@@ -1,3 +1,4 @@
+/* $Id: misc.c,v 1.15 2001/01/20 13:07:29 amai Exp $ */
 /*****************************************************************************/
 /**       Copyright 1988 by Evans & Sutherland Computer Corporation,        **/
 /**                          Salt Lake City, Utah                           **/
@@ -33,24 +34,31 @@
  * mwm - "LessTif Window Manager"
  ***********************************************************************/
 
-#include "mwm.h"
+#include <LTconfig.h>
 
+#include <stdio.h>
 #ifdef HAVE_SYS_TYPES_H
 #include <sys/types.h>
 #endif
 #ifdef HAVE_SYS_TIME_H
 #include <sys/time.h>
-#ifdef TIME_WITH_SYS_TIME
-#include <time.h>
-#endif
 #endif
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
-#ifdef HAVE_SYS_SELECT_H
-#include <sys/select.h>
+
+#include <Xm/Xm.h>
+#include <Xm/MwmUtil.h>
+#include "mwm.h"
+
+#include <X11/keysym.h>
+
+#include <Xm/XmosP.h>
+#if XmVERSION >= 2
+#include <XmI/XmI.h>
 #endif
 
+
 static Time lastTimestamp = CurrentTime;
 
 /*
@@ -221,7 +229,7 @@ window_in_subtree(MwmWindow *win, MwmWindow *sub)
 /*
  * Removes expose events for a specific window from the queue 
  */
-int
+extern int
 MISC_FlushExpose(Window w)
 {
     XEvent dummy;
@@ -235,7 +243,7 @@ MISC_FlushExpose(Window w)
 /*
  * Start/Stops the auto-raise timer
  */
-void
+extern void
 MISC_SetTimer(int delay)
 {
 #ifdef HAVE_SETITIMER
@@ -252,7 +260,7 @@ MISC_SetTimer(int delay)
 /*
  * Records the time of the last processed event. Used in XSetInputFocus
  */
-Boolean
+extern Boolean
 MISC_StashEventTime(XEvent *ev)
 {
     Time NewTimestamp = CurrentTime;
@@ -297,7 +305,7 @@ MISC_StashEventTime(XEvent *ev)
 /*
  * fetch the last saved time
  */
-Time
+extern Time
 MISC_FetchEventTime()
 {
     return lastTimestamp;
@@ -308,7 +316,7 @@ MISC_FetchEventTime()
  * Cleare the CIRCULATED field of the window flags.
  * 
  *****************************************************************************/
-void
+extern void
 MISC_SetFocusSequence(ScreenInfo *scr)
 {
     MwmWindow *temp;
@@ -327,7 +335,7 @@ MISC_SetFocusSequence(ScreenInfo *scr)
  * Grab the pointer and keyboard
  *
  ****************************************************************************/
-Boolean
+extern Boolean
 MISC_Grab(ScreenInfo *scr, int cursor)
 {
     int i = 0, val = 0;
@@ -371,7 +379,7 @@ MISC_Grab(ScreenInfo *scr, int cursor)
  * UnGrab the pointer and keyboard
  *
  ****************************************************************************/
-void
+extern void
 MISC_Ungrab(ScreenInfo *scr)
 {
     Window w;
@@ -400,7 +408,7 @@ MISC_Ungrab(ScreenInfo *scr)
  * 
  * Discard superflous button events during this wait period.
  */
-void
+extern void
 MISC_WaitForButtonsUp(ScreenInfo *scr)
 {
     Bool AllUp = False;
@@ -432,7 +440,7 @@ MISC_WaitForButtonsUp(ScreenInfo *scr)
  * For menus, move, and resize operations, we can effect keyboard 
  * shortcuts by warping the pointer.
  */
-void
+extern void
 MISC_KeyboardShortcut(ScreenInfo *scr, XEvent *event, int ReturnEvent)
 {
     int x, y, x_root, y_root;
@@ -501,7 +509,7 @@ MISC_KeyboardShortcut(ScreenInfo *scr, XEvent *event, int ReturnEvent)
 /*
  * find the root of a window tree
  */
-MwmWindow *
+extern MwmWindow *
 MISC_RootOfTree(MwmWindow *win)
 {
     MwmWindow *anc;
@@ -519,7 +527,7 @@ MISC_RootOfTree(MwmWindow *win)
 /*
  * print the window tree
  */
-void
+extern void
 MISC_PrintTree(ScreenInfo *scr)
 {
     print_window_tree(scr->mwm_root.next, 0);
@@ -528,7 +536,7 @@ MISC_PrintTree(ScreenInfo *scr)
 /*
  * add a window to the window tree
  */
-void
+extern void
 MISC_AddToTree(ScreenInfo *scr, MwmWindow *win)
 {
     MwmWindow *tmp;
@@ -563,7 +571,7 @@ MISC_AddToTree(ScreenInfo *scr, MwmWindow *win)
 /*
  * remove a window from the window tree
  */
-void
+extern void
 MISC_RemoveFromTree(ScreenInfo *scr, MwmWindow *win)
 {
     if (win->ancestor != NULL)
@@ -583,7 +591,7 @@ MISC_RemoveFromTree(ScreenInfo *scr, MwmWindow *win)
  * after starting, fix up the transients whose parent windows weren't
  * captured yet
  */
-void
+extern void
 MISC_FixupTransients(ScreenInfo *scr)
 {
     MwmWindow *tmp, *ptr;
@@ -613,7 +621,7 @@ MISC_FixupTransients(ScreenInfo *scr)
     }
 }
 
-void
+extern void
 MISC_DestroyChildren(ScreenInfo *scr, MwmWindow *win)
 {
     MwmWindow *child, *tmp;
@@ -625,33 +633,8 @@ MISC_DestroyChildren(ScreenInfo *scr, MwmWindow *win)
     }
 }
 
-#if BUILD_FOR_MOTIF
-struct _XmFontListRec {
-    char *tag;
-    XmFontType type;
-    XtPointer font;
-};
-
-XmFontList
-_XmFontListCreateDefault(Display *d)
-{
-    struct _XmFontListRec defaultEntry;
-    XmFontList defaultList;
-    XFontStruct *fs = XLoadQueryFont(d, XmDEFAULT_FONT);
- 
-    defaultEntry.tag = XmFONTLIST_DEFAULT_TAG;
-    defaultEntry.type = XmFONT_IS_FONT;
-    defaultEntry.font = (XtPointer)fs;
- 
-    defaultList = XmFontListAppendEntry(NULL, &defaultEntry);
- 
-    return defaultList;
-}
-
-#endif
-
 /* Debugging purposes only */
-char *
+extern const char *
 _MwmPrintF(int x)
 {
 	switch(x) {
@@ -704,10 +687,10 @@ _MwmPrintF(int x)
 	}
 }
 
-char *
+extern const char *
 _MwmPrintC(int x)
 {
-	static char	res[256];
+	static char res[256];
 
 	res[0] = 0;
 	if (x == C_NO_CONTEXT)
