@@ -1,3 +1,4 @@
+/* $Id: mwm.h,v 1.21 2001/02/07 13:32:25 amai Exp $ */
 /*****************************************************************************/
 /**       Copyright 1988 by Evans & Sutherland Computer Corporation,        **/
 /**                          Salt Lake City, Utah                           **/
@@ -33,37 +34,19 @@
  * mwm - "LessTif Window Manager"
  ***********************************************************************/
 
-#ifndef MWM_H
-#define MWM_H
 
-#include <LTconfig.h>
+#ifndef _MWM_H
+#define _MWM_H
 
-#include <ctype.h>
-#include <stdio.h>
-#ifdef HAVE_STRING_H
-#include <string.h>
-#else
-#define ANSI_STRING
-#include <strings.h>
-#endif
-#include <stdlib.h>
-#include <signal.h>
-#ifdef HAVE_SYS_WAIT_H
-#include <sys/wait.h>
-#endif
+#if 0
 #include <X11/Xlib.h>
 #include <X11/Xatom.h>
-#include <X11/Xutil.h>
 #include <X11/Xresource.h>
-#include <X11/Xproto.h>
 #include <X11/Intrinsic.h>
-#include <X11/IntrinsicP.h>
-#include <X11/keysym.h>
-#include <X11/cursorfont.h>
-#include <X11/extensions/shape.h>
-#include <Xm/XmP.h>
-#include <Xm/XmosP.h>
+
+#include <Xm/Xm.h>
 #include <Xm/MwmUtil.h>
+#endif
 
 /*
  * path components
@@ -84,7 +67,7 @@
 #define MWM_ICONDIR		XINCL_PATH"/X11/bitmaps:"XINCL_PATH"/X11/pixmaps"
 #else
 #define MWM_ICONDIR		"/XFree86/include/X11/bitmaps;/XFree86/include/X11/pixmaps"
-char mwm_icondir[255];
+/* char mwm_icondir[255]; */
 #endif
 #define USAGE \
 "Mwm Ver %s\n\nusage: mwm [-display dpy] [-debug] [-xrm resourcestring] [-multiscreen] [-name name] [-screens name [name [...]]]\n"
@@ -464,15 +447,7 @@ typedef struct FuncKey {
 #define MENU_DONE	1
 #define SUBMENU_DONE	2
 
-
-/*
- * ReapChildren - wait() for all dead child processes
- */
-#ifdef HAVE_WAITPID
-#define ReapChildren()  while ((waitpid(-1, NULL, WNOHANG)) > 0);
-#else
-#define ReapChildren()  while ((wait3(NULL, WNOHANG, NULL)) > 0);
-#endif
+void ReapChildren(void);
 
 #define SIZE_HINDENT 5
 #define SIZE_VINDENT 3
@@ -570,7 +545,7 @@ typedef struct MwmWindow {
     XWindowAttributes attr;		/* the child window attributes */
     XSizeHints      hints;		/* normal hints */
     XWMHints       *wmhints;		/* WM hints */
-    XClassHint      class;
+    XClassHint      classhint;
 
     int             Desk;		/* Tells which desktop this window is on */
     int             FocusDesk;		/* Where (if at all) was it focussed */
@@ -866,7 +841,7 @@ typedef struct MwmInternalInfo {
  * error handler
  */
 #define REDIRECT	01
-#define GENERAL		02
+#define GENERAL	02
 
 /*
  * gotta know which way
@@ -881,7 +856,7 @@ typedef struct MwmInternalInfo {
 extern void	PAN_Initialize(ScreenInfo *scr);
 extern void	PAN_Raise(ScreenInfo *scr);
 extern void	PAN_CheckBounds(ScreenInfo *scr);
-extern void     PAN_PanDesktop(ScreenInfo *scr, int, int,
+extern void PAN_PanDesktop(ScreenInfo *scr, int, int,
 			       int *, int *, int *, int *, Boolean, XEvent *);
 extern Boolean  PAN_IsPannerWindow(ScreenInfo *scr, Window win);
 
@@ -891,14 +866,14 @@ extern void	DT_ChangeDesks(ScreenInfo *scr, int val1, int val2);
 extern void	DT_WindowChangingDesks(ScreenInfo *scr, MwmWindow *t, int val1);
 
 /* Pager functions */
-extern void     PAGER_Clear(ScreenInfo *scr);
-extern void     PAGER_Redraw(ScreenInfo *scr);
-extern void     PAGER_SwitchPage(ScreenInfo *scr, Bool, Bool, XEvent *event);
-extern void     PAGER_UpdateViewPort(ScreenInfo *scr);
-extern void     PAGER_UpdateView(ScreenInfo *scr, MwmWindow * t);
+extern void PAGER_Clear(ScreenInfo *scr);
+extern void PAGER_Redraw(ScreenInfo *scr);
+extern void PAGER_SwitchPage(ScreenInfo *scr, Bool, Bool, XEvent *event);
+extern void PAGER_UpdateViewPort(ScreenInfo *scr);
+extern void PAGER_UpdateView(ScreenInfo *scr, MwmWindow * t);
 extern void	PAGER_Initialize(ScreenInfo *scr, Position x, Position y);
-extern void     PAGER_Update(ScreenInfo *scr, XEvent *event);
-extern void     PAGER_MoveViewPort(ScreenInfo *scr, int newx, int newy, Boolean);
+extern void PAGER_Update(ScreenInfo *scr, XEvent *event);
+extern void PAGER_MoveViewPort(ScreenInfo *scr, int newx, int newy, Boolean);
 
 /* Parser functions */
 extern void	PARSE_mwmrc(ScreenInfo *scr);
@@ -906,19 +881,19 @@ extern int	PARSE_buf(ScreenInfo *scr, char *buf);
 
 /* Resource functions */
 extern void	RES_Initialize(void);
-extern void	RES_GetClientDefaults(ScreenInfo *scr, MwmWindow *win, char *name, char *class);
+extern void	RES_GetClientDefaults(ScreenInfo *scr, MwmWindow *win, char *name_s, char *class_s);
 extern void	RES_GetComponentDefaults(ScreenInfo *scr);
 
 /* Miscellaneous functions */
 extern Boolean	MISC_Grab(ScreenInfo *scr, int);
-extern void     MISC_Ungrab(ScreenInfo *scr);
+extern void MISC_Ungrab(ScreenInfo *scr);
 extern void	MISC_WaitForButtonsUp(ScreenInfo *scr);
-extern int      MISC_FlushExpose(Window w);
-extern void     MISC_SetTimer(int);
+extern int  MISC_FlushExpose(Window w);
+extern void MISC_SetTimer(int);
 extern Boolean  MISC_StashEventTime(XEvent * ev);
 extern Time	MISC_FetchEventTime(void);
-extern void     MISC_SetFocusSequence(ScreenInfo *scr);
-extern void     MISC_KeyboardShortcut(ScreenInfo *scr, XEvent *, int);
+extern void MISC_SetFocusSequence(ScreenInfo *scr);
+extern void MISC_KeyboardShortcut(ScreenInfo *scr, XEvent *, int);
 extern void	MISC_AddToTree(ScreenInfo *scr, MwmWindow *win);
 extern void	MISC_RemoveFromTree(ScreenInfo *scr, MwmWindow *win);
 extern void	MISC_PrintTree(ScreenInfo *scr);
@@ -935,7 +910,7 @@ extern void	PROP_SetBehavior(ScreenInfo *scr, Boolean custom);
 extern void	PROP_ClearBehavior(ScreenInfo *scr);
 extern void	PROP_SetPriorityColors(ScreenInfo *scr);
 extern Boolean	PROP_CheckDesktop(ScreenInfo *scr);
-extern void     PROP_SetState(MwmWindow *, int);
+extern void PROP_SetState(MwmWindow *, int);
 extern void	PROP_SendClientMessage(Window w, Atom a, Time timestamp);
 extern void	PROP_GetMwmHints(MwmWindow *win);
 extern void	PROP_GetMwmMenu(MwmWindow *win);
@@ -956,18 +931,18 @@ extern ScreenInfo *SCREEN_EventToStruct(XEvent *event);
 extern void	WIN_CaptureWindows(ScreenInfo *scr);
 extern void	WIN_ReleaseWindows(ScreenInfo *scr);
 extern void	WIN_MapWindow(ScreenInfo *scr, Window win);
-extern void     WIN_SetFocus(ScreenInfo *scr, Window, MwmWindow *);
+extern void WIN_SetFocus(ScreenInfo *scr, Window, MwmWindow *);
 extern void	WIN_ChangeFocus(ScreenInfo *scr, MwmWindow *t,
 				int DeIconifyOnly);
-extern void     WIN_RestoreWithdrawn(ScreenInfo *scr, MwmWindow *, Boolean);
+extern void WIN_RestoreWithdrawn(ScreenInfo *scr, MwmWindow *, Boolean);
 extern MwmWindow *WIN_WindowToStruct(ScreenInfo *scr, Window target);	
-extern void     WIN_Raise(ScreenInfo *scr, MwmWindow * t);
-extern void     WIN_Lower(ScreenInfo *scr, MwmWindow * t);
+extern void WIN_Raise(ScreenInfo *scr, MwmWindow * t);
+extern void WIN_Lower(ScreenInfo *scr, MwmWindow * t);
 extern void	WIN_ConstrainWindow(ScreenInfo *scr, MwmWindow *win,
 				    int *widthp, int *heightp);
 extern void	WIN_DrawOutline(ScreenInfo *scr, Window win,
 				int x, int y, int wd, int ht);
-extern void     WIN_FreeNames(MwmWindow * tmp, Bool nukename, Bool nukeicon);
+extern void WIN_FreeNames(MwmWindow * tmp, Bool nukename, Bool nukeicon);
 extern void	WIN_DestroyWindow(ScreenInfo *scr, MwmWindow *Tmp_win);
 extern void	WIN_SetFocusInTree(MwmWindow *win);
 
@@ -977,19 +952,19 @@ extern void	COLOR_PushRootColorMap(ScreenInfo *scr);
 extern void	COLOR_PopRootColorMap(ScreenInfo *scr);
 
 /* Decoration functions */
-extern void     DEC_DrawDecorations(ScreenInfo *, MwmWindow *,
+extern void DEC_DrawDecorations(ScreenInfo *, MwmWindow *,
 				    Bool, Bool, Bool, Window);
-extern void     DEC_DrawTitleBar(ScreenInfo *scr, MwmWindow *, Bool, Bool);
-extern void     DEC_DrawShadows(MwmWindow *, Window, int, int, int, int,
+extern void DEC_DrawTitleBar(ScreenInfo *scr, MwmWindow *, Bool, Bool);
+extern void DEC_DrawShadows(MwmWindow *, Window, int, int, int, int,
 				GC, GC);
-extern void     DEC_DrawBorder(MwmWindow *, Window, int, int, int, int,
+extern void DEC_DrawBorder(MwmWindow *, Window, int, int, int, int,
 			       GC, GC);
-extern void     DEC_CreateDecorations(ScreenInfo *scr, MwmWindow *);
-extern void     DEC_ConfigureDecorations(ScreenInfo *scr, MwmWindow *,
+extern void DEC_CreateDecorations(ScreenInfo *scr, MwmWindow *);
+extern void DEC_ConfigureDecorations(ScreenInfo *scr, MwmWindow *,
 					 int, int, int, int, Boolean);
-extern void     DEC_SelectDecorations(ScreenInfo *scr, MwmWindow *t);
-extern void     DEC_ReselectDecorations(ScreenInfo *scr, MwmWindow *t);
-extern void     DEC_SetShape(MwmWindow *, int);
+extern void DEC_SelectDecorations(ScreenInfo *scr, MwmWindow *t);
+extern void DEC_ReselectDecorations(ScreenInfo *scr, MwmWindow *t);
+extern void DEC_SetShape(MwmWindow *, int);
 
 /* Event functions */
 extern void	EVENT_Initialize(void);
@@ -998,18 +973,18 @@ extern int	EVENT_Next(XEvent *event);
 extern void	EVENT_Dispatch(XEvent *event);
 
 /* Function functions */
-extern void     FUNC_Execute(ScreenInfo *scr, int, char *, Window,
+extern void FUNC_Execute(ScreenInfo *scr, int, char *, Window,
 			     MwmWindow *, XEvent *, unsigned long,
 			     long, long, int, int, MenuRoot *);
 
 /* Icon functions */
-extern void     ICON_CreateWindow(ScreenInfo *scr, MwmWindow *tmp,
+extern void ICON_CreateWindow(ScreenInfo *scr, MwmWindow *tmp,
 				  int def_x, int def_y);
-extern void     ICON_DrawWindow(ScreenInfo *scr, MwmWindow *);
-extern void     ICON_UpdateWindow(ScreenInfo *scr, MwmWindow *, Boolean force);
-extern void     ICON_AutoPlace(ScreenInfo *scr, MwmWindow *);
-extern void     ICON_Iconify(ScreenInfo *scr, MwmWindow *, int, int);
-extern void     ICON_DeIconify(ScreenInfo *scr, MwmWindow *);
+extern void ICON_DrawWindow(ScreenInfo *scr, MwmWindow *);
+extern void ICON_UpdateWindow(ScreenInfo *scr, MwmWindow *, Boolean force);
+extern void ICON_AutoPlace(ScreenInfo *scr, MwmWindow *);
+extern void ICON_Iconify(ScreenInfo *scr, MwmWindow *, int, int);
+extern void ICON_DeIconify(ScreenInfo *scr, MwmWindow *);
 
 /* Menu functions */
 extern MenuRoot *MENU_Create(char *name);
@@ -1022,21 +997,21 @@ extern void	MENU_AddItem(ScreenInfo *scr, MenuRoot *menu,
 			     long func_val_1, long func_val_2,
 			     char unit_1, char unit_2);
 extern void	MENU_LinkUp(ScreenInfo *scr);
-extern void     MENU_Realize(ScreenInfo *scr, MenuRoot *);
-extern void     MENU_RealizeMenus(ScreenInfo *scr);
-extern int      MENU_PopupMenu(ScreenInfo *scr, MenuRoot * menu);
-extern int      MENU_WinMenu(ScreenInfo *scr, MenuRoot *menu,
+extern void MENU_Realize(ScreenInfo *scr, MenuRoot *);
+extern void MENU_RealizeMenus(ScreenInfo *scr);
+extern int  MENU_PopupMenu(ScreenInfo *scr, MenuRoot * menu);
+extern int  MENU_WinMenu(ScreenInfo *scr, MenuRoot *menu,
 			     MwmWindow *win, Boolean button, Boolean icon);
 extern void	MENU_Destroy(MenuRoot *menu);
-extern void     MENU_DestroyMenus(ScreenInfo *scr);
-extern void     MENU_Reset(void);
-extern void     MENU_BuildWindowMenu(ScreenInfo *scr, MwmWindow *win);
-extern void     MENU_DestroyWindowMenu(ScreenInfo *scr, MwmWindow *win);
+extern void MENU_DestroyMenus(ScreenInfo *scr);
+extern void MENU_Reset(void);
+extern void MENU_BuildWindowMenu(ScreenInfo *scr, MwmWindow *win);
+extern void MENU_DestroyWindowMenu(ScreenInfo *scr, MwmWindow *win);
 extern char	*MENU_AcceleratorString(ScreenInfo *scr, KeySym key,
 					int modifiers);
 
 /* Movement functions */
-extern void     MOVE_EventLoop(ScreenInfo *scr, MwmWindow *, int, int, int, int,
+extern void MOVE_EventLoop(ScreenInfo *scr, MwmWindow *, int, int, int, int,
 			       int *, int *, Boolean, Boolean);
 extern void	MOVE_Interactive(ScreenInfo *scr, Window *win, MwmWindow *tmp_win,
 				 int *FinalX, int *FinalY, XEvent *eventp);
@@ -1048,11 +1023,11 @@ extern void	RESIZE_EventLoop(ScreenInfo *scr, Window w, MwmWindow *tmp_win,
 
 /* exit functions */
 extern void	MWM_SetErrorHandler(int which);
-extern void     MWM_Done(int, char *);
+extern void MWM_Done(int, const char *);
 
 /* Debugging functions */
-extern char	 *_MwmPrintC(int x);
-extern char	 *_MwmPrintF(int x);
+extern const char	 *_MwmPrintC(int x);
+extern const char	 *_MwmPrintF(int x);
 
 /* Resource default functions */
 extern void	_WmMultiClickTimeDefault(Widget w, int offset, XrmValue *val);
@@ -1130,4 +1105,4 @@ extern Atom     XA_MWM_MESSAGES;
 extern Atom     XA_MWM_MENU;
 extern Atom     XA_MWM_INFO;
 
-#endif /* _MWM_ */
+#endif /* _MWM_H */
