@@ -33,9 +33,13 @@
  * mwm - "LessTif Window Manager"
  ***********************************************************************/
 
+#include <LTconfig.h>
+
+#include <Xm/Xm.h>
+#include <Xm/MwmUtil.h>
 #include "mwm.h"
 
-#include <X11/Intrinsic.h>
+#include <X11/extensions/shape.h>
 
 /*
  * button dimensions
@@ -679,7 +683,7 @@ DEC_ReselectDecorations(ScreenInfo *scr, MwmWindow *t)
 					    CopyFromParent,
 					    valuemask, &attributes);
 
-		XSaveContext(dpy, t->sides[i], MwmContext, (_Xconst char *)t);
+		XSaveContext(dpy, t->sides[i], MwmContext, (XPointer)t);
 	    }
 	}
 	for (i = 0; i < 4; i++)
@@ -743,7 +747,7 @@ DEC_ReselectDecorations(ScreenInfo *scr, MwmWindow *t)
 					      InputOutput, CopyFromParent,
 					      valuemask, &attributes);
 
-		XSaveContext(dpy, t->corners[i], MwmContext, (_Xconst char *)t);
+		XSaveContext(dpy, t->corners[i], MwmContext, (XPointer)t);
 	    }
 	}
 	for (i = 0; i < 4; i++)
@@ -800,7 +804,7 @@ DEC_ReselectDecorations(ScreenInfo *scr, MwmWindow *t)
 				     CopyFromParent,
 				     valuemask, &attributes);
 
-	    XSaveContext(dpy, t->title, MwmContext, (_Xconst char *)t);
+	    XSaveContext(dpy, t->title, MwmContext, (XPointer)t);
 	}
 	XRaiseWindow(dpy, t->title);
     }
@@ -840,7 +844,7 @@ DEC_ReselectDecorations(ScreenInfo *scr, MwmWindow *t)
 				     CopyFromParent,
 				     valuemask, &attributes);
 
-	    XSaveContext(dpy, t->menub, MwmContext, (_Xconst char *)t);
+	    XSaveContext(dpy, t->menub, MwmContext, (XPointer)t);
 	}
 	XRaiseWindow(dpy, t->menub);
     }
@@ -882,7 +886,7 @@ DEC_ReselectDecorations(ScreenInfo *scr, MwmWindow *t)
 					 CopyFromParent,
 					 valuemask, &attributes);
 
-	    XSaveContext(dpy, t->minimizeb, MwmContext, (_Xconst char *)t);
+	    XSaveContext(dpy, t->minimizeb, MwmContext, (XPointer)t);
 	}
 	XRaiseWindow(dpy, t->minimizeb);
     }
@@ -924,7 +928,7 @@ DEC_ReselectDecorations(ScreenInfo *scr, MwmWindow *t)
 					 CopyFromParent,
 					 valuemask, &attributes);
 
-	    XSaveContext(dpy, t->maximizeb, MwmContext, (_Xconst char *)t);
+	    XSaveContext(dpy, t->maximizeb, MwmContext, (XPointer)t);
 	}
 	XRaiseWindow(dpy, t->maximizeb);
     }
