@@ -1,3 +1,4 @@
+/* $Id: icons.c,v 1.14 2001/05/17 13:03:27 amai Exp $ */
 /****************************************************************************
  * This module is mostly all new
  * by Rob Nation
@@ -11,17 +12,29 @@
  *
  ***********************************************************************/
 
-#include "mwm.h"
-#include <XmI/XmXpm.h>
+#include <LTconfig.h>
 
+#include <stdio.h>
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
-
 #ifdef HAVE_FCNTL_H
 #include <fcntl.h>
 #endif
 
+#include <Xm/Xm.h>
+#include <Xm/MwmUtil.h>
+#include "mwm.h"
+
+#include <X11/extensions/shape.h>
+
+#if XmVERSION >= 2
+#include <Xm/XpmP.h>
+#else
+#include <XmI/XmXpm.h>
+#endif
+
+
 #define def_bitmap_width 50
 #define def_bitmap_height 50
 static unsigned char def_bitmap_bits[] =
@@ -57,6 +70,15 @@ static unsigned char def_bitmap_bits[] =
     0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x02, 0xff, 0xff, 0xff, 0xff, 0xff,
     0xff, 0x03};
 
+
+#ifdef __EMX__
+ /* dos-like */
+#define cPATHELEMENTSEP ';'
+#else
+  /* un*x-like */
+#define cPATHELEMENTSEP ':'
+#endif
+
 /*
  * Find the specified icon file somewhere along the given path.
  *
@@ -100,7 +122,7 @@ find_icon_file(char *icon, char *pathlist, int type)
     /* Search each element of the pathlist for the icon file */
     while ((pathlist) && (*pathlist))
     {
-	dir_end = strchr(pathlist, ':');
+	dir_end = strchr(pathlist, cPATHELEMENTSEP);
 	if (dir_end != NULL)
 	{
 	    strncpy(path, pathlist, dir_end - pathlist);
@@ -124,7 +146,7 @@ find_icon_file(char *icon, char *pathlist, int type)
 	    pathlist = dir_end + 1;
     }
     /* Hmm, couldn't find the file.  Return NULL */
-    free(path);
+    XtFree(path);
     return NULL;
 }
 
@@ -206,7 +228,7 @@ get_bitmap_file(ScreenInfo *scr, MwmWindow *tmp_win)
 	tmp_win->icon_p_height = 0;
     }
 
-    free(path);
+    XtFree(path);
 }
 
 /*
@@ -216,7 +238,11 @@ static void
 get_xpm_file(ScreenInfo *scr, MwmWindow *tmp_win)
 {
     XWindowAttributes root_attr;
+#if XmVERSION >= 2
+    XpmAttributes xpm_attributes;
+#else
     _LtXpmAttributes xpm_attributes;
+#endif
     XImage *im, *imshape;
     GC gc;
     char *path = NULL;
@@ -228,12 +254,20 @@ get_xpm_file(ScreenInfo *scr, MwmWindow *tmp_win)
     XGetWindowAttributes(dpy, scr->root_win, &root_attr);
     xpm_attributes.colormap = root_attr.colormap;
     xpm_attributes.closeness = 40000;	/* Allow for "similar" colors */
+#if XmVERSION >= 2
+    xpm_attributes.valuemask = XpmSize | XpmReturnPixels | XpmColormap |
+	XpmCloseness;
+#else
     xpm_attributes.valuemask = _LtXpmSize | _LtXpmReturnPixels | _LtXpmColormap |
 	_LtXpmCloseness;
+#endif
 
 #ifdef NONSTANDARD_CONVERTERS
-    if (_LtXpmReadFileToImage(dpy, path, &im, &imshape, &xpm_attributes) ==
-	_LtXpmSuccess)
+#if XmVERSION >= 2
+    if (XpmReadFileToImage(dpy, path, &im, &imshape, &xpm_attributes) == XpmSuccess)
+#else
+    if (_LtXpmReadFileToImage(dpy, path, &im, &imshape, &xpm_attributes) == _LtXpmSuccess)
+#endif
     {
 
 	tmp_win->icon_pixmap = XCreatePixmap(dpy, scr->root_win,
@@ -271,7 +305,7 @@ get_xpm_file(ScreenInfo *scr, MwmWindow *tmp_win)
     }
 #endif
 
-    free(path);
+    XtFree(path);
 }
 
 /*
@@ -687,7 +721,7 @@ ICON_CreateWindow(ScreenInfo *scr, MwmWindow *tmp_win, int def_x, int def_y)
 				  valuemask, &attributes);
 
 		XSaveContext(dpy, tmp_win->icon_borders[i],
-			     MwmContext, (_Xconst char *)tmp_win);
+			     MwmContext, (XPointer)tmp_win);
 	    }
 	}
 	else
@@ -712,14 +746,14 @@ ICON_CreateWindow(ScreenInfo *scr, MwmWindow *tmp_win, int def_x, int def_y)
 
     if (tmp_win->icon_w != None)
     {
-	XSaveContext(dpy, tmp_win->icon_w, MwmContext, (_Xconst char *)tmp_win);
+	XSaveContext(dpy, tmp_win->icon_w, MwmContext, (XPointer)tmp_win);
 	XDefineCursor(dpy, tmp_win->icon_w, scr->cursors[DEFAULT_CURS]);
 	grab_icon_buttons(scr, tmp_win, tmp_win->icon_w);
 	grab_icon_keys(scr, tmp_win, tmp_win->icon_w);
     }
     if (tmp_win->icon_pixmap_w != None)
     {
-	XSaveContext(dpy, tmp_win->icon_pixmap_w, MwmContext, (_Xconst char *)tmp_win);
+	XSaveContext(dpy, tmp_win->icon_pixmap_w, MwmContext, (XPointer)tmp_win);
 	XDefineCursor(dpy, tmp_win->icon_pixmap_w, scr->cursors[DEFAULT_CURS]);
 	grab_icon_buttons(scr, tmp_win, tmp_win->icon_pixmap_w);
 	grab_icon_keys(scr, tmp_win, tmp_win->icon_pixmap_w);
