@@ -1,3 +1,4 @@
+/* $Id: screens.c,v 1.8 2001/01/20 13:07:29 amai Exp $ */
 /*****************************************************************************/
 /**       Copyright 1988 by Evans & Sutherland Computer Corporation,        **/
 /**                          Salt Lake City, Utah                           **/
@@ -33,8 +34,32 @@
  * mwm - "LessTif Window Manager"
  ***********************************************************************/
 
+#include <LTconfig.h>
+
+#include <stdio.h>
+
+#include <Xm/Xm.h>
+#include <Xm/MwmUtil.h>
 #include "mwm.h"
-#include "X11/bitmaps/stipple"
+
+
+/* amai: Unfortunately X11 forgot about non-case-sensitive filesystems which
+   may have problems to store both "X11/bitmaps/Stipple" and
+   "X11/bitmaps/stipple".
+   So we inline the few lines here instead. */
+#if 0
+#include <X11/bitmaps/stipple>
+#else
+#define stipple_width 16
+#define stipple_height 4
+static char stipple_bits[] = {
+   0x55, 0x55, 0xee, 0xee, 0x55, 0x55, 0xba, 0xbb};
+#endif /* #if 0 */
+
+#include <X11/keysym.h>
+
+#include <Xm/XmP.h>
+
 
 extern Boolean debugging;
 
