@@ -1,3 +1,4 @@
+/* $Id: windows.c,v 1.19 2001/05/17 13:03:27 amai Exp $ */
 /*****************************************************************************/
 /**       Copyright 1988 by Evans & Sutherland Computer Corporation,        **/
 /**                          Salt Lake City, Utah                           **/
@@ -42,13 +43,19 @@
  * mwm - "LessTif Window Manager"
  ***********************************************************************/
 
-#include "mwm.h"
+
+#include <LTconfig.h>
 
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
 
 #include <X11/Intrinsic.h>
+#include <X11/extensions/shape.h>
+
+#include <Xm/Xm.h>
+#include <Xm/MwmUtil.h>
+#include "mwm.h"
 
 /*
  * some macros used by the constraint code
@@ -279,29 +286,29 @@ save_context(MwmWindow *tmp_win)
 {
     int i;
 
-    XSaveContext(dpy, tmp_win->w, MwmContext, (_Xconst char *)tmp_win);
-    XSaveContext(dpy, tmp_win->frame, MwmContext, (_Xconst char *)tmp_win);
-    XSaveContext(dpy, tmp_win->parent, MwmContext, (_Xconst char *)tmp_win);
+    XSaveContext(dpy, tmp_win->w, MwmContext, (XPointer)tmp_win);
+    XSaveContext(dpy, tmp_win->frame, MwmContext, (XPointer)tmp_win);
+    XSaveContext(dpy, tmp_win->parent, MwmContext, (XPointer)tmp_win);
     if (tmp_win->decorations & MWM_DECOR_TITLE)
     {
 	XSaveContext(dpy, tmp_win->title, MwmContext,
-		     (_Xconst char *)tmp_win);
+		     (XPointer)tmp_win);
 	if (tmp_win->menub != None)
 	    XSaveContext(dpy, tmp_win->menub, MwmContext,
-			 (_Xconst char *)tmp_win);
+			 (XPointer)tmp_win);
 	if (tmp_win->minimizeb != None)
 	    XSaveContext(dpy, tmp_win->minimizeb, MwmContext,
-			 (_Xconst char *)tmp_win);
+			 (XPointer)tmp_win);
 	if (tmp_win->maximizeb != None)
 	    XSaveContext(dpy, tmp_win->maximizeb, MwmContext,
-			 (_Xconst char *)tmp_win);
+			 (XPointer)tmp_win);
     }
     if (tmp_win->decorations & MWM_DECOR_BORDER)
     {
 	for (i = 0; i < 4; i++)
 	{
 	    XSaveContext(dpy, tmp_win->sides[i],
-			 MwmContext, (_Xconst char *)tmp_win);
+			 MwmContext, (XPointer)tmp_win);
 	}
     }
     if (tmp_win->decorations & MWM_DECOR_RESIZEH)
@@ -309,7 +316,7 @@ save_context(MwmWindow *tmp_win)
 	for (i = 0; i < 4; i++)
 	{
 	    XSaveContext(dpy, tmp_win->corners[i],
-			 MwmContext, (_Xconst char *)tmp_win);
+			 MwmContext, (XPointer)tmp_win);
 	}
     }
 }
@@ -608,7 +615,7 @@ place_window(ScreenInfo *scr, MwmWindow *tmp_win)
 				 (unsigned int *)&DragHeight,
 				 &JunkBW, &JunkDepth) == 0)
 		{
-		    free((char *)tmp_win);
+		    XtFree((char *)tmp_win);
 		    XUngrabServer(dpy);
 		    return False;
 		}
@@ -699,20 +706,20 @@ add_window(ScreenInfo *scr, Window w)
 
     tmp_win->w = w;
 
-    tmp_win->class.res_name = NoName;
-    tmp_win->class.res_class = NoName;
-    XGetClassHint(dpy, tmp_win->w, &tmp_win->class);
-    if (tmp_win->class.res_name == NULL)
-	tmp_win->class.res_name = NoName;
-    if (tmp_win->class.res_class == NULL)
-	tmp_win->class.res_class = NoName;
+    tmp_win->classhint.res_name = NoName;
+    tmp_win->classhint.res_class = NoName;
+    XGetClassHint(dpy, tmp_win->w, &tmp_win->classhint);
+    if (tmp_win->classhint.res_name == NULL)
+	tmp_win->classhint.res_name = NoName;
+    if (tmp_win->classhint.res_class == NULL)
+	tmp_win->classhint.res_class = NoName;
 
-    RES_GetClientDefaults(scr, tmp_win, tmp_win->class.res_name, tmp_win->class.res_class);
+    RES_GetClientDefaults(scr, tmp_win, tmp_win->classhint.res_name, tmp_win->classhint.res_class);
 
     if (XGetGeometry(dpy, tmp_win->w, &JunkRoot, &JunkX, &JunkY,
 		     &JunkWidth, &JunkHeight, &JunkBW, &JunkDepth) == 0)
     {
-	free((char *)tmp_win);
+	XtFree((char *)tmp_win);
 	return (NULL);
     }
 
@@ -793,7 +800,7 @@ add_window(ScreenInfo *scr, Window w)
 		     &JunkWidth, &JunkHeight,
 		     &JunkBW, &JunkDepth) == 0)
     {
-	free((char *)tmp_win);
+	XtFree((char *)tmp_win);
 	XUngrabServer(dpy);
 	return (NULL);
     }
@@ -801,7 +808,7 @@ add_window(ScreenInfo *scr, Window w)
     XSetWindowBorderWidth(dpy, tmp_win->w, 0);
 
     if (tmp_win->icon_label == NULL)
-	tmp_win->icon_label = tmp_win->class.res_name;
+	tmp_win->icon_label = tmp_win->classhint.res_name;
     tmp_win->icon_active_label = tmp_win->name;
 
     tmp_win->flags &= ~ICONIFIED;
@@ -998,7 +1005,7 @@ lower_children(MwmWindow *win)
 void
 WIN_CaptureWindows(ScreenInfo *scr)
 {
-    int i, j;
+    unsigned int i, j;
     unsigned int nchildren;
     Window root, parent, *children;
 
@@ -1452,7 +1459,7 @@ WIN_Raise(ScreenInfo *scr, MwmWindow *t)
 	XRaiseWindow(dpy, wins[0]);
 
     XRestackWindows(dpy, wins, i);
-    free(wins);
+    XtFree((char *)wins);
 
     PAGER_Clear(scr);
 
@@ -1919,10 +1926,10 @@ WIN_DestroyWindow(ScreenInfo *scr, MwmWindow *tmp)
 
     if (tmp->wmhints)
 	XFree((char *)tmp->wmhints);
-    if (tmp->class.res_name && tmp->class.res_name != NoName)
-	XFree((char *)tmp->class.res_name);
-    if (tmp->class.res_class && tmp->class.res_class != NoName)
-	XFree((char *)tmp->class.res_class);
+    if (tmp->classhint.res_name && tmp->classhint.res_name != NoName)
+	XFree((char *)tmp->classhint.res_name);
+    if (tmp->classhint.res_class && tmp->classhint.res_class != NoName)
+	XFree((char *)tmp->classhint.res_class);
     if (tmp->mwm_hints)
 	XFree((char *)tmp->mwm_hints);
     if (tmp->mwm_menu)
@@ -1933,7 +1940,7 @@ WIN_DestroyWindow(ScreenInfo *scr, MwmWindow *tmp)
     if (tmp->cmap_windows != (Window *)NULL)
 	XFree((void *)tmp->cmap_windows);
 
-    free((char *)tmp);
+    XtFree((char *)tmp);
 
     PAGER_Clear(scr);
     XSync(dpy, 0);
