@@ -1,5 +1,7 @@
 /*
  * Copyright(c) 1992 Bell Communications Research, Inc. (Bellcore)
+ * Copyright (c) 1999-2000 by the LessTif Developers.
+ *
  *                        All rights reserved
  * Permission to use, copy, modify and distribute this material for
  * any purpose and without fee is hereby granted, provided that the
@@ -21,7 +23,7 @@
  *
  * CaptionWidget Author: Andrew Wason, Bellcore, aw@bae.bellcore.com
  *
- * $Id: CaptionP.h,v 1.2 1999/10/23 12:14:09 danny Exp $
+ * $Id: CaptionP.h,v 1.5 2000/06/11 17:58:41 danny Exp $
  */
 
 /*
@@ -31,10 +33,6 @@
 #ifndef _Xbae_CaptionP_h
 #define _Xbae_CaptionP_h
 
-#if defined(__cplusplus) || defined(c_plusplus)
-extern "C" {
-#endif
- 
 #if XmVersion == 1001
 #include <Xm/XmP.h>
 #else
@@ -42,7 +40,10 @@ extern "C" {
 #endif
 #include <Xbae/Caption.h>
 
-
+#ifdef __cplusplus
+extern "C" {
+#endif
+ 
 /*
  * New fields for the Caption widget class record
  */
@@ -92,7 +93,7 @@ typedef struct _XbaeCaptionRec {
     XbaeCaptionPart	caption;
 } XbaeCaptionRec;
 
-#if defined(__cplusplus) || defined(c_plusplus)
+#ifdef __cplusplus
 }
 #endif
 
