@@ -1,6 +1,7 @@
 /*
  * Copyright(c) 1992 Bell Communications Research, Inc. (Bellcore)
  * Copyright(c) 1995-99 Andrew Lister
+ * Copyright (c) 1999-2000 by the LessTif Developers.
  *
  *                        All rights reserved
  * Permission to use, copy, modify and distribute this material for
@@ -21,7 +22,7 @@
  * LOST PROFITS OR OTHER INCIDENTAL OR CONSEQUENTIAL DAMAGES RELAT-
  * ING TO THE SOFTWARE.
  *
- * $Id: Macros.h,v 1.3 2000/01/03 22:33:01 danny Exp $
+ * $Id: Macros.h,v 1.19 2001/05/03 20:22:26 dannybackx Exp $
  */
 
 /*
@@ -31,23 +32,6 @@
 #ifndef _Xbae_Macros_h
 #define _Xbae_Macros_h
 
-#if defined(__cplusplus) || defined(c_plusplus)
-extern "C" {
-#endif
-
-/*
- * Prototype wrapper
- */
-#ifndef P
-#if defined(__STDC__) || defined (__cplusplus)
-#define P(x)		x
-#else
-#define P(x)		()
-#define const
-#define volatile
-#endif
-#endif
-
 #if XmVersion <= 1001
 #include <Xm/XmP.h>
 #else
@@ -56,25 +40,46 @@ extern "C" {
 #include <Xm/TextP.h>
 #include <Xm/PrimitiveP.h>
 
-#if 0
-/*	for BX */
-#define DRAW_SHADOW(dpy, draw, tgc, bgc, sz, x, y, w, h, type)	\
+
+#ifdef __cplusplus
+extern "C" {
+#endif
+
+/*
+ * The third parameter is a string added for debugging purposes.
+ */
+#if	0	/* Danny - DEBUG */
+#define DRAW_SHADOW(dpy, draw, dbg, tgc, bgc, sz, x, y, w, h, type)	\
+    { 									\
+	fprintf(stderr, "_XmDrawShadows(%s)(%d,%d,%d,%d,%d,%d)\n",	\
+		dbg, x, y, w, h, sz, type);				\
+	_XmDrawShadows(dpy, draw, tgc, bgc, x, y, w, h, sz, type);	\
+    }
+#else
+#if 1
+/*
+ * BX (Builder Xcessory - see http://www.ics.com) is linked with a 1.2
+ * version of Motif. Use of an Xbae widget which uses a Motif 2.* function
+ * will crash BX when it tries to use the function.
+ */
+#define DRAW_SHADOW(dpy, draw, dbg, tgc, bgc, sz, x, y, w, h, type)	\
     _XmDrawShadows(dpy, draw, tgc, bgc, x, y, w, h, sz, type)
 
 #else
 #if XmVersion <= 1001
-#define DRAW_SHADOW(dpy, draw, tgc, bgc, sz, x, y, w, h, type)	\
+#define DRAW_SHADOW(dpy, draw, dbg, tgc, bgc, sz, x, y, w, h, type)	\
     _XmDrawShadow(dpy, draw, tgc, bgc, sz, x, y, w, h)
 #else
 #if XmVersion >= 2000
-#define DRAW_SHADOW(dpy, draw, tgc, bgc, sz, x, y, w, h, type)	\
+#define DRAW_SHADOW(dpy, draw, dbg, tgc, bgc, sz, x, y, w, h, type)	\
     XmeDrawShadows(dpy, draw, tgc, bgc, x, y, w, h, sz, type)
 #else
-#define DRAW_SHADOW(dpy, draw, tgc, bgc, sz, x, y, w, h, type)	\
+#define DRAW_SHADOW(dpy, draw, dbg, tgc, bgc, sz, x, y, w, h, type)	\
     _XmDrawShadows(dpy, draw, tgc, bgc, x, y, w, h, sz, type)
 #endif
 #endif
 #endif
+#endif
 
 #if XmVersion <= 1001
 #define DRAW_HIGHLIGHT(dpy, draw, tgc, x, y, w, h, sz, type) \
@@ -138,6 +143,7 @@ extern "C" {
 				 mw->matrix.cell_highlight_thickness + \
 				 mw->matrix.text_shadow_thickness)
 
+#define SOME_ROW_HEIGHT(mw, row)	mw->matrix.row_heights[row]
 #define COLUMN_WIDTH(mw, col)	((mw->matrix.column_widths[col] * \
 				  FONT_WIDTH(mw)) + \
 				 ((int)TEXT_WIDTH_OFFSET(mw) * 2))
@@ -265,6 +271,7 @@ extern "C" {
 				 LABEL_HEIGHT(mw) : 0)
 
 #define COLUMN_POSITION(mw, column) mw->matrix.column_positions[column]
+#define ROW_POSITION(mw, row) mw->matrix.row_positions[row]
  
 #define FIXED_COLUMN_WIDTH(mw)	COLUMN_POSITION(mw, mw->matrix.fixed_columns)
 
@@ -284,10 +291,9 @@ extern "C" {
 #define TRAILING_FIXED_COLUMN_LABEL_OFFSET(mw) (FIXED_COLUMN_LABEL_OFFSET(mw)+\
 						VISIBLE_WIDTH(mw))
 
-#define FIXED_ROW_HEIGHT(mw)	((int)mw->matrix.fixed_rows * ROW_HEIGHT(mw))
+#define	FIXED_ROW_HEIGHT(mw)	xbaeMatrixFixedRowHeight(mw)
 
-#define TRAILING_FIXED_ROW_HEIGHT(mw) ((int)mw->matrix.trailing_fixed_rows * \
-				       ROW_HEIGHT(mw))
+#define TRAILING_FIXED_ROW_HEIGHT(mw) xbaeMatrixTrailingFixedRowHeight(mw)
 
 #define ROW_LABEL_OFFSET(mw)	(COLUMN_LABEL_HEIGHT(mw) + \
 				 HORIZ_SB_OFFSET(mw) + \
@@ -296,6 +302,9 @@ extern "C" {
 #define FIXED_ROW_LABEL_OFFSET(mw) (ROW_LABEL_OFFSET(mw) + \
 				    FIXED_ROW_HEIGHT(mw))
 
+#define UNATTACHED_TRAILING_COLUMNS_OFFSET(mw) (FIXED_COLUMN_LABEL_OFFSET(mw) + \
+					     VISIBLE_WIDTH(mw))
+
 #define UNATTACHED_TRAILING_ROWS_OFFSET(mw) (FIXED_ROW_LABEL_OFFSET(mw) + \
 					     VISIBLE_HEIGHT(mw))
 
@@ -311,6 +320,10 @@ extern "C" {
 					 mw->matrix.trailing_attached_bottom && \
 					 mw->matrix.trailing_fixed_rows) 
 
+#define HAS_ATTACHED_TRAILING_COLUMNS(mw)  (mw->matrix.fill &&  \
+					 mw->matrix.trailing_attached_right && \
+					 mw->matrix.trailing_fixed_columns) 
+
 #define TRAILING_FIXED_ROW_LABEL_OFFSET(mw) (HAS_ATTACHED_TRAILING_ROWS(mw) ? \
 					     ATTACHED_TRAILING_ROWS_OFFSET(mw) : \
 					     UNATTACHED_TRAILING_ROWS_OFFSET(mw))
@@ -319,6 +332,10 @@ extern "C" {
 				      (VERT_DEAD_SPACE_HEIGHT(mw) > 0))
 
 #define NON_FIXED_TOTAL_WIDTH(mw) 	mw->matrix.non_fixed_total_width
+/* Danny
+#define NON_FIXED_TOTAL_HEIGHT(mw) 	mw->matrix.non_fixed_total_height
+ */
+#define NON_FIXED_TOTAL_HEIGHT(mw) 	xbaeCellTotalHeight(mw)
 
 #define CELL_TOTAL_HEIGHT(mw)	((mw->matrix.rows - \
 				  (int) mw->matrix.fixed_rows - \
@@ -435,6 +452,12 @@ extern "C" {
 					       mw->matrix.column_widths); \
 					}
 
+#define xbaeFreeRowHeights(mw)	{ \
+					       if (mw->matrix.row_heights) \
+					       XtFree((XtPointer) \
+					       mw->matrix.row_heights); \
+					}
+
 #define xbaeFreeColumnMaxLengths(mw)	{ \
 					       if (mw->matrix.\
 					       column_max_lengths) \
@@ -467,6 +490,14 @@ extern "C" {
 					       column_button_labels); \
 					}
 
+#define xbaeFreeColumnFontBold(mw)	{ \
+						if (mw->matrix. \
+						column_font_bold) \
+						XtFree((XtPointer) \
+						mw->matrix. \
+						column_font_bold); \
+					}
+
 #define xbaeFreeRowButtonLabels(mw)	{ \
 					       if (mw->matrix. \
 					       row_button_labels) \
@@ -515,10 +546,10 @@ extern "C" {
 					       column_shadow_types); \
 					}
 
-#define CreateColumnPositions(mw)  (int *)XtMalloc((mw->matrix.columns+1)* \
-						   sizeof(int))
-#define YtoRow(mw, y)			((y) / ROW_HEIGHT(mw))
-
+#define CreateRowPositions(mw) \
+	(int *)XtMalloc((mw->matrix.rows+1) * sizeof(int))
+#define CreateColumnPositions(mw) \
+	(int *)XtMalloc((mw->matrix.columns+1) * sizeof(int))
 
 /*
  * Evaluates to 1 if two Rectangles overlap, 0 if no overlap
@@ -565,7 +596,7 @@ extern "C" {
 #endif
 
 #ifdef NEED_24BIT_VISUAL
-#define GC_PARENT_WINDOW(w)	XtWindow(get_shell_ancestor((Widget)w))
+#define GC_PARENT_WINDOW(w)	XtWindow(_XbaeGetShellAncestor((Widget)w))
 #else
 #define GC_PARENT_WINDOW(w)	RootWindowOfScreen(XtScreen(w))
 #endif
@@ -574,6 +605,7 @@ extern "C" {
  * End of array flags for the array type converters
  */
 #define BAD_WIDTH	-1
+#define BAD_HEIGHT	-1
 #define BAD_MAXLENGTH	0
 #define BAD_ALIGNMENT	3	/* see Xm.h */
 #define BAD_PIXEL	0x10000000 /* normally 256 indices */ 
@@ -622,7 +654,7 @@ extern "C" {
 #define IN_GRID_ROW_MODE(mw)	(mw->matrix.grid_type & GRID_MODE_ROW)
 #define IN_GRID_COLUMN_MODE(mw)	(mw->matrix.grid_type & GRID_MODE_COLUMN)
 
-#if defined(__cplusplus) || defined(c_plusplus)
+#ifdef __cplusplus
 }
 #endif
 
