@@ -1,6 +1,8 @@
 /*
  * Copyright(c) 1992 Bell Communications Research, Inc. (Bellcore)
  * Copyright(c) 1995-99 Andrew Lister
+ * Copyright (c) 1999, 2000, 2001 by the LessTif Developers.
+ *
  *                        All rights reserved
  * Permission to use, copy, modify and distribute this material for
  * any purpose and without fee is hereby granted, provided that the
@@ -22,7 +24,7 @@
  *
  * MatrixWidget Author: Andrew Wason, Bellcore, aw@bae.bellcore.com
  *
- * $Id: MatrixP.h,v 1.5 1999/12/28 00:10:24 danny Exp $
+ * $Id: MatrixP.h,v 1.20 2001/05/21 19:34:00 dannybackx Exp $
  */
 
 /*
@@ -32,103 +34,68 @@
 #ifndef _Xbae_MatrixP_h
 #define _Xbae_MatrixP_h
 
-#if defined(__cplusplus) || defined(c_plusplus)
+#include <Xm/ManagerP.h>
+#include <Xbae/Matrix.h>
+
+#ifdef __cplusplus
 extern "C" {
 #endif
 
-#if XmVersion <= 1001
-#	include <Xm/XmP.h>
-#else
-#	include <Xm/ManagerP.h>
-#endif
-
-#include <Xbae/Matrix.h>
-
-#ifndef P
-#if defined(__STDC__) || defined (__cplusplus)
-#define P(x)		x
-#else
-#define P(x)		()
-#define const
-#define volatile
-#endif
-#endif
-
-/*
- * A few definitions we like to use, but those with R4 won't have.
- * From Xfuncproto.h in R5.
- */
-
-#ifndef XlibSpecificationRelease
-# ifndef _XFUNCPROTOBEGIN
-#   ifdef __cplusplus                      /* for C++ V2.0 */
-#     define _XFUNCPROTOBEGIN extern "C" {   /* do not leave open across includes */
-#     define _XFUNCPROTOEND }
-#   else
-#     define _XFUNCPROTOBEGIN
-#     define _XFUNCPROTOEND
-#   endif
-# endif /* _XFUNCPROTOBEGIN */
-#else
-#include <X11/Xfuncproto.h>
-#endif
 
 /*
  * New types for the class methods
  */
-_XFUNCPROTOBEGIN
 
-typedef void (*XbaeMatrixSetCellProc) P((XbaeMatrixWidget, int, int,
-					 const String, Boolean));
+typedef void (*XbaeMatrixSetCellProc)(XbaeMatrixWidget, int, int,
+					 const String, Boolean);
 
-typedef String (*XbaeMatrixGetCellProc) P((XbaeMatrixWidget, int, int));
+typedef String (*XbaeMatrixGetCellProc)(XbaeMatrixWidget, int, int);
 
-typedef void (*XbaeMatrixEditCellProc) P((XbaeMatrixWidget, XEvent *,
-					  int, int, String *, Cardinal));
+typedef void (*XbaeMatrixEditCellProc)(XbaeMatrixWidget, XEvent *,
+					  int, int, String *, Cardinal);
 
-typedef void (*XbaeMatrixSelectCellProc) P((XbaeMatrixWidget, int, int));
+typedef void (*XbaeMatrixSelectCellProc)(XbaeMatrixWidget, int, int);
 
-typedef void (*XbaeMatrixSelectRowProc) P((XbaeMatrixWidget, int));
+typedef void (*XbaeMatrixSelectRowProc)(XbaeMatrixWidget, int);
 
-typedef void (*XbaeMatrixSelectColumnProc) P((XbaeMatrixWidget, int));
+typedef void (*XbaeMatrixSelectColumnProc)(XbaeMatrixWidget, int);
 
-typedef void (*XbaeMatrixDeselectAllProc) P((XbaeMatrixWidget));
+typedef void (*XbaeMatrixDeselectAllProc)(XbaeMatrixWidget);
 
-typedef void (*XbaeMatrixSelectAllProc) P((XbaeMatrixWidget));
+typedef void (*XbaeMatrixSelectAllProc)(XbaeMatrixWidget);
 
-typedef void (*XbaeMatrixDeselectCellProc) P((XbaeMatrixWidget, int, int));
+typedef void (*XbaeMatrixDeselectCellProc)(XbaeMatrixWidget, int, int);
 
-typedef void (*XbaeMatrixDeselectRowProc) P((XbaeMatrixWidget, int));
+typedef void (*XbaeMatrixDeselectRowProc)(XbaeMatrixWidget, int);
 
-typedef void (*XbaeMatrixDeselectColumnProc) P((XbaeMatrixWidget, int));
+typedef void (*XbaeMatrixDeselectColumnProc)(XbaeMatrixWidget, int);
 
-typedef Boolean (*XbaeMatrixCommitEditProc) P((XbaeMatrixWidget, XEvent *,
-					       Boolean));
+typedef Boolean (*XbaeMatrixCommitEditProc)(XbaeMatrixWidget, XEvent *,
+					       Boolean);
 
-typedef void (*XbaeMatrixCancelEditProc) P((XbaeMatrixWidget, Boolean));
+typedef void (*XbaeMatrixCancelEditProc)(XbaeMatrixWidget, Boolean);
 
-typedef void (*XbaeMatrixAddRowsProc) P((XbaeMatrixWidget, int, String *,
-					 String *, Pixel *, Pixel *, int));
+typedef void (*XbaeMatrixAddRowsProc)(XbaeMatrixWidget, int, String *,
+					 String *, Pixel *, Pixel *, int);
 
-typedef void (*XbaeMatrixDeleteRowsProc) P((XbaeMatrixWidget, int, int));
+typedef void (*XbaeMatrixDeleteRowsProc)(XbaeMatrixWidget, int, int);
 
-typedef void (*XbaeMatrixAddColumnsProc) P((XbaeMatrixWidget, int, String *,
+typedef void (*XbaeMatrixAddColumnsProc)(XbaeMatrixWidget, int, String *,
 					    String *, short *, int *,
 					    unsigned char*, unsigned char *,
-					    Pixel *, Pixel *, int));
+					    Pixel *, Pixel *, int);
 
-typedef void (*XbaeMatrixDeleteColumnsProc) P((XbaeMatrixWidget, int, int));
+typedef void (*XbaeMatrixDeleteColumnsProc)(XbaeMatrixWidget, int, int);
 
-typedef void (*XbaeMatrixSetRowColorsProc) P((XbaeMatrixWidget, int, Pixel *,
-					      int, Boolean));
+typedef void (*XbaeMatrixSetRowColorsProc)(XbaeMatrixWidget, int, Pixel *,
+					      int, Boolean);
 
-typedef void (*XbaeMatrixSetColumnColorsProc) P((XbaeMatrixWidget, int,
-						 Pixel *, int, Boolean));
+typedef void (*XbaeMatrixSetColumnColorsProc)(XbaeMatrixWidget, int,
+						 Pixel *, int, Boolean);
 
-typedef void (*XbaeMatrixSetCellColorProc) P((XbaeMatrixWidget, int, int,
-					      Pixel, Boolean));
+typedef void (*XbaeMatrixSetCellColorProc)(XbaeMatrixWidget, int, int,
+					      Pixel, Boolean);
 
-_XFUNCPROTOEND
 
 /*
  * Different than the traversal directions in Xm.h
@@ -185,11 +152,9 @@ externalref XbaeMatrixClassRec xbaeMatrixClassRec;
 #define XbaeInheritSelectCell ((XbaeMatrixSelectCellProc) _XtInherit)
 #define XbaeInheritSelectRow ((XbaeMatrixSelectRowProc) _XtInherit)
 #define XbaeInheritSelectColumn ((XbaeMatrixSelectColumnProc) _XtInherit)
-#if XmVersion >= 1002
 #define XbaeInheritHighlightCell ((XbaeMatrixHighlightCellProc) _XtInherit)
 #define XbaeInheritHighlightRow ((XbaeMatrixHighlightRowProc) _XtInherit)
 #define XbaeInheritHighlightColumn ((XbaeMatrixHighlightColumnProc) _XtInherit)
-#endif
 #define XbaeInheritDeselectAll ((XbaeMatrixDeselectAllProc) _XtInherit)
 #define XbaeInheritSelectAll ((XbaeMatrixSelectAllProc) _XtInherit)
 #define XbaeInheritDeselectCell ((XbaeMatrixDeselectCellProc) _XtInherit)
@@ -259,8 +224,9 @@ typedef struct {
     Boolean	scroll_select;		/* flag to scroll a selected cell    */
     Boolean	**selected_cells;	/* 2D array of selected cells	     */
     Boolean	show_arrows;	        /* sow arrows when text obscured?    */
-    Boolean     *column_button_labels;  /* which column labels are butons */
-    Boolean     *row_button_labels;     /* which row labels are butons */
+    Boolean     *column_font_bold;      /* which columns have bold fonts     */
+    Boolean     *column_button_labels;  /* which column labels are buttons   */
+    Boolean     *row_button_labels;     /* which row labels are buttons      */
     Boolean	traverse_fixed;		/* allow traversal to fixed cells?   */
     Boolean	calc_cursor_position;	/* calculate insert pos from click   */
     unsigned char cell_shadow_type;	/* cell shadow type		     */
@@ -275,9 +241,7 @@ typedef struct {
     unsigned char scrollbar_placement;	/* placement of the scrollbars	     */
     unsigned char selection_policy;	/* as for XmList */
     unsigned char shadow_type;		/* matrix window shadow type	     */
-#if XmVersion >= 1002
     unsigned char **highlighted_cells;	/* 2D array of highlighted cells     */
-#endif
     unsigned char vsb_display_policy;	/* vert scroll bar display policy    */
 
     String	**cells;		/* 2D array of strings		     */
@@ -312,6 +276,8 @@ typedef struct {
     Dimension	visible_columns;	/* number of columns to make visible */
     Dimension	visible_rows;		/* number of rows to make visible    */
 
+    Pixel	all_backgrounds;	/* use this for all cell backgrounds */
+    Pixel	all_foregrounds;	/* use this for all cell foregrounds */
     Pixel	button_label_background; /* color of button label background */
     Pixel	**cell_background;	/* 2D array of Pixels		     */
     Pixel	**colors;		/* 2D array of Pixels		     */
@@ -323,6 +289,7 @@ typedef struct {
     Pixel	selected_background;	/* background for selected cells     */
     Pixel	selected_foreground;	/* foreground for selected cells     */
     Pixel	text_background;	/* background for the "text" field   */
+    Pixel	scroll_background;	/* bacground for scrollbar */
 
     XtTranslations text_translations;	/* translations for textField widget */
 
@@ -333,9 +300,7 @@ typedef struct {
     XtCallbackList leave_cell_callback;	/* called when a cell is left	     */
     XtCallbackList modify_verify_callback; /* verify change to textField     */
 					/* and a draw_cell_callback is set   */
-#if XmVersion > 1001
     XtCallbackList process_drag_callback; /* called when a drag is initiated */
-#endif
     XtCallbackList resize_callback;	/* called when Matrix is resized     */
     XtCallbackList resize_column_callback; /* called when column is resized  */
     XtCallbackList select_cell_callback; /* called when cells are selected   */
@@ -350,9 +315,7 @@ typedef struct {
     /*
      * private state
      */
-#if XmVersion >= 1002
     unsigned char highlight_location;	/* What is being highlighted	     */
-#endif
     short	first_row_offset;	/* hidden first row */
     int		cell_visible_height;	/* height of visible cells in pixels */
     int		column_label_maxlines;	/* max # lines in column labels	     */
@@ -417,6 +380,18 @@ typedef struct {
     short	label_font_height;
     Font	label_fid;
 
+    XtIntervalId traverseId;
+    Cursor	cursor;			/* Used to be global static */
+
+/* Implement variable height rows */
+    short	*row_heights;		/* height of each row in chars     */
+    Boolean	allow_row_resize;	/* can rows dynamically resize?   */
+    int		*row_positions;		/* pixel position of each row */
+    XtCallbackList resize_row_callback; /* called when row is resized  */
+    int		*row_max_heights;	/* max height of each row in pixels   */
+
+/* Whether to use XbaeInput widget */
+    Boolean	useXbaeInput;
 } XbaeMatrixPart;
 
 /*
@@ -430,15 +405,19 @@ typedef struct _XbaeMatrixRec {
     XbaeMatrixPart	matrix;
 } XbaeMatrixRec;
 
-/* provide clean-up for those with R4 */
-#ifndef XlibSpecificationRelease
-# undef _Xconst
-# undef _XFUNCPROTOBEGIN
-# undef _XFUNCPROTOEND
-#endif
 
-#undef P
-#if defined(__cplusplus) || defined(c_plusplus)
+int xbaeCellTotalHeight(XbaeMatrixWidget mw);
+
+/*
+ * Macro replacements
+ */
+extern Dimension xbaeMatrixFixedRowHeight(XbaeMatrixWidget mw);
+extern int YtoRow(XbaeMatrixWidget mw, Position y);
+extern Dimension xbaeMatrixTrailingFixedRowHeight(XbaeMatrixWidget);
+extern Widget _XbaeGetShellAncestor(Widget w);
+extern int xbaeMaxVertScroll(XbaeMatrixWidget);
+
+#ifdef __cplusplus
 }
 #endif
 
