@@ -1,6 +1,7 @@
 /*
  * Copyright(c) 1992 Bell Communications Research, Inc. (Bellcore)
  * Copyright(c) 1999 Andrew Lister
+ * Copyright  1999,2000 by the LessTif Developers.
  *
  *                        All rights reserved
  * Permission to use, copy, modify and distribute this material for
@@ -23,7 +24,7 @@
  *
  * CaptionWidget Author: Andrew Wason, Bellcore, aw@bae.bellcore.com
  * 
- * $Id: Caption.c,v 1.1 1999/08/04 19:46:51 danny Exp $
+ * $Id: Caption.c,v 1.9 2001/04/23 20:19:47 amai Exp $
  */
 
 /*
@@ -35,12 +36,18 @@
 #endif
 
 #include <ctype.h>
+#include <string.h>
 #include <X11/StringDefs.h>
 #include <Xm/XmP.h>
 #include <Xm/Label.h>
 #include <Xbae/Macros.h>
 #include <Xbae/CaptionP.h>
 
+#ifdef	WITH_DMALLOC
+#include <dmalloc.h>
+#endif
+
+
 #ifndef tolower
 #define tolower(c)      ((c) - 'A' + 'a')
 #endif
@@ -85,40 +92,40 @@ static XtResource resources[] = {
 /*
  * Declaration of methods
  */
-static void ClassInitialize P((void));
-static void Initialize P((Widget, Widget, ArgList, Cardinal *));
-static void InsertChild P((Widget));
-static Boolean SetValues P((Widget, Widget, Widget, ArgList, Cardinal *));
-static void SetValuesAlmost P((Widget, Widget, XtWidgetGeometry *,
-			       XtWidgetGeometry *));
-static void GetValuesHook P((Widget, ArgList, Cardinal *));
-static void Resize P((Widget));
-static void ChangeManaged P((Widget));
-static XtGeometryResult GeometryManager P((Widget, XtWidgetGeometry *,
-					   XtWidgetGeometry *));
-static XtGeometryResult QueryGeometry P((Widget, XtWidgetGeometry *,
-					 XtWidgetGeometry *));
+static void ClassInitialize(void);
+static void Initialize(Widget req, Widget n, ArgList args, Cardinal *num_args);
+static void InsertChild(Widget);
+static Boolean SetValues(Widget, Widget, Widget, ArgList, Cardinal *);
+static void SetValuesAlmost(Widget, Widget, XtWidgetGeometry *,
+			       XtWidgetGeometry *);
+static void GetValuesHook(Widget, ArgList, Cardinal *);
+static void Resize(Widget);
+static void ChangeManaged(Widget);
+static XtGeometryResult GeometryManager(Widget, XtWidgetGeometry *,
+					   XtWidgetGeometry *);
+static XtGeometryResult QueryGeometry(Widget, XtWidgetGeometry *,
+					 XtWidgetGeometry *);
 
 /*
  * Private functions
  */
-static void ComputeSize P((XbaeCaptionWidget, Dimension *, Dimension *,
-			    Dimension, Dimension, Dimension));
-static void ComputeUserChildSize P((XbaeCaptionWidget, Dimension, Dimension,
-				     Dimension *, Dimension *, Dimension));
+static void ComputeSize(XbaeCaptionWidget, Dimension *, Dimension *,
+			    Dimension, Dimension, Dimension);
+static void ComputeUserChildSize(XbaeCaptionWidget, Dimension, Dimension,
+				     Dimension *, Dimension *, Dimension);
 
-static void Layout P((XbaeCaptionWidget, Boolean));
-static Boolean CompareStrings P((String, String));
+static void Layout(XbaeCaptionWidget, Boolean);
+static Boolean CompareStrings(String, String);
 
 /*
  * Type converters
  */
-static Boolean CvtStringToLabelPosition P((Display *, XrmValuePtr, Cardinal *,
+static Boolean CvtStringToLabelPosition(Display *, XrmValuePtr, Cardinal *,
 					    XrmValuePtr, XrmValuePtr,
-					    XtPointer *));
-static Boolean CvtStringToLabelAlignment P((Display *, XrmValuePtr,
+					    XtPointer *);
+static Boolean CvtStringToLabelAlignment(Display *, XrmValuePtr,
 					     Cardinal *, XrmValuePtr,
-					     XrmValuePtr, XtPointer *));
+					     XrmValuePtr, XtPointer *);
 
 
 XbaeCaptionClassRec xbaeCaptionClassRec = {
@@ -195,7 +202,7 @@ WidgetClass xbaeCaptionWidgetClass = (WidgetClass)&xbaeCaptionClassRec;
 
 
 static void
-ClassInitialize()
+ClassInitialize(void)
 {
     XtSetTypeConverter(XmRString, XmRLabelAlignment,
                        CvtStringToLabelAlignment, NULL, 0,
@@ -208,10 +215,7 @@ ClassInitialize()
 
 /* ARGSUSED */
 static void
-Initialize(req, n, args, num_args)
-Widget req, n;
-ArgList args;
-Cardinal *num_args;
+Initialize(Widget req, Widget n, ArgList args, Cardinal *num_args)
 {
     XbaeCaptionWidget new = (XbaeCaptionWidget)n;
     /*
@@ -286,8 +290,7 @@ Cardinal *num_args;
 }
 
 static void
-InsertChild(w)
-Widget w;
+InsertChild(Widget w)
 {
     if (((CompositeWidget)XtParent(w))->composite.num_children > 1) {
         XtAppWarningMsg(
@@ -306,10 +309,7 @@ Widget w;
 
 /* ARGSUSED */
 static Boolean
-SetValues(cur, req, nw, args, num_args)
-Widget cur, req, nw;
-ArgList args;
-Cardinal *num_args;
+SetValues(Widget cur, Widget req, Widget nw, ArgList args, Cardinal *num_args)
 {
     XbaeCaptionWidget current = (XbaeCaptionWidget)cur;
     XbaeCaptionWidget new = (XbaeCaptionWidget)nw;
@@ -450,11 +450,10 @@ Cardinal *num_args;
 
 /* ARGSUSED */
 static void
-SetValuesAlmost(o, n, request, reply)
-Widget o;
-Widget n;
-XtWidgetGeometry *request;
-XtWidgetGeometry *reply;
+SetValuesAlmost(Widget o,
+		Widget n,
+		XtWidgetGeometry *request,
+		XtWidgetGeometry *reply)
 {
     XbaeCaptionWidget new = (XbaeCaptionWidget)n;
     /*
@@ -478,10 +477,9 @@ XtWidgetGeometry *reply;
 }
 
 static void
-GetValuesHook(w, args, num_args)
-Widget w;
-ArgList args;
-Cardinal *num_args;
+GetValuesHook(Widget w,
+		ArgList args,
+		Cardinal *num_args)
 {
     XbaeCaptionWidget cw = (XbaeCaptionWidget)w;
     int i;
@@ -504,13 +502,7 @@ Cardinal *num_args;
  * the initiating child (the label never initiates)
  */
 static void
-#if NeedFunctionPrototypes
 Layout(XbaeCaptionWidget cw, Boolean configure)
-#else
-Layout(cw, configure)
-XbaeCaptionWidget cw;
-Boolean configure;
-#endif
 {
     Position label_x = 0, label_y = 0;
     Position user_x = 0, user_y = 0;
@@ -638,8 +630,7 @@ Boolean configure;
 }
 
 static void
-Resize(w)
-Widget w;
+Resize(Widget w)
 {
     Layout((XbaeCaptionWidget)w, True);
 }
@@ -649,20 +640,12 @@ Widget w;
  * of the user's child, compute the width and height of the user's child.
  */
 static void
-#if NeedFunctionPrototypes
-ComputeUserChildSize(XbaeCaptionWidget cw, Dimension cwWidth,
-		     Dimension cwHeight, Dimension *childWidth,
-		     Dimension *childHeight, Dimension childBorderWidth)
-#else
-ComputeUserChildSize(cw, cwWidth, cwHeight, childWidth, childHeight,
-		     childBorderWidth)
-XbaeCaptionWidget cw;
-Dimension cwWidth;
-Dimension cwHeight;
-Dimension *childWidth;
-Dimension *childHeight;
-Dimension childBorderWidth;
-#endif
+ComputeUserChildSize(XbaeCaptionWidget cw,
+		     Dimension cwWidth,
+		     Dimension cwHeight,
+		     Dimension *childWidth,
+		     Dimension *childHeight,
+		     Dimension childBorderWidth)
 {
     int width = cwWidth - 2 * childBorderWidth;
     int height = cwHeight - 2 * childBorderWidth;
@@ -709,19 +692,12 @@ Dimension childBorderWidth;
  * current size of the label child)
  */
 static void
-#if NeedFunctionPrototypes
-ComputeSize(XbaeCaptionWidget cw, Dimension *cwWidth, Dimension *cwHeight,
-	    Dimension childWidth, Dimension childHeight,
+ComputeSize(XbaeCaptionWidget cw,
+	    Dimension *cwWidth,
+	    Dimension *cwHeight,
+	    Dimension childWidth,
+	    Dimension childHeight,
 	    Dimension childBorderWidth)
-#else
-ComputeSize(cw, cwWidth, cwHeight, childWidth, childHeight, childBorderWidth)
-XbaeCaptionWidget cw;
-Dimension *cwWidth;
-Dimension *cwHeight;
-Dimension childWidth;
-Dimension childHeight;
-Dimension childBorderWidth;
-#endif
 {
     childWidth += 2 * childBorderWidth;
     childHeight += 2 * childBorderWidth;
@@ -760,8 +736,7 @@ Dimension childBorderWidth;
 }
 
 static void
-ChangeManaged(w)
-Widget w;
+ChangeManaged(Widget w)
 {
     XbaeCaptionWidget cw = (XbaeCaptionWidget)w;
     Dimension width, height;
@@ -801,9 +776,7 @@ Widget w;
 
 /* ARGSUSED */
 static XtGeometryResult
-GeometryManager(w, desired, allowed)
-Widget w;
-XtWidgetGeometry *desired, *allowed;
+GeometryManager(Widget w, XtWidgetGeometry *desired, XtWidgetGeometry *allowed)
 {
     XbaeCaptionWidget cw = (XbaeCaptionWidget) XtParent(w);
     Dimension save_width, save_height, save_border_width;
@@ -947,9 +920,9 @@ XtWidgetGeometry *desired, *allowed;
 }
 
 static XtGeometryResult
-QueryGeometry(w, proposed, desired)
-Widget w;
-XtWidgetGeometry *proposed, *desired;
+QueryGeometry(Widget w,
+		XtWidgetGeometry *proposed,
+		XtWidgetGeometry *desired)
 {
     XbaeCaptionWidget cw = (XbaeCaptionWidget)w;
 #define Set(bit) (proposed->request_mode & bit)
@@ -1086,8 +1059,7 @@ XtWidgetGeometry *proposed, *desired;
  * string is ignored.
  */
 static Boolean
-CompareStrings(in, test)
-String in, test;
+CompareStrings(String in, String test)
 {
     /*
      * Strip leading whitespace off the in string.
@@ -1113,12 +1085,12 @@ String in, test;
 
 /* ARGSUSED */
 static Boolean
-CvtStringToLabelPosition(dpy, args, num_args, from, to, data)
-Display *dpy;
-XrmValuePtr args;
-Cardinal *num_args;
-XrmValuePtr from, to;
-XtPointer *data;
+CvtStringToLabelPosition(Display *dpy,
+			XrmValuePtr args,
+			Cardinal *num_args,
+			XrmValuePtr from,
+			XrmValuePtr to,
+			XtPointer *data)
 {
     static XbaeLabelPosition position;
 
@@ -1165,12 +1137,12 @@ XtPointer *data;
 
 /* ARGSUSED */
 static Boolean
-CvtStringToLabelAlignment(dpy, args, num_args, from, to, data)
-Display *dpy;
-XrmValuePtr args;
-Cardinal *num_args;
-XrmValuePtr from, to;
-XtPointer *data;
+CvtStringToLabelAlignment(Display *dpy,
+			XrmValuePtr args,
+			Cardinal *num_args,
+			XrmValuePtr from,
+			XrmValuePtr to,
+			XtPointer *data)
 {
     static XbaeLabelAlignment alignment;
 
@@ -1218,11 +1190,10 @@ XtPointer *data;
 }
 
 Widget
-XbaeCreateCaption(parent, name, args, ac) 
-Widget parent;
-String name;
-ArgList args;
-Cardinal ac;
+XbaeCreateCaption(Widget parent,
+		String name,
+		ArgList args,
+		Cardinal ac)
 { 
     return XtCreateWidget(name, xbaeCaptionWidgetClass, parent, args, ac); 
 } 
