@@ -1,6 +1,8 @@
 /*
  * Copyright(c) 1992 Bell Communications Research, Inc. (Bellcore)
  * Copyright(c) 1995-99 Andrew Lister
+ * Copyright  1999,2000,2001 by the LessTif Developers.
+ *
  *                        All rights reserved
  * Permission to use, copy, modify and distribute this material for
  * any purpose and without fee is hereby granted, provided that the
@@ -22,7 +24,7 @@
  *
  * MatrixWidget Author: Andrew Wason, Bellcore, aw@bae.bellcore.com
  *
- * $Id: Create.c,v 1.3 1999/12/12 09:54:57 danny Exp $
+ * $Id: Create.c,v 1.15 2001/04/24 18:18:53 amai Exp $
  */
 
 /*
@@ -33,6 +35,8 @@
 #include <XbaeConfig.h>
 #endif
 
+#include <stdlib.h>
+
 #include <Xm/Xm.h>
 #include <Xbae/MatrixP.h>
 #include <Xbae/Macros.h>
@@ -41,36 +45,28 @@
 #include <Xm/ScrollBar.h>
 #include <Xbae/Create.h>
 
-static Pixmap createInsensitivePixmap P((XbaeMatrixWidget mw));
-
-#ifndef MAXSCREENS
-#define MAXSCREENS 1
+#ifdef	WITH_DMALLOC
+#include <dmalloc.h>
 #endif
 
+
+static Pixmap createInsensitivePixmap(XbaeMatrixWidget mw);
+
 void
-xbaeCopyBackground(widget, offset, value)
-Widget widget;
-int offset;
-XrmValue *value;
+xbaeCopyBackground(Widget widget, int offset, XrmValue *value)
 {
     value->addr = (XtPointer)&(widget->core.background_pixel);
 }
 
 
 void
-xbaeCopyForeground(widget, offset, value)
-Widget widget;
-int offset;
-XrmValue *value;
+xbaeCopyForeground(Widget widget, int offset, XrmValue *value)
 {
     value->addr = (XtPointer)&(((XmManagerWidget)widget)->manager.foreground);
 }
 
 void
-xbaeCopyDoubleClick(widget, offset, value)
-Widget widget;
-int offset;
-XrmValue *value;
+xbaeCopyDoubleClick(Widget widget, int offset, XrmValue *value)
 {
     static int interval;
   
@@ -79,13 +75,14 @@ XrmValue *value;
 }
 
 void
-xbaeCopyCells(mw)
-XbaeMatrixWidget mw;
+xbaeCopyCells(XbaeMatrixWidget mw)
 {
     String **copy = NULL;
     int i, j;
     Boolean empty_row;
 
+    xbaeObjectLock((Widget)mw);
+
     if (mw->matrix.rows && mw->matrix.columns)
     {
 	/*
@@ -137,15 +134,18 @@ XbaeMatrixWidget mw;
 	}
     }
     mw->matrix.cells = copy;
+
+    xbaeObjectUnlock((Widget)mw);
 }
 
 void
-xbaeCopyCellWidgets(mw)
-XbaeMatrixWidget mw;
+xbaeCopyCellWidgets(XbaeMatrixWidget mw)
 {
     Widget **copy = NULL;
     int i, j;
 
+    xbaeObjectLock((Widget)mw);
+
     /*
      * Malloc an array of row pointers
      */
@@ -165,15 +165,17 @@ XbaeMatrixWidget mw;
 	}
     }
     mw->matrix.cell_widgets = copy;
+    xbaeObjectUnlock((Widget)mw);
 }
 
 void
-xbaeCopyCellShadowTypes(mw)
-XbaeMatrixWidget mw;
+xbaeCopyCellShadowTypes(XbaeMatrixWidget mw)
 {
     unsigned char **copy = NULL;
     int i, j;
 
+    xbaeObjectLock((Widget)mw);
+
     if (mw->matrix.rows && mw->matrix.columns)
     {
 	copy = (unsigned char **) XtMalloc(mw->matrix.rows *
@@ -201,15 +203,17 @@ XbaeMatrixWidget mw;
 	    }
     }
     mw->matrix.cell_shadow_types = copy;
+    xbaeObjectUnlock((Widget)mw);
 }
 
 void
-xbaeCopyRowShadowTypes(mw)
-XbaeMatrixWidget mw;
+xbaeCopyRowShadowTypes(XbaeMatrixWidget mw)
 {
     unsigned char *copy = NULL;
     int i;
 
+    xbaeObjectLock((Widget)mw);
+
     if (mw->matrix.rows)
     {
 	copy = (unsigned char *) XtMalloc(mw->matrix.rows *
@@ -229,15 +233,17 @@ XbaeMatrixWidget mw;
 		copy[i] = mw->matrix.row_shadow_types[i];
     }
     mw->matrix.row_shadow_types = copy;
+    xbaeObjectUnlock((Widget)mw);
 }
 
 void
-xbaeCopyColumnShadowTypes(mw)
-XbaeMatrixWidget mw;
+xbaeCopyColumnShadowTypes(XbaeMatrixWidget mw)
 {
     unsigned char *copy = NULL;
     int i;
 
+    xbaeObjectLock((Widget)mw);
+
     if (mw->matrix.columns)
     {
 	copy = (unsigned char *) XtMalloc(mw->matrix.columns *
@@ -257,16 +263,18 @@ XbaeMatrixWidget mw;
 		copy[i] = mw->matrix.column_shadow_types[i];
     }
     mw->matrix.column_shadow_types = copy;
+    xbaeObjectUnlock((Widget)mw);
 }
 
 
 void
-xbaeCopyCellUserData(mw)
-XbaeMatrixWidget mw;
+xbaeCopyCellUserData(XbaeMatrixWidget mw)
 {
     XtPointer **copy = NULL;
     int i, j;
 
+    xbaeObjectLock((Widget)mw);
+
     if (mw->matrix.rows && mw->matrix.columns)
     {
 	copy = (XtPointer **) XtMalloc(mw->matrix.rows * sizeof(XtPointer*));
@@ -280,15 +288,18 @@ XbaeMatrixWidget mw;
 		copy[i][j] = mw->matrix.cell_user_data[i][j];
     }
     mw->matrix.cell_user_data = copy;
+
+    xbaeObjectUnlock((Widget)mw);
 }
 
 void
-xbaeCopyRowUserData(mw)
-XbaeMatrixWidget mw;
+xbaeCopyRowUserData(XbaeMatrixWidget mw)
 {
     XtPointer *copy = NULL;
     int i;
 
+    xbaeObjectLock((Widget)mw);
+
     if (mw->matrix.rows)
     {
 	copy = (XtPointer *) XtMalloc(mw->matrix.rows * sizeof(XtPointer));
@@ -297,15 +308,18 @@ XbaeMatrixWidget mw;
 	    copy[i] = mw->matrix.row_user_data[i];
     }
     mw->matrix.row_user_data = copy;
+
+    xbaeObjectUnlock((Widget)mw);
 }
 
 void
-xbaeCopyColumnUserData(mw)
-XbaeMatrixWidget mw;
+xbaeCopyColumnUserData(XbaeMatrixWidget mw)
 {
     XtPointer *copy = NULL;
     int i;
 
+    xbaeObjectLock((Widget)mw);
+
     if (mw->matrix.columns)
     {
 	copy = (XtPointer *) XtMalloc(mw->matrix.columns * sizeof(XtPointer));
@@ -314,16 +328,19 @@ XbaeMatrixWidget mw;
 	    copy[i] = mw->matrix.column_user_data[i];
     }
     mw->matrix.column_user_data = copy;
+
+    xbaeObjectUnlock((Widget)mw);
 }
 
 void
-xbaeCopyRowLabels(mw)
-XbaeMatrixWidget mw;
+xbaeCopyRowLabels(XbaeMatrixWidget mw)
 {
     String *copy = NULL;
     int i;
     Boolean empty_label;
 
+    xbaeObjectLock((Widget)mw);
+
     if (mw->matrix.rows)
     {
 	copy = (String *) XtMalloc(mw->matrix.rows * sizeof(String));
@@ -343,16 +360,19 @@ XbaeMatrixWidget mw;
 		copy[i] = XtNewString(mw->matrix.row_labels[i]);
     }
     mw->matrix.row_labels = copy;
+
+    xbaeObjectUnlock((Widget)mw);
 }
 
 void
-xbaeCopyColumnLabels(mw)
-XbaeMatrixWidget mw;
+xbaeCopyColumnLabels(XbaeMatrixWidget mw)
 {
     String *copy = NULL;
     int i;
     Boolean empty_column;
     
+    xbaeObjectLock((Widget)mw);
+
     if (mw->matrix.columns)
     {
 	copy = (String *) XtMalloc(mw->matrix.columns * sizeof(String));
@@ -393,16 +413,19 @@ XbaeMatrixWidget mw;
 		    mw->matrix.column_label_lines[i].lines;
     }
     mw->matrix.column_labels = copy;
+
+    xbaeObjectUnlock((Widget)mw);
 }
 
 void
-xbaeCopyColumnWidths(mw)
-XbaeMatrixWidget mw;
+xbaeCopyColumnWidths(XbaeMatrixWidget mw)
 {
     short *copy = NULL;
     int i;
     Boolean bad = False;
 
+    xbaeObjectLock((Widget)mw);
+
     if (mw->matrix.columns)
     {
 	copy = (short *) XtMalloc(mw->matrix.columns * sizeof(short));
@@ -426,16 +449,55 @@ XbaeMatrixWidget mw;
 	}
     }
     mw->matrix.column_widths = copy;
+
+    xbaeObjectUnlock((Widget)mw);
 }
 
 void
-xbaeCopyColumnMaxLengths(mw)
-XbaeMatrixWidget mw;
+xbaeCopyRowHeights(XbaeMatrixWidget mw)
+{
+    short *copy = NULL;
+    int i;
+    Boolean bad = False;
+
+    xbaeObjectLock((Widget)mw);
+
+    if (mw->matrix.rows)
+    {
+	copy = (short *) XtMalloc(mw->matrix.rows * sizeof(short));
+
+	for (i = 0; i < mw->matrix.rows; i++)
+	{
+	    if (!bad && mw->matrix.row_heights[i] == BAD_HEIGHT)
+	    {
+		bad = True;
+		XtAppWarningMsg(
+		    XtWidgetToApplicationContext((Widget) mw),
+		    "copyRowHeights", "tooShort", "XbaeMatrix",
+		    "XbaeMatrix: Row heights array is too short",
+		    NULL, 0);
+		copy[i] = 1;
+	    }
+	    else if (bad)
+		copy[i] = 1;
+	    else
+		copy[i] = mw->matrix.row_heights[i];
+	}
+    }
+    mw->matrix.row_heights = copy;
+
+    xbaeObjectUnlock((Widget)mw);
+}
+
+void
+xbaeCopyColumnMaxLengths(XbaeMatrixWidget mw)
 {
     int *copy = NULL;
     int i;
     Boolean bad = False;
 
+    xbaeObjectLock((Widget)mw);
+
     if (mw->matrix.columns)
     {
 	copy = (int *) XtMalloc(mw->matrix.columns * sizeof(int));
@@ -459,16 +521,19 @@ XbaeMatrixWidget mw;
 	}
     }
     mw->matrix.column_max_lengths = copy;
+
+    xbaeObjectUnlock((Widget)mw);
 }
 
 void
-xbaeCopyColumnAlignments(mw)
-XbaeMatrixWidget mw;
+xbaeCopyColumnAlignments(XbaeMatrixWidget mw)
 {
     unsigned char *copy = NULL;
     int i;
     Boolean bad = False;
 
+    xbaeObjectLock((Widget)mw);
+
     if (mw->matrix.columns)
     {
 	copy = (unsigned char *) XtMalloc(mw->matrix.columns *
@@ -493,15 +558,18 @@ XbaeMatrixWidget mw;
 	}
     }
     mw->matrix.column_alignments = copy;
+
+    xbaeObjectUnlock((Widget)mw);
 }
 
 void
-xbaeCopyColumnButtonLabels(mw)
-XbaeMatrixWidget mw;
+xbaeCopyColumnButtonLabels(XbaeMatrixWidget mw)
 {
     Boolean *copy = NULL;
     int i;
 
+    xbaeObjectLock((Widget)mw);
+
     if (mw->matrix.columns)
     {
 	copy = (Boolean *) XtMalloc(mw->matrix.columns *
@@ -513,15 +581,41 @@ XbaeMatrixWidget mw;
 	}
     }
     mw->matrix.column_button_labels = copy;
+
+    xbaeObjectUnlock((Widget)mw);
 }
 
 void
-xbaeCopyRowButtonLabels(mw)
-XbaeMatrixWidget mw;
+xbaeCopyColumnFontBold(XbaeMatrixWidget mw)
 {
     Boolean *copy = NULL;
     int i;
 
+    xbaeObjectLock((Widget)mw);
+
+    if (mw->matrix.columns)
+    {
+      copy = (Boolean *) XtMalloc(mw->matrix.columns *
+                                  sizeof(Boolean));
+
+      for (i = 0; i < mw->matrix.columns; i++)
+      {
+          copy[i] = mw->matrix.column_font_bold[i];
+      }
+    }
+    mw->matrix.column_font_bold = copy;
+
+    xbaeObjectUnlock((Widget)mw);
+}
+
+void
+xbaeCopyRowButtonLabels(XbaeMatrixWidget mw)
+{
+    Boolean *copy = NULL;
+    int i;
+
+    xbaeObjectLock((Widget)mw);
+
     if (mw->matrix.rows)
     {
 	copy = (Boolean *) XtMalloc(mw->matrix.rows *
@@ -533,16 +627,19 @@ XbaeMatrixWidget mw;
 	}
     }
     mw->matrix.row_button_labels = copy;
+
+    xbaeObjectUnlock((Widget)mw);
 }
 
 void
-xbaeCopyColumnLabelAlignments(mw)
-XbaeMatrixWidget mw;
+xbaeCopyColumnLabelAlignments(XbaeMatrixWidget mw)
 {
     unsigned char *copy = NULL;
     int i;
     Boolean bad = False;
 
+    xbaeObjectLock((Widget)mw);
+
     if (mw->matrix.columns)
     {
 	copy = (unsigned char *) XtMalloc(mw->matrix.columns *
@@ -569,17 +666,19 @@ XbaeMatrixWidget mw;
 	}
     }
     mw->matrix.column_label_alignments = copy;
+
+    xbaeObjectUnlock((Widget)mw);
 }
 
 void
-xbaeCopyColors(mw)
-XbaeMatrixWidget mw;
+xbaeCopyColors(XbaeMatrixWidget mw)
 {
     Pixel **copy = NULL;
     int i, j;
     Boolean badrow = False;
     Boolean badcol;
 
+    xbaeObjectLock((Widget)mw);
 
     if (mw->matrix.rows && mw->matrix.columns)
     {
@@ -632,17 +731,20 @@ XbaeMatrixWidget mw;
 	}
     }
     mw->matrix.colors = copy;
+
+    xbaeObjectUnlock((Widget)mw);
 }
 
 void
-xbaeCopyBackgrounds(mw)
-XbaeMatrixWidget mw;
+xbaeCopyBackgrounds(XbaeMatrixWidget mw)
 {
     Pixel **copy = NULL;
     int i, j;
     Boolean badrow = False;
     Boolean badcol;
 
+    xbaeObjectLock((Widget)mw);
+
     if (mw->matrix.rows && mw->matrix.columns)
     {
 	/*
@@ -708,6 +810,8 @@ XbaeMatrixWidget mw;
 	}
     }
     mw->matrix.cell_background = copy;
+
+    xbaeObjectUnlock((Widget)mw);
 }
 
 /*
@@ -715,19 +819,24 @@ XbaeMatrixWidget mw;
  * represent selected cells if it is NULL.
  */
 void
-xbaeCopySelectedCells(mw)
-XbaeMatrixWidget mw;
+xbaeCopySelectedCells(XbaeMatrixWidget mw)
 {
     Boolean **copy = NULL;
     int i, j;
 
+    xbaeObjectLock((Widget)mw);
+
     if (mw->matrix.rows && mw->matrix.columns)
     {
 	/*
 	 * Malloc an array of row pointers
 	 */
 	mw->matrix.num_selected_cells = 0;
+#if 0
 	copy = (Boolean **) XtMalloc(mw->matrix.rows * sizeof(Boolean *));
+#else
+	copy = (Boolean **) XtCalloc(mw->matrix.rows, sizeof(Boolean *));
+#endif
 
 	/*
 	 * Malloc an array of Booleans for each row pointer
@@ -749,20 +858,23 @@ XbaeMatrixWidget mw;
 		}
     }
     mw->matrix.selected_cells = copy;
+
+    xbaeObjectUnlock((Widget)mw);
 }
 
-#if XmVersion >= 1002
+
 /*
  * Copy the highlightedCells resource. Create a 2D array of Booleans to
  * represent highlighted cells if it is NULL.
  */
 void
-xbaeCopyHighlightedCells(mw)
-XbaeMatrixWidget mw;
+xbaeCopyHighlightedCells(XbaeMatrixWidget mw)
 {
     unsigned char **copy = NULL;
     int i, j;
 
+    xbaeObjectLock((Widget)mw);
+
     if (mw->matrix.rows && mw->matrix.columns)
     {
 	/*
@@ -787,18 +899,21 @@ XbaeMatrixWidget mw;
 		    copy[i][j] = mw->matrix.highlighted_cells[i][j];
     }
     mw->matrix.highlighted_cells = copy;
+
+    xbaeObjectUnlock((Widget)mw);
 }
-#endif
+
 
 /*
  * Create a matrix of Pixels
  */
 void
-xbaeCreateColors(mw)
-XbaeMatrixWidget mw;
+xbaeCreateColors(XbaeMatrixWidget mw)
 {
     int i;
 
+    xbaeObjectLock((Widget)mw);
+
     if (mw->matrix.rows && mw->matrix.columns)
     {
 	/*
@@ -816,6 +931,8 @@ XbaeMatrixWidget mw;
     }
     else
 	mw->matrix.colors = NULL;
+
+    xbaeObjectUnlock((Widget)mw);
 }
 
 /*
@@ -823,40 +940,47 @@ XbaeMatrixWidget mw;
  * XmNsensitive is set to False
  */
 static Pixmap
-createInsensitivePixmap(mw)
-XbaeMatrixWidget mw;
+createInsensitivePixmap(XbaeMatrixWidget mw)
 {
     static char stippleBits[] = { 0x01, 0x02 };
-    static Pixmap stipple[MAXSCREENS] = {(Pixmap)NULL};
+    static Pixmap *stipple = NULL;
     Display *dpy = XtDisplay(mw);
     Screen *scr  = XtScreen (mw);
     int i;
     int maxScreens = ScreenCount(dpy);
     
-    if (maxScreens > MAXSCREENS)
-	maxScreens = MAXSCREENS;
-    
-    if (!stipple[0])
+    xbaeObjectLock((Widget)mw);
+
+    if (!stipple)
     {
+	stipple = malloc(maxScreens*sizeof(Pixmap));
+	if (!stipple) {
+            return (Pixmap) NULL;
+	}
 	for (i = 0 ; i < maxScreens ; i++)
 	    stipple[i] = XCreatePixmapFromBitmapData(
 		dpy, RootWindow(dpy,i), stippleBits, 2, 2, 0, 1, 1);
     }
     for (i = 0; i < maxScreens; i++)
     {
-	if (ScreenOfDisplay(dpy, i) == scr)
+	if (ScreenOfDisplay(dpy, i) == scr) {
+	    xbaeObjectUnlock((Widget)mw);
 	    return stipple[i];
+	}
     }
+
+    xbaeObjectUnlock((Widget)mw);
     return (Pixmap)NULL;
 }
     
 void
-xbaeCreateGridLineGC(mw)
-XbaeMatrixWidget mw;
+xbaeCreateGridLineGC(XbaeMatrixWidget mw)
 {
     XGCValues values;
     XtGCMask mask = GCForeground | GCBackground;
 
+    xbaeObjectLock((Widget)mw);
+
     values.foreground = mw->matrix.grid_line_color;
     values.background = mw->manager.foreground;
     
@@ -871,15 +995,18 @@ XbaeMatrixWidget mw;
     mw->matrix.cell_grid_line_gc = XCreateGC(XtDisplay(mw),
 					     GC_PARENT_WINDOW(mw),
 					     mask, &values);
+
+    xbaeObjectUnlock((Widget)mw);
 }
 
 void
-xbaeCreateDrawGC(mw)
-XbaeMatrixWidget mw;
+xbaeCreateDrawGC(XbaeMatrixWidget mw)
 {
     XGCValues values;
     unsigned long mask = GCForeground | GCFont | GCStipple;
 
+    xbaeObjectLock((Widget)mw);
+
     /*
      * GC for drawing cells. We create it instead of using a cached one,
      * since the foreground may change frequently.
@@ -891,15 +1018,18 @@ XbaeMatrixWidget mw;
     mw->matrix.draw_gc = XCreateGC(XtDisplay(mw),
 				   GC_PARENT_WINDOW(mw),
 				   mask, &values);
+
+    xbaeObjectUnlock((Widget)mw);
 }
 
 void
-xbaeCreatePixmapGC(mw)
-XbaeMatrixWidget mw;
+xbaeCreatePixmapGC(XbaeMatrixWidget mw)
 {
     XGCValues values;
     unsigned long mask = GCForeground | GCGraphicsExposures | GCStipple;
 
+    xbaeObjectLock((Widget)mw);
+
     values.foreground = mw->manager.foreground;
     values.graphics_exposures = False;
     values.stipple = createInsensitivePixmap(mw);
@@ -907,15 +1037,18 @@ XbaeMatrixWidget mw;
     mw->matrix.pixmap_gc = XCreateGC(XtDisplay(mw),
 				     GC_PARENT_WINDOW(mw),
 				     mask, &values);
+
+    xbaeObjectUnlock((Widget)mw);
 }
 
 void
-xbaeCreateLabelGC(mw)
-XbaeMatrixWidget mw;
+xbaeCreateLabelGC(XbaeMatrixWidget mw)
 {
     XGCValues values;
     unsigned long mask = GCForeground | GCFont | GCStipple;
 
+    xbaeObjectLock((Widget)mw);
+
     /*
      * GC for drawing labels
      */
@@ -925,15 +1058,18 @@ XbaeMatrixWidget mw;
     mw->matrix.label_gc = XCreateGC(XtDisplay(mw),
 				    GC_PARENT_WINDOW(mw),
 				    mask, &values);
+
+    xbaeObjectUnlock((Widget)mw);
 }
 
 void
-xbaeCreateLabelClipGC(mw)
-XbaeMatrixWidget mw;
+xbaeCreateLabelClipGC(XbaeMatrixWidget mw)
 {
     XGCValues values;
     unsigned long mask = GCForeground | GCFont | GCStipple;
 
+    xbaeObjectLock((Widget)mw);
+
     /*
      * GC for drawing labels with clipping.
      */
@@ -943,15 +1079,18 @@ XbaeMatrixWidget mw;
     mw->matrix.label_clip_gc = XCreateGC(XtDisplay(mw),
 					 GC_PARENT_WINDOW(mw),
 					 mask, &values);
+
+    xbaeObjectUnlock((Widget)mw);
 }
 
 void
-xbaeCreateTopShadowClipGC(mw)
-XbaeMatrixWidget mw;
+xbaeCreateTopShadowClipGC(XbaeMatrixWidget mw)
 {
     XGCValues values;
     XtGCMask mask = GCForeground | GCBackground;
 
+    xbaeObjectLock((Widget)mw);
+
     /*
      * GC for drawing top shadow inside cells with clipping.
      */
@@ -971,15 +1110,18 @@ XbaeMatrixWidget mw;
     values.function = GXxor;
     mw->matrix.resize_top_shadow_gc = XtGetGC(
 	(Widget) mw, mask, &values);
+
+    xbaeObjectUnlock((Widget)mw);
 }
 
 void
-xbaeCreateBottomShadowClipGC(mw)
-XbaeMatrixWidget mw;
+xbaeCreateBottomShadowClipGC(XbaeMatrixWidget mw)
 {
     XGCValues values;
     XtGCMask mask = GCForeground | GCBackground;
 
+    xbaeObjectLock((Widget)mw);
+
     /*
      * GC for drawing bottom shadow inside cells with clipping.
      */
@@ -999,11 +1141,12 @@ XbaeMatrixWidget mw;
     values.function = GXxor;
     mw->matrix.resize_bottom_shadow_gc = XtGetGC(
 	(Widget) mw, mask, &values);
+
+    xbaeObjectUnlock((Widget)mw);
 }
 
 void
-xbaeNewFont(mw)
-XbaeMatrixWidget mw;
+xbaeNewFont(XbaeMatrixWidget mw)
 {
     XmFontContext context;
     XFontStruct *font;
@@ -1013,6 +1156,8 @@ XbaeMatrixWidget mw;
     XFontStruct **fonts;
     char **font_names;
 
+    xbaeObjectLock((Widget)mw);
+
     /*
      * Make a private copy of the FontList
      */
@@ -1063,11 +1208,12 @@ XbaeMatrixWidget mw;
     }
 
     XmFontListFreeFontContext(context);
+
+    xbaeObjectUnlock((Widget)mw);
 }
 
 void
-xbaeNewLabelFont(mw)
-XbaeMatrixWidget mw;
+xbaeNewLabelFont(XbaeMatrixWidget mw)
 {
     XmFontContext context;
     XFontStruct *font;
@@ -1077,6 +1223,8 @@ XbaeMatrixWidget mw;
     XFontStruct **fonts;
     char **font_names;
 
+    xbaeObjectLock((Widget)mw);
+
     /*
      * Make a private copy of the FontList
      */
@@ -1127,43 +1275,49 @@ XbaeMatrixWidget mw;
     }
 
     XmFontListFreeFontContext(context);
+
+    xbaeObjectUnlock((Widget)mw);
 }
 
 void
-xbaeFreeCells(mw)
-XbaeMatrixWidget mw;
+xbaeFreeCells(XbaeMatrixWidget mw)
 {
     int i, j;
 
     if (!mw->matrix.cells)
 	return;
 
+    xbaeObjectLock((Widget)mw);
+
     /*
      * Free each cell in a row, then free the row and go to the next one
      */
     for (i = 0; i < mw->matrix.rows; i++)
     {
 	for (j = 0; j < mw->matrix.columns; j++)
-	    XtFree((XtPointer) mw->matrix.cells[i][j]);
-	XtFree((XtPointer) mw->matrix.cells[i]);
+	    XtFree((char *) mw->matrix.cells[i][j]);
+	XtFree((char *) mw->matrix.cells[i]);
     }
 
     /*
      * Free the array of row pointers
      */
-    XtFree((XtPointer) mw->matrix.cells);
+    XtFree((char *) mw->matrix.cells);
     mw->matrix.cells = NULL;
+
+    xbaeObjectUnlock((Widget)mw);
 }
 
 void
-xbaeFreeCellWidgets(mw)
-XbaeMatrixWidget mw;
+xbaeFreeCellWidgets(XbaeMatrixWidget mw)
 {
     int i, j;
 
     if (!mw->matrix.cell_widgets)
 	return;
 
+    xbaeObjectLock((Widget)mw);
+
     /*
      * Destroy each cell widget (don't XtFree ! as Linas suggests),
      * then free the row and go to the next one.
@@ -1173,100 +1327,113 @@ XbaeMatrixWidget mw;
 	for (j = 0; j < mw->matrix.columns; j++)
 	    if (mw->matrix.cell_widgets[i][j])
 		XtDestroyWidget(mw->matrix.cell_widgets[i][j]);
-	XtFree((XtPointer) mw->matrix.cell_widgets[i]);
+	XtFree((char *) mw->matrix.cell_widgets[i]);
     }
 
     /*
      * Free the array of row pointers
      */
-    XtFree((XtPointer) mw->matrix.cell_widgets);
+    XtFree((char *) mw->matrix.cell_widgets);
     mw->matrix.cell_widgets = NULL;
+
+    xbaeObjectUnlock((Widget)mw);
 }
 
 void
-xbaeFreeRowLabels(mw)
-XbaeMatrixWidget mw;
+xbaeFreeRowLabels(XbaeMatrixWidget mw)
 {
     int i;
 
     if (!mw->matrix.row_labels)
 	return;
 
-    for (i = 0; i < mw->matrix.rows; i++)
-	XtFree((XtPointer) mw->matrix.row_labels[i]);
+    xbaeObjectLock((Widget)mw);
 
-    XtFree((XtPointer) mw->matrix.row_labels);
+    for (i = 0; i < mw->matrix.rows; i++)
+	XtFree((char *) mw->matrix.row_labels[i]);
+
+    XtFree((char *) mw->matrix.row_labels);
     mw->matrix.row_labels = NULL;
+
+    xbaeObjectUnlock((Widget)mw);
 }
 
 void
-xbaeFreeColumnLabels(mw)
-XbaeMatrixWidget mw;
+xbaeFreeColumnLabels(XbaeMatrixWidget mw)
 {
     int i;
 
     if (!mw->matrix.column_labels)
 	return;
 
+    xbaeObjectLock((Widget)mw);
+
     for (i = 0; i < mw->matrix.columns; i++)
     {
-	XtFree((XtPointer) mw->matrix.column_labels[i]);
-	XtFree((XtPointer) mw->matrix.column_label_lines[i].lengths);
+	XtFree((char *) mw->matrix.column_labels[i]);
+	XtFree((char *) mw->matrix.column_label_lines[i].lengths);
     }
 
-    XtFree((XtPointer) mw->matrix.column_label_lines);
-    XtFree((XtPointer) mw->matrix.column_labels);
+    XtFree((char *) mw->matrix.column_label_lines);
+    XtFree((char *) mw->matrix.column_labels);
     mw->matrix.column_labels = NULL;
+
+    xbaeObjectUnlock((Widget)mw);
 }
 
 
 void
-xbaeFreeColors(mw)
-XbaeMatrixWidget mw;
+xbaeFreeColors(XbaeMatrixWidget mw)
 {
     int i;
 
     if (!mw->matrix.colors)
 	return;
 
+    xbaeObjectLock((Widget)mw);
+
     /*
      * Free each row of Pixels
      */
     for (i = 0; i < mw->matrix.rows; i++)
-	XtFree((XtPointer) mw->matrix.colors[i]);
+	XtFree((char *) mw->matrix.colors[i]);
 
     /*
      * Free the array of row pointers
      */
-    XtFree((XtPointer) mw->matrix.colors);
+    XtFree((char *) mw->matrix.colors);
     mw->matrix.colors = NULL;
+
+    xbaeObjectUnlock((Widget)mw);
 }
 
 void
-xbaeFreeBackgrounds(mw)
-XbaeMatrixWidget mw;
+xbaeFreeBackgrounds(XbaeMatrixWidget mw)
 {
     int i;
 
     if (!mw->matrix.cell_background)
 	return;
 
+    xbaeObjectLock((Widget)mw);
+
     /*
      * Free each row of Pixels
      */
     for (i = 0; i < mw->matrix.rows; i++)
-	XtFree((XtPointer) mw->matrix.cell_background[i]);
+	XtFree((char *) mw->matrix.cell_background[i]);
 
     /*
      * Free the array of row pointers
      */
-    XtFree((XtPointer) mw->matrix.cell_background);
+    XtFree((char *) mw->matrix.cell_background);
     mw->matrix.cell_background = NULL;
+
+    xbaeObjectUnlock((Widget)mw);
 }
 
 void
-xbaeFreeSelectedCells(mw)
-XbaeMatrixWidget mw;
+xbaeFreeSelectedCells(XbaeMatrixWidget mw)
 {
     int i;
 
@@ -1276,44 +1443,52 @@ XbaeMatrixWidget mw;
     if (!mw->matrix.selected_cells)
 	return;
 
+    xbaeObjectLock((Widget)mw);
+
     for (i = 0; i < mw->matrix.rows; i++)
-	XtFree((XtPointer) mw->matrix.selected_cells[i]);
+	XtFree((char *) mw->matrix.selected_cells[i]);
 
     /*
      * Free the array of row pointers
      */
-    XtFree((XtPointer) mw->matrix.selected_cells);
+    XtFree((char *) mw->matrix.selected_cells);
     mw->matrix.selected_cells = NULL;
+
+    xbaeObjectUnlock((Widget)mw);
 }
 
-#if XmVersion >= 1002
+
 void
-xbaeFreeHighlightedCells(mw)
-XbaeMatrixWidget mw;
+xbaeFreeHighlightedCells(XbaeMatrixWidget mw)
 {
     int i;
 
     if (!mw->matrix.highlighted_cells)
 	return;
     
+    xbaeObjectLock((Widget)mw);
+
     /*
      * Free each row of XtPointer pointers
      */
     for (i = 0; i < mw->matrix.rows; i++)
-	XtFree((XtPointer) mw->matrix.highlighted_cells[i]);
+	XtFree((char *) mw->matrix.highlighted_cells[i]);
 
     /*
      * Free the array of row pointers
      */
-    XtFree((XtPointer) mw->matrix.highlighted_cells);
+    XtFree((char *) mw->matrix.highlighted_cells);
     mw->matrix.highlighted_cells = NULL;
+
+    xbaeObjectUnlock((Widget)mw);
 }
-#endif
+
 
 void
-xbaeFreeCellUserData(mw)
-XbaeMatrixWidget mw;
+xbaeFreeCellUserData(XbaeMatrixWidget mw)
 {
+    xbaeObjectLock((Widget)mw);
+
     if (mw->matrix.cell_user_data)
     {
 	int i;
@@ -1322,21 +1497,24 @@ XbaeMatrixWidget mw;
 	 * Free each row of Booleans
 	 */
 	for (i = 0; i < mw->matrix.rows; i++)
-	    XtFree((XtPointer) mw->matrix.cell_user_data[i]);
+	    XtFree((char *) mw->matrix.cell_user_data[i]);
 
 	/*
 	 * Free the array of row pointers
 	 */
-	XtFree((XtPointer) mw->matrix.cell_user_data);
+	XtFree((char *) mw->matrix.cell_user_data);
     }
     mw->matrix.cell_user_data = NULL;
+
+    xbaeObjectUnlock((Widget)mw);
 }
 
 
 void
-xbaeFreeCellShadowTypes(mw)
-XbaeMatrixWidget mw;
+xbaeFreeCellShadowTypes(XbaeMatrixWidget mw)
 {
+    xbaeObjectLock((Widget)mw);
+
     if (mw->matrix.cell_shadow_types)
     {
 	int i;
@@ -1345,13 +1523,14 @@ XbaeMatrixWidget mw;
 	 * Free each row of unsigned char pointers
 	 */
 	for (i = 0; i < mw->matrix.rows; i++)
-	    XtFree((XtPointer) mw->matrix.cell_shadow_types[i]);
+	    XtFree((char *) mw->matrix.cell_shadow_types[i]);
 
 	/*
 	 * Free the array of row pointers
 	 */
-	XtFree((XtPointer) mw->matrix.cell_shadow_types);
+	XtFree((char *) mw->matrix.cell_shadow_types);
     }
     mw->matrix.cell_shadow_types = NULL;
-}
 
+    xbaeObjectUnlock((Widget)mw);
+}
