@@ -1,6 +1,8 @@
 /*
  * Copyright(c) 1992 Bell Communications Research, Inc. (Bellcore)
  * Copyright(c) 1995-99 Andrew Lister
+ * Copyright  1999,2000,2001 by the LessTif Developers.
+ *
  *                        All rights reserved
  * Permission to use, copy, modify and distribute this material for
  * any purpose and without fee is hereby granted, provided that the
@@ -20,55 +22,67 @@
  * LOST PROFITS OR OTHER INCIDENTAL OR CONSEQUENTIAL DAMAGES RELAT-
  * ING TO THE SOFTWARE.
  *
- * $Id: Draw.c,v 1.5 1999/12/12 09:54:56 danny Exp $
+ * $Id: Draw.c,v 1.25 2001/04/24 18:18:53 amai Exp $
  */
 
 #ifdef HAVE_CONFIG_H
 #include <XbaeConfig.h>
 #endif
 
+#include <stdlib.h>
+#include <stdio.h>
+#include <string.h>
+
 #include <Xm/Xm.h>
 #include <Xm/XmP.h>
-#if XmVersion > 1001
 #include <Xm/DrawP.h>
-#endif
 #include <Xbae/MatrixP.h>
 #include <Xbae/Utils.h>
 #include <Xbae/Shadow.h>
 #include <Xbae/Draw.h>
-#include <stdlib.h>
 
-static void xbaeDrawCellString P((XbaeMatrixWidget, int, int, int, int,
-				  String, Pixel, Pixel));
-static void xbaeDrawCellWidget P((XbaeMatrixWidget, int, int, int, int,
-				  Widget, Pixel, Pixel));
-static void xbaeDrawCellPixmap P((XbaeMatrixWidget, int, int, int, int,
+
+#ifdef	WITH_DMALLOC
+#include <dmalloc.h>
+#endif
+
+
+static void xbaeDrawCellString(XbaeMatrixWidget, int, int, int, int,
+				  String, Pixel, Pixel);
+static void xbaeDrawCellWidget(XbaeMatrixWidget, int, int, int, int,
+				  Widget, Pixel, Pixel);
+static void xbaeDrawCellPixmap(XbaeMatrixWidget, int, int, int, int,
 				  Pixmap, Pixmap, int, int, Pixel,
-				  Pixel, int));
+				  Pixel, int);
+static void xbaeDrawString(XbaeMatrixWidget mw, Window win, GC gc,
+			String string, int length,
+			int x, int y,
+			int maxlen, int maxht,
+			unsigned char alignment, Boolean highlight,
+			Boolean bold, Boolean rowLabel,
+			Boolean colLabel, Pixel color);
 
 /*
  * Draw a fixed or non-fixed cell. The coordinates are calculated relative
  * to the correct window and pixmap is copied to that window.
  */
 static void
-xbaeDrawCellPixmap(mw, row, column, x, y, pixmap, mask, width, height, bg,
-		   fg, depth)
-XbaeMatrixWidget mw;
-int row;
-int column;
-int x;
-int y;
-Pixmap pixmap;
-Pixmap mask;
-int width;
-int height;
-Pixel bg;
-Pixel fg;
-int depth;
+xbaeDrawCellPixmap(XbaeMatrixWidget mw,
+		int row,
+		int column,
+		int x,
+		int y,
+		Pixmap pixmap,
+		Pixmap mask,
+		int width,
+		int height,
+		Pixel bg,
+		Pixel fg,
+		int depth)
 {
     int src_x = 0, src_y, dest_x, dest_y;
     int copy_width, copy_height;
-    int cell_height = ROW_HEIGHT(mw);
+    int cell_height = SOME_ROW_HEIGHT(mw, row);
     int cell_width = COLUMN_WIDTH(mw, column);
     Widget w;
     unsigned char alignment = mw->matrix.column_alignments ?
@@ -90,7 +104,6 @@ int depth;
 
     XSetForeground(display, gc, bg);
 
-#if XmVersion >= 1002
     /*
      * If we are only changing the highlighting of a cell, we don't need
      * to do anything other than draw (or undraw) the highlight
@@ -102,10 +115,9 @@ int depth;
 			      cell_height, mw->matrix.highlight_location);
 	return;
     }
-#endif
 
     XFillRectangle(display, win, gc, x, y,
-		   COLUMN_WIDTH(mw, column), ROW_HEIGHT(mw));
+		   COLUMN_WIDTH(mw, column), cell_height);
 
     XSetForeground(display, gc, fg);
     XSetBackground(display, gc, bg);
@@ -215,14 +227,12 @@ int depth;
 		       ax, ay, fill_width, fill_height);
     }
 
-#if XmVersion >= 1002
     if (mw->matrix.highlighted_cells &&
 	mw->matrix.highlighted_cells[row][column])
     {
 	xbaeDrawCellHighlight(mw, win, gc, row, column, x, y, cell_width,
 			      cell_height, HIGHLIGHTING_SOMETHING);
     }
-#endif
     xbaeDrawCellShadow(mw, win, row, column, x, y, cell_width,
 		       cell_height, False, False, False);
 }
@@ -232,26 +242,32 @@ int depth;
  * to the correct window and the cell is drawn in that window.
  */
 static void
-xbaeDrawCellString(mw, row, column, x, y, string, bg, fg)
-XbaeMatrixWidget mw;
-int row, column;
-String string;
-Pixel bg, fg;
+xbaeDrawCellString(XbaeMatrixWidget mw,
+		int row,
+		int column,
+		int x,
+		int y,
+		String string,
+		Pixel bg,
+		Pixel fg)
 {
     GC gc;
     Widget w;
     Window win = xbaeGetCellWindow(mw, &w, row, column);
     Dimension column_width = COLUMN_WIDTH(mw, column);
-    Dimension row_height = ROW_HEIGHT(mw);
+    Dimension row_height = SOME_ROW_HEIGHT(mw, row);
     Dimension width = column_width;
     Dimension height = row_height;
     Boolean selected = mw->matrix.selected_cells ?
 	mw->matrix.selected_cells[row][column] : False;
     String str = string;
-    
+
     if (!win)
 	return;
 
+    if (row_height == 0)
+	return;		/* Quick fix */
+
 #if 0
     /*
      * Probably not needed - time will tell!  If anybody gets a segv on
@@ -323,44 +339,46 @@ Pixel bg, fg;
     /*
      * Draw the string in the cell.
      */
-
     xbaeDrawString(mw, win, gc, str, strlen(str),
-		   x + TEXT_X_OFFSET(mw), y + TEXT_Y_OFFSET(mw),
-		   mw->matrix.column_widths[column],
-		   mw->matrix.column_alignments ?
-		   mw->matrix.column_alignments[column] :
-		   XmALIGNMENT_BEGINNING, selected,
-		   False, False, False, fg);
+		x + TEXT_X_OFFSET(mw), y + TEXT_Y_OFFSET(mw),
+		mw->matrix.column_widths[column],
+		row_height,
+		mw->matrix.column_alignments ?
+		mw->matrix.column_alignments[column] :
+		XmALIGNMENT_BEGINNING, selected,
+		mw->matrix.column_font_bold ? mw->matrix.column_font_bold[column] : 0,
+		False, False, fg);
 
-#if XmVersion >= 1002
     if (mw->matrix.highlighted_cells &&
 	mw->matrix.highlighted_cells[row][column])
     {
 	xbaeDrawCellHighlight(mw, win, gc, row, column, x, y, width, height,
 			      HIGHLIGHTING_SOMETHING);
     }
-#endif
 
     xbaeDrawCellShadow(mw, win, row, column, x, y, COLUMN_WIDTH(mw, column),
-		       ROW_HEIGHT(mw), False, False, False);
+		       SOME_ROW_HEIGHT(mw, row), False, False, False);
 }
 
 /*
  * Draw a user defined widget in the cell
  */
 static void
-xbaeDrawCellWidget(mw, row, column, x, y, widget, bg, fg)
-XbaeMatrixWidget mw;
-int row, column;
-int x, y;
-Widget widget;
-Pixel bg, fg;
+xbaeDrawCellWidget(XbaeMatrixWidget mw,
+		int row,
+		int column,
+		int x,
+		int y,
+		Widget widget,
+		Pixel bg,
+		Pixel fg)
 {
     GC gc;
     Widget w;
     Window win = xbaeGetCellWindow(mw, &w, row, column);
 
     Boolean clipped = False;	/* To get 4.7 to compile. */
+    int	r = SOME_ROW_HEIGHT(mw, row);
 
     if (!win)
 	return;
@@ -369,8 +387,7 @@ Pixel bg, fg;
 
     gc = mw->matrix.draw_gc;
     XSetForeground(XtDisplay(mw), gc, bg);
-    XFillRectangle(XtDisplay(mw), win, gc, x, y,
-		   COLUMN_WIDTH(mw, column), ROW_HEIGHT(mw));
+    XFillRectangle(XtDisplay(mw), win, gc, x, y, COLUMN_WIDTH(mw, column), r);
 
     /*
      * Draw the widget in the cell.
@@ -380,7 +397,7 @@ Pixel bg, fg;
 		 y + mw->matrix.cell_shadow_thickness + mw->matrix.cell_highlight_thickness);
 
     xbaeDrawCellShadow(mw, win, row, column, x, y, COLUMN_WIDTH(mw, column),
-		       ROW_HEIGHT(mw), False, clipped, False);
+		       r, False, clipped, False);
 }
 
 /*
@@ -398,30 +415,23 @@ Pixel bg, fg;
  * Draw a string with specified attributes. We want to avoid having to
  * use a GC clip_mask, so we clip by characters. This complicates the code.
  */
-void
-#if NeedFunctionPrototypes
-xbaeDrawString(XbaeMatrixWidget mw, Window win, GC gc, String string,
-	       int length, int x, int y, int maxlen, unsigned char alignment,
-	       Boolean highlight, Boolean bold, Boolean rowLabel,
-	       Boolean colLabel, Pixel color)
-#else
-xbaeDrawString(mw, win, gc, string, length, x, y, maxlen, alignment,
-	       highlight, bold, rowLabel, colLabel, color)
-XbaeMatrixWidget mw;
-Window win;
-GC gc;
-String string;
-int length;
-int x;
-int y;
-int maxlen;
-unsigned char alignment;
-Boolean highlight;
-Boolean bold;
-Boolean rowLabel;
-Boolean colLabel;
-Pixel color;
-#endif
+	/* FIX ME this should be clipped with the row height for the last row */
+static void
+xbaeDrawString(XbaeMatrixWidget mw,
+		Window win,
+		GC gc,
+		String string,
+		int length,
+		int x,
+		int y,
+		int maxlen,
+		int maxht,		/* Added for row heights */
+		unsigned char alignment,
+		Boolean highlight,
+		Boolean bold,
+		Boolean rowLabel,
+		Boolean colLabel,
+		Pixel color)
 {
     int start, width, maxwidth;
     XFontStruct	*font_struct;
@@ -430,8 +440,12 @@ Pixel color;
     Boolean choppedEnd = False;
     XRectangle *ink_array = NULL;
     XRectangle *logical_array = NULL;
-    int num_chars;
-    XRectangle overall_logical;
+    int num_chars, doclip;
+    XRectangle overall_logical, clipht;
+
+#ifdef	XBAE_VERBOSE
+    fprintf(stderr, "xbaeDrawString(%s) x %d y %d, clip w %d h %d\n", string, x, y, maxlen, maxht);
+#endif
 
     if (rowLabel || colLabel)
     {
@@ -443,6 +457,7 @@ Pixel color;
 	font_struct = mw->matrix.font_struct;
 	font_set = mw->matrix.font_set;
     }
+
     /*
      * Initialize starting character in string
      */
@@ -453,6 +468,28 @@ Pixel color;
     else 
 	maxwidth = maxlen * LABEL_WIDTH(mw);
 
+    /*
+     * Clip height
+     */
+    if (maxht < 0) {
+	doclip = 0;
+    } else {
+	doclip = 1;
+    }
+
+#if 0
+    clipht.height = y + maxht;
+    clipht.width = x + maxwidth;
+    clipht.x = 0;
+    clipht.y = 0;
+
+    if (doclip) {
+	fprintf(stderr, "XSetClipRectangles - x %d y %d w %d h %d\n",
+		x, y, clipht.width, clipht.height);
+	XSetClipRectangles(XtDisplay(mw), gc, 0, 0, &clipht, 1, Unsorted);
+    }
+#endif
+
     if (font_set)
     {
 	ink_array = (XRectangle*)XtMalloc(length * sizeof(XRectangle));
@@ -683,6 +720,20 @@ Pixel color;
 	 */
 	x -= overall_logical.x;
 
+#if 1
+    clipht.height = y + maxht;
+    clipht.width = x + maxwidth;
+    clipht.x = 0;
+    clipht.y = 0;
+
+    if (doclip) {
+#ifdef XBAE_VERBOSE
+	fprintf(stderr, "XSetClipRectangles - x %d y %d w %d h %d\n",
+		x, y, clipht.width, clipht.height);
+#endif
+	XSetClipRectangles(XtDisplay(mw), gc, 0, 0, &clipht, 1, Unsorted);
+    }
+#endif
 	/*
 	 * Now draw the string at x starting at char 'start' and of
 	 * length 'length'
@@ -871,6 +922,20 @@ Pixel color;
 	 * Now draw the string at x starting at char 'start' and of length
 	 * 'length'
 	 */
+#if 1
+    clipht.width = maxwidth + x;
+    clipht.height = maxht + y;
+    clipht.x = 0;
+    clipht.y = 0;
+
+    if (doclip) {
+#ifdef XBAE_VERBOSE
+	fprintf(stderr, "XSetClipRectangles 2 - x %d y %d w %d h %d\n",
+		x, y, clipht.width, clipht.height);
+#endif
+	XSetClipRectangles(XtDisplay(mw), gc, 0, 0, &clipht, 1, Unsorted);
+    }
+#endif
 #ifdef NEED_WCHAR
 	if (TWO_BYTE_FONT(mw))
 	    XDrawString16(XtDisplay(mw), win, gc, x, y, &string[start],
@@ -892,13 +957,18 @@ Pixel color;
 		XDrawString(XtDisplay(mw), win, gc, x - 1, y,
 			    &string[start], length);
     }
+
+    if (doclip) {
+	XSetClipMask(XtDisplay(mw), gc, None);
+    }
 }
 
 void
-xbaeComputeCellColors(mw, row, column, fg, bg)
-XbaeMatrixWidget mw;
-int row, column;
-Pixel *fg, *bg;
+xbaeComputeCellColors(XbaeMatrixWidget mw,
+			int row,
+			int column,
+			Pixel *fg,
+			Pixel *bg)
 {
     Boolean alt = mw->matrix.alt_row_count ?
 	(row / mw->matrix.alt_row_count) % 2 : False;
@@ -945,18 +1015,20 @@ Pixel *fg, *bg;
 }
 
 void
-xbaeDrawCell(mw, row, column)
-XbaeMatrixWidget mw;
-int row, column;
+xbaeDrawCell(XbaeMatrixWidget mw, int row, int column)
 {
     Pixel bg, fg;
     String string;
     int x, y;
-    
+
     if (mw->matrix.disable_redisplay || mw->matrix.rows == 0 ||
 	mw->matrix.columns == 0)
 	return;
 
+#ifdef XBAE_VERBOSE
+    fprintf(stderr, "xbaeDrawCell: %d %d\n", row, column);
+#endif
+
     /*
      * Convert the row/column to the coordinates relative to the correct
      * window
@@ -1001,17 +1073,17 @@ int row, column;
 }
 
 XbaeCellType
-xbaeGetDrawCellValue(mw, row, column, string, pixmap, mask, width,
-		     height, bg, fg, depth)
-XbaeMatrixWidget mw;
-int row;
-int column;
-String *string;
-Pixmap *pixmap;
-Pixmap *mask;
-int *width, *height;
-Pixel *bg, *fg;
-int *depth;
+xbaeGetDrawCellValue(XbaeMatrixWidget mw,
+			int row,
+			int column,
+			String *string,
+			Pixmap *pixmap,
+			Pixmap *mask,
+			int *width,
+			int *height,
+			Pixel *bg,
+			Pixel  *fg,
+			int *depth)
 {
     XbaeMatrixDrawCellCallbackStruct call_data;
 
@@ -1020,7 +1092,7 @@ int *depth;
     call_data.row = row;
     call_data.column = column;
     call_data.width = COLUMN_WIDTH(mw, column) - TEXT_WIDTH_OFFSET(mw) * 2;
-    call_data.height = ROW_HEIGHT(mw) - TEXT_HEIGHT_OFFSET(mw) * 2;
+    call_data.height = SOME_ROW_HEIGHT(mw, row) - TEXT_HEIGHT_OFFSET(mw) * 2;
     call_data.type = XbaeString;
     call_data.string = "";
     call_data.pixmap = (Pixmap)NULL;
@@ -1108,14 +1180,7 @@ int *depth;
  * fixed and non-fixed columns.
  */
 void
-#if NeedFunctionPrototypes
 xbaeDrawColumnLabel(XbaeMatrixWidget mw, int column, Boolean pressed)
-#else
-xbaeDrawColumnLabel(mw, column, pressed)
-XbaeMatrixWidget mw;
-int column;
-Boolean pressed;
-#endif
 {
     String label;
     int labelX, labelY;
@@ -1197,9 +1262,10 @@ Boolean pressed;
 	    xbaeDrawString(mw, XtWindow(mw), gc, label,
 			   mw->matrix.column_label_lines[column].lengths[i],
 			   labelX, labelY, mw->matrix.column_widths[column],
+                         -1,
 			   mw->matrix.column_label_alignments ?
 			   mw->matrix.column_label_alignments[column] :
-			   XmALIGNMENT_BEGINNING, False,
+                         XmALIGNMENT_BEGINNING, False,
 			   mw->matrix.bold_labels, False, True,
 			   mw->matrix.column_label_color);
 	
@@ -1218,16 +1284,10 @@ Boolean pressed;
  * non-fixed rows.
  */
 void
-#if NeedFunctionPrototypes
 xbaeDrawRowLabel(XbaeMatrixWidget mw, int row, Boolean pressed)
-#else
-xbaeDrawRowLabel(mw, row, pressed)
-XbaeMatrixWidget mw;
-int row;
-Boolean pressed;
-#endif
 {
-    int y;
+    int		y;
+    int		 i;
     GC gc;
     Window win = XtWindow(mw);
     Boolean clipped = (row >= (int)mw->matrix.fixed_rows &&
@@ -1239,31 +1299,47 @@ Boolean pressed;
     if (mw->matrix.row_labels[row][0] == '\0' && !button)
 	return;
 
+    /*
+     * FIX ME - This is simplistic.
+     *
+     * Don't draw a label unless there's enough space for it
+     */
+    if (mw->matrix.row_heights[row] < ROW_HEIGHT(mw))
+	return;
+
     /*
      * If the row label is in a fixed row we don't need to account
      * for the vert_origin
      */
-    if (row < (int)mw->matrix.fixed_rows)
-	y = ROW_LABEL_OFFSET(mw) + ROW_HEIGHT(mw) * row + TEXT_Y_OFFSET(mw);
-    else if (row >= TRAILING_VERT_ORIGIN(mw))
+    if (row < (int)mw->matrix.fixed_rows) {
+	if (mw->matrix.row_heights) {
+	    y = ROW_LABEL_OFFSET(mw) + TEXT_Y_OFFSET(mw);
+	    for (i=0; i<row; i++)
+		y += mw->matrix.row_heights[i];
+	} else {
+	    y = ROW_LABEL_OFFSET(mw) + ROW_HEIGHT(mw) * row + TEXT_Y_OFFSET(mw);
+	}
+    } else if (row >= TRAILING_VERT_ORIGIN(mw)) {
 	y = TRAILING_FIXED_ROW_LABEL_OFFSET(mw) +
-	    ROW_HEIGHT(mw) * (row - TRAILING_VERT_ORIGIN(mw)) +
 	    TEXT_Y_OFFSET(mw);
-    else
-	y = ROW_LABEL_OFFSET(mw) + ROW_HEIGHT(mw) * (row - VERT_ORIGIN(mw)) +
-	    LABEL_Y_OFFSET(mw) - mw->matrix.first_row_offset;
+	for (i=TRAILING_VERT_ORIGIN(mw); i<row; i++)
+		y += mw->matrix.row_heights[i];
+    } else {
+	y = ROW_LABEL_OFFSET(mw) + LABEL_Y_OFFSET(mw) - mw->matrix.first_row_offset;
+	for (i=VERT_ORIGIN(mw); i<row; i++)
+		y += mw->matrix.row_heights[i];
+    }
 
     if (clipped)
 	gc = mw->matrix.label_clip_gc;
     else
 	gc = mw->matrix.label_gc;
     
-    if (button)
-    {
+    if (button) {
 	XSetForeground(XtDisplay(mw), gc, mw->matrix.button_label_background);
 	XFillRectangle(XtDisplay(mw), win, gc, VERT_SB_OFFSET(mw),
 		       y - TEXT_Y_OFFSET(mw), ROW_LABEL_WIDTH(mw),
-		       ROW_HEIGHT(mw));
+		       SOME_ROW_HEIGHT(mw, row));
     }
 
     XSetForeground(XtDisplay(mw), gc, mw->matrix.row_label_color);
@@ -1271,16 +1347,17 @@ Boolean pressed;
 
     if (mw->matrix.row_labels[row][0] != '\0')
 	xbaeDrawString(mw, win, gc,
-		       mw->matrix.row_labels[row],
-		       strlen(mw->matrix.row_labels[row]),
-		       TEXT_X_OFFSET(mw) + VERT_SB_OFFSET(mw), y,
-		       mw->matrix.row_label_width,
-		       mw->matrix.row_label_alignment, False,
-		       mw->matrix.bold_labels, True, False,
-		       mw->matrix.row_label_color);
+		mw->matrix.row_labels[row],
+		strlen(mw->matrix.row_labels[row]),
+		TEXT_X_OFFSET(mw) + VERT_SB_OFFSET(mw), y,
+		mw->matrix.row_label_width,	/* width clip */
+		-1,				/* height clip */
+		mw->matrix.row_label_alignment, False,
+		mw->matrix.bold_labels, True, False,
+		mw->matrix.row_label_color);
     
     if (button)
 	xbaeDrawCellShadow(mw, win, row, -1, VERT_SB_OFFSET(mw),
 			   y - TEXT_Y_OFFSET(mw), ROW_LABEL_WIDTH(mw),
-			   ROW_HEIGHT(mw), True, clipped, pressed);
+			   SOME_ROW_HEIGHT(mw, row), True, clipped, pressed);
 }
