@@ -1,5 +1,6 @@
 /*
- * Copyright(c) 1999 Andrew Lister
+ * Copyright (c) 1999 Andrew Lister
+ * Copyright  1999,2000,2001 by the LessTif Developers.
  *
  *                        All rights reserved
  * Permission to use, copy, modify and distribute this material for
@@ -21,35 +22,26 @@
  *
  * Author: Andrew Lister
  *
- * $Id: Input.c,v 1.3 1999/12/11 21:03:11 amai Exp $
+ * $Id: Input.c,v 1.15 2001/05/25 09:07:38 amai Exp $
  */
 
-/*#define DEBUG*/
+/* #define DEBUG */
 
 #ifdef HAVE_CONFIG_H
 #include <XbaeConfig.h>
 #endif
 
-/* For memmove/bcopy */
+#include <stdio.h>
+#include <ctype.h>
 #include <string.h>
 
-#include <ctype.h>
 #include <Xbae/InputP.h>
-#if XmVersion <= 1001
-#include <Xm/XmP.h>
-#else
 #include <Xm/DrawP.h>
+
+#ifdef	WITH_DMALLOC
+#include <dmalloc.h>
 #endif
 
-#if !defined(HAVE_MEMMOVE) && !defined(XBAE_NEED_BCOPY)
-#define XBAE_NEED_BCOPY
-#endif
-
-#if !defined(XBAE_NEED_BCOPY) || defined(SVR4) || defined(VMS)
-#define BCOPY(src, dest, n)     memmove((void *)(dest), (void *)(src), (n))
-#else
-#define BCOPY(src, dest, n)     bcopy((void *)(src), (void *)(dest), (n))
-#endif
 
 #define CHAR_ALPHA		'a' /* Alphabetic characters only */
 #define CHAR_BOTH		'b' /* Both - either character or digit */
@@ -75,23 +67,23 @@
 				 
 /* Declaration of Actions */
 
-/* ARGSUSED */
-static void ToggleOverwriteACT P((Widget, XEvent *,
-				  String *, Cardinal *nparams));
 
-static void validate P((Widget, XtPointer, XtPointer));
+static void ToggleOverwriteACT(Widget, XEvent *,
+				  String *, Cardinal *nparams);
+
+static void validate(Widget, XtPointer, XtPointer);
 
 /* Declaration of Methods */
-static void Initialize P((Widget, Widget, ArgList, Cardinal *));
-static Boolean SetValues P((Widget, Widget, Widget, ArgList, Cardinal *));
-static void Destroy P((Widget));
+static void Initialize(Widget, Widget, ArgList, Cardinal *);
+static Boolean SetValues(Widget, Widget, Widget, ArgList, Cardinal *);
+static void Destroy(Widget);
 
 /* Utility routines */
-static void parsePattern P((XbaeInputWidget, String));
-static int match P((XbaeInputWidget, char *, int));
+static void parsePattern(XbaeInputWidget, String);
+static Boolean match(XbaeInputWidget, char *, Boolean);
 
 /* Internal callbacks */
-static void checkInput P((Widget, XtPointer, XtPointer));
+static void checkInput(Widget, XtPointer, XtPointer);
 
 static char defaultTranslations[] =
 "<Key>osfInsert : ToggleOverwrite()\n\
@@ -120,7 +112,6 @@ static XtResource resources[] = {
       offset(input.validate_callback), XmRCallback, (XtPointer)NULL }
 };
 
-#if XmVersion > 1001
 static XmPrimitiveClassExtRec primitiveClassExtRec = {
     NULL,				/* next_extension	*/
     NULLQUARK,				/* record_type		*/
@@ -130,7 +121,6 @@ static XmPrimitiveClassExtRec primitiveClassExtRec = {
     XmInheritDisplayRectProc,		/* widget_display_rect	*/
     (XmWidgetMarginsProc)_XtInherit	/* widget_margins	*/
 };
-#endif
 
 XbaeInputClassRec xbaeInputClassRec =
 {
@@ -175,11 +165,7 @@ XbaeInputClassRec xbaeInputClassRec =
 	NULL,				/* arm_and_activate	*/
 	NULL,				/* syn resources	*/
 	0,				/* num_syn_resources	*/
-#if XmVersion <= 1001
-	NULL,				/* extension		*/
-#else
-	&primitiveClassExtRec,		/* extension		*/
-#endif
+	(XtPointer)&primitiveClassExtRec, /* extension		*/
     },
     { /* Text class */
 	NULL,				/* extension		*/
@@ -195,12 +181,9 @@ WidgetClass xbaeInputWidgetClass = (WidgetClass) &xbaeInputClassRec;
  * Action routines
  */
 
-/*ARGSUSED*/
+
 static void
-validate(w, cd, cb)
-Widget w;
-XtPointer cd;
-XtPointer cb;
+validate(Widget w, XtPointer cd, XtPointer cb)
 {
     XmAnyCallbackStruct *cbs = (XmAnyCallbackStruct *)cb;
     XmTextVerifyCallbackStruct *vcbs = (XmTextVerifyCallbackStruct *)cb;
@@ -208,14 +191,14 @@ XtPointer cb;
     XbaeInputValidateCallbackStruct call_data;
     String pattern = NULL;
     String value;
-    int ret;
+    Boolean ret;
 
     if (!iw->input.pattern)
 	return;
 
     value = XmTextGetString(w);
 
-    ret = match(iw, value, (int)True);
+    ret = match(iw, value, True);
 
     if (iw->input.auto_fill && iw->input.literal_pending)
     {
@@ -294,13 +277,9 @@ XtPointer cb;
 	XtFree(value);
 }
 
-/* ARGSUSED */
+
 static void
-ToggleOverwriteACT(w, event, params, nparams)
-Widget w;
-XEvent *event;
-String *params;
-Cardinal *nparams;
+ToggleOverwriteACT(Widget w, XEvent *event, String *params, Cardinal *nparams)
 {
     XbaeInputWidget iw = (XbaeInputWidget)w;
 
@@ -318,10 +297,7 @@ Cardinal *nparams;
 
 
 static void
-Initialize(request, new, args, num_args)
-Widget request, new;
-ArgList args;
-Cardinal *num_args;
+Initialize(Widget request, Widget new, ArgList args, Cardinal *num_args)
 {
     XbaeInputWidget iw = (XbaeInputWidget)new;
 
@@ -362,10 +338,8 @@ Cardinal *num_args;
 }
 
 static Boolean
-SetValues(current, request, new, args, num_args)
-Widget current, request, new;
-ArgList args;
-Cardinal *num_args;
+SetValues(Widget current, Widget request, Widget new,
+	  ArgList args, Cardinal *num_args)
 {
     XbaeInputWidget c = (XbaeInputWidget)current;
     XbaeInputWidget n = (XbaeInputWidget)new;
@@ -408,7 +382,7 @@ Cardinal *num_args;
 	if (text && *text)
 	{
 	    *(text + pos) = 0;
-	    (void)match(n, text, (int)False);
+	    match(n, text, False);
 	}
 	else
 	    parsePattern(n, new(pattern));
@@ -423,8 +397,7 @@ Cardinal *num_args;
 }
 
 static void
-Destroy(w)
-Widget w;
+Destroy(Widget w)
 {
     XbaeInputWidget iw = (XbaeInputWidget)w;
 
@@ -433,9 +406,7 @@ Widget w;
 }
 
 static void
-parsePattern(iw, pattern)
-XbaeInputWidget iw;
-String pattern;
+parsePattern(XbaeInputWidget iw, String pattern)
 {
     char *ptr = pattern;
     Boolean optional = False;
@@ -513,18 +484,16 @@ String pattern;
     XtVaSetValues((Widget)iw, XmNmaxLength, iw->input.pattern_length, NULL);
 }
 
-static int
-match(iw, string, exact)
-XbaeInputWidget iw;
-char *string;
-int exact;
+
+static Boolean
+match(XbaeInputWidget iw, char *string, Boolean exact)
 {
     char *tptr, *sptr, *ntptr;
     char *lastopt = NULL;	/* last optional block */
     char *lastreq = NULL;	/* last required block */
     int optional = 0;
     /*int pend_rew = 0;*/
-    int equal = 1;
+    Boolean equal = True;
     int escaped;
     int set_lastopt = 0;
 
@@ -592,13 +561,13 @@ int exact;
 		    set_lastopt = 0;
 		    break;
 		case CHAR_ALPHA:	/* alphabetic */
-		    equal = (int)isalpha(*sptr);
+		    equal = (isalpha(*sptr)==0) ? False : True;
 		    if (equal && set_lastopt)
 			lastopt = sptr;
 		    set_lastopt = 0;
 		    break;
 		case CHAR_BOTH:		/* both: alphabetic or digit */
-		    equal = (int)isalnum(*sptr);
+		    equal = (isalnum(*sptr)==0) ? False : True;
 		    if (equal && set_lastopt)
 			lastopt = sptr;
 		    set_lastopt = 0;
@@ -610,7 +579,7 @@ int exact;
 		    set_lastopt = 0;
 		    break;
 		case CHAR_DIGIT:	/* any digit */
-		    equal = (int)isdigit(*sptr);
+		    equal = (isdigit(*sptr)==0) ? False : True;
 		    if (equal && set_lastopt)
 			lastopt = sptr;
 		    set_lastopt = 0;
@@ -618,7 +587,7 @@ int exact;
 		case CHAR_UPPER:
 		    if (iw->input.convert_case && islower(*sptr))
 			*sptr = toupper(*sptr);
-		    equal = (int)(isalpha(*sptr) && isupper(*sptr));
+		    equal = ((isalpha(*sptr)!=0) && isupper(*sptr));
 		    if (equal && set_lastopt)
 			lastopt = sptr;
 		    set_lastopt = 0;
@@ -626,7 +595,7 @@ int exact;
 		case CHAR_LOWER:
 		    if (iw->input.convert_case && isupper(*sptr))
 			*sptr = tolower(*sptr);
-		    equal = (int)(isalpha(*sptr) && islower(*sptr));
+		    equal = ((isalpha(*sptr)!=0) && islower(*sptr));
 		    if (equal && set_lastopt)
 			lastopt = sptr;
 		    set_lastopt = 0;
@@ -637,7 +606,7 @@ int exact;
 		       char so skip it and then next.
 		    */
 		    tptr = ntptr + 1;
-		    equal = 0;
+		    equal = False;
 		    continue;
 		case CHAR_ESCAPE:
 		    tptr++;
@@ -645,7 +614,7 @@ int exact;
 		    escaped = 1;
 		    /* And fall through to literal */
 		default:		/* treat as a literal */
-		    equal = (int)(*sptr == *tptr);
+		    equal = (*sptr == *tptr);
 		    if (equal && set_lastopt)
 			lastopt = sptr;
 		    set_lastopt = 0;
@@ -694,10 +663,9 @@ int exact;
 		while (*tptr &&
 		       (*tptr != CHAR_EOPTION || *(tptr-1) == CHAR_ESCAPE))
 		    tptr++;
-		if (!*tptr)
-		    (void)fprintf(stderr, "Mismatched parentheses\n");
-		else
-		{
+		if (!*tptr) {
+		    fprintf(stderr, "Mismatched parentheses\n");	/* FIX ME */
+		} else {
 		    tptr++;	/* move past the ']' */
 #ifdef OLD
 		    if (!lastopt)
@@ -810,7 +778,8 @@ int exact;
 		}
 	    }
 	    continue;
-	}
+	} /* if (!equal) */
+
 	if (escaped || (*tptr != CHAR_EOPTION && *tptr != CHAR_SOPTION))
 	{
 #ifdef DEBUG
@@ -885,12 +854,9 @@ int exact;
     return equal;
 }
 
-/*ARGSUSED*/
+
 static void
-checkInput(w, cd, cb)
-Widget w;
-XtPointer cd;
-XtPointer cb;
+checkInput(Widget w, XtPointer cd, XtPointer cb)
 {
     XbaeInputWidget iw = (XbaeInputWidget)w;
     XmTextVerifyCallbackStruct *cbs = (XmTextVerifyCallbackStruct *)cb;
@@ -917,7 +883,9 @@ XtPointer cb;
 	 * to move the cursor to overwrite the text, not to delete the character
 	 * then inserting a new value */
 	if (iw->input.overwrite_mode == True) {
+#if 0
 		fprintf(stderr, "Yow\n");
+#endif
 		XmTextSetInsertionPosition((Widget)iw,(cbs->currInsert - 1));
 		cbs->doit = False;
 	}
@@ -983,7 +951,7 @@ XtPointer cb;
     {
 	char *buf = XtNewString(text);
 	buf[pos] = 0;
-	(void)match(iw, buf, (int)False);
+	match(iw, buf, False);
 	XtFree(buf);
     }
 
@@ -1038,7 +1006,7 @@ XtPointer cb;
 
 	text[pos] = cbs->text->ptr[i];
 
-	if (match(iw, text, (int)False) == 0)
+	if (!match(iw, text, False))
 	{
 	    cbs->doit = False;
 	    XtFree(text);
@@ -1055,12 +1023,9 @@ XtPointer cb;
     XtFree(text);
 }
 
+
 Widget
-XbaeCreateInput(parent, name, args, ac) 
-Widget parent;
-String name;
-ArgList args;
-Cardinal ac;
+XbaeCreateInput(Widget parent, String name, ArgList args, Cardinal ac) 
 { 
     return XtCreateWidget(name, xbaeInputWidgetClass, parent, args, ac); 
 } 
