@@ -1,6 +1,8 @@
 /*
  * Copyright(c) 1992 Bell Communications Research, Inc. (Bellcore)
  * Copyright(c) 1995-99 Andrew Lister
+ * Copyright  1999, 2000, 2001 by the LessTif Developers.
+ *
  *                        All rights reserved
  * Permission to use, copy, modify and distribute this material for
  * any purpose and without fee is hereby granted, provided that the
@@ -20,7 +22,7 @@
  * LOST PROFITS OR OTHER INCIDENTAL OR CONSEQUENTIAL DAMAGES RELAT-
  * ING TO THE SOFTWARE.
  *
- * $Id: ScrollMgr.c,v 1.1 1999/08/04 19:47:00 danny Exp $
+ * $Id: ScrollMgr.c,v 1.24 2001/05/23 20:25:10 dannybackx Exp $
  */
 
 /*
@@ -32,6 +34,8 @@
 #include <XbaeConfig.h>
 #endif
 
+#include <stdio.h>
+
 #include <Xm/Xm.h>
 #include <Xbae/MatrixP.h>
 #include <Xbae/Draw.h>
@@ -40,6 +44,11 @@
 #include <Xbae/Utils.h>
 #include <Xm/ScrollBar.h>
 
+#ifdef	WITH_DMALLOC
+#include <dmalloc.h>
+#endif
+
+
 /*
  * ScrollMgr implementation.
  * When we scroll using XCopyArea, occluding windows will cause GraphicsExpose
@@ -66,7 +75,7 @@
  * Create and initialize a ScrollMgr
  */
 SmScrollMgr
-xbaeSmCreateScrollMgr()
+xbaeSmCreateScrollMgr(void)
 {
     SmScrollMgr scrollMgr = XtNew(SmScrollMgrRec);
 
@@ -83,8 +92,7 @@ xbaeSmCreateScrollMgr()
  * Destroy a ScrollMgr, including any queued scrolls
  */
 void
-xbaeSmDestroyScrollMgr(scrollMgr)
-SmScrollMgr scrollMgr;
+xbaeSmDestroyScrollMgr(SmScrollMgr scrollMgr)
 {
     if (scrollMgr->scroll_queue)
     {
@@ -95,22 +103,19 @@ SmScrollMgr scrollMgr;
 	    SmScrollNode d = node;
 
 	    node = node->next;
-	    XtFree((XtPointer) d);
+	    XtFree((char *) d);
 	}
-	XtFree((XtPointer) node);
+	XtFree((char *) node);
     }
 
-    XtFree((XtPointer) scrollMgr);
+    XtFree((char *) scrollMgr);
 }
 
 /*
  * Record a new scroll request in the ScrollMgr
  */
 void
-xbaeSmAddScroll(scrollMgr, delta_x, delta_y)
-SmScrollMgr scrollMgr;
-int delta_x;
-int delta_y;
+xbaeSmAddScroll(SmScrollMgr scrollMgr, int delta_x, int delta_y)
 {
     SmScrollNode node = XtNew(SmScrollNodeRec);
 
@@ -145,8 +150,7 @@ int delta_y;
  * Remove a scroll from the ScrollMgr queue
  */
 void
-xbaeSmRemoveScroll(scrollMgr)
-SmScrollMgr scrollMgr;
+xbaeSmRemoveScroll(SmScrollMgr scrollMgr)
 {
     if (scrollMgr->scroll_count)
     {
@@ -166,7 +170,7 @@ SmScrollMgr scrollMgr;
 	    node->next->prev = node->prev;
 	    node->prev->next = node->next;
 	}
-	XtFree((XtPointer) node);
+	XtFree((char *) node);
 
 	scrollMgr->scroll_count--;
     }
@@ -176,31 +180,35 @@ SmScrollMgr scrollMgr;
  * Handle an expose event
  */
 void
-xbaeSmScrollEvent(scrollMgr, event)
-SmScrollMgr scrollMgr;
-XEvent *event;
+xbaeSmScrollEvent(SmScrollMgr scrollMgr, XEvent *event)
 {
     switch (event->type)
     {
 
     case Expose:
-
 	/*
 	 * Normal Expose event, translate it into our scrolled
 	 * coordinate system.
 	 */
+#ifdef XBAE_VERBOSE
+	fprintf(stderr, "xbaeSmScrollEvent: Expose\n");
+#endif
+
 	event->xexpose.x += scrollMgr->offset_x;
 	event->xexpose.y += scrollMgr->offset_y;
 	break;
 
     case GraphicsExpose:
-
 	/*
 	 * If we are not scrolling, then this must be the first
 	 * GraphicsExpose event.  Remove the corresponding scroll from the
 	 * queue, and if we have more GraphicsExposes to come, set scrolling
 	 * to True.
 	 */
+#ifdef XBAE_VERBOSE
+	fprintf(stderr, "xbaeSmScrollEvent: GraphicsExpose\n");
+#endif
+
 	if (scrollMgr->scrolling == False)
 	{
 	    xbaeSmRemoveScroll(scrollMgr);
@@ -242,297 +250,394 @@ XEvent *event;
  */
 /* ARGSUSED */
 void
-xbaeScrollVertCB(w, client_data, call_data)
-Widget w;
-XtPointer client_data;
-XmScrollBarCallbackStruct *call_data;
+xbaeScrollVertCB(Widget w, XtPointer client_data, XmScrollBarCallbackStruct *call_data)
 {
-    XbaeMatrixWidget mw = (XbaeMatrixWidget) XtParent(w);
-    Rectangle fixed, nonfixed;
-    int src_y, dest_y, height;
-    int vert_sb_offset = VERT_SB_OFFSET(mw);
-    int row_height = ROW_HEIGHT(mw);
-    int trailing_fixed_row_label_offset = TRAILING_FIXED_ROW_LABEL_OFFSET(mw);
-    int trailing_fixed_column_label_offset =
-	TRAILING_FIXED_COLUMN_LABEL_OFFSET(mw);
-    int trailing_fixed_column_width = TRAILING_FIXED_COLUMN_WIDTH(mw);
-    int row_label_width = ROW_LABEL_WIDTH(mw);
-    int fixed_row_label_offset = FIXED_ROW_LABEL_OFFSET(mw);
-    
-    /*
-     * Not managed yet
-     */
-    if (!XtIsRealized((Widget)mw))
-	return;
+	XbaeMatrixWidget mw = (XbaeMatrixWidget) XtParent(w);
+	Rectangle	fixed, nonfixed;
+	int		src_y, dest_y, height;
+	int		vert_sb_offset = VERT_SB_OFFSET(mw);
+	int		row_height = ROW_HEIGHT(mw);
+	int		trailing_fixed_row_label_offset = TRAILING_FIXED_ROW_LABEL_OFFSET(mw);
+	int		trailing_fixed_column_label_offset = TRAILING_FIXED_COLUMN_LABEL_OFFSET(mw);
+	int		trailing_fixed_column_width = TRAILING_FIXED_COLUMN_WIDTH(mw);
+	int		row_label_width = ROW_LABEL_WIDTH(mw);
+	int		fixed_row_label_offset = FIXED_ROW_LABEL_OFFSET(mw);
+	int		row, i;
 
-    /*
-     * Didn't scroll
-     */
-    if (call_data->value == VERT_ORIGIN(mw))
-	return;
-
-    /*
-     * Scrolled forward. We want to copy a chunk starting at src_y up
-     * to the top (dest_y=0)
-     */
-    else if (call_data->value > VERT_ORIGIN(mw))
-    {
-	dest_y = 0;
-	src_y = (call_data->value - VERT_ORIGIN(mw)) * row_height;
-	height = ClipChild(mw)->core.height - src_y;
-    }
-
-    /*
-     * Scrolled backward. We want to copy a chunk starting at the top
-     * (src_y=0) down to dest_y.
-     */
-    else
-    {
-	dest_y = (VERT_ORIGIN(mw) - call_data->value) * row_height;
-	src_y = 0;
-	height = ClipChild(mw)->core.height - dest_y;
-    }
-
-    /*
-     * The textField needs to scroll along with the cells.
-     */
-    if (XtIsManaged(TextChild(mw)) &&
-		mw->matrix.current_row >= (int)mw->matrix.fixed_rows &&
-		mw->matrix.current_row < TRAILING_VERT_ORIGIN(mw))
-	XtMoveWidget(TextChild(mw),
-		     TextChild(mw)->core.x, TextChild(mw)->core.y +
-		     (VERT_ORIGIN(mw) - call_data->value) * row_height);
-
-    /*
-     * Now we can adjust our vertical origin
-     */
-    VERT_ORIGIN(mw) = call_data->value;
-
-    /*
-     * If we scrolled more than a screenful, just clear and
-     * redraw the whole thing
-     */
-    if (height <= 0)
-    {
 	/*
-	 * Clear the whole clip window.
+	 * Not managed yet
 	 */
-	XClearArea(XtDisplay(mw), XtWindow(ClipChild(mw)),
-		   0, 0,
-		   0 /*Full Width*/, 0 /*Full Height*/,
-		   False);
+	if (!XtIsRealized((Widget)mw))
+		return;
+
+	/* Figure out which row we're in */
+	if (call_data->value == 0) {
+		row = 0;
+		i = 0;
+	} else {
+		row = 0;
+		for (i=1; i<mw->matrix.rows; i++)
+			if (call_data->value <= ROW_POSITION(mw, i)) {
+				row = i;
+				break;
+			}
+	}
+
+#ifdef	XBAE_VERBOSE
+	fprintf(stderr, "xbaeScrollVertCB() - row %d, orig pos %d, should be %d, ClipChild ht %d\n",
+		row, call_data->value, ROW_POSITION(mw, row), ClipChild(mw)->core.height);
+#if 0
+{
+	int i;
+	fprintf(stderr, "Rows : ");
+	for (i=0; i<mw->matrix.rows; i++) {
+	fprintf(stderr, "%d-[%d..%d] ", i, ROW_POSITION(mw, i),
+		ROW_POSITION(mw, i) + SOME_ROW_HEIGHT(mw, i) - 1);
+	}
+	fprintf(stderr, "\n");
+}
+#endif
+#endif
 
 	/*
-	 * Clear the whole Left and Right Clips
+	 * Set the new position exactly on a row boundary
 	 */
-	if (XtIsManaged(LeftClip(mw)))
-	    XClearArea(XtDisplay(mw), XtWindow(LeftClip(mw)),
-		       0, 0,
-		       0 /*Full Width*/, 0 /*Full Height*/,
-		       False);
-	if (XtIsManaged(RightClip(mw)))
-	    XClearArea(XtDisplay(mw), XtWindow(RightClip(mw)),
-		       0, 0,
-		       0 /*Full Width*/, 0 /*Full Height*/,
-		       False);
-	/*
-	 * Redraw all the non-fixed cells in the clip window
-	 */
-	SETRECT(nonfixed,
-		0, 0,
-		ClipChild(mw)->core.width - 1,
-		ClipChild(mw)->core.height - 1);
+	call_data->value = ROW_POSITION(mw, row);
 
 	/*
-	 * Clear non-fixed row labels, if necessary.
-	 * If we don't have a row label width, then
-	 * it would clear the entire width.
+	 * This piece of code figures out in which direction we're scrolling,
+	 * it stores its results in three variables :
+	 *	src_y
+	 *	dest_y
+	 *	height
+	 *
+	 * call_data->value is the pixel position to scroll to.
 	 */
-	if (row_label_width)
-	    XClearArea(XtDisplay(mw), XtWindow(mw),
-		       vert_sb_offset, fixed_row_label_offset,
-		       row_label_width - 1,
-		       VISIBLE_HEIGHT(mw),
-		       False);
+	if (call_data->value == ROW_POSITION(mw, VERT_ORIGIN(mw))) {
+		/*
+		 * Didn't scroll
+		 */
+#ifdef XBAE_VERBOSE
+		fprintf(stderr, "xbaeScrollVertCB: didn't scroll\n");
+#endif
+		return;
+	} else if (call_data->value > ROW_POSITION(mw, VERT_ORIGIN(mw))) {
+		/*
+		 * Scrolled forward. We want to copy a chunk starting at src_y up
+		 * to the top (dest_y=0)
+		 */
+#ifdef XBAE_VERBOSE
+		fprintf(stderr, "xbaeScrollVertCB: scrolled forward\n");
+#endif
 
-	/*
-	 * Clear the trailing filled rows, if necessary
-         */
-	if (IN_GRID_ROW_MODE(mw) && NEED_HORIZ_FILL(mw))
-	    XClearArea(XtDisplay(mw), XtWindow(mw),
-		       trailing_fixed_column_label_offset +
-		       trailing_fixed_column_width,
-		       fixed_row_label_offset,
-		       FILL_HORIZ_WIDTH(mw),
-		       ClipChild(mw)->core.height,
-		       False);
+		dest_y = 0;
+		/* Is it too far ? */
+		if (call_data->value - ROW_POSITION(mw, VERT_ORIGIN(mw))
+				> ClipChild(mw)->core.height) {
+			/* yes */
+			height = -1;
+			src_y = call_data->value;
+		} else {
+			src_y = call_data->value - ROW_POSITION(mw, VERT_ORIGIN(mw));
+			height = ClipChild(mw)->core.height - src_y;
+		}
+	} else {
+		/*
+		 * Scrolled backward. We want to copy a chunk starting at the top
+		 * (src_y=0) down to dest_y.
+		 */
 
-	/*
-	 * Redraw non-fixed row labels and cells in fixed columns
-	 */
-	SETRECT(fixed,
-		vert_sb_offset, fixed_row_label_offset,
-		trailing_fixed_column_label_offset +
-		trailing_fixed_column_width - 1,
-		trailing_fixed_row_label_offset - 1);
-    }
-
-    /*
-     * If we scrolled less than a screenful, we want to copy as many
-     * pixels as we can and then clear and redraw the newly scrolled data.
-     */
-    else
-    {
-	int y_clear = src_y > dest_y ? height : 0;
-
-	/*
-	 * Queue this scroll with the ScrollMgr
-	 */
-	xbaeSmAddScroll(mw->matrix.clip_scroll_mgr, 0, dest_y - src_y);
-
-	/*
-	 * Copy the non-fixed cells in the clip widget
-	 */
-	XCopyArea(XtDisplay(mw),
-		  XtWindow(ClipChild(mw)), XtWindow(ClipChild(mw)),
-		  mw->matrix.draw_gc,
-		  0, src_y,
-		  ClipChild(mw)->core.width, height,
-		  0, dest_y);
-
-	/*
-	 * Clear the newly scrolled chunk of the clip widget
-	 */
-	XClearArea(XtDisplay(mw), XtWindow(ClipChild(mw)),
-		   0, y_clear,
-		   0 /*Full Width*/, ClipChild(mw)->core.height - height,
-		   False);
-
-	/*
-	 * Redraw the non-fixed cells into the new chunk
-	 */
-	SETRECT(nonfixed,
-		0, y_clear,
-		ClipChild(mw)->core.width - 1,
-		(y_clear + (ClipChild(mw)->core.height - height)) - 1);
-
-	/*
-	 * Queue this scroll with the ScrollMgr
-	 */
-	xbaeSmAddScroll(mw->matrix.matrix_scroll_mgr, 0, dest_y - src_y);
-
-	/*
-	 * Copy cells in the fixed columns on LeftClip
-	 */
-	if (XtIsManaged(LeftClip(mw)))
-	    XCopyArea(XtDisplay(mw),
-		      XtWindow(LeftClip(mw)), XtWindow(LeftClip(mw)),
-		      mw->matrix.draw_gc,
-		      0, src_y,
-		      FIXED_COLUMN_WIDTH(mw), height,
-		      0, dest_y);
-
-	/*
-	 * Copy cells in the trailing fixed columns on RightClip
-	 */
-	if (XtIsManaged(RightClip(mw)))
-	    XCopyArea(XtDisplay(mw),
-		      XtWindow(RightClip(mw)), XtWindow(RightClip(mw)),
-		      mw->matrix.draw_gc,
-		      0, src_y,
-		      trailing_fixed_column_width, height,
-		      0, dest_y);
-
-	/*
-	 * Translate coordinates for row labels (on the parent matrix window).
-	 */
-	src_y += fixed_row_label_offset;
-	dest_y += fixed_row_label_offset;
-
-	/*
-	 * Copy the row labels
-	 */
-	if (row_label_width)
-	    XCopyArea(XtDisplay(mw),
-		      XtWindow(mw), XtWindow(mw),
-		      mw->matrix.draw_gc,
-		      vert_sb_offset, src_y,
-		      row_label_width, height,
-		      vert_sb_offset, dest_y);
-	/*
-	 * Copy trailing filled portion of the rows if necessary
-	 */
-	if (IN_GRID_ROW_MODE(mw) && NEED_HORIZ_FILL(mw))
-	{
-	    XCopyArea(XtDisplay(mw), XtWindow(mw), XtWindow(mw),
-		      mw->matrix.draw_gc,
-		      trailing_fixed_column_label_offset +
-		      trailing_fixed_column_width, src_y,
-		      FILL_HORIZ_WIDTH(mw), height,
-		      trailing_fixed_column_label_offset +
-		      trailing_fixed_column_width, dest_y);
+#ifdef XBAE_VERBOSE
+		fprintf(stderr, "xbaeScrollVertCB: scrolled backward\n");
+#endif
+		src_y = 0;
+		/* Is it too far ? */
+		if ((ROW_POSITION(mw, VERT_ORIGIN(mw)) - call_data->value)
+				> ClipChild(mw)->core.height) {
+			/* yes */
+			height = -1;
+			dest_y = call_data->value;
+		} else {
+			int h = ROW_POSITION(mw, VERT_ORIGIN(mw)) - call_data->value;
+			dest_y = call_data->value;
+			dest_y = h;
+			height = ClipChild(mw)->core.height - h;
+		}
 	}
 
 	/*
-	 * Clear newly scrolled chunk of fixed columns on LeftClip
+	 * The textField needs to scroll along with the cells.
 	 */
-	if (XtIsManaged(LeftClip(mw)))
-	    XClearArea(XtDisplay(mw), XtWindow(LeftClip(mw)),
-		       0, y_clear, 0 /* Full Width */,
-		       ClipChild(mw)->core.height - height, False);
-	/*
-	 * Clear newly scrolled chunk of trailing fixed columns on RightClip
-	 */
-	if (XtIsManaged(RightClip(mw)))
-	    XClearArea(XtDisplay(mw), XtWindow(RightClip(mw)),
-		       0, y_clear, 0 /* Full Width */,
-		       ClipChild(mw)->core.height - height, False);
-	/*
-	 * Translate coordinates for row labels on Matrix
-	 */
-	y_clear += fixed_row_label_offset;
+	if (XtIsManaged(TextChild(mw)) &&
+			mw->matrix.current_row >= (int)mw->matrix.fixed_rows &&
+			mw->matrix.current_row < TRAILING_VERT_ORIGIN(mw)) {
+		/* FIX ME */
+		XtMoveWidget(TextChild(mw),
+			TextChild(mw)->core.x, TextChild(mw)->core.y +
+			(VERT_ORIGIN(mw) - call_data->value) * row_height);
+	}
+
+	/* Cell widgets that are in the vertical scrolling region 
+	 * need to scroll along as well... Linas */
+	if (mw->matrix.cell_widgets) {
+		int roe, kol, i;
+
+		for (roe=mw->matrix.fixed_rows; roe < TRAILING_VERT_ORIGIN(mw); roe ++) {
+			for (kol=0; kol < (mw->matrix.columns); kol++) { 
+				Widget w = mw->matrix.cell_widgets[roe][kol];
+				if (w) {
+					if (XtIsManaged (w)) {
+						int x = w->core.x;
+						int y = w->core.y;
+
+						for (i=call_data->value; i<VERT_ORIGIN(mw); i++)
+							y += SOME_ROW_HEIGHT(mw, i);
+
+						XtMoveWidget(w, x, y);
+					}
+				}
+			}
+		}
+	}
 
 	/*
-	 * Clear the newly scrolled chunk of row labels
+	 * Now we can adjust our vertical origin
 	 */
-	if (row_label_width)
-	    XClearArea(XtDisplay(mw), XtWindow(mw),
-		       vert_sb_offset, y_clear,
-		       row_label_width,
-		       CLIP_VERT_VISIBLE_SPACE(mw) +
-		       mw->manager.shadow_thickness,
-		       False);
+	VERT_ORIGIN(mw) = row;
 
 	/*
-	 * Clear the trailing filled rows, if necessary
+	 * If we scrolled more than a screenful, just clear and
+	 * redraw the whole thing
 	 */
-	if ((src_y > dest_y) && IN_GRID_ROW_MODE(mw) && NEED_HORIZ_FILL(mw))
-	    XClearArea(XtDisplay(mw), XtWindow(mw),
-		       trailing_fixed_column_label_offset +
-		       trailing_fixed_column_width, y_clear, 
-		       FILL_HORIZ_WIDTH(mw),
-		       ClipChild(mw)->core.height - height,
-		       False);
- 	
-	/*
-	 * Redraw the new chunk of fixed columns and row labels
-	 */
-	SETRECT(fixed,
-		vert_sb_offset, y_clear,
-		trailing_fixed_column_label_offset +
-		trailing_fixed_column_width - 1,
-		y_clear + CLIP_VERT_VISIBLE_SPACE(mw));
-    }
 
-    /*
-     * Perform the actual redraw. The call to draw the grid shadows
-     * must be done after any XClearAreas() to ensure we don't redraw
-     * more than we have to; that is, we could put the call in the
-     * redraw cells routine, but we would end up occasionally redrawing
-     * more than once.
-     */
-    xbaeRedrawLabelsAndFixed(mw, &fixed);
-    xbaeRedrawCells(mw, &nonfixed);
+	if (height <= 0) {
+		/*
+		 * Clear the whole clip window.
+		 */
+		XClearArea(XtDisplay(mw), XtWindow(ClipChild(mw)),
+			0, 0,
+			0 /*Full Width*/, 0 /*Full Height*/,
+			False);
+
+		/*
+		 * Clear the whole Left and Right Clips
+		 */
+		if (XtIsManaged(LeftClip(mw)))
+			XClearArea(XtDisplay(mw), XtWindow(LeftClip(mw)),
+				0, 0,
+				0 /*Full Width*/, 0 /*Full Height*/,
+				False);
+		if (XtIsManaged(RightClip(mw)))
+			XClearArea(XtDisplay(mw), XtWindow(RightClip(mw)),
+				0, 0,
+				0 /*Full Width*/, 0 /*Full Height*/,
+				False);
+		/*
+		 * Redraw all the non-fixed cells in the clip window
+		 */
+		SETRECT(nonfixed,
+			0, 0,
+			ClipChild(mw)->core.width - 1,
+			ClipChild(mw)->core.height - 1);
+
+		/*
+		 * Clear non-fixed row labels, if necessary.
+		 * If we don't have a row label width, then
+		 * it would clear the entire width.
+		 */
+		if (row_label_width)
+			XClearArea(XtDisplay(mw), XtWindow(mw),
+				vert_sb_offset, fixed_row_label_offset,
+				row_label_width - 1,
+				VISIBLE_HEIGHT(mw),
+				False);
+
+		/*
+		 * Clear the trailing filled rows, if necessary
+		 */
+		if (IN_GRID_ROW_MODE(mw) && NEED_HORIZ_FILL(mw))
+			XClearArea(XtDisplay(mw), XtWindow(mw),
+				trailing_fixed_column_label_offset + trailing_fixed_column_width,
+				fixed_row_label_offset,
+				FILL_HORIZ_WIDTH(mw),
+				ClipChild(mw)->core.height,
+				False);
+
+		/*
+		 * Redraw non-fixed row labels and cells in fixed columns
+		 */
+		SETRECT(fixed,
+			vert_sb_offset, fixed_row_label_offset,
+			trailing_fixed_column_label_offset +
+			trailing_fixed_column_width - 1,
+			trailing_fixed_row_label_offset - 1);
+	} else { 
+		/*
+		 * If we scrolled less than a screenful, we want to copy as many
+		 * pixels as we can and then clear and redraw the newly scrolled data.
+		 */
+		int y_clear = (src_y > dest_y) ? height : 0;
+
+		/*
+		 * Queue this scroll with the ScrollMgr
+		 */
+		xbaeSmAddScroll(mw->matrix.clip_scroll_mgr, 0, dest_y - src_y);
+
+		/*
+		 * Copy the non-fixed cells in the clip widget
+		 */
+		XCopyArea(XtDisplay(mw),
+			XtWindow(ClipChild(mw)), XtWindow(ClipChild(mw)),
+			mw->matrix.draw_gc,
+			0, src_y,
+			ClipChild(mw)->core.width, height,
+			0, dest_y);
+
+		/*
+		 * Clear the newly scrolled chunk of the clip widget
+		 */
+		XClearArea(XtDisplay(mw), XtWindow(ClipChild(mw)),
+			0, y_clear,
+			0 /*Full Width*/, ClipChild(mw)->core.height - height,
+			False);
+
+		/*
+		 * Redraw the non-fixed cells into the new chunk
+		 */
+		SETRECT(nonfixed,
+			0, y_clear,
+			ClipChild(mw)->core.width - 1,
+			(y_clear + (ClipChild(mw)->core.height - height)) - 1);
+#ifdef XBAE_VERBOSE
+	fprintf(stderr, "SetRect(nonfixed) y_clear %d, ClipChild wid %d, ht %d, height %d\n",
+		y_clear, ClipChild(mw)->core.width, ClipChild(mw)->core.height, height);
+#endif
+
+		/*
+		 * Queue this scroll with the ScrollMgr
+		 */
+		xbaeSmAddScroll(mw->matrix.matrix_scroll_mgr, 0, dest_y - src_y);
+
+		/*
+		 * Copy cells in the fixed columns on LeftClip
+		 */
+		if (XtIsManaged(LeftClip(mw)))
+			XCopyArea(XtDisplay(mw),
+				XtWindow(LeftClip(mw)), XtWindow(LeftClip(mw)),
+				mw->matrix.draw_gc,
+				0, src_y,
+				FIXED_COLUMN_WIDTH(mw), height,
+				0, dest_y);
+
+		/*
+		 * Copy cells in the trailing fixed columns on RightClip
+		 */
+		if (XtIsManaged(RightClip(mw)))
+			XCopyArea(XtDisplay(mw),
+				XtWindow(RightClip(mw)), XtWindow(RightClip(mw)),
+				mw->matrix.draw_gc,
+				0, src_y,
+				trailing_fixed_column_width, height,
+				0, dest_y);
+
+		/*
+		 * Translate coordinates for row labels (on the parent matrix window).
+		 */
+		src_y += fixed_row_label_offset;
+		dest_y += fixed_row_label_offset;
+
+		/*
+		 * Copy the row labels
+		 */
+		if (row_label_width)
+			XCopyArea(XtDisplay(mw),
+				XtWindow(mw), XtWindow(mw),
+				mw->matrix.draw_gc,
+				vert_sb_offset, src_y,
+				row_label_width, height,
+				vert_sb_offset, dest_y);
+		/*
+		 * Copy trailing filled portion of the rows if necessary
+		 */
+		if (IN_GRID_ROW_MODE(mw) && NEED_HORIZ_FILL(mw)) {
+			XCopyArea(XtDisplay(mw), XtWindow(mw), XtWindow(mw),
+				mw->matrix.draw_gc,
+				trailing_fixed_column_label_offset +
+				trailing_fixed_column_width, src_y,
+				FILL_HORIZ_WIDTH(mw), height,
+				trailing_fixed_column_label_offset +
+				trailing_fixed_column_width, dest_y);
+		}
+
+		/*
+		 * Clear newly scrolled chunk of fixed columns on LeftClip
+		 */
+		if (XtIsManaged(LeftClip(mw)))
+			XClearArea(XtDisplay(mw), XtWindow(LeftClip(mw)),
+				0, y_clear, 0 /* Full Width */,
+				ClipChild(mw)->core.height - height, False);
+		/*
+		 * Clear newly scrolled chunk of trailing fixed columns on RightClip
+		 */
+		if (XtIsManaged(RightClip(mw)))
+			XClearArea(XtDisplay(mw), XtWindow(RightClip(mw)),
+				0, y_clear, 0 /* Full Width */,
+				ClipChild(mw)->core.height - height, False);
+		/*
+		 * Translate coordinates for row labels on Matrix
+		 */
+		y_clear += fixed_row_label_offset;
+
+		/*
+		 * Clear the newly scrolled chunk of row labels
+		 */
+		if (row_label_width)
+			XClearArea(XtDisplay(mw), XtWindow(mw),
+				vert_sb_offset, y_clear,
+				row_label_width,
+				CLIP_VERT_VISIBLE_SPACE(mw) +
+				mw->manager.shadow_thickness,
+				False);
+
+		/*
+		 * Clear the trailing filled rows, if necessary
+		 */
+		if ((src_y > dest_y) && IN_GRID_ROW_MODE(mw) && NEED_HORIZ_FILL(mw))
+			XClearArea(XtDisplay(mw), XtWindow(mw),
+				trailing_fixed_column_label_offset +
+				trailing_fixed_column_width, y_clear, 
+				FILL_HORIZ_WIDTH(mw),
+				ClipChild(mw)->core.height - height,
+				False);
+
+		/*
+		 * Redraw the new chunk of fixed columns and row labels
+		 */
+		SETRECT(fixed,
+			vert_sb_offset, y_clear,
+			trailing_fixed_column_label_offset +
+			trailing_fixed_column_width - 1,
+			y_clear + CLIP_VERT_VISIBLE_SPACE(mw));
+	}
+
+	/*
+	 * Set the scrollbar to a reasonable position.
+	 */
+	XtVaSetValues(VertScrollChild(mw), XmNvalue, call_data->value, NULL);
+#ifdef	XBAE_VERBOSE
+	fprintf(stderr, "Set Scrollbar to %d (max %d)\n",
+		call_data->value, xbaeMaxVertScroll(mw));
+#endif
+
+	/*
+	 * Perform the actual redraw. The call to draw the grid shadows
+	 * must be done after any XClearAreas() to ensure we don't redraw
+	 * more than we have to; that is, we could put the call in the
+	 * redraw cells routine, but we would end up occasionally redrawing
+	 * more than once.
+	 */
+	xbaeRedrawLabelsAndFixed(mw, &fixed);
+	xbaeRedrawCells(mw, &nonfixed);
 }
 
 /*
@@ -540,10 +645,7 @@ XmScrollBarCallbackStruct *call_data;
  */
 /* ARGSUSED */
 void
-xbaeScrollHorizCB(w, client_data, call_data)
-Widget w;
-XtPointer client_data;
-XmScrollBarCallbackStruct *call_data;
+xbaeScrollHorizCB(Widget w, XtPointer client_data, XmScrollBarCallbackStruct *call_data)
 {
     XbaeMatrixWidget mw = (XbaeMatrixWidget) XtParent(w);
     Rectangle fixed, nonfixed;
@@ -551,8 +653,6 @@ XmScrollBarCallbackStruct *call_data;
     int horiz_sb_offset = HORIZ_SB_OFFSET(mw);
     int trailing_fixed_row_label_offset = TRAILING_FIXED_ROW_LABEL_OFFSET(mw);
     int trailing_fixed_row_height = TRAILING_FIXED_ROW_HEIGHT(mw);
-/*    int trailing_fixed_column_label_offset =
-	TRAILING_FIXED_COLUMN_LABEL_OFFSET(mw);*/
     int column_label_height = COLUMN_LABEL_HEIGHT(mw);
     int fixed_column_label_offset = FIXED_COLUMN_LABEL_OFFSET(mw);
     int fixed_column_width = FIXED_COLUMN_WIDTH(mw);
@@ -560,8 +660,7 @@ XmScrollBarCallbackStruct *call_data;
     Boolean need_vert_fill = NEED_VERT_FILL(mw);
     Boolean has_attached_trailing_rows = HAS_ATTACHED_TRAILING_ROWS(mw);
     Boolean need_vert_dead_space_fill = NEED_VERT_DEAD_SPACE_FILL(mw);
-    
-    /*printf("%d\n", HORIZ_ORIGIN(mw));*/
+
     /*
      * Didn't scroll
      */
@@ -598,11 +697,31 @@ XmScrollBarCallbackStruct *call_data;
 		mw->matrix.current_column < TRAILING_HORIZ_ORIGIN(mw))
     {
 	XtMoveWidget(TextChild(mw),
-		     TextChild(mw)->core.x + (HORIZ_ORIGIN(mw) -
-					      call_data->value),
+		     TextChild(mw)->core.x + (HORIZ_ORIGIN(mw) - call_data->value),
 		     TextChild(mw)->core.y);
     }
 
+    /* Cell widgets that are in the horizontal scrolling region 
+     * need to scroll along as well... Linas */
+    if (mw->matrix.cell_widgets) {
+       int roe, kol;
+
+       for (roe=0; roe < (mw->matrix.rows); roe++) { 
+          for (kol=mw->matrix.fixed_columns;
+               kol < TRAILING_HORIZ_ORIGIN(mw); kol ++) {
+             Widget w = mw->matrix.cell_widgets[roe][kol];
+             if (w) {
+                if (XtIsManaged (w)) {
+                   int x = w->core.x;
+                   int y = w->core.y;
+                   x += HORIZ_ORIGIN(mw) - call_data->value;
+                   XtMoveWidget(w, x, y);
+                }
+             }
+          }
+       }
+    }
+
     /*
      * Now we can adjust our horizontal origin
      */
@@ -668,7 +787,7 @@ XmScrollBarCallbackStruct *call_data;
 			   trailing_fixed_row_label_offset +
 			   trailing_fixed_row_height,
 			   MATRIX_HORIZ_VISIBLE_SPACE(mw), 0, False);
-	    
+
 	    if (need_vert_dead_space_fill)
 		XClearArea(XtDisplay(mw), XtWindow(mw),
 			   fixed_column_label_offset,
@@ -764,7 +883,7 @@ XmScrollBarCallbackStruct *call_data;
 		      src_x, horiz_sb_offset,
 		      width, column_label_height,
 		      dest_x, horiz_sb_offset);
-	
+
 	/*
 	 * Copy trailing filled portion of the columns if necessary
 	 */
@@ -778,7 +897,7 @@ XmScrollBarCallbackStruct *call_data;
 			  width, FILL_VERT_HEIGHT(mw),
 			  dest_x, trailing_fixed_row_label_offset +
 			  trailing_fixed_row_height);
-	    
+
 	    if (need_vert_dead_space_fill)
 		XCopyArea(XtDisplay(mw), XtWindow(mw), XtWindow(mw),
 			  mw->matrix.draw_gc,
@@ -786,7 +905,7 @@ XmScrollBarCallbackStruct *call_data;
 			  width, vert_dead_space_height,
 			  dest_x, unattached_trailing_rows_offset);
 	}
-	
+
 	/*
 	 * Clear newly scrolled chunk of fixed rows on TopClip
 	 */
@@ -856,14 +975,17 @@ XmScrollBarCallbackStruct *call_data;
  * non-fixed cells.
  */
 void
-xbaeRedrawCells(mw, expose)
-XbaeMatrixWidget mw;
-Rectangle *expose;
+xbaeRedrawCells(XbaeMatrixWidget mw, Rectangle *expose)
 {
     int startCol, endCol, startRow, endRow, i, j;
     Rectangle rect;
     Boolean set_mask = False;
 
+#ifdef	XBAE_VERBOSE
+	fprintf(stderr, "xbaeRedrawCells(x %d .. %d, y %d .. %d)\n",
+		expose->x1, expose->x2, expose->y1, expose->y2);
+#endif
+
     if ((mw->matrix.disable_redisplay) || (!mw->matrix.rows) ||
 	(!mw->matrix.columns))
 	return;
@@ -884,11 +1006,16 @@ Rectangle *expose;
      */
     startCol = xbaeXtoCol(mw, rect.x1 + HORIZ_ORIGIN(mw));
     endCol = xbaeXtoCol(mw, rect.x2 + HORIZ_ORIGIN(mw));
-    startRow = YtoRow(mw, rect.y1 + mw->matrix.first_row_offset) +
+    startRow = xbaeYtoRow(mw, rect.y1 + mw->matrix.first_row_offset) +
 	VERT_ORIGIN(mw);
-    endRow = YtoRow(mw, rect.y2 + mw->matrix.first_row_offset) +
+    endRow = xbaeYtoRow(mw, rect.y2 + mw->matrix.first_row_offset) +
 	VERT_ORIGIN(mw);
 
+#ifdef	XBAE_VERBOSE
+    fprintf(stderr, "xbaeRedrawCells(rows %d..%d, cols %d..%d)\n",
+	startRow, endRow, startCol, endCol);
+#endif
+
     SANITY_CHECK_ROW(mw, startRow);
     SANITY_CHECK_ROW(mw, endRow);
     SANITY_CHECK_COLUMN(mw, startCol);
@@ -917,17 +1044,53 @@ Rectangle *expose;
 	xbaeSetClipMask(mw, CLIP_NONE);
 }
 
+/*
+ * These used to be simple macros.
+ */
+Dimension
+xbaeMatrixFixedRowHeight(XbaeMatrixWidget mw)
+{
+	int		i;
+	Dimension	h;
+
+	if (mw->matrix.row_heights) {
+		h = 0;
+		for (i=0; i<mw->matrix.fixed_rows; i++)
+			h += mw->matrix.row_heights[i];
+	} else {
+		h = (int)mw->matrix.fixed_rows * ROW_HEIGHT(mw);
+	}
+
+	return h;
+}
+
+Dimension
+xbaeMatrixTrailingFixedRowHeight(XbaeMatrixWidget mw)
+{
+	int		i;
+	Dimension	h = 0;
+
+	if (mw->matrix.row_heights == 0)
+		return ((int)mw->matrix.trailing_fixed_rows * ROW_HEIGHT(mw));
+
+	for (i= (mw->matrix.rows - mw->matrix.trailing_fixed_rows);
+			i<mw->matrix.rows; i++)
+		h += mw->matrix.row_heights[i];
+	return h;
+}
+
 /*
  * Redraw the row and column labels and the cells in fixed rows/columns
  * that are overlapped by the Rectangle argument.
+ *
+ * FIX ME DANNY
+ * This version is being worked on for variable height rows.
  */
 void
-xbaeRedrawLabelsAndFixed(mw, expose)
-XbaeMatrixWidget mw;
-Rectangle *expose;
+xbaeRedrawLabelsAndFixed(XbaeMatrixWidget mw, Rectangle *expose)
 {
     /*
-     * Set up some local variables to avoid calling too many macros :p
+     * Set up some local variables to avoid calling too many macros
      */
     int horiz_sb_offset = HORIZ_SB_OFFSET(mw);
     int vert_sb_offset = VERT_SB_OFFSET(mw);
@@ -952,17 +1115,16 @@ Rectangle *expose;
     /*
      * Handle the row labels that are in fixed rows
      */
-    if (mw->matrix.rows && mw->matrix.fixed_rows && mw->matrix.row_labels)
-    {
+    if (mw->matrix.rows && mw->matrix.fixed_rows && mw->matrix.row_labels) {
 	Rectangle rect;
-	
+
 	/*
 	 * Get the Rectangle enclosing the fixed row labels
 	 */
 	SETRECT(rect, vert_sb_offset, row_label_offset,
 		vert_sb_offset + row_label_width - 1,
 		fixed_row_label_offset - 1);
-	
+
 	/*
 	 * If the expose Rectangle overlaps, then some labels must be redrawn
 	 */
@@ -981,9 +1143,9 @@ Rectangle *expose;
 	    /*
 	     * Redraw each label that was intersected
 	     */
-	    endRow = YtoRow(mw, intersect.y2 + mw->matrix.first_row_offset);
+	    endRow = xbaeYtoRow(mw, intersect.y2 + mw->matrix.first_row_offset);
 	    SANITY_CHECK_ROW(mw, endRow);
-	    for (i = YtoRow(mw, intersect.y1 + mw->matrix.first_row_offset),
+	    for (i = xbaeYtoRow(mw, intersect.y1 + mw->matrix.first_row_offset),
 		     SANITY_CHECK_ROW(mw, i);
 		 i <= endRow; i++)
 		xbaeDrawRowLabel(mw, i, False);
@@ -994,8 +1156,7 @@ Rectangle *expose;
      * Handle the row labels that are in trailing fixed rows
      */
     if (mw->matrix.rows && mw->matrix.trailing_fixed_rows &&
-	mw->matrix.row_labels)
-    {
+	mw->matrix.row_labels) {
 	Rectangle rect;
 
 	/*
@@ -1024,9 +1185,9 @@ Rectangle *expose;
 	    /*
 	     * Redraw each label that was intersected
 	     */
-	    endRow = YtoRow(mw, intersect.y2) + TRAILING_VERT_ORIGIN(mw);
+	    endRow = xbaeYtoRow(mw, intersect.y2) + TRAILING_VERT_ORIGIN(mw);
 	    SANITY_CHECK_ROW(mw, endRow);
-	    for (i = YtoRow(mw, intersect.y1) + TRAILING_VERT_ORIGIN(mw),
+	    for (i = xbaeYtoRow(mw, intersect.y1) + TRAILING_VERT_ORIGIN(mw),
 		 SANITY_CHECK_ROW(mw, i); i <= endRow; i++)
 		xbaeDrawRowLabel(mw, i, False);
 	}
@@ -1035,8 +1196,7 @@ Rectangle *expose;
     /*
      * Handle row labels that aren't in fixed rows
      */
-    if (mw->matrix.row_labels && mw->matrix.rows)
-    {
+    if (mw->matrix.row_labels && mw->matrix.rows) {
 	Rectangle rect;
 
 	/*
@@ -1063,7 +1223,7 @@ Rectangle *expose;
 
 	    /*
 	     * Translate 'intersect' to take into account any fixed rows.
-	     * This gets it back into the coord system expected by YtoRow().
+	     * This gets it back into the coord system expected by xbaeYtoRow().
 	     */
 	    intersect.y1 += FIXED_ROW_HEIGHT(mw);
 	    intersect.y2 += FIXED_ROW_HEIGHT(mw);
@@ -1071,10 +1231,10 @@ Rectangle *expose;
 	    /*
 	     * Redraw each label that was intersected
 	     */
-	    endRow = YtoRow(mw, intersect.y2) + VERT_ORIGIN(mw);
+	    endRow = xbaeYtoRow(mw, intersect.y2) + VERT_ORIGIN(mw);
 	    SANITY_CHECK_ROW(mw, endRow);
-	    for (i = YtoRow(mw, intersect.y1) + VERT_ORIGIN(mw),
-		 SANITY_CHECK_ROW(mw, i); i <= endRow;	 i++)
+	    for (i = xbaeYtoRow(mw, intersect.y1) + VERT_ORIGIN(mw),
+			 SANITY_CHECK_ROW(mw, i); i <= endRow;	 i++)
 		xbaeDrawRowLabel(mw, i, False);
 	}
     }
@@ -1161,7 +1321,7 @@ Rectangle *expose;
 	    endCol = xbaeXtoTrailingCol(mw, intersect.x2);
 	    SANITY_CHECK_COLUMN(mw, endCol);
 	    for (i = xbaeXtoTrailingCol(mw, intersect.x1),
-		 SANITY_CHECK_COLUMN(mw, i); i <= endCol; i++)
+			SANITY_CHECK_COLUMN(mw, i); i <= endCol; i++)
 		xbaeDrawColumnLabel(mw, i, False);
 	}
     }
@@ -1171,7 +1331,7 @@ Rectangle *expose;
      */
     if (mw->matrix.column_labels && mw->matrix.columns)
     {
-	Rectangle rect;
+	Rectangle rect, corrected_expose;
 
 	/*
 	 * Get the Rectangle enclosing the non-fixed column labels
@@ -1181,10 +1341,18 @@ Rectangle *expose;
 		fixed_column_label_offset + VISIBLE_WIDTH(mw) - 1,
 		horiz_sb_offset + column_label_height - 1);
 
+	/*
+	 * "untransform" the y component of the expose rectangle
+	 * column headings have transfomed x coordinates but original y coordinates
+	 */
+	SETRECT(corrected_expose,
+		expose->x1, expose->y1 - mw->matrix.matrix_scroll_mgr->offset_y,
+		expose->x2, expose->y2 - mw->matrix.matrix_scroll_mgr->offset_y);
+
 	/*
 	 * If the expose Rectangle overlaps, then some labels must be redrawn
 	 */
-	if (OVERLAP(*expose, rect))
+	if (OVERLAP(corrected_expose, rect))
 	{
 	    Rectangle intersect;
 	    int endCol, i;
@@ -1194,7 +1362,7 @@ Rectangle *expose;
 	     * Rectangle along the X axis.  The resulting Rectangle will
 	     * be in 'rect's coordinate system.
 	     */
-	    X_INTERSECT(rect, *expose, intersect);
+	    X_INTERSECT(rect, corrected_expose, intersect);
 
 	    /*
 	     * Translate 'intersect' to take into account any fixed columns.
@@ -1209,7 +1377,7 @@ Rectangle *expose;
 	    endCol = xbaeXtoCol(mw, intersect.x2 + HORIZ_ORIGIN(mw));
 	    SANITY_CHECK_COLUMN(mw, endCol);
 	    for (i = xbaeXtoCol(mw, intersect.x1 + HORIZ_ORIGIN(mw)),
-		 SANITY_CHECK_COLUMN(mw, i); i <= endCol; i++)
+			SANITY_CHECK_COLUMN(mw, i); i <= endCol; i++)
 		xbaeDrawColumnLabel(mw, i, False);
 	}
     }
@@ -1255,11 +1423,11 @@ Rectangle *expose;
 	    startCol = xbaeXtoCol(mw, intersect.x1 + HORIZ_ORIGIN(mw));
 	    endCol = xbaeXtoCol(mw, intersect.x2 + HORIZ_ORIGIN(mw));
 
-	    startRow = YtoRow(mw, intersect.y1) +
+	    startRow = xbaeYtoRow(mw, intersect.y1) +
 		(mw->matrix.fixed_rows
 		 ? 0
 		 : VERT_ORIGIN(mw));
-	    endRow = YtoRow(mw, intersect.y2) +
+	    endRow = xbaeYtoRow(mw, intersect.y2) +
 		(mw->matrix.fixed_rows
 		 ? 0
 		 : VERT_ORIGIN(mw));
@@ -1320,8 +1488,8 @@ Rectangle *expose;
 	    startCol = xbaeXtoCol(mw, intersect.x1 + HORIZ_ORIGIN(mw));
 	    endCol = xbaeXtoCol(mw, intersect.x2 + HORIZ_ORIGIN(mw));
 
-	    startRow = YtoRow(mw, intersect.y1) + TRAILING_VERT_ORIGIN(mw);
-	    endRow = YtoRow(mw, intersect.y2) + TRAILING_VERT_ORIGIN(mw);
+	    startRow = xbaeYtoRow(mw, intersect.y1) + TRAILING_VERT_ORIGIN(mw);
+	    endRow = xbaeYtoRow(mw, intersect.y2) + TRAILING_VERT_ORIGIN(mw);
 
 	    /*
 	     * Redraw each cell that was intersected
@@ -1423,16 +1591,16 @@ Rectangle *expose;
 			     ? 0
 			     : HORIZ_ORIGIN(mw)));
 
-	    startRow = redrawFixedRows ? 0 : YtoRow(mw, intersect.y1) +
+	    startRow = redrawFixedRows ? 0 : xbaeYtoRow(mw, intersect.y1) +
 		VERT_ORIGIN(mw);
 	    if (redrawTrailingFixedRows)
 	    {
-		skipRow = YtoRow(mw, intersect.y2) + VERT_ORIGIN(mw) -
+		skipRow = xbaeYtoRow(mw, intersect.y2) + VERT_ORIGIN(mw) -
 		    mw->matrix.trailing_fixed_rows + 1;
 		endRow = mw->matrix.rows - 1;
 	    }
 	    else
-		endRow = YtoRow(mw, intersect.y2) + VERT_ORIGIN(mw);
+		endRow = xbaeYtoRow(mw, intersect.y2) + VERT_ORIGIN(mw);
 
 	    /*
 	     * Redraw each cell that was intersected
@@ -1561,17 +1729,17 @@ Rectangle *expose;
 	    startCol = xbaeXtoTrailingCol(mw, intersect.x1);
 	    endCol = xbaeXtoTrailingCol(mw, intersect.x2);
 
-	    startRow = redrawFixedRows ? 0 : YtoRow(mw, intersect.y1) +
+	    startRow = redrawFixedRows ? 0 : xbaeYtoRow(mw, intersect.y1) +
 		VERT_ORIGIN(mw);
 
 	    if (redrawTrailingFixedRows)
 	    {
-		skipRow = YtoRow(mw, intersect.y2) + VERT_ORIGIN(mw) -
+		skipRow = xbaeYtoRow(mw, intersect.y2) + VERT_ORIGIN(mw) -
 		    mw->matrix.trailing_fixed_rows + 1;
 		endRow = mw->matrix.rows - 1;
 	    }
 	    else
-		endRow = YtoRow(mw, intersect.y2) + VERT_ORIGIN(mw);
+		endRow = xbaeYtoRow(mw, intersect.y2) + VERT_ORIGIN(mw);
 
 	    /*
 	     * Redraw each cell that was intersected
@@ -1594,7 +1762,7 @@ Rectangle *expose;
 		    if (redrawTrailingFixedRows && (i == skipRow) &&
 			(endRow > skipRow))
 			i = TRAILING_VERT_ORIGIN(mw);
-		    
+
 		    /*
 		     * If we need to clip the vertical fill
 		     */
@@ -1616,6 +1784,7 @@ Rectangle *expose;
 	}
     }
 
+	/* Danny Here */
     /*
      * Draw a shadow just inside row/column labels and around outer edge
      * of clip widget.	We can't use height of clip widget because it is
@@ -1638,12 +1807,12 @@ Rectangle *expose;
 	{
 	    width = mw->core.width - row_label_width -
 		VERT_SB_SPACE(mw);
-	    
+
 	    height = mw->core.height - column_label_height -
 		HORIZ_SB_SPACE(mw) ;
 	}
-	
-	DRAW_SHADOW(XtDisplay(mw), XtWindow(mw),
+
+	DRAW_SHADOW(XtDisplay(mw), XtWindow(mw), "win",
 		    mw->manager.top_shadow_GC,
 		    mw->manager.bottom_shadow_GC,
 		    mw->manager.shadow_thickness,
