@@ -1,6 +1,8 @@
 /*
  * Copyright(c) 1992 Bell Communications Research, Inc. (Bellcore)
  * Copyright(c) 1995-99 Andrew Lister
+ * Copyright  1999, 2000, 2001 by the LessTif Developers.
+ *
  *                        All rights reserved
  * Permission to use, copy, modify and distribute this material for
  * any purpose and without fee is hereby granted, provided that the
@@ -22,7 +24,7 @@
  *
  * MatrixWidget Author: Andrew Wason, Bellcore, aw@bae.bellcore.com
  *
- * $Id: Utils.c,v 1.1 1999/08/04 19:47:01 danny Exp $
+ * $Id: Utils.c,v 1.30 2001/05/29 22:06:06 dannybackx Exp $
  */
 
 /*
@@ -33,22 +35,40 @@
 #include <XbaeConfig.h>
 #endif
 
+#include <stdio.h>
+#include <string.h>
+
 #include <Xm/Xm.h>
 #include <Xbae/MatrixP.h>
 #include <Xbae/Macros.h>
 #include <Xbae/Utils.h>
 #include <Xbae/Actions.h>
 #include <Xm/ScrollBar.h>
+
+#ifdef	WITH_DMALLOC
+#include <dmalloc.h>
+#endif
+
+
 /*
  * Return the top and bottom-most visible non-fixed row
  */
 void
-xbaeGetVisibleRows(mw, top_row, bottom_row)
-XbaeMatrixWidget mw;
-int *top_row, *bottom_row;
+xbaeGetVisibleRows(XbaeMatrixWidget mw, int *top_row, int *bottom_row)
 {
+    int		i;
+    Dimension	h;
+
     *top_row = VERT_ORIGIN(mw) + mw->matrix.fixed_rows;
-    *bottom_row = *top_row + (VISIBLE_HEIGHT(mw) - 1) / ROW_HEIGHT(mw);
+    h = 0;
+    for (i=VERT_ORIGIN(mw); i<mw->matrix.rows && h < VISIBLE_HEIGHT(mw); i++)
+	h += mw->matrix.row_heights[i];
+    *bottom_row = i;
+
+#ifdef	XBAE_VERBOSE
+    fprintf(stderr, "xbaeGetVisibleRows -> top %d bottom %d\n", *top_row, *bottom_row);
+#endif
+
     SANITY_CHECK_ROW(mw, *bottom_row);
 }
 
@@ -56,9 +76,7 @@ int *top_row, *bottom_row;
  * Return the left and right-most visible non-fixed column
  */
 void
-xbaeGetVisibleColumns(mw, left_column, right_column)
-XbaeMatrixWidget mw;
-int *left_column, *right_column;
+xbaeGetVisibleColumns(XbaeMatrixWidget mw, int *left_column, int *right_column)
 {
     *left_column = xbaeXtoCol(mw, FIXED_COLUMN_WIDTH(mw) + HORIZ_ORIGIN(mw));
     *right_column = xbaeXtoCol(mw, FIXED_COLUMN_WIDTH(mw) + HORIZ_ORIGIN(mw) +
@@ -70,9 +88,9 @@ int *left_column, *right_column;
  * the visible non-fixed cells
  */
 void
-xbaeGetVisibleCells(mw, top_row, bottom_row, left_column, right_column)
-XbaeMatrixWidget mw;
-int *top_row, *bottom_row, *left_column, *right_column;
+xbaeGetVisibleCells(XbaeMatrixWidget mw,
+	int *top_row, int *bottom_row,
+	int *left_column, int *right_column)
 {
     xbaeGetVisibleRows(mw, top_row, bottom_row);
     xbaeGetVisibleColumns(mw, left_column, right_column);
@@ -85,8 +103,7 @@ int *top_row, *bottom_row, *left_column, *right_column;
  * If we can't make leftColumn the left column, make it as close as possible.
  */
 void
-xbaeAdjustLeftColumn(mw)
-XbaeMatrixWidget mw;
+xbaeAdjustLeftColumn(XbaeMatrixWidget mw)
 {
     int y;
     int i;
@@ -138,10 +155,9 @@ XbaeMatrixWidget mw;
  * If we can't make topRow the top row, make it as close as possible.
  */
 void
-xbaeAdjustTopRow(mw)
-XbaeMatrixWidget mw;
+xbaeAdjustTopRow(XbaeMatrixWidget mw)
 {
-    int rows_visible = VISIBLE_HEIGHT(mw) / ROW_HEIGHT(mw);
+    int rows_visible = VISIBLE_HEIGHT(mw) / SOME_ROW_HEIGHT(mw, mw->matrix.top_row);
 
     /*
      * If we have less than one full row visible, then count it as a full row
@@ -169,13 +185,12 @@ XbaeMatrixWidget mw;
  * Does not check if the cell is actually visible before clearing it.
  */
 void
-xbaeClearCell(mw, row, column)
-XbaeMatrixWidget mw;
-int row, column;
+xbaeClearCell(XbaeMatrixWidget mw, int row, int column)
 {
     int x, y;
     Boolean fixed = IS_FIXED(mw, row, column);
-    Window win = fixed ? XtWindow(mw) : XtWindow(ClipChild(mw));
+    Widget	cw;	/* Clip widget, not used */
+    Window win = xbaeGetCellWindow(mw, &cw, row, column);
 
     if (!win || mw->matrix.disable_redisplay)
 	return;
@@ -192,17 +207,16 @@ int row, column;
 	 (y < (int)ROW_LABEL_OFFSET(mw))))
 	return;
 #endif
+
     XClearArea(XtDisplay(mw), win, x, y, COLUMN_WIDTH(mw, column),
-	       ROW_HEIGHT(mw), fixed);
+	       SOME_ROW_HEIGHT(mw, row), fixed);
 }
 
 /*
  * Return True if a row is visible on the screen (not scrolled totally off)
  */
 Boolean
-xbaeIsRowVisible(mw, row)
-XbaeMatrixWidget mw;
-int row;
+xbaeIsRowVisible(XbaeMatrixWidget mw, int row)
 {
     /*
      * If we are not in a fixed row or trailing fixed row,
@@ -216,13 +230,13 @@ int row;
 	if (row >= VERT_ORIGIN(mw))
 	{
 	    double height = ((double)ClipChild(mw)->core.height /
-			     ROW_HEIGHT(mw)) + VERT_ORIGIN(mw);
+			     SOME_ROW_HEIGHT(mw, row)) + VERT_ORIGIN(mw);
 	    if (row < height)
 		return True;
 	    
 	    if ((int)ClipChild(mw)->core.height >
 		(int)TEXT_HEIGHT_OFFSET(mw) &&
-		(int)ClipChild(mw)->core.height < ROW_HEIGHT(mw) &&
+		(int)ClipChild(mw)->core.height < SOME_ROW_HEIGHT(mw, row) &&
 		row == VERT_ORIGIN(mw))
 
 		return True;
@@ -238,9 +252,7 @@ int row;
  * Return True if a column is visible on the screen (not scrolled totally off)
  */
 Boolean
-xbaeIsColumnVisible(mw, column)
-XbaeMatrixWidget mw;
-int column;
+xbaeIsColumnVisible(XbaeMatrixWidget mw, int column)
 {
     /*
      * If we are not in a fixed column, see if we are on the screen
@@ -273,9 +285,7 @@ int column;
  * Return True if a cell is visible on the screen (not scrolled totally off)
  */
 Boolean
-xbaeIsCellVisible(mw, row, column)
-XbaeMatrixWidget mw;
-int row, column;
+xbaeIsCellVisible(XbaeMatrixWidget mw, int row, int column)
 {
     return xbaeIsRowVisible(mw, row) && xbaeIsColumnVisible(mw, column);
 }
@@ -284,11 +294,9 @@ int row, column;
  * Scroll a row so it is visible on the screen.
  */
 void
-xbaeMakeRowVisible(mw, row)
-XbaeMatrixWidget mw;
-int row;
+xbaeMakeRowVisible(XbaeMatrixWidget mw, int row)
 {
-    int rows_visible;
+    int rows_visible, rel_row;
     int value, slider_size, increment, page_increment, vert_value;
 
     /*
@@ -302,8 +310,8 @@ int row;
      * Calculate the number of rows visible. If less than one full
      * row is visible, use one full row.
      */
-    row -= mw->matrix.fixed_rows;
-    rows_visible = VISIBLE_HEIGHT(mw) / ROW_HEIGHT(mw);
+    rel_row = row - mw->matrix.fixed_rows;
+    rows_visible = VISIBLE_HEIGHT(mw) / SOME_ROW_HEIGHT(mw, row);
     if (rows_visible == 0)
 	rows_visible = 1;
 
@@ -311,10 +319,10 @@ int row;
      * Figure out the new value of the VSB to scroll this cell
      * onto the screen (the VSB uses row coordinates instead of pixels)
      */
-    if (row < VERT_ORIGIN(mw))
-	vert_value = row;
-    else if (row >= rows_visible + VERT_ORIGIN(mw))
-	vert_value = row - rows_visible + 1;
+    if (rel_row < VERT_ORIGIN(mw))
+	vert_value = rel_row;
+    else if (rel_row >= rows_visible + VERT_ORIGIN(mw))
+	vert_value = rel_row - rows_visible + 1;
     else
 	vert_value = VERT_ORIGIN(mw);
 
@@ -326,7 +334,12 @@ int row;
     {
 	XmScrollBarGetValues(VertScrollChild(mw), &value,
 			     &slider_size, &increment, &page_increment);
-	XmScrollBarSetValues(VertScrollChild(mw), vert_value,
+#ifdef	XBAE_VERBOSE
+	/* Danny */
+	fprintf(stderr, "MakeRowVisible(%d) : Change VSB %d -> %d\n",
+		row, value, ROW_POSITION(mw, vert_value));
+#endif
+	XmScrollBarSetValues(VertScrollChild(mw), ROW_POSITION(mw, vert_value),
 			     slider_size, increment, page_increment, True);
     }
 }
@@ -335,9 +348,7 @@ int row;
  * Scroll a column so it is visible on the screen.
  */
 void
-xbaeMakeColumnVisible(mw, column)
-XbaeMatrixWidget mw;
-int column;
+xbaeMakeColumnVisible(XbaeMatrixWidget mw, int column)
 {
     int value, slider_size, increment, page_increment, x, horiz_value;
 
@@ -363,13 +374,13 @@ int column;
     else if (x + COLUMN_WIDTH(mw, column) >
 	     VISIBLE_WIDTH(mw) + HORIZ_ORIGIN(mw))
     {
-	int off = (x + COLUMN_WIDTH(mw, column)) - (VISIBLE_WIDTH(mw) +
+	int offs = (x + COLUMN_WIDTH(mw, column)) - (VISIBLE_WIDTH(mw) +
 						    HORIZ_ORIGIN(mw));
 
-	if (x - off < HORIZ_ORIGIN(mw))
+	if (x - offs < HORIZ_ORIGIN(mw))
 	    horiz_value = x;
 	else
-	    horiz_value = HORIZ_ORIGIN(mw) + off;
+	    horiz_value = HORIZ_ORIGIN(mw) + offs;
     }
     else
 	horiz_value = HORIZ_ORIGIN(mw);
@@ -391,12 +402,11 @@ int column;
  * Scrolls a fixed or non-fixed cell so it is visible on the screen.
  */
 void
-xbaeMakeCellVisible(mw, row, column)
-XbaeMatrixWidget mw;
-int row, column;
+xbaeMakeCellVisible(XbaeMatrixWidget mw, int row, int column)
 {
     if (!xbaeIsRowVisible(mw, row))
 	xbaeMakeRowVisible(mw, row);
+
     if (!xbaeIsColumnVisible(mw, column))
 	xbaeMakeColumnVisible(mw, column);
 }
@@ -406,9 +416,7 @@ int row, column;
  * drawing non-fixed column labels and fixed rows.
  */
 void
-xbaeSetClipMask(mw, clip_reason)
-XbaeMatrixWidget mw;
-unsigned int clip_reason;
+xbaeSetClipMask(XbaeMatrixWidget mw, unsigned int clip_reason)
 {
     XRectangle r[2];
     int n = 1;
@@ -488,13 +496,11 @@ unsigned int clip_reason;
 		       0, 0, r, n, Unsorted);
 }
 
-
 /*
  * Get the total pixel width of the non-fixed cell area
  */
 void
-xbaeGetCellTotalWidth(mw)
-XbaeMatrixWidget mw;
+xbaeGetCellTotalWidth(XbaeMatrixWidget mw)
 {
     int i, columns;
 
@@ -512,8 +518,7 @@ XbaeMatrixWidget mw;
  * Cache the pixel position of each column
  */
 void
-xbaeGetColumnPositions(mw)
-XbaeMatrixWidget mw;
+xbaeGetColumnPositions(XbaeMatrixWidget mw)
 {
     int i, x;
 
@@ -524,20 +529,24 @@ XbaeMatrixWidget mw;
 }
 
 void
-#if NeedFunctionPrototypes
+xbaeGetRowPositions(XbaeMatrixWidget mw)
+{
+    int i, y;
+
+    for (i = 0, y = 0;
+	 i < mw->matrix.rows;
+	 y += SOME_ROW_HEIGHT(mw, i), i++)
+	ROW_POSITION(mw, i) = y;
+}
+
+void
 xbaeComputeSize(XbaeMatrixWidget mw, Boolean compute_width,
 		Boolean compute_height)
-#else
-xbaeComputeSize(mw, compute_width, compute_height)
-XbaeMatrixWidget mw;
-Boolean compute_width;
-Boolean compute_height;
-#endif
 {
     unsigned long full_width = NON_FIXED_TOTAL_WIDTH(mw) +
 	FIXED_COLUMN_WIDTH(mw) + TRAILING_FIXED_COLUMN_WIDTH(mw) +
 	ROW_LABEL_WIDTH(mw) + 2 * mw->manager.shadow_thickness;
-    unsigned long full_height = CELL_TOTAL_HEIGHT(mw) +
+    unsigned long full_height = xbaeCellTotalHeight(mw) +
 	FIXED_ROW_HEIGHT(mw) + TRAILING_FIXED_ROW_HEIGHT(mw) +
 	COLUMN_LABEL_HEIGHT(mw) + 2 * mw->manager.shadow_thickness;
     unsigned long width, height;
@@ -566,14 +575,21 @@ Boolean compute_height;
      * Otherwise, if the compute_height flag is set, then we are full height.
      * Otherwise we keep whatever height we are.
      */
-    if (mw->matrix.visible_rows)
-	height = mw->matrix.visible_rows * ROW_HEIGHT(mw) +
-	    TRAILING_FIXED_ROW_HEIGHT(mw) + FIXED_ROW_HEIGHT(mw) +
+    if (mw->matrix.visible_rows) {
+	int	i;
+	height = TRAILING_FIXED_ROW_HEIGHT(mw) + FIXED_ROW_HEIGHT(mw) +
 	    COLUMN_LABEL_HEIGHT(mw) + 2 * mw->manager.shadow_thickness;
-    else if (compute_height)
+
+	for (i=0; i<mw->matrix.visible_rows; i++)
+		if (i < mw->matrix.rows)
+			height += SOME_ROW_HEIGHT(mw, i);
+		else
+			height += ROW_HEIGHT(mw);
+    } else if (compute_height) {
 	height = full_height;
-    else
+    } else {
 	height = mw->core.height;
+    }
 
     /*
      * Store our calculated size.
@@ -612,14 +628,18 @@ Boolean compute_height;
      */
     mw->matrix.desired_width = mw->core.width;
     mw->matrix.desired_height = mw->core.height;
+
+#ifdef	XBAE_VERBOSE
+    fprintf(stderr, "xbaeComputeSize -> w %d h %d\n",
+	mw->matrix.desired_width, mw->matrix.desired_height);
+#endif
 }
 
 /*
  * Return the length of the longest row label
  */
 short
-xbaeMaxRowLabel(mw)
-XbaeMatrixWidget mw;
+xbaeMaxRowLabel(XbaeMatrixWidget mw)
 {
     int i;
     short max = 0, len;
@@ -637,9 +657,7 @@ XbaeMatrixWidget mw;
 }
 
 void
-xbaeParseColumnLabel(label, lines)
-String label;
-ColumnLabelLines lines;
+xbaeParseColumnLabel(String label, ColumnLabelLines lines)
 {
     char *nl;
 
@@ -685,6 +703,8 @@ ColumnLabelLines lines;
     }
 }
 
+/* FIX ME : this function contains some as yet unfixed ROW_HEIGHT behaviour.
+   Danny 23/5/2000 */
 /*
  * Convert an x/y pixel position to the row/column cell position it picks.
  * 'cell' specifies whether the x/y coord is relative to the fixed cells
@@ -697,11 +717,8 @@ ColumnLabelLines lines;
  * may be useful nevertheless.
  */
 Boolean
-xbaeXYToRowCol(mw, x, y, row, column, cell)
-XbaeMatrixWidget mw;
-int *x, *y;
-int *row, *column;
-CellType cell;
+xbaeXYToRowCol(XbaeMatrixWidget mw, int *x, int *y,
+		int *row, int *column, CellType cell)
 {
     Rectangle rect;
     unsigned int inBox = CLIP_NONE;
@@ -721,12 +738,55 @@ CellType cell;
     int column_label_offset = COLUMN_LABEL_OFFSET(mw);
     int row_height = ROW_HEIGHT(mw);
     int unattached_trailing_rows_offset = UNATTACHED_TRAILING_ROWS_OFFSET(mw);
+    int	i;
     
     *row = -1;
     *column = -1;
 
-    switch (cell)
-    {
+#ifdef XBAE_VERBOSE
+	fprintf(stderr, "xbaeXYToRowCol(%d,%d, %s)\n",
+		*x, *y,
+		(cell == FixedCell) ? "FixedCell" :
+		(cell == NonFixedCell) ? "NonFixedCell" :
+		(cell == RowLabelCell) ? "RowLabelCell" :
+		(cell == ColumnLabelCell) ? "ColumnLabelCell" : "??");
+#endif
+
+    switch (cell) {
+    case NonFixedCell:
+	/*
+	 * Translate the point to take into account fixed rows or columns.
+	 */
+	*x += FIXED_COLUMN_WIDTH(mw);
+	*y += FIXED_ROW_HEIGHT(mw);
+
+	/*
+	 * Convert the new point to a row/column position
+	 */
+	*row = xbaeYtoRow(mw,
+		*y + mw->matrix.first_row_offset) + VERT_ORIGIN(mw);
+	*column = xbaeXtoCol(mw, *x + HORIZ_ORIGIN(mw));
+
+	/*
+	 * Sanity check the result
+	 */
+	if (*row >= TRAILING_VERT_ORIGIN(mw) ||
+	    *column >= TRAILING_HORIZ_ORIGIN(mw) ||
+	    *row < 0 || *column < 0)
+	    return False;
+
+	/*
+	 * Adjust x,y so they are relative to this cell's origin.
+	 */
+	*x -= COLUMN_POSITION(mw, *column) - HORIZ_ORIGIN(mw);
+
+	for (i=0; i<*row; i++)
+		*y -= SOME_ROW_HEIGHT(mw, i);
+
+	return True;
+	/* NOTREACHED */
+	break;
+
     case FixedCell:
 	/*
 	 * Check the various rectangles enclosing the cells in fixed rows
@@ -995,7 +1055,7 @@ CellType cell;
 		*y += row_height;
 	    }
 	    else
-		*row = YtoRow(mw, *y) +
+		*row = xbaeYtoRow(mw, *y) +
 		    ((((CLIP_FIXED_COLUMNS & inBox) ||
 		       (CLIP_TRAILING_FIXED_COLUMNS & inBox))) &&
 		     !((CLIP_FIXED_ROWS & inBox) ||
@@ -1102,10 +1162,10 @@ CellType cell;
 	if (CLIP_ROW_LABELS & inBox)
 	{
 	    *y -= row_label_offset;
-	    *row = YtoRow(mw, *y);
+	    *row = xbaeYtoRow(mw, *y);
 	    *row += (CLIP_FIXED_ROWS & inBox ? 0 : mw->matrix.top_row);
 	    *row += ((CLIP_TRAILING_FIXED_ROWS & inBox) ?
-		     (CELL_TOTAL_HEIGHT(mw) - VISIBLE_HEIGHT(mw)) /
+		     (xbaeCellTotalHeight(mw) - VISIBLE_HEIGHT(mw)) /
 		     row_height - mw->matrix.top_row: 0);
 	    	    
 	    *column = -1;
@@ -1194,39 +1254,6 @@ CellType cell;
 	/* NOTREACHED */
 	break;
 
-    case NonFixedCell:
-
-	/*
-	 * Translate the point to take into account fixed rows or columns.
-	 */
-	*x += FIXED_COLUMN_WIDTH(mw);
-	*y += FIXED_ROW_HEIGHT(mw);
-
-	/*
-	 * Convert the new point to a row/column position
-	 */
-	*row = YtoRow(mw, *y + mw->matrix.first_row_offset) + VERT_ORIGIN(mw);
-	*column = xbaeXtoCol(mw, *x + HORIZ_ORIGIN(mw));
-
-	/*
-	 * Sanity check the result
-	 */
-	if (*row >= TRAILING_VERT_ORIGIN(mw) ||
-	    *column >= TRAILING_HORIZ_ORIGIN(mw) ||
-	    *row < 0 || *column < 0)
-	    return False;
-
-	/*
-	 * Adjust x,y so they are relative to this cell's origin.
-	 */
-	*x -= COLUMN_POSITION(mw, *column) - HORIZ_ORIGIN(mw);
-	*y %= row_height;
-
-	return True;
-
-	/* NOTREACHED */
-	break;
-
     default:
 	*row = -1;
 	*column = -1;
@@ -1241,11 +1268,8 @@ CellType cell;
  */
 /* ARGSUSED */
 Boolean
-xbaeEventToXY(mw, event, x, y, cell)
-XbaeMatrixWidget mw;
-XEvent *event;
-int *x, *y;
-CellType *cell;
+xbaeEventToXY(XbaeMatrixWidget mw, XEvent *event, int *x, int *y,
+		CellType *cell)
 {
     switch (event->type)
     {
@@ -1273,7 +1297,7 @@ CellType *cell;
 	*x -= FIXED_COLUMN_LABEL_OFFSET(mw);
 	*y -= FIXED_ROW_LABEL_OFFSET(mw);
     }
-    else if (event->xbutton.window == XtWindow(mw))
+    else if (event->xbutton.window == XtWindow(mw)) {
 	if (*x < (int)COLUMN_LABEL_OFFSET(mw) &&
 	    *x > (int)VERT_SB_OFFSET(mw))
 	    *cell = RowLabelCell;
@@ -1282,6 +1306,7 @@ CellType *cell;
 	    *cell = ColumnLabelCell;
 	else
 	    *cell = FixedCell;
+    }
     else if (event->xbutton.window == XtWindow(ClipChild(mw)))
 	*cell = NonFixedCell;
     else if (event->xbutton.window == XtWindow(TextChild(mw)))
@@ -1316,9 +1341,7 @@ CellType *cell;
  * Convert a pixel position to the column it is contained in.
  */
 int
-xbaeXtoCol(mw, x)
-XbaeMatrixWidget mw;
-int x;
+xbaeXtoCol(XbaeMatrixWidget mw, int x)
 {
     int i;
 
@@ -1335,14 +1358,32 @@ int x;
     return i - 1;
 }
 
+int
+xbaeYtoRow(XbaeMatrixWidget mw, int y)
+{
+	int	i, pos;
+
+	for (i=0, pos=0; i<mw->matrix.rows; i++) {
+		pos += mw->matrix.row_heights[i];
+		if (y <= pos) {
+#ifdef	XBAE_VERBOSE
+			fprintf(stderr, "xbaeYtoRow(%d)->%d\n", y, i);
+#endif
+			return i;
+		}
+	}
+#ifdef	XBAE_VERBOSE
+	fprintf(stderr, "xbaeYtoRow(%d)->%d\n", y, mw->matrix.rows);
+#endif
+	return mw->matrix.rows;
+}
+
 /*
  * Convert a pixel position to the trailing
  * fixed column it is contained in
  */
 int
-xbaeXtoTrailingCol(mw, x)
-XbaeMatrixWidget mw;
-int x;
+xbaeXtoTrailingCol(XbaeMatrixWidget mw, int x)
 {
     int i;
 
@@ -1362,13 +1403,10 @@ int x;
  * fixed cells, or the clip window for non-fixed).
  */
 void
-xbaeRowColToXY(mw, row, column, x, y)
-XbaeMatrixWidget mw;
-int row;
-int column;
-int *x;
-int *y;
+xbaeRowColToXY(XbaeMatrixWidget mw, int row, int column, int *x, int *y)
 {
+    int		i;
+
     /*
      * If we are in a fixed cell, calculate x/y relative to Matrixs
      * window (take into account labels etc)
@@ -1406,15 +1444,16 @@ int *y;
 	/*
 	 * Ignore vert_origin if we are in a fixed row
 	 */
-	if (IS_LEADING_FIXED_ROW(mw, row))
-	{
-	    if (IS_FIXED_COLUMN(mw, column))
-		*y = ROW_LABEL_OFFSET(mw) + ROW_HEIGHT(mw) * row;
-	    else
-		*y = ROW_HEIGHT(mw) * row;	/* TopClip */
-	}
-	else if (IS_TRAILING_FIXED_ROW(mw, row))
-	{
+	if (IS_LEADING_FIXED_ROW(mw, row)) {
+	    if (IS_FIXED_COLUMN(mw, column)) {
+		*y = ROW_LABEL_OFFSET(mw);
+	    } else {
+		*y = 0;		/* TopClip */
+	    }
+
+	    for (i=0; i<row; i++)
+		*y += mw->matrix.row_heights[i];
+	} else if (IS_TRAILING_FIXED_ROW(mw, row)) {
 	    int m;
 	    if (IS_FIXED_COLUMN(mw, column))
 		*y = TRAILING_FIXED_ROW_LABEL_OFFSET(mw);
@@ -1422,27 +1461,28 @@ int *y;
 		*y = 0;				/* BottomClip */
 	    for (m = TRAILING_VERT_ORIGIN(mw); m < row; m++)
 		*y += ROW_HEIGHT(mw);
+	} else if (IS_FIXED_COLUMN(mw, column)) {
+	    *y = 0;
+	    for (i=mw->matrix.fixed_rows + VERT_ORIGIN(mw); i<row; i++)
+		*y += mw->matrix.row_heights[i];
+	} else {
+	    *y = ROW_LABEL_OFFSET(mw);
+	    for (i=VERT_ORIGIN(mw); i<row; i++)
+		*y += mw->matrix.row_heights[i];
 	}
-	else if (IS_FIXED_COLUMN(mw, column))
-
-	    *y = ROW_HEIGHT(mw) * ((row - (int)mw->matrix.fixed_rows) -
-				   VERT_ORIGIN(mw));
-	else
-	    *y = ROW_LABEL_OFFSET(mw) + ROW_HEIGHT(mw) *
-		(row - VERT_ORIGIN(mw));
     }
 
     /*
      * If we are not fixed we must account for fixed rows/columns
      * and scrolling origins.
      */
-    else
-    {
+    else {
 	*x = (COLUMN_POSITION(mw, column) -
 	      COLUMN_POSITION(mw, mw->matrix.fixed_columns)) -
 	    HORIZ_ORIGIN(mw);
-	*y = ROW_HEIGHT(mw) * ((row - (int) mw->matrix.fixed_rows) -
-			       VERT_ORIGIN(mw));
+	*y = 0;
+	for (i=mw->matrix.fixed_rows + VERT_ORIGIN(mw); i<row; i++)
+	    *y += mw->matrix.row_heights[i];
 	*y -= mw->matrix.first_row_offset;
     }
 }
@@ -1464,10 +1504,7 @@ int *y;
  * or one of the extra clips which handle the fixed row/col cells.
  */
 Window
-xbaeGetCellWindow(mw, w, row, column)
-XbaeMatrixWidget mw;
-Widget *w;
-int row, column;
+xbaeGetCellWindow(XbaeMatrixWidget mw, Widget *w, int row, int column)
 {
     int posn;
     Window win;
@@ -1533,17 +1570,8 @@ int row, column;
 }
 
 void
-xbaeCalcVertFill(mw, win, x, y, row, column, ax, ay, width, height)
-XbaeMatrixWidget mw;
-Window win;
-int x;
-int y;
-int row;
-int column;
-int *ax;
-int *ay;
-int *width;
-int *height;
+xbaeCalcVertFill(XbaeMatrixWidget mw, Window win, int x, int y, int row, int column,
+	int *ax, int *ay, int *width, int *height)
 {
     *ax = x;
     *width = COLUMN_WIDTH(mw, column);
@@ -1623,20 +1651,11 @@ int *height;
 }
     
 void
-xbaeCalcHorizFill(mw, win, x, y, row, column, ax, ay, width, height)
-XbaeMatrixWidget mw;
-Window win;
-int x;
-int y;
-int row;
-int column;
-int *ax;
-int *ay;
-int *width;
-int *height;
+xbaeCalcHorizFill(XbaeMatrixWidget mw, Window win, int x, int y, int row, int column,
+	int *ax, int *ay, int *width, int *height)
 {
     *ay = y;
-    *height = ROW_HEIGHT(mw);
+    *height = SOME_ROW_HEIGHT(mw, row);
 
     if (win == XtWindow(TopClip(mw)))
     {
@@ -1698,3 +1717,68 @@ int *height;
 		ClipChild(mw)->core.y - *ay;
     }
 }
+
+/*
+ * Below are some functions to deal with a multi-threaded environment.
+ * Use these instead of the Xt stuff, to ensure that we can still cope
+ * with X11r5 where this stuff didn't exist.
+ */
+#ifdef  XtSpecificationRelease
+#if     XtSpecificationRelease > 5
+#define R6plus
+#endif
+#endif
+
+void xbaeObjectLock(Widget w)
+{
+#ifdef  R6plus
+        if (XmIsGadget(w))
+                XtAppLock(XtWidgetToApplicationContext(
+                        XtParent(w)));
+        else
+                XtAppLock(XtWidgetToApplicationContext(w));
+#endif
+}
+
+void
+xbaeObjectUnlock(Widget w)
+{
+#ifdef  R6plus
+        if (XmIsGadget(w))
+                XtAppUnlock(XtWidgetToApplicationContext(
+                        XtParent(w)));
+        else
+                XtAppUnlock(XtWidgetToApplicationContext(w));
+#endif
+}
+
+/* The following 2 functions are here to provide information about the
+   version of this library - since we may often build it as a shared/dynamic
+   library it's important to have this builtin */
+const char *
+XbaeGetVersionTxt(void)
+{
+	return XbaeVersionTxt;
+}
+
+int
+XbaeGetVersionNum(void)
+{
+	return XbaeVersion;
+}
+
+/* The following 2 functions contain/return info about the
+   M*tif/LessTif version this library was built with.
+   Building against one toolkit and linking on runtime against
+   another is likely to fail! */
+const char *
+XbaeGetXmVersionTxt(void)
+{
+        return XmVERSION_STRING;
+}
+
+int
+XbaeGetXmVersionNum(void)
+{
+        return XmVersion;
+}
