@@ -1,6 +1,6 @@
 /**
  *
- * $Id: DisplayFallbackResources.c,v 1.1 1998/06/03 21:54:58 rwscott Exp $
+ * $Id: DisplayFallbackResources.c,v 1.4 2001/03/31 18:09:05 amai Exp $
  *
  * Copyright (C) 1996 Free Software Foundation, Inc.
  *
@@ -22,9 +22,16 @@
  *
  **/
 
+#include "../config.h"
 #include <stdio.h>
 
-static const char rcsid[] = "$Id: DisplayFallbackResources.c,v 1.1 1998/06/03 21:54:58 rwscott Exp $";
+#include "Xlt.h"
+
+#ifdef WITH_DMALLOC
+#include <dmalloc.h>
+#endif
+
+static const char rcsid[] = "$Id: DisplayFallbackResources.c,v 1.4 2001/03/31 18:09:05 amai Exp $";
 
 void 
 XltDisplayFallbackResources(char **Fallback)
