@@ -1,6 +1,6 @@
 /**
  *
- * $Id: DisplayOptions.c,v 1.2 1999/12/12 15:42:33 rwscott Exp $
+ * $Id: DisplayOptions.c,v 1.5 2001/03/31 18:09:05 amai Exp $
  *
  * Copyright (C) 1996 Free Software Foundation, Inc.
  *
@@ -22,15 +22,24 @@
  *
  **/
 
+#include "../config.h"
+
 #include <stdio.h>
+
 #include <X11/Intrinsic.h>
 
-static const char rcsid[] = "$Id: DisplayOptions.c,v 1.2 1999/12/12 15:42:33 rwscott Exp $";
+#include "Xlt.h"
+
+#ifdef WITH_DMALLOC
+#include <dmalloc.h>
+#endif
+
+static const char rcsid[] = "$Id: DisplayOptions.c,v 1.5 2001/03/31 18:09:05 amai Exp $";
 
 void 
 XltDisplayOptions(XrmOptionDescRec *opTable, Cardinal num_options)
 {
-	int i;
+	Cardinal i;
 
 	fprintf(stderr, "The command line options available are as follows:\n");
 	for (i = 0; i < num_options; i++)
