@@ -1,10 +1,10 @@
 /**
  *
- * $Header: /cvsroot/hungry/lesstif/lib/Xlt/lib/FontChooser.c,v 1.8 1999/09/03 00:35:50 rwscott Exp $
+ * $Header: /cvsroot/lesstif/lesstif/lib/Xlt/lib/FontChooser.c,v 1.19 2001/06/02 09:27:45 amai Exp $
  *
- * Copyright (C) 1999 Free Software Foundation, Inc.
+ * Copyright  1999, 2001 by the LessTif developers.
  *
- * This file is part of the GNU LessTif Library.
+ * This file is part of the LessTif Extension Library (Xlt).
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -22,19 +22,28 @@
  *
  **/
 
-static const char rcsid[] = "$Id: FontChooser.c,v 1.8 1999/09/03 00:35:50 rwscott Exp $";
+static const char rcsid[] = "$Header: /cvsroot/lesstif/lesstif/lib/Xlt/lib/FontChooser.c,v 1.19 2001/06/02 09:27:45 amai Exp $";
 
 #include <../config.h>
+
+#include <stdlib.h>
+#include <stdio.h>
+#include <string.h>
+
 #include <Xm/DialogS.h>
 #include <Xm/PushBG.h>
 #include <Xm/XmP.h>
 #include <Xm/MessageBP.h>
 #include <Xm/RowColumn.h>
 #include <Xm/LabelG.h>
+
 #include <FontChooser.h>
 #include <FontChooserP.h>
-#include <stdlib.h>
-#include <stdio.h>
+
+
+#ifdef WITH_DMALLOC
+#include <dmalloc.h>
+#endif
 
 /* Forward Declarations */
 
@@ -42,6 +51,7 @@ static void class_part_initialize(WidgetClass w_class);
 static void initialize(Widget request, Widget new_w, ArgList args, Cardinal *num_args);
 static Boolean set_values(Widget current, Widget request, Widget new_w, ArgList args, Cardinal *num_args);
 static void Destroy(Widget w);
+static void Realize(Widget aw, XtValueMask * value_mask, XSetWindowAttributes * attributes);
 
 /*
  * Resources for the FontChooser class
@@ -100,7 +110,7 @@ XltFontChooserClassRec xltFontChooserClassRec =
 	/* class_inited          */ False,
 	/* initialize            */ initialize,
 	/* initialize_hook       */ NULL,
-	/* realize               */ XtInheritRealize,
+	/* realize               */ Realize,
 	/* actions               */ NULL,
 	/* num_actions           */ 0,
 	/* resources             */ resources,
@@ -156,7 +166,7 @@ XltFontChooserClassRec xltFontChooserClassRec =
     /* XmBulletinBoard class part */
     {
 	/* always_install_accelerators  */	True,
-	/* geo_matrix_create            */	_XmMessageBoxGeoMatrixCreate,
+	/* geo_matrix_create            */	XmInheritGeoMatrixCreate,
 	/* focus_moved_proc             */	XmInheritFocusMovedProc,
 	/* extension                    */	NULL
     },
@@ -169,7 +179,7 @@ XltFontChooserClassRec xltFontChooserClassRec =
 	/* ??? */			   NULL
     }
 };
-/* *INDENT-ON* */
+
 
 WidgetClass xltFontChooserWidgetClass = (WidgetClass)&xltFontChooserClassRec;
 
@@ -227,7 +237,8 @@ ShowCount(Widget fc, int num)
 static void
 DisableAllMenus(Widget fc, int column)
 {
-	int	i, j;
+	int	i;
+      Cardinal j;
 	Widget	m, w;
 
 	for (i=0; i<14; i++)
@@ -255,10 +266,11 @@ EnableMenu(Widget fc, char *s)
 	p = s;
 	for (j=0, s; j<14; j++) {
 		/* Scan for a dash and a string starting at p, store it in field */
-		if (*p != '-') {
-			_XmWarning(fc, "Can't scan font name '%s'\n", s);
-			nogood = 1;
-			break;
+		if (*p != '-') 
+		{
+		    fprintf(stderr, "%s: Can't scan font name '%s'\n", XtName(fc), s);
+		    nogood = 1;
+		    break;
 		}
 		p++;
 		ii=0;
@@ -329,14 +341,15 @@ FCFilter(Widget w, XtPointer client, XtPointer call)
 		XmFontList      fl;
 		XmFontListEntry fle;
 		XFontSet        fs;
-		char            m1[2][80], **missing = (char **)&m1[0];
+		char            m1[2][80], **missing = (char **)&m1[0][0];
 		char            m2[80], *def = &m2[0];
 		int             nmissing = 0;
 
 		fs = XCreateFontSet(XtDisplay(fc), f, &missing, &nmissing, &def);
 
 		if (fs != NULL) {
-                	fle = XmFontListEntryCreate(XmFONTLIST_DEFAULT_TAG, XmFONT_IS_FONTSET, fs);
+                	fle = XmFontListEntryCreate(XmFONTLIST_DEFAULT_TAG,
+			         XmFONT_IS_FONTSET, (XtPointer)fs);
                 	fl = XmFontListAppendEntry(NULL, fle);
 
 			XtVaSetValues(demo, XmNfontList, fl, NULL);
@@ -354,6 +367,67 @@ FCFilter(Widget w, XtPointer client, XtPointer call)
 	}
 }
 
+static void
+Realize(Widget aw, XtValueMask * value_mask, XSetWindowAttributes * attributes)
+{
+	XltFontChooserWidget	fc = (XltFontChooserWidget) aw;
+	char			*f = XtMalloc(256), **list;
+	int			i, j, nfonts;
+
+#define	superclass	(&xmMessageBoxClassRec)
+  (*superclass->core_class.realize) (aw, value_mask, attributes);
+#undef	superclass
+
+	if (! FC_ShowDemo(fc))
+		return;
+
+/* Show the right font initially */
+	f[0] = '\0';
+	for (j=0; j<14; j++) {
+		strcat(f, "-");
+		strcat(f, FC_Setting(fc)[j]);
+	}
+
+	if (FC_FontName(fc))
+		XtFree(FC_FontName(fc));
+	FC_FontName(fc) = f;
+
+/* How many fonts ? */
+	list = XListFonts(XtDisplay(fc), f, 4096, &nfonts);
+	for (i=0; i<nfonts; i++)
+		EnableMenu(aw, list[i]);
+	XFreeFontNames(list);
+
+	if (FC_DemoWidget(fc)) {
+		XmFontList      fl;
+		XmFontListEntry fle;
+		XFontSet        fs;
+		char            m1[2][80], **missing = (char **)&m1[0][0];
+		char            m2[80], *def = &m2[0];
+		int             nmissing = 0;
+
+		fs = XCreateFontSet(XtDisplay(fc), f, &missing, &nmissing, &def);
+
+		if (fs != NULL) {
+                	fle = XmFontListEntryCreate(XmFONTLIST_DEFAULT_TAG,
+			         XmFONT_IS_FONTSET, (XtPointer)fs);
+                	fl = XmFontListAppendEntry(NULL, fle);
+
+			XtVaSetValues(FC_DemoWidget(fc), XmNfontList, fl, NULL);
+
+			XmFontListEntryFree(&fle);
+			XmFontListFree(fl);
+#if 0
+			/* Freeing the font set produces nice little core dumps */
+			XFreeFontSet(XtDisplay(fc), fs);	/* FIX ME leak ?? */
+#endif
+        	}
+
+	}
+
+	ShowCount(aw, nfonts);
+}
+
 static int qsort_helper_num(const void *p, const void *q)
 {
 	int	i, j, r;
@@ -416,7 +490,8 @@ GetFullFontList(Widget w)
 		for (j=0, p=list[i]; j<14; j++) {
 			/* Scan for a dash and a string starting at p, store it in field */
 			if (*p != '-') {
-				_XmWarning(w, "Can't scan font name '%s'\n", list[i]);
+				fprintf(stderr, "%s: Can't scan font name '%s'\n",
+					XtName(w), list[i]);
 				nogood = 1;
 				break;
 			}
@@ -473,8 +548,20 @@ GetFullFontList(Widget w)
 static void
 Destroy(Widget w)
 {
-	if (FC_Menus(w))
-		XtFree((XtPointer)FC_Menus(w));
+int i;
+
+    if (FC_Menus(w))
+    {
+	XtFree((XtPointer)FC_Menus(w));
+    }
+    for (i=0; i<14; i++)
+    {
+	if (FC_Setting(w)[i])
+	{
+	    XtFree(FC_Setting(w)[i]);
+	}
+    }
+    XtFree((char *)FC_Setting(w));
 }
 
 static void
@@ -510,7 +597,6 @@ initialize(Widget request, Widget new_w, ArgList args, Cardinal *num_args)
 
 	ac = 0;
 	form = XmCreateForm(new_w, "form", al, ac);
-	XtManageChild(form);
 
 	p = fn = strdup(FC_FieldNames(new_w));
 	n = XtMalloc(strlen(fn) + 10);
@@ -594,6 +680,7 @@ initialize(Widget request, Widget new_w, ArgList args, Cardinal *num_args)
 	}
 
 	GetFullFontList(new_w);
+	XtManageChild(form);
 }
 
 static Boolean
@@ -638,7 +725,8 @@ XltCreateFontChooserDialog(Widget parent, char *name, Arg *arglist, Cardinal arg
 {
     Widget shell, r;
     char *shell_name;
-    int ac, i;
+    int ac;
+    XtArgVal i;
     Arg *al = (Arg *)XtCalloc(argcount + 1, sizeof(Arg));
 
     ac = 0;
