@@ -1,6 +1,6 @@
 /**
  *
- * $Id: Host.c,v 1.11 1999/11/27 02:10:21 rwscott Exp $
+ * $Id: Host.c,v 1.17 2001/04/28 16:32:47 rwscott Exp $
  *
  * Copyright (C) 1996 Free Software Foundation, Inc.
  *
@@ -23,11 +23,15 @@
  **/
 
 #include "../config.h"
+
 #include <unistd.h>
-#include <fcntl.h>
+/* #include <fcntl.h> */
 #include <stdio.h>
 #include <stdlib.h>
+#include <string.h>
+#ifdef HAVE_SYS_TYPES_H
 #include <sys/types.h>
+#endif
 #include <sys/socket.h>
 #include <sys/time.h>
 #ifdef HAVE_SYS_SELECT_H
@@ -35,17 +39,24 @@
 #endif
 #include <netinet/in.h>
 #include <netdb.h>
+
 #include <X11/StringDefs.h>
+
 #include <Xm/MessageB.h>
 #include <Xm/Form.h>
 #include <Xm/TextF.h>
 #include <Xm/Label.h>
 #include <Xm/RepType.h>
-#include <HostP.h>
+
+#include "HostP.h"
+
+#ifdef WITH_DMALLOC
+#include <dmalloc.h>
+#endif
 
 #undef DEBUG
 
-static const char rcsid[] = "$Id: Host.c,v 1.11 1999/11/27 02:10:21 rwscott Exp $";
+static const char rcsid[] = "$Id: Host.c,v 1.17 2001/04/28 16:32:47 rwscott Exp $";
 
 typedef struct
 {
@@ -68,7 +79,7 @@ static char DefaultHelp[] = "\n"
    Widget methods, forward declarations
  */
 
-static void class_initialize();
+static void class_initialize(void);
 static void class_part_initialize(WidgetClass widget_class);
 static void initialize(Widget request, Widget new_w, ArgList args, Cardinal *num_args);
 static void destroy(Widget w);
@@ -400,8 +411,10 @@ struct servent *sp;
     }
     else
     {
+#ifdef HAVE_GETSERVBYNAME
     	sp = getservbyname(Service, "tcp");
     	if (sp == NULL)
+#endif
     	{
 	    fprintf(stderr, "Service >%s< not found\n", Service);
 	    ListenFd = -1;
@@ -872,8 +885,9 @@ OpenHost(Widget W, Widget w)
  */
 
 static void
-class_initialize()
+class_initialize(void)
 {
+XmRepTypeId id;
 static char *value_names[] =
 {
     "client",
@@ -885,10 +899,18 @@ static unsigned char values[] =
     XltHOST_CLIENT,
     XltHOST_SERVER
 };
+
     /*
        printf("class_initialize()\n");
      */
-     XmRepTypeRegister(XltRHostType, value_names, values, XtNumber(values));
+     id = XmRepTypeRegister(XltRHostType, value_names, values, XtNumber(values));
+     /* rws 13 Apr 2001
+        We need a HostType to String converter
+     XtSetTypeConverter(XltRHostType, XmRString, conv, NULL, 0, XtCacheNone, NULL);
+     */
+     /*
+     XmRepTypeAddReverse(id);
+     */
 }
 
 static void
@@ -1075,8 +1097,12 @@ XltHostSelect(Widget W, Widget w)
 	Widget HostText;
 	Widget PortLabel;
 	Widget PortText;
+	String Name;
 
-	Host_Dialog(w) = XmCreateMessageDialog(W, "HostSelect", NULL, 0);
+	Name = XtMalloc(strlen(XtName(w)) + 7);
+	sprintf(Name, "%sSelect", XtName(w));
+	Host_Dialog(w) = XmCreateMessageDialog(W, Name, NULL, 0);
+	XtFree(Name);
 	Form = XmCreateForm(Host_Dialog(w), "Form", NULL, 0);
 
 	Form1 = XmCreateForm(Form, "HostForm", NULL, 0);
