@@ -1,8 +1,9 @@
 /**
  *
- * $Id: NumEntry.c,v 1.5 1998/11/21 14:35:43 rwscott Exp $
+ * $Id: NumEntry.c,v 1.15 2001/06/02 09:27:45 amai Exp $
  *
  * Copyright (C) 1996 Free Software Foundation, Inc.
+ * Copyright  1999-2001 by the LessTif developers.
  *
  * This file is part of the GNU LessTif Extension Library.
  *
@@ -21,23 +22,30 @@
  * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
  **/
+ 
+#include "../config.h"
+
 #include <stdio.h>
+#include <stdarg.h>
+#include <string.h>
+
 #include <Xm/TextF.h>
 #include <Xm/Label.h>
 #include <Xm/ArrowB.h>
 #include <Xm/RowColumn.h>
 #include <Xm/MessageB.h>
 #include <NumEntryP.h>
+
 #include "calc.h"
 
-#include <stdarg.h>
 
-#undef DEBUG
-#ifdef DEBUG
-#include <stdio.h>
+#ifdef WITH_DMALLOC
+#include <dmalloc.h>
 #endif
 
-static const char rcsid[] = "$Id: NumEntry.c,v 1.5 1998/11/21 14:35:43 rwscott Exp $";
+#undef DEBUG
+
+static const char rcsid[] = "$Id: NumEntry.c,v 1.15 2001/06/02 09:27:45 amai Exp $";
 
 /*
    Widget methods, forward declarations
@@ -125,19 +133,41 @@ static XmSyntheticResource syn_resources[] =
 };
 #undef Offset
 
+static void KeyPad(Widget w, XEvent *event, String *params, Cardinal *num_params);
 static void Up(Widget w, XEvent *event, String *params, Cardinal *num_params);
 static void Down(Widget w, XEvent *event, String *params, Cardinal *num_params);
 
 static XtActionsRec actions[] =
 {
+    {"KeyPad", KeyPad},
     {"Up", Up},
     {"Down", Down},
     {"Help", Help},
 };
 
 static char tm_table[] = "\
-	<Key>osfDown: Down() \n\
-	<Key>osfUp: Up()";
+	<Key>osfDown: Down()\n\
+	<Key>osfUp: Up()\n\
+	!<Btn4Down>: Up()\n\
+	!<Btn5Down>: Down()";
+
+static char text_tm_table[] = "\
+	<Key>KP_Insert: KeyPad()\n\
+	<Key>KP_Delete: KeyPad()\n\
+	<Key>KP_Enter: KeyPad()\n\
+	<Key>KP_End: KeyPad()\n\
+	<Key>KP_Down: KeyPad()\n\
+	<Key>KP_Next: KeyPad()\n\
+	<Key>KP_Left: KeyPad()\n\
+	<Key>KP_Begin: KeyPad()\n\
+	<Key>KP_Right: KeyPad()\n\
+	<Key>KP_Home: KeyPad()\n\
+	<Key>KP_Up: KeyPad()\n\
+	<Key>KP_Prior: KeyPad()\n\
+	<Key>KP_Add: KeyPad()\n\
+	<Key>KP_Divide: KeyPad()\n\
+	<Key>KP_Multiply: KeyPad()\n\
+	<Key>KP_Subtract: KeyPad()";
 
 /*
    Widget class record
@@ -193,7 +223,7 @@ XltNumEntryClassRec xrwsNumEntryClassRec = {
     /* subresource_count */ 0,
     /* constraint_size   */ sizeof(XmFormConstraintRec),
     /* initialize        */ NULL /*constraint_initialize*/,
-    /* destroy           */ destroy /*NULL*/,
+    /* destroy           */ NULL,
     /* set_values        */ NULL /*constraint_set_values*/,
     /* extension         */ NULL,
     },
@@ -246,7 +276,7 @@ _AutoRepeat(Widget w)
     {
 	NumEntry_TimerId(Entry) = XtAppAddTimeOut(XtWidgetToApplicationContext(w),
 						  NumEntry_Delay(Entry),
-						  (void *)_AutoRepeat,
+						  (XtTimerCallbackProc)_AutoRepeat,
 						  w);
 	XtCallCallbacks(w, XmNactivateCallback, NULL);
     }
@@ -269,7 +299,7 @@ AutoRepeat(Widget w, Boolean On)
 	{
 	    NumEntry_TimerId(w) = XtAppAddTimeOut(XtWidgetToApplicationContext(w),
 						  NumEntry_InitialDelay(w),
-						  (void *)_AutoRepeat,
+						  (XtTimerCallbackProc)_AutoRepeat,
 						  button);
 	}
     }
@@ -295,7 +325,7 @@ DoMath(Widget W)
 #endif
     value = XmTextFieldGetString(NumEntry_TextField(W));
     Place = XmTextFieldGetInsertionPosition(NumEntry_TextField(W));
-    result = Calc(value);
+    result = XltCalc(value);
     if (result != NumEntry_Result(W))
     {
 	char buf[1024];
@@ -313,14 +343,15 @@ DoMath(Widget W)
 	{
 	    XtFree(NumEntry_Value(W));
 	    NumEntry_Value(W) = cbs.value;
+	    NumEntry_Result(W) = cbs.dvalue;
 	}
 	else
 	{
 	    XtFree(cbs.value);
 	}
-	XmTextFieldSetString(NumEntry_TextField(W), NumEntry_Value(W));
-	XmTextFieldSetInsertionPosition(NumEntry_TextField(W), Place);
     }
+    XmTextFieldSetString(NumEntry_TextField(W), NumEntry_Value(W));
+    XmTextFieldSetInsertionPosition(NumEntry_TextField(W), Place);
     XtFree(value);
 }
 
@@ -490,6 +521,7 @@ initialize(Widget request, Widget new_w, ArgList args, Cardinal *num_args)
     XtVaSetValues(NumEntry_Label(new_w),
 		  XmNlabelString, NumEntry_LabelString(new_w),
 		  NULL);
+    XtOverrideTranslations(NumEntry_Label(new_w), XtParseTranslationTable(tm_table));
     XtManageChild(NumEntry_Label(new_w));
     /*
        XtVaGetValues(NumEntry_Label(new_w),
@@ -497,14 +529,15 @@ initialize(Widget request, Widget new_w, ArgList args, Cardinal *num_args)
        NULL);
      */
     NumEntry_TextField(new_w) = XmCreateTextField(new_w, "Text", NULL, 0);
-    XtAddCallback(NumEntry_TextField(new_w), XmNvalueChangedCallback, (void *)ValueChange, (XtPointer)NULL);
-    XtAddCallback(NumEntry_TextField(new_w), XmNactivateCallback, (void *)Activate, (XtPointer)NULL);
-    XtAddCallback(NumEntry_TextField(new_w), XmNlosingFocusCallback, (void *)LosingFocus, (XtPointer)NULL);
+    XtAddCallback(NumEntry_TextField(new_w), XmNvalueChangedCallback, (XtCallbackProc)ValueChange, (XtPointer)NULL);
+    XtAddCallback(NumEntry_TextField(new_w), XmNactivateCallback, (XtCallbackProc)Activate, (XtPointer)NULL);
+    XtAddCallback(NumEntry_TextField(new_w), XmNlosingFocusCallback, (XtCallbackProc)LosingFocus, (XtPointer)NULL);
     XtVaSetValues(NumEntry_TextField(new_w),
 		  XmNvalue, NumEntry_Value(new_w),
 		  XmNcolumns, NumEntry_Columns(new_w),
 		  NULL);
     XtOverrideTranslations(NumEntry_TextField(new_w), XtParseTranslationTable(tm_table));
+    XtOverrideTranslations(NumEntry_TextField(new_w), XtParseTranslationTable(text_tm_table));
     XtManageChild(NumEntry_TextField(new_w));
     XtVaGetValues(NumEntry_TextField(new_w),
 		  XmNheight, &h,
@@ -517,10 +550,11 @@ initialize(Widget request, Widget new_w, ArgList args, Cardinal *num_args)
 		  XmNorientation, XmHORIZONTAL,
 		  XmNnumColumns, 1,
 		  NULL);
+    XtOverrideTranslations(NumEntry_RowColumn(new_w), XtParseTranslationTable(tm_table));
     NumEntry_UpArrow(new_w) = XmCreateArrowButton(NumEntry_RowColumn(new_w), "UpArrow", NULL, 0);
-    XtAddCallback(NumEntry_UpArrow(new_w), XmNactivateCallback, (void *)StepValue, (XtPointer)1);
-    XtAddCallback(NumEntry_UpArrow(new_w), XmNarmCallback, (void *)AutoRepeat, (XtPointer)True);
-    XtAddCallback(NumEntry_UpArrow(new_w), XmNdisarmCallback, (void *)AutoRepeat, (XtPointer)False);
+    XtAddCallback(NumEntry_UpArrow(new_w), XmNactivateCallback, (XtCallbackProc)StepValue, (XtPointer)1);
+    XtAddCallback(NumEntry_UpArrow(new_w), XmNarmCallback, (XtCallbackProc)AutoRepeat, (XtPointer)True);
+    XtAddCallback(NumEntry_UpArrow(new_w), XmNdisarmCallback, (XtCallbackProc)AutoRepeat, (XtPointer)False);
     XtVaSetValues(NumEntry_UpArrow(new_w),
 		  XmNwidth, h,
 		  XmNarrowDirection, XmARROW_UP,
@@ -528,9 +562,9 @@ initialize(Widget request, Widget new_w, ArgList args, Cardinal *num_args)
     XtOverrideTranslations(NumEntry_UpArrow(new_w), XtParseTranslationTable(tm_table));
     XtManageChild(NumEntry_UpArrow(new_w));
     NumEntry_DnArrow(new_w) = XmCreateArrowButton(NumEntry_RowColumn(new_w), "DnArrow", NULL, 0);
-    XtAddCallback(NumEntry_DnArrow(new_w), XmNactivateCallback, (void *)StepValue, (XtPointer)-1);
-    XtAddCallback(NumEntry_DnArrow(new_w), XmNarmCallback, (void *)AutoRepeat, (XtPointer)True);
-    XtAddCallback(NumEntry_DnArrow(new_w), XmNdisarmCallback, (void *)AutoRepeat, (XtPointer)False);
+    XtAddCallback(NumEntry_DnArrow(new_w), XmNactivateCallback, (XtCallbackProc)StepValue, (XtPointer)-1);
+    XtAddCallback(NumEntry_DnArrow(new_w), XmNarmCallback, (XtCallbackProc)AutoRepeat, (XtPointer)True);
+    XtAddCallback(NumEntry_DnArrow(new_w), XmNdisarmCallback, (XtCallbackProc)AutoRepeat, (XtPointer)False);
     XtVaSetValues(NumEntry_DnArrow(new_w),
 		  XmNwidth, h,
 		  XmNarrowDirection, XmARROW_DOWN,
@@ -559,7 +593,7 @@ initialize(Widget request, Widget new_w, ArgList args, Cardinal *num_args)
 		  XmNrightWidget, NumEntry_TextField(new_w),
 		  NULL);
     XtManageChild(NumEntry_RowColumn(new_w));
-    XtAddCallback(new_w, XmNhelpCallback, (void *)Help, NULL);
+    XtAddCallback(new_w, XmNhelpCallback, (XtCallbackProc)Help, NULL);
     NumEntry_TimerId(new_w) = (XtIntervalId)NULL;
 }
 
@@ -640,6 +674,18 @@ NEGetColumns(Widget wid, int resource_offset, XtArgVal *value)
 #endif
 }
 
+static void
+KeyPad(Widget w, XEvent *event, String *params, Cardinal *num_params)
+{
+    Widget Entry;
+
+    Entry = w;
+    while (!XltIsNumEntry(Entry))
+	Entry = XtParent(Entry);
+    event->xkey.state |= Mod2Mask;
+    XtCallActionProc(NumEntry_TextField(Entry), "self-insert", event, params, *num_params);
+}
+
 static void
 Up(Widget w, XEvent *event, String *params, Cardinal *num_params)
 {
