@@ -1,4 +1,6 @@
 /*
+ * $Id: SciPlotUtil.c,v 1.6 2001/03/31 18:09:05 amai Exp $
+ *
  * Copyright (C) 1996 by Rob McMullen
  *
  * This library is free software; you can redistribute it and/or
@@ -20,15 +22,19 @@
  *         http://www.ae.utexas.edu/~rwmcm
  */
 
+#include "../config.h"
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <ctype.h>
 #include <unistd.h>
 #include <string.h>
 #include <time.h>
+
 #include <X11/Intrinsic.h>
 #include <X11/Shell.h>
 #include <X11/StringDefs.h>
+
 #include <Xm/Xm.h>
 #include <Xm/Label.h>
 #include <Xm/TextF.h>
@@ -40,6 +46,11 @@
 #include <Xm/PanedW.h>
 
 #include "SciPlot.h"
+#include "SciPlotUtil.h"
+
+#ifdef WITH_DMALLOC
+#include <dmalloc.h>
+#endif
 
 /* Globals */
 static int DialogCount=0;		/* Number of dialogs on screen */
