@@ -1,5 +1,5 @@
 /*
-   $Id: Stroke.c,v 1.3 1999/12/04 21:20:14 amai Exp $
+   $Id: Stroke.c,v 1.6 2001/06/02 09:27:45 amai Exp $
 
    libStroke is a library to enable any X application to use Strokes.
    Copyright (C) 1997  Richard Scott <rwscott@magi.com>
@@ -19,8 +19,10 @@
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
+#include "../config.h"
 
 #include <stdio.h>
+#include <string.h>
 
 #include <X11/IntrinsicP.h>
 #include <X11/StringDefs.h>
@@ -28,6 +30,11 @@
 
 #include <Stroke.h>
 
+#ifdef WITH_DMALLOC
+#include <dmalloc.h>
+#endif
+
+
 typedef struct
 {
     String DefaultHelpString;
