@@ -1,4 +1,4 @@
-/* $Id: test1.c,v 1.1 1999/11/15 23:45:43 rwscott Exp $ */
+/* $Id: test1.c,v 1.2 2000/08/29 21:59:19 dannybackx Exp $ */
 
 #include <stdlib.h>
 #include <SelectionBox.h>
@@ -20,7 +20,7 @@ static char *FallBack[] =
 };
 
 void
-debug()
+debug(void)
 {
 }
 
