@@ -1,8 +1,9 @@
 /**
  *
- * $Header: /cvsroot/hungry/lesstif/lib/Xm-2.0/ComboBox.c,v 1.45 1999/12/30 16:46:37 amai Exp $
+ * $Header: /cvsroot/lesstif/lesstif/lib/Xm-2.0/ComboBox.c,v 1.56 2001/05/22 09:46:21 amai Exp $
  *
  * Copyright (C) 1997 Free Software Foundation, Inc.
+ * Copyright (C) 1997-2001 LessTif Development Team
  *
  * This file is part of the GNU LessTif Library.
  *
@@ -22,12 +23,11 @@
  *
  **/
 
-static const char rcsid[] = "$Header: /cvsroot/hungry/lesstif/lib/Xm-2.0/ComboBox.c,v 1.45 1999/12/30 16:46:37 amai Exp $";
-
+static const char rcsid[] = "$Header: /cvsroot/lesstif/lesstif/lib/Xm-2.0/ComboBox.c,v 1.56 2001/05/22 09:46:21 amai Exp $";
 
 #include <LTconfig.h>
-#include <XmI/XmI.h>
 
+#include <XmI/XmI.h>
 #include <Xm/XmP.h>
 #include <Xm/ListP.h>
 #include <Xm/TextFP.h>
@@ -37,9 +37,11 @@ static const char rcsid[] = "$Header: /cvsroot/hungry/lesstif/lib/Xm-2.0/ComboBo
 #include <Xm/VendorSEP.h>
 
 #include <XmI/MessagesI.h>
-#include <XmI/DebugUtil.h>
 #include <XmI/PopupUtil.h>
 
+#include <XmI/DebugUtil.h>
+
+
 /* 
   MODULE: CombBox.c
   DESCRIPTION:
@@ -112,8 +114,8 @@ static void CheckMinimalSize(Widget w, XtWidgetGeometry *reply);
 #if 0
 static void CheckSetRenderTable(Widget w);
 #endif
-static void ClassInitialize();
-static void ClassPartInitialize(WidgetClass class);
+static void ClassInitialize(void);
+static void ClassPartInitialize(WidgetClass widget_class);
 static void ComboBoxParentProcess(Widget w);
 static void ComputeSize(Widget w, XtWidgetGeometry *reply);
 static void ConstraintDestroy(Widget w);
@@ -337,8 +339,20 @@ static XtResource resources[] = {
      (XtPointer)2
    },
 
+#if XmVersion >= 2001
+   {
+     XmNpositionMode,
+     XmCpositionMode,
+     XmRpositionMode,
+     sizeof(XtEnum),
+     Offset(position_mode),
+     XmRImmediate,
+     (XtPointer)XmZERO_BASED
+   },
+#endif
 };
 
+
 static XmSyntheticResource syn_resources[] = {
    {
       XmNselectedItem,
@@ -520,7 +534,7 @@ CallSelectionCallbacks(Widget w, XtPointer client_data, XtPointer data)
 
    Unused(client_data);
 
-   DEBUGOUT(XdbDebug(__FILE__, w, "%s:CallSelectionCallbacks ...\n",
+   DEBUGOUT(_LtDebugDebug(__FILE__, w, "%s:CallSelectionCallbacks ...\n",
    	__FILE__));
 
    if (CB_SelectionCB(w))
@@ -551,6 +565,7 @@ CallSelectionCallbacks(Widget w, XtPointer client_data, XtPointer data)
    }
 }
 
+
 static void 
 CBActivate(
    Widget w, 
@@ -558,7 +573,7 @@ CBActivate(
    String *params, 
    Cardinal *num_params)
 {
-   DEBUGOUT(XdbDebug(__FILE__, w, "%s:CBActivate ...\n",
+   DEBUGOUT(_LtDebugDebug(__FILE__, w, "%s:CBActivate ...\n",
    	__FILE__));
 #if 0
    FindComboBox();
@@ -572,6 +587,7 @@ CBActivate(
 #endif
 }
 
+
 static void 
 CBArmAndDropDownList(
    Widget w, 
@@ -581,7 +597,7 @@ CBArmAndDropDownList(
 {
    Widget w_passed;
 
-   DEBUGOUT(XdbDebug(__FILE__, w, "%s:CBArmAndDropDownList ...\n",
+   DEBUGOUT(_LtDebugDebug(__FILE__, w, "%s:CBArmAndDropDownList ...\n",
    	__FILE__));
    w_passed = w;
    /* Find self because child may have generated event */
@@ -596,6 +612,7 @@ CBArmAndDropDownList(
    CBDropDownList(w_passed,event,params,num_params);
 }
 
+
 static void 
 CBCancel(
    Widget w, 
@@ -609,6 +626,7 @@ CBCancel(
 #endif
 }
 
+
 static void 
 CBDisarm(
    Widget w,
@@ -622,6 +640,7 @@ CBDisarm(
 #endif
 }
 
+
 static void 
 CBDropDownList(
    Widget w, 
@@ -666,10 +685,11 @@ CBDropDownList(
 #endif
 }
 
+
 static void 
 CBFocusIn(Widget w, XEvent *event, String *params, Cardinal *num_params)
 {
-   DEBUGOUT(XdbDebug(__FILE__, w, "CBFocusIn ...\n"));
+   DEBUGOUT(_LtDebugDebug(__FILE__, w, "CBFocusIn ...\n"));
 
    w = FindComboBox(w);
    if(!XmIsComboBox(w))
@@ -680,10 +700,11 @@ CBFocusIn(Widget w, XEvent *event, String *params, Cardinal *num_params)
    HighlightBorder(w);
 }
 
+
 static void 
 CBFocusOut(Widget w, XEvent *event, String *params, Cardinal *num_params)
 {
-   DEBUGOUT(XdbDebug(__FILE__, w, "CBFocusOut ...\n"));
+   DEBUGOUT(_LtDebugDebug(__FILE__, w, "CBFocusOut ...\n"));
 
    w = FindComboBox(w);
    if(!XmIsComboBox(w))
@@ -698,6 +719,7 @@ CBFocusOut(Widget w, XEvent *event, String *params, Cardinal *num_params)
 #endif
 }
 
+
 #if 0
 static void 
 CBGetSelectedItem(
@@ -712,6 +734,7 @@ CBGetSelectedItem(
 }
 #endif
 
+
 #if 0
 static void 
 CBGetSelectedPos(
@@ -723,6 +746,7 @@ CBGetSelectedPos(
 }
 #endif
    
+
 static void 
 CBListAction(
    Widget w, 
@@ -736,7 +760,7 @@ CBListAction(
    int *selected_pos;
    int item_cnt;
 
-   DEBUGOUT(XdbDebug(__FILE__, w, "CBListAction ...\n"));
+   DEBUGOUT(_LtDebugDebug(__FILE__, w, "CBListAction ...\n"));
 
    w = FindComboBox(w);
    if(!XmIsComboBox(w))
@@ -807,7 +831,7 @@ CBListAction(
          }
       }
 
-      DEBUGOUT(XdbDebug(__FILE__, w, "XmListSelectPos(%s,%d,True)\n",XtName(CB_List(w)),pos));
+      DEBUGOUT(_LtDebugDebug(__FILE__, w, "XmListSelectPos(%s,%d,True)\n",XtName(CB_List(w)),pos));
       XmListSelectPos(CB_List(w), pos, True);
       break;
 
@@ -820,9 +844,10 @@ CBListAction(
       break;
    }
 
-   DEBUGOUT(XdbDebug(__FILE__, w, "CBListAction return\n"));
+   DEBUGOUT(_LtDebugDebug(__FILE__, w, "CBListAction return\n"));
 }
 
+
 #if 0
 static void 
 CBSetSelectedItem(
@@ -838,6 +863,7 @@ CBSetSelectedItem(
 }
 #endif
 
+
 #if 0
 static void 
 CBSetSelectedPos(
@@ -849,6 +875,7 @@ CBSetSelectedPos(
 }
 #endif
 
+
 static void 
 CBTextFocusOut(
    Widget w,
@@ -865,6 +892,7 @@ CBTextFocusOut(
    XtCallActionProc(w, "CBFocusOut", event, NULL, 0);
 }
 
+
 /*
   FUNCTION: ChangeManaged
   SYNOPSIS: static void ChangeManaged(Widget w)
@@ -879,7 +907,7 @@ ChangeManaged(Widget w)
    XtWidgetGeometry geo;
    XtGeometryResult result;
 
-   DEBUGOUT(XdbDebug(__FILE__, w, "ChangeManaged ...\n"));
+   DEBUGOUT(_LtDebugDebug(__FILE__, w, "ChangeManaged ...\n"));
 
 #if 0
    if (!XtIsManaged(w))
@@ -904,6 +932,7 @@ ChangeManaged(Widget w)
    DoLayout(w);
 }
 
+
 /*
   FUNCTION: CheckMinimalSize
   SYNOPSIS: static void CheckMinimalSize(Widget w, XtWidgetGeometry *reply);
@@ -929,6 +958,7 @@ CheckMinimalSize(Widget w, XtWidgetGeometry *reply)
    }
 }
 
+
 #if 0
 static void
 CheckSetRenderTable(Widget w)
@@ -938,6 +968,7 @@ CheckSetRenderTable(Widget w)
 }
 #endif
 
+
 /*
   FUNCTION: ClassInitialize
   SYNOPSIS: static void ClassInitialize()
@@ -946,7 +977,7 @@ CheckSetRenderTable(Widget w)
   END:
 */
 static void
-ClassInitialize()
+ClassInitialize(void)
 {
    /* parse translations */
    parsed_list_translations = XtParseTranslationTable(
@@ -963,12 +994,14 @@ ClassInitialize()
       _XmComboBox_dropDownComboBoxAccelerators);
 }
 
+
 static void
 ClassPartInitialize(WidgetClass widget_class)
 {
    Unused(widget_class);
 }
 
+
 static void
 ComboBoxParentProcess(Widget w)
 {
@@ -983,6 +1016,7 @@ ComboBoxParentProcess(Widget w)
 #endif
 }
 
+
 /*
   FUNCTION: ComputeArrowSize
   SYNOPSIS: static Dimension ComputeArrowSize(Widget w)
@@ -996,7 +1030,7 @@ ComputeArrowSize(Widget w)
    Dimension arrowSize = 0;
    int fudge_factor = 1;
 
-   DEBUGOUT(XdbDebug(__FILE__, w, "ComputeArrowSize ...\n"));
+   DEBUGOUT(_LtDebugDebug(__FILE__, w, "ComputeArrowSize ...\n"));
 
    if (CB_Type(w) == XmDROP_DOWN_LIST
        || CB_Type(w) == XmDROP_DOWN_COMBO_BOX)
@@ -1022,30 +1056,31 @@ ComputeArrowSize(Widget w)
                    ); 
    }
 
-   DEBUGOUT(XdbDebug(__FILE__, w, "ComputeArrowSize: text margin height(%d) ...\n",
+   DEBUGOUT(_LtDebugDebug(__FILE__, w, "ComputeArrowSize: text margin height(%d) ...\n",
       TextF_MarginHeight(CB_EditBox(w))));
-   DEBUGOUT(XdbDebug(__FILE__, w, "ComputeArrowSize: text margin width(%d) ...\n",
+   DEBUGOUT(_LtDebugDebug(__FILE__, w, "ComputeArrowSize: text margin width(%d) ...\n",
       TextF_MarginWidth(CB_EditBox(w))));
-   DEBUGOUT(XdbDebug(__FILE__, w, "ComputeArrowSize: text height(%d) ...\n",
+   DEBUGOUT(_LtDebugDebug(__FILE__, w, "ComputeArrowSize: text height(%d) ...\n",
       XtHeight(CB_EditBox(w))));
-   DEBUGOUT(XdbDebug(__FILE__, w, "ComputeArrowSize: text width(%d) ...\n",
+   DEBUGOUT(_LtDebugDebug(__FILE__, w, "ComputeArrowSize: text width(%d) ...\n",
       XtWidth(CB_EditBox(w))));
-   DEBUGOUT(XdbDebug(__FILE__, w, "ComputeArrowSize: text border width(%d) ...\n",
+   DEBUGOUT(_LtDebugDebug(__FILE__, w, "ComputeArrowSize: text border width(%d) ...\n",
       XtBorderWidth(CB_EditBox(w))));
-   DEBUGOUT(XdbDebug(__FILE__, w, "ComputeArrowSize: text highlight thickness(%d) ...\n",
+   DEBUGOUT(_LtDebugDebug(__FILE__, w, "ComputeArrowSize: text highlight thickness(%d) ...\n",
       Prim_HighlightThickness(CB_EditBox(w))));
-   DEBUGOUT(XdbDebug(__FILE__, w, "ComputeArrowSize: text shadow thickness(%d) ...\n",
+   DEBUGOUT(_LtDebugDebug(__FILE__, w, "ComputeArrowSize: text shadow thickness(%d) ...\n",
       Prim_ShadowThickness(CB_EditBox(w))));
-   DEBUGOUT(XdbDebug(__FILE__, w, "ComputeArrowSize: text ascent (%d) ...\n",
+   DEBUGOUT(_LtDebugDebug(__FILE__, w, "ComputeArrowSize: text ascent (%d) ...\n",
       TextF_FontAscent(CB_EditBox(w))));
-   DEBUGOUT(XdbDebug(__FILE__, w, "ComputeArrowSize: text average char width (%d) ...\n",
+   DEBUGOUT(_LtDebugDebug(__FILE__, w, "ComputeArrowSize: text average char width (%d) ...\n",
       TextF_FontAverageWidth(CB_EditBox(w)))); 
 
-   DEBUGOUT(XdbDebug(__FILE__, w, "ComputeArrowSize: arrow_size(%d) ...\n", arrowSize));
+   DEBUGOUT(_LtDebugDebug(__FILE__, w, "ComputeArrowSize: arrow_size(%d) ...\n", arrowSize));
 
    return arrowSize;
 }
 
+
 /*
   FUNCTION: PreferredGeo
   SYNOPSIS: static void PreferredGeo(Widget w, XtWidgetGeometry *)
@@ -1057,11 +1092,12 @@ static void
 PreferredGeo(Widget w, XtWidgetGeometry *reply)
 {
    XtQueryGeometry(w, NULL, reply);
-   DEBUGOUT(XdbDebug(__FILE__, w,
+   DEBUGOUT(_LtDebugDebug(__FILE__, w,
                      "get preferred size of child %s\n",
-                     XdbWidgetGeometry2String(reply)));
+                     _LtDebugWidgetGeometry2String(reply)));
 }
 
+
 /*
   FUNCTION: AdjustWidth
   SYNOPSIS: static Dimension AdjustWidth(Widget w)
@@ -1087,6 +1123,7 @@ AdjustWidth(Widget w)
    return width;
 }
 
+
 /*
   FUNCTION: AdjustHeight
   SYNOPSIS: static Dimension AdjustHeight(Widget w)
@@ -1112,6 +1149,7 @@ AdjustHeight(Widget w)
    return height;
 }
 
+
 /*
   FUNCTION: ComputeSizeComboBox
   SYNOPSIS: static void ComputeSizeComboBox(Widget w, XtWidgetGeometry *)
@@ -1154,6 +1192,7 @@ ComputeSizeComboBox(Widget w, XtWidgetGeometry *reply)
    reply->height += AdjustHeight(w);
 }
 
+
 /*
   FUNCTION: ComputeSizeDropDown
   SYNOPSIS: static void ComputeSizeDropDown(Widget w, XtWidgetGeometry *)
@@ -1192,9 +1231,9 @@ ComputeSizeDropDown(Widget w, XtWidgetGeometry *reply)
       reply->height = CB_ArrowSize(w);
    }
 
-   DEBUGOUT(XdbDebug(__FILE__, w, "ComputeSize: arrow_spacing(%d) ...\n", 
+   DEBUGOUT(_LtDebugDebug(__FILE__, w, "ComputeSize: arrow_spacing(%d) ...\n", 
             CB_ArrowSpacing(w)));
-   DEBUGOUT(XdbDebug(__FILE__, w, "ComputeSize: arrow_size(%d) ...\n",
+   DEBUGOUT(_LtDebugDebug(__FILE__, w, "ComputeSize: arrow_size(%d) ...\n",
             CB_ArrowSize(w)));
 
    /* add the width and spacing for the arrow button */
@@ -1206,6 +1245,7 @@ ComputeSizeDropDown(Widget w, XtWidgetGeometry *reply)
    reply->height += AdjustHeight(w);
 }
 
+
 /*
   FUNCTION: ComputeSize
   SYNOPSIS: static void ComputeSize(Widget w, XtWidgetGeometry *)
@@ -1216,7 +1256,7 @@ ComputeSizeDropDown(Widget w, XtWidgetGeometry *reply)
 static void
 ComputeSize(Widget w, XtWidgetGeometry *reply)
 {
-   DEBUGOUT(XdbDebug(__FILE__, w, "ComputeSize ...\n"));
+   DEBUGOUT(_LtDebugDebug(__FILE__, w, "ComputeSize ...\n"));
 
    if (CB_Type(w) == XmCOMBO_BOX)
    {
@@ -1232,15 +1272,17 @@ ComputeSize(Widget w, XtWidgetGeometry *reply)
    reply->request_mode = (CWWidth|CWHeight);
 }
 
+
 static void
 ConstraintDestroy(Widget w)
 {
-   DEBUGOUT(XdbDebug(__FILE__, w, "ConstraintDestroy ...\n"));
+   DEBUGOUT(_LtDebugDebug(__FILE__, w, "ConstraintDestroy ...\n"));
 
    /* FIXTHIS: ConstraintDestroy needs to be implemented. */
    Unused(w);
 }
 
+
 /*
   FUNCTION: CreateArrowGC
   SYNOPSIS: static void CreateArrowGC(Widget w)
@@ -1254,7 +1296,7 @@ CreateArrowGC(Widget w)
    XGCValues values;
    XtGCMask mask;
 
-   DEBUGOUT(XdbDebug(__FILE__, w, "CreateArrowGC ...\n"));
+   DEBUGOUT(_LtDebugDebug(__FILE__, w, "CreateArrowGC ...\n"));
 
    mask = GCForeground | GCBackground;
 
@@ -1272,6 +1314,7 @@ CreateArrowGC(Widget w)
    CB_ArrowGC(w) = XtGetGC(w, mask, &values);
 }
 
+
 /*
   FUNCTION: CreateChildren
   SYNOPSIS: static void CreateChildren(Widget w)
@@ -1283,13 +1326,14 @@ CreateArrowGC(Widget w)
 static void
 CreateChildren(Widget w)
 {
-   DEBUGOUT(XdbDebug(__FILE__, w, "CreateChildren ...\n"));
+   DEBUGOUT(_LtDebugDebug(__FILE__, w, "CreateChildren ...\n"));
 
    CreateEditBox(w);
    CreatePulldown(w);
    CreateScrolledList(w);
 }
 
+
 /*
   FUNCTION: CreateEditBox
   SYNOPSIS: static void CreateEditBox(Widget w)
@@ -1304,7 +1348,7 @@ CreateEditBox(Widget w)
    Arg args[10];
    int n = 0;
  
-   DEBUGOUT(XdbDebug(__FILE__, w, "CreateEditBox ...\n"));
+   DEBUGOUT(_LtDebugDebug(__FILE__, w, "CreateEditBox ...\n"));
 
    if (CB_Type(w) == XmDROP_DOWN_LIST)
    {  /* force these settings */
@@ -1353,7 +1397,7 @@ CreatePulldown(Widget w)
    Arg args[10];
    int n = 0;
 
-   DEBUGOUT(XdbDebug(__FILE__, w, "CreatePulldown ...\n"));
+   DEBUGOUT(_LtDebugDebug(__FILE__, w, "CreatePulldown ...\n"));
 
    if (CB_Type(w) == XmDROP_DOWN_LIST
        || CB_Type(w) == XmDROP_DOWN_COMBO_BOX)
@@ -1394,6 +1438,7 @@ CreatePulldown(Widget w)
    }
 }
 
+
 /*
   FUNCTION: CreateScrolledList
   SYNOPSIS: static void CreateScrolledList(Widget parent)
@@ -1407,7 +1452,7 @@ CreateScrolledList(Widget w)
    Arg args[15];
    int n = 0;
 
-   DEBUGOUT(XdbDebug(__FILE__, w, "CreateScrolledList ...\n"));
+   DEBUGOUT(_LtDebugDebug(__FILE__, w, "CreateScrolledList ...\n"));
 
    /* force settings */
    if (CB_Type(w) == XmCOMBO_BOX)
@@ -1458,6 +1503,7 @@ CreateScrolledList(Widget w)
 #endif
 }
 
+
 /*
   FUNCTION: Destroy
   SYNOPSIS: static void Destroy(Widget w)
@@ -1468,7 +1514,7 @@ CreateScrolledList(Widget w)
 static void
 Destroy(Widget w)
 {
-   DEBUGOUT(XdbDebug(__FILE__, w, "Destroy ...\n"));
+   DEBUGOUT(_LtDebugDebug(__FILE__, w, "Destroy ...\n"));
 
    if (CB_SelectedItem(w))
    {
@@ -1508,6 +1554,7 @@ Destroy(Widget w)
    XtReleaseGC(w,CB_ArrowGC(w));
 }
 
+
 /*
   FUNCTION: DetectAndFixBadSettings
   SYNOPSIS: static void DetectAndFixBadSettings(Widget w)
@@ -1529,6 +1576,7 @@ DetectAndFixBadSettings(Widget w)
    }
 }
 
+
 /*
   FUNCTION: DetectAndFixChangesToReadOnlySettings
   SYNOPSIS: static void DetectAndFixChangesToReadOnlySettings(Widget ow, Widget nw)
@@ -1547,6 +1595,7 @@ DetectAndFixChangesToReadOnlySettings(Widget ow, Widget nw)
    }
 }
 
+
 /*
   FUNCTION: DoLayout
   SYNOPSIS: static void DoLayout(Widget w)
@@ -1557,7 +1606,7 @@ DetectAndFixChangesToReadOnlySettings(Widget ow, Widget nw)
 static void
 DoLayout(Widget w)
 {
-   DEBUGOUT(XdbDebug(__FILE__, w, "DoLayout ...\n"));
+   DEBUGOUT(_LtDebugDebug(__FILE__, w, "DoLayout ...\n"));
 
    /* if the kids are not valid then return */
    if(!KidsValid(w))
@@ -1569,6 +1618,7 @@ DoLayout(Widget w)
    AdjustKids(w);
 }
 
+
 /*
   FUNCTION: DrawArrow
   SYNOPSIS: static void DrawArrow(Widget w)
@@ -1593,7 +1643,7 @@ DrawArrow(Widget w)
    Position margin_left, margin_top;
 
 
-   DEBUGOUT(XdbDebug(__FILE__, w, "DrawArrow ...\n"));
+   DEBUGOUT(_LtDebugDebug(__FILE__, w, "DrawArrow ...\n"));
 
    /*
       REMARK:
@@ -1602,7 +1652,7 @@ DrawArrow(Widget w)
       END:
    */
 
-   DEBUGOUT(XdbDebug(__FILE__, 
+   DEBUGOUT(_LtDebugDebug(__FILE__, 
             w, 
             "DrawArrow: x(%d), y(%d), width(%d), height(%d) ...\n",
             (int) CB_HitRect(w).x,
@@ -1641,7 +1691,7 @@ DrawArrow(Widget w)
       normal_arrow_thickness = 1;
    }
 
-   DEBUGOUT(XdbDebug(__FILE__, 
+   DEBUGOUT(_LtDebugDebug(__FILE__, 
             w, 
             "DrawArrow: computed: x(%d), y(%d), width(%d), height(%d)\n",
             (int) normal_arrow_x,
@@ -1693,6 +1743,7 @@ DrawArrow(Widget w)
                   XmSHADOW_ETCHED_OUT_DASH);  /* shadow type */
 }
 
+
 /*
   FUNCTION: DrawBorder
   SYNOPSIS: static void DrawBorder(Widget w)
@@ -1702,10 +1753,10 @@ DrawArrow(Widget w)
   routine Redisplay.
   END:
 */
-void
+static void
 DrawBorder(Widget w)
 {
-   DEBUGOUT(XdbDebug(__FILE__, w, "DrawBorder ...\n"));
+   DEBUGOUT(_LtDebugDebug(__FILE__, w, "DrawBorder ...\n"));
 
    if (CB_Highlighted(w) == True)
    {
@@ -1717,6 +1768,7 @@ DrawBorder(Widget w)
    }
 }
 
+
 static void
 DrawComboBox(Widget w)
 {
@@ -1734,6 +1786,7 @@ DrawComboBox(Widget w)
    DrawBorder(w);
 }
 
+
 /*
   FUNCTION: DrawShadows
   SYNOPSIS: static void DrawShadows(Widget w)
@@ -1748,7 +1801,7 @@ DrawShadows(Widget w)
    Position normal_shadow_x, normal_shadow_y;
    Dimension normal_shadow_width, normal_shadow_height;
 
-   DEBUGOUT(XdbDebug(__FILE__, w, "DrawShadows ...\n"));
+   DEBUGOUT(_LtDebugDebug(__FILE__, w, "DrawShadows ...\n"));
 
    /* adjust for highlight and margins */
    normal_shadow_x = CB_HighlightThickness(w);
@@ -1756,7 +1809,7 @@ DrawShadows(Widget w)
    normal_shadow_width = XtWidth(w) - 2*normal_shadow_x;
    normal_shadow_height = XtHeight(w) - 2*normal_shadow_y;
 
-   DEBUGOUT(XdbDebug(__FILE__, 
+   DEBUGOUT(_LtDebugDebug(__FILE__, 
             w, 
             "DrawShadows: x(%d), y(%d), width(%d), height(%d) ...\n",
             (int) normal_shadow_x,
@@ -1776,6 +1829,7 @@ DrawShadows(Widget w)
                   XmSHADOW_ETCHED_OUT_DASH);  /* shadow type */
 }
 
+
 /*
   FUNCTION: _FindComboBox
   SYNOPSIS: static Widget _FindComboBox(Widget w)
@@ -1790,7 +1844,7 @@ _FindComboBox(Widget w)
 {
    Widget ww = w;
 
-   DEBUGOUT(XdbDebug(__FILE__, w, "_FindComboBox ...\n"));
+   DEBUGOUT(_LtDebugDebug(__FILE__, w, "_FindComboBox ...\n"));
 
    while(ww && !XmIsComboBox(ww))
    {
@@ -1801,6 +1855,7 @@ _FindComboBox(Widget w)
    return ww;
 }
 
+
 /*
   FUNCTION: FindComboBox
   SYNOPSIS: static Widget FindComboBox(Widget w)
@@ -1828,6 +1883,7 @@ FindComboBox(Widget w)
    return self;
 }
 
+
 /*
   FUNCTION: _FindVendorShell
   SYNOPSIS: static Widget _FindVendorShell(Widget w)
@@ -1842,7 +1898,7 @@ _FindVendorShell(Widget w)
 {
    Widget ww = w;
 
-   DEBUGOUT(XdbDebug(__FILE__, w, "_FindVendorShell ...\n"));
+   DEBUGOUT(_LtDebugDebug(__FILE__, w, "_FindVendorShell ...\n"));
 
    while(ww && !XtIsVendorShell(ww))
    {
@@ -1853,12 +1909,13 @@ _FindVendorShell(Widget w)
    return ww;
 }
 
+
 static XtGeometryResult 
 GeometryManager(Widget w, XtWidgetGeometry *request, XtWidgetGeometry *reply)
 {
-   DEBUGOUT(XdbDebug2(__FILE__, XtParent(w), w, "%s:GeometryManager %s\n",
+   DEBUGOUT(_LtDebugDebug2(__FILE__, XtParent(w), w, "%s:GeometryManager %s\n",
    		__FILE__,
-   		XdbWidgetGeometry2String(request)));
+   		_LtDebugWidgetGeometry2String(request)));
 
    Unused(w);
    Unused(request);
@@ -1874,6 +1931,7 @@ GeometryManager(Widget w, XtWidgetGeometry *request, XtWidgetGeometry *reply)
 #endif
 }
 
+
 /*
   FUNCTION: GetEditBoxValue
   SYNOPSIS: static XmString GetEditBoxValue(Widget w)
@@ -1887,7 +1945,7 @@ static XmString
 GetEditBoxValue(Widget w)
 {
    XmString s;
-   _XtString v;
+   char *v;
 
    XtVaGetValues(CB_EditBox(w),XmNvalue, &v, NULL);
    s = XmCvtCTToXmString(v);
@@ -1912,7 +1970,7 @@ HighlightBorder(Widget w)
       return;
    }
 
-   DEBUGOUT(XdbDebug(__FILE__, w, "HighlightBorder ...\n"));
+   DEBUGOUT(_LtDebugDebug(__FILE__, w, "HighlightBorder ...\n"));
 
    XmeDrawHighlight(XtDisplayOfObject(w),
                     XtWindowOfObject(w),
@@ -1926,6 +1984,7 @@ HighlightBorder(Widget w)
    CB_Highlighted(w) = True;
 }
 
+
 /*
   FUNCTION: Initialize
   SYNOPSIS: static void Initialize(Widget,Widget,ArgList,Cardinal *)
@@ -1939,7 +1998,7 @@ Initialize(Widget request,
            ArgList args,
            Cardinal *num_args)
 {
-    DEBUGOUT(XdbDebug(__FILE__, c_new,
+    DEBUGOUT(_LtDebugDebug(__FILE__, c_new,
 		      "%s:Initialize: %i args\n"
 		      "\trequest X %5i Y %5i W %5i H %5i\n"
 		      "\t  c_new X %5i Y %5i W %5i H %5i\n",
@@ -1949,7 +2008,7 @@ Initialize(Widget request,
 		      XtWidth(request), XtHeight(request),
 		      XtX(c_new), XtY(c_new),
 		      XtWidth(c_new), XtHeight(c_new)));
-   DEBUGOUT(XdbPrintArgList(__FILE__, c_new, args, *num_args, False));
+   DEBUGOUT(_LtDebugPrintArgList(__FILE__, c_new, args, *num_args, False));
 
    /* initialize internal data */
    CB_Highlighted(c_new) = False;
@@ -1969,9 +2028,9 @@ Initialize(Widget request,
    /* Set focus change handler */
    SetFocusChangeHandler(c_new);
 
-   DEBUGOUT(XdbDebug(__FILE__, c_new, "Initialize: IdealEBHeight(%d) ...\n",
+   DEBUGOUT(_LtDebugDebug(__FILE__, c_new, "Initialize: IdealEBHeight(%d) ...\n",
             CB_IdealEBHeight(c_new)));
-   DEBUGOUT(XdbDebug(__FILE__, c_new, "Initialize: IdealEBWidth(%d) ...\n",
+   DEBUGOUT(_LtDebugDebug(__FILE__, c_new, "Initialize: IdealEBWidth(%d) ...\n",
             CB_IdealEBWidth(c_new)));
 
    /* The following must be done after creating the children */
@@ -1997,6 +2056,7 @@ Initialize(Widget request,
 
 }
 
+
 /*
   FUNCTION: InsertChild
   SYNOPSIS: static void InsertChild(Widget w)
@@ -2010,7 +2070,7 @@ InsertChild(Widget w)
 {
    Widget parent = XtParent(w);
 
-   DEBUGOUT(XdbDebug2(__FILE__, (Widget)parent, w, "InsertChild\n"));
+   DEBUGOUT(_LtDebugDebug2(__FILE__, (Widget)parent, w, "InsertChild\n"));
 
    /*
       REMARK: 
@@ -2027,6 +2087,7 @@ InsertChild(Widget w)
    (*superclass->composite_class.insert_child)(w);
 }
 
+
 /*
   FUNCTION: ListSelectionCB
   SYNOPSIS: static void ListSelectionCB(Widget w, XtPointer client_data, XtPointer data)
@@ -2042,7 +2103,7 @@ ListSelectionCB(Widget w, XtPointer client_data, XtPointer data)
    Widget cb = (Widget) client_data;
    XmString v;
 
-   DEBUGOUT(XdbDebug(__FILE__, w, "%s:ListSelectionCB ...\n",
+   DEBUGOUT(_LtDebugDebug(__FILE__, w, "%s:ListSelectionCB ...\n",
    	__FILE__));
 
    v = GetEditBoxValue(cb);
@@ -2081,15 +2142,17 @@ ListSelectionCB(Widget w, XtPointer client_data, XtPointer data)
    CBDisarm(cb,cbs->event,NULL,NULL);
 } 
 
+
 static void
 PopdownList(Widget w, XEvent *event)
 {
-   DEBUGOUT(XdbDebug(__FILE__, w, "%s:PopdownList ...\n",
+   DEBUGOUT(_LtDebugDebug(__FILE__, w, "%s:PopdownList ...\n",
    	__FILE__));
 
    XtCallActionProc(CB_ListShell(w), "GrabShellPopdown", event, NULL, 0);
 }
 
+
 /*
   FUNCTION: PopupEH
   SYNOPSIS: static void PopupEH(Widget, XtPointer, XEvent *, Boolean *)
@@ -2103,7 +2166,7 @@ PopupEH(Widget w, XtPointer udata, XEvent *xev, Boolean *pb)
    XButtonPressedEvent *xbp = (XButtonPressedEvent *)xev;
    XRectangle *rect;
 
-   DEBUGOUT(XdbDebug(__FILE__, w, "PopupEH ...\n"));
+   DEBUGOUT(_LtDebugDebug(__FILE__, w, "PopupEH ...\n"));
 
    Unused(udata);
 
@@ -2115,9 +2178,9 @@ PopupEH(Widget w, XtPointer udata, XEvent *xev, Boolean *pb)
       return;
    }
 
-   DEBUGOUT(XdbDebug(__FILE__, w, "PopupEH: hit at: x(%d), y(%d)\n",
+   DEBUGOUT(_LtDebugDebug(__FILE__, w, "PopupEH: hit at: x(%d), y(%d)\n",
             (int)xbp->x,(int)xbp->y));
-   DEBUGOUT(XdbDebug(__FILE__, w, 
+   DEBUGOUT(_LtDebugDebug(__FILE__, w, 
             "PopupEH: hit area: x(%d), y(%d), width(%d), height(%d)\n",
             (int) CB_HitRect(w).x,
             (int) CB_HitRect(w).y,
@@ -2137,7 +2200,7 @@ PopupEH(Widget w, XtPointer udata, XEvent *xev, Boolean *pb)
    /* Is the grab shell managed? */
    if (!XtIsManaged(CB_ListShell(w)))
    {
-      DEBUGOUT(XdbDebug(__FILE__, w, "PopupEH: Not managed so show grab shell\n"));
+      DEBUGOUT(_LtDebugDebug(__FILE__, w, "PopupEH: Not managed so show grab shell\n"));
 
       /* display grab shell */
       XtPopup(CB_ListShell(w), XtGrabNone);
@@ -2147,7 +2210,7 @@ PopupEH(Widget w, XtPointer udata, XEvent *xev, Boolean *pb)
    }
    else
    {
-      DEBUGOUT(XdbDebug(__FILE__, w, "PopupEH: Managed so hide grab shell\n"));
+      DEBUGOUT(_LtDebugDebug(__FILE__, w, "PopupEH: Managed so hide grab shell\n"));
 
       XtUnmanageChild(CB_ListShell(w));
       /*
@@ -2157,6 +2220,7 @@ PopupEH(Widget w, XtPointer udata, XEvent *xev, Boolean *pb)
 #endif
 }
 
+
 /*
   FUNCTION: QueryGeometry
   SYNOPSIS: static XtGeometryResult QueryGeometry(Widget,XtWidgetGeometry *,XtWidgetGeometry *)
@@ -2169,9 +2233,9 @@ QueryGeometry(Widget w, XtWidgetGeometry *proposed, XtWidgetGeometry *answer)
    XtGeometryResult result = XtGeometryYes;
    XtWidgetGeometry geo;
 
-   DEBUGOUT(XdbDebug(__FILE__, w, "%s:QueryGeometry %s\n",
+   DEBUGOUT(_LtDebugDebug(__FILE__, w, "%s:QueryGeometry %s\n",
 		      __FILE__,
-		      XdbWidgetGeometry2String(proposed)));
+		      _LtDebugWidgetGeometry2String(proposed)));
 
    proposed->request_mode &= CWWidth | CWHeight;
    if(proposed->request_mode == 0) 
@@ -2208,6 +2272,7 @@ QueryGeometry(Widget w, XtWidgetGeometry *proposed, XtWidgetGeometry *answer)
    return result;
 }
 
+
 /* 
   FUNCTION: Redisplay
   SYNOPSIS: static void Redisplay(Widget w)
@@ -2218,7 +2283,7 @@ QueryGeometry(Widget w, XtWidgetGeometry *proposed, XtWidgetGeometry *answer)
 static void 
 Redisplay(Widget w, XEvent *event, Region region)
 {
-   DEBUGOUT(XdbDebug(__FILE__, w, "%s:Redisplay ...\n",
+   DEBUGOUT(_LtDebugDebug(__FILE__, w, "%s:Redisplay ...\n",
    	__FILE__));
 
    Unused(event);
@@ -2237,7 +2302,7 @@ Redisplay(Widget w, XEvent *event, Region region)
 static void 
 Resize(Widget w)
 {
-   DEBUGOUT(XdbDebug(__FILE__, w, "%s:Resize %ix%i\n",
+   DEBUGOUT(_LtDebugDebug(__FILE__, w, "%s:Resize %ix%i\n",
    	__FILE__,
    	XtWidth(w), XtHeight(w)));
 
@@ -2255,6 +2320,7 @@ Resize(Widget w)
    (*superclass->core_class.resize)(w);
 }
 
+
 #if 0
 /*
   FUNCTION: SBBtnDownEH()
@@ -2291,10 +2357,11 @@ SBBtnUpEH(Widget w, XtPointer udata, XEvent *xev, Boolean *pb)
 }
 #endif
 
+
 static void
 SetEditBoxValue(Widget w, XmString s)
 {
-   _XtString v;
+   char *v;
 
    /* Convert XmString to a string */
    v = XmCvtXmStringToCT(s);
@@ -2326,6 +2393,7 @@ SetFocusChangeHandler(Widget w)
                              (XtPointer) w);
 }
 
+
 static void
 RemoveFocusChangeHandler(Widget w)
 {
@@ -2353,6 +2421,7 @@ SetHitArea(Widget w)
    SetHitAreaSize(w);
 }
 
+
 /*
   FUNCTION: SetHitAreaHandler
   SYNOPSIS: static void SetHitAreaHandler(Widget w)
@@ -2366,7 +2435,7 @@ SetHitAreaHandler(Widget w)
 {
    EventMask mask;
 
-   DEBUGOUT(XdbDebug(__FILE__, w, "SetHitAreaHandler ...\n"));
+   DEBUGOUT(_LtDebugDebug(__FILE__, w, "SetHitAreaHandler ...\n"));
 
    if (CB_Type(w) == XmDROP_DOWN_LIST
        || CB_Type(w) == XmDROP_DOWN_COMBO_BOX)
@@ -2377,12 +2446,13 @@ SetHitAreaHandler(Widget w)
    }
 }
 
+
 static void
 RemoveHitAreaHandler(Widget w)
 {
    EventMask mask;
 
-   DEBUGOUT(XdbDebug(__FILE__, w, "RemoveHitAreaHandler ...\n"));
+   DEBUGOUT(_LtDebugDebug(__FILE__, w, "RemoveHitAreaHandler ...\n"));
 
    if (CB_Type(w) == XmDROP_DOWN_LIST
        || CB_Type(w) == XmDROP_DOWN_COMBO_BOX)
@@ -2403,7 +2473,7 @@ RemoveHitAreaHandler(Widget w)
 static void
 SetHitAreaSize(Widget w)
 {
-   DEBUGOUT(XdbDebug(__FILE__, w, "SetHitAreaSize ...\n"));
+   DEBUGOUT(_LtDebugDebug(__FILE__, w, "SetHitAreaSize ...\n"));
 
    /* Always use CB_ArrowSize for the width */
    CB_HitRect(w).width = CB_ArrowSize(w);
@@ -2419,6 +2489,7 @@ SetHitAreaSize(Widget w)
                      + CB_ArrowSpacing(w);
 }
 
+
 /*
   FUNCTION: SetIdealTextSize
   SYNOPSIS: static void SetIdealTextSize(Widget w)
@@ -2431,12 +2502,13 @@ SetHitAreaSize(Widget w)
 static void
 SetIdealTextSize(Widget w)
 {
-   DEBUGOUT(XdbDebug(__FILE__, w, "SetIdealTextSize ...\n"));
+   DEBUGOUT(_LtDebugDebug(__FILE__, w, "SetIdealTextSize ...\n"));
 
    CB_IdealEBHeight(w) = XtHeight(CB_EditBox(w));
    CB_IdealEBWidth(w) = XtWidth(CB_EditBox(w));
 }
 
+
 /* 
   FUNCTION: SetValues
   SYNOPSIS: static Boolean SetValues(Widget,Widget,Widget,ArgList,Cardinal *)
@@ -2459,7 +2531,7 @@ SetValues(Widget old,
 
    /* FIXTHIS: */
 
-    DEBUGOUT(XdbDebug(__FILE__, c_new,
+    DEBUGOUT(_LtDebugDebug(__FILE__, c_new,
 		      "%s:SetValues: %i args\n"
 		      "\t    old X %5i Y %5i W %5i H %5i\n"
 		      "\trequest X %5i Y %5i W %5i H %5i\n"
@@ -2472,7 +2544,7 @@ SetValues(Widget old,
 		      XtWidth(request), XtHeight(request),
 		      XtX(c_new), XtY(c_new),
 		      XtWidth(c_new), XtHeight(c_new)));
-    DEBUGOUT(XdbPrintArgList(__FILE__, c_new, args, *num_args, False));
+    DEBUGOUT(_LtDebugPrintArgList(__FILE__, c_new, args, *num_args, False));
 
    DetectAndFixChangesToReadOnlySettings(old, c_new);
    DetectAndFixBadSettings(c_new);
@@ -2484,6 +2556,7 @@ SetValues(Widget old,
    return need_refresh;
 }
 
+
 /*
   FUNCTION: ShellFocusMovedCB
   SYNOPSIS: static void ShellFocusMovedCB(Widget, XtPointer, XtPointer)
@@ -2498,7 +2571,7 @@ ShellFocusMovedCB(Widget w, XtPointer client_data, XtPointer call_data)
    Widget find_self;
    XmFocusMovedCallbackStruct *cbs = (XmFocusMovedCallbackStruct *) call_data;
 
-   DEBUGOUT(XdbDebug(__FILE__, self, "ShellFocusMovedCB ...\n"));
+   DEBUGOUT(_LtDebugDebug(__FILE__, self, "ShellFocusMovedCB ...\n"));
 
    /* we will find ourself if new focus widget is a child */
    find_self = _FindComboBox(cbs->new_focus);
@@ -2520,6 +2593,7 @@ ShellFocusMovedCB(Widget w, XtPointer client_data, XtPointer call_data)
    Unused(call_data);
 } 
 
+
 static void
 ShellPopdownCB(Widget w, XtPointer client_data, XtPointer data) 
 {
@@ -2534,6 +2608,7 @@ ShellPopdownCB(Widget w, XtPointer client_data, XtPointer data)
 #endif
 } 
 
+
 static void
 ShellPopupCB(Widget w, XtPointer client_data, XtPointer data)
 {
@@ -2544,6 +2619,7 @@ ShellPopupCB(Widget w, XtPointer client_data, XtPointer data)
    Unused(data);
 } 
 
+
 /*
   FUNCTION: TextChangedCB
   SYNOPSIS: static void TextChangedCB(Widget w, XtPointer cdata, XtPointer data)
@@ -2554,7 +2630,7 @@ ShellPopupCB(Widget w, XtPointer client_data, XtPointer data)
 static void
 TextChangedCB(Widget w, XtPointer client_data, XtPointer data)
 {
-   DEBUGOUT(XdbDebug(__FILE__, w, "TextChangedCB ...\n"));
+   DEBUGOUT(_LtDebugDebug(__FILE__, w, "TextChangedCB ...\n"));
 
    Unused(w);
    Unused(client_data);
@@ -2579,9 +2655,9 @@ UnhighlightBorder(Widget w)
       return;
    }
 
-   DEBUGOUT(XdbDebug(__FILE__, w, "UnhighlightBorder ...\n"));
+   DEBUGOUT(_LtDebugDebug(__FILE__, w, "UnhighlightBorder ...\n"));
 
-   DEBUGOUT(XdbDebug(__FILE__, 
+   DEBUGOUT(_LtDebugDebug(__FILE__, 
             w, 
             "UnhighlightBorder: x(%d), y(%d), width(%d), height(%d) ...\n",
             (int) highlight_offset,
@@ -2602,6 +2678,7 @@ UnhighlightBorder(Widget w)
    CB_Highlighted(w) = False;
 }
 
+
 /*
   FUNCTION: AdjustKids
   SYNOPSIS: static Boolean AdjustKids(Widget w)
@@ -2624,6 +2701,7 @@ AdjustKids(Widget w)
    }
 }
 
+
 /*
   FUNCTION: AdjustKidsComboBox
   SYNOPSIS: static Boolean AdjustKidsComboBox(Widget w)
@@ -2637,7 +2715,7 @@ AdjustKidsComboBox(Widget w)
    int edit_x, edit_y, edit_height, edit_width;
    int scrolledw_x, scrolledw_y, scrolledw_width, scrolledw_height;
 
-   DEBUGOUT(XdbDebug(__FILE__, w, "AdjustKidsComboBox ...\n"));
+   DEBUGOUT(_LtDebugDebug(__FILE__, w, "AdjustKidsComboBox ...\n"));
 
    /* Perform adjustments to the edit box */
    edit_x = edit_y = 0;
@@ -2727,7 +2805,7 @@ AdjustKidsDropDown(Widget w)
    int edit_x, edit_y, edit_height, edit_width;
    int shell_x, shell_y, shell_height, shell_width;
 
-   DEBUGOUT(XdbDebug(__FILE__, w, "AdjustKidsDropDown ...\n"));
+   DEBUGOUT(_LtDebugDebug(__FILE__, w, "AdjustKidsDropDown ...\n"));
    /*
       FIXTHIS: Confirm if docs are correct.
       REMARK:
@@ -2935,7 +3013,7 @@ ComboBoxFocusMovedCB(Widget w, XtPointer client_data, XtPointer call_data)
    Boolean to_self = False, from_self = False;
    XmFocusMovedCallbackStruct *cbs = (XmFocusMovedCallbackStruct *) call_data;
 
-   DEBUGOUT(XdbDebug(__FILE__, self, "ComboBoxFocusMovedCB ...\n"));
+   DEBUGOUT(_LtDebugDebug(__FILE__, self, "ComboBoxFocusMovedCB ...\n"));
 
    /* continue ? */
    if(!cbs->cont)
@@ -2978,7 +3056,7 @@ ComboBoxFocusMovedCB(Widget w, XtPointer client_data, XtPointer call_data)
   Creates an instance of a XmComboxWidget.
   END:
 */
-Widget 
+extern Widget 
 XmCreateComboBox(Widget parent, char *name, ArgList arglist,
                  Cardinal cnt)
 {
@@ -2997,7 +3075,7 @@ XmCreateComboBox(Widget parent, char *name, ArgList arglist,
   set to XmDROP_DOWN_COMBO_BOX.
   END:
 */
-Widget 
+extern Widget 
 XmCreateDropDownComboBox(Widget parent,
                          char *name,
                          Arg *arglist,
@@ -3024,6 +3102,7 @@ XmCreateDropDownComboBox(Widget parent,
    return w;
 }
 
+
 /* 
   FUNCTION: XmCreateDropDownList
   SYNOPSIS: Widget XmCreateDropDownList(Widget,char *,ArgList,Cardinal)
@@ -3032,7 +3111,7 @@ XmCreateDropDownComboBox(Widget parent,
   set to XmDROP_DOWN_LIST.
   END:
 */
-Widget 
+extern Widget 
 XmCreateDropDownList(Widget parent,
                      char *name,
                      Arg *arglist,
@@ -3059,19 +3138,7 @@ XmCreateDropDownList(Widget parent,
    return w;
 }
 
-/* 
-  FUNCTION: XmComboBoxUpdate
-  SYNOPSIS: void XmComboBoxUpdate(Widget w)
-  DESCRIPTION:
-  Communicates any dependent changes between child widgets.
-  END:
-*/
-void 
-XmComboBoxUpdate(Widget w)
-{
-   /* FIXTHIS: XmComboBoxUpdate needs to be implemented. */
-   Unused(w);
-}
+
 
 /*
   REMARK:
@@ -3099,20 +3166,59 @@ The text field widget is called "Text".
   END:
 */
 
-void
-XmComboBoxAddItem(Widget w, XmString xms, int a, int b)
+#if XmVersion >= 2001
+extern void
+XmComboBoxAddItem(Widget w, XmString xms, int a, Boolean unique)
 {
 	char    *s = NULL;
 	XmStringGetLtoR(xms, XmFONTLIST_DEFAULT_TAG, &s);
 #if 0
-	fprintf(stderr, "XmComboBoxAddItem(%s,%d,%d)\n", s, a, b);
+	fprintf(stderr, "XmComboBoxAddItem(%s,%d,%d)\n", s, a, (int)unique);
 #endif
 	XtFree(s);
-
+      /* FIXME: what about 'unique' ?? */
 	XmListAddItemUnselected(CB_List(w), xms, a);
 }
 
 
+extern void
+XmComboBoxDeletePos(Widget w, int pos)
+{
+  _XmWarning(w, "XmComboBoxDeletePos is not yet implemented!");
+}
+
+
+extern void
+XmComboBoxSelectItem(Widget w, XmString item)
+{
+  _XmWarning(w, "XmComboBoxSelectItem is not yet implemented!");
+}
+
+
+extern void
+XmComboBoxSetItem(Widget w, XmString item)
+{
+  _XmWarning(w, "XmComboBoxSetItem is not yet implemented!");
+}
+
+#endif /* XmVersion >= 2001 */
+
+
+/* 
+  FUNCTION: XmComboBoxUpdate
+  SYNOPSIS: void XmComboBoxUpdate(Widget w)
+  DESCRIPTION:
+  Communicates any dependent changes between child widgets.
+  END:
+*/
+extern void 
+XmComboBoxUpdate(Widget w)
+{
+   /* FIXTHIS: XmComboBoxUpdate needs to be implemented. */
+   Unused(w);
+}
+
+
 /* ================================================================ */
 /* Utility routines the we may want to make public */
 /*
@@ -3164,7 +3270,7 @@ __LTShellSmartPlacement(Widget w, int *ret_x, int *ret_y)
     int x,y,width,height;
     int dpy_width, dpy_height, scr;
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "__LTShellSmartPlacement...\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "__LTShellSmartPlacement...\n"));
 
     /* Is shell realized? */
     if(!XtIsRealized(w))
@@ -3196,7 +3302,7 @@ __LTShellSmartPlacement(Widget w, int *ret_x, int *ret_y)
     *ret_x = x;
     *ret_y = y;
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "__LTShellSmartPlacement...x=%d,y=%d\n",x,y));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "__LTShellSmartPlacement...x=%d,y=%d\n",x,y));
 }
 
 /*
@@ -3232,18 +3338,18 @@ _LTShellSmartPlacement(Widget w)
 static void
 GetValuesHook(Widget w, ArgList args, Cardinal *num_args)
 {
-	int	i;
-	XrmQuark	tf = XrmStringToQuark(XmNtextField), q,
-			l = XrmStringToQuark(XmNlist);
-	XtPointer	*p;
+	Cardinal i;
+	XrmQuark tf = XrmStringToQuark(XmNtextField), q,
+		    l = XrmStringToQuark(XmNlist);
+	XtPointer *p;
 
 	for (i=0; i<*num_args; i++) {
 		q = XrmStringToQuark(args[i].name);
 		if (q == tf) {	/* XmNtextField */
-			p = (XtPointer) args[i].value;
+			p = (XtPointer *) args[i].value;
 			*p = (XtPointer)XtNameToWidget(w, "Text");
 		} else if (q == l) {	/* XmNlist */
-			p = (XtPointer) args[i].value;
+			p = (XtPointer *) args[i].value;
 			*p = (XtPointer)CB_List(w);
 		} else {
 		}
