@@ -1,8 +1,9 @@
 /**
  *
- * $Id: Container.c,v 1.7 1999/10/26 22:22:19 danny Exp $
+ * $Header: /cvsroot/lesstif/lesstif/lib/Xm-2.0/Container.c,v 1.14 2001/05/14 19:46:11 amai Exp $
  *
  * Copyright (C) 1997-1999 Free Software Foundation, Inc.
+ * Copyright (C) 1999-2001 LessTif Development Team
  *
  * This file is part of the GNU LessTif Library.
  *
@@ -23,8 +24,8 @@
  **/
 
 #include <LTconfig.h>
-#include <XmI/XmI.h>
 
+#include <XmI/XmI.h>
 #include <Xm/XmP.h>
 #include <Xm/BaseClassP.h>
 #include <Xm/Container.h>
@@ -33,12 +34,14 @@
 
 #include <XmI/DebugUtil.h>
 
+static const char rcsid[] = "$Header: /cvsroot/lesstif/lesstif/lib/Xm-2.0/Container.c,v 1.14 2001/05/14 19:46:11 amai Exp $";
 
 /* 
   MODULE:	Container.c
   DESCRIPTION:	Implement the XmContainer widget class.
   STATUS:	Not too well yet.
-  END:
+  
+
 */
 
 /*
@@ -341,7 +344,7 @@ XmCreateContainer(Widget parent, String name, ArgList arglist, Cardinal argcount
 static void class_part_initialize(WidgetClass widget_class)
 {
 	/* FIXTHIS: Needs to be implemented */
-	DEBUGOUT(XdbDebug(__FILE__, NULL, "XmContainer class_part_initialize\n"));
+	DEBUGOUT(_LtDebugDebug(__FILE__, NULL, "XmContainer class_part_initialize\n"));
 }
 
 static void
@@ -349,7 +352,7 @@ Initialize(Widget request, Widget new_w, ArgList args, Cardinal *num_args)
 {
 	Dimension	wid, ht;
 	/* FIXTHIS: Needs to be implemented */
-	DEBUGOUT(XdbDebug(__FILE__, new_w, "XmContainer initialize\n"));
+	DEBUGOUT(_LtDebugDebug(__FILE__, new_w, "XmContainer initialize\n"));
 	_XmContainerLayout(new_w, &wid, &ht);
 
 	XtWidth(new_w) = (wid < 100) ? 100 : wid;
@@ -360,7 +363,7 @@ static void
 resize(Widget w)
 {
 	/* FIXTHIS: Needs to be implemented */
-	DEBUGOUT(XdbDebug(__FILE__, w, "XmContainer resize\n"));
+	DEBUGOUT(_LtDebugDebug(__FILE__, w, "XmContainer resize\n"));
 }
 
 
@@ -368,14 +371,14 @@ static Boolean
 SetValues(Widget current, Widget request, Widget new_w, ArgList args, Cardinal *num_args)
 {
 	/* FIXTHIS: Needs to be implemented */
-	DEBUGOUT(XdbDebug(__FILE__, new_w, "XmContainer SetValues\n"));
+	DEBUGOUT(_LtDebugDebug(__FILE__, new_w, "XmContainer SetValues\n"));
 	return True;
 }
 
 static void
 expose(Widget w, XEvent *event, Region region)
 {
-	DEBUGOUT(XdbDebug(__FILE__, w, "XmContainer expose\n"));
+	DEBUGOUT(_LtDebugDebug(__FILE__, w, "XmContainer expose\n"));
 	/* FIXTHIS: Needs to be implemented */
 
 	_XmRedisplayGadgets(w, event, region);
@@ -388,9 +391,9 @@ expose(Widget w, XEvent *event, Region region)
 static void
 _XmContainerLayout(Widget w, Dimension *pwid, Dimension *pht)
 {
-	Dimension	wid, ht;
-	int		i;
-	Position	x, y;
+	Dimension wid, ht;
+	Cardinal i;
+	Position x, y;
 
 	wid = ContainerMarginWidth(w);
 	ht = 0;
@@ -430,7 +433,7 @@ QueryGeometry(Widget w, XtWidgetGeometry *request, XtWidgetGeometry *reply)
 
 	/* FIXTHIS: Needs to be implemented */
 
-	DEBUGOUT(XdbDebug(__FILE__, w, "XmContainer QueryGeometry\n"));
+	DEBUGOUT(_LtDebugDebug(__FILE__, w, "XmContainer QueryGeometry\n"));
 	g = *request;
 
 	_XmContainerLayout(w, &wid, &ht);
@@ -451,7 +454,7 @@ QueryGeometry(Widget w, XtWidgetGeometry *request, XtWidgetGeometry *reply)
 static XtGeometryResult
 GeometryManager(Widget w, XtWidgetGeometry *request, XtWidgetGeometry *reply)
 {
-	DEBUGOUT(XdbDebug(__FILE__, w, "XmContainer GeometryManager\n"));
+	DEBUGOUT(_LtDebugDebug(__FILE__, w, "XmContainer GeometryManager\n"));
 
 	/* FIXTHIS: Needs to be implemented */
 	*reply = *request;
@@ -461,7 +464,7 @@ GeometryManager(Widget w, XtWidgetGeometry *request, XtWidgetGeometry *reply)
 static void
 ChangeManaged(Widget w)
 {
-	DEBUGOUT(XdbDebug(__FILE__, w, "XmContainer ChangeManaged\n"));
+	DEBUGOUT(_LtDebugDebug(__FILE__, w, "XmContainer ChangeManaged\n"));
 
 	/* FIXTHIS: Needs to be implemented */
 	_XmNavigChangeManaged(w);
@@ -473,7 +476,7 @@ static void
 ConstraintInitialize(Widget request, Widget new_w, ArgList args, Cardinal *num_args)
 {
 	/* FIXTHIS: Needs to be implemented */
-	DEBUGOUT(XdbDebug2(__FILE__, XtParent(new_w), new_w, "XmContainer ConstraintInitialize\n"));
+	DEBUGOUT(_LtDebugDebug2(__FILE__, XtParent(new_w), new_w, "XmContainer ConstraintInitialize\n"));
 
 	_XmContainerLayout(XtParent(new_w), 0, 0);
 }
@@ -482,14 +485,70 @@ static void
 ConstraintDestroy(Widget w)
 {
 	/* FIXTHIS: Needs to be implemented */
-	DEBUGOUT(XdbDebug(__FILE__, w, "XmContainer ConstraintDestroy\n"));
+	DEBUGOUT(_LtDebugDebug(__FILE__, w, "XmContainer ConstraintDestroy\n"));
 }
 
 static Boolean
 ConstraintSetValues(Widget current, Widget request, Widget new_w, ArgList args, Cardinal *num_args)
 {
-	DEBUGOUT(XdbDebug(__FILE__, new_w, "XmContainer ConstraintSetValues\n"));
+	DEBUGOUT(_LtDebugDebug(__FILE__, new_w, "XmContainer ConstraintSetValues\n"));
 	/* FIXTHIS: Needs to be implemented */
 	_XmContainerLayout(XtParent(new_w), NULL, NULL);
 	return True;
 }
+
+
+int XmContainerGetItemChildren(Widget w, 
+                               Widget item,
+                               WidgetList *item_children) {
+
+  _XmWarning(NULL, "XmContainerGetItemChildren(): not yet implemented!");
+  return 0;
+}
+
+
+void XmContainerRelayout(Widget w) {
+
+  _XmWarning(NULL, "XmContainerRelayout(): not yet implemented!");
+}
+
+
+void XmContainerReorder(Widget w, WidgetList cwid_list, int cwid_count) {
+
+  _XmWarning(NULL, "XmContainerReorder(): not yet implemented!");
+}
+
+
+Boolean XmContainerCut(Widget w, Time timestamp) {
+
+  _XmWarning(NULL, "XmContainerCut(): not yet implemented!");
+  return False;
+}
+
+
+Boolean XmContainerCopy(Widget w, Time timestamp) {
+
+  _XmWarning(NULL, "XmContainerCopy(): not yet implemented!");
+  return False;
+}
+
+
+Boolean XmContainerPaste(Widget w) {
+
+  _XmWarning(NULL, "XmContainerPaste(): not yet implemented!");
+  return False;
+}
+
+
+Boolean XmContainerCopyLink(Widget w, Time timestamp) {
+
+  _XmWarning(NULL, "XmContainerCopyLink(): not yet implemented!");
+  return False;
+}
+
+
+Boolean XmContainerPasteLink(Widget w) {
+
+  _XmWarning(NULL, "XmContainerPasteLink(): not yet implemented!");
+  return False;
+}
