@@ -1,8 +1,9 @@
 /**
  *
- * $Id: IconG.c,v 1.5 1999/02/07 12:25:46 pgw Exp $
+ * $Header: /cvsroot/lesstif/lesstif/lib/Xm-2.0/IconG.c,v 1.12 2001/05/14 19:46:11 amai Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
+ * Copyright (C) 1995-2001 LessTif Development Team
  *
  * This file is part of the GNU LessTif Library.
  *
@@ -22,11 +23,14 @@
  *
  **/
 
-static const char rcsid[] = "$Id: IconG.c,v 1.5 1999/02/07 12:25:46 pgw Exp $";
+static const char rcsid[] = "$Header: /cvsroot/lesstif/lesstif/lib/Xm-2.0/IconG.c,v 1.12 2001/05/14 19:46:11 amai Exp $";
 
 #include <LTconfig.h>
-#include <XmI/XmI.h>
 
+#include <string.h>
+
+#include <XmI/XmI.h>
+#include <XmI/PixConvI.h>
 #include <Xm/XmP.h>
 #include <Xm/CacheP.h>
 #include <Xm/CascadeB.h>
@@ -37,11 +41,10 @@ static const char rcsid[] = "$Id: IconG.c,v 1.5 1999/02/07 12:25:46 pgw Exp $";
 #include <Xm/IconGP.h>
 #include <Xm/RowColumnP.h>
 #include <Xm/XmosP.h>
-#include <X11/Xfuncs.h>
 #include <X11/ShellP.h>
 
 #include <XmI/DebugUtil.h>
-#include <XmI/PixConvI.h>
+
 
 #ifndef XmUNSPECIFIED
 #define XmUNSPECIFIED (~0)
@@ -54,7 +57,7 @@ static const char rcsid[] = "$Id: IconG.c,v 1.5 1999/02/07 12:25:46 pgw Exp $";
 
 /* Forward Declarations */
 
-static void class_initialize();
+static void class_initialize(void);
 
 static void class_part_initialize(WidgetClass w_class);
 
@@ -426,7 +429,7 @@ secondary_object_create(Widget request, Widget new_w,
     XmWidgetExtData ed;
     int size;
 
-    DEBUGOUT(XdbDebug(__FILE__, new_w,
+    DEBUGOUT(_LtDebugDebug(__FILE__, new_w,
 		      "IconGCacheRec %s being initialized.\n",
 		      XtName(new_w)));
 
@@ -456,7 +459,7 @@ secondary_object_create(Widget request, Widget new_w,
     ed->widget = (Widget)nsec;
     ed->reqWidget = (Widget)rsec;
 
-    bcopy(nsec, rsec, size);
+    memcpy(rsec, nsec, size);
     ((XmExtRec *)rsec)->object.self = (Widget)rsec;
 
     _XmPushWidgetExtData(new_w, ed, XmCACHE_EXTENSION);
@@ -473,14 +476,14 @@ secondary_object_create(Widget request, Widget new_w,
 int
 _XmIconCacheCompare(XtPointer A, XtPointer B)
 {
-    return !bcmp(((XmIconGCacheObjPart *)A),
+    return !memcmp(((XmIconGCacheObjPart *)A),
 		 ((XmIconGCacheObjPart *)B),
 		 sizeof(XmIconGCacheObjPart));
 }
 
 /******************************* GADGET PART *********************************/
 static void
-class_initialize()
+class_initialize(void)
 {
 #if 0
     XtResourceList combined, labels;
@@ -514,10 +517,10 @@ class_initialize()
 			     &labels, &nlabels);
 
     combined = (XtResourceList)XtMalloc(sizeof(XtResource) * ncom);
-    bcopy(labels, combined, nlabels * sizeof(XtResource));
-    bcopy(cache_resources,
-	  &combined[nlabels],
-	  XtNumber(cache_resources) * sizeof(XtResource));
+    memcpy(combined, labels, nlabels * sizeof(XtResource));
+    memcpy(&combined[nlabels],
+           cache_resources,
+	   XtNumber(cache_resources) * sizeof(XtResource));
     XtFree((char *)labels);
 
     xmIconGCacheObjClassRec.object_class.resources = combined;
@@ -575,7 +578,7 @@ static void
 initialize_prehook(Widget request, Widget new_w,
 		   ArgList args, Cardinal *num_args)
 {
-    DEBUGOUT(XdbDebug(__FILE__, new_w, "IconG InitializePrehook\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, new_w, "IconG InitializePrehook\n"));
 }
 
 static void
@@ -584,7 +587,7 @@ initialize_posthook(Widget request, Widget new_w,
 {
     XmWidgetExtData ext;
 
-    DEBUGOUT(XdbDebug(__FILE__, new_w, "IconG InitializePosthook\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, new_w, "IconG InitializePosthook\n"));
 
     /* don't let the null fool you */
 #if 0
@@ -610,7 +613,7 @@ _XmCalcIconGDimensions(Widget w)
 Dimension TextHeight, TextWidth;
 unsigned int IconHeight, IconWidth;
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "_XmCalcIconGDimensions\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "_XmCalcIconGDimensions\n"));
     if (_XmStringIsXmString((XmString)IG_LabelString(w)))
     {
 	IG_LabelString(w) = (XmString)_XmStringCreate((XmString)IG_LabelString(w));
@@ -620,7 +623,7 @@ unsigned int IconHeight, IconWidth;
 		    (_XmString) IG_LabelString(w),
 		    &TextWidth,
 		    &TextHeight);
-    DEBUGOUT(XdbDebug(__FILE__, w, "\t Text width %d height %d\n",
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "\t Text width %d height %d\n",
     		TextWidth, TextHeight));
 
     {
@@ -636,7 +639,7 @@ unsigned int IconHeight, IconWidth;
 		 &IconWidth, &IconHeight,
 		 &tmp, &Depth);
     }
-    DEBUGOUT(XdbDebug(__FILE__, w, "\t Icon width %d height %d\n",
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "\t Icon width %d height %d\n",
     		IconWidth, IconHeight));
 
     XtWidth(w) = TextWidth + IconWidth
@@ -644,7 +647,7 @@ unsigned int IconHeight, IconWidth;
 	    + G_HighlightThickness(w));
     XtHeight(w) = TextHeight > IconHeight ? TextHeight : IconHeight
 	    + 2 * G_HighlightThickness(w);
-    DEBUGOUT(XdbDebug(__FILE__, w, "\t width %d height %d\n",
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "\t width %d height %d\n",
     		XtWidth(w), XtHeight(w)));
 }
 
@@ -652,7 +655,7 @@ static void
 initialize(Widget request, Widget new_w,
 	   ArgList args, Cardinal *num_args)
 {
-    DEBUGOUT(XdbDebug(__FILE__, new_w,
+    DEBUGOUT(_LtDebugDebug(__FILE__, new_w,
 		      "initialize: %i args\n"
 		      "\trequest X %5i Y %5i W %5i H %5i\n"
 		      "\t  new_w X %5i Y %5i W %5i H %5i\n",
@@ -661,7 +664,7 @@ initialize(Widget request, Widget new_w,
 		      XtWidth(request), XtHeight(request),
 		      XtX(new_w), XtY(new_w),
 		      XtWidth(new_w), XtHeight(new_w)));
-    DEBUGOUT(XdbPrintArgList(__FILE__, new_w, args, *num_args, False));
+    DEBUGOUT(_LtDebugPrintArgList(__FILE__, new_w, args, *num_args, False));
 
     /* get the default fontlist if the label was created without one. */
     /* this should come from the renderTable (whatever that is??) FIXME */
@@ -862,7 +865,7 @@ destroy(Widget w)
 static void
 resize(Widget w)
 {
-    DEBUGOUT(XdbDebug(__FILE__, w,"resize\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w,"resize\n"));
 
     if (!XmIsLabelGadget(w))
     {
@@ -897,13 +900,13 @@ set_values_prehook(Widget old, Widget request, Widget new_w,
     ExtObj_ExtensionType(nsec) = XmCACHE_EXTENSION;
 
 #if 0
-    bcopy(LabG_Cache(new_w),
-	  &((XmLabelGCacheObject)nsec)->label_cache,
-	  sizeof(XmLabelGCacheObjPart));
+    memcpy(&((XmLabelGCacheObject)nsec)->label_cache,
+           LabG_Cache(new_w),
+	   sizeof(XmLabelGCacheObjPart));
 #endif
-    bcopy(IG_Cache(new_w),
-	  &((XmIconGCacheObject)nsec)->icon_cache,
-	  sizeof(XmIconGCacheObjPart));
+    memcpy(&((XmIconGCacheObject)nsec)->icon_cache,
+           IG_Cache(new_w),
+	   sizeof(XmIconGCacheObjPart));
 
     ed = (XmWidgetExtData)XtMalloc(sizeof(XmWidgetExtDataRec));
     ed->widget = (Widget)nsec;
@@ -918,7 +921,7 @@ set_values_prehook(Widget old, Widget request, Widget new_w,
 		   (*bce)->secondaryObjectClass->core_class.num_resources,
 		   args, *num_args);
 
-    bcopy(nsec, rsec, size);
+    memcpy(rsec, nsec, size);
 
 #if 0
     LabG_Cache(new_w) = &(((XmIconGCacheObject)nsec)->label_cache);
@@ -989,7 +992,7 @@ set_values(Widget old, Widget request, Widget new_w,
 {
     Boolean refresh_needed = False;
 
-    DEBUGOUT(XdbDebug(__FILE__, new_w,
+    DEBUGOUT(_LtDebugDebug(__FILE__, new_w,
 		      "set_values: %i args\n"
 		      "\t    old X %5i Y %5i W %5i H %5i\n"
 		      "\trequest X %5i Y %5i W %5i H %5i\n"
@@ -1001,7 +1004,7 @@ set_values(Widget old, Widget request, Widget new_w,
 		      XtWidth(request), XtHeight(request),
 		      XtX(new_w), XtY(new_w),
 		      XtWidth(new_w), XtHeight(new_w)));
-    DEBUGOUT(XdbPrintArgList(__FILE__, new_w, args, *num_args, False));
+    DEBUGOUT(_LtDebugPrintArgList(__FILE__, new_w, args, *num_args, False));
 
 #if 0
     if (IG_ArmColor(new_w) != IG_ArmColor(old))
@@ -1175,13 +1178,13 @@ get_values_prehook(Widget new_w, ArgList args, Cardinal *num_args)
     nsec = _XmExtObjAlloc(size);
 
 #if 0
-    bcopy(LabG_Cache(new_w),
-	  &((XmLabelGCacheObject)nsec)->label_cache,
-	  sizeof(XmLabelGCacheObjPart));
+    memcpy(&((XmLabelGCacheObject)nsec)->label_cache,
+           LabG_Cache(new_w),
+	   sizeof(XmLabelGCacheObjPart));
 #endif
-    bcopy(IG_Cache(new_w),
-	  &((XmIconGCacheObject)nsec)->icon_cache,
-	  sizeof(XmIconGCacheObjPart));
+    memcpy(&((XmIconGCacheObject)nsec)->icon_cache,
+           IG_Cache(new_w),
+	   sizeof(XmIconGCacheObjPart));
 
     /*
      * don't do this and ResInd will blow up.
@@ -1225,7 +1228,7 @@ get_values_posthook(Widget new_w, ArgList args, Cardinal *num_args)
 static void
 get_values_hook(Widget w, ArgList args, Cardinal *num_args)
 {
-    DEBUGOUT(XdbDebug(__FILE__, w, "GetValuesHook\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "GetValuesHook\n"));
 }
 
 static void
@@ -1238,7 +1241,7 @@ expose(Widget w, XEvent *event, Region region)
     XRectangle cliprect;
     GC myGC;
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "expose\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "expose\n"));
     /*
      * I'm getting paranoid - Danny - see testXm/filesb/test3
      */
@@ -1264,7 +1267,7 @@ expose(Widget w, XEvent *event, Region region)
 	2 * (/*G_ShadowThickness(w) +*/ G_HighlightThickness(w));
 
     XSetClipRectangles(XtDisplay(w), myGC, 0, 0, &cliprect, 1, Unsorted);
-    DEBUGOUT(XdbDebug(__FILE__, w, "cliprect x %d y %d width %d height %d\n",
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "cliprect x %d y %d width %d height %d\n",
     		cliprect.x, cliprect.y,
     		cliprect.width, cliprect.height));
 
@@ -1341,8 +1344,8 @@ query_geometry(Widget w, XtWidgetGeometry *proposed, XtWidgetGeometry *answer)
 
 #define	Wants(x)	(proposed->request_mode & x)
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "query_geometry %s\n",
-		      XdbWidgetGeometry2String(proposed)));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "query_geometry %s\n",
+		      _LtDebugWidgetGeometry2String(proposed)));
 
     wd = XtWidth(w);
     ht = XtHeight(w);
@@ -1361,7 +1364,7 @@ query_geometry(Widget w, XtWidgetGeometry *proposed, XtWidgetGeometry *answer)
     a.width = XtWidth(w);
     a.height = XtHeight(w);
     a.request_mode = CWWidth | CWHeight;
-    DEBUGOUT(XdbDebug(__FILE__, w,
+    DEBUGOUT(_LtDebugDebug(__FILE__, w,
 		      "IconGadget queried for size: reporting %d %d %08x\n",
 		      XtWidth(w), XtHeight(w), answer));
 
@@ -1457,7 +1460,7 @@ Activate(Widget w, XEvent *event, String *params, Cardinal *num_params)
     XButtonEvent *ev = (XButtonEvent *)event;
     XtExposeProc exp = XtClass(w)->core_class.expose;
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "Activate\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "Activate\n"));
 
     if (IG_Armed(w) == False)
     {
@@ -1504,7 +1507,7 @@ ArmTimeout(XtPointer data, XtIntervalId *id)
     Widget w = (Widget)data;
     XtExposeProc exp = XtClass(w)->core_class.expose;
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "Activate\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "Activate\n"));
 
     if (IG_Armed(w) == False)
     {
@@ -1534,7 +1537,7 @@ ArmTimeout(XtPointer data, XtIntervalId *id)
 static void
 ArmAndActivate(Widget w, XEvent *event, String *params, Cardinal *num_params)
 {
-    DEBUGOUT(XdbDebug(__FILE__, w, "ArmAndActivate\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "ArmAndActivate\n"));
 
     Arm(w, event, params, num_params);
 
@@ -1553,7 +1556,7 @@ EnterWindow(Widget w,
     XtExposeProc exp = XtClass(w)->core_class.expose;
     */
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "EnterWindow()\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "EnterWindow()\n"));
 
 #if 0
     if (!IN_MENU(w))
@@ -1667,7 +1670,7 @@ ButtonUp(Widget w, XEvent *event, String *params, Cardinal *num_params)
     Boolean validButton, poppedUp;
     XmPushButtonCallbackStruct cbs;
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "ButtonUp()\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "ButtonUp()\n"));
 
     /* queue events until the next button event */
     XAllowEvents(XtDisplay(w), SyncPointer, CurrentTime);
@@ -1850,14 +1853,14 @@ Help(Widget w, XEvent *event, String *params, Cardinal *num_params)
 static void
 input_dispatch(Widget gadget, XEvent *event, Mask event_mask)
 {
-    DEBUGOUT(XdbDebug(__FILE__, gadget, "input_dispatch\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, gadget, "input_dispatch\n"));
 #if 0
     Cardinal num_params = 0;
 
     switch (event_mask)
     {
     case XmARM_EVENT:
-	DEBUGOUT(XdbDebug(__FILE__, gadget, "got arm event\n"));
+	DEBUGOUT(_LtDebugDebug(__FILE__, gadget, "got arm event\n"));
 #if 0
 	if (IN_MENU(gadget))
 	{
@@ -1887,7 +1890,7 @@ input_dispatch(Widget gadget, XEvent *event, Mask event_mask)
 	break;
 
     case XmACTIVATE_EVENT:
-	DEBUGOUT(XdbDebug(__FILE__, gadget, "got activate event\n"));
+	DEBUGOUT(_LtDebugDebug(__FILE__, gadget, "got activate event\n"));
 	/*
 	 * The test below is expanded so it also checks for the Shell.
 	 * In tearoff menus, the shell will be a TransientShell, thus we'll
@@ -1922,17 +1925,17 @@ input_dispatch(Widget gadget, XEvent *event, Mask event_mask)
 	break;
 
     case XmENTER_EVENT:
-	DEBUGOUT(XdbDebug(__FILE__, gadget, "got an enter window\n"));
+	DEBUGOUT(_LtDebugDebug(__FILE__, gadget, "got an enter window\n"));
 	EnterWindow(gadget, event, NULL, &num_params);
 	break;
 
     case XmLEAVE_EVENT:
-	DEBUGOUT(XdbDebug(__FILE__, gadget, "got a leave window\n"));
+	DEBUGOUT(_LtDebugDebug(__FILE__, gadget, "got a leave window\n"));
 	LeaveWindow(gadget, event, NULL, &num_params);
 	break;
 
     case XmMOTION_EVENT:
-	DEBUGOUT(XdbDebug(__FILE__, gadget, "got a motion event\n"));
+	DEBUGOUT(_LtDebugDebug(__FILE__, gadget, "got a motion event\n"));
 	break;
 
     case XmFOCUS_IN_EVENT:
