@@ -1,8 +1,9 @@
 /**
  *
- * $Header: /cvsroot/hungry/lesstif/lib/Xm-2.0/PanedW.c,v 1.10 1999/11/19 13:13:16 amai Exp $
+ * $Header: /cvsroot/lesstif/lesstif/lib/Xm-2.0/PanedW.c,v 1.19 2001/05/14 19:46:11 amai Exp $
  *
  * Copyright (C) 1995-1998 Free Software Foundation, Inc.
+ * Copyright (C) 1998-2001 LessTif Development Team
  *
  * This file is part of the GNU LessTif Library.
  *
@@ -22,9 +23,14 @@
  *
  **/
 
-static const char rcsid[] = "$Id: PanedW.c,v 1.10 1999/11/19 13:13:16 amai Exp $";
+static const char rcsid[] = "$Header: /cvsroot/lesstif/lesstif/lib/Xm-2.0/PanedW.c,v 1.19 2001/05/14 19:46:11 amai Exp $";
 
 #include <LTconfig.h>
+
+#include <stdlib.h>
+#include <stdio.h>
+#include <string.h>
+
 #include <XmI/XmI.h>
 
 #include <Xm/XmP.h>
@@ -32,9 +38,6 @@ static const char rcsid[] = "$Id: PanedW.c,v 1.10 1999/11/19 13:13:16 amai Exp $
 #include <Xm/PanedWP.h>
 #include <Xm/SashP.h>
 #include <Xm/SeparatorP.h>
-#include <X11/Xfuncs.h>
-#include <stdlib.h>
-#include <stdio.h>
 
 #include <XmI/DebugUtil.h>
 
@@ -416,7 +419,6 @@ static XmSyntheticResource constraint_syn_resources[] =
 
 static void SashAction(Widget w, XtPointer client_data, XtPointer call_data);
 
-/* *INDENT-OFF* */
 static CompositeClassExtensionRec panedWCompositeExt = {
     /* next_extension */  NULL,
     /* record_type    */  NULLQUARK,
@@ -531,7 +533,6 @@ XmPanedWindowClassRec xmPanedWindowClassRec = {
 	/* extension */ NULL,
     },
 };
-/* *INDENT-ON* */
 
 
 WidgetClass xmPanedWindowWidgetClass = (WidgetClass)&xmPanedWindowClassRec;
@@ -579,6 +580,7 @@ ValidatePaneMin(Widget w)
     }
 }
 
+
 static void
 ValidatePaneMax(Widget w)
 {
@@ -605,6 +607,7 @@ ValidatePaneMax(Widget w)
     }
 }
 
+
 static Boolean
 AllPanesRealized(Widget w)
 {
@@ -628,11 +631,12 @@ AllPanesRealized(Widget w)
 
 
 static void
-class_initialize()
+class_initialize(void)
 {
     _XmPanedWCoreClassExtRec.record_type = XmQmotif;
 }
 
+
 static void
 class_part_initialize(WidgetClass widget_class)
 {
@@ -661,6 +665,7 @@ class_part_initialize(WidgetClass widget_class)
     _XmFastSubclassInit(widget_class, XmPANED_WINDOW_BIT);
 }
 
+
 static void
 CreateFlipGC(Widget w)
 {
@@ -676,6 +681,7 @@ CreateFlipGC(Widget w)
     PW_FlipGC(w) = XtGetGC(w, careabout, &gcv);
 }
 
+
 static void
 initialize(Widget request, Widget new_w,
 	   ArgList args, Cardinal *num_args)
@@ -695,11 +701,12 @@ initialize(Widget request, Widget new_w,
 						     PW_NumSlots(new_w));
 }
 
+
 static void
 constraint_initialize(Widget request, Widget new_w,
 		      ArgList args, Cardinal *num_args)
 {
-    DEBUGOUT(XdbDebug(__FILE__, new_w, "PanedWindow constraint initialize\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, new_w, "PanedWindow constraint initialize\n"));
 
     ValidatePaneMin(new_w);
     ValidatePaneMax(new_w);
@@ -723,9 +730,14 @@ constraint_initialize(Widget request, Widget new_w,
 static void
 destroy(Widget w)
 {
+    if (PW_ManagedChildren(w))
+    {
+	XtFree((String)PW_ManagedChildren(w));
+    }
     XtReleaseGC(w, PW_FlipGC(w));
 }
 
+
 /*
  * FIX ME We don't call a relayout function anywhere yet from set_values.
  */
@@ -737,7 +749,7 @@ set_values(Widget old, Widget request, Widget new_w,
     Boolean redisplay = False;
     Arg argl[3];
 
-    DEBUGOUT(XdbDebug(__FILE__, new_w, "PanedWindow set_valuesn"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, new_w, "PanedWindow set_valuesn"));
 
     if (PW_SeparatorOn(new_w) != PW_SeparatorOn(old))
     {
@@ -760,7 +772,7 @@ set_values(Widget old, Widget request, Widget new_w,
 	XtWidgetGeometry	geo;
 	Arg			arg;
 
-	DEBUGOUT(XdbDebug(__FILE__, new_w, "SetValues: change orientation\n"));
+	DEBUGOUT(_LtDebugDebug(__FILE__, new_w, "SetValues: change orientation\n"));
 
 	/* Tell separators to change orientation too */
 	XtSetArg(arg, XmNorientation,
@@ -841,6 +853,7 @@ set_values(Widget old, Widget request, Widget new_w,
     return redisplay;
 }
 
+
 static void
 expose(Widget w, XEvent *event, Region region)
 {
@@ -866,6 +879,7 @@ expose(Widget w, XEvent *event, Region region)
     _XmRedisplayGadgets(w, event, region);
 }
 
+
 static Boolean
 constraint_set_values(Widget current, Widget request, Widget new_w,
 		      ArgList args, Cardinal *num_args)
@@ -874,7 +888,7 @@ constraint_set_values(Widget current, Widget request, Widget new_w,
     Boolean redisplay = False;
     int i;
 
-    DEBUGOUT(XdbDebug(__FILE__, new_w, "PanedWindow constraint_set_values\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, new_w, "PanedWindow constraint_set_values\n"));
 
     if (!PWC_IsPane(current))
     {
@@ -947,17 +961,18 @@ constraint_set_values(Widget current, Widget request, Widget new_w,
 	}
     }
 
-    DEBUGOUT(XdbDebug(__FILE__, pw,
+    DEBUGOUT(_LtDebugDebug(__FILE__, pw,
 		      "constraint_set_values: after PreferredSize\n"));
     return redisplay;
 }
 
+
 static void
 resize(Widget w)
 {
     XtWidgetGeometry geo;
 
-    DEBUGOUT(XdbDebug(__FILE__, w,
+    DEBUGOUT(_LtDebugDebug(__FILE__, w,
 		      "PanedWindow Resize (%d %d)\n",
 		      XtWidth(w), XtHeight(w)));
 
@@ -968,6 +983,7 @@ resize(Widget w)
     _XmPanedWLayout(w, NULL, NULL, &geo, NULL, NULL, NULL);
 }
 
+
 static void
 realize(Widget w,
 	XtValueMask *value_mask,
@@ -979,7 +995,7 @@ realize(Widget w,
     (*superclass->core_class.realize) (w, value_mask, attributes);
 #undef superclass
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "PanedWindow Realize\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "PanedWindow Realize\n"));
 
     _XmPanedWPreferredSize(w, NULL, NULL, &geo);
 
@@ -1020,6 +1036,7 @@ panedw_insert_position(Widget w)
     return PWC_PositionIndex(w);
 }
 
+
 static void
 insert_child(Widget w)
 {
@@ -1063,6 +1080,7 @@ insert_child(Widget w)
 		XmNseparatorType,	XmSHADOW_ETCHED_IN,
 		XmNorientation,		(PW_Orientation(pw) == XmVERTICAL)
 						? XmHORIZONTAL : XmVERTICAL,
+		XmNnavigationType,	XmNONE,
 					NULL);
 
     PWC_Separator(PWC_Separator(w)) = w;
@@ -1084,8 +1102,11 @@ insert_child(Widget w)
 		  pw);
 
     PW_RecursivelyCalled(pw) = False;
+
+    XtVaSetValues(w, XmNnavigationType, XmTAB_GROUP, NULL);
 }
 
+
 static void
 delete_child(Widget w)
 {
@@ -1101,12 +1122,13 @@ delete_child(Widget w)
 #undef superclass
 }
 
+
 static XtGeometryResult
 query_geometry(Widget w, XtWidgetGeometry *proposed, XtWidgetGeometry *desired)
 {
-    DEBUGOUT(XdbDebug(__FILE__, w,
+    DEBUGOUT(_LtDebugDebug(__FILE__, w,
 		      "QueryGeometry (request %s)\n",
-		      XdbWidgetGeometry2String(proposed)));
+		      _LtDebugWidgetGeometry2String(proposed)));
 
     _XmPanedWPreferredSize(w, NULL, NULL, desired);
 
@@ -1134,6 +1156,7 @@ query_geometry(Widget w, XtWidgetGeometry *proposed, XtWidgetGeometry *desired)
     return XtGeometryAlmost;
 }
 
+
 static XtGeometryResult
 geometry_manager(Widget w,
 		 XtWidgetGeometry *request,
@@ -1144,17 +1167,17 @@ geometry_manager(Widget w,
     Widget pw = XtParent(w);
     XmKidGeometry boxes = NULL, sashes = NULL, seps = NULL;
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "PanedW: geometry manager\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "PanedW: geometry manager\n"));
 
     if (AllPanesRealized(pw))
     {
-	DEBUGOUT(XdbDebug(__FILE__, w,
+	DEBUGOUT(_LtDebugDebug(__FILE__, w,
 			  "All panes realized in geom_manager\n"));
     }
 
     if (AllPanesRealized(pw) && !PWC_AllowResize(w) && XtIsManaged(pw))
     {
-	DEBUGOUT(XdbDebug(__FILE__, w,
+	DEBUGOUT(_LtDebugDebug(__FILE__, w,
 			  "Not allowing resize from geom_manager\n"));
 
 	return XtGeometryNo;
@@ -1169,7 +1192,7 @@ geometry_manager(Widget w,
 
     want = *request;
 
-    DEBUGOUT(XdbDebug(__FILE__, w,
+    DEBUGOUT(_LtDebugDebug(__FILE__, w,
 		      "geom_mgr: child %s wants %d %d\n", XtName(w),
 		      request->width, request->height));
 
@@ -1177,7 +1200,7 @@ geometry_manager(Widget w,
 
     if (_XmMakeGeometryRequest(pw, &geo) == XtGeometryYes)
     {
-	DEBUGOUT(XdbDebug(__FILE__, pw,
+	DEBUGOUT(_LtDebugDebug(__FILE__, pw,
 			  "We wanted %d %d and we got it\n",
 			  geo.width, geo.height));
     }
@@ -1185,7 +1208,7 @@ geometry_manager(Widget w,
     {
 	geo.width = XtWidth(pw);
 	geo.height = XtHeight(pw);
-	DEBUGOUT(XdbDebug(__FILE__, pw,
+	DEBUGOUT(_LtDebugDebug(__FILE__, pw,
 			  "We didn't get our request, but %d %d\n",
 			  geo.width, geo.height));
     }
@@ -1198,6 +1221,7 @@ geometry_manager(Widget w,
 	if ((want.request_mode & CWWidth) && want.width == request->width)
 	{
 	    good++;
+	    want.request_mode &= ~CWWidth;
 	}
     }
     if (Wants(CWHeight))
@@ -1206,6 +1230,7 @@ geometry_manager(Widget w,
 	if ((want.request_mode & CWHeight) && want.height == request->height)
 	{
 	    good++;
+	    want.request_mode &= ~CWHeight;
 	}
     }
 
@@ -1228,7 +1253,7 @@ geometry_manager(Widget w,
 	    }
 	}
 
-	DEBUGOUT(XdbDebug(__FILE__, w,
+	DEBUGOUT(_LtDebugDebug(__FILE__, w,
 		"Child wanted %d and that's ok\n",
 		(PW_Orientation(w) == XmVERTICAL) ? want.height : want.width));
 
@@ -1249,14 +1274,14 @@ geometry_manager(Widget w,
 
     if (good == 0)
     {
-	DEBUGOUT(XdbDebug(__FILE__, w,
+	DEBUGOUT(_LtDebugDebug(__FILE__, w,
 		"Child wanted %d (NO)\n",
 		(PW_Orientation(w) == XmVERTICAL) ? want.height : want.width));
 	return XtGeometryNo;
     }
     else
     {
-	DEBUGOUT(XdbDebug(__FILE__, w,
+	DEBUGOUT(_LtDebugDebug(__FILE__, w,
 		"Child wanted %d (ALMOST)\n",
 		(PW_Orientation(w) == XmVERTICAL) ? want.height : want.width));
 	return XtGeometryAlmost;
@@ -1264,6 +1289,7 @@ geometry_manager(Widget w,
 #undef Wants
 }
 
+
 static int
 pi_sort(const void *a, const void *b)
 {
@@ -1273,17 +1299,18 @@ pi_sort(const void *a, const void *b)
     return PWC_PositionIndex(wa) - PWC_PositionIndex(wb);
 }
 
+
 static void
 change_managed(Widget w)
 {
     int i, j;
     XtWidgetGeometry geo;
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "PanedWindow change_managed\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "PanedWindow change_managed\n"));
 
     if (PW_RecursivelyCalled(w))
     {
-	DEBUGOUT(XdbDebug(__FILE__, w,
+	DEBUGOUT(_LtDebugDebug(__FILE__, w,
 			  "ChangeManaged was recursively called\n"));
 	return;
     }
@@ -1310,7 +1337,7 @@ change_managed(Widget w)
 
 	    if ((PW_NumManagedChildren(w) - i) > 0)
 	    {
-		bcopy(&PW_ManagedChildren(w)[i + 1], &PW_ManagedChildren(w)[i],
+		memcpy(&PW_ManagedChildren(w)[i], &PW_ManagedChildren(w)[i + 1], 
 		      sizeof(Widget) * (PW_NumManagedChildren(w) - i));
 	    }
 	}
@@ -1345,7 +1372,7 @@ change_managed(Widget w)
 		PW_PaneCount(w)++;
 
 		if (PW_Orientation(w) == XmVERTICAL) {
-		    DEBUGOUT(XdbDebug(__FILE__, MGR_Children(w)[i],
+		    DEBUGOUT(_LtDebugDebug(__FILE__, MGR_Children(w)[i],
 			"Setting DHeight in change_managed to %d:"
 			" was %d\n",
 			XtHeight(MGR_Children(w)[i]),
@@ -1354,7 +1381,7 @@ change_managed(Widget w)
 		    PWC_DHeight(MGR_Children(w)[i]) =
 			XtHeight(MGR_Children(w)[i]);
 		} else {
-		    DEBUGOUT(XdbDebug(__FILE__, MGR_Children(w)[i],
+		    DEBUGOUT(_LtDebugDebug(__FILE__, MGR_Children(w)[i],
 			"Setting DWidth in change_managed to %d:"
 			" was %d\n",
 			XtWidth(MGR_Children(w)[i]),
@@ -1368,7 +1395,7 @@ change_managed(Widget w)
 		{
 		    PW_NumSlots(w) *= 2;
 		    PW_ManagedChildren(w) =
-			(WidgetList)XtRealloc((void *)PW_ManagedChildren(w),
+			(WidgetList)XtRealloc((char *)PW_ManagedChildren(w),
 					      sizeof(Widget) * PW_NumSlots(w));
 		}
 
@@ -1442,7 +1469,7 @@ change_managed(Widget w)
 	    CoreConstraints(PW_ManagedChildren(w)[PW_NumManagedChildren(w) - 1]);
     }
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "Compute size in change_managed\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "Compute size in change_managed\n"));
 
     _XmPanedWPreferredSize(w, NULL, NULL, &geo);
 
@@ -1460,6 +1487,7 @@ change_managed(Widget w)
     _XmNavigChangeManaged(w);
 }
 
+
 static Boolean
 traversal_children(Widget mw, Widget **children, Cardinal *num_children)
 {
@@ -1495,6 +1523,7 @@ traversal_children(Widget mw, Widget **children, Cardinal *num_children)
     return True;
 }
 
+
 /*
  * Preferred Size calculation in the XmVERTICAL case.
  *
@@ -1509,7 +1538,7 @@ _XmPanedWVPreferredSize(Widget w, Widget rchild,
     int i;
     Dimension curw, curh;
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "_XmPanedWVPreferredSize\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "_XmPanedWVPreferredSize\n"));
 
     /* Find widest child and compute total height of PanedWindow */
     curw = curh = 0;
@@ -1518,7 +1547,7 @@ _XmPanedWVPreferredSize(Widget w, Widget rchild,
 
 	child = PW_ManagedChildren(w)[i];
 
-	DEBUGOUT(XdbDebug2(__FILE__, w, child,
+	DEBUGOUT(_LtDebugDebug2(__FILE__, w, child,
 			   "_XmPanedWVPreferredSize: child wid %d ht %d "
 			   "pref.ht. %d panemin %d panemax %d\n",
 			   XtWidth(child), XtHeight(child),
@@ -1528,7 +1557,7 @@ _XmPanedWVPreferredSize(Widget w, Widget rchild,
 	if (rchild != NULL && child == rchild && childgeom != NULL &&
 	    childgeom->request_mode & CWWidth)
 	{
-	    DEBUGOUT(XdbDebug(__FILE__, w, "Using childgeom width: %d\n",
+	    DEBUGOUT(_LtDebugDebug(__FILE__, w, "Using childgeom width: %d\n",
 			      childgeom->width));
 	    curw = curw >= childgeom->width ? curw : childgeom->width;
 	}
@@ -1545,7 +1574,7 @@ _XmPanedWVPreferredSize(Widget w, Widget rchild,
 	if (rchild != NULL && child == rchild && childgeom != NULL &&
 	    childgeom->request_mode & CWHeight)
 	{
-	    DEBUGOUT(XdbDebug(__FILE__, w, "Using childgeom height: %d\n",
+	    DEBUGOUT(_LtDebugDebug(__FILE__, w, "Using childgeom height: %d\n",
 			      childgeom->height));
 	    curh += _XmMax(childgeom->height, PWC_PaneMinimum(child));
 	}
@@ -1558,7 +1587,7 @@ _XmPanedWVPreferredSize(Widget w, Widget rchild,
     curh += 2 * PW_MarginHeight(w);
     curw += 2 * PW_MarginWidth(w);
 
-    DEBUGOUT(XdbDebug(__FILE__, w,
+    DEBUGOUT(_LtDebugDebug(__FILE__, w,
 		      "_XmPanedWVPreferredSize: computed %d %d\n",
 		      curw, curh));
 
@@ -1567,6 +1596,7 @@ _XmPanedWVPreferredSize(Widget w, Widget rchild,
     pwg->request_mode = CWWidth | CWHeight;
 }
 
+
 /*
  * Preferred Size calculation in the XmHORIZONTAL case.
  *
@@ -1581,7 +1611,7 @@ _XmPanedWHPreferredSize(Widget w, Widget rchild,
     int i;
     Dimension curw, curh;
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "_XmPanedWPreferredSize\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "_XmPanedWPreferredSize\n"));
 
     /* Find widest child and compute total height of PanedWindow */
     curw = curh = 0;
@@ -1590,7 +1620,7 @@ _XmPanedWHPreferredSize(Widget w, Widget rchild,
 
 	child = PW_ManagedChildren(w)[i];
 
-	DEBUGOUT(XdbDebug2(__FILE__, w, child,
+	DEBUGOUT(_LtDebugDebug2(__FILE__, w, child,
 			   "_XmPanedWHPreferredSize: child wid %d ht %d "
 			   "pref.ht. %d panemin %d panemax %d\n",
 			   XtWidth(child), XtHeight(child),
@@ -1600,7 +1630,7 @@ _XmPanedWHPreferredSize(Widget w, Widget rchild,
 	if (rchild != NULL && child == rchild && childgeom != NULL &&
 	    childgeom->request_mode & CWHeight)
 	{
-	    DEBUGOUT(XdbDebug(__FILE__, w, "Using childgeom height: %d\n",
+	    DEBUGOUT(_LtDebugDebug(__FILE__, w, "Using childgeom height: %d\n",
 			      childgeom->height));
 	    curh = curh >= childgeom->height ? curh : childgeom->height;
 	}
@@ -1617,7 +1647,7 @@ _XmPanedWHPreferredSize(Widget w, Widget rchild,
 	if (rchild != NULL && child == rchild && childgeom != NULL &&
 	    childgeom->request_mode & CWWidth)
 	{
-	    DEBUGOUT(XdbDebug(__FILE__, w, "Using childgeom width: %d\n",
+	    DEBUGOUT(_LtDebugDebug(__FILE__, w, "Using childgeom width: %d\n",
 			      childgeom->width));
 	    curw += _XmMax(childgeom->width, PWC_PaneMinimum(child));
 	}
@@ -1630,7 +1660,7 @@ _XmPanedWHPreferredSize(Widget w, Widget rchild,
     curh += 2 * PW_MarginHeight(w);
     curw += 2 * PW_MarginWidth(w);
 
-    DEBUGOUT(XdbDebug(__FILE__, w,
+    DEBUGOUT(_LtDebugDebug(__FILE__, w,
 		      "_XmPanedWHPreferredSize: computed %d %d\n",
 		      curw, curh));
 
@@ -1639,6 +1669,7 @@ _XmPanedWHPreferredSize(Widget w, Widget rchild,
     pwg->request_mode = CWWidth | CWHeight;
 }
 
+
 static void
 _XmPanedWPreferredSize(Widget w, Widget rchild,
 		       XtWidgetGeometry *childgeom, XtWidgetGeometry *pwg)
@@ -1649,6 +1680,7 @@ _XmPanedWPreferredSize(Widget w, Widget rchild,
 	_XmPanedWHPreferredSize(w, rchild, childgeom, pwg);
 }
 
+
 /*
  * Layout in the XmHORIZONTAL case.
  *
@@ -1669,7 +1701,7 @@ _XmPanedWHLayout(Widget w, Widget rchild,
     XmKidGeometry boxes, sashes, seps;
     int total_delta ;
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "Pane Horizontal Layout\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "Pane Horizontal Layout\n"));
 
     curw = pwg->width;
     curh = pwg->height;
@@ -1872,6 +1904,7 @@ _XmPanedWHLayout(Widget w, Widget rchild,
     }
 }
 
+
 static void
 _XmPanedWVLayout(Widget w, Widget rchild,
 		XtWidgetGeometry *childgeom, XtWidgetGeometry *pwg,
@@ -1886,7 +1919,7 @@ _XmPanedWVLayout(Widget w, Widget rchild,
     XmKidGeometry boxes, sashes, seps;
     int total_delta ;
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "Pane Vertical Layout\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "Pane Vertical Layout\n"));
 
     curw = pwg->width;
     curh = pwg->height;
@@ -2093,6 +2126,7 @@ _XmPanedWVLayout(Widget w, Widget rchild,
     }
 }
 
+
 static void
 _XmPanedWLayout(Widget w, Widget rchild,
 		XtWidgetGeometry *childgeom, XtWidgetGeometry *pwg,
@@ -2105,6 +2139,7 @@ _XmPanedWLayout(Widget w, Widget rchild,
 	_XmPanedWHLayout(w, rchild, childgeom, pwg, boxptr, sashptr, sepptr);
 }
 
+
 static void
 MoveBorderV(Widget pw, Widget sash, int dy)
 {
@@ -2129,14 +2164,14 @@ MoveBorderV(Widget pw, Widget sash, int dy)
 	/* Move up? */
 	if (dy < 0)
 	{
-	    DEBUGOUT(XdbDebug(__FILE__, pw,
+	    DEBUGOUT(_LtDebugDebug(__FILE__, pw,
 			      "Move up: %d %d %d %d %d\n",
 			      PWC_DHeight(panea), dy, PWC_PaneMaximum(panea),
 			      PWC_DHeight(paneb), PWC_PaneMinimum(paneb)));
 
 	    if ((XtHeight(paneb) - dy) < PWC_PaneMaximum(paneb))
 	    {
-		DEBUGOUT(XdbDebug(__FILE__, paneb, "Widget can grow\n"));
+		DEBUGOUT(_LtDebugDebug(__FILE__, paneb, "Widget can grow\n"));
 
 		/*
 		 * can panea absorb the change?  If so, then adjust paneb's
@@ -2144,7 +2179,7 @@ MoveBorderV(Widget pw, Widget sash, int dy)
 		 */
 		if ((XtHeight(panea) + dy) > PWC_PaneMinimum(panea))
 		{
-		    DEBUGOUT(XdbDebug(__FILE__, panea,
+		    DEBUGOUT(_LtDebugDebug(__FILE__, panea,
 				      "Widget can shrink to compensate\n"));
 
 		    cons = PW_BottomPane(pw);
@@ -2172,7 +2207,7 @@ MoveBorderV(Widget pw, Widget sash, int dy)
 		 */
 		else if (XtHeight(panea) > PWC_PaneMinimum(panea))
 		{
-		    DEBUGOUT(XdbDebug(__FILE__, panea,
+		    DEBUGOUT(_LtDebugDebug(__FILE__, panea,
 				      "Widget can partially compensate\n"));
 
 		    dy += XtHeight(panea) - PWC_PaneMinimum(panea);
@@ -2200,7 +2235,7 @@ MoveBorderV(Widget pw, Widget sash, int dy)
 		    sash = PWC_Sash(PW_ManagedChildren(pw)[PWC_Position(panea) - 1]);
 		    panea = PWC_Sash(sash);
 
-		    DEBUGOUT(XdbDebug(__FILE__, paneb,
+		    DEBUGOUT(_LtDebugDebug(__FILE__, paneb,
 				      "And we'll try to pass it off\n"));
 
 		    PW_TopPane(pw) = (XmPanedWindowConstraintPtr)
@@ -2211,7 +2246,7 @@ MoveBorderV(Widget pw, Widget sash, int dy)
 		 */
 		else
 		{
-		    DEBUGOUT(XdbDebug(__FILE__, pw,
+		    DEBUGOUT(_LtDebugDebug(__FILE__, pw,
 				      "No more panes above\n"));
 		    return;
 		}
@@ -2228,7 +2263,7 @@ MoveBorderV(Widget pw, Widget sash, int dy)
 		    sash = PWC_Sash(paneb);
 		    paneb = PW_ManagedChildren(pw)[PWC_Position(PWC_Sash(sash)) + 1];
 
-		    DEBUGOUT(XdbDebug(__FILE__, paneb,
+		    DEBUGOUT(_LtDebugDebug(__FILE__, paneb,
 				      "And we'll try to pass it off\n"));
 
 		    PW_BottomPane(pw) = (XmPanedWindowConstraintPtr)
@@ -2236,7 +2271,7 @@ MoveBorderV(Widget pw, Widget sash, int dy)
 		}
 		else
 		{
-		    DEBUGOUT(XdbDebug(__FILE__, paneb, "Can't grow\n"));
+		    DEBUGOUT(_LtDebugDebug(__FILE__, paneb, "Can't grow\n"));
 		    return;
 		}
 	    }
@@ -2244,7 +2279,7 @@ MoveBorderV(Widget pw, Widget sash, int dy)
 	/* move down? */
 	else if (dy > 0)
 	{
-	    DEBUGOUT(XdbDebug(__FILE__, pw,
+	    DEBUGOUT(_LtDebugDebug(__FILE__, pw,
 			      "Move down: %d %d %d %d %d\n",
 			      PWC_DHeight(panea), dy, PWC_PaneMaximum(panea),
 			      PWC_DHeight(paneb), PWC_PaneMinimum(paneb)));
@@ -2252,7 +2287,7 @@ MoveBorderV(Widget pw, Widget sash, int dy)
 	    /* can panea grow? */
 	    if ((XtHeight(panea) + dy) < PWC_PaneMaximum(panea))
 	    {
-		DEBUGOUT(XdbDebug(__FILE__, panea, "Pane can grow\n"));
+		DEBUGOUT(_LtDebugDebug(__FILE__, panea, "Pane can grow\n"));
 
 		/*
 		 * can paneb absorb the change?  If so, increate panea's
@@ -2260,7 +2295,7 @@ MoveBorderV(Widget pw, Widget sash, int dy)
 		 */
 		if ((XtHeight(paneb) - dy) > PWC_PaneMinimum(paneb))
 		{
-		    DEBUGOUT(XdbDebug(__FILE__, paneb,
+		    DEBUGOUT(_LtDebugDebug(__FILE__, paneb,
 				  "And this pane can shrink to compensate\n"));
 
 		    XtHeight(paneb) -= dy;
@@ -2288,7 +2323,7 @@ MoveBorderV(Widget pw, Widget sash, int dy)
 		 */
 		else if (XtHeight(paneb) > PWC_PaneMinimum(paneb))
 		{
-		    DEBUGOUT(XdbDebug(__FILE__, paneb,
+		    DEBUGOUT(_LtDebugDebug(__FILE__, paneb,
 				  "And this pane can partially compensate\n"));
 
 		    dy -= XtHeight(paneb) - PWC_PaneMinimum(paneb);
@@ -2317,7 +2352,7 @@ MoveBorderV(Widget pw, Widget sash, int dy)
 		    sash = PWC_Sash(paneb);
 		    paneb = PW_ManagedChildren(pw)[PWC_Position(PWC_Sash(sash)) + 1];
 
-		    DEBUGOUT(XdbDebug(__FILE__, paneb,
+		    DEBUGOUT(_LtDebugDebug(__FILE__, paneb,
 				      "And we'll try to pass it off\n"));
 
 		    PW_BottomPane(pw) = (XmPanedWindowConstraintPtr)
@@ -2328,7 +2363,7 @@ MoveBorderV(Widget pw, Widget sash, int dy)
 		 */
 		else
 		{
-		    DEBUGOUT(XdbDebug(__FILE__, pw, "No more panes below\n"));
+		    DEBUGOUT(_LtDebugDebug(__FILE__, pw, "No more panes below\n"));
 		    return;
 		}
 	    }
@@ -2344,7 +2379,7 @@ MoveBorderV(Widget pw, Widget sash, int dy)
 		    sash = PWC_Sash(PW_ManagedChildren(pw)[PWC_Position(panea) - 1]);
 		    panea = PWC_Sash(sash);
 
-		    DEBUGOUT(XdbDebug(__FILE__, paneb,
+		    DEBUGOUT(_LtDebugDebug(__FILE__, paneb,
 				      "And we'll try to pass it off\n"));
 
 		    PW_TopPane(pw) = (XmPanedWindowConstraintPtr)
@@ -2352,7 +2387,7 @@ MoveBorderV(Widget pw, Widget sash, int dy)
 		}
 		else
 		{
-		    DEBUGOUT(XdbDebug(__FILE__, panea, "Pane can't grow\n"));
+		    DEBUGOUT(_LtDebugDebug(__FILE__, panea, "Pane can't grow\n"));
 
 		    return;
 		}
@@ -2361,13 +2396,14 @@ MoveBorderV(Widget pw, Widget sash, int dy)
 	/* no motion, bail out */
 	else
 	{
-	    DEBUGOUT(XdbDebug(__FILE__, pw, "No motion\n"));
+	    DEBUGOUT(_LtDebugDebug(__FILE__, pw, "No motion\n"));
 
 	    return;
 	}
     }
 }
 
+
 static void
 MoveBorderH(Widget pw, Widget sash, int dx)
 {
@@ -2392,14 +2428,14 @@ MoveBorderH(Widget pw, Widget sash, int dx)
 	/* Move up? */
 	if (dx < 0)
 	{
-	    DEBUGOUT(XdbDebug(__FILE__, pw,
+	    DEBUGOUT(_LtDebugDebug(__FILE__, pw,
 			      "Move up: %d %d %d %d %d\n",
 			      PWC_DWidth(panea), dx, PWC_PaneMaximum(panea),
 			      PWC_DWidth(paneb), PWC_PaneMinimum(paneb)));
 
 	    if ((XtWidth(paneb) - dx) < PWC_PaneMaximum(paneb))
 	    {
-		DEBUGOUT(XdbDebug(__FILE__, paneb, "Widget can grow\n"));
+		DEBUGOUT(_LtDebugDebug(__FILE__, paneb, "Widget can grow\n"));
 
 		/*
 		 * can panea absorb the change?  If so, then adjust paneb's
@@ -2407,7 +2443,7 @@ MoveBorderH(Widget pw, Widget sash, int dx)
 		 */
 		if ((XtWidth(panea) + dx) > PWC_PaneMinimum(panea))
 		{
-		    DEBUGOUT(XdbDebug(__FILE__, panea,
+		    DEBUGOUT(_LtDebugDebug(__FILE__, panea,
 				      "Widget can shrink to compensate\n"));
 
 		    cons = PW_BottomPane(pw);
@@ -2435,7 +2471,7 @@ MoveBorderH(Widget pw, Widget sash, int dx)
 		 */
 		else if (XtWidth(panea) > PWC_PaneMinimum(panea))
 		{
-		    DEBUGOUT(XdbDebug(__FILE__, panea,
+		    DEBUGOUT(_LtDebugDebug(__FILE__, panea,
 				      "Widget can partially compensate\n"));
 
 		    dx += XtWidth(panea) - PWC_PaneMinimum(panea);
@@ -2463,7 +2499,7 @@ MoveBorderH(Widget pw, Widget sash, int dx)
 		    sash = PWC_Sash(PW_ManagedChildren(pw)[PWC_Position(panea) - 1]);
 		    panea = PWC_Sash(sash);
 
-		    DEBUGOUT(XdbDebug(__FILE__, paneb,
+		    DEBUGOUT(_LtDebugDebug(__FILE__, paneb,
 				      "And we'll try to pass it off\n"));
 
 		    PW_TopPane(pw) = (XmPanedWindowConstraintPtr)
@@ -2474,7 +2510,7 @@ MoveBorderH(Widget pw, Widget sash, int dx)
 		 */
 		else
 		{
-		    DEBUGOUT(XdbDebug(__FILE__, pw,
+		    DEBUGOUT(_LtDebugDebug(__FILE__, pw,
 				      "No more panes above\n"));
 		    return;
 		}
@@ -2491,7 +2527,7 @@ MoveBorderH(Widget pw, Widget sash, int dx)
 		    sash = PWC_Sash(paneb);
 		    paneb = PW_ManagedChildren(pw)[PWC_Position(PWC_Sash(sash)) + 1];
 
-		    DEBUGOUT(XdbDebug(__FILE__, paneb,
+		    DEBUGOUT(_LtDebugDebug(__FILE__, paneb,
 				      "And we'll try to pass it off\n"));
 
 		    PW_BottomPane(pw) = (XmPanedWindowConstraintPtr)
@@ -2499,7 +2535,7 @@ MoveBorderH(Widget pw, Widget sash, int dx)
 		}
 		else
 		{
-		    DEBUGOUT(XdbDebug(__FILE__, paneb, "Can't grow\n"));
+		    DEBUGOUT(_LtDebugDebug(__FILE__, paneb, "Can't grow\n"));
 		    return;
 		}
 	    }
@@ -2507,7 +2543,7 @@ MoveBorderH(Widget pw, Widget sash, int dx)
 	/* move down? */
 	else if (dx > 0)
 	{
-	    DEBUGOUT(XdbDebug(__FILE__, pw,
+	    DEBUGOUT(_LtDebugDebug(__FILE__, pw,
 			      "Move down: %d %d %d %d %d\n",
 			      PWC_DWidth(panea), dx, PWC_PaneMaximum(panea),
 			      PWC_DWidth(paneb), PWC_PaneMinimum(paneb)));
@@ -2515,7 +2551,7 @@ MoveBorderH(Widget pw, Widget sash, int dx)
 	    /* can panea grow? */
 	    if ((XtWidth(panea) + dx) < PWC_PaneMaximum(panea))
 	    {
-		DEBUGOUT(XdbDebug(__FILE__, panea, "Pane can grow\n"));
+		DEBUGOUT(_LtDebugDebug(__FILE__, panea, "Pane can grow\n"));
 
 		/*
 		 * can paneb absorb the change?  If so, increate panea's
@@ -2523,7 +2559,7 @@ MoveBorderH(Widget pw, Widget sash, int dx)
 		 */
 		if ((XtWidth(paneb) - dx) > PWC_PaneMinimum(paneb))
 		{
-		    DEBUGOUT(XdbDebug(__FILE__, paneb,
+		    DEBUGOUT(_LtDebugDebug(__FILE__, paneb,
 				  "And this pane can shrink to compensate\n"));
 
 		    XtWidth(paneb) -= dx;
@@ -2551,7 +2587,7 @@ MoveBorderH(Widget pw, Widget sash, int dx)
 		 */
 		else if (XtWidth(paneb) > PWC_PaneMinimum(paneb))
 		{
-		    DEBUGOUT(XdbDebug(__FILE__, paneb,
+		    DEBUGOUT(_LtDebugDebug(__FILE__, paneb,
 				  "And this pane can partially compensate\n"));
 
 		    dx -= XtWidth(paneb) - PWC_PaneMinimum(paneb);
@@ -2580,7 +2616,7 @@ MoveBorderH(Widget pw, Widget sash, int dx)
 		    sash = PWC_Sash(paneb);
 		    paneb = PW_ManagedChildren(pw)[PWC_Position(PWC_Sash(sash)) + 1];
 
-		    DEBUGOUT(XdbDebug(__FILE__, paneb,
+		    DEBUGOUT(_LtDebugDebug(__FILE__, paneb,
 				      "And we'll try to pass it off\n"));
 
 		    PW_BottomPane(pw) = (XmPanedWindowConstraintPtr)
@@ -2591,7 +2627,7 @@ MoveBorderH(Widget pw, Widget sash, int dx)
 		 */
 		else
 		{
-		    DEBUGOUT(XdbDebug(__FILE__, pw, "No more panes below\n"));
+		    DEBUGOUT(_LtDebugDebug(__FILE__, pw, "No more panes below\n"));
 		    return;
 		}
 	    }
@@ -2607,7 +2643,7 @@ MoveBorderH(Widget pw, Widget sash, int dx)
 		    sash = PWC_Sash(PW_ManagedChildren(pw)[PWC_Position(panea) - 1]);
 		    panea = PWC_Sash(sash);
 
-		    DEBUGOUT(XdbDebug(__FILE__, paneb,
+		    DEBUGOUT(_LtDebugDebug(__FILE__, paneb,
 				      "And we'll try to pass it off\n"));
 
 		    PW_TopPane(pw) = (XmPanedWindowConstraintPtr)
@@ -2615,7 +2651,7 @@ MoveBorderH(Widget pw, Widget sash, int dx)
 		}
 		else
 		{
-		    DEBUGOUT(XdbDebug(__FILE__, panea, "Pane can't grow\n"));
+		    DEBUGOUT(_LtDebugDebug(__FILE__, panea, "Pane can't grow\n"));
 
 		    return;
 		}
@@ -2624,13 +2660,14 @@ MoveBorderH(Widget pw, Widget sash, int dx)
 	/* no motion, bail out */
 	else
 	{
-	    DEBUGOUT(XdbDebug(__FILE__, pw, "No motion\n"));
+	    DEBUGOUT(_LtDebugDebug(__FILE__, pw, "No motion\n"));
 
 	    return;
 	}
     }
 }
 
+
 static void
 SashAction(Widget w, XtPointer client_data, XtPointer call_data)
 {
@@ -2675,7 +2712,7 @@ SashAction(Widget w, XtPointer client_data, XtPointer call_data)
 		}
 	    }
 
-	    DEBUGOUT(XdbDebug(__FILE__, child,
+	    DEBUGOUT(_LtDebugDebug(__FILE__, child,
 			      "SashAction places child %dx%d, size %d/%d\n",
 			      XtX(child), PWC_DY(child),
 			      XtWidth(child), XtHeight(child)));
@@ -2735,7 +2772,7 @@ SashAction(Widget w, XtPointer client_data, XtPointer call_data)
 		sash = PWC_Sash(child);
 		if (sash)
 		{
-		    DEBUGOUT(XdbDebug(__FILE__, pw, "moving sash\n"));
+		    DEBUGOUT(_LtDebugDebug(__FILE__, pw, "moving sash\n"));
 
 		    if (PW_Orientation(pw) == XmVERTICAL) {
 			Position x;
@@ -2847,6 +2884,7 @@ SashAction(Widget w, XtPointer client_data, XtPointer call_data)
     }
 }
 
+
 Widget
 XmCreatePanedWindow(Widget parent, char *name,
 		    Arg *argList, Cardinal argcount)
