@@ -1,8 +1,9 @@
 /**
  *
- * $Header: /cvsroot/hungry/lesstif/lib/Xm-2.0/SpinBox.c,v 1.24 1999/12/29 18:24:45 danny Exp $
+ * $Header: /cvsroot/lesstif/lesstif/lib/Xm-2.0/SpinBox.c,v 1.34 2001/05/21 11:55:37 amai Exp $
  *
  * Copyright (C) 1997-1998 Free Software Foundation, Inc.
+ * Copyright (C) 1998-2001 LessTif Development Team
  *
  * This file is part of the GNU LessTif Library.
  *
@@ -22,27 +23,27 @@
  *
  **/
 
-static const char rcsid[] = "$Header: /cvsroot/hungry/lesstif/lib/Xm-2.0/SpinBox.c,v 1.24 1999/12/29 18:24:45 danny Exp $";
+static const char rcsid[] = "$Header: /cvsroot/lesstif/lesstif/lib/Xm-2.0/SpinBox.c,v 1.34 2001/05/21 11:55:37 amai Exp $";
+
+#include <LTconfig.h>
 
 #include <stdio.h>
-#include <LTconfig.h>
+
 #include <XmI/XmI.h>
 #include <Xm/XmP.h>
-#include <XmI/DebugUtil.h>
-
 #include <Xm/SpinBP.h>
-
 #include <Xm/Text.h>
 #include <Xm/TextF.h>
-
 #include <Xm/TraitP.h>
 #include <Xm/AccTextT.h>
 #include <Xm/NavigatorT.h>
 #include <Xm/ScrollFrameT.h>
 
+#include <XmI/DebugUtil.h>
+
 /* Forward Declarations */
 
-static void class_initialize();
+static void class_initialize(void);
 
 static void class_part_initialize(WidgetClass w_class);
 
@@ -280,7 +281,6 @@ char _XmSpinBox_ChildAccelerators[] =
 	"<Key>osfBeginData:		SpinBFirst()\n"
 	"<Key>osfEndData:		SpinBLast()\n";
 
-/* *INDENT-OFF* */
 static XmBaseClassExtRec _XmSpinBoxCoreClassExtRec = {
     /* next_extension            */ NULL,
     /* record_type               */ NULLQUARK,                             
@@ -396,7 +396,6 @@ XmSpinBoxClassRec xmSpinBoxClassRec = {
 	/* extension */ NULL,
     },
 };
-/* *INDENT-ON* */
 
 
 WidgetClass xmSpinBoxWidgetClass = (WidgetClass)&xmSpinBoxClassRec;
@@ -409,12 +408,14 @@ XmNavigatorTraitRec _XmSPBNavigatorTraitRec = {
 };
 
 static void
-class_initialize()
+class_initialize(void)
 {
     _XmSpinBoxCoreClassExtRec.record_type = XmQmotif;
 
-    XmeTraitSet(xmSpinBoxWidgetClass, XmQTnavigator, &_XmSPBNavigatorTraitRec);
-    XmeTraitSet(xmSpinBoxWidgetClass, XmQTnavigator, &_XmSPBNavigatorTraitRec);
+    XmeTraitSet((XtPointer)xmSpinBoxWidgetClass, XmQTnavigator,
+                (XtPointer)&_XmSPBNavigatorTraitRec);
+    XmeTraitSet((XtPointer)xmSpinBoxWidgetClass, XmQTnavigator,
+                (XtPointer)&_XmSPBNavigatorTraitRec);
 }
 
 static void
@@ -453,11 +454,11 @@ initialize(Widget request, Widget nw,
 {
 	XmScrollFrameTrait	t;
 
-	DEBUGOUT(XdbDebug(__FILE__, nw,
+	DEBUGOUT(_LtDebugDebug(__FILE__, nw,
 		      "SpinBox initialize: w/h %d %d\n",
 		      XtWidth(nw), XtHeight(nw)));
 #if 0
-	DEBUGOUT(XdbPrintArgList(__FILE__, nw, args, *num_args, False));
+	DEBUGOUT(_LtDebugPrintArgList(__FILE__, nw, args, *num_args, False));
 #endif
 
 	SPB_TextW(nw) = NULL;
@@ -471,7 +472,7 @@ initialize(Widget request, Widget nw,
 	}
 
 	/* Find out if our parent is a ScrollFrame */
-	t = (XmScrollFrameTrait) XmeTraitGet(XtClass(XtParent(nw)),
+	t = (XmScrollFrameTrait) XmeTraitGet((XtPointer)XtClass(XtParent(nw)),
 					XmQTscrollFrame);
 	if (t) {
 		t->init(XtParent(nw), _XmSPBTraitMoveCB, nw);	/* ??? FIX ME */
@@ -490,7 +491,7 @@ SetValues(Widget old, Widget request, Widget nw,
     Boolean	need_refresh = False,
 		need_layout = False;
 
-    DEBUGOUT(XdbDebug(__FILE__, nw, "SetValues()\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, nw, "SetValues()\n"));
 
 #define	NE(x)	(x(old) != x(nw))
 
@@ -506,8 +507,8 @@ SetValues(Widget old, Widget request, Widget nw,
     if (NE(SPB_UpX) || NE(SPB_UpY)
      || NE(SPB_UpWidth) || NE(SPB_UpHeight)
      || NE(SPB_DownX) || NE(SPB_DownY)
-     || NE(SPB_DownWidth) || NE(SPB_DownHeight))
-		;
+     || NE(SPB_DownWidth) || NE(SPB_DownHeight)) {
+    }
 
     if (need_layout) {
 	Layout(nw, NULL);
@@ -521,7 +522,7 @@ SetValues(Widget old, Widget request, Widget nw,
 static void
 expose(Widget w, XEvent *event, Region region)
 {
-	DEBUGOUT(XdbDebug(__FILE__, w, "Expose\n"));
+	DEBUGOUT(_LtDebugDebug(__FILE__, w, "Expose\n"));
 
 	_XmRedisplayGadgets(w, event, region);
 
@@ -550,7 +551,7 @@ expose(Widget w, XEvent *event, Region region)
 static void
 resize(Widget w)
 {
-	DEBUGOUT(XdbDebug(__FILE__, w, "Resize\n"));
+	DEBUGOUT(_LtDebugDebug(__FILE__, w, "Resize\n"));
 }
 
 /*
@@ -564,26 +565,28 @@ ConstraintInitialize(Widget request, Widget nw,
 {
     Widget sb = XtParent(nw);
 
-	DEBUGOUT(XdbDebug2(__FILE__, sb, nw, "ConstraintInitialize\n"));
+	DEBUGOUT(_LtDebugDebug2(__FILE__, sb, nw, "ConstraintInitialize\n"));
 #if 0
-	DEBUGOUT(XdbPrintArgList(__FILE__, nw, args, *num_args, False));
+	DEBUGOUT(_LtDebugPrintArgList(__FILE__, nw, args, *num_args, False));
 #endif
 
-		if (SPBC_Numeric(nw))
-			DEBUGOUT(XdbDebug(__FILE__, nw,
+		if (SPBC_Numeric(nw)) {
+			DEBUGOUT(_LtDebugDebug(__FILE__, nw,
 				"ConstraintInitialize(XmNUMERIC) min %d max %d inc %d\n",
 				SPBC_Minimum(nw),
 				SPBC_Maximum(nw),
 				SPBC_Increment(nw) ));
-		else
-			DEBUGOUT(XdbDebug(__FILE__, nw,
+		}
+		else {
+			DEBUGOUT(_LtDebugDebug(__FILE__, nw,
 				"ConstraintInitialize(XmSTRING) num_values %d\n",
 				SPBC_NumValues(nw)));
+		}
 
 	if (SPBC_ArrowSensitivity(nw) == XmARROWS_DEFAULT_SENSITIVITY)
 		SPBC_ArrowSensitivity(nw) = SPB_DefaultSensitivity(sb);
 
-	if (XmeTraitGet(XtClass(nw), XmQTaccessTextual))
+	if (XmeTraitGet((XtPointer)XtClass(nw), XmQTaccessTextual))
 		_XmSpinBoxShowValue(sb, nw);
 }
 
@@ -594,9 +597,9 @@ ConstraintSetValues(Widget curr, Widget req, Widget nw,
 	Widget	sb = XtParent(nw);
 	Boolean	r = False;
 
-	DEBUGOUT(XdbDebug2(__FILE__, sb, nw, "ConstraintSetValues\n"));
+	DEBUGOUT(_LtDebugDebug2(__FILE__, sb, nw, "ConstraintSetValues\n"));
 #if 0
-	DEBUGOUT(XdbPrintArgList(__FILE__, nw, args, *num_args, False));
+	DEBUGOUT(_LtDebugPrintArgList(__FILE__, nw, args, *num_args, False));
 #endif
 
 #define	NE(x)	(x(curr) != x(nw))
@@ -628,7 +631,7 @@ static void
 PreferredSize(	Widget w, Widget instig, XtWidgetGeometry *ig,
 				Dimension *wid, Dimension *ht)
 {
-	int		i;
+	Cardinal i;
 	Position	xx, yy;
 	Dimension	hh = 0;
 
@@ -643,7 +646,7 @@ PreferredSize(	Widget w, Widget instig, XtWidgetGeometry *ig,
 				hh = XtHeight(child);
 
 			xx += XtWidth(child) + SPB_Spacing(w);
-			DEBUGOUT(XdbDebug2(__FILE__, w, child,
+			DEBUGOUT(_LtDebugDebug2(__FILE__, w, child,
 				"PreferredSize: add width %d\n",
 				XtWidth(child)));
 	}
@@ -664,7 +667,7 @@ PreferredSize(	Widget w, Widget instig, XtWidgetGeometry *ig,
 	*wid = xx;
 	*ht = hh + 2 * SPB_MarginHeight(w);;
 
-	DEBUGOUT(XdbDebug(__FILE__, w,
+	DEBUGOUT(_LtDebugDebug(__FILE__, w,
 	"PreferredSize => %d %d (ArrowSize %d MarginWidth %d"
 	" MarginHeight %d Spacing %d)\n",
 		*wid, *ht,
@@ -680,11 +683,11 @@ PreferredSize(	Widget w, Widget instig, XtWidgetGeometry *ig,
 static void
 Layout(Widget w, Widget instig)
 {
-	int		i;
+	Cardinal i;
 	Position	xx, yy;
 	Dimension	hh = 0;
 
-	DEBUGOUT(XdbDebug(__FILE__, w, "XmSpinBox Layout\n"));
+	DEBUGOUT(_LtDebugDebug(__FILE__, w, "XmSpinBox Layout\n"));
 
 	for (i=0; i<MGR_NumChildren(w); i++)
 	    if (XtHeight(MGR_Children(w)[i]) > hh)
@@ -770,7 +773,7 @@ Layout(Widget w, Widget instig)
 	    SPB_UpHeight(w) = SPB_ArrowSize(w);
 	}
 
-	DEBUGOUT(XdbDebug(__FILE__, w,
+	DEBUGOUT(_LtDebugDebug(__FILE__, w,
 	    "SpinBoxLayout: down at %d %d up at %d %d, spb size %d %d\n",
 	    SPB_DownX(w), SPB_DownY(w), SPB_UpX(w), SPB_UpY(w),
 	    XtWidth(w), XtHeight(w)));
@@ -784,7 +787,7 @@ QueryGeometry(Widget w, XtWidgetGeometry *proposed, XtWidgetGeometry *answer)
 
 #define	Wants(x)	((geo.request_mode & x) == x)
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "QueryGeometry\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "QueryGeometry\n"));
 
     geo = *proposed;
 
@@ -798,16 +801,16 @@ QueryGeometry(Widget w, XtWidgetGeometry *proposed, XtWidgetGeometry *answer)
 
     if (Wants(CWHeight) && (geo.height != answer->height))
     {
-	DEBUGOUT(XdbDebug(__FILE__, w, "QueryGeometry => No.\n"));
+	DEBUGOUT(_LtDebugDebug(__FILE__, w, "QueryGeometry => No.\n"));
 	return XtGeometryNo;
     }
     if (Wants(CWWidth) && (geo.width != answer->width))
     {
-	DEBUGOUT(XdbDebug(__FILE__, w, "QueryGeometry => No.\n"));
+	DEBUGOUT(_LtDebugDebug(__FILE__, w, "QueryGeometry => No.\n"));
 	return XtGeometryNo;
     }
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "QueryGeometry => Yes.\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "QueryGeometry => Yes.\n"));
     return XtGeometryYes;
 }
 
@@ -816,7 +819,7 @@ GeometryManager(Widget w,
 		 XtWidgetGeometry *request,
 		 XtWidgetGeometry *reply)
 {
-	DEBUGOUT(XdbDebug(__FILE__, w, "GeometryManager\n"));
+	DEBUGOUT(_LtDebugDebug(__FILE__, w, "GeometryManager\n"));
 
 	/* FIX ME */
 
@@ -830,17 +833,17 @@ ChangeManaged(Widget w)
 	XtWidgetGeometry	geo;
 	XtGeometryResult	r;
 
-	DEBUGOUT(XdbDebug(__FILE__, w, "ChangeManaged\n"));
+	DEBUGOUT(_LtDebugDebug(__FILE__, w, "ChangeManaged\n"));
 
 	/* Make sure that our size is right */
 	PreferredSize(w, NULL, NULL, &geo.width, &geo.height);
 	geo.request_mode = CWWidth | CWHeight;
 
 	r = _XmMakeGeometryRequest(w, &geo);
-	DEBUGOUT(XdbDebug(__FILE__, w,
+	DEBUGOUT(_LtDebugDebug(__FILE__, w,
 		"ChangeManaged Geometry Request %s -> %s\n",
-		XdbWidgetGeometry2String(&geo),
-		XdbGeometryResult2String(r)));
+		_LtDebugWidgetGeometry2String(&geo),
+		_LtDebugGeometryResult2String(r)));
 
 	if (r == XtGeometryYes) {
 		/* FIX ME */
@@ -884,7 +887,7 @@ SpinBModify(Widget w, XEvent *evp, int reason)
 	tw = SPB_TextW(w);
 
 	if (tw == NULL) {
-		DEBUGOUT(XdbDebug(__FILE__, w, "SpinBModify: no text widget\n"));
+		DEBUGOUT(_LtDebugDebug(__FILE__, w, "SpinBModify: no text widget\n"));
 		return;
 	}
 
@@ -926,7 +929,7 @@ SpinBModify(Widget w, XEvent *evp, int reason)
 		s = _XmSpinBoxNumericString(tw);
 		xms = XmStringCreateSimple(s);
 
-		DEBUGOUT(XdbDebug(__FILE__, w,
+		DEBUGOUT(_LtDebugDebug(__FILE__, w,
 			"SpinBModify(NUMERIC) %d [%d,%d]\n",
 			SPBC_Position(tw), SPBC_Minimum(tw), SPBC_Maximum(tw)));
 	} else {
@@ -954,7 +957,7 @@ SpinBModify(Widget w, XEvent *evp, int reason)
 
 		xms = XmStringCopy(SPBC_Values(tw)[SPBC_Position(tw)]);
 
-		DEBUGOUT(XdbDebug(__FILE__, w,
+		DEBUGOUT(_LtDebugDebug(__FILE__, w,
 			"SpinBModify(STRING) %d [%d,%d]\n",
 			SPBC_Position(tw), 0, SPBC_NumValues(tw)));
 	}
@@ -992,7 +995,7 @@ _XmSpinBoxTimer(XtPointer client, XtIntervalId *timerid)
 {
 	Widget	spb = (Widget)client;
 
-	DEBUGOUT(XdbDebug(__FILE__, spb, "_XmSpinBoxTimer\n"));
+	DEBUGOUT(_LtDebugDebug(__FILE__, spb, "_XmSpinBoxTimer\n"));
 
 	if (SPB_UpArrowPressed(spb))
 			SpinBModify(spb, NULL, XmCR_SPIN_NEXT);
@@ -1008,7 +1011,7 @@ SpinBArm(Widget w, XEvent *event, String *params, Cardinal *num_params)
 {
 	Widget	tw;
 
-	DEBUGOUT(XdbDebug(__FILE__, w, "SpinBArm at %d %d\n",
+	DEBUGOUT(_LtDebugDebug(__FILE__, w, "SpinBArm at %d %d\n",
 		event->xbutton.x, event->xbutton.y));
 
 	if ((tw = SPB_TextW(w)) == NULL)
@@ -1069,7 +1072,7 @@ SpinBArm(Widget w, XEvent *event, String *params, Cardinal *num_params)
 static void
 SpinBDisarm(Widget w, XEvent *event, String *params, Cardinal *num_params)
 {
-	DEBUGOUT(XdbDebug(__FILE__, w, "SpinBDisarm\n"));
+	DEBUGOUT(_LtDebugDebug(__FILE__, w, "SpinBDisarm\n"));
 
 	if (SPB_TimerId(w))
 		XtRemoveTimeOut(SPB_TimerId(w));
@@ -1082,37 +1085,37 @@ SpinBDisarm(Widget w, XEvent *event, String *params, Cardinal *num_params)
 static void
 SpinBFirst(Widget w, XEvent *event, String *params, Cardinal *num_params)
 {
-	DEBUGOUT(XdbDebug(__FILE__, w, "SpinBFirst\n"));
+	DEBUGOUT(_LtDebugDebug(__FILE__, w, "SpinBFirst\n"));
 }
 
 static void
 SpinBLast(Widget w, XEvent *event, String *params, Cardinal *num_params)
 {
-	DEBUGOUT(XdbDebug(__FILE__, w, "SpinBLast\n"));
+	DEBUGOUT(_LtDebugDebug(__FILE__, w, "SpinBLast\n"));
 }
 
 static void
 SpinBLeft(Widget w, XEvent *event, String *params, Cardinal *num_params)
 {
-	DEBUGOUT(XdbDebug(__FILE__, w, "SpinBLeft\n"));
+	DEBUGOUT(_LtDebugDebug(__FILE__, w, "SpinBLeft\n"));
 }
 
 static void
 SpinBRight(Widget w, XEvent *event, String *params, Cardinal *num_params)
 {
-	DEBUGOUT(XdbDebug(__FILE__, w, "SpinBRight\n"));
+	DEBUGOUT(_LtDebugDebug(__FILE__, w, "SpinBRight\n"));
 }
 
 static void
 SpinBNext(Widget w, XEvent *event, String *params, Cardinal *num_params)
 {
-	DEBUGOUT(XdbDebug(__FILE__, w, "SpinBNext\n"));
+	DEBUGOUT(_LtDebugDebug(__FILE__, w, "SpinBNext\n"));
 }
 
 static void
 SpinBPrior(Widget w, XEvent *event, String *params, Cardinal *num_params)
 {
-	DEBUGOUT(XdbDebug(__FILE__, w, "SpinBPrior\n"));
+	DEBUGOUT(_LtDebugDebug(__FILE__, w, "SpinBPrior\n"));
 }
 
 static void
@@ -1120,7 +1123,7 @@ _XmSpinBoxChildFocus(Widget w, XtPointer client, XtPointer call)
 {
 	Widget	spb = (Widget)client;
 
-	DEBUGOUT(XdbDebug2(__FILE__, spb, w, "_XmSpinBoxChildFocus\n"));
+	DEBUGOUT(_LtDebugDebug2(__FILE__, spb, w, "_XmSpinBoxChildFocus\n"));
 
 	SPB_TextW(spb) = w;
 }
@@ -1130,17 +1133,17 @@ InsertChild(Widget w)
 {
     Widget	spb = XtParent(w);
 
-    DEBUGOUT(XdbDebug2(__FILE__, (Widget)spb, w, "SpinBox InsertChild\n"));
+    DEBUGOUT(_LtDebugDebug2(__FILE__, (Widget)spb, w, "SpinBox InsertChild\n"));
 
 #define superclass (&xmManagerClassRec)
     (*superclass->composite_class.insert_child) (w);
 #undef superclass
 
-	if (XmeTraitGet(XtClass(w), XmQTaccessTextual)) {
+	if (XmeTraitGet((XtPointer)XtClass(w), XmQTaccessTextual)) {
 #if 0
 		Arg		a;
 
-		DEBUGOUT(XdbDebug2(__FILE__, spb, w,
+		DEBUGOUT(_LtDebugDebug2(__FILE__, spb, w,
 			"Adding accelerators to child\n"));
 
 		XtSetArg(a, XmNtranslations,
@@ -1190,12 +1193,13 @@ _XmSpinBoxShowValue(Widget spb, Widget t)
 	XmString	xms;
 	XmAccessTextualTrait	trait;
 
-	DEBUGOUT(XdbDebug(__FILE__, spb, "_XmSpinBoxShowValue\n"));
+	DEBUGOUT(_LtDebugDebug(__FILE__, spb, "_XmSpinBoxShowValue\n"));
 
 	if (spb == NULL || t == NULL)
 		return;
 
-	trait = (XmAccessTextualTrait)XmeTraitGet(XtClass(t), XmQTaccessTextual);
+	trait = (XmAccessTextualTrait)XmeTraitGet((XtPointer)XtClass(t),
+	                                           XmQTaccessTextual);
 
 	if (trait == NULL) {
 	    _XmError(t, "_XmSpinBoxShowValue: no trait XmQTaccessTextual\n");
@@ -1224,7 +1228,7 @@ _XmSpinBoxShowValue(Widget spb, Widget t)
 			XmTextSetString(t, s);
 			XtFree(s);
 		} else
-			trait->setValue(t, xms, XmFORMAT_XmSTRING);
+			trait->setValue(t, (XtPointer)xms, XmFORMAT_XmSTRING);
 		/* Do *not* free xms */
 	}
 
@@ -1233,19 +1237,19 @@ _XmSpinBoxShowValue(Widget spb, Widget t)
 static void
 SpinBoxEnter(Widget w, XEvent *evp, String *par, Cardinal *npar)
 {
-	DEBUGOUT(XdbDebug(__FILE__, w, "SpinBoxEnter\n"));
+	DEBUGOUT(_LtDebugDebug(__FILE__, w, "SpinBoxEnter\n"));
 }
 
 static void
 SpinBoxFocusIn(Widget w, XEvent *evp, String *par, Cardinal *npar)
 {
-	DEBUGOUT(XdbDebug(__FILE__, w, "SpinBoxFocusIn\n"));
+	DEBUGOUT(_LtDebugDebug(__FILE__, w, "SpinBoxFocusIn\n"));
 }
 
 static void
 SpinBoxFocusOut(Widget w, XEvent *evp, String *par, Cardinal *npar)
 {
-	DEBUGOUT(XdbDebug(__FILE__, w, "SpinBoxFocusOut\n"));
+	DEBUGOUT(_LtDebugDebug(__FILE__, w, "SpinBoxFocusOut\n"));
 }
 
 /*
@@ -1257,7 +1261,7 @@ _XmSPBTraitChangeMoveCB(Widget nav,
 			XtPointer closure,
 			Boolean setUnset)
 {
-	DEBUGOUT(XdbDebug(__FILE__, nav,
+	DEBUGOUT(_LtDebugDebug(__FILE__, nav,
 		"_XmSPBTraitChangeMoveCB (%s)\n",
 		setUnset ? "set" : "unset"));
 
@@ -1272,7 +1276,7 @@ _XmSPBTraitSetValue(Widget nav, XmNavigatorData navigatorData, Boolean notify)
 {
 	Widget	tw;
 
-	DEBUGOUT(XdbDebug(__FILE__, nav, "_XmSPBTraitSetValue\n"));
+	DEBUGOUT(_LtDebugDebug(__FILE__, nav, "_XmSPBTraitSetValue\n"));
 
 	tw = SPB_TextW(nav);
 	if (tw == NULL)
@@ -1301,7 +1305,7 @@ _XmSPBTraitSetValue(Widget nav, XmNavigatorData navigatorData, Boolean notify)
 static int
 _XmSPBTraitGetValue(Widget nav, XmNavigatorData navigatorData)
 {
-	DEBUGOUT(XdbDebug(__FILE__, nav, "_XmSPBTraitGetValue\n"));
+	DEBUGOUT(_LtDebugDebug(__FILE__, nav, "_XmSPBTraitGetValue\n"));
 	return 0;
 }
 
@@ -1311,7 +1315,7 @@ _XmSPBTraitGetValue(Widget nav, XmNavigatorData navigatorData)
 static void
 _XmSPBTraitMoveCB(Widget nav, XtPointer client, XtPointer call)
 {
-	DEBUGOUT(XdbDebug(__FILE__, nav, "_XmSPBTraitMoveCB\n"));
+	DEBUGOUT(_LtDebugDebug(__FILE__, nav, "_XmSPBTraitMoveCB\n"));
 
 	/*
 	 * This gets called when a navigator (e.g. Notebook) that we're
@@ -1320,3 +1324,24 @@ _XmSPBTraitMoveCB(Widget nav, XtPointer client, XtPointer call)
 	 * By getting this call we can change our appearance (e.g. page number).
 	 */
 }
+
+
+extern int
+XmSpinBoxValidatePosition(Widget textfield,
+                          int *position)
+{
+   int oldpos;
+
+   if (SPBC_Numeric(textfield)) {
+	oldpos = SPBC_Position(textfield);
+	*position = oldpos;
+        _XmWarning(NULL, "XmSpinBoxValidatePosition(): not yet implemented!");
+        return XmCURRENT_VALUE;
+   }
+   else {
+        /* non-numeric */
+	oldpos = SPBC_Position(textfield);
+	*position = oldpos;
+	return XmCURRENT_VALUE;
+   }
+}
