@@ -1,8 +1,9 @@
 /**
  *
- * $Id: ArrowB.c,v 1.30 1999/03/06 05:26:17 gritton Exp $
+ * $Id: ArrowB.c,v 1.36 2001/05/21 08:36:51 amai Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
+ * Copyright (C) 1995-2001 LessTif Development Team
  *
  * This file is part of the GNU LessTif Library.
  *
@@ -22,11 +23,11 @@
  *
  **/
 
-static const char rcsid[] = "$Id: ArrowB.c,v 1.30 1999/03/06 05:26:17 gritton Exp $";
+static const char rcsid[] = "$Id: ArrowB.c,v 1.36 2001/05/21 08:36:51 amai Exp $";
 
 #include <LTconfig.h>
-#include <XmI/XmI.h>
 
+#include <XmI/XmI.h>
 #include <Xm/XmP.h>
 #include <Xm/ArrowBP.h>
 #include <Xm/RepType.h>
@@ -62,7 +63,7 @@ static XmActivatableTraitRec _XmArrowBTraitRec = {
 
 /* Forward Declarations */
 #if XmVERSION > 1
-static void class_initialize();
+static void class_initialize(void);
 #endif
 static void class_part_initialize(WidgetClass w_class);
 
@@ -158,7 +159,6 @@ static void MultiArm(Widget w, XEvent *event,
 		     String *params, Cardinal *num_params);
 
 
-/* *INDENT-OFF* */
 static XtActionsRec actions[] = {
     {"Activate",                Activate},
     {"MultiActivate",           MultiActivate},
@@ -229,17 +229,16 @@ XmArrowButtonClassRec xmArrowButtonClassRec = {
 	/* extension */ NULL
     }
 };
-/* *INDENT-ON* */
 
 
 WidgetClass xmArrowButtonWidgetClass = (WidgetClass)&xmArrowButtonClassRec;
 
 #if XmVERSION > 1
 static void
-class_initialize()
+class_initialize(void)
 {
-    if (! XmeTraitSet(xmArrowButtonWidgetClass, XmQTactivatable,
-		&_XmArrowBTraitRec)) {
+    if (! XmeTraitSet((XtPointer)xmArrowButtonWidgetClass, XmQTactivatable,
+		(XtPointer)&_XmArrowBTraitRec)) {
 	_XmWarning(NULL,
 	    "XmArrowButtonWidget ClassInitialize: XmeTraitSet failed\n");
     }
@@ -331,6 +330,7 @@ destroy(Widget w)
     if (AB_Timer(w) != 0)
     {
 	XtRemoveTimeOut(AB_Timer(w));
+	AB_Timer(w) = 0;
     }
 
     XtReleaseGC(w, AB_ArrowGC(w));
@@ -351,7 +351,7 @@ set_values(Widget old, Widget request, Widget new_w,
 			     AB_MultiClick(new_w), new_w))
 	AB_MultiClick(new_w) = AB_MultiClick(old);
 
-    DEBUGOUT(XdbDebug(__FILE__, new_w, "ArrowB set_values\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, new_w, "ArrowB set_values\n"));
 
     if (AB_Direction(old) != AB_Direction(new_w) ||
 #if XmVERSION > 1
@@ -471,7 +471,7 @@ Arm(Widget w, XEvent *event, String *params, Cardinal *num_params)
     int margin;
     XmArrowButtonCallbackStruct cbs;
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "Arm\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "Arm\n"));
 
     if (!AB_Armed(w))
     {
@@ -520,7 +520,7 @@ Activate(Widget w, XEvent *event, String *params, Cardinal *num_params)
     XmArrowButtonCallbackStruct cbs;
     XButtonEvent *ev = (XButtonEvent *)event;
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "Activate 000\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "Activate 000\n"));
 
     if (XtIsRealized(w))
     {
@@ -565,7 +565,7 @@ Disarm(Widget w, XEvent *event, String *params, Cardinal *num_params)
     int margin;
     XmArrowButtonCallbackStruct cbs;
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "Disarm\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "Disarm\n"));
 
     if (AB_Armed(w))
     {
@@ -606,7 +606,7 @@ ArmTimeout(XtPointer data, XtIntervalId *id)
     int margin;
     Widget w = (Widget)data;
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "ArmTimeout\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "ArmTimeout\n"));
 
     AB_Timer(w) = 0;
 
@@ -648,7 +648,7 @@ ArmAndActivate(Widget w, XEvent *event, String *params, Cardinal *num_params)
 {
     XmArrowButtonCallbackStruct cbs;
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "ArmAndActivate\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "ArmAndActivate\n"));
 
     /* Arm, Activate, and Disarm now, but draw the disarmed state later */
 
@@ -678,6 +678,7 @@ ArmAndActivate(Widget w, XEvent *event, String *params, Cardinal *num_params)
     if (AB_Timer(w) != 0)
     {
 	XtRemoveTimeOut(AB_Timer(w));
+	AB_Timer(w) = 0;
     }
 
     AB_Timer(w) = XtAppAddTimeOut(XtWidgetToApplicationContext(w),
@@ -689,7 +690,7 @@ EnterWindow(Widget w, XEvent *event, String *params, Cardinal *num_params)
 {
     int margin = Prim_ShadowThickness(w) + Prim_HighlightThickness(w);
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "EnterWindow\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "EnterWindow\n"));
 
     if (AB_Armed(w))
     {
@@ -713,7 +714,7 @@ LeaveWindow(Widget w, XEvent *event, String *params, Cardinal *num_params)
 {
     int margin = Prim_ShadowThickness(w) + Prim_HighlightThickness(w);
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "LeaveWindow\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "LeaveWindow\n"));
 
     if (AB_Armed(w))
     {
@@ -735,7 +736,7 @@ LeaveWindow(Widget w, XEvent *event, String *params, Cardinal *num_params)
 static void
 MultiArm(Widget w, XEvent *event, String *params, Cardinal *num_params)
 {
-    DEBUGOUT(XdbDebug(__FILE__, w, "MultiArm\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "MultiArm\n"));
 
     if (AB_MultiClick(w) == XmMULTICLICK_KEEP)
     {
@@ -750,7 +751,7 @@ MultiActivate(Widget w, XEvent *event, String *params, Cardinal *num_params)
     XButtonEvent *ev = (XButtonEvent *)event;
     XmPushButtonCallbackStruct cbs;
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "PushB: MultiClick\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "PushB: MultiClick\n"));
 
     if (AB_MultiClick(w) == XmMULTICLICK_KEEP)
     {
