@@ -1,8 +1,9 @@
 /**
  *
- * $Id: BaseClass.c,v 1.12 1999/11/18 19:47:54 amai Exp $
+ * $Id: BaseClass.c,v 1.24 2001/05/17 13:04:49 amai Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
+ * Copyright (C) 1995-2001 LessTif Development Team
  *
  * This file is part of the GNU LessTif Library.
  *
@@ -22,11 +23,14 @@
  *
  **/
 
-static const char rcsid[] = "$Id: BaseClass.c,v 1.12 1999/11/18 19:47:54 amai Exp $";
+static const char rcsid[] = "$Id: BaseClass.c,v 1.24 2001/05/17 13:04:49 amai Exp $";
 
 #include <LTconfig.h>
-#include <XmI/XmI.h>
 
+#include <stdio.h>
+#include <string.h>
+
+#include <XmI/XmI.h>
 #include <Xm/XmP.h>
 #include <Xm/BaseClassP.h>
 #include <Xm/DisplayP.h>
@@ -37,24 +41,23 @@ static const char rcsid[] = "$Id: BaseClass.c,v 1.12 1999/11/18 19:47:54 amai Ex
 #include <Xm/DropTransP.h>
 #include <Xm/VendorSEP.h>
 #include <Xm/DropSMgrP.h>
+#include <Xm/DialogS.h>
 
 #include <X11/RectObjP.h>
 #include <X11/Xutil.h>
-#include <X11/Xfuncs.h>
-#include <X11/Xfuncproto.h>
 
 #include <XmI/DebugUtil.h>
 
-#include <stdio.h>
 
 /* For the prototype police */
 
 extern XmWrapperData	_XmGetWrapperData(WidgetClass wc);
 extern void		_XmFreeWrapperData(XmWrapperData data);
 
+/* Some global data */
 XrmQuark XmQmotif = 0;
-int _XmInheritClass;
-XmBaseClassExt *_Xm_fastPtr;
+int _XmInheritClass = 0;
+XmBaseClassExt *_Xm_fastPtr=0;
 
 /*
  * credit where credit is due.  A good portion of the changes made today come
@@ -62,7 +65,7 @@ XmBaseClassExt *_Xm_fastPtr;
  * like the coding style, though -- that's my fault (MLM).
  * Danny gets the credit for the XContext hint, though.
  */
-XmGenericClassExt *
+extern XmGenericClassExt *
 _XmGetClassExtensionPtr(XmGenericClassExt *listHeadPtr,
 			XrmQuark owner)
 {
@@ -76,7 +79,8 @@ _XmGetClassExtensionPtr(XmGenericClassExt *listHeadPtr,
     return ext;
 }
 
-Boolean
+
+extern Boolean
 _XmIsSlowSubclass(WidgetClass wc,
 		  unsigned int bit)
 {
@@ -97,12 +101,13 @@ _XmIsSlowSubclass(WidgetClass wc,
     return False;
 }
 
+
 /*
  * Check a widget class for being a real (core) LessTif widget class. We
  * consider application defined classes as not being standard (beep) widget
  * classes.
  */
-Boolean
+extern Boolean
 _XmIsStandardMotifWidgetClass(WidgetClass wc)
 {
     XmBaseClassExt *bclass, *super;
@@ -164,12 +169,13 @@ _XmIsStandardMotifWidgetClass(WidgetClass wc)
     return False;
 }
 
+
 /*
  * according to Harald, this is for backwards compatibility only.  A local
  * function, BaseClassPartInitialize(), provides the functionality that
  * used to be here.
  */
-void
+extern void
 _XmBaseClassPartInitialize(WidgetClass wc)
 {
 }
@@ -205,6 +211,7 @@ XmWidgetExtDataStackRec, *XmWidgetExtDataStackPtr;
  */
 static XContext extContexts[XmDEFAULT_EXTENSION + 1] = { 0 };
 
+
 static XContext
 FindAssociatedContext(unsigned int extType)
 {
@@ -223,12 +230,13 @@ FindAssociatedContext(unsigned int extType)
     return extContexts[extType];
 }
 
+
 /*
  * nondestructively fetch an Extension data record.  Hey Danny, this
  * looks like a good way to find the VendorShellExt without looping
  * through Composite children!  Whadaya think?
  */
-XmWidgetExtData
+extern XmWidgetExtData
 _XmGetWidgetExtData(Widget widget, unsigned char extType)
 {
     XContext c = FindAssociatedContext(extType);
@@ -242,15 +250,17 @@ _XmGetWidgetExtData(Widget widget, unsigned char extType)
     return node->data;
 }
 
-void
+
+extern void
 _XmFreeWidgetExtData(Widget widget)
 {
 }
 
+
 /*
  * apparently, these can be chained...
  */
-void
+extern void
 _XmPushWidgetExtData(Widget widget,
 		     XmWidgetExtData data,
 		     unsigned char extType)
@@ -266,10 +276,11 @@ _XmPushWidgetExtData(Widget widget,
 
     node->next = head;
 
-    XSaveContext(XtDisplay(widget), (XID)widget, c, (_Xconst char *)node);
+    XSaveContext(XtDisplay(widget), (XID)widget, c, (XPointer)node);
 }
 
-void
+
+extern void
 _XmPopWidgetExtData(Widget widget,
 		    XmWidgetExtData *dataRtn,
 		    unsigned char extType)
@@ -295,7 +306,7 @@ _XmPopWidgetExtData(Widget widget,
     if (node->next)
     {
 	XSaveContext(XtDisplay(widget), (XID)widget, c,
-		     (_Xconst char *)node->next);
+		     (XPointer)node->next);
     }
     else
     {
@@ -307,6 +318,7 @@ _XmPopWidgetExtData(Widget widget,
     XtFree((char *)node);
 }
 
+
 /* -------------------------------------------------------------
  * Wrapper functions.
  */
@@ -354,6 +366,7 @@ _XmGetWrapperData(WidgetClass wc)
     return bce->wrapperData;
 }
 
+
 /*
  * This isn't really necessary! Better fold this into the source at the
  * appropiate places. Or make it a simple #define --aldi
@@ -361,12 +374,13 @@ _XmGetWrapperData(WidgetClass wc)
  * MLM: True, but I understand the sentiment.  The four functions taken
  * together "black box" the interface.  Good information hiding...
  */
-void
+extern void
 _XmFreeWrapperData(XmWrapperData data)
 {
     XtFree((char *)data);
 }
 
+
 /*
  * Push information about the hooked function pointer on a per-widget-class
  * stack.
@@ -428,7 +442,7 @@ _XmPushWrapperData(WidgetClass wc)
     {
 	wrapper = (XmWrapperData)XtMalloc(sizeof(XmWrapperDataRec));
 
-	bcopy(bce->wrapperData, wrapper, sizeof(XmWrapperDataRec));
+	memcpy(wrapper, bce->wrapperData, sizeof(XmWrapperDataRec));
 
 	wrapper->widgetClass = wc;
 	wrapper->next = bce->wrapperData;
@@ -437,7 +451,7 @@ _XmPushWrapperData(WidgetClass wc)
     {
 	wrapper = (XmWrapperData)XtMalloc(sizeof(XmWrapperDataRec));
 
-	bzero(wrapper, sizeof(XmWrapperDataRec));
+	memset(wrapper, 0, sizeof(XmWrapperDataRec));
     }
 
     bce->wrapperData = wrapper;
@@ -445,6 +459,7 @@ _XmPushWrapperData(WidgetClass wc)
     return wrapper;
 }
 
+
 /*
  * Pop data from the per-widget-class stack. The wrapper data is *NOT*
  * freed but instead a pointer to it is returned to the caller. Thus, it's
@@ -469,6 +484,7 @@ _XmPopWrapperData(WidgetClass wc)
     return wrapper;
 }
 
+
 /* ---------------------------------------------------------------------
  * Following is the realize wrapper stuff. Whereas the wrapper system
  * for hooking the class_part_init, initialize, set_values and get_values
@@ -632,6 +648,7 @@ static XtGeometryHandler GeometryWrappers[] =
     GeometryHandlerWrapper9
 };
 
+
 /*
  * Returns a widget class' depth in relation to the vendor shell widget
  * class. Any direct subclass of the vendor shell widget class is
@@ -674,6 +691,7 @@ RealizeDepth(WidgetClass MyWc)
     return Depth;
 }
 
+
 /*
  * Almost the same as the previous function, but this time the levels
  * are the distance from the rectObject widget class.
@@ -715,6 +733,7 @@ ResizeDepth(WidgetClass MyWc)
     return Depth;
 }
 
+
 /*
  * Same as above, but this time for the geometry handler stuff.
  */
@@ -773,6 +792,7 @@ static void (*ObjectClassPartInit)(WidgetClass c) = NULL;
 static Boolean (*ObjectSetValues)(Widget old, Widget request, Widget new_w,
 				  ArgList args, Cardinal *num_args) = NULL;
 
+
 /*
  * Much the same as XtIsSubclass(). But this time it checks whether a
  * widget *class* is a subclass of some other class. This function is
@@ -797,6 +817,7 @@ _XmIsSubclassOf(WidgetClass wc, WidgetClass sclass)
     return False;
 }
 
+
 /*
  * figure out if this widget class is one of those that needs inheritance
  * wrapper help.
@@ -896,6 +917,7 @@ ResolveWrappers(WidgetClass wc, WidgetClass sclass)
 #undef C_C
 }
 
+
 static XmBaseClassExt *
 BaseClassPartInitialize(WidgetClass wc)
 {
@@ -954,12 +976,12 @@ BaseClassPartInitialize(WidgetClass wc)
 	sext = _XmGetBaseClassExtPtr(sclass, XmQmotif);
     }
 
-    DEBUGOUT(XdbDebug(__FILE__, NULL,
+    DEBUGOUT(_LtDebugDebug(__FILE__, NULL,
 		      "BaseClassPartInit on %s with %s base ext rec.\n",
 		      wc->core_class.class_name,
 		      (ext && *ext) ? "a" : "NO!"));
 
-    DEBUGOUT(XdbDebug(__FILE__, NULL,
+    DEBUGOUT(_LtDebugDebug(__FILE__, NULL,
 		      "BaseClassPartInit: sclass %s with %s base ext rec.\n",
 		      wc->core_class.superclass->core_class.class_name,
 		      (sext && *sext) ? "a" : "NO!"));
@@ -1025,7 +1047,7 @@ BaseClassPartInitialize(WidgetClass wc)
 	 * to indicate that this class had no base extension...
 	 */
 	XmBaseClassExt p;
-	DEBUGOUT(XdbDebug(__FILE__, NULL,
+	DEBUGOUT(_LtDebugDebug(__FILE__, NULL,
 			  "BaseClassPartInit(): %s got bce record on the fly\n",
 			  wc->core_class.class_name));
 
@@ -1052,7 +1074,7 @@ BaseClassPartInitialize(WidgetClass wc)
      * Let us be notorious and always clean the flags. The flags have
      * to be set during the class_part_init method of a widget class.
      */
-    bzero((*ext)->flags, 32);
+    memset((*ext)->flags, 0, 32);
 
     /*
      * If the superclass hasn't a base class extension record, use
@@ -1147,6 +1169,7 @@ BaseClassPartInitialize(WidgetClass wc)
     return ext;
 }
 
+
 /*
  * A wrapper shrinked over a widget class' realize procedure. This is
  * called from the RealizeWrapper<#> funcions with the last param
@@ -1201,7 +1224,8 @@ RealizeWrapper(Widget w, XtValueMask *ValueMask,
 
     ext = _XmGetWidgetExtData(w, XmSHELL_EXTENSION);
 
-    if (ext != NULL && ext->widget != NULL)
+/*    if (ext != NULL && ext->widget != NULL && !XmIsDialogShell(w)) */
+    if (ext != NULL && ext->widget != NULL && wc != xmDialogShellWidgetClass)
     {
 	/*
 	 * Check for a callback list to be executed (the only exception
@@ -1213,6 +1237,7 @@ RealizeWrapper(Widget w, XtValueMask *ValueMask,
     }
 }
 
+
 static void
 ResizeWrapper(Widget w, int IntentedWrapperDepth)
 {
@@ -1268,6 +1293,7 @@ ResizeWrapper(Widget w, int IntentedWrapperDepth)
     }
 }
 
+
 /*
  * fun with GeoUtils showed me why this was broken.  Remember how a 
  * geometry manager is called with the child requesting the change?
@@ -1283,8 +1309,8 @@ GeometryHandlerWrapper(Widget w, XtWidgetGeometry *request,
     static Boolean recurse = False;
     XtGeometryResult ret;
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "GeometryHandlerWrapper(%d) : request %s\n",
-		      IntentedWrapperDepth, XdbWidgetGeometry2String(request)));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "GeometryHandlerWrapper(%d) : request %s\n",
+		      IntentedWrapperDepth, _LtDebugWidgetGeometry2String(request)));
 
     WindUp = GeomDepth(wc) - IntentedWrapperDepth;
 
@@ -1326,6 +1352,7 @@ GeometryHandlerWrapper(Widget w, XtWidgetGeometry *request,
     return ret;
 }
 
+
 /* ---------------------------------------------------------------------
  * Following is the implementation of the hook wrapper mechanism. In
  * order to do pre- and post-processing before/after a widget's class
@@ -1342,6 +1369,7 @@ GeometryHandlerWrapper(Widget w, XtWidgetGeometry *request,
  * pointer and calls the post-hook function. Any questions remaining...?
  */
 
+
 /*
  * This is the ClassPartInit method's post-hook. It calls the last
  * ClassPartInit method in the chain and then restores the original
@@ -1377,6 +1405,7 @@ ClassPartInitLeafWrapper(WidgetClass widget_class)
     wrap->classPartInitLeaf = NULL;
 }
 
+
 /*
  * This ClassPartInit pre-hook is called as the first method whenever a
  * new widget class gets initialized.
@@ -1430,6 +1459,7 @@ ClassPartInitRootWrapper(WidgetClass widget_class)
     }
 }
 
+
 /*
  * Initialize posthook stuff.
  * it is called just before the last init has been called and then redirects
@@ -1488,6 +1518,7 @@ InitializeLeafWrapper(Widget request, Widget new_w,
     }
 }
 
+
 /*
  * The initialze pre-hook is called on any fresh widget instance before
  * any other initialize method has ever seen this new_w widget.
@@ -1544,6 +1575,7 @@ InitializeRootWrapper(Widget request, Widget new_w,
     }
 }
 
+
 /*
  * This is the set_values post-hook. It is called as the last set_values
  * method in the chain and in turn calls the real last set_values method
@@ -1594,6 +1626,7 @@ SetValuesLeafWrapper(Widget old, Widget request, Widget new_w,
     return refresh;
 }
 
+
 /*
  * This SetValues pre-hook is called before any set_values methods whenever
  * a XtSetValues() call occures. We then call the registered pre-hook for
@@ -1666,6 +1699,7 @@ SetValuesRootWrapper(Widget old, Widget request, Widget new_w,
     return refresh;
 }
 
+
 /*
  * The GetValues post-hook is called after any get_values_hook methods of
  * the current widget's class and superclasses. But it is called BEFORE any
@@ -1703,6 +1737,7 @@ GetValuesLeafWrapper(Widget w, ArgList args, Cardinal *num_args)
     }
 }
 
+
 /*
  * The GetValues pre-hook is called before any other get_values_hook method.
  * But there's a problem with the post-hook function. It isn't called as the
@@ -1754,6 +1789,7 @@ GetValuesRootWrapper(Widget w, ArgList args, Cardinal *num_args)
     }
 }
 
+
 /*
  * Initialize the dreaded extension mechanism. This new extension is needed
  * in LessTif in order to pre- or postprocess some toolkit operations. This
@@ -1762,7 +1798,7 @@ GetValuesRootWrapper(Widget w, ArgList args, Cardinal *num_args)
  * MLM: Possibly, but I'm gonna make sure it's called by putting in the
  * lesstif core widgets -- Gadget, Primitive, Manager, VendorShell.
  */
-void
+extern void
 _XmInitializeExtensions(void)
 {
     /* We want to use the same as Motif, so our
@@ -1787,6 +1823,7 @@ _XmInitializeExtensions(void)
     }
 }
 
+
 /*
  * Assumption: widgets that have subresources use this as a helper
  * function when their getSecResData BaseClassExt function is called.
@@ -1795,7 +1832,7 @@ _XmInitializeExtensions(void)
  * relationship to testXm/baseclass/test5.c.  Now to figure out who uses
  * it.
  */
-Cardinal
+extern Cardinal
 _XmSecondaryResourceData(XmBaseClassExt bce,
 			 XmSecondaryResourceData **secResDataRtn,
 			 XtPointer client_data,
@@ -1838,6 +1875,7 @@ _XmSecondaryResourceData(XmBaseClassExt bce,
     return 1;
 }
 
+
 /*
  * Testing shows that calling this function before and after instantiation
  * of a widget results in uncompiled resources.  So I assume that this
@@ -1847,14 +1885,14 @@ _XmSecondaryResourceData(XmBaseClassExt bce,
  * an object (the "owner" field in the BaseClassExt isn't set until
  * class_intialize), thus the following crap.
  */
-void
+extern void
 _XmTransformSubResources(XtResourceList comp_resources,
 			 Cardinal num_comp_resources,
 			 XtResourceList *resources,
 			 Cardinal *num_resources)
 {
     XtResourceList res;
-    int i;
+    Cardinal i;
 
     if (num_comp_resources == 0)
     {
@@ -1867,7 +1905,7 @@ _XmTransformSubResources(XtResourceList comp_resources,
 
     if ((int)(comp_resources[0].resource_offset) >= 0)
     {
-	bcopy(comp_resources, res, sizeof(XtResource) * num_comp_resources);
+	memcpy(res, comp_resources, sizeof(XtResource) * num_comp_resources);
     }
     else
     {
@@ -1899,19 +1937,24 @@ _XmTransformSubResources(XtResourceList comp_resources,
     *num_resources = num_comp_resources;
 }
 
+
 /*
  * lo and behold, there is a man page for this one.
  * According to OSF/Motif Programmer's Reference, page 1-539, there are
  * three things to free: the resources, the block, and the block array
  * (see testXm/baseclass/test5.c).
  */
-Cardinal
+extern Cardinal
 XmGetSecondaryResourceData(WidgetClass w_class,
 			   XmSecondaryResourceData **secondaryDataRtn)
 
 {
     XmBaseClassExt *bce = NULL;
 
+    DEBUGOUT(_LtDebugDebug(__FILE__, NULL,
+		      "XmGetSecondaryResourceData() called on %s.\n",
+		      w_class->core_class.class_name));
+
     bce = _XmGetBaseClassExtPtr(w_class, XmQmotif);
 
     if (!bce || !*bce)
