@@ -1,8 +1,9 @@
 /**
  *
- * $Id: CascadeB.c,v 1.148 1999/11/19 00:58:34 rwscott Exp $
+ * $Id: CascadeB.c,v 1.158 2001/05/17 10:04:53 amai Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
+ * Copyright (C) 1995-2001 LessTif Development Team
  *
  * This file is part of the GNU LessTif Library.
  *
@@ -22,12 +23,14 @@
  *
  **/
 
-static const char rcsid[] = "$Id: CascadeB.c,v 1.148 1999/11/19 00:58:34 rwscott Exp $";
+static const char rcsid[] = "$Id: CascadeB.c,v 1.158 2001/05/17 10:04:53 amai Exp $";
 
 #include <LTconfig.h>
+
+#include <stdarg.h>
+
 #include <XmI/XmI.h>
 #include <XmI/MacrosI.h>
-
 #include <Xm/XmP.h>
 #include <Xm/CascadeBP.h>
 #include <Xm/CascadeBGP.h>
@@ -38,73 +41,45 @@ static const char rcsid[] = "$Id: CascadeB.c,v 1.148 1999/11/19 00:58:34 rwscott
 #include <Xm/MenuUtilP.h>
 #include <Xm/ScreenP.h>
 
-#if STDC_HEADERS
-#include <stdarg.h>
-#else
-#include <varargs.h>
-#endif
-
 #include <XmI/DebugUtil.h>
 
-#include <stdio.h>
-
-#ifndef XmUNSPECIFIED
-#define XmUNSPECIFIED	(~0)
-#endif
 
 #define CASCADE_SPACING	4
 
 /* Forward Declarations */
 
 static void class_initialize(void);
-
 static void class_part_initialize(WidgetClass w_class);
-
 static void initialize(Widget request, Widget new_w,
 		       ArgList args, Cardinal *num_args);
-
 static void initialize_prehook(Widget request, Widget new_w,
 			       ArgList args, Cardinal *num_args);
-
 static void initialize_posthook(Widget request, Widget new_w,
 				ArgList args, Cardinal *num_args);
-
 static void resize(Widget w);
-
 static void destroy(Widget w);
-
 static void expose(Widget w, XEvent *event, Region region);
-
 static Boolean set_values(Widget current, Widget request, Widget new_w,
 			  ArgList args, Cardinal *num_args);
-
 static void CleanupMenuBar(Widget w, XEvent *event,
 			   String *params, Cardinal *num_params);
-
 static void StartDrag(Widget, XEvent *, String *, Cardinal *);
-
 static void KeySelect(Widget, XEvent *, String *, Cardinal *);
-
 static void ArmAndActivate(Widget w, XEvent *event,
 			   String *params, Cardinal *num_params);
-
 static void _XmCBMenuBarSelect(Widget, XEvent *, String *, Cardinal *);
-
 static void _XmCBMenuBarDoSelect(Widget, XEvent *, String *, Cardinal *);
-
 static void MenuBarEnter(Widget, XEvent *, String *, Cardinal *);
-
 static void MenuBarLeave(Widget, XEvent *, String *, Cardinal *);
-
 static void DelayedArm(Widget, XEvent *, String *, Cardinal *);
-
 static void CheckDisarm(Widget, XEvent *, String *, Cardinal *);
 
 #if 0
 static void ArmAndPost(Widget, XEvent *, String *, Cardinal *);
 #endif
 
-static void MenuProcEntry(int proc, Widget rc,...);
+static void MenuProcEntry(int proc, Widget rc, ...);
+
 
 /*
  * Resources for the cascade button class
@@ -161,6 +136,7 @@ static XtResource resources[] =
     }
 };
 
+
 static XtActionsRec actions[] =
 {
     {"DelayedArm", DelayedArm},
@@ -175,7 +151,7 @@ static XtActionsRec actions[] =
     {"Help", _XmCBHelp},
 };
 
-/* *INDENT-OFF* */
+
 static XmBaseClassExtRec _XmCascadeBCoreClassExtRec = {
     /* next_extension            */ NULL,
     /* record_type               */ NULLQUARK,
@@ -270,7 +246,6 @@ XmCascadeButtonClassRec xmCascadeButtonClassRec = {
 	/* extension */ NULL
     }
 };
-/* *INDENT-ON* */
 
 
 WidgetClass xmCascadeButtonWidgetClass = (WidgetClass)&xmCascadeButtonClassRec;
@@ -288,26 +263,28 @@ class_initialize(void)
     popup_trans = XtParseTranslationTable(_XmCascadeB_p_events);
 }
 
+
 static void
 class_part_initialize(WidgetClass widget_class)
 {
     _XmFastSubclassInit(widget_class, XmCASCADE_BUTTON_BIT);
 }
 
+
 static void
 initialize_prehook(Widget request, Widget new_w,
 		   ArgList args, Cardinal *num_args)
 {
-    DEBUGOUT(XdbDebug(__FILE__, new_w, "%s:initialize_prehook()\n", __FILE__));
-    DEBUGOUT(XdbDebug("MENUT", new_w, "%s:initialize_prehook(%d)\n", __FILE__, __LINE__));
+    DEBUGOUT(_LtDebugDebug(__FILE__, new_w, "%s:initialize_prehook()\n", __FILE__));
+    DEBUGOUT(_LtDebugDebug("MENUT", new_w, "%s:initialize_prehook(%d)\n", __FILE__, __LINE__));
 
     _XmSaveCoreClassTranslations(new_w);
 
     if (XmIsRowColumn(XtParent(new_w)) &&
 	RC_Type(XtParent(new_w)) == XmMENU_BAR)
     {
-	DEBUGOUT(XdbDebug(__FILE__, new_w, "    MenuBar translations\n"));
-	DEBUGOUT(XdbDebug0("MENUT", new_w, "    MenuBar translations\n"));
+	DEBUGOUT(_LtDebugDebug(__FILE__, new_w, "    MenuBar translations\n"));
+	DEBUGOUT(_LtDebugDebug0("MENUT", new_w, "    MenuBar translations\n"));
 
 	CoreClassTranslations(new_w) = (String)menub_trans;
     }
@@ -315,19 +292,20 @@ initialize_prehook(Widget request, Widget new_w,
 	     (RC_Type(XtParent(new_w)) == XmMENU_POPUP ||
 	      RC_Type(XtParent(new_w)) == XmMENU_PULLDOWN))
     {
-	DEBUGOUT(XdbDebug(__FILE__, new_w, "    Popup translations\n"));
-	DEBUGOUT(XdbDebug0("MENUT", new_w, "    Popup translations\n"));
+	DEBUGOUT(_LtDebugDebug(__FILE__, new_w, "    Popup translations\n"));
+	DEBUGOUT(_LtDebugDebug0("MENUT", new_w, "    Popup translations\n"));
 
 	CoreClassTranslations(new_w) = (String)popup_trans;
     }
     else
     {
-	DEBUGOUT(XdbDebug(__FILE__, new_w, "    Default translations\n"));
-	DEBUGOUT(XdbDebug0("MENUT", new_w, "    Default translations\n"));
+	DEBUGOUT(_LtDebugDebug(__FILE__, new_w, "    Default translations\n"));
+	DEBUGOUT(_LtDebugDebug0("MENUT", new_w, "    Default translations\n"));
     }
 
 }
 
+
 static void
 initialize_posthook(Widget request, Widget new_w,
 		    ArgList args, Cardinal *num_args)
@@ -335,6 +313,7 @@ initialize_posthook(Widget request, Widget new_w,
     _XmRestoreCoreClassTranslations(new_w);
 }
 
+
 static void
 place_cascade(Widget w)
 {
@@ -362,6 +341,7 @@ place_cascade(Widget w)
     }
 }
 
+
 static void
 size_cascade(Widget w)
 {
@@ -382,11 +362,12 @@ size_cascade(Widget w)
     }
 }
 
+
 static void
 initialize(Widget request, Widget new_w, ArgList args, Cardinal *num_args)
 {
-    DEBUGOUT(XdbDebug(__FILE__, new_w, "%s:initialize()\n", __FILE__));
-    DEBUGOUT(XdbDebug("MENUT", new_w, "%s:initialize(%d)\n", __FILE__, __LINE__));
+    DEBUGOUT(_LtDebugDebug(__FILE__, new_w, "%s:initialize()\n", __FILE__));
+    DEBUGOUT(_LtDebugDebug("MENUT", new_w, "%s:initialize(%d)\n", __FILE__, __LINE__));
 
     CB_SetArmed(new_w, False);
     CB_ArmedPixmap(new_w) = None;
@@ -475,10 +456,11 @@ initialize(Widget request, Widget new_w, ArgList args, Cardinal *num_args)
     LabClass_MenuProcs(XtClass(new_w)) = MenuProcEntry;
 }
 
+
 static void
 resize(Widget w)
 {
-    DEBUGOUT(XdbDebug(__FILE__, w, "%s:resize(%d) - %dx%d\n",
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "%s:resize(%d) - %dx%d\n",
 			__FILE__, __LINE__,
 			XtWidth(w), XtHeight(w)));
 
@@ -492,14 +474,16 @@ resize(Widget w)
     }
 }
 
+
 static void
 destroy(Widget w)
 {
-    DEBUGOUT(XdbDebug(__FILE__, w, "%s:destroy()\n", __FILE__));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "%s:destroy()\n", __FILE__));
 
     if (CB_Timer(w) != 0)
     {
 	XtRemoveTimeOut(CB_Timer(w));
+	CB_Timer(w) = 0;
     }
 
     if (CB_ArmedPixmap(w))
@@ -509,13 +493,14 @@ destroy(Widget w)
     }
 }
 
+
 static Boolean
 set_values(Widget old, Widget request, Widget new_w,
 	   ArgList args, Cardinal *num_args)
 {
     Boolean refresh_needed = False;
 
-    DEBUGOUT(XdbDebug(__FILE__, new_w,
+    DEBUGOUT(_LtDebugDebug(__FILE__, new_w,
 		      "%s:set_values() - %i args\n"
 		      "\t    old X %5i Y %5i W %5i H %5i\n"
 		      "\trequest X %5i Y %5i W %5i H %5i\n"
@@ -528,7 +513,7 @@ set_values(Widget old, Widget request, Widget new_w,
 		      XtWidth(request), XtHeight(request),
 		      XtX(new_w), XtY(new_w),
 		      XtWidth(new_w), XtHeight(new_w)));
-    DEBUGOUT(XdbPrintArgList(__FILE__, new_w, args, *num_args, False));
+    DEBUGOUT(_LtDebugPrintArgList(__FILE__, new_w, args, *num_args, False));
 
     if (CB_MapDelay(new_w) < 0)
     {
@@ -563,16 +548,18 @@ set_values(Widget old, Widget request, Widget new_w,
 	{
 	    if (CB_CascadePixmap(new_w) == XmUNSPECIFIED_PIXMAP &&
 		CB_Submenu(new_w))
+	    {
 		_XmCreateArrowPixmaps(new_w);
+	    }
 	    size_cascade(new_w);
+	    refresh_needed = True;
 	}
 
-	refresh_needed = True;
     }
 
     if (XtSensitive(new_w) != XtSensitive(old))
     {
-	DEBUGOUT(XdbDebug(__FILE__, new_w,
+	DEBUGOUT(_LtDebugDebug(__FILE__, new_w,
 			  "SetValues: sensitive changes to %d\n",
 			  XtSensitive(new_w)));
 
@@ -678,6 +665,7 @@ set_values(Widget old, Widget request, Widget new_w,
     return refresh_needed;
 }
 
+
 static void
 draw_cascade(Widget w)
 {
@@ -704,12 +692,13 @@ draw_cascade(Widget w)
     }
 }
 
+
 static void
 expose(Widget w, XEvent *event, Region region)
 {
-    DEBUGOUT(XdbDebug(__FILE__, w, "%s:expose() Armed(%d) type %s\n",
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "%s:expose() Armed(%d) type %s\n",
 		      __FILE__,
-		      CB_IsArmed(w), XdbMenuType2String(Lab_MenuType(w))));
+		      CB_IsArmed(w), _LtDebugMenuType2String(Lab_MenuType(w))));
 
     /* The label's expose routine should be called after the cascade button's
      * expose routine. */
@@ -721,6 +710,7 @@ expose(Widget w, XEvent *event, Region region)
 #undef superclass
 }
 
+
 /* ACTION PROCS */
 /*
  * CleanupMenuBar
@@ -729,7 +719,7 @@ expose(Widget w, XEvent *event, Region region)
 static void
 CleanupMenuBar(Widget w, XEvent *event, String *params, Cardinal *num_params)
 {
-    DEBUGOUT(XdbDebug(__FILE__, w, "CleanupMenuBar()\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "CleanupMenuBar()\n"));
 
     if (w)			/* Try not to crash */
     {
@@ -738,10 +728,11 @@ CleanupMenuBar(Widget w, XEvent *event, String *params, Cardinal *num_params)
     }
 }
 
+
 static void
 KeySelect(Widget w, XEvent *event, String *params, Cardinal *num_params)
 {
-    DEBUGOUT(XdbDebug(__FILE__, w, "KeySelect\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "KeySelect\n"));
 
     /* If we're in keyboard traversal mode, and if we have a submenu
        attached, call the cascading callback, post the submenu and
@@ -751,7 +742,8 @@ KeySelect(Widget w, XEvent *event, String *params, Cardinal *num_params)
        in the cascade */
 }
 
-void
+
+extern void
 _XmCascadingPopup(Widget w, XEvent *event, Boolean doCascade)
 {
     /* FIX ME: doCascade? */
@@ -768,7 +760,10 @@ _XmCascadingPopup(Widget w, XEvent *event, Boolean doCascade)
     Widget sub_pane;
     Boolean was_torn;
 
-	sub_pane = XmIsGadget(w) ?  CBG_Submenu(w) : CB_Submenu(w);
+	if (XmIsGadget(w))
+	    sub_pane = CBG_Submenu(w);
+	else
+	    sub_pane = CB_Submenu(w);
 	if (sub_pane)
 	{
 	    RCClass_MenuProcs(XtClass(sub_pane))(XmMENU_RESTORE_TEAROFF_TO_MENUSHELL,
@@ -781,11 +776,10 @@ _XmCascadingPopup(Widget w, XEvent *event, Boolean doCascade)
 
     XFlush(XtDisplay(w));
 
-    XtCallCallbackList(w,
-		       (XmIsGadget(w)
-			? CBG_CascadeCall(w)
-			: CB_CascadeCall(w)),
-		       &cbs);
+    if(XmIsGadget(w))
+          XtCallCallbackList(w, CBG_CascadeCall(w), &cbs);
+    else
+          XtCallCallbackList(w, CB_CascadeCall(w),  &cbs);
 
     if (doCascade)
     {
@@ -793,6 +787,7 @@ _XmCascadingPopup(Widget w, XEvent *event, Boolean doCascade)
     }
 }
 
+
 /*
  *    _XmCBMenuBarDoSelect()
  *
@@ -802,22 +797,22 @@ _XmCascadingPopup(Widget w, XEvent *event, Boolean doCascade)
  *      calls the callbacks in XmNactivateCallback, activates the
  *      CascadeButton, and unposts all posted menus in the cascade.
  */
-void
+static void
 _XmCBMenuBarDoSelect(Widget w, XEvent *event,
 		     String *params, Cardinal *num_params)
 {
 Boolean validButton;
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "%s:_XmCBMenuBarDoSelect(%d)\n\t%s posted mine %s\n",
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "%s:_XmCBMenuBarDoSelect(%d)\n\t%s posted mine %s\n",
 	__FILE__, __LINE__,
 	RC_PopupPosted(XtParent(w)) ? XtName(RC_PopupPosted(XtParent(w))) : "nothing",
 	CB_Submenu(w) ? XtName(CB_Submenu(w)) : "none"
 	));
-    DEBUGOUT(XdbDebug("MENU", w, "%s:_XmCBMenuBarDoSelect(%d)\n\t%s posted mine %s %s\n",
+    DEBUGOUT(_LtDebugDebug("MENU", w, "%s:_XmCBMenuBarDoSelect(%d)\n\t%s posted mine %s %s\n",
 	__FILE__, __LINE__,
 	RC_PopupPosted(XtParent(w)) ? XtName(RC_PopupPosted(XtParent(w))) : "nothing",
 	CB_Submenu(w) ? XtName(CB_Submenu(w)) : "none",
-	event ? XdbEventType2String(event->xany.type) : "no-event"
+	event ? _LtDebugEventType2String(event->xany.type) : "no-event"
 	));
     if (event && event->xany.type == ButtonRelease)
     {
@@ -869,18 +864,19 @@ Boolean validButton;
     }
 }
 
-void
+
+static void
 _XmCBMenuBarSelect(Widget w, XEvent *event,
 		   String *params, Cardinal *num_params)
 {
     Boolean validButton;
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "%s:_XmCBMenuBarSelect(%d)\n\t%s posted mine %s\n",
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "%s:_XmCBMenuBarSelect(%d)\n\t%s posted mine %s\n",
 	__FILE__, __LINE__,
 	RC_PopupPosted(XtParent(w)) ? XtName(RC_PopupPosted(XtParent(w))) : "nothing",
 	CB_Submenu(w) ? XtName(CB_Submenu(w)) : "none"
 	));
-    DEBUGOUT(XdbDebug("MENU", w, "%s:_XmCBMenuBarSelect(%d)\n\t%s posted mine %s\n",
+    DEBUGOUT(_LtDebugDebug("MENU", w, "%s:_XmCBMenuBarSelect(%d)\n\t%s posted mine %s\n",
 	__FILE__, __LINE__,
 	RC_PopupPosted(XtParent(w)) ? XtName(RC_PopupPosted(XtParent(w))) : "nothing",
 	CB_Submenu(w) ? XtName(CB_Submenu(w)) : "none"
@@ -899,7 +895,7 @@ _XmCBMenuBarSelect(Widget w, XEvent *event,
 	XmProcessTraversal(w, XmTRAVERSE_CURRENT);
 	if (CB_Submenu(w))
 	{
-	int i;
+	Cardinal i;
 	Widget tmp = RC_PopupPosted(XtParent(w));
 
 	    if (RC_PopupPosted(XtParent(w)))
@@ -962,7 +958,7 @@ _XmCBMenuBarSelect(Widget w, XEvent *event,
 	}
 	else
 	{
-	int i;
+	Cardinal i;
 	Boolean poppedUp;
 
 	    _XmCascadingPopup(w, event, False);
@@ -983,17 +979,18 @@ _XmCBMenuBarSelect(Widget w, XEvent *event,
     }
 }
 
+
 #if 0
 static void
 ArmAndPost(Widget w, XEvent *event, String *params, Cardinal *num_params)
 {
-    DEBUGOUT(XdbDebug(__FILE__, w, "ArmAndPost()\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "ArmAndPost()\n"));
 
     XmProcessTraversal(w, XmTRAVERSE_CURRENT);
 
     if (!RC_IsArmed(CBG_Submenu(w)))
     {
-	DEBUGOUT(XdbDebug(__FILE__, w, "MENU_ARM\n"));
+	DEBUGOUT(_LtDebugDebug(__FILE__, w, "MENU_ARM\n"));
 
 	RCClass_MenuProcs(XtClass(XtParent(w)))(XmMENU_ARM, XtParent(w), NULL);
     }
@@ -1002,19 +999,20 @@ ArmAndPost(Widget w, XEvent *event, String *params, Cardinal *num_params)
 }
 #endif
 
+
 static void
 MenuBarEnter(Widget w, XEvent *event, String *params, Cardinal *num_params)
 {
-    DEBUGOUT(XdbDebug(__FILE__, w, "MenuBarEnter\n\t%s %s %s %s posted posting %s\n",
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "MenuBarEnter\n\t%s %s %s %s posted posting %s\n",
 	_XmGetInDragMode(w) ? "Dragging" : "Not Dragging",
-	XdbRcType2String(Lab_MenuType(w)),
+	_LtDebugRcType2String(Lab_MenuType(w)),
 	RC_IsArmed(XtParent(w)) ? "Armed" : "Not Armed",
 	RC_PopupPosted(XtParent(w)) ? XtName(RC_PopupPosted(XtParent(w))) : "nothing",
 	CB_Submenu(w) ? XtName(CB_Submenu(w)) : "nothing"
 	));
-    DEBUGOUT(XdbDebug("MENU", w, "MenuBarEnter\n\t%s %s %s %s posted posting %s\n",
+    DEBUGOUT(_LtDebugDebug("MENU", w, "MenuBarEnter\n\t%s %s %s %s posted posting %s\n",
 	_XmGetInDragMode(w) ? "Dragging" : "Not Dragging",
-	XdbRcType2String(Lab_MenuType(w)),
+	_LtDebugRcType2String(Lab_MenuType(w)),
 	RC_IsArmed(XtParent(w)) ? "Armed" : "Not Armed",
 	RC_PopupPosted(XtParent(w)) ? XtName(RC_PopupPosted(XtParent(w))) : "nothing",
 	CB_Submenu(w) ? XtName(CB_Submenu(w)) : "nothing"
@@ -1065,19 +1063,20 @@ MenuBarEnter(Widget w, XEvent *event, String *params, Cardinal *num_params)
     }
 }
 
+
 static void
 MenuBarLeave(Widget w, XEvent *event, String *params, Cardinal *num_params)
 {
-    DEBUGOUT(XdbDebug(__FILE__, w, "MenuBarLeave\n\t%s %s %s %s posted mine %s\n",
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "MenuBarLeave\n\t%s %s %s %s posted mine %s\n",
 	_XmGetInDragMode(w) ? "Dragging" : "Not Dragging",
-	XdbRcType2String(Lab_MenuType(w)),
+	_LtDebugRcType2String(Lab_MenuType(w)),
 	RC_IsArmed(XtParent(w)) ? "Armed" : "Not Armed",
 	RC_PopupPosted(XtParent(w)) ? XtName(RC_PopupPosted(XtParent(w))) : "nothing",
 	CB_Submenu(w) ? XtName(CB_Submenu(w)) : "nothing"
 	));
-    DEBUGOUT(XdbDebug("MENU", w, "MenuBarLeave\n\t%s %s %s %s posted mine %s\n",
+    DEBUGOUT(_LtDebugDebug("MENU", w, "MenuBarLeave\n\t%s %s %s %s posted mine %s\n",
 	_XmGetInDragMode(w) ? "Dragging" : "Not Dragging",
-	XdbRcType2String(Lab_MenuType(w)),
+	_LtDebugRcType2String(Lab_MenuType(w)),
 	RC_IsArmed(XtParent(w)) ? "Armed" : "Not Armed",
 	RC_PopupPosted(XtParent(w)) ? XtName(RC_PopupPosted(XtParent(w))) : "nothing",
 	CB_Submenu(w) ? XtName(CB_Submenu(w)) : "nothing"
@@ -1102,13 +1101,14 @@ MenuBarLeave(Widget w, XEvent *event, String *params, Cardinal *num_params)
     }
 }
 
+
 static void
 StartDrag(Widget w, XEvent *event, String *params, Cardinal *num_params)
 {
     Boolean validButton;
     Boolean poppedUp;
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "StartDrag()\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "StartDrag()\n"));
 
     /* queue events up until the next button event. */
     XAllowEvents(XtDisplay(w), SyncPointer, CurrentTime);
@@ -1128,7 +1128,7 @@ StartDrag(Widget w, XEvent *event, String *params, Cardinal *num_params)
     {
 	if (CB_Submenu(w) && RC_PopupPosted(XtParent(w)) == CB_Submenu(w))
 	{
-	int i;
+	Cardinal i;
 
 	    for (i = 0; i < MGR_NumChildren(CB_Submenu(w)); i++)
 	    {
@@ -1140,7 +1140,7 @@ StartDrag(Widget w, XEvent *event, String *params, Cardinal *num_params)
 	    RC_MenuShellPopdown(w, event, &poppedUp);
 	    _XmCascadingPopup(w, event, True);
 	    {
-	    int i;
+	    Cardinal i;
 	    Widget menu = XtParent(w);
 
 		for (i = 0; i < MGR_NumChildren(menu); i++)
@@ -1155,22 +1155,24 @@ StartDrag(Widget w, XEvent *event, String *params, Cardinal *num_params)
     }
 }
 
+
 static void
 CascadePopupHandler(XtPointer clientData, XtIntervalId *id)
 {
     Widget w = (Widget)clientData;
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "CascadePopupHandler\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "CascadePopupHandler\n"));
 
     CB_Timer(w) = 0;
 
     _XmCascadingPopup(w, NULL, True);	/* FIX ME: NULL? */
 }
 
+
 static void
 DelayedArm(Widget w, XEvent *event, String *params, Cardinal *num_params)
 {
-    DEBUGOUT(XdbDebug(__FILE__, w, "DelayedArm()\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "DelayedArm()\n"));
 
     /* Signal server to release next pointer event */
     XAllowEvents(XtDisplay(w), SyncPointer, CurrentTime);
@@ -1195,6 +1197,7 @@ DelayedArm(Widget w, XEvent *event, String *params, Cardinal *num_params)
     }
 }
 
+
 static void
 CheckDisarm(Widget w, XEvent *event, String *params, Cardinal *num_params)
 {
@@ -1202,7 +1205,7 @@ CheckDisarm(Widget w, XEvent *event, String *params, Cardinal *num_params)
     int y = ((XLeaveWindowEvent *)event)->y_root;
     Widget subpane;
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "CheckDisarm()\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "CheckDisarm()\n"));
 
     XAllowEvents(XtDisplay(w), SyncPointer, CurrentTime); /* NEW */
 
@@ -1234,10 +1237,11 @@ CheckDisarm(Widget w, XEvent *event, String *params, Cardinal *num_params)
     }
 }
 
-void
+
+extern void
 _XmCBHelp(Widget w, XEvent *event, String *params, Cardinal *num_params)
 {
-    DEBUGOUT(XdbDebug(__FILE__, w, "_XmCBHelp\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "_XmCBHelp\n"));
 
     /* clean up things */
 
@@ -1245,10 +1249,11 @@ _XmCBHelp(Widget w, XEvent *event, String *params, Cardinal *num_params)
     XtCallActionProc(w, "PrimitiveHelp", event, params, *num_params);
 }
 
+
 static void
 ArmAndActivate(Widget w, XEvent *event, String *params, Cardinal *num_params)
 {
-    DEBUGOUT(XdbDebug(__FILE__, w, "%s:ArmAndActivate()\n",
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "%s:ArmAndActivate()\n",
 		__FILE__));
     if (Lab_MenuType(w) == XmMENU_BAR)
     {
@@ -1264,28 +1269,17 @@ ArmAndActivate(Widget w, XEvent *event, String *params, Cardinal *num_params)
     }
     else /* pullright */
     {
-	CascadePopupHandler(w, NULL);
+	CascadePopupHandler((XtPointer)w, NULL);
     }
 }
 
 
-#if STDC_HEADERS
 static void
-MenuProcEntry(int proc, Widget w,...)
+MenuProcEntry(int proc, Widget w, ...)
 {
     va_list arg_list;
 
     va_start(arg_list, w);
-#else
-MenuProcEntry(proc, w, va_alist)
-     int proc;
-     Widget w;
-     va_dcl
-{
-    va_list arglist;
-
-    va_start(arglist);
-#endif
 
     switch (proc)
     {
@@ -1318,10 +1312,11 @@ MenuProcEntry(proc, w, va_alist)
     va_end(arg_list);
 }
 
-void
+
+extern void
 XmCascadeButtonHighlight(Widget w, Boolean highlight)
 {
-    DEBUGOUT(XdbDebug(__FILE__, w,
+    DEBUGOUT(_LtDebugDebug(__FILE__, w,
 	     "XmCascadeButtonHighlight(hl %d, armed %d, apm 0x%X, cpm 0x%X)\n",
 		      highlight,
 		      XmIsPrimitive(w) ? CB_IsArmed(w) : CBG_IsArmed(w),
@@ -1355,7 +1350,8 @@ XmCascadeButtonHighlight(Widget w, Boolean highlight)
     }
 }
 
-Widget
+
+extern Widget
 XmCreateCascadeButton(Widget parent, char *name,
 		      Arg *arglist, Cardinal argcount)
 {
