@@ -1,8 +1,9 @@
 /**
  *
- * $Id: CascadeBG.c,v 1.132 1999/11/19 00:58:35 rwscott Exp $
+ * $Id: CascadeBG.c,v 1.144 2001/05/20 17:49:38 amai Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
+ * Copyright (C) 1995-2001 LessTif Development Team
  *
  * This file is part of the GNU LessTif Library.
  *
@@ -22,11 +23,14 @@
  *
  **/
 
-static const char rcsid[] = "$Id: CascadeBG.c,v 1.132 1999/11/19 00:58:35 rwscott Exp $";
+static const char rcsid[] = "$Id: CascadeBG.c,v 1.144 2001/05/20 17:49:38 amai Exp $";
 
 #include <LTconfig.h>
-#include <XmI/XmI.h>
 
+#include <stdarg.h>
+#include <string.h>
+
+#include <XmI/XmI.h>
 #include <Xm/XmP.h>
 #include <Xm/CacheP.h>
 #include <Xm/CascadeB.h>	/* For XmCascadeButtonHighlight */
@@ -37,13 +41,6 @@ static const char rcsid[] = "$Id: CascadeBG.c,v 1.132 1999/11/19 00:58:35 rwscot
 #include <Xm/RowColumnP.h>
 #include <Xm/ScreenP.h>
 #include <Xm/MenuUtilP.h>
-#include <X11/Xfuncs.h>
-
-#if STDC_HEADERS
-#include <stdarg.h>
-#else
-#include <varargs.h>
-#endif
 
 #include <XmI/DebugUtil.h>
 
@@ -105,6 +102,8 @@ static void ArmAndActivate(Widget w, XEvent *event,
 
 static void MenuProcEntry(int proc, Widget rc,...);
 
+static void CascadePopupHandler(XtPointer clientData, XtIntervalId *id);
+
 /*
  * cache resources
  */
@@ -124,7 +123,6 @@ static XtResource cache_resources[] =
     }
 };
 
-/* *INDENT-OFF* */
 XmCascadeButtonGCacheObjClassRec xmCascadeButtonGCacheObjClassRec = {
     /* Object class part */
     {
@@ -337,7 +335,6 @@ XmCascadeButtonGadgetClassRec xmCascadeButtonGadgetClassRec = {
 	/* extension */ NULL
     },
 };
-/* *INDENT-ON* */
 
 
 WidgetClass xmCascadeButtonGadgetClass =
@@ -351,9 +348,9 @@ secondary_object_create(Widget request, Widget new_w,
     XmBaseClassExt *bce;
     XtPointer nsec, rsec;
     XmWidgetExtData ed;
-    int size;
+    Cardinal size;
 
-    DEBUGOUT(XdbDebug(__FILE__, new_w,
+    DEBUGOUT(_LtDebugDebug(__FILE__, new_w,
 		      "CascadeButtonGCacheRec %s being initialized.\n",
 		      XtName(new_w)));
 
@@ -383,7 +380,7 @@ secondary_object_create(Widget request, Widget new_w,
     ed->widget = (Widget)nsec;
     ed->reqWidget = (Widget)rsec;
 
-    bcopy(nsec, rsec, size);
+    memcpy(rsec, nsec, size);
 
     ((XmExtRec *)rsec)->object.self = (Widget)rsec;
 
@@ -401,7 +398,7 @@ secondary_object_create(Widget request, Widget new_w,
 int
 _XmCascadeBCacheCompare(XtPointer A, XtPointer B)
 {
-    return !bcmp(((XmCascadeButtonGCacheObjPart *)A),
+    return !memcmp(((XmCascadeButtonGCacheObjPart *)A),
 		 ((XmCascadeButtonGCacheObjPart *)B),
 		 sizeof(XmCascadeButtonGCacheObjPart));
 }
@@ -417,7 +414,7 @@ _XmArrowPixmapCacheCompare(XtPointer A, XtPointer B)
      * different shadow thicknesses imply different pixmap sizes (at least
      * for the same size font).
      */
-    return !bcmp(A, B, XtOffsetOf(XmArrowPixmap, pixmap));
+    return !memcmp(A, B, XtOffsetOf(XmArrowPixmap, pixmap));
 }
 
 void
@@ -440,7 +437,7 @@ _XmArrowPixmapCacheDelete(XtPointer data)
 			(Pixmap)data);
 		    list->prev->next = list->next;
 		    list->next->prev = list->prev;
-		    XtFree((XtPointer)list);
+		    XtFree((char *)list);
 		}
 		return;
 	    }
@@ -512,12 +509,12 @@ _XmCreateArrowPixmaps(Widget w)
     cpart.top_shadow_color = ts;
     cpart.bottom_shadow_color = bs;
     cp = (XmArrowPixmap *)
-	_XmCachePart(&arrow_pixmap_cache, &cpart, sizeof cpart);
+	_XmCachePart(&arrow_pixmap_cache, (XtPointer)&cpart, sizeof cpart);
 
     cpart.top_shadow_color = bs;
     cpart.bottom_shadow_color = ts;
     acp = (XmArrowPixmap *)
-	_XmCachePart(&arrow_pixmap_cache, &cpart, sizeof cpart);
+	_XmCachePart(&arrow_pixmap_cache, (XtPointer)&cpart, sizeof cpart);
 
     /* If the cached pixmaps don't exist, create them */
     if (cp->pixmap == XmUNSPECIFIED_PIXMAP
@@ -602,10 +599,10 @@ class_initialize(void)
 
     combined = (XtResourceList)XtMalloc(sizeof(XtResource) * ncom);
 
-    bcopy(labels, combined, nlabels * sizeof(XtResource));
-    bcopy(cache_resources,
-	  &combined[nlabels],
-	  XtNumber(cache_resources) * sizeof(XtResource));
+    memcpy(combined, labels, nlabels * sizeof(XtResource));
+    memcpy(&combined[nlabels],
+           cache_resources,
+ 	   XtNumber(cache_resources) * sizeof(XtResource));
 
     XtFree((char *)labels);
 
@@ -625,7 +622,7 @@ initialize_posthook(Widget request, Widget new_w,
 {
     XmWidgetExtData ext;
 
-    DEBUGOUT(XdbDebug(__FILE__, new_w, "CascadeBG InitializePosthook\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, new_w, "CascadeBG InitializePosthook\n"));
 
     /* don't let the null fool you */
     LabG_Cache(new_w) = (XmLabelGCacheObjPart *)
@@ -856,10 +853,10 @@ resize(Widget w)
 static void
 destroy(Widget w)
 {
-    _XmCacheDelete((XtPointer)CBG_Cache(w));
     if (CBG_Timer(w) != 0)
     {
 	XtRemoveTimeOut(CBG_Timer(w));
+	CBG_Timer(w) = 0;
     }
 
     if (CBG_ArmedPixmap(w))
@@ -867,6 +864,8 @@ destroy(Widget w)
 	_XmArrowPixmapCacheDelete((XtPointer)CBG_ArmedPixmap(w));
 	_XmArrowPixmapCacheDelete((XtPointer)CBG_CascadePixmap(w));
     }
+
+   _XmCacheDelete((XtPointer)CBG_Cache(w));
 }
 
 static Boolean
@@ -875,7 +874,7 @@ set_values_prehook(Widget old, Widget request, Widget new_w,
 {
     XmBaseClassExt *bce;
     XmWidgetExtData ed;
-    int size;
+    Cardinal size;
     XtPointer nsec, rsec;
 
     bce = _XmGetBaseClassExtPtr(XtClass(new_w), XmQmotif);
@@ -895,12 +894,12 @@ set_values_prehook(Widget old, Widget request, Widget new_w,
     ExtObj_LogicalParent(nsec) = new_w;
     ExtObj_ExtensionType(nsec) = XmCACHE_EXTENSION;
 
-    bcopy(LabG_Cache(new_w),
-	  &((XmLabelGCacheObject)nsec)->label_cache,
+    memcpy(&((XmLabelGCacheObject)nsec)->label_cache,
+	  LabG_Cache(new_w),
 	  sizeof(XmLabelGCacheObjPart));
-    bcopy(CBG_Cache(new_w),
-	  &((XmCascadeButtonGCacheObject)nsec)->cascade_button_cache,
-	  sizeof(XmCascadeButtonGCacheObjPart));
+    memcpy(&((XmCascadeButtonGCacheObject)nsec)->cascade_button_cache,
+           CBG_Cache(new_w),
+	   sizeof(XmCascadeButtonGCacheObjPart));
 
     ed = (XmWidgetExtData)XtMalloc(sizeof(XmWidgetExtDataRec));
 
@@ -916,7 +915,7 @@ set_values_prehook(Widget old, Widget request, Widget new_w,
 		   (*bce)->secondaryObjectClass->core_class.num_resources,
 		   args, *num_args);
 
-    bcopy(nsec, rsec, size);
+    memcpy(rsec, nsec, size);
 
     LabG_Cache(new_w) = &(((XmCascadeButtonGCacheObject)nsec)->label_cache);
     LabG_Cache(request) = &(((XmCascadeButtonGCacheObject)rsec)->label_cache);
@@ -984,7 +983,7 @@ set_values(Widget old, Widget request, Widget new_w,
 {
     Boolean refresh_needed = False;
 
-    DEBUGOUT(XdbDebug(__FILE__, new_w, "set_values()\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, new_w, "set_values()\n"));
 
     if (CBG_MapDelay(new_w) < 0)
     {
@@ -1025,11 +1024,12 @@ set_values(Widget old, Widget request, Widget new_w,
 	    if (LabG_MenuType(new_w) != XmMENU_OPTION &&
 		CBG_CascadePixmap(new_w) == XmUNSPECIFIED_PIXMAP &&
 		CBG_Submenu(new_w))
+	    {
 		_XmCreateArrowPixmaps(new_w);
+	    }
 	    size_cascade(new_w);
+	    refresh_needed = True;
 	}
-
-	refresh_needed = True;
     }
 
     if (XtSensitive(new_w) != XtSensitive(old))
@@ -1142,7 +1142,7 @@ get_values_prehook(Widget new_w, ArgList args, Cardinal *num_args)
 {
     XmBaseClassExt *bce;
     XmWidgetExtData ed;
-    int size;
+    Cardinal size;
     XtPointer nsec;
 
     bce = _XmGetBaseClassExtPtr(XtClass(new_w), XmQmotif);
@@ -1150,12 +1150,12 @@ get_values_prehook(Widget new_w, ArgList args, Cardinal *num_args)
 
     nsec = _XmExtObjAlloc(size);
 
-    bcopy(LabG_Cache(new_w),
-	  &((XmLabelGCacheObject)nsec)->label_cache,
-	  sizeof(XmLabelGCacheObjPart));
-    bcopy(CBG_Cache(new_w),
-	  &((XmCascadeButtonGCacheObject)nsec)->cascade_button_cache,
-	  sizeof(XmCascadeButtonGCacheObjPart));
+    memcpy(&((XmLabelGCacheObject)nsec)->label_cache,
+           LabG_Cache(new_w),
+	   sizeof(XmLabelGCacheObjPart));
+    memcpy(&((XmCascadeButtonGCacheObject)nsec)->cascade_button_cache,
+           CBG_Cache(new_w),
+	   sizeof(XmCascadeButtonGCacheObjPart));
 
     /*
      * don't do this and ResInd will blow up.
@@ -1262,8 +1262,8 @@ expose(Widget w,
        XEvent *event,
        Region region)
 {
-    DEBUGOUT(XdbDebug(__FILE__, w, "expose() Armed(%d) type %s\n",
-		      CBG_IsArmed(w), XdbMenuType2String(LabG_MenuType(w))));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "expose() Armed(%d) type %s\n",
+		      CBG_IsArmed(w), _LtDebugMenuType2String(LabG_MenuType(w))));
 
     draw_cascade(w);
 
@@ -1286,16 +1286,16 @@ DoSelect(Widget w, XEvent *event, String *params, Cardinal *num_params)
 {
 Boolean validButton;
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "%s:DoSelect(%d)\n\t%s posted mine %s\n",
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "%s:DoSelect(%d)\n\t%s posted mine %s\n",
 	__FILE__, __LINE__,
 	RC_PopupPosted(XtParent(w)) ? XtName(RC_PopupPosted(XtParent(w))) : "nothing",
 	CBG_Submenu(w) ? XtName(CBG_Submenu(w)) : "none"
 	));
-    DEBUGOUT(XdbDebug("MENU", w, "%s:DoSelect(%d)\n\t%s posted mine %s %s\n",
+    DEBUGOUT(_LtDebugDebug("MENU", w, "%s:DoSelect(%d)\n\t%s posted mine %s %s\n",
 	__FILE__, __LINE__,
 	RC_PopupPosted(XtParent(w)) ? XtName(RC_PopupPosted(XtParent(w))) : "nothing",
 	CBG_Submenu(w) ? XtName(CBG_Submenu(w)) : "none",
-	event ? XdbEventType2String(event->xany.type) : "no-event"
+	event ? _LtDebugEventType2String(event->xany.type) : "no-event"
 	));
     if (event && event->xany.type == ButtonRelease)
     {
@@ -1352,12 +1352,12 @@ MenuBarSelect(Widget w, XEvent *event, String *params, Cardinal *num_params)
 {
     Boolean validButton;
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "%s:MenuBarSelect(%d)\n\t%s posted mine %s\n",
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "%s:MenuBarSelect(%d)\n\t%s posted mine %s\n",
 	__FILE__, __LINE__,
 	RC_PopupPosted(XtParent(w)) ? XtName(RC_PopupPosted(XtParent(w))) : "nothing",
 	CBG_Submenu(w) ? XtName(CBG_Submenu(w)) : "none"
 	));
-    DEBUGOUT(XdbDebug("MENU", w, "%s:MenuBarSelect(%d)\n\t%s posted mine %s\n",
+    DEBUGOUT(_LtDebugDebug("MENU", w, "%s:MenuBarSelect(%d)\n\t%s posted mine %s\n",
 	__FILE__, __LINE__,
 	RC_PopupPosted(XtParent(w)) ? XtName(RC_PopupPosted(XtParent(w))) : "nothing",
 	CBG_Submenu(w) ? XtName(CBG_Submenu(w)) : "none"
@@ -1376,7 +1376,7 @@ MenuBarSelect(Widget w, XEvent *event, String *params, Cardinal *num_params)
 	XmProcessTraversal(w, XmTRAVERSE_CURRENT);
 	if (RC_PopupPosted(XtParent(w)))
 	{
-	int i;
+	Cardinal i;
 
 	    for (i = 0; i < MGR_NumChildren(RC_PopupPosted(XtParent(w))); i++)
 	    {
@@ -1413,8 +1413,8 @@ MenuBarSelect(Widget w, XEvent *event, String *params, Cardinal *num_params)
 static void
 ArmAndPost(Widget w, XEvent *event, String *params, Cardinal *num_params)
 {
-    DEBUGOUT(XdbDebug(__FILE__, w, "ArmAndPost(): %p %p\n", w, CBG_Submenu(w)));
-    DEBUGOUT(XdbDebug("MENU", w, "%s:ArmAndPost(%d)\n\t%s posted mine %s\n",
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "ArmAndPost(): %p %p\n", w, CBG_Submenu(w)));
+    DEBUGOUT(_LtDebugDebug("MENU", w, "%s:ArmAndPost(%d)\n\t%s posted mine %s\n",
 	__FILE__, __LINE__,
 	RC_PopupPosted(XtParent(w)) ? XtName(RC_PopupPosted(XtParent(w))) : "nothing",
 	CBG_Submenu(w) ? XtName(CBG_Submenu(w)) : "none"
@@ -1425,7 +1425,7 @@ ArmAndPost(Widget w, XEvent *event, String *params, Cardinal *num_params)
 #if 0
     if (!RC_IsArmed(CBG_Submenu(w)))
     {
-	DEBUGOUT(XdbDebug(__FILE__, w, "MENU_ARM\n"));
+	DEBUGOUT(_LtDebugDebug(__FILE__, w, "MENU_ARM\n"));
 
 	RCClass_MenuProcs(XtClass(XtParent(w)))(XmMENU_ARM, XtParent(w), NULL);
     }
@@ -1444,16 +1444,16 @@ ArmAndPost(Widget w, XEvent *event, String *params, Cardinal *num_params)
 static void
 MenuBarEnter(Widget w, XEvent *event, String *params, Cardinal *num_params)
 {
-    DEBUGOUT(XdbDebug(__FILE__, w, "MenuBarEnter\n\t%s %s %s %s posted posting %s\n",
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "MenuBarEnter\n\t%s %s %s %s posted posting %s\n",
 	_XmGetInDragMode(w) ? "Dragging" : "Not Dragging",
-	XdbRcType2String(LabG_MenuType(w)),
+	_LtDebugRcType2String(LabG_MenuType(w)),
 	RC_IsArmed(XtParent(w)) ? "Armed" : "Not Armed",
 	RC_PopupPosted(XtParent(w)) ? XtName(RC_PopupPosted(XtParent(w))) : "nothing",
 	CBG_Submenu(w) ? XtName(CBG_Submenu(w)) : "nothing"
 	));
-    DEBUGOUT(XdbDebug("MENU", w, "MenuBarEnter\n\t%s %s %s %s posted posting %s\n",
+    DEBUGOUT(_LtDebugDebug("MENU", w, "MenuBarEnter\n\t%s %s %s %s posted posting %s\n",
 	_XmGetInDragMode(w) ? "Dragging" : "Not Dragging",
-	XdbRcType2String(LabG_MenuType(w)),
+	_LtDebugRcType2String(LabG_MenuType(w)),
 	RC_IsArmed(XtParent(w)) ? "Armed" : "Not Armed",
 	RC_PopupPosted(XtParent(w)) ? XtName(RC_PopupPosted(XtParent(w))) : "nothing",
 	CBG_Submenu(w) ? XtName(CBG_Submenu(w)) : "nothing"
@@ -1533,7 +1533,7 @@ StartDrag(Widget w, XEvent *event, String *params, Cardinal *num_params)
     Boolean validButton;
     Boolean poppedUp;
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "StartDrag()\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "StartDrag()\n"));
 
     XAllowEvents(XtDisplay(w), SyncPointer, CurrentTime);
 
@@ -1551,7 +1551,7 @@ StartDrag(Widget w, XEvent *event, String *params, Cardinal *num_params)
     /* Was it the right button? */
     RC_MenuButton(w, event, &validButton);
 
-    DEBUGOUT(XdbDebug("MENU", w, "%s:StartDrag(%d) - %s posted mine %s\n",
+    DEBUGOUT(_LtDebugDebug("MENU", w, "%s:StartDrag(%d) - %s posted mine %s\n",
 	__FILE__, __LINE__,
 	RC_PopupPosted(XtParent(w)) ? XtName(RC_PopupPosted(XtParent(w))) : "None",
 	CBG_Submenu(w) ? XtName(CBG_Submenu(w)) : "None"
@@ -1560,7 +1560,7 @@ StartDrag(Widget w, XEvent *event, String *params, Cardinal *num_params)
     {
 	if (CBG_Submenu(w) && RC_PopupPosted(XtParent(w)) == CBG_Submenu(w))
 	{
-	int i;
+	Cardinal i;
 
 	    for (i = 0; i < MGR_NumChildren(CBG_Submenu(w)); i++)
 	    {
@@ -1572,7 +1572,7 @@ StartDrag(Widget w, XEvent *event, String *params, Cardinal *num_params)
 	    RC_MenuShellPopdown(w, event, &poppedUp);
 	    _XmCascadingPopup(w, event, True);
 	    {
-	    int i;
+	    Cardinal i;
 	    Widget menu = XtParent(w);
 
 		for (i = 0; i < MGR_NumChildren(menu); i++)
@@ -1594,7 +1594,7 @@ CascadePopupHandler(XtPointer clientData, XtIntervalId *id)
 
     CBG_Timer(w) = 0;
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "CascadePopupHandler()\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "CascadePopupHandler()\n"));
 
     _XmCascadingPopup(w, NULL, True);	/* FIX ME: NULL? */
 }
@@ -1603,7 +1603,7 @@ CascadePopupHandler(XtPointer clientData, XtIntervalId *id)
 static void
 DelayedArm(Widget w, XEvent *event, String *params, Cardinal *num_params)
 {
-    DEBUGOUT(XdbDebug(__FILE__, w, "DelayedArm()\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "DelayedArm()\n"));
 
     XAllowEvents(XtDisplay(w), SyncPointer, CurrentTime);
 
@@ -1634,7 +1634,7 @@ CheckDisarm(Widget w, XEvent *event, String *params, Cardinal *num_params)
     int y = ((XLeaveWindowEvent *)event)->y_root;
     Widget subpane;
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "CheckDisarm() - %i %i %i\n", x, y, XtHeight(w)));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "CheckDisarm() - %i %i %i\n", x, y, XtHeight(w)));
 
     XAllowEvents(XtDisplay(w), SyncPointer, CurrentTime);
 
@@ -1690,7 +1690,7 @@ Help(Widget w, XEvent *event, String *params, Cardinal *num_params)
 static void
 ArmAndActivate(Widget w, XEvent *event, String *params, Cardinal *num_params)
 {
-    DEBUGOUT(XdbDebug(__FILE__, w, "ArmAndActivate\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "ArmAndActivate\n"));
 
     MGR_ActiveChild(XtParent(w)) = w;
     if (LabG_MenuType(w) == XmMENU_BAR)
@@ -1711,17 +1711,17 @@ ArmAndActivate(Widget w, XEvent *event, String *params, Cardinal *num_params)
 
 	if (CBG_Submenu(w) && !RC_IsArmed(CBG_Submenu(w)))
 	{
-	    DEBUGOUT(XdbDebug(__FILE__, w, "MENU_ARM\n"));
+	    DEBUGOUT(_LtDebugDebug(__FILE__, w, "MENU_ARM\n"));
 
 	    RCClass_MenuProcs(XtClass(XtParent(w)))(XmMENU_ARM,
 						    XtParent(w), NULL);
 	}
 
-	CascadePopupHandler(w, NULL);
+	CascadePopupHandler((XtPointer)w, NULL);
     }
     else /* pullright */
     {
-	CascadePopupHandler(w, NULL);
+	CascadePopupHandler((XtPointer)w, NULL);
     }
 }
 
@@ -1802,7 +1802,6 @@ visual_change(Widget w,
     return False;
 }
 
-#if STDC_HEADERS
 static void
 MenuProcEntry(int proc, Widget w,...)
 {
@@ -1812,19 +1811,6 @@ MenuProcEntry(int proc, Widget w,...)
     */
 
     va_start(arg_list, w);
-#else
-MenuProcEntry(proc, w, va_alist)
-     int proc;
-     Widget w;
-     va_dcl
-{
-    va_list arglist;
-    /*
-    XEvent *event = NULL;
-    */
-
-    va_start(arglist);
-#endif
 
     switch (proc)
     {
@@ -1862,14 +1848,14 @@ MenuProcEntry(proc, w, va_alist)
 void
 XmCascadeButtonGadgetHighlight(Widget w, Boolean highlight)
 {
-    DEBUGOUT(XdbDebug(__FILE__, w,
+    DEBUGOUT(_LtDebugDebug(__FILE__, w,
 		      "XmCascadeButtonGadgetHighlight(hl %d, armed %d,"
 		      " apm 0x%X, cpm 0x%X), %s, geo %d %d %dx%d\n",
 		      highlight,
 		      XmIsPrimitive(w) ? CB_IsArmed(w) : CBG_IsArmed(w),
 		  XmIsPrimitive(w) ? CB_ArmedPixmap(w) : CBG_ArmedPixmap(w),
 	      XmIsPrimitive(w) ? CB_CascadePixmap(w) : CBG_CascadePixmap(w),
-		      XdbMenuType2String(LabG_MenuType(w)),
+		      _LtDebugMenuType2String(LabG_MenuType(w)),
 		      XtX(w), XtY(w), XtWidth(w), XtHeight(w)
 	     ));
 
