@@ -1,8 +1,9 @@
 /**
  *
- * $Id: CutPaste.c,v 1.9 1999/06/13 08:33:33 danny Exp $
+ * $Id: CutPaste.c,v 1.22 2001/05/14 19:46:08 amai Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
+ * Copyright (C) 1995-2001 LessTif Development Team
  *
  * This file is part of the GNU LessTif Library.
  *
@@ -22,22 +23,24 @@
  *
  **/
 
-static const char rcsid[] = "$Id: CutPaste.c,v 1.9 1999/06/13 08:33:33 danny Exp $";
+static const char rcsid[] = "$Id: CutPaste.c,v 1.22 2001/05/14 19:46:08 amai Exp $";
 
 #include <LTconfig.h>
-#include <XmI/XmI.h>
 
-#include <X11/Xfuncs.h>
+#include <stdio.h>
+#include <string.h>
+
+#include <XmI/XmI.h>
 
 #include <Xm/XmP.h>
 #include <Xm/AtomMgr.h>
 #include <Xm/CutPaste.h>
 #include <XmI/CutPasteI.h>
 #include <XmI/AtomMgrI.h>
-#include <stdio.h>
 
 #include <XmI/DebugUtil.h>
 
+
 #define UNLOCKED	0
 #define LOCKED		1
 
@@ -52,51 +55,35 @@ static const char rcsid[] = "$Id: CutPaste.c,v 1.9 1999/06/13 08:33:33 danny Exp
  * internal functions
  */
 static Atom _XmClipboardGetAtomFromId(Display *display, int id);
-
 static void _XmClipboardDeleteId(Display *display, int id);
-
 static void _XmClipboardDeleteItemLabel(Display *display, Window window,
 					int id);
-
 static void _XmClipboardDeleteMarked(Display *display, Window window,
 				     XmClipboard * clip);
-
 static void _XmClipboardDeleteFormat(Display *display, int id);
-
 static int _XmClipboardRegisterFormat(Display *display, char *format_name,
 				      int format_len);
-
 static void _XmClipboardDeleteFormats(Display *display, Window window, int id);
-
 static void _XmClipboardDeleteItem(Display *display, Window window,
 				   XmClipboard * clip, unsigned item);
-
 static int _XmClipboardGetWindowProperty(Display *display, Window win,
 					 Atom item, unsigned **data, int *len,
 					 int *type_return, int *format_return,
-					 Boolean delete);
-
+					 Boolean deleteflag);
 static Boolean _XmClipboardSearchForWindow(Display *display, Window root,
 					   Window window);
-
 static Time _XmClipboardGetCurrentTime(Display *display);
-
 static Atom _XmClipboardGetAtomFromFormat(Display *display, char *format_name);
-
 static Boolean _XmClipboardGetLenFromFormat(Display *display,
 					    char *format_name,
 					    int *format);
-
 static Boolean _XmClipboardWindowExists(Display *display, Window window);
-
 static int _XmClipboardFindItem(Display *display, int id,
 				unsigned **data, int *len,
 				int *format_return, unsigned new_item);
-
 static Boolean _XmClipboardIsMarkedForDelete(Display *display,
 					     XmClipboard * clip,
 					     int item);
-
 static XmClipboardFormat *_XmClipboardFindFormat(Display *display,
 						 XmClipboard * clip,
 						 char *format_name,
@@ -105,60 +92,38 @@ static XmClipboardFormat *_XmClipboardFindFormat(Display *display,
 						 int *namelen,
 						 int *num_formats,
 						 int *format_len);
-
 static void _XmClipboardReplaceItem(Display *display, int id,
 				    unsigned *data, int len,
 				    int mode, int format, Boolean free);
-
 static int _XmClipboardRetrieveItem(Display *display, int id,
 				    int extra_size, int size,
 				    unsigned **data, int *len,
 				    int *format_return,
 				    unsigned new_item, Boolean truncate);
-
 static Boolean _XmClipboardSendMessage(Display *display, Window window,
 				       XmClipboardFormat * format, int id);
-
 static void _XmAssertClipboardSelection(Display *display, Window window,
 					XmClipboard * clip, Time locktime);
-
 static Boolean _XmClipboardGetSelection(Display *display, Window window,
 					char *format_name, XmClipboard * clip,
 					unsigned **data, int *len);
-
 static void _XmClipboardMarkItem(Display *display, XmClipboard * clip,
 				 unsigned id, Boolean delete_on);
-
 static void _XmClipboardSetNextItemId(Display *display, int item_id);
-
 static unsigned _XmClipboardGetNewItemId(Display *display);
-
 static void _XmClipboardSetAccess(Display *display, Window window);
-
 static XmClipboard *_XmClipboardOpen(Display *display, int extra_size);
-
 static void _XmClipboardClose(Display *display, XmClipboard * clip);
-
 static int _XmClipboardLock(Display *display, Window window);
-
 static int _XmClipboardUnlock(Display *display, Window window,
 			      Boolean remove_all_locks);
-
 static void _XmClipboardEventHandler(Widget widget, XtPointer client_data,
 				     XEvent *event, Boolean *dontSwallow);
-
 static Window _XmClipboardInitializeSelection(Display *display,
 					      XmClipboard * clip,
 					      Window window, Time locktime);
-
 static Boolean _XmClipboardWeOwnSelection(Display *display, XmClipboard * clip);
 
-/* And one external function for the prototype police */
-
-extern int XmClipboardWithdrawFormat(Display *display, Window window,
-				     int data_id);
-
-
 
 /******************************* low level ******************************/
 
@@ -167,6 +132,8 @@ _XmClipboardGetAtomFromId(Display *display, int id)
 {
     char buf[128];
 
+    DEBUGOUT(_LtDebugDebug(__FILE__, NULL, "_XmClipboardGetAtomFromId(id=%i)\n",
+                      id));
     if (id == XmCLIP_HEADER)
     {
 	return XmInternAtom(display, _XA_MOTIF_CLIP_HEADER, False);
@@ -181,6 +148,7 @@ _XmClipboardGetAtomFromId(Display *display, int id)
     return XmInternAtom(display, buf, False);
 }
 
+
 static Boolean
 _XmClipboardIsMarkedForDelete(Display *display, XmClipboard * clip, int item)
 {
@@ -215,6 +183,7 @@ _XmClipboardIsMarkedForDelete(Display *display, XmClipboard * clip, int item)
     return ret;
 }
 
+
 static void
 _XmClipboardDeleteId(Display *display, int id)
 {
@@ -225,6 +194,7 @@ _XmClipboardDeleteId(Display *display, int id)
     XDeleteProperty(display, DefaultRootWindow(display), atom);
 }
 
+
 static void
 _XmClipboardDeleteItemLabel(Display *display, Window window, int id)
 {
@@ -260,6 +230,7 @@ _XmClipboardDeleteItemLabel(Display *display, Window window, int id)
     XtFree((char *)item);
 }
 
+
 static int
 _XmClipboardRegisterFormat(Display *display, char *format_name, int format_len)
 {
@@ -286,6 +257,7 @@ _XmClipboardRegisterFormat(Display *display, char *format_name, int format_len)
     return XmClipboardSuccess;
 }
 
+
 static void
 _XmClipboardDeleteFormat(Display *display, int id)
 {
@@ -351,6 +323,7 @@ _XmClipboardDeleteFormat(Display *display, int id)
 			    32, True);
 }
 
+
 static void
 _XmClipboardDeleteFormats(Display *display, Window window, int id)
 {
@@ -359,7 +332,7 @@ _XmClipboardDeleteFormats(Display *display, Window window, int id)
     Atom hdr;
     XmClipboardItem *item;
 
-    DEBUGOUT(XdbDebug(__FILE__, NULL, "_XmClipboardDeleteFormats\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, NULL, "_XmClipboardDeleteFormats\n"));
 
     _XmClipboardFindItem(display, id, &data, &len, NULL, XmCLIP_PROP_ITEM);
 
@@ -419,6 +392,7 @@ _XmClipboardDeleteFormats(Display *display, Window window, int id)
     XtFree((char *)data);
 }
 
+
 static void
 _XmClipboardDeleteItem(Display *display, Window window,
 		       XmClipboard * clip, unsigned item)
@@ -528,6 +502,7 @@ _XmClipboardDeleteItem(Display *display, Window window,
     _XmClipboardDeleteId(display, item);
 }
 
+
 static void
 _XmClipboardDeleteMarked(Display *display, Window window, XmClipboard * clip)
 {
@@ -546,13 +521,14 @@ _XmClipboardDeleteMarked(Display *display, Window window, XmClipboard * clip)
     }
 }
 
+
 static int
 _XmClipboardGetWindowProperty(Display *display, Window win, Atom item,
 			      unsigned **data, int *len,
 			      int *type_return, int *format_return,
-			      Boolean delete)
+			      Boolean deleteflag)
 {
-    int transfersize;
+    long transfersize;
     long offset;
     Atom actual_type;
     int actual_format;
@@ -560,19 +536,20 @@ _XmClipboardGetWindowProperty(Display *display, Window win, Atom item,
     unsigned long bytes_after = 1;
     unsigned char *prop;
     unsigned *ret_buf;
+    Bool req_delete=False;
 
     transfersize = XMaxRequestSize(display);
     *data = NULL;
     *len = 0;
 
-    ret_buf = (unsigned *)XtMalloc(8);		/* FIX ME : 8 : what's this ? */
+    ret_buf = (unsigned *)XtMalloc(8); /* minimum size */
     ret_buf[0] = 0;
     offset = 0;
     alloc_size = 0;
 
     while (bytes_after)
     {
-	if (XGetWindowProperty(display, win, item, offset, transfersize, False,
+	if (XGetWindowProperty(display, win, item, offset, transfersize, req_delete,
 			       AnyPropertyType, &actual_type, &actual_format,
 			       &nitems, &bytes_after, &prop) != Success)
 	{
@@ -583,7 +560,7 @@ _XmClipboardGetWindowProperty(Display *display, Window win, Atom item,
 
 	if (prop == NULL || nitems == 0)
 	{
-	    if (delete)
+	    if (deleteflag)
 	    {
 		XDeleteProperty(display, win, item);
 	    }
@@ -594,7 +571,7 @@ _XmClipboardGetWindowProperty(Display *display, Window win, Atom item,
 
 	    XtFree((char *)ret_buf);
 
-	    DEBUGOUT(XdbDebug(__FILE__, NULL,
+	    DEBUGOUT(_LtDebugDebug(__FILE__, NULL,
 		"_XmClipboardGetWindowProperty: XmClipboardFail (alloc_size %d)\n",
 			alloc_size));
 	    return XmClipboardFail;
@@ -618,7 +595,7 @@ _XmClipboardGetWindowProperty(Display *display, Window win, Atom item,
 
 	ret_buf = (unsigned *)XtRealloc((char *)ret_buf,
 					alloc_size + alloc_incr + 1);
-	bcopy(prop, &ret_buf[offset], alloc_incr);
+	memcpy(&ret_buf[offset], prop, alloc_incr);
 	alloc_size += alloc_incr;
 
 	switch (actual_format)
@@ -645,7 +622,7 @@ _XmClipboardGetWindowProperty(Display *display, Window win, Atom item,
     *data = ret_buf;
     *len = alloc_size;
 
-    if (delete)
+    if (deleteflag)
     {
 	XDeleteProperty(display, win, item);
     }
@@ -660,12 +637,13 @@ _XmClipboardGetWindowProperty(Display *display, Window win, Atom item,
 	*type_return = actual_type;
     }
 
-    DEBUGOUT(XdbDebug(__FILE__, NULL,
+    DEBUGOUT(_LtDebugDebug(__FILE__, NULL,
 	"_XmClipboardGetWindowProperty: XmClipboardSuccess (alloc_size %d)\n",
 		alloc_size));
     return XmClipboardSuccess;
 }
 
+
 static Boolean
 _XmClipboardSearchForWindow(Display *display, Window root, Window window)
 {
@@ -707,6 +685,7 @@ _XmClipboardSearchForWindow(Display *display, Window root, Window window)
     return found;
 }
 
+
 static Time
 _XmClipboardGetCurrentTime(Display *display)
 {
@@ -728,6 +707,7 @@ _XmClipboardGetCurrentTime(Display *display)
     return event.xproperty.time;
 }
 
+
 static Bool
 _XmClipboardDataIsReady(Display *display, XEvent *event, XPointer arg)
 {
@@ -758,6 +738,7 @@ _XmClipboardDataIsReady(Display *display, XEvent *event, XPointer arg)
     return False;
 }
 
+
 static Boolean
 _XmClipboardRequestDataAndWait(Display *display, Window win,
 			       XmClipboardFormat * format)
@@ -768,7 +749,7 @@ _XmClipboardRequestDataAndWait(Display *display, Window win,
     XmMatchFormat arg;
     int status;
 
-    DEBUGOUT(XdbDebug(__FILE__, NULL, "_XmClipboardRequestDataAndWait\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, NULL, "_XmClipboardRequestDataAndWait\n"));
 
     XGetWindowAttributes(display, root, &attr);
 
@@ -785,7 +766,7 @@ _XmClipboardRequestDataAndWait(Display *display, Window win,
     arg.id = format->format_id;
 
     status = XCheckIfEvent(display, &event, _XmClipboardDataIsReady,
-			   (XtPointer)&arg);
+			   (XPointer)&arg);
 
     if (arg.window == None)
     {
@@ -807,6 +788,7 @@ _XmClipboardRequestDataAndWait(Display *display, Window win,
     return True;
 }
 
+
 static Boolean
 _XmClipboardWindowExists(Display *display, Window window)
 {
@@ -849,6 +831,7 @@ _XmClipboardWindowExists(Display *display, Window window)
     return found;
 }
 
+
 static int
 _XmClipboardFindItem(Display *display, int id, unsigned **data, int *len,
 		     int *format_return, unsigned prop_id)
@@ -889,6 +872,7 @@ _XmClipboardFindItem(Display *display, int id, unsigned **data, int *len,
     return XmClipboardFail;
 }
 
+
 static XmClipboardFormat *
 _XmClipboardFindFormat(Display *display, XmClipboard * clip,
 		       char *format_name, long item_id, int index,
@@ -1042,6 +1026,7 @@ _XmClipboardFindFormat(Display *display, XmClipboard * clip,
     return format;
 }
 
+
 static void
 _XmClipboardReplaceItem(Display *display, int id, unsigned *data, int len,
 			int mode, int format, Boolean free)
@@ -1102,6 +1087,7 @@ _XmClipboardReplaceItem(Display *display, int id, unsigned *data, int len,
     }
 }
 
+
 static int
 _XmClipboardRetrieveItem(Display *display, int id, int extra_size, int size,
 			 unsigned **data, int *len, int *format_return,
@@ -1131,7 +1117,7 @@ _XmClipboardRetrieveItem(Display *display, int id, int extra_size, int size,
 
     if (status == XmClipboardSuccess)
     {
-	bcopy(ldata, buf, *len);
+	memcpy(buf, ldata, *len);
     }
 
     *data = buf;
@@ -1145,6 +1131,7 @@ _XmClipboardRetrieveItem(Display *display, int id, int extra_size, int size,
     return status;
 }
 
+
 static unsigned
 _XmClipboardGetNewItemId(Display *display)
 {
@@ -1162,6 +1149,7 @@ _XmClipboardGetNewItemId(Display *display)
     return ret;
 }
 
+
 static Atom
 _XmClipboardGetAtomFromFormat(Display *display, char *format_name)
 {
@@ -1179,6 +1167,7 @@ _XmClipboardGetAtomFromFormat(Display *display, char *format_name)
     return atom;
 }
 
+
 static Boolean
 _XmClipboardGetLenFromFormat(Display *display, char *format_name, int *format)
 {
@@ -1215,6 +1204,7 @@ _XmClipboardGetLenFromFormat(Display *display, char *format_name, int *format)
     return ret;
 }
 
+
 static Window
 _XmClipboardInitializeSelection(Display *display, XmClipboard * clip,
 				Window window, Time locktime)
@@ -1243,6 +1233,7 @@ _XmClipboardInitializeSelection(Display *display, XmClipboard * clip,
     return clipowner;
 }
 
+
 static Boolean
 _XmClipboardWeOwnSelection(Display *display, XmClipboard * clip)
 {
@@ -1261,6 +1252,7 @@ _XmClipboardWeOwnSelection(Display *display, XmClipboard * clip)
     return False;
 }
 
+
 static void
 _XmAssertClipboardSelection(Display *display, Window window,
 			    XmClipboard * clip, Time locktime)
@@ -1273,7 +1265,7 @@ _XmAssertClipboardSelection(Display *display, Window window,
 
     w = XtWindowToWidget(display, window);
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "_XmAssertClipboardSelection\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "_XmAssertClipboardSelection\n"));
 
     if (w == NULL || clip->current_item == 0)
     {
@@ -1290,6 +1282,7 @@ _XmAssertClipboardSelection(Display *display, Window window,
     XtAddEventHandler(w, None, True, _XmClipboardEventHandler, NULL);
 }
 
+
 static Bool
 _XmClipboardSelectionIsReady(Display *display, XEvent *event, XPointer arg)
 {
@@ -1353,13 +1346,13 @@ _XmClipboardSelectionIsReady(Display *display, XEvent *event, XPointer arg)
 
     buf = (unsigned *)XtMalloc(len + match->len);
 
-    bcopy(match->data, buf, match->len);
+    memcpy(buf, match->data, match->len);
 
     XtFree((char *)match->data);
 
     match->data = buf;
 
-    bcopy(data, (char *)buf + match->len, len);
+    memcpy((char *)buf + match->len, data, len);
 
     XtFree((char *)data);
 
@@ -1373,6 +1366,7 @@ _XmClipboardSelectionIsReady(Display *display, XEvent *event, XPointer arg)
     return True;
 }
 
+
 static Boolean
 _XmClipboardGetSelection(Display *display, Window window, char *format_name,
 			 XmClipboard * clip, unsigned **data, int *len)
@@ -1456,6 +1450,7 @@ _XmClipboardGetSelection(Display *display, Window window, char *format_name,
     return False;
 }
 
+
 static void
 _XmClipboardSetNextItemId(Display *display, int item_id)
 {
@@ -1505,6 +1500,7 @@ _XmClipboardSetNextItemId(Display *display, int item_id)
 			    PropModeReplace, 32, True);
 }
 
+
 static void
 _XmClipboardMarkItem(Display *display, XmClipboard * clip,
 		     unsigned id, Boolean delete_on)
@@ -1538,6 +1534,7 @@ _XmClipboardMarkItem(Display *display, XmClipboard * clip,
 
 }
 
+
 static void
 _XmClipboardSetAccess(Display *display, Window window)
 {
@@ -1551,6 +1548,7 @@ _XmClipboardSetAccess(Display *display, Window window)
 		    PropModeReplace, (unsigned char *)"yes", 3);
 }
 
+
 static XmClipboard *
 _XmClipboardOpen(Display *display, int extra_size)
 {
@@ -1612,6 +1610,7 @@ _XmClipboardOpen(Display *display, int extra_size)
     return clip;
 }
 
+
 static void
 _XmClipboardClose(Display *display, XmClipboard * clip)
 {
@@ -1621,6 +1620,7 @@ _XmClipboardClose(Display *display, XmClipboard * clip)
 			    PropModeReplace, 32, True);
 }
 
+
 static int
 _XmClipboardLock(Display *display, Window window)
 {
@@ -1719,7 +1719,8 @@ _XmClipboardLock(Display *display, Window window)
 	}
     }
 
-    _XmClipboardReplaceItem(display, XmCLIP_LOCK, (unsigned *)lock, 8,
+    _XmClipboardReplaceItem(display, XmCLIP_LOCK, (unsigned *)lock,
+			    sizeof(XmClipboardLockRec),
 			    PropModeReplace, 32, False);
 
     _XmClipboardSetAccess(display, window);
@@ -1729,6 +1730,7 @@ _XmClipboardLock(Display *display, Window window)
     return XmClipboardSuccess;
 }
 
+
 static int
 _XmClipboardUnlock(Display *display, Window window, Boolean remove_all_locks)
 {
@@ -1772,7 +1774,7 @@ _XmClipboardUnlock(Display *display, Window window, Boolean remove_all_locks)
 
     if (lock->lockdepth > UNLOCKED)
     {
-	len = 8;
+	len = sizeof(XmClipboardLockRec);
     }
     else
     {
@@ -1794,6 +1796,7 @@ _XmClipboardUnlock(Display *display, Window window, Boolean remove_all_locks)
     return XmClipboardSuccess;
 }
 
+
 static void
 _XmClipboardEventHandler(Widget widget, XtPointer client_data,
 			 XEvent *ev, Boolean *dontSwallow)
@@ -1804,7 +1807,7 @@ _XmClipboardEventHandler(Widget widget, XtPointer client_data,
     int reason, len;
     XmCutPasteProc cb;
 
-    DEBUGOUT(XdbDebug(__FILE__, widget, "Call to ClipboardEventHandler\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, widget, "Call to ClipboardEventHandler\n"));
 
     if (event->type != ClientMessage)
     {
@@ -1821,7 +1824,7 @@ _XmClipboardEventHandler(Widget widget, XtPointer client_data,
     fid = event->data.l[1];
     pid = event->data.l[2];
 
-    DEBUGOUT(XdbDebug(__FILE__, widget, "ClipboardEventHandler: Find Item\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, widget, "ClipboardEventHandler: Find Item\n"));
 
     status = _XmClipboardFindItem(event->display, fid, &data, &len,
 				  NULL, XmCLIP_PROP_FORMAT);
@@ -1831,7 +1834,7 @@ _XmClipboardEventHandler(Widget widget, XtPointer client_data,
 	return;
     }
 
-    DEBUGOUT(XdbDebug(__FILE__, widget, "ClipboardEventHandler: Found Item\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, widget, "ClipboardEventHandler: Found Item\n"));
 
     cb = ((XmClipboardFormat *) data)->callback;
 
@@ -1839,7 +1842,7 @@ _XmClipboardEventHandler(Widget widget, XtPointer client_data,
 
     if (!cb)
     {
-	DEBUGOUT(XdbDebug(__FILE__, widget,
+	DEBUGOUT(_LtDebugDebug(__FILE__, widget,
 			  "ClipboardEventHandler: No Callback\n"));
 
 	return;
@@ -1859,7 +1862,7 @@ _XmClipboardEventHandler(Widget widget, XtPointer client_data,
     }
     else
     {
-	DEBUGOUT(XdbDebug(__FILE__, widget,
+	DEBUGOUT(_LtDebugDebug(__FILE__, widget,
 			  "ClipboardEventHandler: Bad Atom\n"));
 
 	return;
@@ -1869,7 +1872,7 @@ _XmClipboardEventHandler(Widget widget, XtPointer client_data,
 
     if (reason == XmCR_CLIPBOARD_DATA_REQUEST)
     {
-	DEBUGOUT(XdbDebug(__FILE__, widget,
+	DEBUGOUT(_LtDebugDebug(__FILE__, widget,
 			  "ClipboardEventHandler: Data Request\n"));
 
 	return;
@@ -1884,6 +1887,7 @@ _XmClipboardEventHandler(Widget widget, XtPointer client_data,
 			    PropModeReplace, 32, True);
 }
 
+
 static Boolean
 _XmClipboardSendMessage(Display *display, Window window,
 			XmClipboardFormat * format, int id)
@@ -1894,7 +1898,7 @@ _XmClipboardSendMessage(Display *display, Window window,
     unsigned *data;
     int len;
 
-    DEBUGOUT(XdbDebug(__FILE__, NULL, "_XmClipboardSendMEssage: %d\n", id));
+    DEBUGOUT(_LtDebugDebug(__FILE__, NULL, "_XmClipboardSendMEssage: %d\n", id));
 
     if (format->widget_window == None)
     {
@@ -1930,7 +1934,7 @@ _XmClipboardSendMessage(Display *display, Window window,
 	event.data.l[2] = format->private_id;
 	if (format->window == window && format->display == display)
 	{
-	    DEBUGOUT(XdbDebug(__FILE__, NULL,
+	    DEBUGOUT(_LtDebugDebug(__FILE__, NULL,
 			      "Call _XmClipboardEventHandler direct\n"));
 
 	    _XmClipboardEventHandler(format->widget, NULL,
@@ -1950,7 +1954,8 @@ _XmClipboardSendMessage(Display *display, Window window,
 
 /*************************** public functions **************************/
 
-int
+#if XmVERSION == 1
+extern int
 XmClipboardBeginCopy(Display *display,
 		     Window window,
 		     XmString clip_label,
@@ -1961,8 +1966,10 @@ XmClipboardBeginCopy(Display *display,
     return XmClipboardStartCopy(display, window, clip_label, CurrentTime,
 				widget, callback, item_id);
 }
+#endif
 
-int
+
+extern int
 XmClipboardCancelCopy(Display *display,
 		      Window window,
 		      long item_id)
@@ -2004,7 +2011,8 @@ XmClipboardCancelCopy(Display *display,
     return XmClipboardSuccess;
 }
 
-int
+
+extern int
 XmClipboardCopy(Display *display,
 		Window window,
 		long item_id,
@@ -2024,7 +2032,7 @@ XmClipboardCopy(Display *display,
     int fmt;
     XtPointer nbuf;
 
-    DEBUGOUT(XdbDebug(__FILE__, NULL,
+    DEBUGOUT(_LtDebugDebug(__FILE__, NULL,
 	"XmClipboardCopy(win 0x%X, item_id %ld, format %s)\n",
 	window, item_id, format_name));
 
@@ -2140,7 +2148,7 @@ XmClipboardCopy(Display *display,
 
     if (buffer)
     {
-	bcopy(buffer, nbuf, length);
+	memcpy(nbuf, buffer, length);
     }
 
     format->length += length;
@@ -2157,13 +2165,13 @@ XmClipboardCopy(Display *display,
     }
 
     _XmClipboardClose(display, clip);
-
     _XmClipboardUnlock(display, window, False);
 
     return XmClipboardSuccess;
 }
 
-int
+
+extern int
 XmClipboardCopyByName(Display *display,
 		      Window window,
 		      long data_id,
@@ -2229,7 +2237,7 @@ XmClipboardCopyByName(Display *display,
 
 	bufstart = (unsigned char *)idata + ilen - length;
 
-	bcopy(buffer, bufstart, length);
+	memcpy(bufstart, buffer, length);
 
 	_XmClipboardReplaceItem(display, format->data_id, idata, ilen,
 				PropModeReplace, fmt, True);
@@ -2248,7 +2256,8 @@ XmClipboardCopyByName(Display *display,
     return XmClipboardSuccess;
 }
 
-int
+
+extern int
 XmClipboardEndCopy(Display *display,
 		   Window window,
 		   long item_id)
@@ -2259,7 +2268,7 @@ XmClipboardEndCopy(Display *display,
     int len;
     XmClipboardItem *item;
 
-    DEBUGOUT(XdbDebug(__FILE__, NULL,
+    DEBUGOUT(_LtDebugDebug(__FILE__, NULL,
 	"XmClipboardEndCopy(win 0x%X, item %ld)\n",
 	window, item_id));
 
@@ -2310,7 +2319,7 @@ XmClipboardEndCopy(Display *display,
 
     if (item && item->widget)
     {
-	DEBUGOUT(XdbDebug(__FILE__, NULL, "Add EventHandler from EndCopy\n"));
+	DEBUGOUT(_LtDebugDebug(__FILE__, NULL, "Add EventHandler from EndCopy\n"));
 
 	XtAddEventHandler(item->widget, None, True,
 			  _XmClipboardEventHandler, NULL);
@@ -2330,7 +2339,8 @@ XmClipboardEndCopy(Display *display,
     return XmClipboardSuccess;
 }
 
-int
+
+extern int
 XmClipboardEndRetrieve(Display *display,
 		       Window window)
 {
@@ -2348,7 +2358,8 @@ XmClipboardEndRetrieve(Display *display,
     return XmClipboardSuccess;
 }
 
-int
+
+extern int
 XmClipboardInquireCount(Display *display,
 			Window window,
 			int *count,
@@ -2398,7 +2409,7 @@ XmClipboardInquireCount(Display *display,
 	    return XmClipboardFail;
 	}
 
-	/* FIX ME */
+	/* FIX ME:  This is supposed to be the ICCCM way */
     }
 
     if (max_length != NULL)
@@ -2423,7 +2434,8 @@ XmClipboardInquireCount(Display *display,
     return XmClipboardSuccess;
 }
 
-int
+
+extern int
 XmClipboardInquireFormat(Display *display,
 			 Window window,
 			 int index,
@@ -2473,13 +2485,13 @@ XmClipboardInquireFormat(Display *display,
 	    return XmClipboardFail;
 	}
 
-	/* FIX ME */
+	/* FIX ME: This is supposed to be the ICCCM way of things */
     }
 
     if (name)
     {
 	len = strlen(name) <= buffer_len ? strlen(name) : buffer_len;
-	strncpy(format_name_buf, name, len);
+	strncpy((char *)format_name_buf, name, len);
 
 	if (copied_len)
 	{
@@ -2503,7 +2515,8 @@ XmClipboardInquireFormat(Display *display,
     return XmClipboardSuccess;
 }
 
-int
+
+extern int
 XmClipboardInquireLength(Display *display,
 			 Window window,
 			 char *format_name,
@@ -2563,13 +2576,13 @@ XmClipboardInquireLength(Display *display,
     }
 
     _XmClipboardClose(display, clip);
-
     _XmClipboardUnlock(display, window, False);
 
     return XmClipboardSuccess;
 }
 
-int
+
+extern int
 XmClipboardInquirePendingItems(Display *display,
 			       Window window,
 			       char *format_name,
@@ -2650,14 +2663,16 @@ XmClipboardInquirePendingItems(Display *display,
     return XmClipboardSuccess;
 }
 
-int
+
+extern int
 XmClipboardLock(Display *display,
 		Window window)
 {
     return _XmClipboardLock(display, window);
 }
 
-int
+
+extern int
 XmClipboardRegisterFormat(Display *display,
 			  char *format_name,
 			  int format_length)
@@ -2718,7 +2733,8 @@ XmClipboardRegisterFormat(Display *display,
     return XmClipboardFail;
 }
 
-int
+
+extern int
 XmClipboardRetrieve(Display *display,
 		    Window window,
 		    char *format_name,
@@ -2728,23 +2744,25 @@ XmClipboardRetrieve(Display *display,
 		    long *private_id)
 {
     XmClipboard *clip;
-    int namelen, numformats, formatlen, dlen, privid = 0, status, format_id;
+    int namelen, numformats, formatlen, dlen, status, format_id;
+    int privid = 0;
     int fetchlen = 0;
     XmClipboardFormat *format;
     Boolean got_format = False;
-    unsigned *data;
+    unsigned *data=NULL;
     unsigned char *bufstart = NULL;
-    Boolean partial_copy = False, got_data = False;
+    Boolean partial_copy = False;
+    Boolean got_data = False;
 
     status = _XmClipboardLock(display, window);
     if (status == XmClipboardLocked)
     {
-	DEBUGOUT(XdbDebug(__FILE__, XtWindowToWidget(display, window),
+	DEBUGOUT(_LtDebugDebug(__FILE__, XtWindowToWidget(display, window),
 		"XmClipboardRetrieve => XmClipboardLocked\n"));
 	return status;
     }
 
-    DEBUGOUT(XdbDebug(__FILE__, XtWindowToWidget(display, window),
+    DEBUGOUT(_LtDebugDebug(__FILE__, XtWindowToWidget(display, window),
 	"XmClipboardRetrieve(fmt %s, maxlen %d)\n",
 	format_name ? format_name : "(null)", length));
 
@@ -2757,7 +2775,7 @@ XmClipboardRetrieve(Display *display,
 
     if (_XmClipboardWeOwnSelection(display, clip))
     {
-	DEBUGOUT(XdbDebug(__FILE__, XtWindowToWidget(display, window),
+	DEBUGOUT(_LtDebugDebug(__FILE__, XtWindowToWidget(display, window),
 	    "XmClipboardRetrieve: we own selection\n"));
 
 	format = _XmClipboardFindFormat(display, clip, format_name, 0, 0,
@@ -2767,14 +2785,14 @@ XmClipboardRetrieve(Display *display,
 
 	if (format != NULL)
 	{
-	    DEBUGOUT(XdbDebug(__FILE__, XtWindowToWidget(display, window),
+	    DEBUGOUT(_LtDebugDebug(__FILE__, XtWindowToWidget(display, window),
 		"XmClipboardRetrieve: format != NULL\n"));
 
 	    format_id = format->format_id;
 
 	    if (format->by_name)
 	    {
-		DEBUGOUT(XdbDebug(__FILE__, XtWindowToWidget(display, window),
+		DEBUGOUT(_LtDebugDebug(__FILE__, XtWindowToWidget(display, window),
 		    "XmClipboardRetrieve: by name\n"));
 
 		got_format = _XmClipboardRequestDataAndWait(display, window,
@@ -2782,7 +2800,7 @@ XmClipboardRetrieve(Display *display,
 
 		if (got_format)
 		{
-		    DEBUGOUT(XdbDebug(__FILE__,
+		    DEBUGOUT(_LtDebugDebug(__FILE__,
 			XtWindowToWidget(display, window),
 			"XmClipboardRetrieve: got format\n"));
 
@@ -2798,7 +2816,7 @@ XmClipboardRetrieve(Display *display,
 
 	    if (got_format)
 	    {
-		DEBUGOUT(XdbDebug(__FILE__, XtWindowToWidget(display, window),
+		DEBUGOUT(_LtDebugDebug(__FILE__, XtWindowToWidget(display, window),
 			"XmClipboardRetrieve: got format 2\n"));
 
 		_XmClipboardFindItem(display, format->data_id, &data, &dlen,
@@ -2816,9 +2834,9 @@ XmClipboardRetrieve(Display *display,
 		    fetchlen = dlen - format->offset;
 		}
 
-		DEBUGOUT(XdbDebug(__FILE__, XtWindowToWidget(display, window),
+		DEBUGOUT(_LtDebugDebug(__FILE__, XtWindowToWidget(display, window),
 			"XmClipboardRetrieve: fetchlen %d partial_copy %s\n",
-			fetchlen, XdbBoolean2String(partial_copy)));
+			fetchlen, _LtDebugBoolean2String(partial_copy)));
 
 		if (clip->retrieve_locked)
 		{
@@ -2844,44 +2862,45 @@ XmClipboardRetrieve(Display *display,
     }
     else
     {
-	DEBUGOUT(XdbDebug(__FILE__, XtWindowToWidget(display, window),
+	DEBUGOUT(_LtDebugDebug(__FILE__, XtWindowToWidget(display, window),
 	    "XmClipboardRetrieve: we don't own selection !!\n"));
 
-	_XmClipboardGetSelection(display, window, format_name, clip,
-				 &data, &dlen);
+        /* amai: At least when clipboard is messed up from remaining LT bugs
+                 I saw got_data==NULL here ... */
+        got_data = _XmClipboardGetSelection(display, window,
+                                            format_name, clip,
+  	                                    &data, &dlen);
+	if (got_data) {
 
-	bufstart = (unsigned char *)data + clip->offset;
-
-	if (dlen - clip->offset > length)
-	{
-	    partial_copy = True;
-	    fetchlen = length;
-	}
-	else
-	{
-	    fetchlen = dlen - clip->offset;
-	}
-
-	if (clip->retrieve_locked)
-	{
-
-	    if (partial_copy)
-	    {
-		clip->offset += fetchlen;
-	    }
-	    else
-	    {
-		clip->offset = 0;
-	    }
-	}
-
-	got_data = True;
-    }
+		bufstart = (unsigned char *)data + clip->offset;
+		if (dlen - clip->offset > length)
+		{
+		    partial_copy = True;
+		    fetchlen = length;
+		}
+		else
+		{
+		    fetchlen = dlen - clip->offset;
+		}
+	
+		if (clip->retrieve_locked)
+		{
+	
+		    if (partial_copy)
+		    {
+			clip->offset += fetchlen;
+		    }
+		    else
+		    {
+			clip->offset = 0;
+		    }
+		}
+	} /* if (got_data) */
+    } /* else */
 
     if (got_data)
     {
-	bcopy(bufstart, buffer, fetchlen);
-
+	memcpy(buffer, bufstart, fetchlen);
 	XtFree((char *)data);
     }
 
@@ -2896,7 +2915,6 @@ XmClipboardRetrieve(Display *display,
     }
 
     _XmClipboardClose(display, clip);
-
     _XmClipboardUnlock(display, window, False);
 
     if (fetchlen == 0)
@@ -2912,7 +2930,8 @@ XmClipboardRetrieve(Display *display,
     return XmClipboardSuccess;
 }
 
-int
+
+extern int
 XmClipboardStartCopy(Display *display,
 		     Window window,
 		     XmString clip_label,
@@ -2926,7 +2945,7 @@ XmClipboardStartCopy(Display *display,
     unsigned new_item_id, new_label_id;
     int llen;
 
-    DEBUGOUT(XdbDebug(__FILE__, NULL,
+    DEBUGOUT(_LtDebugDebug(__FILE__, NULL,
 	"XmClipboardStartCopy(win 0x%X, clip 0x%X, time %ld)\n",
 	window, clip_label, timestamp));
 
@@ -2990,7 +3009,8 @@ XmClipboardStartCopy(Display *display,
     return XmClipboardSuccess;
 }
 
-int
+
+extern int
 XmClipboardStartRetrieve(Display *display,
 			 Window window,
 			 Time timestamp)
@@ -3013,6 +3033,7 @@ XmClipboardStartRetrieve(Display *display,
     return XmClipboardSuccess;
 }
 
+
 int
 XmClipboardUndoCopy(Display *display, Window window)
 {
@@ -3065,19 +3086,20 @@ XmClipboardUndoCopy(Display *display, Window window)
     }
 
     _XmClipboardClose(display, clip);
-
     _XmClipboardUnlock(display, window, False);
 
     return XmClipboardSuccess;
 }
 
-int
+
+extern int
 XmClipboardUnlock(Display *display, Window window, Boolean remove_all_locks)
 {
     return _XmClipboardUnlock(display, window, remove_all_locks);
 }
 
-int
+
+extern int
 XmClipboardWithdrawFormat(Display *display,
 			  Window window,
 			  int data_id)
@@ -3088,7 +3110,6 @@ XmClipboardWithdrawFormat(Display *display,
     }
 
     _XmClipboardDeleteFormat(display, data_id);
-
     _XmClipboardUnlock(display, window, False);
 
     return XmClipboardSuccess;
