@@ -1,10 +1,11 @@
 /**
  *
- * $Id: DialogS.c,v 1.21 1999/08/07 15:32:52 rwscott Exp $
+ * $Id: DialogS.c,v 1.29 2001/05/14 19:46:08 amai Exp $
  *
  * derived from Xt Vendor class.c 
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
+ * Copyright (C) 1995-2001 LessTif Development Team
  * Copyright 1989 Massachusetts Institute of Technology
  *
  * This file is part of the GNU LessTif Library.
@@ -25,9 +26,7 @@
  *
  **/
 
-static const char rcsid[] = "$Id: DialogS.c,v 1.21 1999/08/07 15:32:52 rwscott Exp $";
-
-#include <stdio.h>
+static const char rcsid[] = "$Id: DialogS.c,v 1.29 2001/05/14 19:46:08 amai Exp $";
 
 /*
  * This controls questionable behavior.  It will go away
@@ -36,15 +35,17 @@ static const char rcsid[] = "$Id: DialogS.c,v 1.21 1999/08/07 15:32:52 rwscott E
  */
 
 #include <LTconfig.h>
-#include <XmI/XmI.h>
 
+#include <stdio.h>
+#include <string.h>
+
+#include <XmI/XmI.h>
 #include <Xm/XmP.h>
 #include <Xm/ExtObjectP.h>
 #include <Xm/DialogSP.h>
 #include <Xm/DialogSEP.h>
 #include <X11/StringDefs.h>
 #include <X11/ShellP.h>
-#include <X11/Xfuncs.h>
 #include <Xm/VendorSP.h>
 #include <Xm/VendorSEP.h>
 #include <Xm/BulletinBP.h>
@@ -52,6 +53,7 @@ static const char rcsid[] = "$Id: DialogS.c,v 1.21 1999/08/07 15:32:52 rwscott E
 
 #include <XmI/DebugUtil.h>
 
+
 static void class_initialize(void);
 
 static void class_part_initialize(WidgetClass wclass);
@@ -250,7 +252,49 @@ WidgetClass xmDialogShellExtObjectClass = (WidgetClass)&xmDialogShellExtClassRec
 static void 
 class_initialize(void)
 {
+    int ncom, addon;
+    XtResourceList combined, shells;
+    Cardinal nshells, i, j;
+
     _XmDialogSCoreClassExtRec.record_type = XmQmotif;
+
+    ncom = XtNumber(ext_resources) +
+                xmVendorShellExtClassRec.object_class.num_resources;
+
+    _XmTransformSubResources(xmVendorShellExtClassRec.object_class.resources,
+                             xmVendorShellExtClassRec.object_class.
+				num_resources,
+                             &shells, &nshells);
+
+    combined = (XtResourceList)XtMalloc(sizeof(XtResource) * ncom);
+
+    memcpy(combined, shells, nshells * sizeof(XtResource));
+
+    /* can't just copy.  We know we need to drop one */
+    for (i = 0, addon = 0; i < XtNumber(ext_resources); i++)
+    {
+	for (j = 0; j < nshells; j++)
+	{
+	    if (strcmp(combined[j].resource_name,
+		       ext_resources[i].resource_name) == 0)
+	    {
+		combined[j] = ext_resources[i];
+		break;
+	    }
+	}
+
+	if (j != nshells)
+	{
+	    combined[nshells + addon] = ext_resources[i];
+	    addon++;
+	}
+    }
+
+    XtFree((char *)shells);
+
+    xmDialogShellExtClassRec.object_class.resources = combined;
+    xmDialogShellExtClassRec.object_class.num_resources = nshells + addon;
+
 }
 
 static void 
@@ -292,11 +336,12 @@ initialize(Widget request,
     XtRealizeWidget(new_w);
 }
 
+
 static Widget
 GetChild(Widget w)
 {
     CompositeWidget cw = (CompositeWidget)w;
-    int i;
+    Cardinal i;
     extern WidgetClass xmVendorShellExtObjectClass;	/* FIX ME */
 
     /*
@@ -323,7 +368,7 @@ change_managed(Widget w)
 
     child = GetChild(w);
 
-    DEBUGOUT(XdbDebug2(__FILE__, w, child, "ChangeManaged\n"));
+    DEBUGOUT(_LtDebugDebug2(__FILE__, w, child, "ChangeManaged\n"));
 
     /* not doing this was why popup dialogs wouldn't take focus */
     XtSetKeyboardFocus(w, child);
@@ -332,10 +377,10 @@ change_managed(Widget w)
     {
 	Boolean InitialPlacement = False;
 
-	DEBUGOUT(XdbDebug2(__FILE__, w, child, "... was not managed\n"));
+	DEBUGOUT(_LtDebugDebug2(__FILE__, w, child, "... was not managed\n"));
 	if (!XtIsRealized(child))
 	{
-	    DEBUGOUT(XdbDebug(__FILE__, child, "... Realizing\n"));
+	    DEBUGOUT(_LtDebugDebug(__FILE__, child, "... Realizing\n"));
 
 	    XtRealizeWidget(child);
 
@@ -352,13 +397,13 @@ change_managed(Widget w)
 	/* Dialog's child should be at 0,0 */
 	if (XtX(child) != 0 || XtY(child) != 0)
 	{
-	    DEBUGOUT(XdbDebug2(__FILE__, w, child,
+	    DEBUGOUT(_LtDebugDebug2(__FILE__, w, child,
 		    "Child position %d,%d set to 0,0\n",
 		    XtX(child), XtY(child)));
 	    _XmMoveObject(child, 0, 0);
 	}
 
-	DEBUGOUT(XdbDebug2(__FILE__, w, child, "Shell size %d %d %d %d\n",
+	DEBUGOUT(_LtDebugDebug2(__FILE__, w, child, "Shell size %d %d %d %d\n",
 			   XtWidth(child), XtHeight(child),
 			   XtX(w), XtY(w)));
 
@@ -373,7 +418,7 @@ change_managed(Widget w)
 	    px = XtX(p) + (XtWidth(p) - XtWidth(child)) / 2;
 	    py = XtY(p) + (XtHeight(p) - XtHeight(child)) / 2;
 
-	    DEBUGOUT(XdbDebug(__FILE__, w, "def. pos. %d %d\n", px, py));
+	    DEBUGOUT(_LtDebugDebug(__FILE__, w, "def. pos. %d %d\n", px, py));
 
 	    if (px < 0)
 	    {
@@ -387,24 +432,25 @@ change_managed(Widget w)
 	    XtMoveWidget(w, px, py);
 	    BB_DefaultPosition(child) = False;
 	}
+
+	if (XmIsBulletinBoard(child))
+	    _XmBulletinBoardMap(child);
+
 	if (CoreMappedWhenManaged(w))
 	{
-	    DEBUGOUT(XdbDebug(__FILE__, w, "XtPopup\n"));
+	    DEBUGOUT(_LtDebugDebug(__FILE__, w, "XtPopup\n"));
 
 	    XtPopup(w, XtGrabNone);
 
-	    DEBUGOUT(XdbDebug2(__FILE__, w, child,
+	    DEBUGOUT(_LtDebugDebug2(__FILE__, w, child,
 			       "Mapping shell, just to be sure\n"));
 
-	    if (XmIsBulletinBoard(child))
-		_XmBulletinBoardMap(child);
-
 	    XtMapWidget(w);
 	}
     }
     else
     {
-	DEBUGOUT(XdbDebug(__FILE__, (Widget)w, "XtPopDown\n"));
+	DEBUGOUT(_LtDebugDebug(__FILE__, (Widget)w, "XtPopDown\n"));
 	XtPopdown((Widget)w);
 
 	/* rws 1 Jun 1997
@@ -414,7 +460,7 @@ change_managed(Widget w)
 	 * MLM: enabled this again, but moved it to *after* the popdown.  This
 	 * fixes certain cases (like nedit->open->ok, with no file selected).
 	 */
-	DEBUGOUT(XdbDebug(__FILE__, w, "XtUnmapWidget\n"));
+	DEBUGOUT(_LtDebugDebug(__FILE__, w, "XtUnmapWidget\n"));
 	if (XmIsBulletinBoard(child))
 		_XmBulletinBoardUnmap(child);
 
@@ -427,7 +473,7 @@ insert_child(Widget w)
 {
     CompositeWidget p = (CompositeWidget)XtParent(w);
 
-    DEBUGOUT(XdbDebug2(__FILE__, (Widget)p, w, "insert_child\n"));
+    DEBUGOUT(_LtDebugDebug2(__FILE__, (Widget)p, w, "insert_child\n"));
 
 #define superclass (&transientShellClassRec)
     (*superclass->composite_class.insert_child) (w);
@@ -467,8 +513,8 @@ delete_child(Widget w)
 {
     Widget s = XtParent(w);
 
-    DEBUGOUT(XdbDebug2(__FILE__, s, w, "delete_child\n"));
-    DEBUGOUT(XdbDebug2("RWS", s, w,"%s:delete_child(%d)\n",
+    DEBUGOUT(_LtDebugDebug2(__FILE__, s, w, "delete_child\n"));
+    DEBUGOUT(_LtDebugDebug2("RWS", s, w,"%s:delete_child(%d)\n",
     	__FILE__, __LINE__
     	));
 
@@ -485,10 +531,10 @@ delete_child(Widget w)
     if (XtIsManaged(w) || MGR_NumChildren(s) == 1)
     {
 #if 1
-	DEBUGOUT(XdbDebug(__FILE__, s, "XtUnmapWidget\n"));
+	DEBUGOUT(_LtDebugDebug(__FILE__, s, "XtUnmapWidget\n"));
 	XtUnmapWidget(s);
 #else
-	DEBUGOUT(XdbDebug(__FILE__, s, "XtPopdown\n"));
+	DEBUGOUT(_LtDebugDebug(__FILE__, s, "XtPopdown\n"));
 	XtPopdown(s);
 #endif
     }
@@ -502,15 +548,15 @@ static void
 WmProtocolHandler(Widget w, XtPointer client, XtPointer call)
 {
     XmVendorShellExtObject ve = (XmVendorShellExtObject)client;
-    int i;
+    Cardinal i;
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "Dialog's WmProtocolHandler\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "Dialog's WmProtocolHandler\n"));
 
     switch (VSEP_DeleteResponse(ve))
     {
     case XmDESTROY:
 	XtDestroyWidget(w);
-	DEBUGOUT(XdbDebug(__FILE__, w,
+	DEBUGOUT(_LtDebugDebug(__FILE__, w,
 			  "WmProtocolHandler(DeleteResponse XmDESTROY)\n"));
 	break;
 
@@ -520,7 +566,7 @@ WmProtocolHandler(Widget w, XtPointer client, XtPointer call)
 	{
 	    if (XtIsManaged(MGR_Children(w)[i]))
 	    {
-		DEBUGOUT(XdbDebug2(__FILE__, w, MGR_Children(w)[i],
+		DEBUGOUT(_LtDebugDebug2(__FILE__, w, MGR_Children(w)[i],
 				   "XtUnmanageChild(child)\n"));
 		XtUnmanageChild(MGR_Children(w)[i]);
 		return;
@@ -529,7 +575,7 @@ WmProtocolHandler(Widget w, XtPointer client, XtPointer call)
 	break;
 
     case XmDO_NOTHING:
-	DEBUGOUT(XdbDebug(__FILE__, w,
+	DEBUGOUT(_LtDebugDebug(__FILE__, w,
 			  "WmProtocolHandler(DeleteResponse XmNO_NOTHING)\n"));
 	return;
     }
@@ -607,12 +653,12 @@ StructureNotifyHandler(Widget w, XtPointer closure,
     switch (event->type)
     {
     case ConfigureNotify:
-	DEBUGOUT(XdbDebug(__FILE__, w, "CONFIGURE NOTIFY: layout %d %d %d %d\n",
+	DEBUGOUT(_LtDebugDebug(__FILE__, w, "CONFIGURE NOTIFY: layout %d %d %d %d\n",
 			  cev->x, cev->y, cev->width, cev->height));
 	break;
 
     case MapNotify:
-	DEBUGOUT(XdbDebug(__FILE__, w, "MAP NOTIFY: window %08x\n",
+	DEBUGOUT(_LtDebugDebug(__FILE__, w, "MAP NOTIFY: window %08x\n",
 			  mev->window));
 	/* rws 22 Sep 1998
 	   This is causing nedit's File->Open to un-map as soon as it is
@@ -629,17 +675,17 @@ StructureNotifyHandler(Widget w, XtPointer closure,
 	break;
 
     case UnmapNotify:
-	DEBUGOUT(XdbDebug(__FILE__, w, "UNMAP NOTIFY: window %08x\n",
+	DEBUGOUT(_LtDebugDebug(__FILE__, w, "UNMAP NOTIFY: window %08x\n",
 			  uev->window));
 	break;
 
     case ReparentNotify:
-	DEBUGOUT(XdbDebug(__FILE__, w, "REPARENT NOTIFY: window %08x\n",
+	DEBUGOUT(_LtDebugDebug(__FILE__, w, "REPARENT NOTIFY: window %08x\n",
 			  rev->window));
 	break;
 
     default:
-	DEBUGOUT(XdbDebug(__FILE__, w, "Got UNKNOWN TYPE: %d\n",
+	DEBUGOUT(_LtDebugDebug(__FILE__, w, "Got UNKNOWN TYPE: %d\n",
 			  event->type));
 	break;
     }
