@@ -1,8 +1,9 @@
 /**
  *
- * $Id: Display.c,v 1.15 2000/01/05 23:44:43 rwscott Exp $
+ * $Id: Display.c,v 1.22 2001/05/17 13:04:49 amai Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
+ * Copyright (C) 1995-2001 LessTif Development Team
  *
  * This file is part of the GNU LessTif Library.
  *
@@ -22,9 +23,10 @@
  *
  **/
 
-static const char rcsid[] = "$Id: Display.c,v 1.15 2000/01/05 23:44:43 rwscott Exp $";
+static const char rcsid[] = "$Id: Display.c,v 1.22 2001/05/17 13:04:49 amai Exp $";
 
 #include <LTconfig.h>
+
 #include <XmI/XmI.h>
 #include <XmI/MacrosI.h>
 #include <XmI/AtomMgrI.h>
@@ -38,6 +40,7 @@ static const char rcsid[] = "$Id: Display.c,v 1.15 2000/01/05 23:44:43 rwscott E
 
 #include <XmI/DebugUtil.h>
 
+
 #define Offset(field) XtOffsetOf(XmDisplayRec, display.field)
 
 static XtResource resources[] =
@@ -242,7 +245,7 @@ DisplaySuicide(Widget w, XtPointer ClientData, XtPointer CallData)
 {
     Display *Dsp;
 
-    DEBUGOUT(XdbDebug(__FILE__, (Widget)ClientData,
+    DEBUGOUT(_LtDebugDebug(__FILE__, (Widget)ClientData,
 		      "XmDisplay Connection Close-Down\n"));
 
     Dsp = XtDisplay((Widget)ClientData);
@@ -306,7 +309,7 @@ initialize(Widget request, Widget new_w,
     XPointer FirstDisplayWidget;
     XmDisplay dpy = (XmDisplay)new_w;
 
-    DEBUGOUT(XdbDebug(__FILE__, new_w,
+    DEBUGOUT(_LtDebugDebug(__FILE__, new_w,
 		      "%s:initialize(%d) - %i args\n"
 		      "\trequest X %5i Y %5i W %5i H %5i\n"
 		      "\t  new_w X %5i Y %5i W %5i H %5i\n",
@@ -316,7 +319,7 @@ initialize(Widget request, Widget new_w,
 		      XtWidth(request), XtHeight(request),
 		      XtX(new_w), XtY(new_w),
 		      XtWidth(new_w), XtHeight(new_w)));
-    DEBUGOUT(XdbPrintArgList(__FILE__, new_w, args, *num_args, False));
+    DEBUGOUT(_LtDebugPrintArgList(__FILE__, new_w, args, *num_args, False));
 
     dpy->display.shellCount = 0;
     dpy->display.numModals = 0;
@@ -347,7 +350,7 @@ initialize(Widget request, Widget new_w,
     else
     {
 	XSaveContext(XtDisplay(new_w), PDWC_RID_DisplayWidget,
-		     PerDisplayWidgetContext, (_Xconst char *)new_w);
+		     PerDisplayWidgetContext, (XPointer)new_w);
     }
 
     /*
@@ -396,9 +399,9 @@ destroy(Widget w)
 {
 
     if (CoreBeingDestroyed(w))
-	DEBUGOUT(XdbDebug(__FILE__, w, "Destroy(being_destroyed)\n"));
+	DEBUGOUT(_LtDebugDebug(__FILE__, w, "Destroy(being_destroyed)\n"));
     else {
-	DEBUGOUT(XdbDebug(__FILE__, w, "Destroy\n"));
+	DEBUGOUT(_LtDebugDebug(__FILE__, w, "Destroy\n"));
 
 	/* Hack */
 	return;
@@ -422,7 +425,7 @@ static Boolean
 set_values(Widget old, Widget request, Widget new_w,
 	   ArgList args, Cardinal *num_args)
 {
-    DEBUGOUT(XdbDebug(__FILE__, new_w,
+    DEBUGOUT(_LtDebugDebug(__FILE__, new_w,
 		      "%s:set_values(%d) - %i args\n"
 		      "\t    old X %5i Y %5i W %5i H %5i\n"
 		      "\trequest X %5i Y %5i W %5i H %5i\n"
@@ -435,7 +438,7 @@ set_values(Widget old, Widget request, Widget new_w,
 		      XtWidth(request), XtHeight(request),
 		      XtX(new_w), XtY(new_w),
 		      XtWidth(new_w), XtHeight(new_w)));
-    DEBUGOUT(XdbPrintArgList(__FILE__, new_w, args, *num_args, False));
+    DEBUGOUT(_LtDebugPrintArgList(__FILE__, new_w, args, *num_args, False));
 
     return False;
 }
@@ -458,7 +461,7 @@ RefreshMapping(Widget w, XEvent *event,
      * look up the offending display widget and refresh the virtual
      * key bindings. Happy refreshing...
      */
-    DEBUGOUT(XdbDebug(__FILE__, w, "Refreshing virtual key bindings.\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "Refreshing virtual key bindings.\n"));
 
     _XmRefreshVirtKeys(XmGetXmDisplay(event->xmapping.display));
 }
@@ -527,26 +530,34 @@ _XmGetDropSiteManagerObject(XmDisplay xmDisplay)
 {
     XmDropSiteManagerObject dsm = NULL;
 
-    DEBUGOUT(XdbDebug(__FILE__, (Widget)xmDisplay, "%s:_XmGetDropSiteManagerObject(%d) - %s 0x%p\n",
+    DEBUGOUT(_LtDebugDebug(__FILE__, (Widget)xmDisplay, "%s:_XmGetDropSiteManagerObject(%d) - %s 0x%p\n",
     	__FILE__, __LINE__,
     	Display_DSM(xmDisplay) ? "Old" : "New", Display_DSM(xmDisplay)));
-    if (Display_DSM(xmDisplay) == NULL)
+    if (!XmIsDisplay(xmDisplay))
     {
-	DEBUGOUT(XdbDebug0(__FILE__, NULL, "%s:_XmGetDropSiteManagerObject(%d) - creating\n",
-	    __FILE__, __LINE__
-	    ));
-	dsm = (XmDropSiteManagerObject)XtCreateWidget("DropSiteManager",
-				       Display_DropSiteManagerClass(xmDisplay),
-						   (Widget)xmDisplay, NULL, 0);
-	DEBUGOUT(XdbDebug2(__FILE__, (Widget)xmDisplay, (Widget)dsm, "%s:_XmGetDropSiteManagerObject(%d) - created 0x%p\n",
-	    __FILE__, __LINE__, dsm
-	    ));
-
-	Display_DSM(xmDisplay) = dsm;
+    	_XmWarning((Widget)xmDisplay, "%s:_XmGetDropSiteManagerObject(%d) called without an XmDisplay",
+    		__FILE__, __LINE__);
     }
     else
     {
-	dsm = Display_DSM(xmDisplay);
+	if (Display_DSM(xmDisplay) == NULL)
+	{
+	    DEBUGOUT(_LtDebugDebug0(__FILE__, NULL, "%s:_XmGetDropSiteManagerObject(%d) - creating\n",
+		__FILE__, __LINE__
+		));
+	    dsm = (XmDropSiteManagerObject)XtCreateWidget("DropSiteManager",
+					   Display_DropSiteManagerClass(xmDisplay),
+						       (Widget)xmDisplay, NULL, 0);
+	    DEBUGOUT(_LtDebugDebug2(__FILE__, (Widget)xmDisplay, (Widget)dsm, "%s:_XmGetDropSiteManagerObject(%d) - created 0x%p\n",
+		__FILE__, __LINE__, dsm
+		));
+
+	    Display_DSM(xmDisplay) = dsm;
+	}
+	else
+	{
+	    dsm = Display_DSM(xmDisplay);
+	}
     }
 
     return dsm;
@@ -557,7 +568,7 @@ _XmGetDragProtocolStyle(Widget w)
 {
     Widget disp = XmGetXmDisplay(XtDisplay(w));
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "%s:_XmGetDragProtocolStyle(%d)\n",
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "%s:_XmGetDragProtocolStyle(%d)\n",
     	__FILE__, __LINE__));
     switch (Display_DragReceiverProtocolStyle(disp))
     {
@@ -586,7 +597,7 @@ _XmGetDragProtocolStyle(Widget w)
 unsigned char
 _XmGetDragTrackingMode(Widget w)
 {
-    DEBUGOUT(XdbDebug(__FILE__, w, "%s:_XmGetDragTrackingMode(%d)\n",
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "%s:_XmGetDragTrackingMode(%d)\n",
     	__FILE__, __LINE__));
     return 0;
 }
@@ -595,14 +606,14 @@ Widget
 _XmGetDragContextFromHandle(Widget w, Atom iccHandle)
 {
     Widget disp = XmGetXmDisplay(XtDisplay(w)), dc;
-    int i;
+    Cardinal c;
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "%s:_XmGetDragContextFromHandle(%d)\n",
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "%s:_XmGetDragContextFromHandle(%d)\n",
     	__FILE__, __LINE__));
-    for (i = 0; i < MGR_NumChildren(disp); i++)
+    for (c = 0; c < MGR_NumChildren(disp); c++)
     {
 
-	dc = MGR_Children(disp)[i];
+	dc = MGR_Children(disp)[c];
 
 	if (XmIsDragContext(dc))
 	{
@@ -626,7 +637,7 @@ _XmGetXmDisplayClass(void)
 {
     if (__XmDisplayClass == NULL)
     {
-	DEBUGOUT(XdbDebug0(__FILE__, NULL, "%s:_XmGetXmDisplayClass(%d)\n",
+	DEBUGOUT(_LtDebugDebug0(__FILE__, NULL, "%s:_XmGetXmDisplayClass(%d)\n",
 	    __FILE__, __LINE__));
 	__XmDisplayClass = (WidgetClass)&xmDisplayClassRec;
     }
@@ -645,7 +656,7 @@ _XmSetXmDisplayClass(WidgetClass wc)
 {
     WidgetClass OldDisplayClass, SuperClass;
 
-    DEBUGOUT(XdbDebug0(__FILE__, NULL, "%s:_XmSetXmDisplayClass(%d)\n",
+    DEBUGOUT(_LtDebugDebug0(__FILE__, NULL, "%s:_XmSetXmDisplayClass(%d)\n",
     	__FILE__, __LINE__));
     OldDisplayClass = __XmDisplayClass;
     SuperClass = wc;
@@ -673,9 +684,9 @@ Widget
 XmGetDragContext(Widget w, Time time)
 {
     Widget disp = XmGetXmDisplay(XtDisplay(w)), dc = NULL;
-    int i;
+    Cardinal i;
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "%s:XmGetDragContext(%d)\n",
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "%s:XmGetDragContext(%d)\n",
     	__FILE__, __LINE__));
     for (i = 0; i < MGR_NumChildren(disp); i++)
     {
