@@ -1,8 +1,9 @@
 /**
  *
- * $Id: DragIcon.c,v 1.12 2000/01/05 23:44:45 rwscott Exp $
+ * $Id: DragIcon.c,v 1.16 2001/05/17 13:04:49 amai Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
+ * Copyright (C) 1995-2001 LessTif Development Team 
  *
  * This file is part of the GNU LessTif Library.
  *
@@ -22,11 +23,11 @@
  *
  **/
 
-static const char rcsid[] = "$Id: DragIcon.c,v 1.12 2000/01/05 23:44:45 rwscott Exp $";
+static const char rcsid[] = "$Id: DragIcon.c,v 1.16 2001/05/17 13:04:49 amai Exp $";
 
 #include <LTconfig.h>
-#include <XmI/XmI.h>
 
+#include <XmI/XmI.h>
 #include <Xm/XmP.h>
 #include <Xm/DragDrop.h>
 #include <Xm/DragIconP.h>
@@ -581,7 +582,7 @@ static XmQuarkToCursorRec quarkToCursorTable[] =
 static void
 class_initialize(void)
 {
-    DEBUGOUT(XdbDebug0(__FILE__, NULL, "%s:class_initialize(%d)\n",
+    DEBUGOUT(_LtDebugDebug0(__FILE__, NULL, "%s:class_initialize(%d)\n",
     	__FILE__, __LINE__));
     _XmDragIconRectClassExtRec.record_type = XmQmotif;
 }
@@ -589,7 +590,7 @@ class_initialize(void)
 static void
 class_part_initialize(WidgetClass widget_class)
 {
-    DEBUGOUT(XdbDebug0(__FILE__, NULL, "%s:class_part_initialize(%d)\n",
+    DEBUGOUT(_LtDebugDebug0(__FILE__, NULL, "%s:class_part_initialize(%d)\n",
     	__FILE__, __LINE__));
     _XmFastSubclassInit(widget_class, XmDRAG_ICON_BIT);
 }
@@ -606,7 +607,7 @@ initialize(Widget request, Widget new_w,
     unsigned int width, height;
     XImage *image;
 
-    DEBUGOUT(XdbDebug(__FILE__, new_w,
+    DEBUGOUT(_LtDebugDebug(__FILE__, new_w,
 		      "%s:initialize(%d) - %i args\n"
 		      "\trequest X %5i Y %5i W %5i H %5i\n"
 		      "\t  new_w X %5i Y %5i W %5i H %5i\n",
@@ -616,7 +617,7 @@ initialize(Widget request, Widget new_w,
 		      XtWidth(request), XtHeight(request),
 		      XtX(new_w), XtY(new_w),
 		      XtWidth(new_w), XtHeight(new_w)));
-    DEBUGOUT(XdbPrintArgList(__FILE__, new_w, args, *num_args, False));
+    DEBUGOUT(_LtDebugPrintArgList(__FILE__, new_w, args, *num_args, False));
 
     pscr = XtScreenOfObject(XtParent(new_w));
 
@@ -754,7 +755,7 @@ initialize(Widget request, Widget new_w,
 static void
 destroy(Widget w)
 {
-    DEBUGOUT(XdbDebug(__FILE__, w, "%s:destroy(%d)\n",
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "%s:destroy(%d)\n",
     	__FILE__, __LINE__));
 
     if (DI_Region(w))
@@ -776,7 +777,7 @@ set_values(Widget current, Widget request, Widget new_w,
 {
     Boolean refresh = False;
 
-    DEBUGOUT(XdbDebug(__FILE__, new_w,
+    DEBUGOUT(_LtDebugDebug(__FILE__, new_w,
 		      "%s:set_values(%d) - %i args\n"
 		      "\t    old X %5i Y %5i W %5i H %5i\n"
 		      "\trequest X %5i Y %5i W %5i H %5i\n"
@@ -789,7 +790,7 @@ set_values(Widget current, Widget request, Widget new_w,
 		      XtWidth(request), XtHeight(request),
 		      XtX(new_w), XtY(new_w),
 		      XtWidth(new_w), XtHeight(new_w)));
-    DEBUGOUT(XdbPrintArgList(__FILE__, new_w, args, *num_args, False));
+    DEBUGOUT(_LtDebugPrintArgList(__FILE__, new_w, args, *num_args, False));
 
     if (DI_Depth(new_w) != DI_Depth(current) ||
 	DI_Width(new_w) != DI_Width(current) ||
@@ -821,7 +822,7 @@ _XmGetTextualDragIcon(Widget w)
 {
     Widget widget;
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "_XmGetTextualDragIcon\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "_XmGetTextualDragIcon\n"));
 
     if (textIconContext == None)
     {
@@ -838,7 +839,7 @@ _XmGetTextualDragIcon(Widget w)
 			      NULL, 0);
 
     XSaveContext(XtDisplay(w), XmTextualDragIconRID, textIconContext,
-		 (_Xconst char *)widget);
+		 (XPointer)widget);
 
     return widget;
 }
@@ -848,7 +849,7 @@ _XmDestroyDefaultDragIcon(XmDragIconObject icon)
 {
     Screen *scr = XtScreenOfObject(XtParent(icon));
 
-    DEBUGOUT(XdbDebug0(__FILE__, NULL, "_XmDestroyDefaultDragIcon\n"));
+    DEBUGOUT(_LtDebugDebug0(__FILE__, NULL, "_XmDestroyDefaultDragIcon\n"));
 
     if (DI_Pixmap(icon) != XmUNSPECIFIED_PIXMAP)
     {
@@ -868,7 +869,7 @@ _XmDestroyDefaultDragIcon(XmDragIconObject icon)
 Boolean
 _XmDragIconIsDirty(XmDragIconObject icon)
 {
-    DEBUGOUT(XdbDebug0(__FILE__, NULL, "_XmDragIconIsDirty\n"));
+    DEBUGOUT(_LtDebugDebug0(__FILE__, NULL, "_XmDragIconIsDirty\n"));
 
     return DI_IsDirty(icon);
 }
@@ -878,7 +879,7 @@ _XmDragIconClean(XmDragIconObject icon1,
 		 XmDragIconObject icon2,
 		 XmDragIconObject icon3)
 {
-    DEBUGOUT(XdbDebug0(__FILE__, NULL, "_XmDragIconClean\n"));
+    DEBUGOUT(_LtDebugDebug0(__FILE__, NULL, "_XmDragIconClean\n"));
 
     if (icon1)
 	DI_IsDirty(icon1) = False;
@@ -894,7 +895,7 @@ XmCreateDragIcon(Widget widget,
 		 ArgList arglist,
 		 Cardinal argcount)
 {
-    DEBUGOUT(XdbDebug(__FILE__, widget, "XmCreateDragIcon\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, widget, "XmCreateDragIcon\n"));
 
     return XtCreateWidget(name, xmDragIconObjectClass,
 			  widget, arglist, argcount);
