@@ -1,9 +1,10 @@
-/** -*- Mode: C; tab-width: 8; indent-tabs-mode: nil -*-
+/**
  *
- * $Id: DragOverS.c,v 1.17 2000/01/05 23:44:45 rwscott Exp $
+ * $Id: DragOverS.c,v 1.22 2001/05/14 19:46:08 amai Exp $
  *
  * derived from Xt Vendor class.c 
  *
+ * Copyright (C) 1995-2001 LessTif Development Team 
  * Copyright (C) 1995 Free Software Foundation, Inc.
  * Copyright 1989 Massachusetts Institute of Technology
  *
@@ -25,11 +26,13 @@
  *
  **/
 
-static const char rcsid[] = "$Id: DragOverS.c,v 1.17 2000/01/05 23:44:45 rwscott Exp $";
+static const char rcsid[] = "$Id: DragOverS.c,v 1.22 2001/05/14 19:46:08 amai Exp $";
 
 #include <LTconfig.h>
-#include <XmI/XmI.h>
 
+#include <stdlib.h>
+
+#include <XmI/XmI.h>
 #include <Xm/XmP.h>
 #include <Xm/XmosP.h>
 #include <Xm/VendorSP.h>
@@ -200,7 +203,7 @@ class_initialize()
 static void 
 class_part_initialize(WidgetClass widget_class)
 {
-    DEBUGOUT(XdbDebug0(__FILE__, NULL, "%s:class_part_initialize(%d)\n",
+    DEBUGOUT(_LtDebugDebug0(__FILE__, NULL, "%s:class_part_initialize(%d)\n",
     	__FILE__, __LINE__));
     _XmFastSubclassInit(widget_class, XmDRAG_OVER_SHELL_BIT);
 }
@@ -210,7 +213,7 @@ static void
 initialize(Widget request, Widget new_w,
 	   ArgList args, Cardinal *num_args)
 {
-    DEBUGOUT(XdbDebug(__FILE__, new_w,
+    DEBUGOUT(_LtDebugDebug(__FILE__, new_w,
 		      "%s:initialize(%d) - %i args\n"
 		      "\trequest X %5i Y %5i W %5i H %5i\n"
 		      "\t  new_w X %5i Y %5i W %5i H %5i\n",
@@ -220,7 +223,7 @@ initialize(Widget request, Widget new_w,
 		      XtWidth(request), XtHeight(request),
 		      XtX(new_w), XtY(new_w),
 		      XtWidth(new_w), XtHeight(new_w)));
-    DEBUGOUT(XdbPrintArgList(__FILE__, new_w, args, *num_args, False));
+    DEBUGOUT(_LtDebugPrintArgList(__FILE__, new_w, args, *num_args, False));
 
     XtWidth(new_w) = 0;
     XtHeight(new_w) = 0;
@@ -265,7 +268,7 @@ initialize(Widget request, Widget new_w,
 static void
 destroy(Widget w)
 {
-    DEBUGOUT(XdbDebug(__FILE__, w, "%s:destroy(%d)\n",
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "%s:destroy(%d)\n",
     	__FILE__, __LINE__));
 }
 
@@ -275,7 +278,7 @@ set_values(Widget old, Widget request, Widget new_w,
 {
     Widget dc = XtParent(new_w);	/* this had BETTER be a DragContext */
 
-    DEBUGOUT(XdbDebug(__FILE__, new_w,
+    DEBUGOUT(_LtDebugDebug(__FILE__, new_w,
 		      "%s:set_values(%d) - %i args\n"
 		      "\t    old X %5i Y %5i W %5i H %5i\n"
 		      "\trequest X %5i Y %5i W %5i H %5i\n"
@@ -288,7 +291,7 @@ set_values(Widget old, Widget request, Widget new_w,
 		      XtWidth(request), XtHeight(request),
 		      XtX(new_w), XtY(new_w),
 		      XtWidth(new_w), XtHeight(new_w)));
-    DEBUGOUT(XdbPrintArgList(__FILE__, new_w, args, *num_args, False));
+    DEBUGOUT(_LtDebugPrintArgList(__FILE__, new_w, args, *num_args, False));
 
     if (DO_Mode(new_w) != DO_Mode(old))
     {
@@ -314,7 +317,7 @@ set_values(Widget old, Widget request, Widget new_w,
 static void
 expose(Widget w, XEvent *event, Region region)
 {
-    DEBUGOUT(XdbDebug(__FILE__, w, "%s:expose(%d)\n",
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "%s:expose(%d)\n",
     	__FILE__, __LINE__));
 }
 
@@ -543,7 +546,7 @@ static void
 copy_to_window(Widget w, XmDragIconObject mixedIcon, Window win,
 	       Position x, Position y)
 {
-    DEBUGOUT(XdbDebug(__FILE__, w, "XmDragOver: COPY TO WINDOW\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "XmDragOver: COPY TO WINDOW\n"));
 
     XSetFunction(XtDisplay(w), DO_RootBlend(w).gc, GXcopy);
 
@@ -1407,7 +1410,7 @@ change_window(Widget w)
 static void
 melt_cursor(Widget w)
 {
-  DEBUGOUT(XdbDebug( __FILE__, w, "Melting not implemented.\n" ));
+  DEBUGOUT(_LtDebugDebug( __FILE__, w, "Melting not implemented.\n" ));
 }
 
 /*
@@ -1426,7 +1429,7 @@ zap_cursor(Widget w)
     int zap_count, idx;
     int adjust_w, adjust_h;
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "zap_cursor is implemented.\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "zap_cursor is implemented.\n"));
     zap_count = 0;
     gc = DO_RootBlend(w).gc;
 
@@ -1450,7 +1453,7 @@ zap_cursor(Widget w)
     adjust_h = (DO_InitialY(w) - (XtY(w) + (XtHeight(w)>>1))) / ZAP_EFFECT2;
     adjust_w = (DO_InitialX(w) - (XtX(w) + (XtWidth(w)>>1))) / ZAP_EFFECT2;
 
-    gc_values.foreground = 1;
+    gc_values.foreground = (~0);
     gc_values.function = GXxor;
     gc_values.clip_mask = (Pixmap)NULL;
     XChangeGC( DisplayOfScreen(screen), gc,
@@ -1509,7 +1512,7 @@ zap_cursor(Widget w)
     	    RootWindowOfScreen(screen),
 	    segments[0].x2, segments[0].y2 );
 
-    	gc_values.foreground = 1;
+    	gc_values.foreground = (~0);
     	gc_values.function = GXxor;
     	XChangeGC( DisplayOfScreen(screen), gc,
 	    	GCFunction | GCForeground,
@@ -1532,7 +1535,7 @@ _XmDragOverHide(Widget w, Position clipOriginX, Position clipOriginY,
     Widget dc = XtParent(w);	/* this had BETTER be a DragContext */
     Boolean clip_on = False;
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "%s:_XmDragOverHide(%d)\n",
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "%s:_XmDragOverHide(%d)\n",
     	__FILE__, __LINE__));
     if (!DO_IsVisible(w) || DC_BlendModel(dc) == XmBLEND_NONE ||
 	DO_ActiveMode(w) == XmCURSOR)
@@ -1581,7 +1584,7 @@ _XmDragOverShow(Widget w, Position clipOriginX, Position clipOriginY,
     Widget dc = XtParent(w);	/* this had BETTER be a DragContext */
     Boolean clip_on = False;
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "%s:_XmDragOverShow(%d)\n",
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "%s:_XmDragOverShow(%d)\n",
     	__FILE__, __LINE__));
     if (!DO_IsVisible(w) || DC_BlendModel(dc) == XmBLEND_NONE ||
 	DO_ActiveMode(w) == XmCURSOR)
@@ -1650,7 +1653,7 @@ _XmDragOverMove(Widget w, Position x, Position y)
     DO_HotX(w) = x;
     DO_HotY(w) = y;
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "XmDragOverMove\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "XmDragOverMove\n"));
 
     if (!DO_IsVisible(w) || DC_BlendModel(dc) == XmBLEND_NONE ||
 	DO_ActiveMode(w) == XmCURSOR)
@@ -1728,7 +1731,7 @@ _XmDragOverChange(Widget w, unsigned char dropSiteStatus)
     XmDragIconObject src_icon = NULL, op_icon = NULL, state_icon = NULL;
     Boolean src_pix = True, dirty, change;
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "%s:_XmDragOverChange(%d)\n",
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "%s:_XmDragOverChange(%d)\n",
     	__FILE__, __LINE__));
     DO_CursorState(w) = dropSiteStatus;
 
@@ -1909,7 +1912,7 @@ _XmDragOverFinish(Widget w, unsigned char completionStatus)
     Widget dc = XtParent(w);	/* this had BETTER be a drag context */
     Display *disp = XtDisplay(w);
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "%s:_XmDragOverFinish(%d)\n",
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "%s:_XmDragOverFinish(%d)\n",
     	__FILE__, __LINE__));
     if (DC_BlendModel(dc) == XmBLEND_NONE)
     {
@@ -1939,7 +1942,7 @@ _XmDragOverFinish(Widget w, unsigned char completionStatus)
 Cursor
 _XmDragOverGetActiveCursor(Widget w)
 {
-    DEBUGOUT(XdbDebug(__FILE__, w, "%s:_XmDragOverGetActiveCursor(%d)\n",
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "%s:_XmDragOverGetActiveCursor(%d)\n",
     	__FILE__, __LINE__));
     return DO_ActiveCursor(w);
 }
@@ -1947,7 +1950,7 @@ _XmDragOverGetActiveCursor(Widget w)
 void
 _XmDragOverSetInitialPosition(Widget w, Position initialX, Position initialY)
 {
-    DEBUGOUT(XdbDebug(__FILE__, w, "%s:_XmDragOverSetInitialPosition(%d)\n",
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "%s:_XmDragOverSetInitialPosition(%d)\n",
     	__FILE__, __LINE__));
     DO_InitialX(w) = initialX;
     DO_InitialY(w) = initialY;
