@@ -1,8 +1,9 @@
 /**
  *
- * $Id: DragUnder.c,v 1.4 2000/01/05 23:44:46 rwscott Exp $
+ * $Id: DragUnder.c,v 1.6 2001/05/14 19:46:08 amai Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
+ * Copyright (C) 1995-2001 LessTif Development Team 
  *
  * This file is part of the GNU LessTif Library.
  *
@@ -22,19 +23,20 @@
  *
  **/
 
-static const char rcsid[] = "$Id: DragUnder.c,v 1.4 2000/01/05 23:44:46 rwscott Exp $";
+static const char rcsid[] = "$Id: DragUnder.c,v 1.6 2001/05/14 19:46:08 amai Exp $";
 
 #include <LTconfig.h>
-#include <XmI/XmI.h>
 
+#include <XmI/XmI.h>
 #include <Xm/XmP.h>
 #include <XmI/DragDropI.h>
 
 #include <XmI/DebugUtil.h>
 
+
 void
 _XmDragUnderAnimation(Widget w, XtPointer clientData, XtPointer callData)
 {
-    DEBUGOUT(XdbDebug(__FILE__, w, "%s:_XmDragUnderAnimation(%d)\n",
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "%s:_XmDragUnderAnimation(%d)\n",
     	__FILE__, __LINE__));
 }
