@@ -1,8 +1,9 @@
 /**
  *
- * $Id: DrawnB.c,v 1.39 1999/09/11 20:06:34 rwscott Exp $
+ * $Id: DrawnB.c,v 1.44 2001/05/20 17:49:38 amai Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
+ * Copyright (C) 1995-2001 LessTif Development Team 
  *
  * This file is part of the GNU LessTif Library.
  *
@@ -22,11 +23,11 @@
  *
  **/
 
-static const char rcsid[] = "$Id: DrawnB.c,v 1.39 1999/09/11 20:06:34 rwscott Exp $";
+static const char rcsid[] = "$Id: DrawnB.c,v 1.44 2001/05/20 17:49:38 amai Exp $";
 
 #include <LTconfig.h>
-#include <XmI/XmI.h>
 
+#include <XmI/XmI.h>
 #include <Xm/XmP.h>
 #include <Xm/DrawnBP.h>
 #include <Xm/RepType.h>
@@ -53,7 +54,7 @@ static XmActivatableTraitRec _XmDrawnBTraitRec = {
 
 /* Forward Declarations */
 #if XmVERSION > 1
-static void class_initialize();
+static void class_initialize(void);
 #endif
 static void class_part_initialize(WidgetClass w_class);
 
@@ -257,10 +258,10 @@ WidgetClass xmDrawnButtonWidgetClass = (WidgetClass)&xmDrawnButtonClassRec;
 
 #if XmVERSION > 1
 static void
-class_initialize()
+class_initialize(void)
 {
-    if (! XmeTraitSet(xmDrawnButtonWidgetClass, XmQTactivatable,
-		&_XmDrawnBTraitRec)) {
+    if (! XmeTraitSet((XtPointer)xmDrawnButtonWidgetClass, XmQTactivatable,
+		(XtPointer)&_XmDrawnBTraitRec)) {
 	_XmWarning(NULL,
 	    "XmDrawnButton ClassInitialize: XmeTraitSet failed\n");
     }
@@ -277,7 +278,7 @@ static void
 initialize(Widget request, Widget new_w,
 	   ArgList args, Cardinal *num_args)
 {
-    DEBUGOUT(XdbDebug(__FILE__, new_w,
+    DEBUGOUT(_LtDebugDebug(__FILE__, new_w,
 		      "initialize: %i args\n"
 		      "\trequest X %5i Y %5i W %5i H %5i\n"
 		      "\t  new_w X %5i Y %5i W %5i H %5i\n",
@@ -286,7 +287,7 @@ initialize(Widget request, Widget new_w,
 		      XtWidth(request), XtHeight(request),
 		      XtX(new_w), XtY(new_w),
 		      XtWidth(new_w), XtHeight(new_w)));
-    DEBUGOUT(XdbPrintArgList(__FILE__, new_w, args, *num_args, False));
+    DEBUGOUT(_LtDebugPrintArgList(__FILE__, new_w, args, *num_args, False));
 
     /* check the RepType resources */
     if (!XmRepTypeValidValue(XmRepTypeGetId(XmRMultiClick),
@@ -306,6 +307,7 @@ destroy(Widget w)
     if (DB_Timer(w) != 0)
     {
 	XtRemoveTimeOut(DB_Timer(w));
+	DB_Timer(w) = 0;
     }
 }
 
@@ -331,7 +333,7 @@ set_values(Widget old, Widget request, Widget new_w,
 {
     Boolean refresh_needed = False;
 
-    DEBUGOUT(XdbDebug(__FILE__, new_w,
+    DEBUGOUT(_LtDebugDebug(__FILE__, new_w,
 		      "set_values: %i args\n"
 		      "\t    old X %5i Y %5i W %5i H %5i\n"
 		      "\trequest X %5i Y %5i W %5i H %5i\n"
@@ -343,7 +345,7 @@ set_values(Widget old, Widget request, Widget new_w,
 		      XtWidth(request), XtHeight(request),
 		      XtX(new_w), XtY(new_w),
 		      XtWidth(new_w), XtHeight(new_w)));
-    DEBUGOUT(XdbPrintArgList(__FILE__, new_w, args, *num_args, False));
+    DEBUGOUT(_LtDebugPrintArgList(__FILE__, new_w, args, *num_args, False));
 
     /* check the RepType resources */
     if (!XmRepTypeValidValue(XmRepTypeGetId(XmRMultiClick),
@@ -384,7 +386,7 @@ expose(Widget w, XEvent *event, Region region)
 {
     XmDrawnButtonCallbackStruct cbs;
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "Expose (%dx%d%+d%+d)\n",
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "Expose (%dx%d%+d%+d)\n",
 		      XtWidth(w), XtHeight(w),XtX(w),XtY(w)));
 
 #define superclass (&xmLabelClassRec)
@@ -455,7 +457,7 @@ resize(Widget w)
 {
     XmDrawnButtonCallbackStruct cbs;
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "Resize (%dx%d%+d%+d)\n",
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "Resize (%dx%d%+d%+d)\n",
 		      XtWidth(w), XtHeight(w),XtX(w),XtY(w)));
 
 #define superclass (&xmLabelClassRec)
@@ -576,7 +578,7 @@ ArmTimeout(XtPointer data, XtIntervalId *id)
 {
     Widget w = (Widget)data;
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "ArmTimeout\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "ArmTimeout\n"));
 
     DB_Timer(w) = 0;
 
@@ -592,7 +594,7 @@ ArmAndActivate(Widget w, XEvent *event, String *params, Cardinal *num_params)
 {
     XmDrawnButtonCallbackStruct cbs;
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "ArmAndActivate\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "ArmAndActivate\n"));
 
     /* Arm, Activate, and Disarm now, but draw the disarmed state later */
 
@@ -624,6 +626,7 @@ ArmAndActivate(Widget w, XEvent *event, String *params, Cardinal *num_params)
     if (DB_Timer(w) != 0)
     {
 	XtRemoveTimeOut(DB_Timer(w));
+	DB_Timer(w) = 0;
     }
 
     DB_Timer(w) = XtAppAddTimeOut(XtWidgetToApplicationContext(w),
@@ -671,7 +674,7 @@ LeaveWindow(Widget w, XEvent *event, String *params, Cardinal *num_params)
 static void
 MultiArm(Widget w, XEvent *event, String *params, Cardinal *num_params)
 {
-    DEBUGOUT(XdbDebug(__FILE__, w, "MultiArm\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "MultiArm\n"));
 
     if (DB_MultiClick(w) == XmMULTICLICK_KEEP)
     {
@@ -685,7 +688,7 @@ MultiActivate(Widget w, XEvent *event, String *params, Cardinal *num_params)
     XButtonEvent *ev = (XButtonEvent *)event;
     XmDrawnButtonCallbackStruct cbs;
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "DrawnB: MultiClick\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "DrawnB: MultiClick\n"));
 
     if (DB_MultiClick(w) == XmMULTICLICK_KEEP)
     {
