@@ -1,8 +1,9 @@
-/** -*- Mode: C; tab-width: 8; indent-tabs-mode: nil -*-
+/*
  *
- * $Id: DropSMgrI.c,v 1.3 2000/01/05 23:44:47 rwscott Exp $
+ * $Id: DropSMgrI.c,v 1.6 2001/05/14 19:46:08 amai Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
+ * Copyright (C) 1995-2001 LessTif Development Team
  *
  * This file is part of the GNU LessTif Library.
  *
@@ -22,9 +23,10 @@
  *
  **/
 
-static const char rcsid[] = "$Id: DropSMgrI.c,v 1.3 2000/01/05 23:44:47 rwscott Exp $";
+static const char rcsid[] = "$Id: DropSMgrI.c,v 1.6 2001/05/14 19:46:08 amai Exp $";
 
 #include <LTconfig.h>
+
 #include <XmI/XmI.h>
 
 #include <Xm/DropSMgrP.h>
@@ -33,10 +35,10 @@ static const char rcsid[] = "$Id: DropSMgrI.c,v 1.3 2000/01/05 23:44:47 rwscott
 #include <Xm/DragCP.h>
 #include <Xm/DropTrans.h>
 #include <Xm/XmP.h>
-#include <X11/Xfuncs.h>
 
 #include <XmI/DebugUtil.h>
 
+
 #define Offset(f)	XtOffsetOf(XmDropSiteInfoRec, f)
 XtResource _XmDSResources[] =
 {
@@ -110,56 +112,62 @@ XtResource _XmDSResources[] =
 
 int _XmNumDSResources = XtNumber(_XmDSResources);
 
+
 /* netscape 4.5PR1 wants these; dunno what they're supposed to do -jac */
-void 
+extern void 
 _XmDSIAddChild(void* a, void* b, int c)
 {
-    DEBUGOUT(XdbDebug(__FILE__, NULL, "_XmDSIAddChild(%p, %p, %d) - stub!\n",
+    DEBUGOUT(_LtDebugDebug(__FILE__, NULL, "_XmDSIAddChild(%p, %p, %d) - stub!\n",
 		      a, b, c));
 }
 
-void 
+
+extern void 
 _XmDSIRemoveChild(void* a, void* b)
 {
-    DEBUGOUT(XdbDebug0(__FILE__, NULL, "_XmDSIRemoveChild(%p, %p) - stub!\n",
+    DEBUGOUT(_LtDebugDebug0(__FILE__, NULL, "_XmDSIRemoveChild(%p, %p) - stub!\n",
 		      a, b));
 }
 
-void 
+
+extern void 
 _XmDSIDestroy(void* a, int b)
 {
-    DEBUGOUT(XdbDebug0(__FILE__, NULL, "_XmDSIDestroy(%p, %d) - stub!\n",
+    DEBUGOUT(_LtDebugDebug0(__FILE__, NULL, "_XmDSIDestroy(%p, %d) - stub!\n",
 		      a, b));
 }
 
-int 
+
+extern int 
 _XmDSIGetBorderWidth(void* a)
 {
-    DEBUGOUT(XdbDebug0(__FILE__, NULL, "_XmDSIGetBorderWidth(%p) - stub!\n",
+    DEBUGOUT(_LtDebugDebug0(__FILE__, NULL, "_XmDSIGetBorderWidth(%p) - stub!\n",
 		      a));
     return 0;
 }
 
-int 
+
+extern int 
 _XmDSIReplaceChild(void* a)
 {
-    DEBUGOUT(XdbDebug0(__FILE__, NULL, "_XmDSIReplaceChild(%p) - stub!\n",
+    DEBUGOUT(_LtDebugDebug0(__FILE__, NULL, "_XmDSIReplaceChild(%p) - stub!\n",
 		      a));
     return 0;
 }
 
-int 
+
+extern int
 _XmDSIGetChildPosition(void* a, void* b)
 {
-    DEBUGOUT(XdbDebug0(__FILE__, NULL, "_XmDSIGetChildPosition(%p) - stub!\n",
+    DEBUGOUT(_LtDebugDebug0(__FILE__, NULL, "_XmDSIGetChildPosition(%p) - stub!\n",
 		      a, b));
     return 0;
 }
 
-void 
+
+extern void 
 _XmDSISwapChildren(void* a, void* b, void* c)
 {
-    DEBUGOUT(XdbDebug0(__FILE__, NULL, "_XmDSISwapChildren(%p, %p, %p) - stub!\n",
+    DEBUGOUT(_LtDebugDebug0(__FILE__, NULL, "_XmDSISwapChildren(%p, %p, %p) - stub!\n",
 		      a, b, c));
 }
-
