@@ -1,11 +1,9 @@
-#define MOZILLA
-#undef PATTERN_DANNY
-#undef	SAVE_PATTERN
 /**
  *
- * $Header: /cvsroot/hungry/lesstif/lib/Xm/FileSB.c,v 1.78 1999/08/17 22:09:50 rwscott Exp $
+ * $Header: /cvsroot/lesstif/lesstif/lib/Xm/FileSB.c,v 1.105 2001/05/14 19:46:08 amai Exp $
  *
  * Copyright (C) 1997 Free Software Foundation, Inc.
+ * Copyright (C) 1997-2001 LessTif Development Team
  *
  * This file is part of the GNU LessTif Library.
  *
@@ -25,14 +23,23 @@
  *
  **/
 
-static const char rcsid[] = "$Header: /cvsroot/hungry/lesstif/lib/Xm/FileSB.c,v 1.78 1999/08/17 22:09:50 rwscott Exp $";
+static const char rcsid[] = "$Header: /cvsroot/lesstif/lesstif/lib/Xm/FileSB.c,v 1.105 2001/05/14 19:46:08 amai Exp $";
 
 #include <LTconfig.h>
-#include <XmI/XmI.h>
-#include <XmI/MacrosI.h>
 
 #include <stdlib.h>
+#include <string.h>
+#ifdef HAVE_UNISTD_H
 #include <unistd.h>
+#endif
+#ifdef HAVE_LIBGEN_H
+#include <libgen.h>    /* for basename() */
+#else
+#include "LTmisc.h"
+#endif
+
+#include <XmI/XmI.h>
+#include <XmI/MacrosI.h>
 #include <Xm/XmP.h>
 #include <Xm/List.h>
 #include <Xm/SelectioBP.h>
@@ -45,21 +52,22 @@ static const char rcsid[] = "$Header: /cvsroot/hungry/lesstif/lib/Xm/FileSB.c,v
 #include <Xm/TextF.h>
 #include <Xm/DialogS.h>
 #include <Xm/TransltnsP.h>
-#include <X11/Xfuncs.h>
 
 #include <XmI/DebugUtil.h>
 
+
+/* #defines being used: */
+
+/* see below for more info */
+#define MOZILLA
+
 /* Forward Declarations */
 
 static void class_initialize(void);
-
 static void class_part_initialize(WidgetClass w_class);
-
 static void initialize(Widget request, Widget new_w,
 		       ArgList args, Cardinal *num_args);
-
 static void destroy(Widget w);
-
 static Boolean set_values(Widget current, Widget request, Widget new_w,
 			  ArgList args, Cardinal *num_args);
 
@@ -70,23 +78,14 @@ static void get_values_hook(Widget w, ArgList args, Cardinal *num_args);
 
 static XmGeoMatrix geoMatrixCreate(Widget _w, Widget _from,
 				   XtWidgetGeometry *_pref);
-
 static void focus_moved_proc(Widget wid, XtPointer client_data, XtPointer data);
-
 static Boolean noGeoRequest(XmGeoMatrix _geoSpec);
-
 static void defaultDirSearchProc(Widget widget, XtPointer search_data);
-
 static void defaultFileSearchProc(Widget widget, XtPointer search_data);
-
 static void defaultQualifySearchDataProc(Widget widget, XtPointer input_data, XtPointer output_data);
-
 static void _XmFileSelectionSearch(Widget w);
-
 static void _XmFsbButton(Widget w, XtPointer client, XtPointer call);
-
 static void _XmFsbFileSelect(Widget w, XtPointer client, XtPointer call);
-
 static XtAccelerators text_accelerators_parsed = NULL;
 
 
@@ -201,29 +200,33 @@ static XtResource resources[] =
 	XmNfileListItemCount, XmCItemCount, XmRInt,
 	sizeof(int), SBOffset(list_item_count),
 	XmRImmediate, (XtPointer)XmUNSPECIFIED
-    }
+    },
+#if XmVERSION > 1
+    {
+	XmNpathMode, XmCPathMode, XmRPathMode,
+	sizeof(XtEnum), Offset(path_mode),
+	XmRImmediate, (XtPointer)XmPATH_MODE_FULL
+    },
+    {
+	XmNfileFilterStyle, XmCFileFilterStyle, XmRFileFilterStyle,
+	sizeof(XtEnum), Offset(file_filter_style),
+	XmRImmediate, (XtPointer)XmFILTER_NONE
+    },
+#endif
 };
 
 /* Synthetic resources */
-void FSBGetDirListItemCount(Widget wid, int resource_offset, XtArgVal *value);
+static void FSBGetDirListItemCount(Widget wid, int resource_offset, XtArgVal *value);
+static void FSBGetDirListItems(Widget wid, int resource_offset, XtArgVal *value);
+static void FSBGetDirListLabelString(Widget wid, int resource_offset, XtArgVal *value);
+static void FSBGetDirMask(Widget wid, int resource_offset, XtArgVal *value);
+static void FSBGetDirectory(Widget wid, int resource_offset, XtArgVal *value);
+static void FSBGetFilterLabelString(Widget wid, int resource_offset, XtArgVal *value);
+static void FSBGetListItemCount(Widget wid, int resource_offset, XtArgVal *value);
+static void FSBGetListItems(Widget wid, int resource_offset, XtArgVal *value);
+static void FSBGetNoMatchString(Widget wid, int resource_offset, XtArgVal *value);
+static void FSBGetPattern(Widget wid, int resource_offset, XtArgVal *value);
 
-void FSBGetDirListItems(Widget wid, int resource_offset, XtArgVal *value);
-
-void FSBGetDirListLabelString(Widget wid, int resource_offset, XtArgVal *value);
-
-void FSBGetDirMask(Widget wid, int resource_offset, XtArgVal *value);
-
-void FSBGetDirectory(Widget wid, int resource_offset, XtArgVal *value);
-
-void FSBGetFilterLabelString(Widget wid, int resource_offset, XtArgVal *value);
-
-void FSBGetListItemCount(Widget wid, int resource_offset, XtArgVal *value);
-
-void FSBGetListItems(Widget wid, int resource_offset, XtArgVal *value);
-
-void FSBGetNoMatchString(Widget wid, int resource_offset, XtArgVal *value);
-
-void FSBGetPattern(Widget wid, int resource_offset, XtArgVal *value);
 
 static XmSyntheticResource syn_resources[] =
 {
@@ -299,7 +302,6 @@ static XtActionsRec actions[] =
 	{"SelectionBoxRestore", FileSelectionBoxRestore},
 };
 
-/* *INDENT-OFF* */
 #if 0
 static XmBaseClassExtRec _XmFileSBCoreClassExtRec = {
     /* next_extension            */ NULL,
@@ -402,14 +404,14 @@ XmFileSelectionBoxClassRec xmFileSelectionBoxClassRec = {
     /* XmSelectionBox part */
     {
 	/* Motif has something called a list_callback here */
-	/* extension */ NULL,
+	/* list_callback                */ NULL,
+	/* extension                    */ NULL,
     },
     /* XmFileSelectionBox part */
     {
-	/* extension */ NULL,
+	/* extension                    */ NULL,
     }
 };
-/* *INDENT-ON* */
 
 
 WidgetClass xmFileSelectionBoxWidgetClass = (WidgetClass)&xmFileSelectionBoxClassRec;
@@ -445,8 +447,9 @@ initialize(Widget request, Widget new_w,
     int argc;
     XmString files = NULL;
     XtTranslations save_acc;
+    int dir_set_flag;
 
-    DEBUGOUT(XdbDebug(__FILE__, new_w,
+    DEBUGOUT(_LtDebugDebug(__FILE__, new_w,
 		      "initialize: %i args\n"
 		      "\trequest X %5i Y %5i W %5i H %5i\n"
 		      "\t  new_w X %5i Y %5i W %5i H %5i\n",
@@ -455,7 +458,7 @@ initialize(Widget request, Widget new_w,
 		      XtWidth(request), XtHeight(request),
 		      XtX(new_w), XtY(new_w),
 		      XtWidth(new_w), XtHeight(new_w)));
-    DEBUGOUT(XdbPrintArgList(__FILE__, new_w, args, *num_args, False));
+    DEBUGOUT(_LtDebugPrintArgList(__FILE__, new_w, args, *num_args, False));
 
     SB_DialogType(new_w) = XmDIALOG_FILE_SELECTION;
     FS_StateFlags(new_w) = 0;
@@ -687,6 +690,7 @@ initialize(Widget request, Widget new_w,
      *
      *		--mu@trends.net, 1998.07.04
      */
+    dir_set_flag = 0;
     if (_XmStringIsSpecified(FS_DirMask(new_w)))
     {
 	String mask;
@@ -696,12 +700,6 @@ initialize(Widget request, Widget new_w,
             String pat = _XmOSFindPatternPart(mask);
 	    if (!_XmStringIsSpecified(FS_Pattern(new_w))) {
 		FS_Pattern(new_w) = XmStringCreateLocalized(pat);
-#ifdef PATTERN_DANNY
-		fprintf(stderr,
-			"FileSB Initialize() pattern uninitialised, "
-			"take mask '%s'\n",
-			mask);
-#endif
 	    }
 
             if (pat > mask && !_XmStringIsSpecified(FS_Directory(new_w)))
@@ -714,10 +712,21 @@ initialize(Widget request, Widget new_w,
         	 */
         	*pat = '\0';
         	FS_Directory(new_w) = XmStringCreateLocalized(mask);
+                dir_set_flag = 1;
             }
 
             XtFree(mask);
         }
+
+        FS_DirMask(new_w) = XmStringCopy(FS_DirMask(new_w));
+    }
+
+    if (dir_set_flag == 0) { /* FS_Directory was not set above */
+        if (_XmStringIsSpecified(FS_Directory(new_w))) {
+            FS_Directory(new_w) = XmStringCopy(FS_Directory(new_w));
+        }
+        else
+            FS_Directory(new_w) = NULL;
     }
 
     _XmFileSelectionSearch(new_w);
@@ -728,29 +737,30 @@ initialize(Widget request, Widget new_w,
 static void
 destroy(Widget w)
 {
-    DEBUGOUT(XdbDebug("RWS", w,"%s:destroy(%d)\n",
+    DEBUGOUT(_LtDebugDebug("RWS", w,"%s:destroy(%d)\n",
     	__FILE__, __LINE__
     	));
+
     XmStringFree(FS_DirListLabelString(w));
     XmStringFree(FS_FileListLabelString(w));
     XmStringFree(FS_FilterLabelString(w));
     XmStringFree(FS_NoMatchString(w));
 	/* T. Straumann: don't forget to free these */
-	XmStringFree(FS_DirMask(w));
-	XmStringFree(FS_Pattern(w));
-	XmStringFree(FS_Directory(w));
+    XmStringFree(FS_DirMask(w));
+    XmStringFree(FS_Pattern(w));
+    XmStringFree(FS_Directory(w));
 }
 
 static Boolean
 set_values(Widget old, Widget request, Widget new_w,
 	   ArgList args, Cardinal *num_args)
 {
-    Arg al[10];
+    Arg al[6];
     int ac;
     Boolean refresh = False;
-	Boolean reSearch;
+    Boolean reSearch;
 
-    DEBUGOUT(XdbDebug(__FILE__, new_w,
+    DEBUGOUT(_LtDebugDebug(__FILE__, new_w,
 		      "set_values: %i args\n"
 		      "\t    old X %5i Y %5i W %5i H %5i\n"
 		      "\trequest X %5i Y %5i W %5i H %5i\n"
@@ -762,7 +772,7 @@ set_values(Widget old, Widget request, Widget new_w,
 		      XtWidth(request), XtHeight(request),
 		      XtX(new_w), XtY(new_w),
 		      XtWidth(new_w), XtHeight(new_w)));
-    DEBUGOUT(XdbPrintArgList(__FILE__, new_w, args, *num_args, False));
+    DEBUGOUT(_LtDebugPrintArgList(__FILE__, new_w, args, *num_args, False));
 
     /* this is required */
     BB_InSetValues(new_w) = True;
@@ -784,7 +794,7 @@ set_values(Widget old, Widget request, Widget new_w,
      */
     if (FS_DirSpec(old) == FS_DirSpec(new_w))
     {
-    int i;
+    Cardinal i;
 
     	for (i=0; i<*num_args; i++)
     	{
@@ -863,26 +873,6 @@ set_values(Widget old, Widget request, Widget new_w,
 
     }
 
-#ifdef PATTERN_DANNY
-{
-  int r;
-     r = XmStringCompare(FS_Pattern(new_w), FS_Pattern(old));
-     if (! r) {
-	fprintf(stderr, "FileSB: SetValues: Pattern differs (r %d), old %p '%s'"
-		r,
-		FS_Pattern(old), XdbXmString2String(FS_Pattern(old)));
-	fprintf(stderr,", new %p '%s'\n",
-		FS_Pattern(new_w), XdbXmString2String(FS_Pattern(new_w)));
-    } else {
-	fprintf(stderr, "FileSB: SetValues: Pattern equal (r %d) %p '%s',",
-		r,
-		FS_Pattern(old), XdbXmString2String(FS_Pattern(old)));
-	fprintf(stderr," %p '%s'\n",
-		FS_Pattern(new_w), XdbXmString2String(FS_Pattern(new_w)));
-    }
-}
-#endif
-
 	/* T. Straumann: if any new string is supplied, we must 
 	 *				 free the old one and make a copy of the user
 	 *				 supplied value.
@@ -937,9 +927,6 @@ set_values(Widget old, Widget request, Widget new_w,
 
 	if ( reSearch || (FS_FileTypeMask(new_w) != FS_FileTypeMask(old)))
     {
-#ifdef PATTERN_DANNY
-	fprintf(stderr, "FileSB: SetValues requires another Search()\n");
-#endif
 	_XmFileSelectionSearch(new_w);
     }
 
@@ -960,11 +947,11 @@ set_values(Widget old, Widget request, Widget new_w,
 /* Motif has one of these */
 static void get_values_hook(Widget w, ArgList args, Cardinal *num_args)
 {
-    DEBUGOUT(XdbDebug(__FILE__, w,
+    DEBUGOUT(_LtDebugDebug(__FILE__, w,
 		      "%s:get_values_hook() - %i args\n",
 		      __FILE__,
 		      *num_args));
-    DEBUGOUT(XdbPrintArgList(__FILE__, w, args, *num_args, False));
+    DEBUGOUT(_LtDebugPrintArgList(__FILE__, w, args, *num_args, False));
 
 }
 #endif
@@ -1075,8 +1062,8 @@ _XmTextFix(XmGeoMatrix geoSpec,
     default:
 	break;
     }
-    DEBUGOUT(XdbDebug2(__FILE__, XtParent(rowPtr->kid), rowPtr->kid, "_XmTextFix() - %s %dx%d%+d%+d\n",
-    		XdbGeoAction2String(action),
+    DEBUGOUT(_LtDebugDebug2(__FILE__, XtParent(rowPtr->kid), rowPtr->kid, "_XmTextFix() - %s %dx%d%+d%+d\n",
+    		_LtDebugGeoAction2String(action),
     		rowPtr->box.width,
     		rowPtr->box.height,
     		rowPtr->box.x,
@@ -1092,8 +1079,8 @@ _XmDirListFix(XmGeoMatrix geoSpec,
 	      XmGeoMajorLayout layoutPtr,
 	      XmKidGeometry rowPtr)
 {
-    DEBUGOUT(XdbDebug2(__FILE__, XtParent(rowPtr->kid), rowPtr->kid, "_XmDirListFix() - %s\n",
-    		XdbGeoAction2String(action)));
+    DEBUGOUT(_LtDebugDebug2(__FILE__, XtParent(rowPtr->kid), rowPtr->kid, "_XmDirListFix() - %s\n",
+    		_LtDebugGeoAction2String(action)));
 
     switch (action)
     {
@@ -1119,8 +1106,8 @@ _XmDirListFix(XmGeoMatrix geoSpec,
 		    rowPtr->box.width = (&rowPtr[-3])->box.width;
 		}
 		}
-    DEBUGOUT(XdbDebug2(__FILE__, XtParent(rowPtr->kid), rowPtr->kid, "_XmDirListFix() - %s %dx%d%+d%+d\n",
-    		XdbGeoAction2String(action),
+    DEBUGOUT(_LtDebugDebug2(__FILE__, XtParent(rowPtr->kid), rowPtr->kid, "_XmDirListFix() - %s %dx%d%+d%+d\n",
+    		_LtDebugGeoAction2String(action),
     		rowPtr->box.width,
     		rowPtr->box.height,
     		rowPtr->box.x,
@@ -1148,8 +1135,8 @@ _XmDirListFix(XmGeoMatrix geoSpec,
 			    GetPreferredListWidth(SW_WorkWindow(rowPtr->kid)) +
 				      rowPtr->box.width);
 		}
-    DEBUGOUT(XdbDebug2(__FILE__, XtParent(rowPtr->kid), rowPtr->kid, "_XmDirListFix() - %s %dx%d%+d%+d\n",
-    		XdbGeoAction2String(action),
+    DEBUGOUT(_LtDebugDebug2(__FILE__, XtParent(rowPtr->kid), rowPtr->kid, "_XmDirListFix() - %s %dx%d%+d%+d\n",
+    		_LtDebugGeoAction2String(action),
     		rowPtr->box.width,
     		rowPtr->box.height,
     		rowPtr->box.x,
@@ -1223,14 +1210,14 @@ _XmDirListFix(XmGeoMatrix geoSpec,
 		- geoSpec->margin_w
 		- layoutPtr->row.space_between
 		- 2 * dirList->box.border_width;
-    DEBUGOUT(XdbDebug2(__FILE__, XtParent(rowPtr->kid), rowPtr->kid, "_XmDirListFix() - %s %dx%d%+d%+d\n",
-    		XdbGeoAction2String(action),
+    DEBUGOUT(_LtDebugDebug2(__FILE__, XtParent(rowPtr->kid), rowPtr->kid, "_XmDirListFix() - %s %dx%d%+d%+d\n",
+    		_LtDebugGeoAction2String(action),
     		rowPtr->box.width,
     		rowPtr->box.height,
     		rowPtr->box.x,
     		rowPtr->box.y ));
-    DEBUGOUT(XdbDebug2(__FILE__, XtParent(rowPtr->kid), fileList->kid, "_XmDirListFix() - %s %dx%d%+d%+d\n",
-    		XdbGeoAction2String(action),
+    DEBUGOUT(_LtDebugDebug2(__FILE__, XtParent(rowPtr->kid), fileList->kid, "_XmDirListFix() - %s %dx%d%+d%+d\n",
+    		_LtDebugGeoAction2String(action),
     		fileList->box.width,
     		fileList->box.height,
     		fileList->box.x,
@@ -1263,9 +1250,9 @@ geoMatrixCreate(Widget _w, Widget _from, XtWidgetGeometry *_pref)
     XmGeoMatrix geoSpec;
     register XmGeoRowLayout layoutPtr;
     register XmKidGeometry boxPtr;
-    Cardinal numKids;
+    Cardinal numKids, i, nextras;
     Boolean newRow;
-    int nrows, i, nextras;
+    int nrows;
     Widget *extras = NULL;
 
     numKids = MGR_NumChildren(_w);
@@ -1696,22 +1683,44 @@ geoMatrixCreate(Widget _w, Widget _from, XtWidgetGeometry *_pref)
 	layoutPtr->fit_mode = XmGEO_WRAP;
 	if (LabG_RecomputeSize(SB_HelpButton(_w)))
 	{
+#if 1
+	    /* rws 8 Jul 2000
+	       filesb/test6
+	     */
 	    if (SB_List(_w) && XtIsManaged(XtParent(SB_List(_w))))
 	    {
-	    layoutPtr->even_width = 6;
-	    layoutPtr->even_height = 6;
+		layoutPtr->even_width = 6;
+		layoutPtr->even_height = 6;
 	    }
 	    else
 	    {
-	    layoutPtr->even_width = 4;
-	    layoutPtr->even_height = 4;
+		layoutPtr->even_width = 4;
+		layoutPtr->even_height = 4;
 	    }
+#endif
 	}
 	else
 	{
 	    layoutPtr->even_width = 1;
 	    layoutPtr->even_height = 1;
 	}
+#if 1
+	/* rws 8 Jul 2000
+	   filesb/test15 when pressing Filter
+	 */
+	if (SB_OkButton(_w) && !LabG_RecomputeSize(SB_OkButton(_w)))
+	{
+	    layoutPtr->even_width = 1;
+	    if (XtIsRealized(_w))
+	    {
+		layoutPtr->even_height = 1;
+	    }
+	    else
+	    {
+		layoutPtr->even_height = 4;
+	    }
+	}
+#endif
 	layoutPtr->space_above = BB_MarginHeight(_w);
 	newRow = True;
     }
@@ -1738,7 +1747,7 @@ focus_moved_proc(Widget wid, XtPointer client_data, XtPointer data)
     XmFocusMovedCallback cbs = (XmFocusMovedCallback)data;
     Widget bb = (Widget)client_data;
 
-    DEBUGOUT(XdbDebug2(__FILE__, wid, bb, "%s:focus_moved_proc() - %s -> %s\n",
+    DEBUGOUT(_LtDebugDebug2(__FILE__, wid, bb, "%s:focus_moved_proc() - %s -> %s\n",
     	__FILE__,
     	cbs->old_focus ? XtName(cbs->old_focus) : "NULL",
     	cbs->new_focus ? XtName(cbs->new_focus) : "NULL"));
@@ -1767,7 +1776,7 @@ noGeoRequest(XmGeoMatrix _geoSpec)
 	/*
 	return True;
 	*/
-    DEBUGOUT(XdbDebug("FSB", _geoSpec->composite, "%s:noGeoRequest(%d) - instigator %s\n",
+    DEBUGOUT(_LtDebugDebug("FSB", _geoSpec->composite, "%s:noGeoRequest(%d) - instigator %s\n",
     	__FILE__, __LINE__,
     	_geoSpec->instigator ? XtName(_geoSpec->instigator) : "NULL"));
 
@@ -1792,6 +1801,31 @@ noGeoRequest(XmGeoMatrix _geoSpec)
 
 /******************************* SEARCHING **********************************/
 
+
+#if XmVERSION > 1
+/* Keep it short and simple - make it a function (instead of a macro)
+   for easier debugging purposes but give it a more than reasonable chance
+   to be optimized out (or inlined in :-)  ... */
+static Boolean
+FileIsHidden(String name) {
+
+  const char *ptr;
+  
+  ptr=basename(name);
+  if (*ptr=='.') {
+     ptr++;
+     if ( (*ptr=='\0') || ((*ptr=='.')&&(*(ptr+1)=='\0')) )
+       /* keep special entries visible */
+        return False;
+     else
+        return True;
+  }
+  else
+     return False;
+}
+#endif
+
+
 static void
 defaultDirSearchProc(Widget widget,
 		     XtPointer data)
@@ -1799,7 +1833,7 @@ defaultDirSearchProc(Widget widget,
     String *entries, dir, pat;
     XmFileSelectionBoxCallbackStruct *cbs = (XmFileSelectionBoxCallbackStruct *)data;
     unsigned int numents, numalloc, argc, i, max;
-    Arg argl[5];
+    Arg argl[3];
     XmString *table;
 
     if (!XmStringGetLtoR(cbs->dir, XmFONTLIST_DEFAULT_TAG, &dir))
@@ -1807,8 +1841,9 @@ defaultDirSearchProc(Widget widget,
 	dir = XtNewString("");
     }
 
-    entries = NULL;
-    numents = numalloc = 0;
+    entries  = NULL;
+    numents  = 0;
+    numalloc = 0;
 
     /*
      * I think pattern should always be "*" for directories.
@@ -1827,6 +1862,33 @@ defaultDirSearchProc(Widget widget,
 
     max = 64;
     table = (XmString *)XtCalloc(max, sizeof(XmString));
+   
+#if XmVERSION > 1
+    /* we use two versions of this loop for better performance */
+    if (FS_FileFilterStyle(widget)==XmFILTER_HIDDEN_FILES) {
+
+        int visible=0;
+    
+        for (i = 0; i < numents; i++)
+         {
+	 if (FileIsHidden(entries[i])) {
+	    XtFree(entries[i]);
+	 }
+	 else {
+	   if (i == max)
+	    {
+	    max *= 2;
+	    table = (XmString *)XtRealloc((char *)table, max * sizeof(XmString));
+	    }
+	    table[visible] = XmStringCreateLtoR(entries[i], XmFONTLIST_DEFAULT_TAG);
+	    visible++;
+	    XtFree(entries[i]);
+         } /* not hidden */
+        } /* for */
+	numents = visible;
+    }  /* XmFILTER_HIDDEN_FILES */
+    else
+#endif
     for (i = 0; i < numents; i++)
     {
 	if (i == max)
@@ -1834,11 +1896,10 @@ defaultDirSearchProc(Widget widget,
 	    max *= 2;
 	    table = (XmString *)XtRealloc((char *)table, max * sizeof(XmString));
 	}
-
 	table[i] = XmStringCreateLtoR(entries[i], XmFONTLIST_DEFAULT_TAG);
-
 	XtFree(entries[i]);
     }
+
     if (entries)
     {
 	XtFree((char *)entries);
@@ -1865,37 +1926,38 @@ defaultFileSearchProc(Widget widget, XtPointer data)
     XmFileSelectionBoxCallbackStruct *cbs =
 	(XmFileSelectionBoxCallbackStruct *)data;
     unsigned int numents, numalloc, argc, i, max;
-    Arg argl[5];
+    Arg argl[3];
     XmString *table;
 
-    DEBUGOUT(XdbDebug(__FILE__, widget, "defaultFileSearchProc\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, widget, "defaultFileSearchProc\n"));
 
     if (!XmStringGetLtoR(cbs->dir, XmFONTLIST_DEFAULT_TAG, &dir))
     {
 	dir = XtNewString("");
-	DEBUGOUT(XdbDebug(__FILE__, widget,
+	DEBUGOUT(_LtDebugDebug(__FILE__, widget,
 			  "defaultFileSearchProc: empty directory\n"));
     }
     else
     {
-	DEBUGOUT(XdbDebug(__FILE__, widget,
+	DEBUGOUT(_LtDebugDebug(__FILE__, widget,
 			  "defaultFileSearchProc: directory '%s'\n", dir));
     }
 
     if (!XmStringGetLtoR(cbs->pattern, XmFONTLIST_DEFAULT_TAG, &pat))
     {
 	pat = XtNewString("*");
-	DEBUGOUT(XdbDebug(__FILE__, widget,
+	DEBUGOUT(_LtDebugDebug(__FILE__, widget,
 			  "defaultFileSearchProc: default pattern '*'\n"));
     }
     else
     {
-	DEBUGOUT(XdbDebug(__FILE__, widget,
+	DEBUGOUT(_LtDebugDebug(__FILE__, widget,
 			  "defaultFileSearchProc: pattern '%s'\n", pat));
     }
 
-    entries = NULL;
-    numents = numalloc = 0;
+    entries  = NULL;
+    numents  = 0;
+    numalloc = 0;
 
     _XmOSBuildFileList(dir, pat, FS_FileTypeMask(widget),  /* FIX ME: leaks */
 		       &entries, &numents, &numalloc);
@@ -1908,17 +1970,45 @@ defaultFileSearchProc(Widget widget, XtPointer data)
     max = 64;
     table = (XmString *)XtCalloc(max, sizeof(XmString));
 
-    for (i = 0; i < numents; i++)
-    {
-	if (i == max)
-	{
-	    max *= 2;
-	    table = (XmString *)XtRealloc((char *)table,
+#if XmVERSION > 1
+    /* we use two versions of this loop for better performance */
+    if (FS_FileFilterStyle(widget)==XmFILTER_HIDDEN_FILES) {
+
+        int visible=0;
+
+        for (i = 0; i < numents; i++)
+        {
+	 if (FileIsHidden(entries[i])) {
+	    XtFree(entries[i]);
+	 } else {
+   	   if (i == max)
+ 	    {
+	     max *= 2;
+	     table = (XmString *)XtRealloc((char *)table,
 					  max * sizeof(XmString));
-	}
-	table[i] = XmStringCreateLtoR(entries[i], XmFONTLIST_DEFAULT_TAG);
-	XtFree(entries[i]);
-    }
+ 	    }
+	    table[visible] = XmStringCreateLtoR(entries[i], XmFONTLIST_DEFAULT_TAG);
+	    visible++;
+	    XtFree(entries[i]);
+         } /* not hidden */
+	 } /* for */
+    numents = visible;
+    } /* XmFILTER_HIDDEN_FILES */
+    else
+#endif
+     for (i = 0; i < numents; i++)
+        {
+   	 if (i == max)
+ 	{
+	     max *= 2;
+	     table = (XmString *)XtRealloc((char *)table,
+					  max * sizeof(XmString));
+ 	  }
+	  table[i] = XmStringCreateLtoR(entries[i], XmFONTLIST_DEFAULT_TAG);
+	  XtFree(entries[i]);
+        }
+
+
     if (entries)
     {
 	XtFree((char *)entries);
@@ -1956,7 +2046,7 @@ defaultQualifySearchDataProc(Widget w,
     String mask = NULL;
     String pdir, ppat;
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "defaultQualifySearchDataProc\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "defaultQualifySearchDataProc\n"));
 
     if (in->dir)
     {
@@ -2094,11 +2184,8 @@ _XmFileSelectionSearch(Widget w)
     Arg argl[5];
     int argc;
     char *value;
-#ifdef	SAVE_PATTERN
-    XmString	x;
-#endif
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "_XmFileSelectionSearch()\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "_XmFileSelectionSearch()\n"));
 
     /*
      * setup the callback structures
@@ -2118,8 +2205,8 @@ _XmFileSelectionSearch(Widget w)
 	in.mask = FS_DirMask(w);
     }
     in.mask_length = XmStringLength(in.mask);
-    DEBUGOUT(XdbDebug(__FILE__, w, "\tdirMask %s (%i)\n", 
-    		XdbXmString2String(FS_DirMask(w)),in.mask_length));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "\tdirMask %s (%i)\n", 
+    		_LtDebugXmString2String(FS_DirMask(w)),in.mask_length));
 
     in.dir = FS_Directory(w);
     if (!_XmStringIsSpecified(in.dir))
@@ -2128,54 +2215,37 @@ _XmFileSelectionSearch(Widget w)
 	in.dir = FS_Directory(w);
     }
     in.dir_length = XmStringLength(in.dir);
-    DEBUGOUT(XdbDebug(__FILE__, w, "\tdirectory %s (%i)\n", 
-    		XdbXmString2String(FS_Directory(w)),in.dir_length));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "\tdirectory %s (%i)\n", 
+    		_LtDebugXmString2String(FS_Directory(w)),in.dir_length));
 
-#ifdef	SAVE_PATTERN
-    /*
-     * Too many strange things are happening
-     * Play it safe, only pass a copy
-     * Leak for now ...
-     */
-    x = FS_Pattern(w);
-    in.pattern = XmStringCopy(FS_Pattern(w));
-#else
     in.pattern = FS_Pattern(w);
-#endif
     if (!_XmStringIsSpecified(in.pattern))
     {
-#ifdef PATTERN_DANNY
-	fprintf(stderr, "FileSB _XmFileSelectionSearch: set pattern to ''\n");
-#endif
     	FS_Pattern(w) = XmStringCreateSimple("");
 	in.pattern = FS_Pattern(w);
     }
     in.pattern_length = XmStringLength(in.pattern);
-    DEBUGOUT(XdbDebug(__FILE__, w, "\tpattern %s (%i)\n", 
-    		XdbXmString2String(FS_Pattern(w)),in.pattern_length));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "\tpattern %s (%i)\n", 
+    		_LtDebugXmString2String(FS_Pattern(w)),in.pattern_length));
 
     /*
      * qualification
      */
-    DEBUGOUT(XdbDebug(__FILE__, w, "_XmFileSelectionSearch() - Calling Qualify\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "_XmFileSelectionSearch() - Calling Qualify\n"));
     (*FS_QualifySearchDataProc(w)) (w, (XtPointer)&in, (XtPointer)&out);
 
-#ifdef	SAVE_PATTERN
-    FS_Pattern(w) = x;
-#endif
-
 #ifdef MOZILLA
     /* rws 21 Jun 1998
        Mozilla installs its own dir search proc.  The first thing it does is
        to get the XmNdirectory and compare this to the last time it was 
        called.  If it is the same, the search is not done.
      */
-    DEBUGOUT(XdbDebug(__FILE__, w, "_XmFileSelectionSearch()\n\tdirectory %s\n",
-		      XdbXmString2String(FS_Directory(w))));
-    DEBUGOUT(XdbDebug(__FILE__, w, "_XmFileSelectionSearch()\n\tin.dir %s\n",
-		      XdbXmString2String(in.dir)));
-    DEBUGOUT(XdbDebug(__FILE__, w, "_XmFileSelectionSearch()\n\tout.dir %s\n",
-		      XdbXmString2String(out.dir)));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "_XmFileSelectionSearch()\n\tdirectory %s\n",
+		      _LtDebugXmString2String(FS_Directory(w))));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "_XmFileSelectionSearch()\n\tin.dir %s\n",
+		      _LtDebugXmString2String(in.dir)));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "_XmFileSelectionSearch()\n\tout.dir %s\n",
+		      _LtDebugXmString2String(out.dir)));
     if (FS_Directory(w))
     {
 	XmStringFree(FS_Directory(w));
@@ -2196,7 +2266,7 @@ _XmFileSelectionSearch(Widget w)
     if ((FS_StateFlags(w) & XmFS_DIR_SEARCH_PROC) == 0)
     {
 	FS_StateFlags(w) |= XmFS_DIR_SEARCH_PROC;
-	DEBUGOUT(XdbDebug(__FILE__, w, "_XmFileSelectionSearch() - Calling DirSearch\n"));
+	DEBUGOUT(_LtDebugDebug(__FILE__, w, "_XmFileSelectionSearch() - Calling DirSearch\n"));
 	(*FS_DirSearchProc(w)) (w, (XtPointer)&out);
 	if (FS_ListUpdated(w))
 	{
@@ -2215,7 +2285,7 @@ _XmFileSelectionSearch(Widget w)
 	if ((FS_StateFlags(w) & XmFS_IN_FILE_SEARCH) == 0)
 	{
 	    FS_StateFlags(w) |= XmFS_IN_FILE_SEARCH;
-	    DEBUGOUT(XdbDebug(__FILE__, w, "_XmFileSelectionSearch() - Calling FileSearch\n"));
+	    DEBUGOUT(_LtDebugDebug(__FILE__, w, "_XmFileSelectionSearch() - Calling FileSearch\n"));
 	    (*FS_FileSearchProc(w)) (w, (XtPointer)&out);
 	    if (FS_ListUpdated(w) && SB_ListItemCount(w) == 0)
 	    {
@@ -2283,16 +2353,9 @@ _XmFileSelectionSearch(Widget w)
     FS_Directory(w) = XmStringCopy(out.dir);
     if (FS_Pattern(w))
     {
-#ifdef PATTERN_DANNY
-	fprintf(stderr, "FileSB _XmFileSelectionSearch: free pattern\n");
-#endif
 	XmStringFree(FS_Pattern(w));
     }
     FS_Pattern(w) = XmStringCopy(out.pattern);
-#ifdef PATTERN_DANNY
-    fprintf(stderr, "FileSB _XmFileSelectionSearch: change pattern (%s)\n",
-	XdbXmString2String(FS_Pattern(w)));
-#endif
 
     if (out.value)
     {
@@ -2342,7 +2405,7 @@ XmCreateFileSelectionDialog(Widget parent,
     Widget d, ret;
     char *s;
     Arg *al;
-    int ac, i;
+    Cardinal ac, i;
 
     s = _XmMakeDialogName(name);
 
@@ -2366,7 +2429,7 @@ XmCreateFileSelectionDialog(Widget parent,
     XtAddCallback(ret, XmNdestroyCallback,
 		  _XmDestroyParentCallback,
 		  (XtPointer)ret);
-    XtFree((XtPointer)al);
+    XtFree((char *)al);
 
     return ret;
 }
@@ -2432,8 +2495,8 @@ XmFileSelectionDoSearch(Widget w, XmString dirmask)
        	XmFileSelectionBoxCallbackStruct in, out;
        	String mask;
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "XmFileSelectionDoSearch() - %s\n",
-    		XdbXmString2String(dirmask)));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "XmFileSelectionDoSearch() - %s\n",
+    		_LtDebugXmString2String(dirmask)));
 
 	in.reason = 0;
 	in.event = NULL;
@@ -2481,10 +2544,6 @@ XmFileSelectionDoSearch(Widget w, XmString dirmask)
 	XmStringFree(FS_Directory(w));
 	FS_DirMask(w) = out.mask;
 	FS_Pattern(w) = out.pattern;
-#ifdef PATTERN_DANNY
-	fprintf(stderr, "XmFileSelectionDoSearch: change pattern (%s)\n",
-		XdbXmString2String(FS_Pattern(w)));
-#endif
 	FS_Directory(w) = out.dir;
 	if (out.value)
 	{
@@ -2547,7 +2606,7 @@ _XmFsbButton(Widget w, XtPointer client, XtPointer call)
 	{
 	    char *s = XmTextFieldGetString(SB_Text(sb));
 
-	    DEBUGOUT(XdbDebug(__FILE__, w, "_XmFsbButton OK '%s'\n", s));
+	    DEBUGOUT(_LtDebugDebug(__FILE__, w, "_XmFsbButton OK '%s'\n", s));
 
 	    cbs.value = XmStringCreateSimple(s);
 
@@ -2565,8 +2624,8 @@ _XmFsbButton(Widget w, XtPointer client, XtPointer call)
 
 	    cbs.value = XmStringCopy(ls->item);
 
-	    DEBUGOUT(XdbDebug(__FILE__, w, "_XmFsbButton LIST '%s'\n",
-			      XdbXmString2String(cbs.value)));
+	    DEBUGOUT(_LtDebugDebug(__FILE__, w, "_XmFsbButton LIST '%s'\n",
+			      _LtDebugXmString2String(cbs.value)));
 	}
 
 	cbs.length = XmStringLength(cbs.value);
@@ -2601,7 +2660,7 @@ _XmFsbButton(Widget w, XtPointer client, XtPointer call)
     {
 	cbs.reason = XmCR_CANCEL;
 
-	DEBUGOUT(XdbDebug(__FILE__, w, "_XmFsbButton CANCEL\n"));
+	DEBUGOUT(_LtDebugDebug(__FILE__, w, "_XmFsbButton CANCEL\n"));
 
 	XtCallCallbackList((Widget)sb, SB_CancelCallback(sb), &cbs);
     }
@@ -2610,7 +2669,7 @@ _XmFsbButton(Widget w, XtPointer client, XtPointer call)
        	XmFileSelectionBoxCallbackStruct in, out;
        	String mask;
 
-	DEBUGOUT(XdbDebug(__FILE__, w, "_XmFsbButton FILTER\n"));
+	DEBUGOUT(_LtDebugDebug(__FILE__, w, "_XmFsbButton FILTER\n"));
 
 	in.reason = a->reason;
 	in.event = a->event;
@@ -2630,10 +2689,6 @@ _XmFsbButton(Widget w, XtPointer client, XtPointer call)
 	XmStringFree(FS_Directory(sb));
 	FS_DirMask(sb) = out.mask;
 	FS_Pattern(sb) = out.pattern;
-#ifdef PATTERN_DANNY
-	fprintf(stderr, "_XmFsbButton: change pattern (%s)\n",
-		XdbXmString2String(FS_Pattern(w)));
-#endif
 	FS_Directory(sb) = out.dir;
 
 	if (XmStringGetLtoR(out.mask, XmFONTLIST_DEFAULT_TAG, &mask))
@@ -2648,7 +2703,7 @@ _XmFsbButton(Widget w, XtPointer client, XtPointer call)
     else if (w == SB_HelpButton(sb))
     {
 	cbs.reason = XmCR_HELP;
-	DEBUGOUT(XdbDebug(__FILE__, w, "_XmFsbButton HELP\n"));
+	DEBUGOUT(_LtDebugDebug(__FILE__, w, "_XmFsbButton HELP\n"));
 	XtCallCallbackList((Widget)sb, sb->manager.help_callback, &cbs);
     }
     else if (w == FS_DirList(sb))
@@ -2658,7 +2713,7 @@ _XmFsbButton(Widget w, XtPointer client, XtPointer call)
        	XmFileSelectionBoxCallbackStruct in, out;
        	String mask;
 
-	DEBUGOUT(XdbDebug(__FILE__, w, "_XmFsbButton DirList\n"));
+	DEBUGOUT(_LtDebugDebug(__FILE__, w, "_XmFsbButton DirList\n"));
 
 	in.reason = ls->reason;
 	in.event = ls->event;
@@ -2679,10 +2734,6 @@ _XmFsbButton(Widget w, XtPointer client, XtPointer call)
 	XmStringFree(FS_Directory(sb));
 	FS_DirMask(sb) = out.mask;
 	FS_Pattern(sb) = out.pattern;
-#ifdef PATTERN_DANNY
-	fprintf(stderr, "_XmFsbButton: change pattern (%s)\n",
-		XdbXmString2String(FS_Pattern(w)));
-#endif
 	FS_Directory(sb) = out.dir;
 #endif
 
@@ -2706,10 +2757,6 @@ _XmFsbButton(Widget w, XtPointer client, XtPointer call)
 	    XmStringFree(FS_Directory(sb));
 	    FS_DirMask(sb) = out.mask;
 	    FS_Pattern(sb) = out.pattern;
-#ifdef PATTERN_DANNY
-	fprintf(stderr, "_XmFsbButton: change pattern (%s)\n",
-		XdbXmString2String(FS_Pattern(w)));
-#endif
 	    FS_Directory(sb) = out.dir;
 #endif
 	    _XmFileSelectionSearch((Widget)sb);
@@ -2719,7 +2766,7 @@ _XmFsbButton(Widget w, XtPointer client, XtPointer call)
     }
     else
     {
-	DEBUGOUT(XdbDebug(__FILE__, w, "_XmFsbButton : untreated case !!\n"));
+	DEBUGOUT(_LtDebugDebug(__FILE__, w, "_XmFsbButton : untreated case !!\n"));
     }
 
     /* AutoUnmanage */
@@ -2735,7 +2782,7 @@ _XmFsbButton(Widget w, XtPointer client, XtPointer call)
 		!CoreBeingDestroyed(sb))
 	    {
 		XtUnmanageChild((Widget)sb);
-		DEBUGOUT(XdbDebug(__FILE__, (Widget)sb, "FSB AutoUnmanage\n"));
+		DEBUGOUT(_LtDebugDebug(__FILE__, (Widget)sb, "FSB AutoUnmanage\n"));
 
 		XtCallCallbackList(shell,
 				   Shell_PopdownCallback(shell),
@@ -2780,7 +2827,7 @@ _XmFsbFileSelect(Widget w, XtPointer client, XtPointer call)
      */
     if (FS_StateFlags(fsb) & XmFS_NO_MATCH)
     {
-	DEBUGOUT(XdbDebug(__FILE__, w,
+	DEBUGOUT(_LtDebugDebug(__FILE__, w,
 			  "_XmFsbFileSelect (Selection with XmFS_NO_MATCH)\n"));
 
         /*
@@ -2793,31 +2840,41 @@ _XmFsbFileSelect(Widget w, XtPointer client, XtPointer call)
 
     if (SB_Text(fsb) == NULL)
     {
-	DEBUGOUT(XdbDebug(__FILE__, w,
+	DEBUGOUT(_LtDebugDebug(__FILE__, w,
 			  "_XmFsbFileSelect (No TextField)\n"));
 	return;
     }
 
     if (lp == NULL || lp->item == NULL)
     {
-	DEBUGOUT(XdbDebug(__FILE__, w,
+	DEBUGOUT(_LtDebugDebug(__FILE__, w,
 			  "_XmFsbFileSelect (NULL)\n"));
 	return;
     }
 
     if (!XmStringGetLtoR(lp->item, XmFONTLIST_DEFAULT_TAG, &s))
     {
-	DEBUGOUT(XdbDebug(__FILE__, w,
+	DEBUGOUT(_LtDebugDebug(__FILE__, w,
 			  "_XmFsbFileSelect (Couldn't convert to string)\n"));
 	return;
     }
 
-    DEBUGOUT(XdbDebug(__FILE__, w,
+    DEBUGOUT(_LtDebugDebug(__FILE__, w,
 		      "_XmFsbFileSelect '%s'\n", s));
     XmTextFieldSetString(SB_Text(fsb), s);
     XmTextFieldSetInsertionPosition(SB_Text(fsb),
 				    XmTextFieldGetLastPosition(SB_Text(fsb)));
     XtFree(s);
+    if (SB_TextString(fsb))
+    {
+    	XmStringFree(SB_TextString(fsb));
+    }
+    SB_TextString(fsb) = XmStringCopy(lp->item);
+    if (FS_DirMask(fsb))
+    {
+    	XmStringFree(FS_DirMask(fsb));
+    }
+    FS_DirMask(fsb) = XmStringCopy(lp->item);
 }
 
 /*
@@ -2879,7 +2936,7 @@ void
 FSBGetDirectory(Widget wid, int resource_offset, XtArgVal *value)
 {
     /* According to Ken this is always a copy */
-    DEBUGOUT(XdbDebug(__FILE__, wid, "FSBGetDirectory()\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, wid, "FSBGetDirectory()\n"));
     *value = (XtArgVal)XmStringCopy(FS_Directory(wid));
 }
 
@@ -2909,14 +2966,16 @@ FSBGetListItems(Widget wid, int resource_offset, XtArgVal *value)
 void 
 FSBGetNoMatchString(Widget wid, int resource_offset, XtArgVal *value)
 {
-    /* FIX ME : a copy or the real thing ? */
-    *value = (XtArgVal)FS_NoMatchString(wid);
+    /* Return a copy of the internal string because the */
+    /* application expects to be able to free the returned string. */
+    *value = (XtArgVal)XmStringCopy(FS_NoMatchString(wid));
 }
 
 void 
 FSBGetPattern(Widget wid, int resource_offset, XtArgVal *value)
 {
-    /* FIX ME : a copy or the real thing ? */
+    /* Return a copy of the internal string because the */
+    /* application expects to be able to free the returned string. */
     *value = (XtArgVal)XmStringCopy(FS_Pattern(wid));
 }
 
@@ -2930,7 +2989,7 @@ FileSelectionBoxUpOrDown(Widget w, XEvent *event, String *params, Cardinal *num_
     int *position_list, position_count, new_position, old_position;
     /* char *selected_text; */
     /* XmFileSelectionBoxCallbackStruct in, out; */
-    DEBUGOUT(XdbDebug(__FILE__, w, "FileSelectionBoxUpOrDown %s\n",
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "FileSelectionBoxUpOrDown %s\n",
 		      params ? *params: "NULL"));
 
     if (*num_params != 1)
@@ -3036,14 +3095,13 @@ FileSelectionBoxUpOrDown(Widget w, XEvent *event, String *params, Cardinal *num_
 static void
 FileSelectionBoxRestore(Widget w, XEvent *event, String *params, Cardinal *num_params)
 {
-    /* Arg args[2]; */
     Widget text, list, focus;
     /* XmStringTable items; */
     /* int item_count, *position_list, position_count; */
     char *selected_text;
     /* XmAnyCallbackStruct in; */
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "FileSelectionBoxRestore\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "FileSelectionBoxRestore\n"));
 
     focus = XmGetFocusWidget(w);
     if (focus == FS_FilterText(w))
