@@ -1,8 +1,9 @@
 /**
  *
- * $Id: Form.c,v 1.164 2000/01/04 23:44:13 rwscott Exp $
+ * $Id: Form.c,v 1.172 2001/05/14 19:46:08 amai Exp $
  *
  * Copyright (C) 1996 Free Software Foundation, Inc.
+ * Copyright (C) 1996-2000 LessTif Development Team
  *
  * This file is part of the GNU LessTif Library.
  *
@@ -22,7 +23,7 @@
  *
  **/
 
-static const char rcsid[] = "$Id: Form.c,v 1.164 2000/01/04 23:44:13 rwscott Exp $";
+static const char rcsid[] = "$Id: Form.c,v 1.172 2001/05/14 19:46:08 amai Exp $";
 
 #include <LTconfig.h>
 #include <XmI/XmI.h>
@@ -375,22 +376,23 @@ XmFormClassRec xmFormClassRec =
 	/* extension                    */ NULL
     }
 };
-/* *INDENT-ON* */
 
 
 WidgetClass xmFormWidgetClass = (WidgetClass)&xmFormClassRec;
 
+
 static void
 class_part_initialize(WidgetClass widget_class)
 {
     _XmFastSubclassInit(widget_class, XmFORM_BIT);
 }
 
+
 static void
 initialize(Widget request, Widget new_w,
 	   ArgList args, Cardinal *num_args)
 {
-    DEBUGOUT(XdbDebug(__FILE__, new_w,
+    DEBUGOUT(_LtDebugDebug(__FILE__, new_w,
 		      "%s:initialize(%d) - %i args\n"
 		      "\trequest X %5i Y %5i W %5i H %5i\n"
 		      "\t  new_w X %5i Y %5i W %5i H %5i\n",
@@ -400,7 +402,7 @@ initialize(Widget request, Widget new_w,
 		      XtWidth(request), XtHeight(request),
 		      XtX(new_w), XtY(new_w),
 		      XtWidth(new_w), XtHeight(new_w)));
-    DEBUGOUT(XdbPrintArgList(__FILE__, new_w, args, *num_args, False));
+    DEBUGOUT(_LtDebugPrintArgList(__FILE__, new_w, args, *num_args, False));
 
     if (BB_MarginWidth(new_w) == XmINVALID_DIMENSION)
     {
@@ -412,6 +414,7 @@ initialize(Widget request, Widget new_w,
     }
 }
 
+
 #if 0
 static void
 realize(Widget w,
@@ -430,7 +433,7 @@ realize(Widget w,
    of -2, because of the topOffset, and BulletinBoard was then enforcing
    a margin of 0.
  */
-    DEBUGOUT(XdbDebug(__FILE__, w, "%s:realize(%d) - %dx%d\n",
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "%s:realize(%d) - %dx%d\n",
 		      __FILE__, __LINE__,
 		      XtWidth(w), XtHeight(w)));
 
@@ -446,7 +449,7 @@ set_values(Widget old, Widget request, Widget new_w,
 {
     Boolean relayout = False, refresh = False;
 
-    DEBUGOUT(XdbDebug(__FILE__, new_w,
+    DEBUGOUT(_LtDebugDebug(__FILE__, new_w,
 		      "%s:set_values(%d) - %i args\n"
 		      "\t    old X %5i Y %5i W %5i H %5i\n"
 		      "\trequest X %5i Y %5i W %5i H %5i\n"
@@ -459,7 +462,7 @@ set_values(Widget old, Widget request, Widget new_w,
 		      XtWidth(request), XtHeight(request),
 		      XtX(new_w), XtY(new_w),
 		      XtWidth(new_w), XtHeight(new_w)));
-    DEBUGOUT(XdbPrintArgList(__FILE__, new_w, args, *num_args, False));
+    DEBUGOUT(_LtDebugPrintArgList(__FILE__, new_w, args, *num_args, False));
 
     BB_InSetValues(new_w) = True;
 
@@ -509,12 +512,13 @@ set_values(Widget old, Widget request, Widget new_w,
     return refresh;
 }
 
+
 static void
 resize(Widget w)
 {
     Dimension wd, ht;
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "%s:resize(%d) - %dx%d\n",
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "%s:resize(%d) - %dx%d\n",
 		      __FILE__, __LINE__,
 		      XtWidth(w), XtHeight(w)));
 
@@ -526,6 +530,7 @@ resize(Widget w)
     _XmFormConfigureChildren(w, NULL, NULL);
 }
 
+
 /*
  * query_geometry
  *
@@ -546,9 +551,9 @@ query_geometry(Widget w, XtWidgetGeometry *proposed, XtWidgetGeometry *desired)
     Dimension wd, ht;
     XtWidgetGeometry pp;
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "%s:query_geometry(%d) - proposed %s\n",
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "%s:query_geometry(%d) - proposed %s\n",
 		      __FILE__, __LINE__,
-		      XdbWidgetGeometry2String(proposed)));
+		      _LtDebugWidgetGeometry2String(proposed)));
 
     pp = *proposed;
 
@@ -583,12 +588,13 @@ query_geometry(Widget w, XtWidgetGeometry *proposed, XtWidgetGeometry *desired)
     desired->width = wd == 0 ? 1 : wd;
     desired->height = ht == 0 ? 1 : ht;
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "%s:query_geometry(%d) - desired %s\n",
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "%s:query_geometry(%d) - desired %s\n",
 		      __FILE__, __LINE__,
-		      XdbWidgetGeometry2String(desired)));
+		      _LtDebugWidgetGeometry2String(desired)));
     return _XmGMReplyToQueryGeometry(w, &pp, desired);
 }
 
+
 /*
  * GeometryManager Method
  *
@@ -619,18 +625,21 @@ geometry_manager(Widget w, XtWidgetGeometry *request, XtWidgetGeometry *reply)
 
 #define       Wants(x)        (request->request_mode & x)
 
+    DEBUGOUT(_LtDebugDebug2("GM", f, w, "%s:geometry_manager(%d) - request %s\n",
+		       __FILE__, __LINE__,
+		       _LtDebugWidgetGeometry2String(request)));
     if ((request->request_mode & (CWX | CWY | CWWidth | CWHeight)) == 0)
     {
 	Form_ProcessingConstraints(f) = False;
 	*reply = *request;
 	return XtGeometryYes;
     }
-    DEBUGOUT(XdbDebug2(__FILE__, f, w, "%s:geometry_manager(%d) - request %s\n",
+    DEBUGOUT(_LtDebugDebug2(__FILE__, f, w, "%s:geometry_manager(%d) - request %s\n",
 		       __FILE__, __LINE__,
-		       XdbWidgetGeometry2String(request)));
-    DEBUGOUT(XdbDebug2("GM", f, w, "%s:geometry_manager(%d) - request %s\n",
+		       _LtDebugWidgetGeometry2String(request)));
+    DEBUGOUT(_LtDebugDebug2("GM", f, w, "%s:geometry_manager(%d) - request %s\n",
 		       __FILE__, __LINE__,
-		       XdbWidgetGeometry2String(request)));
+		       _LtDebugWidgetGeometry2String(request)));
 
     mr = *request;
     /*
@@ -667,11 +676,11 @@ geometry_manager(Widget w, XtWidgetGeometry *request, XtWidgetGeometry *reply)
     /* Ask form's layout algorithm what it thinks about this */
     _XmFormLayout(f, w, &mr, &wd, &ht);
 
-    DEBUGOUT(XdbDebug2(__FILE__, f, w,
+    DEBUGOUT(_LtDebugDebug2(__FILE__, f, w,
 		       "GeometryManager: Form Layout makes child geo"
 		       " %dx%d form %dx%d\n",
 		       mr.width, mr.height, wd, ht));
-    DEBUGOUT(XdbDebug2("GM", f, w,
+    DEBUGOUT(_LtDebugDebug2("GM", f, w,
 		       "GeometryManager: Form Layout makes child geo"
 		       " %dx%d%+i%+i form %dx%d %dx%d\n",
 		       mr.width, mr.height, mr.x, mr.y, wd, ht,
@@ -685,7 +694,7 @@ geometry_manager(Widget w, XtWidgetGeometry *request, XtWidgetGeometry *reply)
 	 */
 	_XmFormLayout(f, w, &mr, &wd, &ht);
 
-	DEBUGOUT(XdbDebug2(__FILE__, (Widget)f, w,
+	DEBUGOUT(_LtDebugDebug2(__FILE__, (Widget)f, w,
 			   "GeometryManager: Form Second Layout makes child geo"
 			   " %dx%d form %dx%d\n",
 			   mr.width, mr.height, wd, ht));
@@ -696,7 +705,7 @@ geometry_manager(Widget w, XtWidgetGeometry *request, XtWidgetGeometry *reply)
 	wd = XtWidth(f);
 	ht = XtHeight(f);
 	_XmFormLayout(f, w, &mr, &wd, &ht);
-	DEBUGOUT(XdbDebug2("GM", f, w,
+	DEBUGOUT(_LtDebugDebug2("GM", f, w,
 			   "GeometryManager: Form Layout 2 makes child geo"
 			   " %dx%d form %dx%d %dx%d\n",
 			   mr.width, mr.height, wd, ht,
@@ -707,12 +716,14 @@ geometry_manager(Widget w, XtWidgetGeometry *request, XtWidgetGeometry *reply)
      * Now that form has looked at the geometry, tell our caller what
      * happened
      */
+    *reply = *request;
     if (Wants(CWX))
     {
 	ask++;
 	if ((mr.request_mode & CWX) && mr.x == request->x)
 	{
 	    good++;
+	    reply->request_mode &= ~CWX;
 	}
     }
     if (Wants(CWY))
@@ -721,6 +732,7 @@ geometry_manager(Widget w, XtWidgetGeometry *request, XtWidgetGeometry *reply)
 	if ((mr.request_mode & CWY) && mr.y == request->y)
 	{
 	    good++;
+	    reply->request_mode &= ~CWY;
 	}
     }
     if (Wants(CWWidth))
@@ -729,6 +741,7 @@ geometry_manager(Widget w, XtWidgetGeometry *request, XtWidgetGeometry *reply)
 	if ((mr.request_mode & CWWidth) && mr.width == request->width)
 	{
 	    good++;
+	    reply->request_mode &= ~CWWidth;
 	}
     }
     if (Wants(CWHeight))
@@ -737,9 +750,11 @@ geometry_manager(Widget w, XtWidgetGeometry *request, XtWidgetGeometry *reply)
 	if ((mr.request_mode & CWHeight) && mr.height == request->height)
 	{
 	    good++;
+	    reply->request_mode &= ~CWHeight;
 	}
     }
 
+    /*
     if (reply != NULL)
     {
 	int mode;
@@ -748,6 +763,7 @@ geometry_manager(Widget w, XtWidgetGeometry *request, XtWidgetGeometry *reply)
 	reply->request_mode = CWHeight;
 	reply->height = 1;
     }
+    */
 
     if (good == ask)
     {
@@ -761,7 +777,7 @@ geometry_manager(Widget w, XtWidgetGeometry *request, XtWidgetGeometry *reply)
     }
     else
     {
-	DEBUGOUT(XdbDebug2("GM", f, w,
+	DEBUGOUT(_LtDebugDebug2("GM", f, w,
 			   "GeometryManager: "
 			   " %dx%d %dx%d form %dx%d\n",
 			   wd, ht,
@@ -782,6 +798,7 @@ geometry_manager(Widget w, XtWidgetGeometry *request, XtWidgetGeometry *reply)
 #undef        Wants
 }
 
+
 /*
  * change_managed method
  *
@@ -797,16 +814,21 @@ change_managed(Widget w)
 {
     Dimension wd, ht;
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "change_managed %s %s\n",
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "change_managed %s %s\n",
 		      Form_InitialWidth(w) ? "InitialWidth" : " ",
 		      Form_InitialHeight(w) ? "InitialHeight" : ""));
-    DEBUGOUT(XdbDebug("DX", w, "change_managed %s %s %dx%d\n",
+    DEBUGOUT(_LtDebugDebug("GM", w, "change_managed %s %s %dx%d %s\n",
 		      Form_InitialWidth(w) ? "InitialWidth" : " ",
 		      Form_InitialHeight(w) ? "InitialHeight" : "",
-		      XtWidth(w), XtHeight(w)));
+		      XtWidth(w), XtHeight(w),
+		      XtIsManaged(w) ? "Managed" : "Not Managed"));
 
+    if (XtIsRealized(w) && !XtIsManaged(w))
     {
-    int i;
+	return;
+    }
+    {
+    Cardinal i;
 
     	for (i = 0; i < MGR_NumChildren(w); i++)
     	{
@@ -832,6 +854,8 @@ change_managed(Widget w)
 
     if (_XmFormGeomRequest(w, &wd, &ht) == XtGeometryYes)
     {
+	DEBUGOUT(_LtDebugDebug("DX", w, "change_managed Got yes %dx%d\n",
+		      XtWidth(w), XtHeight(w)));
 	/* Why layout again?  Cause we might have gotten a compromise */
 	_XmFormLayout(w, NULL, NULL, &wd, &ht);
 
@@ -839,6 +863,8 @@ change_managed(Widget w)
     }
     else
     {
+	DEBUGOUT(_LtDebugDebug("DX", w, "change_managed Got no  %dx%d\n",
+		      XtWidth(w), XtHeight(w)));
 	/*
 	 * MLM: I kept adding this, and then taking it out.
 	 * Once and for all: this *has* to be here.  Above, we
@@ -858,12 +884,13 @@ change_managed(Widget w)
     }
 
     _XmNavigChangeManaged(w);
-    DEBUGOUT(XdbDebug("DX", w, "change_managed %dx%d\n",
+    DEBUGOUT(_LtDebugDebug("DX", w, "change_managed %dx%d\n",
 		      XtWidth(w), XtHeight(w)));
 
     return;
 }
 
+
 static int
 _XmGetOffset(Widget w, int i)
 {
@@ -895,6 +922,7 @@ int offset;
     return(offset);
 }
 
+
 /*
  * Called in the process of adding a child
  */
@@ -906,7 +934,7 @@ constraint_initialize(Widget request, Widget new_w,
     Widget f = XtParent(new_w);
     int i;
 
-    DEBUGOUT(XdbDebug2(__FILE__, f, new_w,
+    DEBUGOUT(_LtDebugDebug2(__FILE__, f, new_w,
 		       "%s:constraint_initialize(%d) - %i args\n"
 		       "\trequest X %5i Y %5i W %5i H %5i\n"
 		       "\t  new_w X %5i Y %5i W %5i H %5i\n"
@@ -922,7 +950,7 @@ constraint_initialize(Widget request, Widget new_w,
 		       XtIsManaged(new_w) ? "Managed" : "Not Managed",
 		       XtIsRealized(new_w) ? "Realized" : "Not Realized"
 		       ));
-    DEBUGOUT(XdbPrintArgList(__FILE__, new_w, args, *num_args, False));
+    DEBUGOUT(_LtDebugPrintArgList(__FILE__, new_w, args, *num_args, False));
 
     /* do this for all four contraints (TOP, BOTTOM, LEFT, RIGHT) */
     for (i = 0; i < 4; i++)
@@ -958,6 +986,7 @@ constraint_initialize(Widget request, Widget new_w,
     FCP_PrefH(con) = XmINVALID_DIMENSION;
 }
 
+
 static Boolean
 constraint_set_values(Widget current, Widget request, Widget new_w,
 		      ArgList args, Cardinal *num_args)
@@ -968,7 +997,7 @@ constraint_set_values(Widget current, Widget request, Widget new_w,
     Boolean changed = False;
     int i;
 
-    DEBUGOUT(XdbDebug2(__FILE__, (Widget)f, new_w,
+    DEBUGOUT(_LtDebugDebug2(__FILE__, (Widget)f, new_w,
 		       "%s:constraint_set_values(%d) - %i args\n"
 		       "\tcurrent X %5i Y %5i W %5i H %5i\n"
 		       "\trequest X %5i Y %5i W %5i H %5i\n"
@@ -983,8 +1012,8 @@ constraint_set_values(Widget current, Widget request, Widget new_w,
 		       XtX(new_w), XtY(new_w),
 		       XtWidth(new_w), XtHeight(new_w),
 		       XtWidth(XtParent(new_w)), XtHeight(XtParent(new_w))));
-    DEBUGOUT(XdbPrintArgList(__FILE__, new_w, args, *num_args, False));
-    DEBUGOUT(XdbDebug2("DX", (Widget)f, new_w,
+    DEBUGOUT(_LtDebugPrintArgList(__FILE__, new_w, args, *num_args, False));
+    DEBUGOUT(_LtDebugDebug2("DX", (Widget)f, new_w,
 		       "%s:constraint_set_values(%d) - %i args\n"
 		       "\tcurrent X %5i Y %5i W %5i H %5i\n"
 		       "\trequest X %5i Y %5i W %5i H %5i\n"
@@ -999,7 +1028,7 @@ constraint_set_values(Widget current, Widget request, Widget new_w,
 		       XtX(new_w), XtY(new_w),
 		       XtWidth(new_w), XtHeight(new_w),
 		       XtWidth(XtParent(new_w)), XtHeight(XtParent(new_w))));
-    DEBUGOUT(XdbPrintArgList("DX", new_w, args, *num_args, False));
+    DEBUGOUT(_LtDebugPrintArgList("DX", new_w, args, *num_args, False));
 
     Form_ProcessingConstraints(f) = True;
     for (i = 0; i < 4; i++)
@@ -1067,7 +1096,7 @@ constraint_set_values(Widget current, Widget request, Widget new_w,
 
 	changed = False;
 
-	DEBUGOUT(XdbDebug2(__FILE__, (Widget)f, new_w,
+	DEBUGOUT(_LtDebugDebug2(__FILE__, (Widget)f, new_w,
 			   "constraint_set_values - refused resize\n"));
     }
 
@@ -1075,7 +1104,7 @@ constraint_set_values(Widget current, Widget request, Widget new_w,
     /* scrolledwindow/test13 */
     /* form/test15 */
     /* Xinvest when centering in the frame */
-    DEBUGOUT(XdbDebug2("DX", (Widget)f, new_w,"%s %s %s %s\n",
+    DEBUGOUT(_LtDebugDebug2("DX", (Widget)f, new_w,"%s %s %s %s\n",
     	changed ? "True" : "False",
     	XtIsRealized(f) ? "True" : "False",
     	XtIsManaged(f) ? "True" : "False",
@@ -1094,7 +1123,7 @@ constraint_set_values(Widget current, Widget request, Widget new_w,
 	Dimension wd, ht;
 	XtWidgetGeometry cg;
 
-	DEBUGOUT(XdbDebug(__FILE__, f,
+	DEBUGOUT(_LtDebugDebug(__FILE__, f,
 			  "GeomRequest in constraint_set_values\n"));
 
 	if (XtWidth(f) == 0 && XtHeight(f) == 0)
@@ -1138,7 +1167,7 @@ constraint_set_values(Widget current, Widget request, Widget new_w,
     }
 
     return False;
-    return changed;
+    /* return changed; */
 }
 
 /************************* LAYOUT METHODS *****************************/
@@ -1199,6 +1228,7 @@ constraint_set_values(Widget current, Widget request, Widget new_w,
 #define	AttachedWd	FormC_W
 #define	AttachedHt	FormC_H
 
+
 /*
 
  */
@@ -1210,10 +1240,10 @@ SetW(Widget f, Widget child, int p, int line)
 
 #ifdef	PRINT_ASSIGNMENTS
 #ifdef	PRINT_LINENO
-    DEBUGOUT(XdbDebug2(__FILE__, (Widget)f, child,
+    DEBUGOUT(_LtDebugDebug2(__FILE__, (Widget)f, child,
 		       "set width to %d (line %d)\n", p, line));
 #else
-    DEBUGOUT(XdbDebug2(__FILE__, (Widget)f, child, "set width to %d\n", p));
+    DEBUGOUT(_LtDebugDebug2(__FILE__, (Widget)f, child, "set width to %d\n", p));
 #endif
 #endif
     FCP_W(con) = p;
@@ -1226,15 +1256,16 @@ SetH(Widget f, Widget child, int p, int line)
 
 #ifdef	PRINT_ASSIGNMENTS
 #ifdef	PRINT_LINENO
-    DEBUGOUT(XdbDebug2(__FILE__, (Widget)f, child,
+    DEBUGOUT(_LtDebugDebug2(__FILE__, (Widget)f, child,
 		       "set height to %d (line %d)\n", p, line));
 #else
-    DEBUGOUT(XdbDebug2(__FILE__, (Widget)f, child, "set height to %d\n", p));
+    DEBUGOUT(_LtDebugDebug2(__FILE__, (Widget)f, child, "set height to %d\n", p));
 #endif
 #endif
     FCP_H(con) = p;
 }
 
+
 static void
 SetX(Widget f, Widget child, int p, int line)
 {
@@ -1242,15 +1273,16 @@ SetX(Widget f, Widget child, int p, int line)
 
 #ifdef	PRINT_ASSIGNMENTS
 #ifdef	PRINT_LINENO
-    DEBUGOUT(XdbDebug2(__FILE__, (Widget)f, child,
+    DEBUGOUT(_LtDebugDebug2(__FILE__, (Widget)f, child,
 		       "set x to %d (line %d)\n", p, line));
 #else
-    DEBUGOUT(XdbDebug2(__FILE__, (Widget)f, child, "set x to %d\n", p));
+    DEBUGOUT(_LtDebugDebug2(__FILE__, (Widget)f, child, "set x to %d\n", p));
 #endif
 #endif
     FCP_X(con) = p;
 }
 
+
 static void
 SetY(Widget f, Widget child, int p, int line)
 {
@@ -1258,38 +1290,40 @@ SetY(Widget f, Widget child, int p, int line)
 
 #ifdef	PRINT_ASSIGNMENTS
 #ifdef	PRINT_LINENO
-    DEBUGOUT(XdbDebug2(__FILE__, (Widget)f, child,
+    DEBUGOUT(_LtDebugDebug2(__FILE__, (Widget)f, child,
 		       "set y to %d (line %d)\n", p, line));
 #else
-    DEBUGOUT(XdbDebug2(__FILE__, (Widget)f, child, "set y to %d\n", p));
+    DEBUGOUT(_LtDebugDebug2(__FILE__, (Widget)f, child, "set y to %d\n", p));
 #endif
 #endif
     FCP_Y(con) = p;
 }
 
+
 static void
 SetFW(Widget f, Widget child, int p, int line)
 {
 #ifdef	PRINT_ASSIGNMENTS
 #ifdef	PRINT_LINENO
-    DEBUGOUT(XdbDebug2(__FILE__, (Widget)f, child,
+    DEBUGOUT(_LtDebugDebug2(__FILE__, (Widget)f, child,
 		       "set form width to %d (line %d)\n", p, line));
 #else
-    DEBUGOUT(XdbDebug2(__FILE__, (Widget)f, child,
+    DEBUGOUT(_LtDebugDebug2(__FILE__, (Widget)f, child,
 		       "set form width to %d\n", p));
 #endif
 #endif
 }
 
+
 static void
 SetFH(Widget f, Widget child, int p, int line)
 {
 #ifdef	PRINT_ASSIGNMENTS
 #ifdef	PRINT_LINENO
-    XdbDebug2(__FILE__, (Widget)f, child,
+    _LtDebugDebug2(__FILE__, (Widget)f, child,
 	      "set form height to %d (line %d)\n", p, line);
 #else
-    DEBUGOUT(XdbDebug2(__FILE__, (Widget)f, child,
+    DEBUGOUT(_LtDebugDebug2(__FILE__, (Widget)f, child,
 		       "set form height to %d\n", p));
 #endif
 #endif
@@ -1297,6 +1331,7 @@ SetFH(Widget f, Widget child, int p, int line)
 
 #endif
 
+
 static void
 _XmCheckAttachedWidget(Widget f, int i, XmFormConstraints con)
 {
@@ -1315,7 +1350,7 @@ _XmCheckAttachedWidget(Widget f, int i, XmFormConstraints con)
 	{
 	    if (XtParent(w) == (Widget)f)
 	    {
-		DEBUGOUT(XdbDebug2(__FILE__, (Widget)f,
+		DEBUGOUT(_LtDebugDebug2(__FILE__, (Widget)f,
 				   FCP_Atta(con, i).w,
 				   "Replace child by %s\n", XtName(w)));
 #ifdef	LESSTIF_VERBOSE
@@ -1405,12 +1440,12 @@ _XmCheckAttachedWidget(Widget f, int i, XmFormConstraints con)
 static void
 XmFormFindPreferred(Widget f, Widget cw, XtWidgetGeometry *cg)
 {
-    int i;
+    Cardinal i;
     Boolean changed;		/* is needed because of SETX macros */
 
-    DEBUGOUT(XdbDebug(__FILE__, (Widget)f, "XmFormFindPreferred() - start\n"));
-    DEBUGOUT(XdbDebug2(__FILE__, (Widget)f, cw, "XmFormFindPreferred() - request %s\n",
-		       XdbWidgetGeometry2String(cg)));
+    DEBUGOUT(_LtDebugDebug(__FILE__, (Widget)f, "XmFormFindPreferred() - start\n"));
+    DEBUGOUT(_LtDebugDebug2(__FILE__, (Widget)f, cw, "XmFormFindPreferred() - request %s\n",
+		       _LtDebugWidgetGeometry2String(cg)));
     for (i = 0; i < MGR_NumChildren(f); i++)
     {
 	Widget child = MGR_Children(f)[i];
@@ -1501,7 +1536,7 @@ XmFormFindPreferred(Widget f, Widget cw, XtWidgetGeometry *cg)
 	}
     }
 
-    DEBUGOUT(XdbDebug(__FILE__, (Widget)f, "XmFormFindPreferred() - end\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, (Widget)f, "XmFormFindPreferred() - end\n"));
 }
 
 /*
@@ -1513,38 +1548,39 @@ XmFormFindPreferred(Widget f, Widget cw, XtWidgetGeometry *cg)
     {									\
 	if (FCP_Atta(con, x).w == NULL)					\
 	{								\
-	    DEBUGOUT(XdbDebug0(__FILE__, (Widget)f,  "%s(%s)\t",	\
-			       XdbAttachment2String(FCP_Atta(con, x).type),	\
+	    DEBUGOUT(_LtDebugDebug0(__FILE__, (Widget)f,  "%s(%s)\t",	\
+			       _LtDebugAttachment2String(FCP_Atta(con, x).type),	\
 			       "(null)"));				\
 	}								\
 	else								\
 	{								\
-	    DEBUGOUT(XdbDebug0(__FILE__, (Widget)f,  "%s(%s,%d)\t",	\
-			       XdbAttachment2String(FCP_Atta(con, x).type),	\
+	    DEBUGOUT(_LtDebugDebug0(__FILE__, (Widget)f,  "%s(%s,%d)\t",	\
+			       _LtDebugAttachment2String(FCP_Atta(con, x).type),	\
 			       XtName(FCP_Atta(con, x).w),		\
 			       FCP_Atta(con, x).offset));		\
 	}								\
     }									\
     else if (FCP_Atta(con, x).type == XmATTACH_POSITION)		\
     {									\
-	DEBUGOUT(XdbDebug0(__FILE__, (Widget)f, "%s(%d/%d,%d)\t",	\
-		 XdbAttachment2String(FCP_Atta(con, x).type),		\
+	DEBUGOUT(_LtDebugDebug0(__FILE__, (Widget)f, "%s(%d/%d,%d)\t",	\
+		 _LtDebugAttachment2String(FCP_Atta(con, x).type),		\
 		 FCP_Atta(con, x).percent, Form_FractionBase(f),		\
 		 FCP_Atta(con, x).offset));				\
     }									\
     else								\
     {									\
-	DEBUGOUT(XdbDebug0(__FILE__, (Widget)f,  "%s(%d)\t",		\
-		 XdbAttachment2String(FCP_Atta(con, x).type),		\
+	DEBUGOUT(_LtDebugDebug0(__FILE__, (Widget)f,  "%s(%d)\t",		\
+		 _LtDebugAttachment2String(FCP_Atta(con, x).type),		\
 		 FCP_Atta(con, x).offset));				\
     }
 
-void
+
+extern void
 XmFormPrintAttachmentReport(Widget f)
 {
-    int i;
+    Cardinal i;
 
-    DEBUGOUT(XdbDebug(__FILE__, (Widget)f,
+    DEBUGOUT(_LtDebugDebug(__FILE__, (Widget)f,
 		      "Attachment Report : (Top,Bottom,Left,Right)\n"));
 
     for (i = 0; i < MGR_NumChildren(f); i++)
@@ -1552,20 +1588,21 @@ XmFormPrintAttachmentReport(Widget f)
 	Widget child = MGR_Children(f)[i];
 	XmFormConstraints con = (XmFormConstraints)CoreConstraints(child);
 
-	DEBUGOUT(XdbDebug0(__FILE__, (Widget)f,
+	DEBUGOUT(_LtDebugDebug0(__FILE__, (Widget)f,
 			   "child %s\t\t", XtName(child)));
 
 	PRINTIT(TOP);
 	PRINTIT(BOTTOM);
-	DEBUGOUT(XdbDebug0(__FILE__, (Widget)f, "\n\t\t\t\t"));
+	DEBUGOUT(_LtDebugDebug0(__FILE__, (Widget)f, "\n\t\t\t\t"));
 	PRINTIT(LEFT);
 	PRINTIT(RIGHT);
 
-	DEBUGOUT(XdbDebug0(__FILE__, (Widget)f, "\n"));
+	DEBUGOUT(_LtDebugDebug0(__FILE__, (Widget)f, "\n"));
     }
 }
 #undef	PRINTIT
 
+
 /*
  * XmFormPath
  *
@@ -1692,11 +1729,12 @@ XmFormPath(Widget f, Widget c, int dir, XmFormAttachment atta)
     return (x);
 }
 
+
 static void
 XmFormAllPaths(Widget f, Dimension *ww, Dimension *hh)
 {
     int y;
-    int i;
+    Cardinal i;
     XmFormAttachmentRec atta[4];
 
     *hh = 0;
@@ -1723,7 +1761,8 @@ XmFormAllPaths(Widget f, Dimension *ww, Dimension *hh)
 		atta[BOTTOM].type == XmATTACH_POSITION)
 	    {
 		/* test 2 test30 nedit */
-		y = (y * Form_FractionBase(f) / (atta[BOTTOM].percent -
+		if (atta[BOTTOM].percent != atta[TOP].percent)
+			y = (y * Form_FractionBase(f) / (atta[BOTTOM].percent -
 						 atta[TOP].percent));
 		atta[TOP].tempValue = 0;
 	    }
@@ -1732,24 +1771,33 @@ XmFormAllPaths(Widget f, Dimension *ww, Dimension *hh)
 	    int y1;
 
 		/* test18 test19 test27 */
-		y1 = (int)(((10 * (y) * Form_FractionBase(f) /
-			    atta[BOTTOM].percent) + 5) / 10);
-		atta[TOP].tempValue = -y + (y1 * atta[BOTTOM].percent) / Form_FractionBase(f);
+		/* Hack divide by 0 */
+		if (atta[BOTTOM].percent == 0)
+			y1 = (int)(((10 * (y) * Form_FractionBase(f)) + 5) / 10);
+		else
+			y1 = (int)(((10 * (y) * Form_FractionBase(f) / atta[BOTTOM].percent) + 5) / 10);
+		/* Hack divide by 0 */
+		if (Form_FractionBase(f) == 0)
+			atta[TOP].tempValue = 100;	/* Does this make sense ? */
+		else
+			atta[TOP].tempValue = -y + (y1 * atta[BOTTOM].percent) / Form_FractionBase(f);
 		y = y1;
 	    }
 	    else if (atta[TOP].type == XmATTACH_POSITION)
 	    {
 		/* test28 test29 */
-		y = (int)(((10 * (y) * Form_FractionBase(f) /
-			  (Form_FractionBase(f) - atta[TOP].percent)) + 5) / 10);
-		atta[TOP].tempValue = 0;
-		atta[TOP].tempValue = (y * atta[TOP].percent) / Form_FractionBase(f);
+		/* Hack divide by 0 */
+		if (Form_FractionBase(f) != atta[TOP].percent)
+			y = (int)(((10 * (y) * Form_FractionBase(f) / (Form_FractionBase(f) - atta[TOP].percent)) + 5) / 10);
+		/* Hack divide by 0 */
+		if (Form_FractionBase(f) != 0)
+			atta[TOP].tempValue = (y * atta[TOP].percent) / Form_FractionBase(f);
 	    }
 	    else
 	    {
 		y += atta[TOP].tempValue;
 	    }
-	    DEBUGOUT(XdbDebug2(__FILE__, (Widget)f, child,
+	    DEBUGOUT(_LtDebugDebug2(__FILE__, (Widget)f, child,
 			       "XmFormAllPaths H %5i %i %i\n", y, atta[TOP].tempValue, atta[BOTTOM].tempValue));
 	    if (y > *hh)
 	    {
@@ -1781,33 +1829,43 @@ XmFormAllPaths(Widget f, Dimension *ww, Dimension *hh)
 	    if (atta[LEFT].type == XmATTACH_POSITION &&
 		atta[RIGHT].type == XmATTACH_POSITION)
 	    {
-		y = (y * Form_FractionBase(f) / (atta[RIGHT].percent -
-						 atta[LEFT].percent));
+		/* Hack divide by 0 */
+		if (atta[RIGHT].percent != atta[LEFT].percent)
+			y = (y * Form_FractionBase(f) / (atta[RIGHT].percent - atta[LEFT].percent));
 		atta[LEFT].tempValue = 0;
 	    }
 	    else if (atta[RIGHT].type == XmATTACH_POSITION)
 	    {
 	    int y1;
 
-		y1 = ((10 * (y) * Form_FractionBase(f) /
-		      atta[RIGHT].percent) + 5) / 10;
-		atta[LEFT].tempValue = 0;
-		atta[LEFT].tempValue = -y + (y1 * atta[RIGHT].percent) / Form_FractionBase(f);
+		/* Hack divide by 0 */
+		if (atta[RIGHT].percent != 0)
+			y1 = ((10 * (y) * Form_FractionBase(f) / atta[RIGHT].percent) + 5) / 10;
+		else
+			y1 = y;
+		/* Hack divide by 0 */
+		if (Form_FractionBase(f) != 0)
+			atta[LEFT].tempValue = -y + (y1 * atta[RIGHT].percent) / Form_FractionBase(f);
+		else
+			atta[LEFT].tempValue = y;
 		y = y1;
 	    }
 	    else if (atta[LEFT].type == XmATTACH_POSITION)
 	    {
 		/* test13 test31 mgdiff */
-		y = ((10 * (y) * Form_FractionBase(f) /
-		      (Form_FractionBase(f) - atta[LEFT].percent)) + 5) / 10;
-		atta[LEFT].tempValue = 0;
-		atta[LEFT].tempValue = ((y * atta[LEFT].percent) / Form_FractionBase(f));
+		/* Hack divide by 0 */
+		if (Form_FractionBase(f) != atta[LEFT].percent)
+			y = ((10 * (y) * Form_FractionBase(f) / (Form_FractionBase(f) - atta[LEFT].percent)) + 5) / 10;
+		if (Form_FractionBase(f) != 0)
+			atta[LEFT].tempValue = ((y * atta[LEFT].percent) / Form_FractionBase(f));
+		else
+			atta[LEFT].tempValue = y;
 	    }
 	    else
 	    {
 		y += atta[LEFT].tempValue;
 	    }
-	    DEBUGOUT(XdbDebug2(__FILE__, (Widget)f, child,
+	    DEBUGOUT(_LtDebugDebug2(__FILE__, (Widget)f, child,
 			       "XmFormAllPaths W %5i %i %i\n", y, atta[LEFT].tempValue, atta[RIGHT].tempValue));
 	    if (y > *ww)
 	    {
@@ -1816,10 +1874,11 @@ XmFormAllPaths(Widget f, Dimension *ww, Dimension *hh)
 	}
     }
 
-    DEBUGOUT(XdbDebug(__FILE__, (Widget)f,
+    DEBUGOUT(_LtDebugDebug(__FILE__, (Widget)f,
 		      "XmFormAllPaths returning %d %d\n", *ww, *hh));
 }
 
+
 /*
  * _XmFormLayout() - run the XmForm layout algorithm
  *
@@ -1871,19 +1930,19 @@ _XmFormLayout(Widget f, Widget cw, XtWidgetGeometry *cg,
 {
     int number_of_iterations = 0;
     Boolean changed;
-    int i;
+    Cardinal i;
     Dimension fw, fh;		/* Form dimensions */
     Boolean WidthChangeMode = True, HeightChangeMode = True;
     Boolean ParentChangeMode = True;
 
 #define	Wants(x)	(cg->request_mode & x)
 
-    DEBUGOUT(XdbDebug(__FILE__, (Widget)f,
+    DEBUGOUT(_LtDebugDebug(__FILE__, (Widget)f,
 		      "_XmFormLayout with width %d height %d\n",
 		      XtWidth(f), XtHeight(f)));
-    DEBUGOUT(XdbDebug2(__FILE__, (Widget)f, cw,
+    DEBUGOUT(_LtDebugDebug2(__FILE__, (Widget)f, cw,
 		       "_XmFormLayout request %s\n",
-		       XdbWidgetGeometry2String(cg)));
+		       _LtDebugWidgetGeometry2String(cg)));
 
     /*
      * Special case ... should this be a special case (maybe our code below
@@ -1892,7 +1951,7 @@ _XmFormLayout(Widget f, Widget cw, XtWidgetGeometry *cg,
      */
     if (_XmGeoCount_kids((CompositeRec *)f) == 0)
     {
-	DEBUGOUT(XdbDebug(__FILE__, (Widget)f,
+	DEBUGOUT(_LtDebugDebug(__FILE__, (Widget)f,
 			  "_XmFormLayout: Form has no managed kids!\n"));
 
 	*width = XtWidth(f);
@@ -1931,12 +1990,12 @@ _XmFormLayout(Widget f, Widget cw, XtWidgetGeometry *cg,
 	ParentChangeMode = False;
     }
 
-    DEBUGOUT(XdbDebug(__FILE__, (Widget)f,
+    DEBUGOUT(_LtDebugDebug(__FILE__, (Widget)f,
 		      "XmFormLayout\n\tWidthChangeMode  %s\n"
 		      "\tHeightChangeMode %s\n\tParentChangeMode %s\n",
-		      XdbBoolean2String(WidthChangeMode),
-		      XdbBoolean2String(HeightChangeMode),
-		      XdbBoolean2String(ParentChangeMode)));
+		      _LtDebugBoolean2String(WidthChangeMode),
+		      _LtDebugBoolean2String(HeightChangeMode),
+		      _LtDebugBoolean2String(ParentChangeMode)));
 
     for (i = 0; i < MGR_NumChildren(f); i++)
     {
@@ -1982,7 +2041,7 @@ _XmFormLayout(Widget f, Widget cw, XtWidgetGeometry *cg,
 	if (FCP_Atta(con, LEFT).type == XmATTACH_NONE &&
 	    FCP_Atta(con, RIGHT).type == XmATTACH_NONE)
 	{
-	    DEBUGOUT(XdbDebug2(__FILE__, (Widget)f, child,
+	    DEBUGOUT(_LtDebugDebug2(__FILE__, (Widget)f, child,
 			       "left & right were NONE\n"));
 
 	    if (Form_RubberPos(f) == True)
@@ -1994,7 +2053,7 @@ _XmFormLayout(Widget f, Widget cw, XtWidgetGeometry *cg,
 		    (FCP_Atta(con, LEFT).value / Form_FractionBase(f));
 		    */
 
-		DEBUGOUT(XdbDebug2(__FILE__, (Widget)f, child,
+		DEBUGOUT(_LtDebugDebug2(__FILE__, (Widget)f, child,
 				   "Left set to ATTACH_POSITION %d\n",
 				   FCP_Atta(con, LEFT).percent));
 	    }
@@ -2006,7 +2065,7 @@ _XmFormLayout(Widget f, Widget cw, XtWidgetGeometry *cg,
 		 */
 		FCP_Atta(con, LEFT).offset = XtX(child) - BB_MarginWidth(f);
 
-		DEBUGOUT(XdbDebug2(__FILE__, (Widget)f, child,
+		DEBUGOUT(_LtDebugDebug2(__FILE__, (Widget)f, child,
 				   "Left set to ATTACH_FORM %d\n",
 				   FCP_Atta(con, LEFT).offset));
 	    }
@@ -2019,7 +2078,7 @@ _XmFormLayout(Widget f, Widget cw, XtWidgetGeometry *cg,
 	if (FCP_Atta(con, TOP).type == XmATTACH_NONE &&
 	    FCP_Atta(con, BOTTOM).type == XmATTACH_NONE)
 	{
-	    DEBUGOUT(XdbDebug2(__FILE__, (Widget)f, child,
+	    DEBUGOUT(_LtDebugDebug2(__FILE__, (Widget)f, child,
 			       "top & bottom were NONE\n"));
 
 	    if (Form_RubberPos(f) == True)
@@ -2031,7 +2090,7 @@ _XmFormLayout(Widget f, Widget cw, XtWidgetGeometry *cg,
 		    (FCP_Atta(con, TOP).value / Form_FractionBase(f));
 		    */
 
-		DEBUGOUT(XdbDebug2(__FILE__, (Widget)f, child,
+		DEBUGOUT(_LtDebugDebug2(__FILE__, (Widget)f, child,
 				   "Top set to ATTACH_POSITION %d\n",
 				   FCP_Atta(con, TOP).percent));
 	    }
@@ -2040,7 +2099,7 @@ _XmFormLayout(Widget f, Widget cw, XtWidgetGeometry *cg,
 		FCP_Atta(con, TOP).type = XmATTACH_FORM;
 		FCP_Atta(con, TOP).offset = XtY(child) - BB_MarginHeight(f);
 
-		DEBUGOUT(XdbDebug2(__FILE__, (Widget)f, child,
+		DEBUGOUT(_LtDebugDebug2(__FILE__, (Widget)f, child,
 				   "Top set to ATTACH_FORM %d\n",
 				   FCP_Atta(con, TOP).offset));
 	    }
@@ -2053,16 +2112,16 @@ _XmFormLayout(Widget f, Widget cw, XtWidgetGeometry *cg,
 #endif
 
 #ifdef PRINT_REPORT
-    DEBUGOUT(XdbDebug(__FILE__, (Widget)f,
+    DEBUGOUT(_LtDebugDebug(__FILE__, (Widget)f,
 		      "Initial geometry : %d %d\n", fw, fh));
-    DEBUGOUT(XdbDebug0(__FILE__, (Widget)f, "Initial child geometry :\n"));
+    DEBUGOUT(_LtDebugDebug0(__FILE__, (Widget)f, "Initial child geometry :\n"));
 
     for (i = 0; i < MGR_NumChildren(f); i++)
     {
 	Widget child = MGR_Children(f)[i];
 	XmFormConstraints con = (XmFormConstraints)CoreConstraints(child);
 
-	DEBUGOUT(XdbDebug0(__FILE__, (Widget)f,
+	DEBUGOUT(_LtDebugDebug0(__FILE__, (Widget)f,
 			   "Child #%d (%s)\tx %d y %d w %d h %d\n",
 			   i, XtName(child),
 			   FCP_X(con), FCP_Y(con),
@@ -2081,7 +2140,7 @@ _XmFormLayout(Widget f, Widget cw, XtWidgetGeometry *cg,
      */
     do
     {
-	DEBUGOUT(XdbDebug(__FILE__, (Widget)f,
+	DEBUGOUT(_LtDebugDebug(__FILE__, (Widget)f,
 			  "Layout iteration %d\n", number_of_iterations));
 
 	changed = False;
@@ -2263,7 +2322,7 @@ _XmFormLayout(Widget f, Widget cw, XtWidgetGeometry *cg,
 
 		    if (FCP_Y(con) != pos)
 		    {
-			DEBUGOUT(XdbDebug(__FILE__, (Widget)f,
+			DEBUGOUT(_LtDebugDebug(__FILE__, (Widget)f,
 					  "Computed pos as %d\n", pos));
 
 			if (!HeightChangeMode && FCP_Atta(con, BOTTOM).type != XmATTACH_NONE)
@@ -2822,7 +2881,7 @@ _XmFormLayout(Widget f, Widget cw, XtWidgetGeometry *cg,
 	_XmWarning((Widget)f,
 		   "Layout algorithm bailing out after %d iterations",
 		   MAX_ITERATIONS);
-	DEBUGOUT(XdbDebug(__FILE__, (Widget)f,
+	DEBUGOUT(_LtDebugDebug(__FILE__, (Widget)f,
 			  "Bailing out after %d iterations",
 			  MAX_ITERATIONS));
     }
@@ -2831,14 +2890,14 @@ _XmFormLayout(Widget f, Widget cw, XtWidgetGeometry *cg,
      * handy/dandy debug data.
      */
 #ifdef	PRINT_REPORT
-    DEBUGOUT(XdbDebug(__FILE__, (Widget)f,
+    DEBUGOUT(_LtDebugDebug(__FILE__, (Widget)f,
 		      "XmFormLayout results : form size %d %d\n", fw, fh));
     for (i = 0; i < MGR_NumChildren(f); i++)
     {
 	Widget child = MGR_Children(f)[i];
 	XmFormConstraints con = (XmFormConstraints)CoreConstraints(child);
 
-	DEBUGOUT(XdbDebug0(__FILE__, (Widget)f,
+	DEBUGOUT(_LtDebugDebug0(__FILE__, (Widget)f,
 			   "\tChild #%d (%s)\tx %d y %d w %d h %d %s\n",
 			   i, XtName(child), FCP_X(con),
 			   FCP_Y(con), FCP_W(con), FCP_H(con),
@@ -2871,6 +2930,7 @@ _XmFormLayout(Widget f, Widget cw, XtWidgetGeometry *cg,
     }
 }
 
+
 static XtGeometryResult
 _XmFormGeomRequest(Widget f, Dimension *width, Dimension *height)
 {
@@ -2883,7 +2943,7 @@ _XmFormGeomRequest(Widget f, Dimension *width, Dimension *height)
     request.height = *height;
     request.border_width = XtBorderWidth(f);
 
-    DEBUGOUT(XdbDebug(__FILE__, (Widget)f,
+    DEBUGOUT(_LtDebugDebug(__FILE__, (Widget)f,
 		      "Form is making a GeometryRequest to %s!!!!!!!!!\n",
 		      XtName(XtParent(f))));
 
@@ -2908,24 +2968,24 @@ _XmFormGeomRequest(Widget f, Dimension *width, Dimension *height)
     {
 	result = _XmMakeGeometryRequest((Widget)f, &request);
 
-	DEBUGOUT(XdbDebug(__FILE__, (Widget)f,
+	DEBUGOUT(_LtDebugDebug(__FILE__, (Widget)f,
 			  "_XmMakeGeometryRequest (w %d h %d) => %s\n",
 			  request.width, request.height,
-			  XdbGeometryResult2String(result)));
-	DEBUGOUT(XdbDebug("GM", (Widget)f,
+			  _LtDebugGeometryResult2String(result)));
+	DEBUGOUT(_LtDebugDebug("GM", (Widget)f,
 			  "_XmMakeGeometryRequest (w %d h %d) => %s\n",
 			  request.width, request.height,
-			  XdbGeometryResult2String(result)));
+			  _LtDebugGeometryResult2String(result)));
 
 	if (result == XtGeometryYes)
 	{
 	    *width = request.width;
 	    *height = request.height;
 
-	    DEBUGOUT(XdbDebug(__FILE__, (Widget)f,
+	    DEBUGOUT(_LtDebugDebug(__FILE__, (Widget)f,
 			      "Got Yes for _XmMakeGeometryRequest: %d %d\n",
 			      *width, *height));
-	    DEBUGOUT(XdbDebug("GM", (Widget)f,
+	    DEBUGOUT(_LtDebugDebug("GM", (Widget)f,
 			      "Got Yes for _XmMakeGeometryRequest: %d %d\n",
 			      *width, *height));
 	}
@@ -2944,12 +3004,13 @@ _XmFormGeomRequest(Widget f, Dimension *width, Dimension *height)
     return result;
 }
 
+
 static void
 _XmFormConfigureChildren(Widget f, Widget cw, XtWidgetGeometry *cg)
 {
-    int i;
+    Cardinal i;
 
-    DEBUGOUT(XdbDebug(__FILE__, (Widget)f,
+    DEBUGOUT(_LtDebugDebug(__FILE__, (Widget)f,
 		      "_XmFormConfigureChildren:  Child is %s\n",
 		      cw ? XtName(cw) : "(null)"));
 
@@ -2963,7 +3024,7 @@ _XmFormConfigureChildren(Widget f, Widget cw, XtWidgetGeometry *cg)
 	    continue;
 	if (cw && cw == child)
 	{
-	    DEBUGOUT(XdbDebug(__FILE__, (Widget)f,
+	    DEBUGOUT(_LtDebugDebug(__FILE__, (Widget)f,
 			      "_XmFormConfigureChildren:  Child is %s\n",
 			      cw ? XtName(cw) : "(null)"));
 
@@ -3001,6 +3062,7 @@ _XmFormConfigureChildren(Widget f, Widget cw, XtWidgetGeometry *cg)
 #undef	Wants
 }
 
+
 static void
 _XmFormExportOffset(Widget widget, int offset, XtArgVal *value)
 {
@@ -3010,6 +3072,7 @@ _XmFormExportOffset(Widget widget, int offset, XtArgVal *value)
     }
 }
 
+
 static XmImportOperator
 _XmFormImportOffset(Widget widget, int offset, XtArgVal *value)
 {
@@ -3018,7 +3081,8 @@ _XmFormImportOffset(Widget widget, int offset, XtArgVal *value)
 
 /************************ EXTERNAL INTERFACE ****************************/
 
-Widget
+
+extern Widget
 XmCreateForm(Widget parent, char *name,
 	     Arg *arglist, Cardinal argCount)
 {
@@ -3026,13 +3090,15 @@ XmCreateForm(Widget parent, char *name,
 			  arglist, argCount);
 }
 
-Widget
+
+extern Widget
 XmCreateFormDialog(Widget parent, char *name,
 		   Arg *arglist, Cardinal argcount)
 {
     Widget shell, r;
     char *shell_name;
-    int ac, i;
+    int ac;
+    Cardinal i;
     Arg *al = (Arg *)XtCalloc(argcount + 1, sizeof(Arg));
 
     ac = 0;
@@ -3049,7 +3115,7 @@ XmCreateFormDialog(Widget parent, char *name,
     XtFree(shell_name);
 
     r = XmCreateForm(shell, name, al, ac);
-    XtFree((XtPointer)al);
+    XtFree((char *)al);
 
     return r;
 }
