@@ -1,8 +1,9 @@
 /** 
  *
- * $Id: Frame.c,v 1.47 2000/01/09 00:12:33 rwscott Exp $
+ * $Id: Frame.c,v 1.53 2001/05/14 19:46:08 amai Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
+ * Copyright (C) 1995-2001 LessTif Development Team
  *
  * This file is part of the GNU LessTif Library.
  *
@@ -22,7 +23,7 @@
  *
  **/
 
-static const char rcsid[] = "$Id: Frame.c,v 1.47 2000/01/09 00:12:33 rwscott Exp $";
+static const char rcsid[] = "$Id: Frame.c,v 1.53 2001/05/14 19:46:08 amai Exp $";
 
 /*
  * Warning : this widget has a small amount of Motif 2.0 support.
@@ -420,7 +421,7 @@ set_values(Widget current, Widget request, Widget new_w,
 
     if (redisplay && XtIsRealized(new_w) && XtIsManaged(new_w))
     {
-	DEBUGOUT(XdbDebug(__FILE__, new_w,
+	DEBUGOUT(_LtDebugDebug(__FILE__, new_w,
 			  "Frame set_values %p W %i H %i\n",
 			  new_w, XtWidth(new_w), XtHeight(new_w)));
 
@@ -455,7 +456,7 @@ set_values(Widget current, Widget request, Widget new_w,
 static void
 resize(Widget w)
 {
-    DEBUGOUT(XdbDebug(__FILE__, w,
+    DEBUGOUT(_LtDebugDebug(__FILE__, w,
 		      "Frame resize (%d %d)\n", XtWidth(w), XtHeight(w)));
 
 #if CON_FROM_RESIZE
@@ -535,7 +536,7 @@ query_geometry(Widget w, XtWidgetGeometry *request, XtWidgetGeometry *reply)
     XtGeometryResult r;
     XtWidgetGeometry rr = *request;
 
-    DEBUGOUT(XdbDebug(__FILE__, w,
+    DEBUGOUT(_LtDebugDebug(__FILE__, w,
 		      "Frame query_geometry %p W %i H %i\n",
 		       w, XtWidth(w), XtHeight(w)));
 
@@ -563,16 +564,16 @@ query_geometry(Widget w, XtWidgetGeometry *request, XtWidgetGeometry *reply)
 	r = XtGeometryAlmost;
     }
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "QueryGeometry [%s] => %s [%s]\n",
-		      XdbWidgetGeometry2String(&rr),
-		      XdbGeometryResult2String(r),
-		      XdbWidgetGeometry2String(reply)));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "QueryGeometry [%s] => %s [%s]\n",
+		      _LtDebugWidgetGeometry2String(&rr),
+		      _LtDebugGeometryResult2String(r),
+		      _LtDebugWidgetGeometry2String(reply)));
 
     return r;
 #else
     Dimension wd, ht;
 
-    DEBUGOUT(XdbDebug(__FILE__, w,
+    DEBUGOUT(_LtDebugDebug(__FILE__, w,
 		      "Frame query_geometry %p W %i H %i\n",
 		       w, XtWidth(w), XtHeight(w)));
 
@@ -602,8 +603,8 @@ geometry_manager(Widget w,
     Widget fw = XtParent(w);
     XtWidgetGeometry wants;
 
-    DEBUGOUT(XdbDebug2(__FILE__, fw, w, "Frame geometry_manager [%s]\n",
-		       XdbWidgetGeometry2String(request)));
+    DEBUGOUT(_LtDebugDebug2(__FILE__, fw, w, "Frame geometry_manager [%s]\n",
+		       _LtDebugWidgetGeometry2String(request)));
 
     wants = *request;
     *reply = wants;
@@ -611,7 +612,7 @@ geometry_manager(Widget w,
     if (FrameC_ChildType(w) != XmFRAME_WORKAREA_CHILD &&
 	FrameC_ChildType(w) != XmFRAME_TITLE_CHILD)
     {
-	DEBUGOUT(XdbDebug(__FILE__, w, "Frame geometry_manager => No\n"));
+	DEBUGOUT(_LtDebugDebug(__FILE__, w, "Frame geometry_manager => No\n"));
 
 	return XtGeometryNo;
     }
@@ -620,14 +621,14 @@ geometry_manager(Widget w,
     {
 	_XmWarning(w, "%s(%d):geometry_manager() - Widget requesting %s from %s\n\tReturning XtGeometryNo\n",
 		__FILE__, __LINE__, 
-		XdbWidgetGeometry2String(request),
+		_LtDebugWidgetGeometry2String(request),
 		XtName(fw));
 	return XtGeometryNo;
     }
 
     /* Calculate desired Frame geometry in wd/ht from requested child
      * geometry in "request" */
-    DEBUGOUT(XdbDebug(__FILE__, fw,
+    DEBUGOUT(_LtDebugDebug(__FILE__, fw,
 		      "Frame geometry_manager %p W %i H %i\n",
 		       fw, XtWidth(fw), XtHeight(fw)));
 
@@ -642,40 +643,50 @@ geometry_manager(Widget w,
 	geo.request_mode = CWWidth | CWHeight | CWBorderWidth;
 	geo.border_width = XtBorderWidth(fw);
 
-	DEBUGOUT(XdbDebug(__FILE__, fw,
+	DEBUGOUT(_LtDebugDebug(__FILE__, fw,
 			  "Frame requesting geo: %s : is %d %d\n",
-			  XdbWidgetGeometry2String(&geo),
+			  _LtDebugWidgetGeometry2String(&geo),
 			  XtWidth(fw), XtHeight(fw)));
 
 	if (_XmMakeGeometryRequest(fw, &geo) == XtGeometryYes)
 	{
-	    DEBUGOUT(XdbDebug(__FILE__, fw,
+	    DEBUGOUT(_LtDebugDebug(__FILE__, fw,
 			      "Got geo %d %d : is %d %d\n",
 			      geo.width, geo.height,
 			      XtWidth(fw), XtHeight(fw)));
 	}
 	else
 	{
-	    DEBUGOUT(XdbDebug(__FILE__, fw,
+	    DEBUGOUT(_LtDebugDebug(__FILE__, fw,
 			      "No. Geo is %d %d : WH: %d %d\n",
 			      geo.width, geo.height,
 			      XtWidth(fw), XtHeight(fw)));
 	}
 
-	DEBUGOUT(XdbDebug2(__FILE__, fw, w,
+	DEBUGOUT(_LtDebugDebug2(__FILE__, fw, w,
 			"geometry_manager : _XmFrameConfigureChildren => %s\n",
-			   XdbWidgetGeometry2String(reply)));
+			   _LtDebugWidgetGeometry2String(reply)));
 
+	_XmClearShadowType(fw, 
+	    Frame_OldWidth(fw), 
+	    Frame_OldHeight(fw),
+	    Frame_OldShadowThickness(fw),
+	    0);
 	_XmFrameConfigureChildren(fw, w, reply, XtWidth(fw), XtHeight(fw),
 				  False);
     }
     else
     {
+	_XmClearShadowType(fw, 
+	    Frame_OldWidth(fw), 
+	    Frame_OldHeight(fw),
+	    Frame_OldShadowThickness(fw),
+	    0);
 	_XmFrameConfigureChildren(fw, w, reply, XtWidth(fw), XtHeight(fw),
 				  False);
-	DEBUGOUT(XdbDebug2(__FILE__, fw, w,
+	DEBUGOUT(_LtDebugDebug2(__FILE__, fw, w,
 			 "geometry_manager determined child size at %d [%s]\n",
-			   __LINE__, XdbWidgetGeometry2String(reply)));
+			   __LINE__, _LtDebugWidgetGeometry2String(reply)));
     }
 
     Frame_OldShadowX(fw) = 0;
@@ -694,10 +705,13 @@ geometry_manager(Widget w,
     Frame_OldWidth(fw) = XtWidth(fw);
     Frame_OldHeight(fw) = XtHeight(fw);
 
+    DEBUGOUT(_LtDebugDebug2(__FILE__, fw, w,
+			   "Frame geometry_manager => (%s) (%s)\n",
+			   _LtDebugWidgetGeometry2String(&wants), _LtDebugWidgetGeometry2String(reply)));
     if ((wants.request_mode & (CWWidth | CWHeight)) == (CWWidth | CWHeight) &&
 	reply->width == wants.width && reply->height == wants.height)
     {
-	DEBUGOUT(XdbDebug2(__FILE__, fw, w,
+	DEBUGOUT(_LtDebugDebug2(__FILE__, fw, w,
 			   "Frame geometry_manager => Yes (%d %d)\n",
 			   reply->width, reply->height));
 
@@ -714,17 +728,34 @@ geometry_manager(Widget w,
     }
     else if (reply->width == XtWidth(w) && reply->height == XtHeight(w))
     {
-	DEBUGOUT(XdbDebug2(__FILE__, fw, w,
+	DEBUGOUT(_LtDebugDebug2(__FILE__, fw, w,
 			   "Frame geometry_manager same as current=> No\n"));
 
 	return XtGeometryNo;
     }
     else
     {
-	DEBUGOUT(XdbDebug2(__FILE__, fw, w,
+	DEBUGOUT(_LtDebugDebug2(__FILE__, fw, w,
 			   "Frame geometry_manager => Almost (%d %d)\n",
 			   reply->width, reply->height));
 
+	reply->request_mode &= wants.request_mode;
+	if (reply->request_mode & CWX && request->x == wants.x)
+	{
+	    reply->request_mode &= ~CWX;
+	}
+	if (reply->request_mode & CWY && request->y == wants.y)
+	{
+	    reply->request_mode &= ~CWY;
+	}
+	if (reply->request_mode & CWWidth && request->width == wants.width)
+	{
+	    reply->request_mode &= ~CWWidth;
+	}
+	if (reply->request_mode & CWHeight && request->height == wants.height)
+	{
+	    reply->request_mode &= ~CWHeight;
+	}
 	return XtGeometryAlmost;
     }
 }
@@ -732,7 +763,7 @@ geometry_manager(Widget w,
 static void
 change_managed(Widget w)
 {
-    int i;
+    Cardinal i;
     Widget child;
     Dimension wd, ht;
     XtWidgetGeometry request;
@@ -762,16 +793,21 @@ change_managed(Widget w)
 	    }
 	    else
 	    {
-		DEBUGOUT(XdbDebug(__FILE__, w,
+		DEBUGOUT(_LtDebugDebug(__FILE__, w,
 				  "XmFrame illegal child type resource\n"));
 	    }
 	}
     }
 
-    DEBUGOUT(XdbDebug(__FILE__, w,
+    DEBUGOUT(_LtDebugDebug(__FILE__, w,
 		      "Frame change_managed %p W %i H %i\n",
 		       w, XtWidth(w), XtHeight(w)));
 
+    _XmClearShadowType(w, 
+    	Frame_OldWidth(w), 
+    	Frame_OldHeight(w),
+    	Frame_OldShadowThickness(w),
+    	0);
     _XmFrameComputeSize(w, NULL, NULL, &wd, &ht);
 
     request.request_mode = (CWWidth | CWHeight | CWBorderWidth);
@@ -813,10 +849,10 @@ constraint_initialize(Widget request, Widget new_w,
 		      ArgList args, Cardinal *num_args)
 {
 #if XmVERSION > 1
-    int	i;
+    Cardinal i;
 #endif
 
-    DEBUGOUT(XdbDebug2(__FILE__, XtParent(new_w), new_w, "%s:constraint_initialize()\n", __FILE__));
+    DEBUGOUT(_LtDebugDebug2(__FILE__, XtParent(new_w), new_w, "%s:constraint_initialize()\n", __FILE__));
 
 #if XmVERSION > 1
     if (num_args)
@@ -841,7 +877,7 @@ constraint_initialize(Widget request, Widget new_w,
 
 static void constraint_destroy(Widget w)
 {
-    DEBUGOUT(XdbDebug(__FILE__, w, "%s:constraint_destroy()\n", __FILE__));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "%s:constraint_destroy()\n", __FILE__));
 }
 
 static Boolean
@@ -851,7 +887,7 @@ constraint_set_values(Widget current, Widget request, Widget new_w,
     Boolean redisplay = False;
 
 #if XmVERSION > 1
-    int	i;
+    Cardinal i;
 
     if (num_args)
 	for (i=0; i<*num_args; i++) {
@@ -885,29 +921,29 @@ _XmFrameComputeSize(Widget w, Widget instig, XtWidgetGeometry *desired,
     Dimension curw, curh;
     XtWidgetGeometry geo;
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "_XmFrameComputeSize"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "_XmFrameComputeSize"));
 
     if (instig)
     {
-	DEBUGOUT(XdbDebug0(__FILE__, w, " Instig %s (desired geo %s)",
-			   XtName(instig), XdbWidgetGeometry2String(desired)));
+	DEBUGOUT(_LtDebugDebug0(__FILE__, w, " Instig %s (desired geo %s)",
+			   XtName(instig), _LtDebugWidgetGeometry2String(desired)));
     }
 
     if (Frame_TitleArea(w))
     {
-	DEBUGOUT(XdbDebug0(__FILE__, w, " Title %s wid %d ht %d",
+	DEBUGOUT(_LtDebugDebug0(__FILE__, w, " Title %s wid %d ht %d",
 		       XtName(Frame_TitleArea(w)), XtWidth(Frame_TitleArea(w)),
 			   XtHeight(Frame_TitleArea(w))));
     }
 
     if (Frame_WorkArea(w))
     {
-	DEBUGOUT(XdbDebug0(__FILE__, w, " Work %s wid %d ht %d",
+	DEBUGOUT(_LtDebugDebug0(__FILE__, w, " Work %s wid %d ht %d",
 			 XtName(Frame_WorkArea(w)), XtWidth(Frame_WorkArea(w)),
 			   XtHeight(Frame_WorkArea(w))));
     }
 
-    DEBUGOUT(XdbDebug0(__FILE__, w, "\n"));
+    DEBUGOUT(_LtDebugDebug0(__FILE__, w, "\n"));
 
     curw = curh = 0;
 
@@ -997,7 +1033,7 @@ _XmFrameComputeSize(Widget w, Widget instig, XtWidgetGeometry *desired,
 	*wd = XtWidth(w);
 	*ht = XtHeight(w);
 
-	DEBUGOUT(XdbDebug2(__FILE__, w, instig,
+	DEBUGOUT(_LtDebugDebug2(__FILE__, w, instig,
 			   "Not processing constraints %d %d\n", *wd, *ht));
     }
     else
@@ -1006,9 +1042,9 @@ _XmFrameComputeSize(Widget w, Widget instig, XtWidgetGeometry *desired,
 	*ht = curh;
     }
 
-    DEBUGOUT(XdbDebug2(__FILE__, w, instig,
+    DEBUGOUT(_LtDebugDebug2(__FILE__, w, instig,
 		       "_XmFrameComputeSize (%s) => %d %d : %d %d\n",
-		       XdbWidgetGeometry2String(desired), curw, curh,
+		       _LtDebugWidgetGeometry2String(desired), curw, curh,
 		       *wd, *ht));
 }
 
@@ -1019,7 +1055,7 @@ _XmFrameConfigureChildren(Widget w, Widget instig, XtWidgetGeometry *desired,
     int title_x = 0, title_y = 0;
     int workarea_x = 0, workarea_y = 0;
 
-    DEBUGOUT(XdbDebug2(__FILE__, w, instig,
+    DEBUGOUT(_LtDebugDebug2(__FILE__, w, instig,
 		       "_XmFrameConfigureChildren (Frame geo %d %d)\n",
 		       curw, curh));
 
@@ -1089,8 +1125,9 @@ _XmFrameConfigureChildren(Widget w, Widget instig, XtWidgetGeometry *desired,
 
 	if (Frame_TitleArea(w) == instig)
 	{
-	    DEBUGOUT(XdbDebug(__FILE__, w, "Instig is title\n"));
+	    DEBUGOUT(_LtDebugDebug(__FILE__, w, "Instig is title\n"));
 
+#if 0
 	    desired->request_mode = CWBorderWidth | CWWidth | CWHeight | CWX | CWY;
 	    desired->x = title_x;
 	    desired->y = title_y;
@@ -1100,11 +1137,25 @@ _XmFrameConfigureChildren(Widget w, Widget instig, XtWidgetGeometry *desired,
 		       - 2 * Frame_MarginWidth(w)
 		       - 2 * FrameC_ChildHSpacing(Frame_TitleArea(w)));
 	    desired->height = XtHeight(Frame_TitleArea(w));	/* HERE */
+#else
+	    /* rws 3 Jan 2000
+	       frame/test9
+	     */
+	    desired->x = desired->request_mode & CWX ? desired->x : title_x;
+	    desired->y = desired->request_mode & CWY ? desired->y : title_y;
+	    desired->width =
+		_XmMin(desired->request_mode & CWWidth ? desired->width : XtWidth(Frame_TitleArea(w)),
+		       curw - 2 * MGR_ShadowThickness(w)
+		       - 2 * Frame_MarginWidth(w)
+		       - 2 * FrameC_ChildHSpacing(Frame_TitleArea(w)));
+	    desired->height = desired->request_mode & CWHeight ? desired->height : XtHeight(Frame_TitleArea(w));	/* HERE */
+	    desired->request_mode = CWBorderWidth | CWWidth | CWHeight | CWX | CWY;
+#endif
 	    desired->border_width = XtBorderWidth(Frame_TitleArea(w));
 	}
 	else if (commit)
 	{
-	    DEBUGOUT(XdbDebug(__FILE__, w, "Configure title\n"));
+	    DEBUGOUT(_LtDebugDebug(__FILE__, w, "Configure title\n"));
 
 	    _XmConfigureObject(Frame_TitleArea(w),
 			       title_x, title_y,
@@ -1157,7 +1208,7 @@ _XmFrameConfigureChildren(Widget w, Widget instig, XtWidgetGeometry *desired,
 	}
 	else if (commit)
 	{
-	    DEBUGOUT(XdbDebug(__FILE__, w, "Configure work area\n"));
+	    DEBUGOUT(_LtDebugDebug(__FILE__, w, "Configure work area\n"));
 
 	    _XmConfigureObject(Frame_WorkArea(w),
 			       workarea_x, workarea_y,
