@@ -1,8 +1,9 @@
 /**
  *
- * $Id: Gadget.c,v 1.16 1999/02/02 18:23:10 gritton Exp $
+ * $Id: Gadget.c,v 1.18 2001/05/14 19:46:08 amai Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
+ * Copyright (C) 1995-2001 LessTif Development Team
  *
  * This file is part of the GNU LessTif Library.
  *
@@ -22,16 +23,17 @@
  *
  **/
 
-static const char rcsid[] = "$Id: Gadget.c,v 1.16 1999/02/02 18:23:10 gritton Exp $";
+static const char rcsid[] = "$Id: Gadget.c,v 1.18 2001/05/14 19:46:08 amai Exp $";
 
 #include <LTconfig.h>
-#include <XmI/XmI.h>
 
+#include <string.h>
+
+#include <XmI/XmI.h>
 #include <Xm/XmP.h>
 #include <Xm/GadgetP.h>
 #include <Xm/ExtObjectP.h>
 #include <Xm/RepType.h>
-#include <string.h>
 
 #include <XmI/DebugUtil.h>
 
@@ -508,68 +510,68 @@ input_dispatch(Widget w,
 	       XEvent *event,
 	       Mask event_mask)
 {
-    DEBUGOUT(XdbDebug(__FILE__, w,
+    DEBUGOUT(_LtDebugDebug(__FILE__, w,
 		      "Inside base Gadget input_dispatch() routine "
 		      "(you really should write one for this gadget... :)\n"));
 
     if (event_mask & XmENTER_EVENT)
     {
-	DEBUGOUT(XdbDebug(__FILE__, w,
+	DEBUGOUT(_LtDebugDebug(__FILE__, w,
 			  "Enter event sent to gadget %d\n", w));
     }
     else if (event_mask & XmLEAVE_EVENT)
     {
-	DEBUGOUT(XdbDebug(__FILE__, w,
+	DEBUGOUT(_LtDebugDebug(__FILE__, w,
 			  "Leave event sent to gadget %d\n", w));
     }
     else if (event_mask & XmFOCUS_IN_EVENT)
     {
-	DEBUGOUT(XdbDebug(__FILE__, w,
+	DEBUGOUT(_LtDebugDebug(__FILE__, w,
 			  "Focus in event sent to gadget %d\n", w));
     }
     else if (event_mask & XmFOCUS_OUT_EVENT)
     {
-	DEBUGOUT(XdbDebug(__FILE__, w,
+	DEBUGOUT(_LtDebugDebug(__FILE__, w,
 			  "Focus out event sent to gadget %d\n", w));
     }
     else if (event_mask & XmMOTION_EVENT)
     {
-	DEBUGOUT(XdbDebug(__FILE__, w,
+	DEBUGOUT(_LtDebugDebug(__FILE__, w,
 			  "Motion event sent to gadget %d\n", w));
     }
     else if (event_mask & XmARM_EVENT)
     {
-	DEBUGOUT(XdbDebug(__FILE__, w,
+	DEBUGOUT(_LtDebugDebug(__FILE__, w,
 			  "Arm event sent to gadget %d\n", w));
     }
     else if (event_mask & XmACTIVATE_EVENT)
     {
-	DEBUGOUT(XdbDebug(__FILE__, w,
+	DEBUGOUT(_LtDebugDebug(__FILE__, w,
 			  "Activate event sent to gadget %d\n", w));
     }
     else if (event_mask & XmHELP_EVENT)
     {
-	DEBUGOUT(XdbDebug(__FILE__, w,
+	DEBUGOUT(_LtDebugDebug(__FILE__, w,
 			  "Help event sent to gadget %d\n", w));
     }
     else if (event_mask & XmKEY_EVENT)
     {
-	DEBUGOUT(XdbDebug(__FILE__, w,
+	DEBUGOUT(_LtDebugDebug(__FILE__, w,
 			  "Key event sent to gadget %d\n", w));
     }
     else if (event_mask & XmMULTI_ARM_EVENT)
     {
-	DEBUGOUT(XdbDebug(__FILE__, w,
+	DEBUGOUT(_LtDebugDebug(__FILE__, w,
 			  "Multi arm event sent to gadget %d\n", w));
     }
     else if (event_mask & XmMULTI_ACTIVATE_EVENT)
     {
-	DEBUGOUT(XdbDebug(__FILE__, w,
+	DEBUGOUT(_LtDebugDebug(__FILE__, w,
 			  "Multi activate event sent to gadget %d\n", w));
     }
     else if (event_mask & XmBDRAG_EVENT)
     {
-	DEBUGOUT(XdbDebug(__FILE__, w,
+	DEBUGOUT(_LtDebugDebug(__FILE__, w,
 			  "BDrag event sent to gadget %d\n", w));
     }
 }
@@ -665,7 +667,7 @@ static void
 gadget_border_unhighlight(Widget w)
 {
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "gadget_border_unhighlight\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "gadget_border_unhighlight\n"));
 
     /* with zero width, we don't need this... */
     if (G_HighlightThickness(w) == 0)
@@ -697,7 +699,7 @@ static void
 gadget_border_highlight(Widget w)
 {
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "gadget_border_highlight\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "gadget_border_highlight\n"));
 
     /* with zero width, we don't need this... */
     if (G_HighlightThickness(w) == 0)
