@@ -1,8 +1,9 @@
 /**
  *
- * $Id: LTCvt.c,v 1.8 1999/09/03 00:17:33 rwscott Exp $
+ * $Id: LTCvt.c,v 1.21 2001/05/14 19:46:09 amai Exp $
  *
- * Copyright (C) 1995 Free Software Foundation, Inc.
+ * Copyright  1995 Free Software Foundation, Inc.
+ * Copyright  1995-2001 LessTif Development Team
  *
  * This file is part of the GNU LessTif Library.
  *
@@ -22,35 +23,61 @@
  *
  **/
 
-static const char rcsid[] = "$Id: LTCvt.c,v 1.8 1999/09/03 00:17:33 rwscott Exp $";
-
-#ifdef NONSTANDARD_CONVERTERS 
+static const char rcsid[] = "$Id: LTCvt.c,v 1.21 2001/05/14 19:46:09 amai Exp $";
 
 #include <LTconfig.h>
+
+#ifdef NONSTANDARD_CONVERTERS
+
+#include <stdio.h>
+#include <string.h>
+
 #include <XmI/XmI.h>
 #include <XmI/HashI.h>
-#include <X11/Xfuncs.h>
 #include <XmI/ImageCacheI.h>
-
 #include <Xm/XmP.h>
 #include <Xm/ScreenP.h>
-
-#include<XmI/XmXpm.h>
+#if XmVERSION >= 2
+#include <Xm/XpmP.h>
+#else
+#include <XmI/XmXpm.h>
+#endif
 
 #include <XmI/DebugUtil.h>
 
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
 
 static char *_search_path = NULL;
 
-extern void LTCreateSearchPath(void);
-extern Boolean LTAddPixmapToCache(char *pixmap_name, Pixmap pixmap,
-				  Screen *screen,
-				  Pixel foreground, Pixel background,
-				  int depth, int width, int height,
-				  int hot_x, int hot_y);
+/* Prototypes to ensure proper type for the definitions below: */
+static Boolean
+_XmNSECvtStringToBitmap (Display*, XrmValue*, Cardinal*, XrmValue*, XrmValue*, XtPointer*);
+static Boolean
+_XmNSECvtStringToPixmap (Display*, XrmValue*, Cardinal*, XrmValue*, XrmValue*, XtPointer*);
+static Boolean
+_XmNSECvtShapeStyleToString (Display*, XrmValue*, Cardinal*, XrmValue*, XrmValue*, XtPointer*);
+static Boolean
+_XmNSECvtBooleanToString (Display*, XrmValue*, Cardinal*, XrmValue*, XrmValue*, XtPointer*);
+static Boolean
+_XmNSECvtBoolToString (Display*, XrmValue*, Cardinal*, XrmValue*, XrmValue*, XtPointer*);
+static Boolean
+_XmNSECvtAttachmentToString (Display*, XrmValue*, Cardinal*, XrmValue*, XrmValue*, XtPointer*);
+static Boolean
+_XmNSECvtPositionToString (Display*, XrmValue*, Cardinal*, XrmValue*, XrmValue*, XtPointer*);
+static Boolean
+_XmNSECvtCardinalToString (Display*, XrmValue*, Cardinal*, XrmValue*, XrmValue*, XtPointer*);
+static Boolean
+_XmNSECvtPixelToString (Display*, XrmValue*, Cardinal*, XrmValue*, XrmValue*, XtPointer*);
+static Boolean
+_XmNSECvtColorToString (Display*, XrmValue*, Cardinal*, XrmValue*, XrmValue*, XtPointer*);
+static Boolean
+_XmNSECvtUnsignedCharToString (Display*, XrmValue*, Cardinal*, XrmValue*, XrmValue*, XtPointer*);
+static Boolean
+_XmNSECvtDimensionToString (Display*, XrmValue*, Cardinal*, XrmValue*, XrmValue*, XtPointer*);
+static Boolean
+_XmNSECvtXmStringToString (Display*, XrmValue*, Cardinal*, XrmValue*, XrmValue*, XtPointer*);
+static Boolean
+_XmNSECvtWidgetToString (Display*, XrmValue*, Cardinal*, XrmValue*, XrmValue*, XtPointer*);
+
 
 /*
  * _XmNSEGetPixmap is not part of the OSF/Motif API.
@@ -64,7 +91,11 @@ _XmNSEGetPixmap(Screen *screen, char *fname)
 {
     static Colormap _cmap;
 
+#if XmVERSION >= 2
+    XpmAttributes xpm_attrib;
+#else
     _LtXpmAttributes xpm_attrib;
+#endif
     Pixmap pmap;
     Pixmap mask;
 
@@ -96,7 +127,7 @@ _XmNSEGetPixmap(Screen *screen, char *fname)
 
     if (_search_path == NULL)
     {
-	LTCreateSearchPath();
+	_LTCreateSearchPath();
     }
 
     /*
@@ -129,6 +160,16 @@ _XmNSEGetPixmap(Screen *screen, char *fname)
 	return XmUNSPECIFIED_PIXMAP;
     }
 
+#if XmVERSION >= 2
+    xpm_attrib.colormap = _cmap;
+    xpm_attrib.closeness = 40000;
+    xpm_attrib.valuemask = XpmSize | XpmReturnPixels
+	| XpmColormap | XpmCloseness;
+
+    if (XpmReadFileToPixmap(dpy, w, pathname_to_pixmap,
+			       &pmap, &mask,
+			       &xpm_attrib) == XpmSuccess)
+#else
     xpm_attrib.colormap = _cmap;
     xpm_attrib.closeness = 40000;
     xpm_attrib.valuemask = _LtXpmSize | _LtXpmReturnPixels
@@ -137,8 +178,9 @@ _XmNSEGetPixmap(Screen *screen, char *fname)
     if (_LtXpmReadFileToPixmap(dpy, w, pathname_to_pixmap,
 			       &pmap, &mask,
 			       &xpm_attrib) == _LtXpmSuccess)
+#endif
     {
-	LTAddPixmapToCache(fname, pmap, screen, 0, 0, 0, 0, 0, 0, 0);
+	_LTAddPixmapToCache(fname, pmap, screen, 0, 0, 0, 0, 0, 0, 0);
 	/* FIX ME! */
     }
     else
@@ -220,11 +262,73 @@ _XmNSECvtStringToPixmap(Display *dpy,
 	}
     }
 
-    converter_data = converter_data;	/* keeps compiler happy */
-
     return True;
 }
 
+/*
+ * _XmNSECvtStringToPixmap is not part of the OSF/Motif API.
+ * This routine is used to convert a string to a pixmap.
+ * This is done by calling the _XmNSEGetPixmap routine with the
+ * supplied string.  
+ */
+static Boolean
+_XmNSECvtStringToBitmap(Display *dpy,
+		        XrmValue *args,
+		        Cardinal *num_args,
+		        XrmValue *from,
+		        XrmValue *to,
+		        XtPointer *converter_data)
+{
+	static Pixmap _pmap;
+	Screen *screen;
+	char *name;
+
+	DEBUGOUT(_LtDebugDebug(__FILE__, NULL, "_XmNSECvtStringToBitmap(from=%s) called\n",
+		(char *)from->addr));
+
+	if (_XmGetDefaultDisplay() == NULL) {
+		return False;
+	}
+
+	if (*num_args != 1) {
+		XtWarningMsg("wrongParameters", "cvtStringToBitmap", "XtToolkitError",
+			"String to Bitmap conversion needs screen argument",
+			(String *)NULL,
+			(Cardinal *)NULL);
+	}
+
+	/* get arguments */
+	screen = *((Screen **)args[0].addr);
+	name = (char *)from->addr;
+
+	/* over kill check */
+	if (name == NULL || strcmp(name, "None") == 0
+			|| strcmp(name, "XmUNSPECIFIED_PIXMAP") == 0) {
+		_pmap = XmUNSPECIFIED_PIXMAP;
+	} else {
+		fprintf(stderr, "_XmNSECvtStringToBitmap(from=%s) called\n",
+			(char *)from->addr);
+		_pmap = XmUNSPECIFIED_PIXMAP;		/* FIX ME */
+
+		return False;
+/*		_pmap = _XmNSEGetPixmap(screen, name);	*/
+	}
+
+	if (to->addr == NULL) {
+		to->addr = (XPointer)&_pmap;
+		to->size = sizeof(Pixmap);
+	} else {
+		if (to->size >= sizeof(Pixmap)) {
+			*((Pixmap *)to->addr) = _pmap;
+			to->size = sizeof(Pixmap);
+		} else {
+			XtDisplayStringConversionWarning(dpy, (char *)from->addr, XmRPixmap);
+		}
+	}
+
+	return True;
+}
+
 /* Begin stuff copied from X11/Xmu/Converters.h */
 #define XtRShapeStyle "ShapeStyle"
 #define XtERectangle "Rectangle"
@@ -249,8 +353,8 @@ typedef struct
     int value;
     char *name;
     int size;
-}
-ConversionMap;
+} ConversionMap;
+
 
 static ConversionMap boolean_map[] =
 {
@@ -262,6 +366,7 @@ static ConversionMap boolean_map[] =
     {False, XtEno, sizeof(XtEno)},
 };
 
+
 static ConversionMap shape_style_map[] =
 {
     {XmuShapeRectangle, XtERectangle, sizeof(XtERectangle)},
@@ -287,7 +392,7 @@ static ConversionMap shape_style_map[] =
 		strcpy((char *) toVal->addr, value);	\
 	    }							\
 	    else {						\
-		toVal->addr = (XtPointer)value;			\
+		toVal->addr = (XPointer)value;			\
 	    }							\
 	    toVal->size = size; 				\
 	    return True;					\
@@ -309,6 +414,7 @@ static ConversionMap shape_style_map[] =
 	    return True;					\
 	}
 
+
 static void
 XtDisplayConversionWarning(String type, XtPointer from, String to)
 {
@@ -322,6 +428,7 @@ XtDisplayConversionWarning(String type, XtPointer from, String to)
 		 params, &num_params);
 }
 
+
 static Boolean
 _XmNSECvtShapeStyleToString(Display *dpy, XrmValue *args, Cardinal *numArgs,
 			    XrmValue *fromVal, XrmValue *toVal,
@@ -339,10 +446,11 @@ _XmNSECvtShapeStyleToString(Display *dpy, XrmValue *args, Cardinal *numArgs,
 	}
     }
 
-    XtDisplayConversionWarning(XtRShapeStyle, (XtPointer)*i, XtRString);
+    XtDisplayConversionWarning(XtRShapeStyle, (XtPointer)(long)*i, XtRString);
     return False;
 }
 
+
 static Boolean
 _XmNSECvtBooleanToString(Display *dpy, XrmValue *args, Cardinal *numArgs,
 			 XrmValue *fromVal, XrmValue *toVal, XtPointer *data)
@@ -363,6 +471,7 @@ _XmNSECvtBooleanToString(Display *dpy, XrmValue *args, Cardinal *numArgs,
     return False;
 }
 
+
 static Boolean
 _XmNSECvtBoolToString(Display *dpy, XrmValue *args, Cardinal *numArgs,
 		      XrmValue *fromVal, XrmValue *toVal, XtPointer *data)
@@ -383,6 +492,29 @@ _XmNSECvtBoolToString(Display *dpy, XrmValue *args, Cardinal *numArgs,
     return False;
 }
 
+
+static Boolean
+_XmNSECvtAttachmentToString(Display *dpy, XrmValue *args, Cardinal *numArgs,
+			  XrmValue *fromVal, XrmValue *toVal, XtPointer *data)
+{
+    int size;
+    const char *buffer;
+
+    if (*numArgs != 0)
+    {
+	XtAppWarningMsg(XtDisplayToApplicationContext(dpy),
+		  XtNwrongParameters, "cvtattachmentToString", XtCXtToolkitError,
+		      "Attachment to String conversion needs no extra arguments",
+			(String *)NULL, (Cardinal *)NULL);
+    }
+
+    /* buffer must not be freed! */
+    buffer = _LtDebugAttachment2String(*(Position *)fromVal->addr);
+    size = strlen(buffer);
+    string_convert_done(buffer);
+}
+
+
 static Boolean
 _XmNSECvtPositionToString(Display *dpy, XrmValue *args, Cardinal *numArgs,
 			  XrmValue *fromVal, XrmValue *toVal, XtPointer *data)
@@ -403,6 +535,7 @@ _XmNSECvtPositionToString(Display *dpy, XrmValue *args, Cardinal *numArgs,
     string_convert_done(buffer);
 }
 
+
 static Boolean
 _XmNSECvtIntToString(Display *dpy, XrmValue *args, Cardinal *numArgs,
 		     XrmValue *fromVal, XrmValue *toVal, XtPointer *data)
@@ -423,6 +556,7 @@ _XmNSECvtIntToString(Display *dpy, XrmValue *args, Cardinal *numArgs,
     string_convert_done(buffer);
 }
 
+
 static Boolean
 _XmNSECvtCardinalToString(Display *dpy, XrmValue *args, Cardinal *numArgs,
 			  XrmValue *fromVal, XrmValue *toVal, XtPointer *data)
@@ -443,6 +577,7 @@ _XmNSECvtCardinalToString(Display *dpy, XrmValue *args, Cardinal *numArgs,
     string_convert_done(buffer);
 }
 
+
 static Boolean
 _XmNSECvtPixelToString(Display *dpy, XrmValue *args, Cardinal *numArgs,
 		       XrmValue *fromVal, XrmValue *toVal, XtPointer *data)
@@ -463,6 +598,7 @@ _XmNSECvtPixelToString(Display *dpy, XrmValue *args, Cardinal *numArgs,
     string_convert_done(buffer);
 }
 
+
 static Boolean
 _XmNSECvtColorToString(Display *dpy, XrmValue *args, Cardinal *numArgs,
 		       XrmValue *fromVal, XrmValue *toVal, XtPointer *data)
@@ -484,6 +620,7 @@ _XmNSECvtColorToString(Display *dpy, XrmValue *args, Cardinal *numArgs,
     string_convert_done(buffer);
 }
 
+
 static Boolean
 _XmNSECvtUnsignedCharToString(Display *dpy, XrmValue *args, Cardinal *numArgs,
 			      XrmValue *fromVal, XrmValue *toVal,
@@ -506,6 +643,7 @@ _XmNSECvtUnsignedCharToString(Display *dpy, XrmValue *args, Cardinal *numArgs,
     string_convert_done(buffer);
 }
 
+
 static Boolean
 _XmNSECvtDimensionToString(Display *dpy, XrmValue *args, Cardinal *numArgs,
 			   XrmValue *fromVal, XrmValue *toVal, XtPointer *data)
@@ -526,6 +664,7 @@ _XmNSECvtDimensionToString(Display *dpy, XrmValue *args, Cardinal *numArgs,
     string_convert_done(buffer);
 }
 
+
 static Boolean
 _XmNSECvtXmStringToString(Display *dpy, XrmValue *args, Cardinal *numArgs,
 			  XrmValue *fromVal, XrmValue *toVal, XtPointer *data)
@@ -533,7 +672,7 @@ _XmNSECvtXmStringToString(Display *dpy, XrmValue *args, Cardinal *numArgs,
     int size;
     char *s;
 
-    DEBUGOUT(XdbDebug(__FILE__, NULL, "_XmNSECvtXmStringToString\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, NULL, "_XmNSECvtXmStringToString\n"));
 
     if (*numArgs != 0)
     {
@@ -548,11 +687,40 @@ _XmNSECvtXmStringToString(Display *dpy, XrmValue *args, Cardinal *numArgs,
 	/* There's a memory leak here : nobody does free(s) */
 	size = strlen(s);
 	string_convert_done(s);
-    }
-    return False;		/* Conversion failed */
+    } 
+    else
+       return False;		/* Conversion failed */
 }
 
-/* this is needed for the string to pixmap converter */
+
+static Boolean
+_XmNSECvtWidgetToString(Display *dpy, XrmValue *args, Cardinal *numArgs,
+			  XrmValue *fromVal, XrmValue *toVal, XtPointer *data)
+{
+    int size;
+    Widget w;
+    static char *s = NULL; /* amai: a small(?) memory leak - see below */
+
+    DEBUGOUT(_LtDebugDebug(__FILE__, NULL, "_XmNSECvtWidgetToString\n"));
+
+    if (*numArgs != 0)
+    {
+	XtAppWarningMsg(XtDisplayToApplicationContext(dpy),
+		 XtNwrongParameters, "cvtWidgetToString", XtCXtToolkitError,
+		      "Widget to String conversion needs extra arguments",
+			(String *)NULL, (Cardinal *)NULL);
+    }
+
+    if (s)
+      XtFree(s);
+    w = *(Widget *)fromVal->addr;
+    s = XtNewString(XtName(w));
+    size = strlen(s);
+    string_convert_done(s);
+}
+
+
+/* this is needed for the string to pixmap converters */
 static XtConvertArgRec args[] =
 {
     {
@@ -562,29 +730,48 @@ static XtConvertArgRec args[] =
     }
 };
 
+
 /*
  * register nonstandard extensions, if any
  */
-void
+extern void
 _XmRegisterNSEConverters(void)
 {
-#if 1
     XtSetTypeConverter(XmRString,	/* source type */
 		       XmRPixmap,	/* target type */
 		       _XmNSECvtStringToPixmap,	/* converter routine */
-		       args,	/* args for converter routine */
+		       args,		/* args for converter routine */
 		       XtNumber(args),	/* number of args for converter */
 		       XtCacheNone,	/* caching instructions */
-		       NULL);	/* destructor function */
-#endif
+		       NULL);		/* destructor function */
 
+/* Danny FIX ME should I do this ? */
     XtSetTypeConverter(XmRString,	/* source type */
-		       XmRXmBackgroundPixmap,	/* target type */
-		       _XmNSECvtStringToPixmap,	/* converter routine */
-		       args,	/* args for converter routine */
+		       XmRBitmap,	/* target type */
+		       _XmNSECvtStringToBitmap,	/* converter routine */
+		       args,		/* args for converter routine */
 		       XtNumber(args),	/* number of args for converter */
 		       XtCacheNone,	/* caching instructions */
-		       NULL);	/* destructor function */
+		       NULL);		/* destructor function */
+/* End Danny */
+
+#if 0
+/* amai: see thread about DragIcon/Ishmail */
+    XtSetTypeConverter(XmRString,
+		       XmRBitmap,
+		       _XmNSECvtStringToBitmap,
+		       args,
+		       XtNumber(args),
+		       XtCacheNone,
+		       NULL);
+#endif
+    XtSetTypeConverter(XmRString,
+		       XmRXmBackgroundPixmap,
+		       _XmNSECvtStringToPixmap,
+		       args,
+		       XtNumber(args),
+		       XtCacheNone,
+		       NULL);
 
     /* Converters for reverse-direction Editres below */
     XtSetTypeConverter(XtRShapeStyle, XtRString, _XmNSECvtShapeStyleToString,
@@ -599,6 +786,8 @@ _XmRegisterNSEConverters(void)
 		       NULL, 0, XtCacheNone, NULL);
     XtSetTypeConverter(XtRBool, XtRString, _XmNSECvtBoolToString,
 		       NULL, 0, XtCacheNone, NULL);
+    XtSetTypeConverter(XmRAttachment, XtRString, _XmNSECvtAttachmentToString,
+		       NULL, 0, XtCacheNone, NULL);
     XtSetTypeConverter(XtRPosition, XtRString, _XmNSECvtPositionToString,
 		       NULL, 0, XtCacheNone, NULL);
     XtSetTypeConverter(XtRInt, XtRString, _XmNSECvtIntToString,
@@ -616,6 +805,8 @@ _XmRegisterNSEConverters(void)
 		       NULL, 0, XtCacheNone, NULL);
     XtSetTypeConverter(XmRXmString, XtRString, _XmNSECvtXmStringToString,
 		       NULL, 0, XtCacheNone, NULL);
+    XtSetTypeConverter(XmRWidget, XtRString, _XmNSECvtWidgetToString,
+		       NULL, 0, XtCacheNone, NULL);
 }
 
 #endif /* NONSTANDARD_CONVERTERS */
