@@ -1,3 +1,4 @@
+/* $Id: LTV6Editres.c,v 1.14 2001/05/14 19:46:09 amai Exp $ */
 /* $XConsortium: EditresCom.c,v 1.33 94/04/17 20:16:05 rws Exp $ */
 
 /*
@@ -32,8 +33,10 @@
  */
 
 #include <LTconfig.h>
-#include <XmI/DebugUtil.h>
-#include <X11/Xfuncproto.h>
+
+#include <stdio.h>
+#include <string.h>
+
 #include <X11/IntrinsicP.h>	/* To get into the composite and core widget
 				   structures. */
 #include <X11/ObjectP.h>	/* For XtIs<Classname> macros. */
@@ -41,12 +44,30 @@
 #include <X11/ShellP.h>		/* for Application Shell Widget class. */
 
 #include <X11/Xatom.h>
-#include <X11/Xos.h>		/* for strcpy declaration */
-#include <XmI/LTV6EditresP.h>
 #include <X11/Xmd.h>
+#include <X11/keysym.h>
 #include <Xm/XmP.h>
+#include <XmI/XmI.h>
+#include <XmI/LTV6EditresP.h>
+
+#include <XmI/DebugUtil.h>
+
+
+/* local prototypes */
+static void _LesstifEditResPutString8(ProtocolStream *stream, char *str);
+static void _LesstifEditResPut8(ProtocolStream *stream, unsigned int value);
+static void _LesstifEditResPut16(ProtocolStream *stream, unsigned int value);
+static void _LesstifEditResPut32(ProtocolStream *stream, unsigned long value);
+static void _LesstifEditResPutWidgetInfo(ProtocolStream *stream, WidgetInfo *info);
+static void _LesstifEditResResetStream(ProtocolStream *stream);
+
+static Boolean _LesstifEditResGet8(ProtocolStream *stream, unsigned char *val);
+static Boolean _LesstifEditResGet16(ProtocolStream *stream, unsigned short *val);
+static Boolean _LesstifEditResGetSigned16(ProtocolStream *stream, short *val);
+static Boolean _LesstifEditResGet32(ProtocolStream *stream, unsigned long *val);
+static Boolean _LesstifEditResGetString8(ProtocolStream *stream, char **str);
+static Boolean _LesstifEditResGetWidgetInfo(ProtocolStream *stream, WidgetInfo *info);
 
-#include <stdio.h>
 
 #define _LesstifEditResPutBool _LesstifEditResPut8
 #define _LesstifEditResPutResourceType _LesstifEditResPut8
@@ -204,7 +225,6 @@ XInternAtoms(Display *dpy, char **names, int count, Bool ifExists, Atom *aret)
 /*
  * avoid linking with Xmu
  */
-#include <X11/keysym.h>
 
 static void
 XmuCopyISOLatin1Lowered(char *dst, XmConst char *src)
@@ -227,12 +247,14 @@ XmuCopyISOLatin1Lowered(char *dst, XmConst char *src)
     *dest = '\0';
 }
 
+
 /************************************************************
  *
  * Resource Editor Communication Code
  *
  ************************************************************/
 
+
 /*    Function Name: _XmNSEEditResCheckMessages (_XmNSEEditResCheckMessages)
  *    Description: This callback routine is set on all shell widgets,
  *                   and checks to see if a client message event
@@ -244,10 +266,11 @@ XmuCopyISOLatin1Lowered(char *dst, XmConst char *src)
  *    Returns: none.
  */
 
-/* ARGSUSED */
-void
-_XmNSEEditResCheckMessages(Widget w, XtPointer data, XEvent *event,
-			   Boolean *cont)
+extern void
+_XmNSEEditResCheckMessages(Widget w,
+                           XtPointer data, 
+                           XEvent *event,
+                           Boolean *cont)
 {
     Time time;
     ResIdent ident;
@@ -298,6 +321,7 @@ _XmNSEEditResCheckMessages(Widget w, XtPointer data, XEvent *event,
     }
 }
 
+
 /*    Function Name: BuildEvent
  *    Description: Takes the info out the protocol stream an constructs
  *                   the proper event structure.
@@ -311,6 +335,7 @@ _XmNSEEditResCheckMessages(Widget w, XtPointer data, XEvent *event,
 
 #define ERROR_MESSAGE ("Client: Improperly formatted protocol request")
 
+
 static EditresEvent *
 BuildEvent(Widget w, Atom sel, XtPointer data, ResIdent ident,
 	   unsigned long length)
@@ -465,6 +490,7 @@ BuildEvent(Widget w, Atom sel, XtPointer data, ResIdent ident,
     return (NULL);
 }
 
+
 /*      Function Name: FreeEvent
  *    Description: Frees the event structure and any other pieces
  *                   in it that need freeing.
@@ -490,6 +516,7 @@ FreeEvent(EditresEvent *event)
     XtFree((char *)event);
 }
 
+
 /*      Function Name: GetCommand
  *    Description: Gets the Command out of the selection asserted by the
  *                   resource manager.
@@ -498,7 +525,6 @@ FreeEvent(EditresEvent *event)
  *      Returns: none.
  */
 
-/* ARGSUSED */
 static void
 GetCommand(Widget w, XtPointer data, Atom *selection, Atom *type,
 	   XtPointer value, unsigned long *length, int *format)
@@ -516,6 +542,7 @@ GetCommand(Widget w, XtPointer data, Atom *selection, Atom *type,
     }
 }
 
+
 /*    Function Name: ExecuteCommand
  *    Description: Executes a command string received from the 
  *                   resource editor.
@@ -527,7 +554,6 @@ GetCommand(Widget w, XtPointer data, Atom *selection, Atom *type,
  * NOTES:  munges str
  */
 
-/* ARGSUSED */
 static void
 ExecuteCommand(Widget w, Atom sel, ResIdent ident, EditresEvent * event)
 {
@@ -590,6 +616,7 @@ ExecuteCommand(Widget w, Atom sel, ResIdent ident, EditresEvent * event)
     }
 }
 
+
 /*    Function Name: ConvertReturnCommand
  *    Description: Converts a selection.
  *    Arguments: w - the widget that owns the selection.
@@ -602,7 +629,6 @@ ExecuteCommand(Widget w, Atom sel, ResIdent ident, EditresEvent * event)
  *    Returns: True if conversion was sucessful.
  */
 
-/* ARGSUSED */
 static Boolean
 ConvertReturnCommand(Widget w, Atom *selection, Atom *target, Atom *type_ret,
 		     XtPointer *value_ret, unsigned long *length_ret,
@@ -623,19 +649,20 @@ ConvertReturnCommand(Widget w, Atom *selection, Atom *target, Atom *type_ret,
     return (True);
 }
 
+
 /*      Function Name: CommandDone
  *    Description: done with the selection.
  *      Arguments: *** UNUSED ***
  *      Returns: none.
  */
 
-/* ARGSUSED */
 static void
 CommandDone(Widget widget, Atom *selection, Atom *target)
 {
     /* Keep the toolkit from automaticaly freeing the selection value */
 }
 
+
 /*    Function Name: SendFailure
  *    Description: Sends a failure message.
  *    Arguments: w - the widget to own the selection.
@@ -697,6 +724,7 @@ BuildReturnPacket(ResIdent ident, EditresCommand command,
     return ((XtPointer)stream->real_top);
 }
 
+
 /*      Function Name: SendCommand
  *    Description: Builds a return command line.
  *      Arguments: w - the widget to own the selection.
@@ -724,6 +752,7 @@ SendCommand(Widget w, Atom sel, ResIdent ident, EditresCommand command,
 		   ConvertReturnCommand, NULL, CommandDone);
 }
 
+
 /************************************************************
  *
  * Generic Utility Functions.
@@ -744,7 +773,7 @@ static int
 FindChildren(Widget parent, Widget **children, Boolean normal, Boolean popup)
 {
     CompositeWidget cw = (CompositeWidget)parent;
-    int i, num_children, current = 0;
+    Cardinal i, num_children, current = 0;
 
     num_children = 0;
 
@@ -773,6 +802,7 @@ FindChildren(Widget parent, Widget **children, Boolean normal, Boolean popup)
     return (num_children);
 }
 
+
 /*      Function Name: IsChild
  *    Description: check to see of child is a child of parent.
  *      Arguments: top - the top of the tree.
@@ -805,6 +835,7 @@ IsChild(Widget top, Widget parent, Widget child)
     return (False);
 }
 
+
 /*      Function Name: VerifyWidget
  *    Description: Makes sure all the widgets still exist.
  *      Arguments: w - any widget in the tree.
@@ -844,6 +875,7 @@ VerifyWidget(Widget w, WidgetInfo *info)
     return (NULL);
 }
 
+
 /************************************************************
  *
  * Code to Perform SetValues operations.
@@ -851,6 +883,7 @@ VerifyWidget(Widget w, WidgetInfo *info)
  ************************************************************/
 
 
+
 /*      Function Name:  DoSetValues
  *    Description: performs the setvalues requested.
  *      Arguments: w - a widget in the tree.
@@ -893,6 +926,7 @@ DoSetValues(Widget w, EditresEvent *event, ProtocolStream *stream)
     return (NULL);
 }
 
+
 /*    Function Name: HandleToolkitErrors
  *    Description: Handles X Toolkit Errors.
  *    Arguments: name - name of the error.
@@ -903,7 +937,6 @@ DoSetValues(Widget w, EditresEvent *event, ProtocolStream *stream)
  *    Returns: none.
  */
 
-/* ARGSUSED */
 static void
 HandleToolkitErrors(String name, String type, String class, String msg,
 		    String *params, Cardinal *num_params)
@@ -918,7 +951,7 @@ HandleToolkitErrors(String name, String type, String class, String msg,
 	sprintf(buf, msg, params[0]);
     else if (streq(name, "conversionFailed") || streq(name, "conversionError"))
     {
-	if (streq(info->event->value, XtRString))
+	if (streq((char *)info->event->value, XtRString))
 	    sprintf(buf,
 		    "Could not convert the string '%s' for the `%s' resource.",
 		    (char *)info->event->value, info->event->name);
@@ -939,6 +972,7 @@ HandleToolkitErrors(String name, String type, String class, String msg,
     _LesstifEditResPutString8(info->stream, buf);
 }
 
+
 /*      Function Name: ExecuteSetValues
  *    Description: Performs a setvalues for a given command.
  *      Arguments: w - the widget to perform the set_values on.
@@ -988,7 +1022,7 @@ ExecuteSetValues(Widget w, SetValuesEvent *sv_event, WidgetInfo *entry,
 
 #define TOOLKIT_TYPE ("Xt")
 
-/* ARGSUSED */
+
 static char *
 DumpWidgets(Widget w, EditresEvent *event, ProtocolStream *stream)
 {
@@ -1022,6 +1056,7 @@ DumpWidgets(Widget w, EditresEvent *event, ProtocolStream *stream)
     return (NULL);
 }
 
+
 /*    Function Name: DumpChildren
  *    Description: Adds a child's name to the list.
  *    Arguments: w - the widget to dump.
@@ -1053,6 +1088,7 @@ isApplicationShell(Widget w)
     return False;
 }
 
+
 static void
 DumpChildren(Widget w, ProtocolStream *stream, unsigned short *count)
 {
@@ -1097,6 +1133,7 @@ DumpChildren(Widget w, ProtocolStream *stream, unsigned short *count)
     XtFree((char *)children);
 }
 
+
 /************************************************************
  *
  * Code for getting the geometry of widgets.
@@ -1140,6 +1177,7 @@ DoGetGeometry(Widget w, EditresEvent *event, ProtocolStream *stream)
     return (NULL);
 }
 
+
 /*    Function Name: ExecuteGetGeometry
  *    Description: Gets the geometry for each widget specified.
  *    Arguments: w - the widget to get geom on.
@@ -1213,6 +1251,7 @@ ExecuteGetGeometry(Widget w, ProtocolStream *stream)
     _LesstifEditResPut16(stream, border_width);
 }
 
+
 /************************************************************
  *
  * Code for executing FindChild.
@@ -1278,6 +1317,7 @@ PositionInChild(Widget child, int x, int y)
 	(y <= (child_y + (Position)height + 2 * (Position)border_width));
 }
 
+
 /*      Function Name: _FindChild
  *    Description: Finds the child that actually contatians the point shown.
  *      Arguments: parent - a widget that is known to contain the point
@@ -1310,6 +1350,7 @@ _FindChild(Widget parent, int x, int y)
     return (parent);
 }
 
+
 /*    Function Name: DoFindChild
  *    Description: finds the child that contains the location specified.
  *    Arguments: w - a widget in the tree.
@@ -1342,6 +1383,7 @@ DoFindChild(Widget w, EditresEvent *event, ProtocolStream *stream)
     return (NULL);
 }
 
+
 /************************************************************
  *
  * Procedures for performing GetResources.
@@ -1388,6 +1430,7 @@ DoGetResources(Widget w, EditresEvent *event, ProtocolStream *stream)
     return (NULL);
 }
 
+
 /*    Function Name: ExecuteGetResources.
  *    Description: Gets the resources for any individual widget.
  *    Arguments: w - the widget to get resources on.
@@ -1446,6 +1489,7 @@ ExecuteGetResources(Widget w, ProtocolStream *stream)
     }
 }
 
+
 /*
  *    Function Name: DumpValues
  *    Description: Returns resource values to the resource editor.
@@ -1454,7 +1498,6 @@ ExecuteGetResources(Widget w, ProtocolStream *stream)
  *    Returns: NULL
  */
 
-/* ARGSUSED */
 static char *
 DumpValues(Widget w, EditresEvent *event, ProtocolStream *stream)
 {
@@ -1490,6 +1533,7 @@ DumpValues(Widget w, EditresEvent *event, ProtocolStream *stream)
     return (NULL);
 }
 
+
 /************************************************************
  *
  * Code for inserting values into the protocol stream.
@@ -1534,6 +1578,7 @@ InsertWidget(ProtocolStream * stream, Widget w)
     XtFree((char *)widget_list);
 }
 
+
 /************************************************************
  *
  * All of the following routines are public.
@@ -1547,7 +1592,7 @@ InsertWidget(ProtocolStream * stream, Widget w)
  *    Returns: none.
  */
 
-void
+static void
 _LesstifEditResPutString8(ProtocolStream *stream, char *str)
 {
     int i, len = strlen(str);
@@ -1557,6 +1602,7 @@ _LesstifEditResPutString8(ProtocolStream *stream, char *str)
 	_LesstifEditResPut8(stream, *str);
 }
 
+
 /*    Function Name: _LesstifEditResPut8
  *    Description: Inserts an 8 bit integer into the protocol stream.
  *    Arguments: stream - stream to insert string into.
@@ -1564,7 +1610,7 @@ _LesstifEditResPutString8(ProtocolStream *stream, char *str)
  *    Returns: none
  */
 
-void
+static void
 _LesstifEditResPut8(ProtocolStream *stream, unsigned int value)
 {
     unsigned char temp;
@@ -1591,13 +1637,14 @@ _LesstifEditResPut8(ProtocolStream *stream, unsigned int value)
  *    Returns: void
  */
 
-void
+static void
 _LesstifEditResPut16(ProtocolStream *stream, unsigned int value)
 {
     _LesstifEditResPut8(stream, (value >> XER_NBBY) & BYTE_MASK);
     _LesstifEditResPut8(stream, value & BYTE_MASK);
 }
 
+
 /*    Function Name: _LesstifEditResPut32
  *    Description: Inserts a 32 bit integer into the protocol stream.
  *    Arguments: stream - stream to insert string into.
@@ -1605,7 +1652,7 @@ _LesstifEditResPut16(ProtocolStream *stream, unsigned int value)
  *    Returns: void
  */
 
-void
+static void
 _LesstifEditResPut32(ProtocolStream *stream, unsigned long value)
 {
     int i;
@@ -1614,6 +1661,7 @@ _LesstifEditResPut32(ProtocolStream *stream, unsigned long value)
 	_LesstifEditResPut8(stream, (value >> (XER_NBBY * i)) & BYTE_MASK);
 }
 
+
 /*    Function Name: _LesstifEditResPutWidgetInfo
  *    Description: Inserts the widget info into the protocol stream.
  *    Arguments: stream - stream to insert widget info into.
@@ -1621,7 +1669,7 @@ _LesstifEditResPut32(ProtocolStream *stream, unsigned long value)
  *    Returns: none
  */
 
-void
+static void
 _LesstifEditResPutWidgetInfo(ProtocolStream *stream, WidgetInfo *info)
 {
     unsigned int i;
@@ -1631,19 +1679,21 @@ _LesstifEditResPutWidgetInfo(ProtocolStream *stream, WidgetInfo *info)
 	_LesstifEditResPut32(stream, info->ids[i]);
 }
 
+
 /************************************************************
  *
  * Code for retrieving values from the protocol stream.
  *
  ************************************************************/
 
+
 /*    Function Name: _LesstifEditResResetStream
  *    Description: resets the protocol stream
  *    Arguments: stream - the stream to reset.
  *    Returns: none.
  */
 
-void
+static void
 _LesstifEditResResetStream(ProtocolStream *stream)
 {
     stream->current = stream->top;
@@ -1658,6 +1708,7 @@ _LesstifEditResResetStream(ProtocolStream *stream)
     }
 }
 
+
 /*
  * NOTE NOTE NOTE NOTE NOTE NOTE NOTE NOTE NOTE NOTE NOTE 
  *
@@ -1675,7 +1726,7 @@ _LesstifEditResResetStream(ProtocolStream *stream)
  *    Returns: True if sucessful.
  */
 
-Boolean
+static Boolean
 _LesstifEditResGet8(ProtocolStream *stream, unsigned char *val)
 {
     if (stream->size < (stream->current - stream->top))
@@ -1685,6 +1736,7 @@ _LesstifEditResGet8(ProtocolStream *stream, unsigned char *val)
     return (True);
 }
 
+
 /*    Function Name: _LesstifEditResGet16
  *    Description: Retrieves an unsigned 16 bit value
  *                 from the protocol stream.
@@ -1693,7 +1745,7 @@ _LesstifEditResGet8(ProtocolStream *stream, unsigned char *val)
  *    Returns: True if sucessful.
  */
 
-Boolean
+static Boolean
 _LesstifEditResGet16(ProtocolStream *stream, unsigned short *val)
 {
     unsigned char temp1, temp2;
@@ -1706,6 +1758,7 @@ _LesstifEditResGet16(ProtocolStream *stream, unsigned short *val)
     return (True);
 }
 
+
 /*      Function Name: _LesstifEditResGetSigned16
  *    Description: Retrieves an signed 16 bit value from the protocol stream.
  *      Arguments: stream.
@@ -1713,7 +1766,7 @@ _LesstifEditResGet16(ProtocolStream *stream, unsigned short *val)
  *      Returns: True if sucessful.
  */
 
-Boolean
+static Boolean
 _LesstifEditResGetSigned16(ProtocolStream *stream, short *val)
 {
     unsigned char temp1, temp2;
@@ -1734,6 +1787,7 @@ _LesstifEditResGetSigned16(ProtocolStream *stream, short *val)
     return (True);
 }
 
+
 /*    Function Name: _LesstifEditResGet32
  *    Description: Retrieves an unsigned 32 bit value
  *                 from the protocol stream.
@@ -1742,7 +1796,7 @@ _LesstifEditResGetSigned16(ProtocolStream *stream, short *val)
  *    Returns: True if sucessful.
  */
 
-Boolean
+static Boolean
 _LesstifEditResGet32(ProtocolStream *stream, unsigned long *val)
 {
     unsigned short temp1, temp2;
@@ -1756,6 +1810,7 @@ _LesstifEditResGet32(ProtocolStream *stream, unsigned long *val)
     return (True);
 }
 
+
 /*    Function Name: _LesstifEditResGetString8
  *    Description: Retrieves an 8 bit string value from the protocol stream.
  *    Arguments: stream - the protocol stream
@@ -1763,7 +1818,7 @@ _LesstifEditResGet32(ProtocolStream *stream, unsigned long *val)
  *    Returns: True if retrieval was successful.
  */
 
-Boolean
+static Boolean
 _LesstifEditResGetString8(ProtocolStream *stream, char **str)
 {
     unsigned short len;
@@ -1797,7 +1852,7 @@ _LesstifEditResGetString8(ProtocolStream *stream, char **str)
  *    Returns: True if retrieval was successful.
  */
 
-Boolean
+static Boolean
 _LesstifEditResGetWidgetInfo(ProtocolStream *stream, WidgetInfo *info)
 {
     unsigned int i;
@@ -1837,7 +1892,7 @@ _LesstifEditResGetWidgetInfo(ProtocolStream *stream, WidgetInfo *info)
  *    Returns: True if conversion was sucessful.
  */
 
-/* ARGSUSED */
+
 static Boolean
 CvtStringToBlock(Display *dpy, XrmValue *args, Cardinal *num_args,
 		 XrmValue *from_val, XrmValue *to_val,
@@ -1913,6 +1968,7 @@ LoadResources(Widget w)
 			      XtNumber(resources), NULL, (Cardinal)0);
 }
 
+
 static void
 _LesstifEditResCvtWarningHandler(String name,
 				 String type,
@@ -1925,6 +1981,7 @@ _LesstifEditResCvtWarningHandler(String name,
     return;
 }
 
+
 static void
 _LesstifEditResGetStringValues(Widget w, Arg *warg, int numargs)
 {
@@ -2012,7 +2069,7 @@ _LesstifEditResGetStringValues(Widget w, Arg *warg, int numargs)
     }
 
 
-    DEBUGOUT(XdbDebug(__FILE__, w,
+    DEBUGOUT(_LtDebugDebug(__FILE__, w,
 		      "_LesstifEditResGetStringValues: converting from %s\n",
 		      res->resource_type));
 
