@@ -1,10 +1,11 @@
 /**
- * $Id: LTglob.c,v 1.9 2000/01/09 13:02:12 amai Exp $
+ * $Id: LTglob.c,v 1.26 2001/05/14 19:46:09 amai Exp $
  *
  * derived from GNU libc glob.c
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  * Copyright (C) 1991, 1992, 1993 Free Software Foundation, Inc.
+ * Copyright (C) 1995-2001 LessTif Development Team
  *
  * This file is part of the GNU LessTif Library.
  *
@@ -24,27 +25,23 @@
  *
  **/
 
-static const char rcsid[] = "$Id: LTglob.c,v 1.9 2000/01/09 13:02:12 amai Exp $";
-
-#if defined (_AIX) && !defined (__GNUC__)
-#pragma alloca
-#endif
+static const char rcsid[] = "$Id: LTglob.c,v 1.26 2001/05/14 19:46:09 amai Exp $";
 
 #include <LTconfig.h>
-#include <X11/Intrinsic.h>
-#include <X11/Xfuncs.h>
 
+#include <stdlib.h>
+#include <string.h>
+#include <ctype.h>
 #include <errno.h>
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <stdio.h>
-
-
-#ifdef	STDC_HEADERS
 #include <stddef.h>
+#ifdef HAVE_SYS_TYPES_H
+#include <sys/types.h>
+#endif
+#ifdef HAVE_SYS_STAT_H
+#include <sys/stat.h>
 #endif
 
-#ifdef	HAVE_UNISTD_H
+#ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #ifndef POSIX
 #ifdef	_POSIX_VERSION
@@ -53,14 +50,7 @@ static const char rcsid[] = "$Id: LTglob.c,v 1.9 2000/01/09 13:02:12 amai Exp $"
 #endif
 #endif
 
-#if !defined(__GNU_LIBRARY__) && !defined(STDC_HEADERS)
-extern int errno;
-#endif
-
-#ifndef	NULL
-#define	NULL	0
-#endif
-
+#include <X11/Intrinsic.h>  /* for Xt*alloc() */
 
 #if	defined (POSIX) || defined (HAVE_DIRENT_H) || defined (__GNU_LIBRARY__)
 #include <dirent.h>
@@ -91,101 +81,24 @@ extern int errno;
 #define REAL_DIR_ENTRY(dp) (dp->d_ino != 0)
 #endif /* POSIX */
 
-#if	(defined (STDC_HEADERS) || defined (__GNU_LIBRARY__))
-#include <stdlib.h>
-#include <string.h>
-#define	ANSI_STRING
-#else /* No standard headers.  */
+#include "LTglob.h"
 
-#ifdef HAVE_STRING_H
-#include <string.h>
-#define	ANSI_STRING
-#else
-#include <strings.h>
-#endif
-#ifdef	HAVE_MEMORY_H
-#include <memory.h>
-#endif
-
-extern void qsort();
-
-#endif /* Standard headers.  */
-
-#ifdef __EMX__   /* -> OS/2 drive letters support */
-#define  INCL_DOS
-#define  INCL_DOSMISC
-#define  INCL_ERRORS
-#include <os2.h>
-#endif
-
-#ifndef	HAVE_STRCOLL
-#define	strcoll	strcmp
-#endif
+#include <XmI/DebugUtil.h>
 
 
-#if 0
-#ifndef	__GNU_LIBRARY__
-#ifdef	__GNUC__
-__inline
-#endif
-static void *
-my_realloc(void *p,
-	   unsigned int n)
-{
-    /* These casts are the for sake of the broken Ultrix compiler,
-       which warns of illegal pointer combinations otherwise.  */
-    if (p == NULL)
-	return (char *)XtMalloc(n);
-    return (char *)XtRealloc(p, n);
-}
-#define	realloc	my_realloc
-#endif
-#endif
-
-
-#if !defined(__alloca)
-#ifdef	__GNUC__
-#undef	alloca
-#define	alloca(n)	__builtin_alloca (n)
-#else /* Not GCC.  */
-#if	defined (sparc) || defined (HAVE_ALLOCA_H)
-#include <alloca.h>
-#else /* Not sparc or HAVE_ALLOCA_H.  */
-#ifndef	_AIX
-extern char *alloca(size_t size_to_allocate);
-#endif /* Not _AIX.  */
-#endif /* sparc or HAVE_ALLOCA_H.  */
-#endif /* GCC.  */
-
-#define	__alloca	alloca
-
-#endif
-
-#if 0
-/* rws 5 Nov 1998
-   What is all this for??  IMO a link to a directory is still a directory.
+/*
+   rws 5 Nov 1998
+   IMO a link to a directory is still a directory.
    We shouldn't care whether it is a link or not, therefore I think stat is
    the correct call here, whether we have lstat or not.  If I'm wrong, write
    a test and prove it by doing a FileSelectionSearch on a directory with
    links to directories and comparing the results.
- */
 
-#ifndef __GNU_LIBRARY__
-#  define __lstat lstat
-#  ifndef HAVE_LSTAT
-#    define lstat stat
-#  endif
-#  ifdef STAT_MACROS_BROKEN
-#    undef S_ISDIR
-#  endif
-#  ifndef S_ISDIR
-#    define S_ISDIR(mode) (((mode) & S_IFMT) == S_IFDIR)
-#  endif
-#endif
+#  define LTstat lstat
+*/
 
-#else
+#define LTstat stat
 
-#define __lstat stat
 #ifdef STAT_MACROS_BROKEN
 #  undef S_ISDIR
 #endif
@@ -193,43 +106,190 @@ extern char *alloca(size_t size_to_allocate);
 #  define S_ISDIR(mode) (((mode) & S_IFMT) == S_IFDIR)
 #endif
 
-#endif
 
-#ifndef	STDC_HEADERS
-#undef	size_t
-#define	size_t	unsigned int
-#endif
+#if 1
+/* amai: We don't use external hooks. If you want to do so -
+   also change <LTglob.h> */
+static void *(*__glob_opendir_hook) (const char *directory)=NULL;
+static const char *(*__glob_readdir_hook) (void * stream)=NULL;
+static void (*__glob_closedir_hook) (void * stream)=NULL;
+#else
+void *(*__glob_opendir_hook) (const char *directory);
+const char *(*__glob_readdir_hook) (void * stream);
+void (*__glob_closedir_hook) (void * stream);
+#endif /* */
 
-/* Some system header files erroneously define these.
-   We want our own definitions from <fnmatch.h> to take precedence.  */
-#undef	FNM_PATHNAME
-#undef	FNM_NOESCAPE
-#undef	FNM_PERIOD
-#include <LTfnmatch.h>
+/* just for convenience: */
+typedef int (* errfunc_t)(const char *, int);
+
+static int glob_pattern_p (const char *pattern, int quote);
+static int glob_in_dir (const char *pattern, const char *directory,
+                        int flags, errfunc_t errfunc, glob_t *pglob);
+static int prefix_array (const char *prefix, char **array, size_t n);
+static int collated_compare (const void *, const void *);
+
+
+
+
+/* Match STRING against the filename pattern PATTERN, returning zero if
+   it matches, nonzero if not.
+   Derived from GNU libc fnmatch.c  */
+static int
+_Lesstif_fnmatch(const char *pattern,
+		 const char *string,
+		 int flags)
+{
+    register const char *p = pattern, *n = string;
+    register char c;
+
+/* Note that this evalutes C many times.  */
+#define FOLD(c)	((flags & FNM_CASEFOLD) && isupper (c) ? tolower (c) : (c))
+
+    while ((c = *p++) != '\0')
+    {
+	c = FOLD(c);
+
+	switch (c)
+	{
+	case '?':
+	    if (*n == '\0')
+		return FNM_NOMATCH;
+	    else if ((flags & FNM_FILE_NAME) && *n == '/')
+		return FNM_NOMATCH;
+	    else if ((flags & FNM_PERIOD) && *n == '.' &&
+		     (n == string || ((flags & FNM_FILE_NAME) && n[-1] == '/')))
+		return FNM_NOMATCH;
+	    break;
+
+	case '\\':
+	    if (!(flags & FNM_NOESCAPE))
+	    {
+		c = *p++;
+		c = FOLD(c);
+	    }
+	    if (FOLD(*n) != c)
+		return FNM_NOMATCH;
+	    break;
+
+	case '*':
+	    if ((flags & FNM_PERIOD) && *n == '.' &&
+		(n == string || ((flags & FNM_FILE_NAME) && n[-1] == '/')))
+		return FNM_NOMATCH;
+
+	    for (c = *p++; c == '?' || c == '*'; c = *p++, ++n)
+		if (((flags & FNM_FILE_NAME) && *n == '/') ||
+		    (c == '?' && *n == '\0'))
+		    return FNM_NOMATCH;
+
+	    if (c == '\0')
+		return 0;
+
+	    {
+		char c1 = (!(flags & FNM_NOESCAPE) && c == '\\') ? *p : c;
+		c1 = FOLD(c1);
+		for (--p; *n != '\0'; ++n)
+		    if ((c == '[' || FOLD(*n) == c1) &&
+			_Lesstif_fnmatch(p, n, flags & ~FNM_PERIOD) == 0)
+			return 0;
+		return FNM_NOMATCH;
+	    }
+
+	case '[':
+	    {
+		/* Nonzero if the sense of the character class is inverted.  */
+		register int c_not;
+
+		if (*n == '\0')
+		    return FNM_NOMATCH;
+
+		if ((flags & FNM_PERIOD) && *n == '.' &&
+		    (n == string || ((flags & FNM_FILE_NAME) && n[-1] == '/')))
+		    return FNM_NOMATCH;
+
+		c_not = (*p == '!' || *p == '^');
+		if (c_not)
+		    ++p;
+
+		c = *p++;
+		for (;;)
+		{
+		    register char cstart = c, cend = c;
+
+		    if (!(flags & FNM_NOESCAPE) && c == '\\')
+			cstart = cend = *p++;
+
+		    cstart = cend = FOLD(cstart);
+
+		    if (c == '\0')
+			/* [ (unterminated) loses.  */
+			return FNM_NOMATCH;
+
+		    c = *p++;
+		    c = FOLD(c);
+
+		    if ((flags & FNM_FILE_NAME) && c == '/')
+			/* [/] can never match.  */
+			return FNM_NOMATCH;
+
+		    if (c == '-' && *p != ']')
+		    {
+			cend = *p++;
+			if (!(flags & FNM_NOESCAPE) && cend == '\\')
+			    cend = *p++;
+			if (cend == '\0')
+			    return FNM_NOMATCH;
+			cend = FOLD(cend);
+
+			c = *p++;
+		    }
+
+		    if (FOLD(*n) >= cstart && FOLD(*n) <= cend)
+			goto matched;
+
+		    if (c == ']')
+			break;
+		}
+		if (!c_not)
+		    return FNM_NOMATCH;
+		break;
+
+	      matched:;
+		/* Skip the rest of the [...] that already matched.  */
+		while (c != ']')
+		{
+		    if (c == '\0')
+			/* [... (unterminated) loses.  */
+			return FNM_NOMATCH;
+
+		    c = *p++;
+		    if (!(flags & FNM_NOESCAPE) && c == '\\')
+			/* XXX 1003.2d11 is unclear if this is right.  */
+			++p;
+		}
+		if (c_not)
+		    return FNM_NOMATCH;
+	    }
+	    break;
+
+	default:
+	    if (c != FOLD(*n))
+		return FNM_NOMATCH;
+	}
+
+	++n;
+    }
+
+    if (*n == '\0')
+	return 0;
+
+    if ((flags & FNM_LEADING_DIR) && *n == '/')
+	/* The FNM_LEADING_DIR flag says that "foo*" matches "foobar/frobozz".  */
+	return 0;
+
+    return FNM_NOMATCH;
+} /* _Lesstif_fnmatch() */
 
-/* Some system header files erroneously define these.
-   We want our own definitions from <glob.h> to take precedence.  */
-#undef	GLOB_ERR
-#undef	GLOB_MARK
-#undef	GLOB_NOSORT
-#undef	GLOB_DOOFFS
-#undef	GLOB_NOCHECK
-#undef	GLOB_APPEND
-#undef	GLOB_NOESCAPE
-#undef	GLOB_PERIOD
-#include <LTglob.h>
-
-__ptr_t(*__glob_opendir_hook) __P((const char *directory));
-     const char *(*__glob_readdir_hook) __P((__ptr_t stream));
-     void (*__glob_closedir_hook) __P((__ptr_t stream));
 
-     static int glob_pattern_p __P((const char *pattern, int quote));
-     static int glob_in_dir __P((const char *pattern, const char *directory,
-				 int flags,
-				 int (*errfunc) __P((const char *, int)),
-				 glob_t *pglob));
-     static int prefix_array __P((const char *prefix, char **array, size_t n));
-     static int collated_compare __P((const __ptr_t, const __ptr_t));
 
 /* Do glob searching for PATTERN, placing results in PGLOB.
    The bits defined above may be set in FLAGS.
@@ -241,12 +301,11 @@ __ptr_t(*__glob_opendir_hook) __P((const char *directory));
    Otherwise, `glob' returns zero.
  */
 int
- _Lesstif_glob(const char *pattern, int flags,
-	       int (*errfunc) __P((const char *, int)),
-	       glob_t *pglob)
+_Lesstif_glob(const char *pattern, int flags,
+	      errfunc_t errfunc, glob_t *pglob)
 {
-    const char *filename;
     char *dirname;
+    const char *filename;
     size_t dirlen;
     int status;
     int oldcount;
@@ -257,26 +316,31 @@ int
 	return -1;
     }
 
+    DEBUGOUT(_LtDebugDebug(__FILE__, NULL,
+		      "_Lesstif_glob(%s, %i)\n", pattern, flags));
+
     /* Find the filename.  */
     filename = strrchr(pattern, '/');
     if (filename == NULL)
     {
+        dirname=XtMalloc(2);
 	filename = pattern;
-	dirname = (char *)".";
+	strcpy(dirname, ".");
 	dirlen = 0;
     }
     else if (filename == pattern)
     {
 	/* "/pattern".  */
-	dirname = (char *)"/";
+        dirname=XtMalloc(2);
+	strcpy(dirname, "/");
 	dirlen = 1;
 	++filename;
     }
     else
     {
 	dirlen = filename - pattern;
-	dirname = (char *)__alloca(dirlen + 1);
-	bcopy(pattern, dirname, dirlen);
+	dirname = XtMalloc(dirlen + 1);
+	memcpy(dirname, pattern, dirlen);
 	dirname[dirlen] = '\0';
 	++filename;
     }
@@ -288,6 +352,7 @@ int
 	if (val == 0)
 	    pglob->gl_flags = (pglob->gl_flags & ~GLOB_MARK) |
 		(flags & GLOB_MARK);
+	XtFree(dirname);
 	return val;
     }
 
@@ -307,12 +372,16 @@ int
 	glob_t dirs;
 	register int i;
 
+        DEBUGOUT(_LtDebugDebug(__FILE__, NULL,
+		      "glob_pattern(%s, !(flags & GLOB_NOESCAPE))!=NULL\n", dirname));
 	status = _Lesstif_glob(dirname,
 			 ((flags & (GLOB_ERR | GLOB_NOCHECK | GLOB_NOESCAPE)) |
 			  GLOB_NOSORT),
-			       errfunc, &dirs);
-	if (status != 0)
+			  errfunc, &dirs);
+	if (status != 0) {
+     	    XtFree(dirname);
 	    return status;
+        }
 
 	/* We have successfully globbed the preceding directory name.
 	   For each name we found, call glob_in_dir on it and FILENAME,
@@ -321,20 +390,6 @@ int
 	{
 	    int oldcount;
 
-#ifdef	SHELL
-	    {
-		/* Make globbing interruptible in the bash shell. */
-		extern int interrupt_state;
-
-		if (interrupt_state)
-		{
-		    _Lesstif_globfree(&dirs);
-		    _Lesstif_globfree(&files);
-		    return GLOB_ABEND;
-		}
-	    }
-#endif /* SHELL.  */
-
 	    oldcount = pglob->gl_pathc;
 	    status = glob_in_dir(filename, dirs.gl_pathv[i],
 				 (flags | GLOB_APPEND) & ~GLOB_NOCHECK,
@@ -347,6 +402,7 @@ int
 	    {
 		_Lesstif_globfree(&dirs);
 		_Lesstif_globfree(pglob);
+		XtFree(dirname);
 		return status;
 	    }
 
@@ -357,6 +413,7 @@ int
 	    {
 		_Lesstif_globfree(&dirs);
 		_Lesstif_globfree(pglob);
+		XtFree(dirname);
 		return GLOB_NOSPACE;
 	    }
 	}
@@ -370,9 +427,11 @@ int
 	    {
 		size_t len = strlen(pattern) + 1;
 		char *patcopy = (char *)XtMalloc(len);
-		if (patcopy == NULL)
+		if (patcopy == NULL) {
+		    XtFree(dirname);
 		    return GLOB_NOSPACE;
-		bcopy(pattern, patcopy, len);
+		}
+		memcpy(patcopy, pattern, len);
 
 		pglob->gl_pathv =
 		    (char **)XtRealloc((char *)pglob->gl_pathv,
@@ -384,6 +443,7 @@ int
 		if (pglob->gl_pathv == NULL)
 		{
 		    XtFree(patcopy);
+		    XtFree(dirname);
 		    return GLOB_NOSPACE;
 		}
 
@@ -395,15 +455,22 @@ int
 		pglob->gl_pathv[pglob->gl_pathc] = NULL;
 		pglob->gl_flags = flags;
 	    }
-	    else
+	    else {
+	        XtFree(dirname);
 		return GLOB_NOMATCH;
+	    }
 	}
-    }
+    } /* if (glob_pattern_p(dirname, !(flags & GLOB_NOESCAPE))) */
     else
     {
+        DEBUGOUT(_LtDebugDebug(__FILE__, NULL,
+		      "glob_pattern(%s, !(flags & GLOB_NOESCAPE))=NULL\n", dirname));
+
 	status = glob_in_dir(filename, dirname, flags, errfunc, pglob);
-	if (status != 0)
+	if (status != 0) {
+ 	    XtFree(dirname);
 	    return status;
+	}
 
 	if (dirlen > 0)
 	{
@@ -413,14 +480,15 @@ int
 			     pglob->gl_pathc - oldcount))
 	    {
 		_Lesstif_globfree(pglob);
+		XtFree(dirname);
 		return GLOB_NOSPACE;
 	    }
 	}
-    }
+    } /* else */
 
     if (!(flags & GLOB_NOSORT))
 	/* Sort the vector.  */
-	qsort((__ptr_t) & pglob->gl_pathv[oldcount],
+	qsort((void *) & pglob->gl_pathv[oldcount],
 	      pglob->gl_pathc - oldcount,
 	      sizeof(char *), collated_compare);
 
@@ -433,14 +501,17 @@ int
 	   order (`"./" > "../"' but `"." < ".."'). */
 	int i;
 	struct stat st;
+
+        DEBUGOUT(_LtDebugDebug(__FILE__, NULL,
+		      "_Lesstif_glob(): GLOB_NOSORT\n"));
 	for (i = oldcount; i < pglob->gl_pathc; ++i)
-	    if (__lstat(pglob->gl_pathv[i], &st) == 0 &&
+	    if (LTstat(pglob->gl_pathv[i], &st) == 0 &&
 		S_ISDIR(st.st_mode))
 		strcat(pglob->gl_pathv[i], "/");
     }
-
+    XtFree(dirname);
     return 0;
-}
+} /* _Lesstif_glob() */
 
 
 /* Free storage allocated in PGLOB by a previous `glob' call.  */
@@ -450,17 +521,18 @@ _Lesstif_globfree(register glob_t *pglob)
     if (pglob->gl_pathv != NULL)
     {
 	register int i;
+
 	for (i = 0; i < pglob->gl_pathc; ++i)
 	    if (pglob->gl_pathv[i] != NULL)
-		XtFree((__ptr_t) pglob->gl_pathv[i]);
-	XtFree((__ptr_t) pglob->gl_pathv);
+		XtFree((char *) pglob->gl_pathv[i]);
+	XtFree((char *) pglob->gl_pathv);
     }
 }
 
 
 /* Do a collated comparison of A and B.  */
 static int
-collated_compare(const __ptr_t a, const __ptr_t b)
+collated_compare(const void * a, const void * b)
 {
     const char *const s1 = *(const char *const *const)a;
     const char *const s2 = *(const char *const *const)b;
@@ -503,14 +575,14 @@ prefix_array(const char *dirname, char **array, size_t n)
 	if (new_w == NULL)
 	{
 	    while (i > 0)
-		XtFree((__ptr_t) array[--i]);
+		XtFree((char *) array[--i]);
 	    return 1;
 	}
 
-	bcopy(dirname, new_w, dirlen);
+	memcpy(new_w, dirname, dirlen);
 	new_w[dirlen] = '/';
-	bcopy(array[i], &new_w[dirlen + 1], eltlen);
-	XtFree((__ptr_t) array[i]);
+	memcpy(&new_w[dirlen + 1], array[i], eltlen);
+	XtFree((char *) array[i]);
 	array[i] = new_w;
     }
 
@@ -526,6 +598,8 @@ glob_pattern_p(const char *pattern, int quote)
     register const char *p;
     int open = 0;
 
+    DEBUGOUT(_LtDebugDebug(__FILE__, NULL,
+		      "glob_pattern_p(%s, %i)\n", pattern, quote));
     for (p = pattern; *p != '\0'; ++p)
 	switch (*p)
 	{
@@ -549,7 +623,7 @@ glob_pattern_p(const char *pattern, int quote)
 	}
 
     return 0;
-}
+} /* glob_pattern_p() */
 
 
 /* Like `glob', but PATTERN is a final pathname component,
@@ -559,19 +633,21 @@ glob_pattern_p(const char *pattern, int quote)
  */
 static int
 glob_in_dir(const char *pattern, const char *directory, int flags,
-	    int (*errfunc) __P((const char *, int)),
-	    glob_t *pglob)
+	    errfunc_t errfunc, glob_t *pglob)
 {
-    __ptr_t stream;
-
+    void *stream;
     struct globlink
     {
 	struct globlink *next;
 	char *name;
     };
-    struct globlink *names = NULL, *tail = NULL;
+    struct globlink *names = NULL;
     size_t nfound = 0;
 
+
+    DEBUGOUT(_LtDebugDebug(__FILE__, NULL,
+		      "glob_in_dir(%s, %s, %i,)\n",
+		      pattern, directory, flags));    
     if (!glob_pattern_p(pattern, !(flags & GLOB_NOESCAPE)))
     {
 	stream = NULL;
@@ -582,7 +658,7 @@ glob_in_dir(const char *pattern, const char *directory, int flags,
 	flags |= GLOB_MAGCHAR;
 
 	stream = (__glob_opendir_hook ? (*__glob_opendir_hook) (directory)
-		  : (__ptr_t) opendir(directory));
+		  : (void *) opendir(directory));
 	if (stream == NULL)
 	{
 	    if ((errfunc != NULL && (*errfunc) (directory, errno)) ||
@@ -622,28 +698,18 @@ glob_in_dir(const char *pattern, const char *directory, int flags,
 			    ((flags & GLOB_NOESCAPE) ? FNM_NOESCAPE : 0)) == 0)
 		{
 		    struct globlink *new_w
-		    = (struct globlink *)__alloca(sizeof(struct globlink));
+		    = (struct globlink *)XtMalloc(sizeof(struct globlink));
+
 		    if (len == 0)
 			len = strlen(name);
 		    new_w->name
 			= (char *)XtMalloc(len + ((flags & GLOB_MARK) ? 1 : 0) + 1);
 		    if (new_w->name == NULL)
 			goto memory_error;
-		    bcopy(name, (__ptr_t) new_w->name, len);
+		    memcpy((void *) new_w->name, name, len);
 		    new_w->name[len] = '\0';
-#ifdef READDIR_LEXICOGRAPHIC_ORDER
-		    if (names == NULL)
-			names = tail = new_w;
-		    else
-		    {
-			tail->next = new_w;
-			tail = new_w;
-		    }
-		    new_w->next = NULL;
-#else
 		    new_w->next = names;
 		    names = new_w;
-#endif
 		    ++nfound;
 		}
 	    }
@@ -653,12 +719,12 @@ glob_in_dir(const char *pattern, const char *directory, int flags,
     {
 	size_t len = strlen(pattern);
 	nfound = 1;
-	names = (struct globlink *)__alloca(sizeof(struct globlink));
+	names = (struct globlink *)XtMalloc(sizeof(struct globlink));
 	names->next = NULL;
 	names->name = (char *)XtMalloc(len + 1);
 	if (names->name == NULL)
 	    goto memory_error;
-	bcopy(pattern, names->name, len);
+	memcpy(names->name, pattern, len);
 	names->name[len] = '\0';
     }
 
@@ -684,69 +750,32 @@ glob_in_dir(const char *pattern, const char *directory, int flags,
     if (stream != NULL)
     {
 	int save = errno;
+
 	if (__glob_closedir_hook)
 	    (*__glob_closedir_hook) (stream);
 	else
-	    (void)closedir((DIR *) stream);
+	    closedir((DIR *) stream);
 	errno = save;
     }
+    if (names) XtFree((char *)names);
     return nfound == 0 ? GLOB_NOMATCH : 0;
 
   memory_error:
     {
 	int save = errno;
+
 	if (__glob_closedir_hook)
 	    (*__glob_closedir_hook) (stream);
 	else
-	    (void)closedir((DIR *) stream);
+	    closedir((DIR *) stream);
 	errno = save;
     }
     while (names != NULL)
     {
 	if (names->name != NULL)
-	    XtFree((__ptr_t) names->name);
+	    XtFree((char *) names->name);
 	names = names->next;
     }
+    if (names) XtFree((char *)names);
     return GLOB_NOSPACE;
-}
-
-#ifdef __EMX__
-/* Under OS/2 we may need to deal with drive letters ...*/
-
-static char *GetDriveMap(void)
-{
-  /* returns a string containing a list of all drives: e.g. "C: D: E:" */
-
-  static char drivelist[256];   /* so we'll never have a memory leak beyond 256 bytes */
-  static char tmpstr[4];
-  ULONG  CurDrive;
-  ULONG  DriveMap;
-  int    dnum;
-  int    start = 3;
- 
-  drivelist[0] = '\0';
-  tmpstr[1] = ':';
-  tmpstr[2] = ' ';
-  tmpstr[3] = '\0';
-/*
-  DosError(0);
-*/
-  DosQueryCurrentDisk(&CurDrive, &DriveMap);
-  DriveMap>>=start-1;
-  for (dnum = start; dnum <= 26; dnum++) {
-     if ((DriveMap & (ULONG)1)) {
-        tmpstr[0] = (char)dnum+'A'-1;
-        strcat(drivelist, tmpstr);
-   }
-      DriveMap>>=1; 
-  }
- 
-  if (drivelist[strlen(drivelist)-1]== ' ')
-    drivelist[strlen(drivelist)-1] = '\0';
-
-/*
-  DosError(1);
-*/
-  return drivelist;
-}
-#endif
+} /* glob_in_dir() */
