@@ -1,8 +1,9 @@
 /**
  *
- * $Header: /cvsroot/hungry/lesstif/lib/Xm/Label.c,v 1.107 2000/01/03 22:32:57 danny Exp $
+ * $Header: /cvsroot/lesstif/lesstif/lib/Xm/Label.c,v 1.119 2001/05/14 19:46:09 amai Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
+ * Copyright (C) 1995-2001 LessTif Development Team
  *
  * This file is part of the GNU LessTif Library.
  *
@@ -22,11 +23,14 @@
  *
  **/
 
-static const char rcsid[] = "$Header: /cvsroot/hungry/lesstif/lib/Xm/Label.c,v 1.107 2000/01/03 22:32:57 danny Exp $";
+static const char rcsid[] = "$Header: /cvsroot/lesstif/lesstif/lib/Xm/Label.c,v 1.119 2001/05/14 19:46:09 amai Exp $";
 
 #include <LTconfig.h>
-#include <XmI/XmI.h>
 
+#include <stdio.h>
+#include <string.h>
+
+#include <XmI/XmI.h>
 #include <Xm/XmP.h>
 #include <Xm/XmosP.h>
 #include <Xm/AtomMgr.h>
@@ -42,8 +46,6 @@ static const char rcsid[] = "$Header: /cvsroot/hungry/lesstif/lib/Xm/Label.c,v 1
 #include <XmI/AtomMgrI.h>
 #include <XmI/DragDropI.h>
 
-#include <assert.h>
-
 #include <XmI/DebugUtil.h>
 
 /* Forward Declarations */
@@ -396,11 +398,11 @@ class_part_initialize(WidgetClass widget_class)
     (XmLabelWidgetClass)(widget_class->core_class.superclass);
     XmPrimitiveClassExt ext, *extptr, *sextptr;
 
-    DEBUGOUT(XdbDebug0(__FILE__, NULL, "%s:class_part_initialize(%d) - %s -> %s\n",
+    DEBUGOUT(_LtDebugDebug0(__FILE__, NULL, "%s:class_part_initialize(%d) - %s -> %s\n",
 		__FILE__, __LINE__,
 		swc->core_class.class_name,
 		widget_class->core_class.class_name));
-    DEBUGOUT(XdbDebug0("MENUT", NULL, "%s:class_part_initialize(%d) - %s -> %s\n",
+    DEBUGOUT(_LtDebugDebug0("MENUT", NULL, "%s:class_part_initialize(%d) - %s -> %s\n",
 		__FILE__, __LINE__,
 		swc->core_class.class_name,
 		widget_class->core_class.class_name));
@@ -408,34 +410,34 @@ class_part_initialize(WidgetClass widget_class)
 
     if (lwc->label_class.menuProcs == XmInheritMenuProc)
     {
-	DEBUGOUT(XdbDebug(__FILE__, NULL, "    XmInheritMenuProc\n"));
+	DEBUGOUT(_LtDebugDebug(__FILE__, NULL, "    XmInheritMenuProc\n"));
 	lwc->label_class.menuProcs =
 	    swc->label_class.menuProcs;
     }
     if (lwc->label_class.setOverrideCallback == XmInheritSetOverrideCallback)
     {
-	DEBUGOUT(XdbDebug(__FILE__, NULL, "    XmInheritSetOverrideCallback\n"));
+	DEBUGOUT(_LtDebugDebug(__FILE__, NULL, "    XmInheritSetOverrideCallback\n"));
 	lwc->label_class.setOverrideCallback =
 	    swc->label_class.setOverrideCallback;
     }
     if (lwc->label_class.translations == XtInheritTranslations)
     {
-	DEBUGOUT(XdbDebug(__FILE__, NULL, "    XtInheritTranslations\n"));
-	DEBUGOUT(XdbDebug0("MENUT", NULL, "    XtInheritTranslations\n"));
+	DEBUGOUT(_LtDebugDebug(__FILE__, NULL, "    XtInheritTranslations\n"));
+	DEBUGOUT(_LtDebugDebug0("MENUT", NULL, "    XtInheritTranslations\n"));
 	lwc->label_class.translations =
 	    swc->label_class.translations;
     }
     else if (lwc->label_class.translations != NULL)
     {
-	DEBUGOUT(XdbDebug(__FILE__, NULL, "    supplied by subclass\n"));
-	DEBUGOUT(XdbDebug0("MENUT", NULL, "    supplied by subclass\n"));
+	DEBUGOUT(_LtDebugDebug(__FILE__, NULL, "    supplied by subclass\n"));
+	DEBUGOUT(_LtDebugDebug0("MENUT", NULL, "    supplied by subclass\n"));
 	lwc->label_class.translations =
 	    (String)XtParseTranslationTable(lwc->label_class.translations);
     }
     else
     {
-	DEBUGOUT(XdbDebug(__FILE__, NULL, "    Use menu traversal translations\n"));
-	DEBUGOUT(XdbDebug0("MENUT", NULL, "    Use menu traversal translations\n"));
+	DEBUGOUT(_LtDebugDebug(__FILE__, NULL, "    Use menu traversal translations\n"));
+	DEBUGOUT(_LtDebugDebug0("MENUT", NULL, "    Use menu traversal translations\n"));
 	lwc->label_class.translations =
 	    (String)XtParseTranslationTable(_XmLabel_menu_traversal_events);
     }
@@ -517,7 +519,7 @@ CreateInsensitiveGC(Widget w)
     values.graphics_exposures = False;
     values.foreground = Prim_Foreground(w);
     values.background = XtBackground(w);
-    values.fill_style = FillOpaqueStippled;
+    values.fill_style = FillStippled;
     values.ts_x_origin = values.ts_y_origin = 0;
     values.stipple = XmGetPixmapByDepth(XtScreen(w), "50_foreground", 1, 0, 1);
 
@@ -530,7 +532,7 @@ initialize_prehook(Widget request,
 		   ArgList args,
 		   Cardinal *num_args)
 {
-    DEBUGOUT(XdbDebug(__FILE__, new_w, "Initialize Prehook\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, new_w, "Initialize Prehook\n"));
 
     _XmSaveCoreClassTranslations(new_w);
 
@@ -551,7 +553,7 @@ static void
 initialize_posthook(Widget request, Widget new_w,
 		    ArgList args, Cardinal *num_args)
 {
-    DEBUGOUT(XdbDebug(__FILE__, new_w, "Initialize Posthook\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, new_w, "Initialize Posthook\n"));
 
     _XmRestoreCoreClassTranslations(new_w);
 }
@@ -560,7 +562,7 @@ static void
 initialize(Widget request, Widget new_w,
 	   ArgList args, Cardinal *num_args)
 {
-    DEBUGOUT(XdbDebug(__FILE__, new_w,
+    DEBUGOUT(_LtDebugDebug(__FILE__, new_w,
 		      "%s:initialize(%d) - %i args\n"
 		      "\trequest X %5i Y %5i W %5i H %5i\n"
 		      "\t  new_w X %5i Y %5i W %5i H %5i\n",
@@ -570,7 +572,7 @@ initialize(Widget request, Widget new_w,
 		      XtWidth(request), XtHeight(request),
 		      XtX(new_w), XtY(new_w),
 		      XtWidth(new_w), XtHeight(new_w)));
-    DEBUGOUT(XdbPrintArgList(__FILE__, new_w, args, *num_args, False));
+    DEBUGOUT(_LtDebugPrintArgList(__FILE__, new_w, args, *num_args, False));
 
     Lab_SkipCallback(new_w) = False;
 
@@ -593,14 +595,14 @@ initialize(Widget request, Widget new_w,
     /* Force the traversal and highlight on enter resources if in a menu */
     if (Lab_MenuType(new_w) != XmWORK_AREA)
     {
-	DEBUGOUT(XdbDebug(__FILE__, new_w, "    merge in Label Class Translations\n"));
+	DEBUGOUT(_LtDebugDebug(__FILE__, new_w, "    merge in Label Class Translations\n"));
 #if 0
 MLM: WHY TRAVERSAL?
 	Prim_TraversalOn(new_w) = False;
 #endif
 	Prim_HighlightOnEnter(new_w) = False;
 
-	DEBUGOUT(XdbDebug("MENUT", new_w, "%s:initialize(%d) - Override translations\n",
+	DEBUGOUT(_LtDebugDebug("MENUT", new_w, "%s:initialize(%d) - Override translations\n",
 	    __FILE__, __LINE__
 	    ));
 	XtOverrideTranslations(new_w,
@@ -769,7 +771,7 @@ set_values(Widget old, Widget request, Widget new_w,
     Boolean refresh_needed = False, relayout_needed = False,
 	recalc_needed = False;
 
-    DEBUGOUT(XdbDebug(__FILE__, new_w,
+    DEBUGOUT(_LtDebugDebug(__FILE__, new_w,
 		      "%s(%d):set_values: %i args\n"
 		      "\t    old X %5i Y %5i W %5i H %5i\n"
 		      "\trequest X %5i Y %5i W %5i H %5i\n"
@@ -781,7 +783,7 @@ set_values(Widget old, Widget request, Widget new_w,
 		      XtWidth(request), XtHeight(request),
 		      XtX(new_w), XtY(new_w),
 		      XtWidth(new_w), XtHeight(new_w)));
-    DEBUGOUT(XdbPrintArgList(__FILE__, new_w, args, *num_args, False));
+    DEBUGOUT(_LtDebugPrintArgList(__FILE__, new_w, args, *num_args, False));
 
     /* Silently discard accelerator for non-menu labels and cascades */
     if ((Lab_MenuType(new_w) != XmMENU_PULLDOWN &&
@@ -1036,7 +1038,7 @@ expose(Widget w, XEvent *event, Region region)
     GC myGC;
     Pixmap mypm;
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "Expose\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "Expose\n"));
 
     /*
      * I'm becoming paranoid - Danny - see testXm/filesb/test3
@@ -1086,7 +1088,7 @@ expose(Widget w, XEvent *event, Region region)
     /* Draw the accelerator without clipping */
     if (Lab_AcceleratorText(w))
     {
-	DEBUGOUT(XdbDebug(__FILE__, w,
+	DEBUGOUT(_LtDebugDebug(__FILE__, w,
 			  "Expose: acctextrect %d %d %dx%d\n",
 			  Lab_AccTextRect(w).x, Lab_AccTextRect(w).y,
 			  Lab_AccTextRect(w).width, Lab_AccTextRect(w).height));
@@ -1127,7 +1129,7 @@ expose(Widget w, XEvent *event, Region region)
 	- ((Prim_ShadowThickness(w) + Prim_HighlightThickness(w)) << 1)
 	- Lab_MarginTop(w) - Lab_MarginBottom(w);
 
-    DEBUGOUT(XdbDebug(__FILE__, w,
+    DEBUGOUT(_LtDebugDebug(__FILE__, w,
 		      "Expose: cliprect %d %d %dx%d\n",
 		      cliprect.x, cliprect.y,
 		      cliprect.width, cliprect.height));
@@ -1174,6 +1176,12 @@ expose(Widget w, XEvent *event, Region region)
 	mypm = XtSensitive(w)
 	    ? Lab_Pixmap(w)
 	    : Lab_PixmapInsensitive(w);
+#if XmVERSION > 1
+	if (mypm == XmUNSPECIFIED_PIXMAP)
+	{
+	    mypm = Lab_Pixmap(w);
+	}
+#endif
 	if (mypm > XmUNSPECIFIED_PIXMAP &&
 	    cliprect.x + cliprect.width > Lab_TextRect_x(w) &&
 	    cliprect.x < Lab_TextRect_x(w) + Lab_TextRect_width(w) &&
@@ -1202,14 +1210,106 @@ expose(Widget w, XEvent *event, Region region)
 		cliprect.height =
 		    (Lab_TextRect_y(w) + Lab_TextRect_height(w)) - cliprect.y;
 	    }
+#if XmVERSION > 1
+	    {
+		unsigned int tmp, tmpw, tmph;
+		int tmpx, tmpy;
+		int Depth;
+		Window tmpwin;
+
+		/* rws 4 Mar 2000
+		   According to the comments in label/test12, this does not
+		   happen. We should be adding the pixmap to our pixmap
+		   cache, and getting the info from there instead of a
+		   round trip server query.
+		 */
+		/* rws 10 Mar 2000
+		   Apparently a Motif 2 thing.
+		 */
+		/*
+		   eaf 3 Apr 2000 use the cache for this.  Tests indicate
+		   that Motif does this in set_values, but I like waiting
+		   until we actually need the information.
+		 */
+		{
+		  char *name ;
+		  Pixel fg, bg ;
+		  int hotx, hoty ;
+		  u_int wid, hgt ;
+
+		  if( !_XmGetPixmapData(XtScreen(w), mypm, &name,
+		  	&Depth, &fg, &bg, &hotx, &hoty, &wid, &hgt) )
+		  {
+		    /* This pixmap has never been seen before.  Generate
+		     * a unique name and add it to the cache.  The cache
+		     * mechanism will obtain the depth.  Then we can try
+		     * again.
+		     */
+		    char newname[64] ;
+		    sprintf(newname, "--anon pixmap %p-%x",
+		      (void *)XtScreen(w),
+		      (unsigned)mypm) ;
+		    (void) _XmInstallPixmap(mypm, XtScreen(w), newname,
+			  Prim_Foreground(w), XtBackground(w)) ;
+
+		    /* Ok, let's try again. */
+		    if( !_XmGetPixmapData(XtScreen(w), mypm, &name,
+			  &Depth, &fg, &bg, &hotx, &hoty, &wid, &hgt) )
+		    {
+		      /* Still not found?
+		       * Don't panic, ask the server directly.
+		       */
+		      unsigned uDepth;
+		      XGetGeometry(XtDisplayOfObject(w), mypm, &tmpwin,
+				&tmpx, &tmpy, &tmpw, &tmph, &tmp, &uDepth);
+		      Depth=uDepth;
+		    }
+		  }
+		}
+
+		if (Depth == 1)
+		{
+		    XCopyPlane(XtDisplay(w),
+			  mypm,
+			  XtWindow(w),
+			  myGC,
+			  cliprect.x - Lab_TextRect_x(w),
+			  cliprect.y - Lab_TextRect_y(w),
+			  cliprect.width, cliprect.height,
+			  cliprect.x, cliprect.y,
+			  1);
+		}
+		else if (Depth > 1)
+		{
+		    XCopyArea(XtDisplay(w),
+			  mypm,
+			  XtWindow(w),
+			  myGC,
+			  cliprect.x - Lab_TextRect_x(w),
+			  cliprect.y - Lab_TextRect_y(w),
+			  cliprect.width, cliprect.height,
+			  cliprect.x, cliprect.y);
+		}
+	    }
+	    if (!XtSensitive(w) && Lab_PixmapInsensitive(w) == XmUNSPECIFIED_PIXMAP)
+	    {
+	    	/* label/test14 */
+	    	XFillRectangle(XtDisplay(w), XtWindow(w),
+	    		myGC,
+		  cliprect.x,
+		  cliprect.y,
+		  cliprect.width, cliprect.height);
+	    }
+#else
 	    XCopyArea(XtDisplay(w),
-		      mypm,
-		      XtWindow(w),
-		      myGC,
-		      cliprect.x - Lab_TextRect_x(w),
-		      cliprect.y - Lab_TextRect_y(w),
-		      cliprect.width, cliprect.height,
-		      cliprect.x, cliprect.y);
+		  mypm,
+		  XtWindow(w),
+		  myGC,
+		  cliprect.x - Lab_TextRect_x(w),
+		  cliprect.y - Lab_TextRect_y(w),
+		  cliprect.width, cliprect.height,
+		  cliprect.x, cliprect.y);
+#endif
 	}
     }
 }
@@ -1302,7 +1402,7 @@ resize(Widget w)
 				 - Lab_AccTextRect(w).height) >> 1)
 	    + Lab_MarginTop(w);
 #if 0
-	DEBUGOUT(XdbDebug(__FILE__, w,
+	DEBUGOUT(_LtDebugDebug(__FILE__, w,
 			  "Accelerator @ %d %d (x = %d - %d - %d - %d)\n",
 			  Lab_AccTextRect(w).x, Lab_AccTextRect(w).y,
 			  XtWidth(w), Lab_Shadow(w), Lab_Highlight(w),
@@ -1318,7 +1418,7 @@ query_geometry(Widget w, XtWidgetGeometry *proposed, XtWidgetGeometry *answer)
     XtWidgetGeometry a;
     Dimension wd, ht;
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "query_geometry\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "query_geometry\n"));
 
     wd = XtWidth(w);
     ht = XtHeight(w);
@@ -1391,8 +1491,8 @@ query_geometry(Widget w, XtWidgetGeometry *proposed, XtWidgetGeometry *answer)
 	*answer = a;
     }
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "query_geometry => %s\n",
-		      XdbWidgetGeometry2String(answer)));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "query_geometry => %s\n",
+		      _LtDebugWidgetGeometry2String(answer)));
 
     if ((proposed->request_mode & (CWWidth | CWHeight)) ==
 	(CWWidth | CWHeight) &&
@@ -1532,10 +1632,16 @@ drag_convert_proc(Widget w, Atom *selection,
 {
     Atom COMPOUND_TEXT;
     Atom MOTIF_DROP;
+    Widget source;
+    char *ct, *text;
+    XmString label;
+    Atom TEXT;
+    int len;
 
     _XmObjectLock(w);
     COMPOUND_TEXT = XmInternAtom(XtDisplay(w), _XA_COMPOUND_TEXT, False);
     MOTIF_DROP = XmInternAtom(XtDisplay(w), _XA_MOTIF_DROP, False);
+    TEXT = XmInternAtom(XtDisplay(w), _XA_TEXT, False);
 
     if (*selection != MOTIF_DROP)
     {
@@ -1543,43 +1649,97 @@ drag_convert_proc(Widget w, Atom *selection,
 	return False;
     }
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "We're dealing with a motif drop\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "We're dealing with a motif drop\n"));
+
+    XtVaGetValues(w,
+    	XmNsourceWidget, &source,
+    	NULL);
+    XtVaGetValues(source,
+    	XmNlabelString, &label,
+    	NULL);
+
+    ct = XmCvtXmStringToCT(label);
+    XmStringFree(label);
+    text = XtNewString(ct);
+    len = strlen(text);
+
+    if (*target == XA_STRING){
+        *length_return = len;
+        *value_return = text;
+        *type_return = XA_STRING;
+    }
+    else if (*target == COMPOUND_TEXT || *target == TEXT){
+
+	    XTextProperty prop;
+	    char *buf;
+	    int ret;
+
+	    buf = XtMalloc(len + 1);
+            strncpy(buf, text, len);
+	    buf[len] = '\0';
+
+	    ret = XmbTextListToTextProperty(XtDisplay(w), &buf, 1,
+	    			XCompoundTextStyle, &prop);
+		
+	    XtFree(buf);
+	    if (ret != 0){
+	    	*length_return = 0;
+	    	*value_return  = NULL;
+	    }
+	    else{
+	    	buf = XtMalloc(prop.nitems + 1);
+		strncpy(buf, prop.value, prop.nitems);
+		buf[prop.nitems] = '\0';
+	    	*length_return = prop.nitems;
+	    	*value_return  = buf;
+	    }
+	   *type_return = COMPOUND_TEXT;
+
+    }
+    else
+	return False;
+
+    *format_return = 8;
 
     _XmObjectUnlock(w);
-    return False;
+    return True;
 }
 
 static void
 drag_drop_finish(Widget w, XtPointer client_data, XtPointer call_data)
 {
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "%s:drag_drop_finish(%d)\n",
+    	__FILE__, __LINE__));
 }
 
 static void
 ProcessDrag(Widget w, XEvent *event, String *params, Cardinal *num_params)
 {
-    Atom export_target;
+    Atom export_target[3];
     Arg args[10];
     int n = 0;
     Widget dc;
 
     _XmObjectLock(w);
-    DEBUGOUT(XdbDebug(__FILE__, w, "Processing a drag-drop request\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "Processing a drag-drop request\n"));
 
     if (Lab_IsPixmap(w))
     {
-	export_target = XmInternAtom(XtDisplay(w),
+	export_target[0] = XmInternAtom(XtDisplay(w),
 				     _XA_PIXMAP,
 				     False);
+        XtSetArg(args[n], XmNexportTargets, export_target); n++;
+        XtSetArg(args[n], XmNnumExportTargets, 1); n++;
     }
     else
     {
-	export_target = XmInternAtom(XtDisplay(w),
-				     _XA_COMPOUND_TEXT,
-				     False);
+	export_target[0] = XmInternAtom(XtDisplay(w), _XA_COMPOUND_TEXT, False);
+	export_target[1] = XmInternAtom(XtDisplay(w), _XA_TEXT, False);
+	export_target[2] = XA_STRING;
+        XtSetArg(args[n], XmNexportTargets, export_target); n++;
+        XtSetArg(args[n], XmNnumExportTargets, 3); n++;
     }
 
-    XtSetArg(args[n], XmNexportTargets, &export_target); n++;
-    XtSetArg(args[n], XmNnumExportTargets, 1); n++;
     XtSetArg(args[n], XmNdragOperations, XmDROP_COPY); n++;
     XtSetArg(args[n], XmNconvertProc, drag_convert_proc); n++;
     XtSetArg(args[n], XmNclientData, w); n++;
@@ -1658,9 +1818,17 @@ _XmCalcLabelDimensions(Widget w)
     }
     else
     {
-	_XmLabelGetPixmapSize(w, (XtIsSensitive(w)
-				  ? Lab_Pixmap(w)
-				  : Lab_PixmapInsensitive(w)),
+    Pixmap mypm;
+
+	mypm = XtIsSensitive(w) ? Lab_Pixmap(w) : Lab_PixmapInsensitive(w);
+#if XmVERSION > 1
+	if (mypm == XmUNSPECIFIED_PIXMAP)
+	{
+	    mypm = Lab_Pixmap(w);
+	}
+#endif
+
+	_XmLabelGetPixmapSize(w, mypm,
 			      &Lab_TextRect_width(w),
 			      &Lab_TextRect_height(w));
     }
