@@ -1,8 +1,9 @@
 /**
  *
- * $Id: LabelG.c,v 1.88 1999/07/27 23:37:32 rwscott Exp $
+ * $Id: LabelG.c,v 1.99 2001/05/14 19:46:09 amai Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
+ * Copyright (C) 1995-2001 LessTif Development Team
  *
  * This file is part of the GNU LessTif Library.
  *
@@ -22,9 +23,12 @@
  *
  **/
 
-static const char rcsid[] = "$Id: LabelG.c,v 1.88 1999/07/27 23:37:32 rwscott Exp $";
+static const char rcsid[] = "$Id: LabelG.c,v 1.99 2001/05/14 19:46:09 amai Exp $";
 
 #include <LTconfig.h>
+
+#include <string.h>
+
 #include <XmI/XmI.h>
 
 #include <Xm/XmP.h>
@@ -40,7 +44,6 @@ static const char rcsid[] = "$Id: LabelG.c,v 1.88 1999/07/27 23:37:32 rwscott Ex
 #include <Xm/ScreenP.h>
 #include <Xm/DragIconP.h>
 #include <XmI/AtomMgrI.h>
-#include <X11/Xfuncs.h>
 
 #include <XmI/DebugUtil.h>
 
@@ -470,9 +473,9 @@ secondary_object_create(Widget request,
     XmBaseClassExt *bce;
     XtPointer nsec, rsec;
     XmWidgetExtData ed;
-    int size;
+    Cardinal size;
 
-    DEBUGOUT(XdbDebug(__FILE__, new_w,
+    DEBUGOUT(_LtDebugDebug(__FILE__, new_w,
 		      "LabelGCacheRec %s being initialized.\n", XtName(new_w)));
 
     bce = _XmGetBaseClassExtPtr(XtClass(new_w), XmQmotif);
@@ -501,7 +504,7 @@ secondary_object_create(Widget request,
     ed->widget = (Widget)nsec;
     ed->reqWidget = (Widget)rsec;
 
-    bcopy(nsec, rsec, size);
+    memcpy(rsec, nsec, size);
     ((XmExtRec *)rsec)->object.self = (Widget)rsec;
 
     _XmPushWidgetExtData(new_w, ed, XmCACHE_EXTENSION);
@@ -513,86 +516,11 @@ secondary_object_create(Widget request,
 int
 _XmLabelCacheCompare(XtPointer A, XtPointer B)
 {
-    return !bcmp(((XmLabelGCacheObjPart *)A),
+    return !memcmp(((XmLabelGCacheObjPart *)A),
 		 ((XmLabelGCacheObjPart *)B),
 		 sizeof(XmLabelGCacheObjPart));
 }
 
-void
-_LtLabelGAccomodateIndicator(Widget w, Widget req, int iwidth, int iheight,
-			     int odd, int left, int sv)
-{
-    int tm, bm, minm;
-
-    /* Increase margins as necessary to fit an indicator for ToggleBG and
-     * CascadeBG.  They may be decreased as well if called from set_values.
-     * The external widget size may change if the requested size was default
-     * or recomputeSize is set in set_values.
-     */
-
-    /* Make room on the left or right side as needed */
-    if (left)
-    {
-	iwidth -= LabG_MarginLeft(w);
-	if (iwidth > 0 ||
-	    (iwidth && sv && LabG_MarginLeft(w) == LabG_MarginLeft(req)))
-	if (iwidth > 0)
-	{
-	    LabG_MarginLeft(w) += iwidth;
-	    LabG_TextRect_x(w) += iwidth;
-	    LabG_AccTextRect(w).x += iwidth;
-	    if (!XtWidth(req) || (sv && LabG_RecomputeSize(w)))
-		XtWidth(w) += iwidth;
-	}
-    }
-    else
-    {
-	iwidth -= LabG_MarginRight(w);
-	if (iwidth > 0 ||
-	    (iwidth && sv && LabG_MarginRight(w) == LabG_MarginRight(req)))
-	{
-	    LabG_MarginRight(w) += iwidth;
-	    if (!XtWidth(req) || (sv && LabG_RecomputeSize(w)))
-		XtWidth(w) += iwidth;
-	}
-    }
-
-    /* Make room vertically.  An odd-sized adjustment may put the odd bit
-     * on the bottom (or it may not).  The set_values case gets complex
-     * as the top may be modified but not the bottom, or vice versa.
-     */
-    iheight -=
-	LabG_TextRect_height(w) + LabG_MarginTop(w) + LabG_MarginBottom(w);
-
-    tm = iheight / 2;
-    minm = sv && LabG_MarginTop(w) == LabG_MarginTop(req)
-	? XmDEFAULT_TOP_MARGIN - (int)LabG_MarginTop(w)
-	: 0;
-    if (tm < minm)
-	tm = minm;
-    LabG_MarginTop(w) += tm;
-
-    bm = (iheight + odd) / 2;
-    minm = sv && bm < LabG_MarginBottom(w) == LabG_MarginBottom(req)
-	? XmDEFAULT_BOTTOM_MARGIN - (int)LabG_MarginBottom(w)
-	: 0;
-    if (bm < minm)
-	bm = minm;
-    LabG_MarginBottom(w) += bm;
-
-    if (!XtHeight(req) || (sv && LabG_RecomputeSize(w)))
-    {
-	LabG_TextRect_y(w) += tm;
-	LabG_AccTextRect(w).y += tm;
-	XtHeight(w) += tm + bm;
-    }
-    else if (tm != bm)
-    {
-	LabG_TextRect_y(w) += (tm - bm) / 2;
-	LabG_AccTextRect(w).y += (tm - bm) / 2;
-    }
-}
-
 void
 _XmCalcLabelGDimensions(Widget w)
 {
@@ -609,9 +537,17 @@ _XmCalcLabelGDimensions(Widget w)
     }
     else
     {
-	_XmLabelGetPixmapSize(w, (XtIsSensitive(w)
-				  ? LabG_Pixmap(w)
-				  : LabG_PixmapInsensitive(w)),
+    Pixmap mypm;
+
+	mypm = XtIsSensitive(w) ? LabG_Pixmap(w) : LabG_PixmapInsensitive(w);
+#if XmVERSION > 1
+	if (mypm == XmUNSPECIFIED_PIXMAP)
+	{
+	    mypm = LabG_Pixmap(w);
+	}
+#endif
+
+	_XmLabelGetPixmapSize(w, mypm,
 			      &LabG_TextRect_width(w),
 			      &LabG_TextRect_height(w));
     }
@@ -650,7 +586,7 @@ _XmAssignLabG_MarginHeight(XmLabelGadget lw, Dimension value)
 {
     XmBaseClassExt *bce;
     XmWidgetExtData ed;
-    int size;
+    Cardinal size;
     XtPointer nsec;
 
     if ((ed = _XmGetWidgetExtData((Widget)lw, XmCACHE_EXTENSION)) != NULL)
@@ -674,9 +610,9 @@ _XmAssignLabG_MarginHeight(XmLabelGadget lw, Dimension value)
     ExtObj_LogicalParent(nsec) = (Widget)lw;
     ExtObj_ExtensionType(nsec) = XmCACHE_EXTENSION;
 
-    bcopy(LabG_Cache((Widget)lw),
-	  &((XmLabelGCacheObject)nsec)->label_cache,
-	  sizeof(XmLabelGCacheObjPart));
+    memcpy(&((XmLabelGCacheObject)nsec)->label_cache,
+           LabG_Cache((Widget)lw),
+	   sizeof(XmLabelGCacheObjPart));
 
     ed = (XmWidgetExtData)XtMalloc(sizeof(XmWidgetExtDataRec));
     ed->widget = (Widget)nsec;
@@ -695,7 +631,7 @@ _XmAssignLabG_MarginWidth(XmLabelGadget lw, Dimension value)
 {
     XmBaseClassExt *bce;
     XmWidgetExtData ed;
-    int size;
+    Cardinal size;
     XtPointer nsec;
 
     if ((ed = _XmGetWidgetExtData((Widget)lw, XmCACHE_EXTENSION)) != NULL)
@@ -719,9 +655,9 @@ _XmAssignLabG_MarginWidth(XmLabelGadget lw, Dimension value)
     ExtObj_LogicalParent(nsec) = (Widget)lw;
     ExtObj_ExtensionType(nsec) = XmCACHE_EXTENSION;
 
-    bcopy(LabG_Cache((Widget)lw),
-	  &((XmLabelGCacheObject)nsec)->label_cache,
-	  sizeof(XmLabelGCacheObjPart));
+    memcpy(&((XmLabelGCacheObject)nsec)->label_cache,
+           LabG_Cache((Widget)lw),
+	   sizeof(XmLabelGCacheObjPart));
 
     ed = (XmWidgetExtData)XtMalloc(sizeof(XmWidgetExtDataRec));
     ed->widget = (Widget)nsec;
@@ -738,7 +674,7 @@ _XmAssignLabG_MarginLeft(XmLabelGadget lw, Dimension value)
 {
     XmBaseClassExt *bce;
     XmWidgetExtData ed;
-    int size;
+    Cardinal size;
     XtPointer nsec;
 
     if ((ed = _XmGetWidgetExtData((Widget)lw, XmCACHE_EXTENSION)) != NULL)
@@ -762,9 +698,9 @@ _XmAssignLabG_MarginLeft(XmLabelGadget lw, Dimension value)
     ExtObj_LogicalParent(nsec) = (Widget)lw;
     ExtObj_ExtensionType(nsec) = XmCACHE_EXTENSION;
 
-    bcopy(LabG_Cache((Widget)lw),
-	  &((XmLabelGCacheObject)nsec)->label_cache,
-	  sizeof(XmLabelGCacheObjPart));
+    memcpy(&((XmLabelGCacheObject)nsec)->label_cache,
+           LabG_Cache((Widget)lw),
+	   sizeof(XmLabelGCacheObjPart));
 
     ed = (XmWidgetExtData)XtMalloc(sizeof(XmWidgetExtDataRec));
     ed->widget = (Widget)nsec;
@@ -781,7 +717,7 @@ _XmAssignLabG_MarginRight(XmLabelGadget lw, Dimension value)
 {
     XmBaseClassExt *bce;
     XmWidgetExtData ed;
-    int size;
+    Cardinal size;
     XtPointer nsec;
 
     if ((ed = _XmGetWidgetExtData((Widget)lw, XmCACHE_EXTENSION)) != NULL)
@@ -805,9 +741,9 @@ _XmAssignLabG_MarginRight(XmLabelGadget lw, Dimension value)
     ExtObj_LogicalParent(nsec) = (Widget)lw;
     ExtObj_ExtensionType(nsec) = XmCACHE_EXTENSION;
 
-    bcopy(LabG_Cache((Widget)lw),
-	  &((XmLabelGCacheObject)nsec)->label_cache,
-	  sizeof(XmLabelGCacheObjPart));
+    memcpy(&((XmLabelGCacheObject)nsec)->label_cache,
+           LabG_Cache((Widget)lw),
+	   sizeof(XmLabelGCacheObjPart));
 
     ed = (XmWidgetExtData)XtMalloc(sizeof(XmWidgetExtDataRec));
     ed->widget = (Widget)nsec;
@@ -824,7 +760,7 @@ _XmAssignLabG_MarginTop(XmLabelGadget lw, Dimension value)
 {
     XmBaseClassExt *bce;
     XmWidgetExtData ed;
-    int size;
+    Cardinal size;
     XtPointer nsec;
 
     if ((ed = _XmGetWidgetExtData((Widget)lw, XmCACHE_EXTENSION)) != NULL)
@@ -848,9 +784,9 @@ _XmAssignLabG_MarginTop(XmLabelGadget lw, Dimension value)
     ExtObj_LogicalParent(nsec) = (Widget)lw;
     ExtObj_ExtensionType(nsec) = XmCACHE_EXTENSION;
 
-    bcopy(LabG_Cache((Widget)lw),
-	  &((XmLabelGCacheObject)nsec)->label_cache,
-	  sizeof(XmLabelGCacheObjPart));
+    memcpy(&((XmLabelGCacheObject)nsec)->label_cache,
+           LabG_Cache((Widget)lw),
+	   sizeof(XmLabelGCacheObjPart));
 
     ed = (XmWidgetExtData)XtMalloc(sizeof(XmWidgetExtDataRec));
     ed->widget = (Widget)nsec;
@@ -867,7 +803,7 @@ _XmAssignLabG_MarginBottom(XmLabelGadget lw, Dimension value)
 {
     XmBaseClassExt *bce;
     XmWidgetExtData ed;
-    int size;
+    Cardinal size;
     XtPointer nsec;
 
     if ((ed = _XmGetWidgetExtData((Widget)lw, XmCACHE_EXTENSION)) != NULL)
@@ -891,9 +827,9 @@ _XmAssignLabG_MarginBottom(XmLabelGadget lw, Dimension value)
     ExtObj_LogicalParent(nsec) = (Widget)lw;
     ExtObj_ExtensionType(nsec) = XmCACHE_EXTENSION;
 
-    bcopy(LabG_Cache((Widget)lw),
-	  &((XmLabelGCacheObject)nsec)->label_cache,
-	  sizeof(XmLabelGCacheObjPart));
+    memcpy(&((XmLabelGCacheObject)nsec)->label_cache,
+           LabG_Cache((Widget)lw),
+	   sizeof(XmLabelGCacheObjPart));
 
     ed = (XmWidgetExtData)XtMalloc(sizeof(XmWidgetExtDataRec));
     ed->widget = (Widget)nsec;
@@ -914,17 +850,77 @@ drag_convert_proc(Widget w, Atom *selection,
 {
     Atom COMPOUND_TEXT;
     Atom MOTIF_DROP;
+    Widget source;
+    char *ct, *text;
+    XmString label;
+    Atom TEXT;
+    int len;
 
+    _XmObjectLock(w);
     COMPOUND_TEXT = XmInternAtom(XtDisplay(w), _XA_COMPOUND_TEXT, False);
     MOTIF_DROP = XmInternAtom(XtDisplay(w), _XA_MOTIF_DROP, False);
+    TEXT = XmInternAtom(XtDisplay(w), _XA_TEXT, False);
 
     if (*selection != MOTIF_DROP)
     {
+	_XmObjectUnlock(w);
 	return False;
     }
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "We're dealing with a motif drop\n"));
-    return False;
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "We're dealing with a motif drop\n"));
+
+    XtVaGetValues(w,
+    	XmNsourceWidget, &source,
+    	NULL);
+    XtVaGetValues(source,
+    	XmNlabelString, &label,
+    	NULL);
+
+    ct = XmCvtXmStringToCT(label);
+    XmStringFree(label);
+    text = XtNewString(ct);
+    len = strlen(text);
+
+    if (*target == XA_STRING){
+        *length_return = len;
+        *value_return = text;
+        *type_return = XA_STRING;
+    }
+    else if (*target == COMPOUND_TEXT || *target == TEXT){
+
+	    XTextProperty prop;
+	    char *buf;
+	    int ret;
+
+	    buf = XtMalloc(len + 1);
+            strncpy(buf, text, len);
+	    buf[len] = '\0';
+
+	    ret = XmbTextListToTextProperty(XtDisplay(w), &buf, 1,
+	    			XCompoundTextStyle, &prop);
+		
+	    XtFree(buf);
+	    if (ret != 0){
+	    	*length_return = 0;
+	    	*value_return  = NULL;
+	    }
+	    else{
+	    	buf = XtMalloc(prop.nitems + 1);
+		strncpy(buf, prop.value, prop.nitems);
+		buf[prop.nitems] = '\0';
+	    	*length_return = prop.nitems;
+	    	*value_return  = buf;
+	    }
+	   *type_return = COMPOUND_TEXT;
+
+    }
+    else
+	return False;
+
+    *format_return = 8;
+
+    _XmObjectUnlock(w);
+    return True;
 }
 
 static void
@@ -936,28 +932,30 @@ void
 _XmProcessDrag(Widget w, XEvent *event,
 	       String *params, Cardinal *num_params)
 {
-    Atom export_target;
+    Atom export_target[3];
     Arg args[10];
     int n = 0;
     Widget dc;
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "Processing a drag-drop request\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "Processing a drag-drop request\n"));
 
     if (LabG_IsPixmap(w))
     {
-	export_target = XmInternAtom(XtDisplay(w),
+	export_target[0] = XmInternAtom(XtDisplay(w),
 				     _XA_PIXMAP,
 				     False);
+        XtSetArg(args[n], XmNexportTargets, export_target); n++;
+        XtSetArg(args[n], XmNnumExportTargets, 1); n++;
     }
     else
     {
-	export_target = XmInternAtom(XtDisplay(w),
-				     _XA_COMPOUND_TEXT,
-				     False);
+	export_target[0] = XmInternAtom(XtDisplay(w), _XA_COMPOUND_TEXT, False);
+	export_target[1] = XmInternAtom(XtDisplay(w), _XA_TEXT, False);
+	export_target[2] = XA_STRING;
+        XtSetArg(args[n], XmNexportTargets, export_target); n++;
+        XtSetArg(args[n], XmNnumExportTargets, 3); n++;
     }
 
-    XtSetArg(args[n], XmNexportTargets, &export_target); n++;
-    XtSetArg(args[n], XmNnumExportTargets, 1); n++;
     XtSetArg(args[n], XmNdragOperations, XmDROP_COPY); n++;
     XtSetArg(args[n], XmNconvertProc, drag_convert_proc); n++;
     XtSetArg(args[n], XmNclientData, w); n++;
@@ -1075,7 +1073,7 @@ CreateInsensitiveGC(Widget w)
     values.graphics_exposures = False;
     values.foreground = XmParentForeground(w);
     values.background = XmParentBackground(w);
-    values.fill_style = FillOpaqueStippled;
+    values.fill_style = FillStippled;
     values.ts_x_origin = values.ts_y_origin = 0;
     values.stipple = XmGetPixmapByDepth(XtScreen(w), "50_foreground", 1, 0, 1);
 
@@ -1087,7 +1085,7 @@ static void
 initialize_prehook(Widget request, Widget new_w,
 		   ArgList args, Cardinal *num_args)
 {
-    DEBUGOUT(XdbDebug(__FILE__, new_w, "LabelG InitializePrehook\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, new_w, "LabelG InitializePrehook\n"));
 }
 #endif
 
@@ -1099,7 +1097,7 @@ initialize_posthook(Widget request,
 {
     XmWidgetExtData ext;
 
-    DEBUGOUT(XdbDebug(__FILE__, new_w, "LabelG InitializePosthook\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, new_w, "LabelG InitializePosthook\n"));
 
     /* don't let the null fool you */
     LabG_Cache(new_w) = (XmLabelGCacheObjPart *)
@@ -1119,7 +1117,7 @@ initialize(Widget request, Widget new_w,
 {
     XmString xmstring;
 
-    DEBUGOUT(XdbDebug(__FILE__, new_w,
+    DEBUGOUT(_LtDebugDebug(__FILE__, new_w,
 		      "%s:initialize(%d) - %i args\n"
 		      "\trequest X %5i Y %5i W %5i H %5i\n"
 		      "\t  new_w X %5i Y %5i W %5i H %5i\n",
@@ -1129,7 +1127,7 @@ initialize(Widget request, Widget new_w,
 		      XtWidth(request), XtHeight(request),
 		      XtX(new_w), XtY(new_w),
 		      XtWidth(new_w), XtHeight(new_w)));
-    DEBUGOUT(XdbPrintArgList(__FILE__, new_w, args, *num_args, False));
+    DEBUGOUT(_LtDebugPrintArgList(__FILE__, new_w, args, *num_args, False));
 
     LabG_SkipCallback(new_w) = False;
 
@@ -1304,6 +1302,7 @@ resize(Widget w)
 
     /* Make sure there's room for the accelerator */
     if (LabG_AcceleratorText(w))
+    {
 	if (LabG_StringDirection(w) == XmSTRING_DIRECTION_L_TO_R)
 	{
 	    if (LabG_MarginRight(w) <
@@ -1311,7 +1310,7 @@ resize(Widget w)
 	    {
 		_XmAssignLabG_MarginRight((XmLabelGadget)w,
 		    LABEL_ACC_PAD + LabG_AccTextRect(w).width);
-		DEBUGOUT(XdbDebug(__FILE__, w,
+		DEBUGOUT(_LtDebugDebug(__FILE__, w,
 				  "resize: set right margin to %d\n",
 				  LabG_MarginRight(w)));
 		_XmReCacheLabG(w);
@@ -1324,18 +1323,23 @@ resize(Widget w)
 	    {
 		_XmAssignLabG_MarginLeft((XmLabelGadget)w,
 		    LABEL_ACC_PAD + LabG_AccTextRect(w).width);
-		DEBUGOUT(XdbDebug(__FILE__, w,
+		DEBUGOUT(_LtDebugDebug(__FILE__, w,
 				  "resize: set left margin to %d\n",
 				  LabG_MarginLeft(w)));
 		_XmReCacheLabG(w);
 	    }
 	}
+    }
 
     /* Set our preferred dimensions if they are currently zero */
     if (!XtWidth(w))
+    {
 	preferred_size(w, &XtWidth(w), NULL);
+    }
     if (!XtHeight(w))
+    {
 	preferred_size(w, NULL, &XtHeight(w));
+    }
 
     /* An RtoL stringDirection reverses the effect of alignment here */
     switch (LabG_Alignment(w) ^ (LabG_StringDirection(w) << 1))
@@ -1403,7 +1407,7 @@ set_values_prehook(Widget old, Widget request, Widget new_w,
 {
     XmBaseClassExt *bce;
     XmWidgetExtData ed;
-    int size;
+    Cardinal size;
     XtPointer nsec, rsec;
 
     bce = _XmGetBaseClassExtPtr(XtClass(new_w), XmQmotif);
@@ -1423,9 +1427,9 @@ set_values_prehook(Widget old, Widget request, Widget new_w,
     ExtObj_LogicalParent(nsec) = new_w;
     ExtObj_ExtensionType(nsec) = XmCACHE_EXTENSION;
 
-    bcopy(LabG_Cache(new_w),
-	  &((XmLabelGCacheObject)nsec)->label_cache,
-	  sizeof(XmLabelGCacheObjPart));
+    memcpy(&((XmLabelGCacheObject)nsec)->label_cache,
+           LabG_Cache(new_w),
+	   sizeof(XmLabelGCacheObjPart));
 
     ed = (XmWidgetExtData)XtMalloc(sizeof(XmWidgetExtDataRec));
     ed->widget = (Widget)nsec;
@@ -1440,7 +1444,7 @@ set_values_prehook(Widget old, Widget request, Widget new_w,
 		   (*bce)->secondaryObjectClass->core_class.num_resources,
 		   args, *num_args);
 
-    bcopy(nsec, rsec, size);
+    memcpy(rsec, nsec, size);
     ((XmExtRec *)rsec)->object.self = (Widget)rsec;
 
     LabG_Cache(new_w) = &(((XmLabelGCacheObject)nsec)->label_cache);
@@ -1494,7 +1498,7 @@ set_values(Widget old, Widget request, Widget new_w,
     Boolean refresh_needed = False, relayout_needed = False,
 	recalc_needed = False;
 
-    DEBUGOUT(XdbDebug(__FILE__, new_w, "LabelGadget SetValues %d %d\n",
+    DEBUGOUT(_LtDebugDebug(__FILE__, new_w, "LabelGadget SetValues %d %d\n",
 		      LabG_MarginRight(new_w), LabG_MarginRight(old)));
 
     /* Silently discard accelerator for non-menu labels and cascades */
@@ -1732,7 +1736,7 @@ set_values(Widget old, Widget request, Widget new_w,
 	}
 
 	/* Put the internal parts in the right places.
-	 * Since the size hasn't changed yes, use the old size.
+	 * Since the size hasn't changed yet, use the old size.
 	 * If it changes, resize will be called again by Xt.
 	 */
 
@@ -1755,7 +1759,7 @@ get_values_prehook(Widget new_w, ArgList args, Cardinal *num_args)
 {
     XmBaseClassExt *bce;
     XmWidgetExtData ed;
-    int size;
+    Cardinal size;
     XtPointer nsec;
 
     bce = _XmGetBaseClassExtPtr(XtClass(new_w), XmQmotif);
@@ -1763,9 +1767,9 @@ get_values_prehook(Widget new_w, ArgList args, Cardinal *num_args)
 
     nsec = _XmExtObjAlloc(size);
 
-    bcopy(LabG_Cache(new_w),
-	  &((XmLabelGCacheObject)nsec)->label_cache,
-	  sizeof(XmLabelGCacheObjPart));
+    memcpy(&((XmLabelGCacheObject)nsec)->label_cache,
+           LabG_Cache(new_w),
+	   sizeof(XmLabelGCacheObjPart));
 
     /*
      * don't do this and ResInd will blow up.
@@ -1810,7 +1814,7 @@ get_values_posthook(Widget new_w, ArgList args, Cardinal *num_args)
 static void
 get_values_hook(Widget w, ArgList args, Cardinal *num_args)
 {
-    DEBUGOUT(XdbDebug(__FILE__, w, "LabelG: GetValuesHook\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "LabelG: GetValuesHook\n"));
 }
 #endif
 
@@ -1891,11 +1895,11 @@ expose(Widget w,
 	- ((G_ShadowThickness(w) + G_HighlightThickness(w)) << 1)
 	- LabG_MarginTop(w) - LabG_MarginBottom(w);
 
-    DEBUGOUT(XdbDebug(__FILE__, w,
+    DEBUGOUT(_LtDebugDebug(__FILE__, w,
              "Expose: cliprect %d %d %dx%d\n",
             cliprect.x, cliprect.y,
              cliprect.width, cliprect.height));
-    DEBUGOUT(XdbDebug(__FILE__, w,
+    DEBUGOUT(_LtDebugDebug(__FILE__, w,
              "Expose: acctextrect %d %d %dx%d\n",
             LabG_AccTextRect(w).x, LabG_AccTextRect(w).y,
              LabG_AccTextRect(w).width, LabG_AccTextRect(w).height));
@@ -1942,6 +1946,12 @@ expose(Widget w,
 	mypm = XtSensitive(w)
 	    ? LabG_Pixmap(w)
 	    : LabG_PixmapInsensitive(w);
+#if XmVERSION > 1
+	if (mypm == XmUNSPECIFIED_PIXMAP)
+	{
+	    mypm = LabG_Pixmap(w);
+	}
+#endif
 	cliprect.x -= XtX(w);
 	cliprect.y -= XtY(w);
 	if (mypm > XmUNSPECIFIED_PIXMAP &&
@@ -1972,6 +1982,63 @@ expose(Widget w,
 		cliprect.height = (LabG_TextRect_y(w)
 				   + LabG_TextRect_height(w)) - cliprect.y;
 	    }
+#if XmVERSION > 1
+	    {
+		unsigned int tmp, tmpw, tmph;
+		int tmpx, tmpy;
+		unsigned Depth;
+		Window tmpwin;
+
+		/* rws 4 Mar 2000
+		   According to the comments in label/test12, this does not
+		   happen. We should be adding the pixmap to our pixmap
+		   cache, and getting the info from there instead of a
+		   round trip server query.
+		 */
+		/* rws 10 Mar 2000
+		   Apparently a Motif 2 thing.
+		 */
+		XGetGeometry(XtDisplayOfObject(w),
+			     mypm,
+			     &tmpwin,
+			     &tmpx, &tmpy,
+			     &tmpw, &tmph,
+			     &tmp, &Depth);
+
+		if (Depth == 1)
+		{
+		    XCopyPlane(XtDisplay(w),
+			  mypm,
+			  XtWindow(w),
+			  myGC,
+			  cliprect.x - LabG_TextRect_x(w),
+			  cliprect.y - LabG_TextRect_y(w),
+			  cliprect.width, cliprect.height,
+			  cliprect.x + XtX(w), cliprect.y + XtY(w),
+			  1);
+		}
+		else if (Depth > 1)
+		{
+		    XCopyArea(XtDisplay(w),
+			  mypm,
+			  XtWindow(w),
+			  myGC,
+			  cliprect.x - LabG_TextRect_x(w),
+			  cliprect.y - LabG_TextRect_y(w),
+			  cliprect.width, cliprect.height,
+			  cliprect.x + XtX(w), cliprect.y + XtY(w));
+		}
+	    }
+	    if (!XtSensitive(w) && LabG_PixmapInsensitive(w) == XmUNSPECIFIED_PIXMAP)
+	    {
+	    	/* label/test14 */
+	    	XFillRectangle(XtDisplay(w), XtWindow(w),
+	    		myGC,
+		  cliprect.x,
+		  cliprect.y,
+		  cliprect.width, cliprect.height);
+	    }
+#else
 	    XCopyArea(XtDisplay(w),
 		      mypm,
 		      XtWindow(w),
@@ -1980,6 +2047,7 @@ expose(Widget w,
 		      cliprect.y - LabG_TextRect_y(w),
 		      cliprect.width, cliprect.height,
 		      cliprect.x + XtX(w), cliprect.y + XtY(w));
+#endif
 	}
     }
 }
@@ -2052,7 +2120,7 @@ query_geometry(Widget w, XtWidgetGeometry *proposed, XtWidgetGeometry *answer)
     a.width = XtWidth(w);
     a.height = XtHeight(w);
     a.request_mode = CWWidth | CWHeight;
-    DEBUGOUT(XdbDebug(__FILE__, w,
+    DEBUGOUT(_LtDebugDebug(__FILE__, w,
 		      "LabelGadget queried for size: reporting %d %d %08x\n",
 		      XtWidth(w), XtHeight(w), answer));
 
@@ -2173,11 +2241,11 @@ input_dispatch(Widget gadget,
     switch (event_mask)
     {
     case XmHELP_EVENT:
-	DEBUGOUT(XdbDebug(__FILE__, gadget, "LabelGadget got help event\n"));
+	DEBUGOUT(_LtDebugDebug(__FILE__, gadget, "LabelGadget got help event\n"));
 	break;
 
     case XmBDRAG_EVENT:
-	DEBUGOUT(XdbDebug(__FILE__, gadget, "LabelGadget got bdrag event\n"));
+	DEBUGOUT(_LtDebugDebug(__FILE__, gadget, "LabelGadget got bdrag event\n"));
 	_XmProcessDrag(gadget, event, NULL, NULL);
 	break;
     }
