@@ -1,8 +1,9 @@
 /**
  *
- * $Id: MainW.c,v 1.53 1999/09/11 20:02:14 rwscott Exp $
+ * $Id: MainW.c,v 1.61 2001/05/20 17:49:38 amai Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
+ * Copyright (C) 1995-2001 LessTif Development Team
  *
  * This file is part of the GNU LessTif Library.
  *
@@ -22,11 +23,14 @@
  *
  **/
 
-static const char rcsid[] = "$Id: MainW.c,v 1.53 1999/09/11 20:02:14 rwscott Exp $";
+static const char rcsid[] = "$Id: MainW.c,v 1.61 2001/05/20 17:49:38 amai Exp $";
 
 #include <LTconfig.h>
-#include <XmI/XmI.h>
+
 #include <stdio.h>
+#include <string.h>
+
+#include <XmI/XmI.h>
 #include <Xm/XmP.h>
 #include <Xm/ScrollBarP.h>
 #include <Xm/BaseClassP.h>
@@ -36,7 +40,6 @@ static const char rcsid[] = "$Id: MainW.c,v 1.53 1999/09/11 20:02:14 rwscott Exp
 #include <Xm/MainWP.h>
 #include <Xm/SeparatoG.h>
 #include <Xm/SeparatoGP.h>
-#include <X11/Xfuncs.h>
 
 #include <XmI/DebugUtil.h>
 
@@ -49,10 +52,8 @@ static void class_initialize();
 #endif
 
 static void class_part_initialize(WidgetClass w_class);
-
 static void initialize(Widget request, Widget new_w,
 		       ArgList args, Cardinal *num_args);
-
 static void resize(Widget w);
 
 #if 0
@@ -63,19 +64,13 @@ static void realize(Widget w, Mask *value_mask,
 static XtGeometryResult query_geometry(Widget w,
 				       XtWidgetGeometry *proposed,
 				       XtWidgetGeometry *answer);
-
 static Boolean set_values(Widget current, Widget request, Widget new_w,
 			  ArgList args, Cardinal *num_args);
-
 static XtGeometryResult geometry_manager(Widget w,
 					 XtWidgetGeometry *request,
 					 XtWidgetGeometry *reply);
-
 static void change_managed(Widget w);
-
 static void insert_child(Widget w);
-
-
 static void _XmMainWindowPreferredSize(Widget w, Widget child,
 				       XtWidgetGeometry *cg, XmMWValues * vals);
 static XtGeometryResult _XmMainWindowGeomRequest(Widget w, XmMWValues * vals);
@@ -154,7 +149,6 @@ static XmSyntheticResource syn_resources[] =
 };
 
 /* Add Actions and Translations -- FIX ME */
-/* *INDENT-OFF* */
 #if 0
 static XmBaseClassExtRec _XmMainWindowCoreClassExtRec = {
     /* next_extension            */ NULL,
@@ -264,7 +258,6 @@ XmMainWindowClassRec xmMainWindowClassRec = {
 	/* extension */ NULL,	
     },
 };
-/* *INDENT-ON* */
 
 
 WidgetClass xmMainWindowWidgetClass = (WidgetClass)&xmMainWindowClassRec;
@@ -393,7 +386,7 @@ set_values(Widget old, Widget request, Widget new_w,
     Boolean relayout = False;
 #define NE(x)	(x(old) != x(new_w))
 
-    DEBUGOUT(XdbDebug(__FILE__, new_w,
+    DEBUGOUT(_LtDebugDebug(__FILE__, new_w,
 		      "%s:set_values(%d) - %i args\n"
 		      "\t    old X %5i Y %5i W %5i H %5i\n"
 		      "\trequest X %5i Y %5i W %5i H %5i\n"
@@ -406,7 +399,7 @@ set_values(Widget old, Widget request, Widget new_w,
 		      XtWidth(request), XtHeight(request),
 		      XtX(new_w), XtY(new_w),
 		      XtWidth(new_w), XtHeight(new_w)));
-    DEBUGOUT(XdbPrintArgList(__FILE__, new_w, args, *num_args, False));
+    DEBUGOUT(_LtDebugPrintArgList(__FILE__, new_w, args, *num_args, False));
 
     if (NE(MW_CommandLoc))
     {
@@ -417,7 +410,7 @@ set_values(Widget old, Widget request, Widget new_w,
 
     if (NE(SW_WorkWindow))
     {
-	DEBUGOUT(XdbDebug(__FILE__, new_w,
+	DEBUGOUT(_LtDebugDebug(__FILE__, new_w,
 			  "SetValues: Changed the work window !\n"));
 
 	if (SW_ClipWindow(new_w))
@@ -447,7 +440,7 @@ set_values(Widget old, Widget request, Widget new_w,
 
     if (NE(MW_CommandWindow))
     {
-	DEBUGOUT(XdbDebug(__FILE__, new_w,
+	DEBUGOUT(_LtDebugDebug(__FILE__, new_w,
 			  "SetValues: Changed the command window !\n"));
 
 	if (SW_VisualPolicy(new_w) == XmCONSTANT &&
@@ -475,7 +468,7 @@ set_values(Widget old, Widget request, Widget new_w,
 
     if (NE(MW_MessageWindow))
     {
-	DEBUGOUT(XdbDebug(__FILE__, new_w,
+	DEBUGOUT(_LtDebugDebug(__FILE__, new_w,
 			  "SetValues: Changed the message window !\n"));
 
 	if (SW_VisualPolicy(new_w) == XmCONSTANT &&
@@ -503,7 +496,7 @@ set_values(Widget old, Widget request, Widget new_w,
 
     if (NE(MW_MenuBar))
     {
-	DEBUGOUT(XdbDebug(__FILE__, new_w,
+	DEBUGOUT(_LtDebugDebug(__FILE__, new_w,
 			  "SetValues: Changed the menu bar !\n"));
 
 	if (SW_VisualPolicy(new_w) == XmCONSTANT &&
@@ -597,7 +590,7 @@ resize(Widget w)
 {
     XmMWValues vals;
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "%s:resize(%d) (%dx%d)\n",
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "%s:resize(%d) (%dx%d)\n",
 		      __FILE__, __LINE__,
 		      XtWidth(w), XtHeight(w)));
 
@@ -626,7 +619,7 @@ realize(Widget w, Mask *value_mask, XSetWindowAttributes *attributes)
        XmMWValues vals;
      */
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "Realize ...\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "Realize ...\n"));
 
 #define superclass (&xmManagerClassRec)
     (*superclass->core_class.realize) (w, value_mask, attributes);
@@ -654,7 +647,7 @@ realize(Widget w, Mask *value_mask, XSetWindowAttributes *attributes)
 
     _XmMainWindowConfigureChildren(w, NULL, NULL, &vals);
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "Realize => size %d %d\n",
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "Realize => size %d %d\n",
 		      XtWidth(w), XtHeight(w)));
 #endif
 }
@@ -734,7 +727,7 @@ CreateManagedList(Widget w, Widget **managed, int *num_managed,
 {
     if (XtIsComposite(w) && MGR_NumChildren(w) > 0)
     {
-	int i;
+	Cardinal i;
 	for (i = 0; i < MGR_NumChildren(w); i++)
 	{
 	    CreateManagedList(MGR_Children(w)[i], managed, num_managed,
@@ -748,7 +741,7 @@ CreateManagedList(Widget w, Widget **managed, int *num_managed,
 	    /* Allocate more space */
 	    *num_allocated += (*num_allocated / 2) + 2;
 	    *managed =
-		(WidgetList)XtRealloc((XtPointer)*managed,
+		(WidgetList)XtRealloc((char *)*managed,
 				      (unsigned)(*num_allocated *
 						 sizeof(Widget)));
 	}
@@ -761,7 +754,8 @@ ReparentChild(Widget w, Widget child)
 {
 
     Widget old_parent = XtParent(child), *managed;
-    int i, j, num_managed, num_allocated;
+    Cardinal i, j;
+    int num_managed, num_allocated;
     Boolean is_realized;
 
     if (old_parent == w)
@@ -785,7 +779,7 @@ ReparentChild(Widget w, Widget child)
 	/* Allocate more space */
 	MGR_NumSlots(w) += (MGR_NumSlots(w) / 2) + 2;
 	MGR_Children(w) =
-	    (WidgetList)XtRealloc((XtPointer)MGR_Children(w),
+	    (WidgetList)XtRealloc((char *)MGR_Children(w),
 				  (unsigned)(MGR_NumSlots(w) *
 					     sizeof(Widget)));
     }
@@ -820,7 +814,7 @@ change_managed(Widget w)
 {
     XmMWValues vals;
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "ChangeManaged\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "ChangeManaged\n"));
     /*
      * It is possible that some of the MainWindows children is made children
      * of ClipWindow by the ScrolledWindow. Steal them back.
@@ -830,7 +824,7 @@ change_managed(Widget w)
     {
 	XmDrawingAreaWidget cw = SW_ClipWindow(w);
 	Widget mw_children[3];
-	int i, num_mw_children = 0;
+	Cardinal i, num_mw_children = 0;
 
 	for (i = 0; i < MGR_NumChildren(cw); i++)
 	{
@@ -965,7 +959,7 @@ _XmMainWindowPreferredSize(Widget w, Widget child, XtWidgetGeometry *cg,
 
     int i, nchld, nsep = 0;
 
-    bzero(vals, sizeof(XmMWValues));
+    memset(vals, 0, sizeof(XmMWValues));
 
     /* ignore the instigator if resizing */
     if (SW_FromResize(w))
@@ -1068,8 +1062,8 @@ _XmMainWindowPreferredSize(Widget w, Widget child, XtWidgetGeometry *cg,
      */
 
     /* Talk */
-    DEBUGOUT(XdbDebug2(__FILE__, w, child, "_XmMainWindowPreferredSize request %s\n",
-		       XdbWidgetGeometry2String(cg)));
+    DEBUGOUT(_LtDebugDebug2(__FILE__, w, child, "_XmMainWindowPreferredSize request %s\n",
+		       _LtDebugWidgetGeometry2String(cg)));
 
     /* T. Straumann: do querying the kids using a loop (I'm lazy) */
 
@@ -1158,7 +1152,7 @@ _XmMainWindowPreferredSize(Widget w, Widget child, XtWidgetGeometry *cg,
 		Dimension bw = XtBorderWidth(chld[i]);
 		XtWidgetGeometry rcg, rcg_r;
 
-		DEBUGOUT(XdbDebug2(__FILE__, w, chld[i],
+		DEBUGOUT(_LtDebugDebug2(__FILE__, w, chld[i],
 		       "_XmMainWindowPreferredSize: %s current wid %d ht %d\n",
 			 chldwh[i].name, XtWidth(chld[i]), XtHeight(chld[i])));
 
@@ -1192,14 +1186,14 @@ _XmMainWindowPreferredSize(Widget w, Widget child, XtWidgetGeometry *cg,
 		    {
 			chldwh[i].w = cg->width;
 			childNotHappy = True;
-			DEBUGOUT(XdbDebug0(__FILE__, w, "\tbut it wants width %d\n",
+			DEBUGOUT(_LtDebugDebug0(__FILE__, w, "\tbut it wants width %d\n",
 					   cg->width));
 		    }
 		    if (Wants(CWHeight) && chldwh[i].h != cg->height)
 		    {
 			chldwh[i].h = cg->height;
 			childNotHappy = True;
-			DEBUGOUT(XdbDebug0(__FILE__, w, "\tbut it wants height %d\n",
+			DEBUGOUT(_LtDebugDebug0(__FILE__, w, "\tbut it wants height %d\n",
 					   cg->height));
 		    }
 		    /* T. Straumann: allow changes of border width */
@@ -1213,7 +1207,7 @@ _XmMainWindowPreferredSize(Widget w, Widget child, XtWidgetGeometry *cg,
 		chldwh[i].w += 2 * bw;
 		chldwh[i].h += 2 * bw;
 
-		DEBUGOUT(XdbDebug2(__FILE__, w, chld[i],
+		DEBUGOUT(_LtDebugDebug2(__FILE__, w, chld[i],
 				   "%s w %d h %d\n",
 				   chldwh[i].name, chldwh[i].w, chldwh[i].h));
 
@@ -1375,10 +1369,10 @@ _XmMainWindowGeomRequest(Widget w, XmMWValues * vals)
     geo.height = vals->MwH;
     geo.request_mode = CWWidth | CWHeight;
 
-    DEBUGOUT(XdbDebug(__FILE__, w,
+    DEBUGOUT(_LtDebugDebug(__FILE__, w,
 		      "_XmMainWindowGeomRequests: request geo %s: "
 		      "am %d %d given: %d %d\n",
-		      XdbWidgetGeometry2String(&geo),
+		      _LtDebugWidgetGeometry2String(&geo),
 		      XtWidth(w), XtHeight(w),
 		      SW_GivenWidth(w), SW_GivenHeight(w)));
 
@@ -1392,9 +1386,9 @@ _XmMainWindowGeomRequest(Widget w, XmMWValues * vals)
 	vals->MwW = XtWidth(w);
 	vals->MwH = XtHeight(w);
 
-	DEBUGOUT(XdbDebug(__FILE__, w,
+	DEBUGOUT(_LtDebugDebug(__FILE__, w,
 			  "_XmMainWindowGeomRequests CONF got %s\n",
-			  XdbWidgetGeometry2String(&geo)));
+			  _LtDebugWidgetGeometry2String(&geo)));
     }
 
     return res;
@@ -1436,7 +1430,7 @@ _XmMainWindowLayout(Widget w, Widget child, XtWidgetGeometry *cg,
 
     if (MW_CommandLoc(w) == XmCOMMAND_ABOVE_WORKSPACE)
     {
-	DEBUGOUT(XdbDebug(__FILE__, w,
+	DEBUGOUT(_LtDebugDebug(__FILE__, w,
 			  "_XmMainWindowLayout COMMAND_ABOVE_WORKSPACE\n"));
 	if (VALID(MW_CommandWindow(w)))
 	{
@@ -1489,7 +1483,7 @@ _XmMainWindowLayout(Widget w, Widget child, XtWidgetGeometry *cg,
     }
     else
     {				/* XmCOMMAND_BELOW_WORKSPACE */
-	DEBUGOUT(XdbDebug(__FILE__, w,
+	DEBUGOUT(_LtDebugDebug(__FILE__, w,
 			  "_XmMainWindowLayout COMMAND_BELOW_WORKSPACE\n"));
 	/*
 	 * Start working from the bottom immediately, eventually getting
@@ -1602,8 +1596,8 @@ _XmMainWindowLayout(Widget w, Widget child, XtWidgetGeometry *cg,
 	    cg->request_mode = CWWidth | CWHeight | CWX | CWY;
 	}
 
-	DEBUGOUT(XdbDebug2(__FILE__, w, child, "feedback => %s\n",
-			   XdbWidgetGeometry2String(cg)));
+	DEBUGOUT(_LtDebugDebug2(__FILE__, w, child, "feedback => %s\n",
+			   _LtDebugWidgetGeometry2String(cg)));
     }
 }
 
@@ -1621,7 +1615,7 @@ _XmMainWindowLayout(Widget w, Widget child, XtWidgetGeometry *cg,
  */
 #define	CONF(wid, xx, yy, ww, hh, child, cg)				\
     {								\
-	DEBUGOUT(XdbDebug2(__FILE__, w, wid,                    \
+	DEBUGOUT(_LtDebugDebug2(__FILE__, w, wid,                    \
                            "CONF: x %d y %d w %d h %d\n",   \
 			   xx, yy, ww, hh));	\
 	if (child && wid == child)		\
@@ -1691,9 +1685,9 @@ geometry_manager(Widget w,
     XtWidgetGeometry wants;
     XtGeometryResult rval;
 
-    DEBUGOUT(XdbDebug2(__FILE__, XtParent(w), w,
+    DEBUGOUT(_LtDebugDebug2(__FILE__, XtParent(w), w,
 		       "geometry_manager request %s\n",
-		       XdbWidgetGeometry2String(desired)));
+		       _LtDebugWidgetGeometry2String(desired)));
 
 #define Wants(flag)     (wants.request_mode & flag)
 
@@ -1701,7 +1695,7 @@ geometry_manager(Widget w,
 
     if (Wants(XtCWQueryOnly))
     {
-	DEBUGOUT(XdbDebug(__FILE__, w,
+	DEBUGOUT(_LtDebugDebug(__FILE__, w,
 			  "Geometry Mgr Query Only Unimplemented\n"));
 	return XtGeometryYes;
     }
@@ -1714,17 +1708,17 @@ geometry_manager(Widget w,
      */
     if (SW_ScrollPolicy(mw) == XmAUTOMATIC && w == (Widget)SW_ClipWindow(mw))
     {
-	DEBUGOUT(XdbDebug2(__FILE__, mw, w,
+	DEBUGOUT(_LtDebugDebug2(__FILE__, mw, w,
 			   "BEGIN AUTOMATIC FAKE\n\n"));
 
-	DEBUGOUT(XdbDebug2(__FILE__, mw, w,
+	DEBUGOUT(_LtDebugDebug2(__FILE__, mw, w,
 			   "geometry_manager: resize WorkWindow: %d %d\n",
 			   wants.width, wants.height));
 	/* bypass the clipwindow; the _real_ instigator is the work window */
 	_XmMainWindowPreferredSize(mw, SW_WorkWindow(mw), &wants, &vals);
 	_XmMainWindowLayout(mw, SW_WorkWindow(mw), &wants, &vals);
 	_XmMainWindowConfigureChildren(mw, SW_WorkWindow(mw), &wants, &vals);
-	DEBUGOUT(XdbDebug2(__FILE__, mw, w, "END AUTOMATIC FAKE\n"));
+	DEBUGOUT(_LtDebugDebug2(__FILE__, mw, w, "END AUTOMATIC FAKE\n"));
 	/* consult ScrolledW.c about this return value */
 	return XtGeometryNo;
     }
@@ -1750,10 +1744,16 @@ geometry_manager(Widget w,
     /*
      * Try this : if we have a difference, just report "Almost"
      */
-#define DO_RETURN_ALMOST
+#undef DO_RETURN_ALMOST
     /* till 26 may 1999; re-enabled the `almost' return value.
      * This fixed `thot' and I couldn't observe anything strange
-     * happening whith grace 5.0.2
+     * happening with grace 5.0.2
+     * amai 20010228: well, situation is that #undefining it
+     * makes emacs 20.x to start up at least instead of an
+     * (almost, finished by crash probably) endless loop.
+     * Grace 5.1.x (CVS) seems to be still happy, I played with the
+     * Zoom function - no problem showed up. I haven't checked 
+     * Thot so far.
      */
 #ifdef DO_RETURN_ALMOST
     /* rws 20 Nov 1998
@@ -1807,15 +1807,31 @@ geometry_manager(Widget w,
     }
     else if (rval == XtGeometryAlmost)
     {
-	DEBUGOUT(XdbDebug2(__FILE__, mw, w,
+	DEBUGOUT(_LtDebugDebug2(__FILE__, mw, w,
 		       "geometry_manager : child wants %s gets %s => Almost\n",
-			   XdbWidgetGeometry2String(&wants),
-			   XdbWidgetGeometry2String(desired)));
+			   _LtDebugWidgetGeometry2String(&wants),
+			   _LtDebugWidgetGeometry2String(desired)));
     }
 
     /* Strip off unwanted bits */
 
     allowed->request_mode &= wants.request_mode;
+    if (allowed->request_mode & CWX && desired->x == allowed->x)
+    {
+    	allowed->request_mode &= ~CWX;
+    }
+    if (allowed->request_mode & CWY && desired->y == allowed->y)
+    {
+    	allowed->request_mode &= ~CWY;
+    }
+    if (allowed->request_mode & CWWidth && desired->width == allowed->width)
+    {
+    	allowed->request_mode &= ~CWWidth;
+    }
+    if (allowed->request_mode & CWHeight && desired->height == allowed->height)
+    {
+    	allowed->request_mode &= ~CWHeight;
+    }
     return rval;
 #undef	Wants
 }
@@ -1863,20 +1879,20 @@ XmMainWindowSetAreas(Widget widget,
     Arg args[5];
     int n = 0;
 
-    DEBUGOUT(XdbDebug(__FILE__, widget, "XmMainWindowSetAreas ["));
+    DEBUGOUT(_LtDebugDebug(__FILE__, widget, "XmMainWindowSetAreas ["));
 
 #define	P(cw, t)						\
     if (cw)							\
-	DEBUGOUT(XdbDebug0(__FILE__, widget, t, XtName(cw)));	\
+	{ DEBUGOUT(_LtDebugDebug0(__FILE__, widget, t, XtName(cw))); }\
     else							\
-	DEBUGOUT(XdbDebug0(__FILE__, widget, t, ": NULL"));
+	{ DEBUGOUT(_LtDebugDebug0(__FILE__, widget, t, ": NULL")); }
 
     P(menu_bar, " MenuBar %s");
     P(command_window, " CommandWindow %s");
     P(horizontal_scrollbar, " Hor.Scrollbar %s");
     P(vertical_scrollbar, " Vert.Scrollbar %s");
     P(work_region, " WorkRegion %s");
-    DEBUGOUT(XdbDebug0(__FILE__, widget, "]\n"));
+    DEBUGOUT(_LtDebugDebug0(__FILE__, widget, "]\n"));
 
 #if 1
     if (menu_bar)
@@ -2011,13 +2027,13 @@ insert_child(Widget w)
 
 	if (XmIsRowColumn(w) && RC_Type(w) == XmMENU_BAR)
 	{
-	    DEBUGOUT(XdbDebug2(__FILE__, p, w,
+	    DEBUGOUT(_LtDebugDebug2(__FILE__, p, w,
 			       "insert_child : this is the menu bar\n"));
 	    MW_MenuBar(p) = w;
 	}
 	else if (XmIsSeparator(w) || XmIsSeparatorGadget(w))
 	{
-	    DEBUGOUT(XdbDebug2(__FILE__, p, w,
+	    DEBUGOUT(_LtDebugDebug2(__FILE__, p, w,
 			       "insert_child : this is a separator\n"));
 	}
 	else if (MW_MessageWindow(p) == NULL &&
@@ -2028,7 +2044,7 @@ insert_child(Widget w)
 		 w != (Widget)SW_HSB(p) &&
 		 w != (Widget)SW_VSB(p))
 	{
-	    DEBUGOUT(XdbDebug2(__FILE__, p, w,
+	    DEBUGOUT(_LtDebugDebug2(__FILE__, p, w,
 			       "insert_child : this is the message window\n"));
 	    MW_MessageWindow(p) = w;
 	}
@@ -2051,7 +2067,7 @@ insert_child(Widget w)
 	    w != (Widget)SW_HSB(p) &&
 	    w != (Widget)SW_VSB(p))
 	{
-	    DEBUGOUT(XdbDebug2(__FILE__, p, w,
+	    DEBUGOUT(_LtDebugDebug2(__FILE__, p, w,
 			       "insert_child : this is the message window\n"));
 	    MW_MessageWindow(p) = w;
 	}
