@@ -1,8 +1,9 @@
 /**
  *
- * $Id: Manager.c,v 1.80 2000/01/13 17:20:25 rwscott Exp $
+ * $Id: Manager.c,v 1.95 2001/05/20 17:49:38 amai Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
+ * Copyright (C) 1995-2001 LessTif Development Team
  *
  * This file is part of the GNU LessTif Library.
  *
@@ -22,15 +23,13 @@
  *
  **/
 
-static const char rcsid[] = "$Id: Manager.c,v 1.80 2000/01/13 17:20:25 rwscott Exp $";
+static const char rcsid[] = "$Id: Manager.c,v 1.95 2001/05/20 17:49:38 amai Exp $";
 
 #include <LTconfig.h>
 
 #include <stdio.h>
 #include <stdlib.h>
-#ifdef HAVE_STRINGS_H
-#include <strings.h>
-#endif
+#include <string.h>
 
 #include <XmI/XmI.h>
 #include <XmI/TraversalI.h>
@@ -42,11 +41,8 @@ static const char rcsid[] = "$Id: Manager.c,v 1.80 2000/01/13 17:20:25 rwscott E
 #include <Xm/RepType.h>
 #include <Xm/TransltnsP.h>
 #include <X11/keysym.h>
-#include <X11/Xfuncs.h>
 #include <Xm/VirtKeysP.h>
 
-#include <XmI/DebugUtil.h>
-
 /* For determining class type */
 #include <Xm/CascadeBP.h>
 #include <Xm/CascadeBGP.h>
@@ -76,6 +72,9 @@ static XmSpecifyLayoutDirectionTraitRec _XmManagerLayoutTraitRec =
 
 #endif
 
+#include <XmI/DebugUtil.h>
+
+
 /* Forward Declarations */
 
 static void class_initialize(void);
@@ -336,15 +335,12 @@ static XtResource resources[] =
     }
 
 #if XmVERSION > 1
-#if LESSTIF_BACK_COMPAT
-#else
     ,
     {
 	XmNpopupHandlerCallback, XmCCallback, XmRCallback,
 	sizeof(XtCallbackList), Offset(popup_handler_callback),
 	XtRPointer, (XtPointer)NULL
     }
-#endif
     ,
     {
 	XmNlayoutDirection, XmCLayoutDirection, XmRDirection,
@@ -399,7 +395,7 @@ static XmSyntheticResource syn_resources[] =
     },
 #endif
 };
-/* *INDENT-OFF* */
+
 static XmBaseClassExtRec _XmManagerCoreClassExtRec = {
     /* next_extension            */ NULL,
     /* record_type               */ NULLQUARK,                             
@@ -511,7 +507,7 @@ XmManagerClassRec xmManagerClassRec = {
 	/* extension                    */ (XtPointer)&_XmManagerClassExtRec
     },
 };
-/* *INDENT-ON* */
+
 
 
 
@@ -637,8 +633,8 @@ class_part_initialize(WidgetClass widget_class)
 #if XmVERSION > 1
     /* FIX ME: do I need to worry about subclasses */
     /* initialize traits */
-    if (!XmeTraitSet(xmManagerWidgetClass, XmQTspecifyLayoutDirection,
-		     &_XmManagerLayoutTraitRec))
+    if (!XmeTraitSet((XtPointer)xmManagerWidgetClass, XmQTspecifyLayoutDirection,
+		     (XtPointer)&_XmManagerLayoutTraitRec))
     {
 	_XmWarning(NULL, "XmManager ClassInitialize: XmeTraitSet failed\n");
     }
@@ -649,7 +645,7 @@ static void
 CreateHighlightGC(Widget mw)
 {
     XGCValues values;
-    unsigned long mask;
+    unsigned long mask,dynamic,unused;
 
     mask = GCForeground | GCBackground;
     values.foreground = MGR_HighlightColor(mw);
@@ -671,7 +667,15 @@ CreateHighlightGC(Widget mw)
     values.join_style = JoinMiter;
     values.cap_style = CapButt;
 
+    /* these may be changed in _XmDrawHighlight() called from
+     * gadget_border_highlight() */
+    dynamic = GCLineWidth | GCLineStyle | GCJoinStyle | GCDashList;
+
+    /* have no idea ! */
+    unused = 0;
+
     MGR_HighlightGC(mw) = XtGetGC((Widget)mw, mask, &values);
+
 }
 
 static void
@@ -845,13 +849,17 @@ initialize(Widget request,
 static void
 destroy(Widget w)
 {
-    DEBUGOUT(XdbDebug("RWS", w, "%s:destroy(%d)\n",
+    DEBUGOUT(_LtDebugDebug("RWS", w, "%s:destroy(%d)\n",
 		      __FILE__, __LINE__
 	     ));
     XtReleaseGC(w, MGR_TopShadowGC(w));
     XtReleaseGC(w, MGR_BottomShadowGC(w));
     XtReleaseGC(w, MGR_HighlightGC(w));
     XtReleaseGC(w, MGR_BackgroundGC(w));
+    if (MGR_KeyboardList(w))
+    {
+        XtFree((char *)MGR_KeyboardList(w));
+    }
 
     if (MGR_EventHandlerAdded(w))
     {
@@ -871,7 +879,7 @@ realize(Widget w, XtValueMask *value_mask, XSetWindowAttributes *attributes)
        This causes a parse error with gcc on AIX???
        Boolean wz = False, hz = False; */
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "%s:realize(%d) - %dx%d\n",
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "%s:realize(%d) - %dx%d\n",
 		      __FILE__, __LINE__,
 		      XtWidth(w), XtHeight(w)));
 
@@ -920,7 +928,7 @@ set_values(Widget old,
 {
     Boolean need_refresh = False;
 
-    DEBUGOUT(XdbDebug(__FILE__, new_w, "SetValues()\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, new_w, "SetValues()\n"));
 
 #if 0
     if (XtWidth(new_w) != XtWidth(old) && XtWidth(new_w) == 0)
@@ -1025,7 +1033,7 @@ geometry_manager(Widget w,
 		 XtWidgetGeometry *reply)
 {
     /* Motif does not have this method */
-    DEBUGOUT(XdbDebug(__FILE__, w, "geometry_manager()\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "geometry_manager()\n"));
 
     if (reply)
     {
@@ -1061,7 +1069,7 @@ changed_managed(Widget w)
 {
     /* Motif does not have this method */
     /* FIX ME */
-    DEBUGOUT(XdbDebug(__FILE__, w, "ChangeManaged()\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "ChangeManaged()\n"));
 
 
     _XmNavigChangeManaged(w);
@@ -1089,7 +1097,7 @@ constraint_initialize(Widget request,
 		|| G_EventMask(new_w) & XmENTER_EVENT
 		|| G_EventMask(new_w) & XmLEAVE_EVENT)
 	    {
-		DEBUGOUT(XdbDebug(__FILE__, manager, "adding event handler\n"));
+		DEBUGOUT(_LtDebugDebug(__FILE__, manager, "adding event handler\n"));
 
 		XtAddEventHandler(manager,
 				  PointerMotionMask | EnterWindowMask |
@@ -1112,7 +1120,7 @@ _XmGadgetArm(Widget w,
     XButtonEvent *ev = (XButtonEvent *)event;
     Widget g = (Widget)_XmInputForGadget(w, ev->x, ev->y);
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "Inside _XmGadgetArm()\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "Inside _XmGadgetArm()\n"));
 
     if (g)
     {
@@ -1141,7 +1149,7 @@ _XmGadgetActivate(Widget w,
 {
     Widget g = (Widget)MGR_SelectedGadget(w);
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "Inside _XmGadgetActivate()\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "Inside _XmGadgetActivate()\n"));
 
     if (g)
     {
@@ -1162,7 +1170,7 @@ _XmGadgetMultiArm(Widget w,
     XButtonEvent *ev = (XButtonEvent *)event;
     Widget g = (Widget)_XmInputForGadget(w, ev->x, ev->y);
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "Inside _XmGadgetMultiArm()\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "Inside _XmGadgetMultiArm()\n"));
 
     if (g)
     {
@@ -1204,7 +1212,7 @@ _XmGadgetMultiActivate(Widget w,
 {
     Widget g = (Widget)MGR_SelectedGadget(w);
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "Inside _XmGadgetMultiActivate()\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "Inside _XmGadgetMultiActivate()\n"));
 
     if (g)
     {
@@ -1239,7 +1247,7 @@ _XmGadgetDrag(Widget w,
     XButtonEvent *ev = (XButtonEvent *)event;
     Widget g = (Widget)_XmInputForGadget(w, ev->x, ev->y);
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "Inside _XmGadgetDrag()\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "Inside _XmGadgetDrag()\n"));
 
     if (g)
     {
@@ -1257,7 +1265,7 @@ _XmGadgetSelect(Widget w,
 {
     Widget g = MGR_ActiveChild(w);
 
-    DEBUGOUT(XdbDebug2(__FILE__, w, g, "Inside _XmGadgetSelect()\n"));
+    DEBUGOUT(_LtDebugDebug2(__FILE__, w, g, "Inside _XmGadgetSelect()\n"));
 
     if (g && XmIsGadget(g))
     {
@@ -1279,7 +1287,7 @@ _XmManagerParentActivate(Widget w,
     XmParentProcessDataRec data;
     XmManagerWidgetClass mwc = (XmManagerWidgetClass)w->core.widget_class;
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "Inside ManagerParentActivate()\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "Inside ManagerParentActivate()\n"));
 
     data.input_action.process_type = XmINPUT_ACTION;
     data.input_action.event = event;
@@ -1359,7 +1367,7 @@ _XmGadgetTraverseRight(Widget w,
 {
     Widget g = MGR_ActiveChild(w);
 
-    DEBUGOUT(XdbDebug(__FILE__, g,
+    DEBUGOUT(_LtDebugDebug(__FILE__, g,
 		      "Manager XmProcessTraversal(TRAVERSE_RIGHT)\n"));
 
     XmProcessTraversal(g, XmTRAVERSE_RIGHT);
@@ -1386,7 +1394,7 @@ _XmGadgetHelp(Widget w,
     Widget cur;
     XmAnyCallbackStruct cbs;
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "Inside _XmGadgetHelp()\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "Inside _XmGadgetHelp()\n"));
 
     if (g)
     {
@@ -1431,8 +1439,8 @@ _XmGadgetKeyInput(Widget w,
 {
     Widget g = MGR_ActiveChild(w);
 
-    DEBUGOUT(XdbDebug2(__FILE__, w, g, "Inside _XmGadgetKeyInput()\n"));
-    DEBUGOUT(XdbDebug2(__FILE__, w, g,
+    DEBUGOUT(_LtDebugDebug2(__FILE__, w, g, "Inside _XmGadgetKeyInput()\n"));
+    DEBUGOUT(_LtDebugDebug2(__FILE__, w, g,
 		       "event type: %d code %08x\n",
 		       event->type, event->xkey.keycode));
 
@@ -1452,7 +1460,7 @@ _XmGadgetButtonMotion(Widget w,
 {
     Widget g = MGR_ActiveChild(w);
 
-    DEBUGOUT(XdbDebug(__FILE__, w,
+    DEBUGOUT(_LtDebugDebug(__FILE__, w,
 		      "Inside _XmGadgetButtonMotion(), gadget %s\n",
 		      g ? XtName(g) : "(null)"));
 
@@ -1473,7 +1481,7 @@ _XmManagerParentCancel(Widget w,
     XmParentProcessDataRec data;
     XmManagerWidgetClass mwc = (XmManagerWidgetClass)w->core.widget_class;
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "ManagerParentCancel\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "ManagerParentCancel\n"));
 
     data.input_action.process_type = XmINPUT_ACTION;
     data.input_action.event = event;
@@ -1518,7 +1526,7 @@ _XmSocorro(Widget w, XEvent *event, String *params, Cardinal *num_params)
 {
     XmAnyCallbackStruct cbs;
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "Inside _XmSocorro()\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "Inside _XmSocorro()\n"));
 
     cbs.reason = XmCR_HELP;
     cbs.event = event;
@@ -1567,12 +1575,12 @@ _XmManagerEventHandler(Widget w,
     g = _XmInputForGadget(w, event->xmotion.x, event->xmotion.y);
     if (g == NULL)
     {
-	DEBUGOUT(XdbDebug(__FILE__, w,
+	DEBUGOUT(_LtDebugDebug(__FILE__, w,
 			  "### _XmManagerEventHandler(gadget NULL)\n"));
     }
     else
     {
-	DEBUGOUT(XdbDebug2(__FILE__, w,
+	DEBUGOUT(_LtDebugDebug2(__FILE__, w,
 			   (Widget)g, "### _XmManagerEventHandler()\n"));
     }
 
@@ -1612,7 +1620,7 @@ _XmManagerEventHandler(Widget w,
 
 	if (MGR_HighlightedWidget(w) != NULL)
 	{
-	    DEBUGOUT(XdbDebug(__FILE__, w, "Sending LEAVE event to gadget\n"));
+	    DEBUGOUT(_LtDebugDebug(__FILE__, w, "Sending LEAVE event to gadget\n"));
 	    _XmDispatchGadgetInput(MGR_HighlightedWidget(w),
 				   event,
 				   XmLEAVE_EVENT);
@@ -1635,7 +1643,7 @@ _XmManagerEventHandler(Widget w,
 
 	if (MGR_HighlightedWidget(w) == NULL)
 	{
-	    DEBUGOUT(XdbDebug(__FILE__, w, "Sending ENTER event to gadget\n"));
+	    DEBUGOUT(_LtDebugDebug(__FILE__, w, "Sending ENTER event to gadget\n"));
 
 	    _XmDispatchGadgetInput((Widget)g,
 				   event,
@@ -1648,14 +1656,14 @@ _XmManagerEventHandler(Widget w,
 	    if (MGR_HighlightedWidget(w) != (Widget)g)
 	    {
 
-		DEBUGOUT(XdbDebug(__FILE__, w,
+		DEBUGOUT(_LtDebugDebug(__FILE__, w,
 				  "Sending LEAVE event to old gadget\n"));
 
 		_XmDispatchGadgetInput(MGR_HighlightedWidget(w),
 				       event,
 				       XmLEAVE_EVENT);
 
-		DEBUGOUT(XdbDebug(__FILE__, w,
+		DEBUGOUT(_LtDebugDebug(__FILE__, w,
 				  "Sending ENTER event to new gadget\n"));
 
 		_XmDispatchGadgetInput((Widget)g,
@@ -1671,7 +1679,7 @@ _XmManagerEventHandler(Widget w,
 void
 _XmDestroyParentCallback(Widget w, XtPointer client, XtPointer call)
 {
-    DEBUGOUT(XdbDebug2("NEDIT", XtParent(w), w,
+    DEBUGOUT(_LtDebugDebug2("NEDIT", XtParent(w), w,
 		       "%s:_XmDestroyParentCallback(%d)\n",
 		       __FILE__, __LINE__));
     XtDestroyWidget(XtParent(w));
@@ -1739,12 +1747,12 @@ _XmAcceleratorHandler(Widget w, XtPointer client_data,
     XtActionProc act = NULL;
     Widget comp = (Widget)client_data;
 
-    DEBUGOUT(XdbDebug(__FILE__, w,
+    DEBUGOUT(_LtDebugDebug(__FILE__, w,
 		      "Got Accelerator key: type %d code %04x "
 		      "state %08x data %08x\n",
 		      event->type, event->xkey.keycode, event->xkey.state,
 		      client_data));
-    DEBUGOUT(XdbDebug("NMEM", w,
+    DEBUGOUT(_LtDebugDebug("NMEM", w,
 		      "Got Accelerator key: type %d code %04x "
 		      "state %08x data %08x\n",
 		      event->type, event->xkey.keycode, event->xkey.state,
@@ -1753,19 +1761,19 @@ _XmAcceleratorHandler(Widget w, XtPointer client_data,
     if (!XmIsManager(w) ||
 	(event->type != KeyPress && event->type != KeyRelease))
     {
-	DEBUGOUT(XdbDebug(__FILE__, w,
+	DEBUGOUT(_LtDebugDebug(__FILE__, w,
 			  "Bad match for event.  Not manager or bad event.\n"));
 	return;
     }
 
-    DEBUGOUT(XdbDebug(__FILE__, w,
+    DEBUGOUT(_LtDebugDebug(__FILE__, w,
 		    "Got Accelerator key: code: %08x state: %08x cd %08x %s\n",
-		      event->xkey.keycode, event->xkey.state, client_data,
-		      XtName(client_data)));
-    DEBUGOUT(XdbDebug("NMEM", w,
+		      event->xkey.keycode, event->xkey.state, comp,
+		      XtName(comp)));
+    DEBUGOUT(_LtDebugDebug("NMEM", w,
 		    "Got Accelerator key: code: %08x state: %08x cd %08x %s\n",
-		      event->xkey.keycode, event->xkey.state, client_data,
-		      XtName(client_data)));
+		      event->xkey.keycode, event->xkey.state, comp,
+		      XtName(comp)));
 
     /* JAC added this test.  Occasionally in nedit the accelerators cause
        segfaults.  The comp value passed has 0 for all fields in the
@@ -1792,7 +1800,7 @@ _XmAcceleratorHandler(Widget w, XtPointer client_data,
     if (!comp->core.widget_class)
     {
 	_XmWarning(NULL, "XmManager _XmAcceleratorHandler: bad news!!!\n");
-	DEBUGOUT(XdbDebug(__FILE__, w,
+	DEBUGOUT(_LtDebugDebug(__FILE__, w,
 			  "_XmAcceleratorHandler: called for w=%s"
 			  " with invalid comp, ignoring\n",
 			  XtName(w)));
@@ -1811,7 +1819,7 @@ _XmAcceleratorHandler(Widget w, XtPointer client_data,
      * --mu@trends.net, 1998.09.06 */
     if (!XtIsSensitive(comp))
     {
-	DEBUGOUT(XdbDebug(__FILE__, w,
+	DEBUGOUT(_LtDebugDebug(__FILE__, w,
 			"_XmAcceleratorHandler: %s is insensitive, ignoring\n",
 			  XtName(comp)));
 	return;
@@ -1819,14 +1827,15 @@ _XmAcceleratorHandler(Widget w, XtPointer client_data,
 
     for (i = 0; i < MGR_NumKeyboardEntries(w); i++)
     {
-	if (			/*MGR_KeyboardList(w)[i].eventType == event->type && */
-	       MGR_KeyboardList(w)[i].key == event->xkey.keycode &&
-	       MGR_KeyboardList(w)[i].modifiers == event->xkey.state &&
-	       MGR_KeyboardList(w)[i].component == client_data)
+	if (/*MGR_KeyboardList(w)[i].eventType == event->type && */
+	    MGR_KeyboardList(w)[i].key == event->xkey.keycode &&
+	    MGR_KeyboardList(w)[i].modifiers == event->xkey.state &&
+	    MGR_KeyboardList(w)[i].component == comp &&
+	    XtIsManaged(comp))
 	{
 	    Cardinal num_params = 0;
 
-	    DEBUGOUT(XdbDebug(__FILE__, w,
+	    DEBUGOUT(_LtDebugDebug(__FILE__, w,
 			      "MATCH!!!! "
 			      "code %08x state %08x name: %s:%s mnem: %d\n",
 			      event->xkey.keycode, event->xkey.state,
@@ -1835,7 +1844,7 @@ _XmAcceleratorHandler(Widget w, XtPointer client_data,
 			      : "(nil)",
 			      comp ? XtName(comp) : "(nil)",
 			      MGR_KeyboardList(w)[i].isMnemonic));
-	    DEBUGOUT(XdbDebug("NMEM", w,
+	    DEBUGOUT(_LtDebugDebug("NMEM", w,
 			      "MATCH!!!! "
 			      "code %08x state %08x name: %s:%s mnem: %d\n",
 			      event->xkey.keycode, event->xkey.state,
@@ -1847,35 +1856,57 @@ _XmAcceleratorHandler(Widget w, XtPointer client_data,
 
 	    /* Make sure XmGetPostedFromWidget works: ? */
 
+   /* amai: The following "if (act==NULL)" catches are just dumb
+            attempts to prevent some rare crashes, e.g. as seen with
+   	    NEdit. They really should be replaced by something better.
+   	    Actually the only confirmed crash location is the
+   	    first one, however I just decided to add a "full set"
+	    of checks here ...
+	    The _XmWarning()s should help us to get additional feedback
+	    on that issue!
+	    FIX ME ! */
+
 	    if (!MGR_KeyboardList(w)[i].isMnemonic || RC_Type(XtParent(comp)) == XmMENU_BAR ||
 		(MGR_KeyboardList(w)[i].isMnemonic && XtParent(comp) == RC_PopupPosted(w))
 		)
 	    {
-		DEBUGOUT(XdbDebug2(__FILE__, w, MGR_KeyboardList(w)[i].component,
+		DEBUGOUT(_LtDebugDebug2(__FILE__, w, MGR_KeyboardList(w)[i].component,
 				   "menu showing\n"));
-		DEBUGOUT(XdbDebug2("NMEM", w, MGR_KeyboardList(w)[i].component,
+		DEBUGOUT(_LtDebugDebug2("NMEM", w, MGR_KeyboardList(w)[i].component,
 				   "menu showing\n"));
 		if (XmIsPrimitive(MGR_KeyboardList(w)[i].component))
 		{
 		    act = PrimC_ArmAndActivate(XtClass(comp));
-
-		    (*act) (comp, event, NULL, &num_params);
+		    if (act==NULL) {
+                       _XmWarning(w, "%s:_XmAcceleratorHandler(%d) - act==NULL for %s", 
+		               __FILE__, __LINE__, comp->core.widget_class->core_class.class_name);
+		    } else {
+                       (*act) (comp, event, NULL, &num_params);
+		    }
 		}
 		else if (XmIsGadget(MGR_KeyboardList(w)[i].component))
 		{
 		    act = GC_ArmAndActivate(XtClass(comp));
-
-		    (*act) (comp, event, NULL, &num_params);
+		    if (act==NULL) {
+                       _XmWarning(w, "%s:_XmAcceleratorHandler(%d) - act==NULL for gadget", 
+		               __FILE__, __LINE__);
+	            } else {
+                       (*act) (comp, event, NULL, &num_params);
+		    }
 		}
 		else if (XmIsRowColumn(w))
 		{
 		    act = RCClass_ArmAndActivate(XtClass(comp));
-
-		    (*act) (comp, event, NULL, &num_params);
+		    if (act==NULL) {
+                       _XmWarning(w, "%s:_XmAcceleratorHandler(%d) - act==NULL for RowColumn", 
+		               __FILE__, __LINE__);
+		    } else {
+                       (*act) (comp, event, NULL, &num_params);
+		    }
 		}
 		else
 		{
-		    DEBUGOUT(XdbDebug(__FILE__, MGR_KeyboardList(w)[i].component,
+		    DEBUGOUT(_LtDebugDebug(__FILE__, MGR_KeyboardList(w)[i].component,
 				      "Not a widget/gadget\n"));
 
 		    XtCallActionProc(comp, "Activate", event, NULL, 0);
@@ -1885,9 +1916,9 @@ _XmAcceleratorHandler(Widget w, XtPointer client_data,
 	    }
 	    else
 	    {
-		DEBUGOUT(XdbDebug2(__FILE__, w, MGR_KeyboardList(w)[i].component,
+		DEBUGOUT(_LtDebugDebug2(__FILE__, w, MGR_KeyboardList(w)[i].component,
 				   "menu is not showing\n"));
-		DEBUGOUT(XdbDebug2("NMEM", w, MGR_KeyboardList(w)[i].component,
+		DEBUGOUT(_LtDebugDebug2("NMEM", w, MGR_KeyboardList(w)[i].component,
 				   "menu is not showing\n"));
 	    }
 
@@ -1921,14 +1952,14 @@ AddKeyboardEntry(Widget m, Widget w, XmKeyboardData *item)
 	{
 	    MGR_SizeKeyboardList(m) *= 2;
 	    MGR_KeyboardList(m) =
-		(XmKeyboardData *)XtRealloc((XtPointer)MGR_KeyboardList(m),
+		(XmKeyboardData *)XtRealloc((char *)MGR_KeyboardList(m),
 					    sizeof(XmKeyboardData) *
 					    MGR_SizeKeyboardList(m));
 	}
 
 	if (MGR_SizeKeyboardList(m) > i)
 	{
-	    bzero(&MGR_KeyboardList(m)[i],
+	    memset(&MGR_KeyboardList(m)[i], 0,
 		  (MGR_SizeKeyboardList(m) - i) * sizeof(XmKeyboardData));
 	}
     }
@@ -1942,16 +1973,16 @@ AddKeyboardEntry(Widget m, Widget w, XmKeyboardData *item)
 	_XmError(m, "This should not happen: %d\n", i);
     }
 
-    bcopy(item, &MGR_KeyboardList(m)[i], sizeof(XmKeyboardData));
+    memcpy(&MGR_KeyboardList(m)[i], item, sizeof(XmKeyboardData));
 
     MGR_NumKeyboardEntries(m)++;
 
-    DEBUGOUT(XdbDebug(__FILE__, m,
+    DEBUGOUT(_LtDebugDebug(__FILE__, m,
 		      "GrabKey: key %04x Sym: %04x modifiers %08x %s %s\n",
 		      item->key, item->keysym, item->modifiers,
 		      item->needGrab ? "Grabbing" : "Not Grabbing",
 		      item->isMnemonic ? "mnemonic" : "not mnemonic"));
-    DEBUGOUT(XdbDebug("NMEM", m,
+    DEBUGOUT(_LtDebugDebug("NMEM", m,
 		      "GrabKey: key %04x Sym: %04x modifiers %08x %s %s\n",
 		      item->key, item->keysym, item->modifiers,
 		      item->needGrab ? "Grabbing" : "Not Grabbing",
@@ -1997,18 +2028,19 @@ DeleteKeyboardEntry(Widget m, Widget w, Boolean isMnemonic)
 
 	    if (i < MGR_NumKeyboardEntries(m) - 1)
 	    {
-		bcopy(&MGR_KeyboardList(m)[i + 1],
-		      &MGR_KeyboardList(m)[i],
-		      sizeof(XmKeyboardData) *
+		memcpy(&MGR_KeyboardList(m)[i],
+                       &MGR_KeyboardList(m)[i + 1],
+		       sizeof(XmKeyboardData) *
 		        (MGR_NumKeyboardEntries(m) - i - 1));
 
-		bzero(&MGR_KeyboardList(m)[MGR_NumKeyboardEntries(m) - 1],
+		memset(&MGR_KeyboardList(m)[MGR_NumKeyboardEntries(m) - 1], 0,
 		      sizeof(XmKeyboardData));
 	    }
 	    /* end of list */
 	    else
 	    {
-		bzero(&MGR_KeyboardList(m)[i], sizeof(XmKeyboardData));
+		memset(&MGR_KeyboardList(m)[i], 0,
+		       sizeof(XmKeyboardData));
 	    }
 
 	    MGR_NumKeyboardEntries(m)--;
@@ -2026,9 +2058,9 @@ _XmManagerInstallAccelerator(Widget m, Widget w, String s)
     unsigned int modifiers = 0;
     XmKeyboardData data;
 
-    DEBUGOUT(XdbDebug2(__FILE__, m, w,
+    DEBUGOUT(_LtDebugDebug2(__FILE__, m, w,
 		       "_XmManagerInstallAccelerator(%s)\n", s));
-    DEBUGOUT(XdbDebug2("NMEM", m, w,
+    DEBUGOUT(_LtDebugDebug2("NMEM", m, w,
 		       "_XmManagerInstallAccelerator(%s)\n", s));
 
     /* Do we really have an accelerator ? */
@@ -2067,7 +2099,7 @@ _XmManagerInstallAccelerator(Widget m, Widget w, String s)
     }
 
 
-    DEBUGOUT(XdbDebug(__FILE__, w,
+    DEBUGOUT(_LtDebugDebug(__FILE__, w,
 		      "_XmManagerInstallAccelerator found manager %s\n",
 		      XtName(mw)));
 
@@ -2084,7 +2116,7 @@ _XmManagerInstallAccelerator(Widget m, Widget w, String s)
     data.needGrab = True;
     if (data.key == 0)
     {
-#if 0
+#if 1
     KeySym real_sym;
     Modifiers real_mod;
 
@@ -2149,8 +2181,8 @@ _XmManagerInstallMnemonic(Widget m, Widget w, KeySym mn)
 	return;
     }
 
-    DEBUGOUT(XdbDebug2(__FILE__, m, w, "_XmManagerInstallMnemonic(%c)\n", mn));
-    DEBUGOUT(XdbDebug2("NMEM", m, w, "_XmManagerInstallMnemonic(%c)\n", mn));
+    DEBUGOUT(_LtDebugDebug2(__FILE__, m, w, "_XmManagerInstallMnemonic(%c)\n", mn));
+    DEBUGOUT(_LtDebugDebug2("NMEM", m, w, "_XmManagerInstallMnemonic(%c)\n", mn));
 
     /* If we're dealing with the menu bar, store the stuff in the highest
      * manager widget that you can find (with all the accelerators) */
@@ -2182,8 +2214,8 @@ _XmManagerInstallMnemonic(Widget m, Widget w, KeySym mn)
 	    }
 	}
 
-	DEBUGOUT(XdbDebug2(__FILE__, mw, w, "Store mnemonic info\n"));
-	DEBUGOUT(XdbDebug2("NMEM", mw, w, "Store mnemonic info\n"));
+	DEBUGOUT(_LtDebugDebug2(__FILE__, mw, w, "Store mnemonic info\n"));
+	DEBUGOUT(_LtDebugDebug2("NMEM", mw, w, "Store mnemonic info\n"));
 	m = mw;
     }
 
@@ -2207,7 +2239,8 @@ _XmManagerUninstallMnemonic(Widget m, Widget w)
 {
     Widget mw, sh;
 
-    if (XmIsRowColumn(m) && RC_Type(m) == XmMENU_BAR)
+    if (XmIsRowColumn(m) &&
+	(RC_Type(m) == XmMENU_PULLDOWN || RC_Type(m) == XmMENU_BAR || RC_Type(m) == XmMENU_OPTION))
     {
 	/* Find the manager widget just underneath the shell */
 	for (mw = m; XtParent(mw); mw = sh)
@@ -2215,7 +2248,8 @@ _XmManagerUninstallMnemonic(Widget m, Widget w)
 	    sh = XtParent(mw);
 	    if (XtIsSubclass(sh, applicationShellWidgetClass) ||
 		XtIsSubclass(sh, xmDialogShellWidgetClass) ||
-		XtIsSubclass(sh, topLevelShellWidgetClass))
+		XtIsSubclass(sh, topLevelShellWidgetClass) ||
+		(RC_Type(m) != XmMENU_BAR && XmIsRowColumn(mw) && RC_Type(mw) == XmMENU_BAR))
 	    {
 		break;
 	    }
@@ -2243,18 +2277,18 @@ widget_navigable(Widget w)
 	    MGR_NavigationType(w) == XmEXCLUSIVE_TAB_GROUP ||
 	    (MGR_NavigationType(w) == XmTAB_GROUP && !_XmShellIsExclusive(w)))
 	{
-	    DEBUGOUT(XdbDebug(__FILE__, w,
+	    DEBUGOUT(_LtDebugDebug(__FILE__, w,
 			  "WidgetNavigable => XmDESCENDANTS_TAB_NAVIGABLE\n"));
 
 	    return XmDESCENDANTS_TAB_NAVIGABLE;
 	}
-	DEBUGOUT(XdbDebug(__FILE__, w,
+	DEBUGOUT(_LtDebugDebug(__FILE__, w,
 			  "WidgetNavigable => XmDESCENDANTS_NAVIGABLE\n"));
 
 	return XmDESCENDANTS_NAVIGABLE;
     }
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "WidgetNavigable => XmNOT_NAVIGABLE\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "WidgetNavigable => XmNOT_NAVIGABLE\n"));
 
     return XmNOT_NAVIGABLE;
 }
@@ -2264,7 +2298,7 @@ _XmManagerHighlightPixmapDefault(Widget w, int offset, XrmValue *value)
 {
     static Pixmap pixmap;
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "_XmManagerHighlightPixmapDefault\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "_XmManagerHighlightPixmapDefault\n"));
 
     pixmap = XmUNSPECIFIED_PIXMAP;
 
@@ -2283,7 +2317,7 @@ _XmManagerTopShadowPixmapDefault(Widget w, int offset, XrmValue *value)
 {
     static Pixmap pixmap;
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "_XmManagerTopShadowPixmapDefault\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "_XmManagerTopShadowPixmapDefault\n"));
 
     pixmap = XmUNSPECIFIED_PIXMAP;
 
