@@ -1,8 +1,9 @@
 /**
  *
- * $Id: MenuUtil.c,v 1.60 1999/04/01 00:51:18 rwscott Exp $
+ * $Id: MenuUtil.c,v 1.64 2001/05/14 19:46:09 amai Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
+ * Copyright (C) 1995-2001 LessTif Development Team
  *
  * This file is part of the GNU LessTif Library.
  *
@@ -22,9 +23,10 @@
  *
  **/
 
-static const char rcsid[] = "$Id: MenuUtil.c,v 1.60 1999/04/01 00:51:18 rwscott Exp $";
+static const char rcsid[] = "$Id: MenuUtil.c,v 1.64 2001/05/14 19:46:09 amai Exp $";
 
 #include <LTconfig.h>
+
 #include <XmI/XmI.h>
 
 #include <Xm/XmP.h>
@@ -40,7 +42,8 @@ static const char rcsid[] = "$Id: MenuUtil.c,v 1.60 1999/04/01 00:51:18 rwscott
 
 #include <XmI/DebugUtil.h>
 
-Boolean
+
+extern Boolean
 _XmMenuGetInPMMode(Widget w)
 {
     XmMenuState state = _XmGetMenuState(w);
@@ -48,7 +51,8 @@ _XmMenuGetInPMMode(Widget w)
     return state->MU_InPMMode;
 }
 
-void
+
+extern void
 _XmMenuSetInPMMode(Widget w, Boolean flag)
 {
     XmMenuState state = _XmGetMenuState(w);
@@ -56,7 +60,8 @@ _XmMenuSetInPMMode(Widget w, Boolean flag)
     state->MU_InPMMode = flag;
 }
 
-Boolean
+
+extern Boolean
 _XmGetInDragMode(Widget w)
 {
     XmMenuState state = _XmGetMenuState(w);
@@ -64,17 +69,19 @@ _XmGetInDragMode(Widget w)
     return state->MU_InDragMode;
 }
 
-void
+
+extern void
 _XmSetInDragMode(Widget w, Boolean flag)
 {
     XmMenuState state = _XmGetMenuState(w);
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "_XmSetInDragMode() - %s\n",
-    		XdbBoolean2String(flag)));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "_XmSetInDragMode() - %s\n",
+    		_LtDebugBoolean2String(flag)));
     state->MU_InDragMode = flag;
 }
 
-Widget
+
+extern Widget
 _XmGetRC_PopupPosted(Widget rc)
 {
     if (!XmIsRowColumn(rc))
@@ -92,12 +99,13 @@ _XmGetRC_PopupPosted(Widget rc)
     }
 }
 
+
 /*
  * A wrapper around the Xt Intrinsic's pointer grabbing. This
  * one retries a failing grab for several times and if all fails
  * spits out a warning message.
  */
-int
+extern int
 _XmGrabPointer(Widget w,
 	       int owner_events,
 	       unsigned int event_mask,
@@ -109,7 +117,7 @@ _XmGrabPointer(Widget w,
 {
     int result, retries;
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "_XmGrabPointer()\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "_XmGrabPointer()\n"));
 
     for (retries = 4; retries >= 0; retries--)
     {
@@ -123,7 +131,7 @@ _XmGrabPointer(Widget w,
 	    return result;
 	}
 
-	DEBUGOUT(XdbDebug(__FILE__, w, "_XmGrabPointer => %s, trying again\n",
+	DEBUGOUT(_LtDebugDebug(__FILE__, w, "_XmGrabPointer => %s, trying again\n",
 			  (result == AlreadyGrabbed) ? "AlreadyGrabbed" :
 			  (result == GrabInvalidTime) ? "GrabInvalidTime" :
 			  (result == GrabNotViewable) ? "GrabNotViewable" :
@@ -140,19 +148,21 @@ _XmGrabPointer(Widget w,
     return result;
 }
 
-void
+
+extern void
 _XmUngrabPointer(Widget w, Time t)
 {
-    DEBUGOUT(XdbDebug(__FILE__, w, "_XmUngrabPointer\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "_XmUngrabPointer\n"));
 
     XtUngrabPointer(XmIsGadget(w) ? XtParent(w) : w, t);
 }
 
+
 /*
  * Same as the _XmGrabPointer wrapper above, but this time for grabbing
  * the keyboard.
  */
-int
+extern int
 _XmGrabKeyboard(Widget widget,
 		int owner_events,
 		int pointer_mode,
@@ -161,7 +171,7 @@ _XmGrabKeyboard(Widget widget,
 {
     int result, retries;
 
-    DEBUGOUT(XdbDebug(__FILE__, widget, "_XmGrabKeyboard()\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, widget, "_XmGrabKeyboard()\n"));
 
     for (retries = 4; retries >= 0; retries--)
     {
@@ -175,7 +185,7 @@ _XmGrabKeyboard(Widget widget,
 	    return result;
 	}
 
-	DEBUGOUT(XdbDebug(__FILE__, widget,
+	DEBUGOUT(_LtDebugDebug(__FILE__, widget,
 			  "_XmGrabKeyboard : trying again\n"));
 
 	if (retries)
@@ -189,15 +199,17 @@ _XmGrabKeyboard(Widget widget,
     return result;
 }
 
-void
+
+extern void
 _XmUngrabKeyboard(Widget w, Time t)
 {
-    DEBUGOUT(XdbDebug(__FILE__, w, "_XmUngrabKeyboard\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "_XmUngrabKeyboard\n"));
 
     XtUngrabKeyboard(XmIsGadget(w) ? XtParent(w) : w, t);
 }
 
-void
+
+extern void
 _XmMenuEscape(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 {
     /* Widget cb = NULL; */
@@ -210,8 +222,8 @@ _XmMenuEscape(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 
     _XmRecordEvent(ev);
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "_XmMenuEscape()\n"));
-    DEBUGOUT(XdbDebug("ESC", w, "_XmMenuEscape()\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "_XmMenuEscape()\n"));
+    DEBUGOUT(_LtDebugDebug("ESC", w, "_XmMenuEscape()\n"));
     if (XmIsRowColumn(w))
     {
 	rc = (XmRowColumnWidget)w;
@@ -220,8 +232,8 @@ _XmMenuEscape(Widget w, XEvent *ev, String *params, Cardinal *num_params)
     {
 	rc = (XmRowColumnWidget)XtParent(w);
     }
-    DEBUGOUT(XdbDebug(__FILE__, w, "    Menu      %s (%s)\n", XtName(rc), XdbRcType2String(RC_Type(rc))));
-    DEBUGOUT(XdbDebug("ESC", w, "    Menu      %s (%s)\n", XtName(rc), XdbRcType2String(RC_Type(rc))));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "    Menu      %s (%s)\n", XtName(rc), _LtDebugRcType2String(RC_Type(rc))));
+    DEBUGOUT(_LtDebugDebug("ESC", w, "    Menu      %s (%s)\n", XtName(rc), _LtDebugRcType2String(RC_Type(rc))));
 
     if (RC_Type(rc) == XmMENU_BAR)
     {
@@ -240,26 +252,26 @@ _XmMenuEscape(Widget w, XEvent *ev, String *params, Cardinal *num_params)
     }
     else if (_XmIsActiveTearOff((Widget)rc))
     {
-	DEBUGOUT(XdbDebug(__FILE__, w, "    Menu is TornOff\n"));
-	DEBUGOUT(XdbDebug("ESC", w, "    Menu is TornOff\n"));
+	DEBUGOUT(_LtDebugDebug(__FILE__, w, "    Menu is TornOff\n"));
+	DEBUGOUT(_LtDebugDebug("ESC", w, "    Menu is TornOff\n"));
 	if (RC_PopupPosted(rc))
 	{
 	Boolean poppedUp;
 	Widget menu = RC_PopupPosted(rc);
 
-	    DEBUGOUT(XdbDebug(__FILE__, w, "    Menu has %s posted\n", XtName(menu)));
-	    DEBUGOUT(XdbDebug("ESC", w, "    Menu has %s posted\n", XtName(menu)));
+	    DEBUGOUT(_LtDebugDebug(__FILE__, w, "    Menu has %s posted\n", XtName(menu)));
+	    DEBUGOUT(_LtDebugDebug("ESC", w, "    Menu has %s posted\n", XtName(menu)));
 	    while (RC_PopupPosted(menu))
 	    {
 		menu = RC_PopupPosted(menu);
-		DEBUGOUT(XdbDebug(__FILE__, w, "    which has %s posted\n", XtName(menu)));
+		DEBUGOUT(_LtDebugDebug(__FILE__, w, "    which has %s posted\n", XtName(menu)));
 	    }
 	    RC_MenuShellPopdown(RC_CascadeBtn(menu), ev, &poppedUp);
 	}
 	else
 	{
-	    DEBUGOUT(XdbDebug(__FILE__, w, "    Menu has nothing posted\n"));
-	    DEBUGOUT(XdbDebug("ESC", w, "    Menu has nothing posted\n"));
+	    DEBUGOUT(_LtDebugDebug(__FILE__, w, "    Menu has nothing posted\n"));
+	    DEBUGOUT(_LtDebugDebug("ESC", w, "    Menu has nothing posted\n"));
 	    if (RC_TornOff(rc))
 	    {
 		RCClass_MenuProcs(XtClass(rc))(XmMENU_DISARM, (Widget)rc, NULL);
@@ -291,8 +303,8 @@ _XmMenuEscape(Widget w, XEvent *ev, String *params, Cardinal *num_params)
     Widget cb = RC_CascadeBtn(rc);
     unsigned char type = XmIsGadget(cb) ? LabG_MenuType(cb) : Lab_MenuType(cb);
 
-	DEBUGOUT(XdbDebug(__FILE__, w, "    Posted by %s (%s)\n", XtName(cb), XdbRcType2String(type)));
-	DEBUGOUT(XdbDebug("ESC", w, "    Posted by %s (%s)\n", XtName(cb), XdbRcType2String(type)));
+	DEBUGOUT(_LtDebugDebug(__FILE__, w, "    Posted by %s (%s)\n", XtName(cb), _LtDebugRcType2String(type)));
+	DEBUGOUT(_LtDebugDebug("ESC", w, "    Posted by %s (%s)\n", XtName(cb), _LtDebugRcType2String(type)));
 
 	if (type == XmMENU_BAR)
 	{
@@ -310,19 +322,20 @@ _XmMenuEscape(Widget w, XEvent *ev, String *params, Cardinal *num_params)
     {
     Boolean poppedUp;
 
-	DEBUGOUT(XdbDebug(__FILE__, w, "    must be a popup\n"));
+	DEBUGOUT(_LtDebugDebug(__FILE__, w, "    must be a popup\n"));
     	RC_MenuButtonPopdown(MGR_ActiveChild(rc), ev, &poppedUp);
     }
     return;
 }
 
-void
+
+extern void
 _XmMenuTraverseLeft(Widget w,
 		    XEvent *ev,
 		    String *params,
 		    Cardinal *num_params)
 {
-    DEBUGOUT(XdbDebug(__FILE__, w, "_XmMenuTraverseLeft()\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "_XmMenuTraverseLeft()\n"));
 
     if (!_XmIsEventUnique(ev)) return;
     _XmRecordEvent(ev);
@@ -343,13 +356,14 @@ _XmMenuTraverseLeft(Widget w,
     }
 }
 
-void
+
+extern void
 _XmMenuTraverseRight(Widget w,
 		     XEvent *ev,
 		     String *params,
 		     Cardinal *num_params)
 {
-    DEBUGOUT(XdbDebug(__FILE__, w, "_XmMenuTraverseRight()\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "_XmMenuTraverseRight()\n"));
 
     if (!_XmIsEventUnique(ev)) return;
     _XmRecordEvent(ev);
@@ -363,13 +377,14 @@ _XmMenuTraverseRight(Widget w,
     }
 }
 
-void
+
+extern void
 _XmMenuTraverseUp(Widget w,
 		  XEvent *ev,
 		  String *params,
 		  Cardinal *num_params)
 {
-    DEBUGOUT(XdbDebug(__FILE__, w, "_XmMenuTraverseUp()\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "_XmMenuTraverseUp()\n"));
 
     if (!_XmIsEventUnique(ev)) return;
     _XmRecordEvent(ev);
@@ -383,14 +398,15 @@ _XmMenuTraverseUp(Widget w,
     }
 }
 
-void
+
+extern void
 _XmMenuTraverseDown(Widget w,
 		    XEvent *ev,
 		    String *params,
 		    Cardinal *num_params)
 {
-    DEBUGOUT(XdbDebug(__FILE__, w, "_XmMenuTraverseDown()\n"));
-    DEBUGOUT(XdbDebug2("TRAV", XtParent(w), w, "_XmMenuTraverseDown() - %s %s\n",
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "_XmMenuTraverseDown()\n"));
+    DEBUGOUT(_LtDebugDebug2("TRAV", XtParent(w), w, "_XmMenuTraverseDown() - %s %s\n",
     	_XmIsEventUnique(ev) ? "unique" : "not-unique",
     	MGR_ActiveChild(XtParent(w)) ? XtName(MGR_ActiveChild(XtParent(w))) : "no-active"
     	));
@@ -407,14 +423,15 @@ _XmMenuTraverseDown(Widget w,
     }
 }
 
-void
+
+extern void
 _XmRC_GadgetTraverseDown(Widget w,
 			 XEvent *ev,
 			 String *params,
 			 Cardinal *num_params)
 {
-    DEBUGOUT(XdbDebug(__FILE__, w, "_XmRC_GadgetTraverseDown()\n"));
-    DEBUGOUT(XdbDebug("TRAV", w, "_XmRC_GadgetTraverseDown() - %s %s\n",
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "_XmRC_GadgetTraverseDown()\n"));
+    DEBUGOUT(_LtDebugDebug("TRAV", w, "_XmRC_GadgetTraverseDown() - %s %s\n",
     	_XmIsEventUnique(ev) ? "unique" : "not-unique",
     	MGR_ActiveChild((w)) ? XtName(MGR_ActiveChild((w))) : "no-active"
     	));
@@ -427,13 +444,14 @@ _XmRC_GadgetTraverseDown(Widget w,
     }
 }
 
-void
+
+extern void
 _XmRC_GadgetTraverseUp(Widget w,
 		       XEvent *ev,
 		       String *params,
 		       Cardinal *num_params)
 {
-    DEBUGOUT(XdbDebug(__FILE__, w, "_XmRC_GadgetTraverseUp()\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "_XmRC_GadgetTraverseUp()\n"));
 
     if (!_XmIsEventUnique(ev)) return;
     _XmRecordEvent(ev);
@@ -443,13 +461,14 @@ _XmRC_GadgetTraverseUp(Widget w,
     }
 }
 
-void
+
+extern void
 _XmRC_GadgetTraverseLeft(Widget w,
 			 XEvent *ev,
 			 String *params,
 			 Cardinal *num_params)
 {
-    DEBUGOUT(XdbDebug(__FILE__, w, "_XmRC_GadgetTraverseLeft()\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "_XmRC_GadgetTraverseLeft()\n"));
 
     if (!_XmIsEventUnique(ev)) return;
     _XmRecordEvent(ev);
@@ -459,13 +478,14 @@ _XmRC_GadgetTraverseLeft(Widget w,
     }
 }
 
-void
+
+extern void
 _XmRC_GadgetTraverseRight(Widget w,
 			  XEvent *ev,
 			  String *params,
 			  Cardinal *num_params)
 {
-    DEBUGOUT(XdbDebug(__FILE__, w, "_XmRC_GadgetTraverseRight()\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "_XmRC_GadgetTraverseRight()\n"));
 
     if (!_XmIsEventUnique(ev)) return;
     _XmRecordEvent(ev);
@@ -475,29 +495,33 @@ _XmRC_GadgetTraverseRight(Widget w,
     }
 }
 
+
 /*
  * apparently has something to do with the MenuProcEntry.
  */
-XtPointer
+extern XtPointer
 _XmGetMenuProcContext(void)
 {
     return NULL;
 }
 
+
 /*
  * apparently has something to do with the MenuProcEntry.
  */
-void
+extern void
 _XmSaveMenuProcContext(XtPointer address)
 {
 }
 
+
 extern void
 _XmSetMenuTraversal(Widget wid, Boolean traversalOn)
 {
 }
 
-Widget
+
+extern Widget
 _XmMenuNextItem(Widget menu, Widget current_item)
 {
     int current_item_index;
@@ -544,6 +568,7 @@ _XmMenuNextItem(Widget menu, Widget current_item)
     return(MGR_Children(menu)[next_item_index]);
 }
 
+
 static Widget
 _XmMenuPrevItem(Widget menu, Widget current_item)
 {
@@ -581,7 +606,8 @@ _XmMenuPrevItem(Widget menu, Widget current_item)
     return(MGR_Children(menu)[next_item_index]);
 }
 
-void
+
+extern void
 _XmMenuDisarmItem(Widget w)
 {
     if (w &&
@@ -592,11 +618,12 @@ _XmMenuDisarmItem(Widget w)
     }
 }
 
-void
+
+extern void
 _XmMenuArmItem(Widget w)
 {
-    DEBUGOUT(XdbDebug(__FILE__, w, "_XmMenuArmItem()\n"));
-    DEBUGOUT(XdbDebug("MENU", w, "_XmMenuArmItem() - %s\n"
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "_XmMenuArmItem()\n"));
+    DEBUGOUT(_LtDebugDebug("MENU", w, "_XmMenuArmItem() - %s\n"
     	));
     if (w && (!XtIsSensitive(w) || !XtIsManaged(w) ||
 	    (!(XmIsLabel(w) && (((XmLabelClassRec *)(XtClass(w)))->label_class.menuProcs) && Prim_TraversalOn(w)) &&
@@ -628,26 +655,27 @@ _XmMenuArmItem(Widget w)
     }
 }
 
-void
+
+extern void
 _XmMenuTraversalHandler(Widget w, Widget pw, XmTraversalDirection direction)
 {
-    DEBUGOUT(XdbDebug2(__FILE__, w, pw, "_XmMenuTraversalHandler()\n"));
+    DEBUGOUT(_LtDebugDebug2(__FILE__, w, pw, "_XmMenuTraversalHandler()\n"));
     switch(direction)
     {
     case XmTRAVERSE_DOWN:
     	if ((XmIsCascadeButton(pw) && CB_Submenu(pw)) && RC_Type(w) == XmMENU_BAR)
     	{
 	    _XmWarning(pw, "%s(%i) - Traversal down in MENU_BAR not written yet!", __FILE__,__LINE__);
-	    DEBUGOUT(XdbDebug2(__FILE__, w, pw, "    Post %s\n", XtName(CB_Submenu(pw))));
-	    DEBUGOUT(XdbDebug2(__FILE__, w, pw, "    Arm %s\n", XtName(MGR_Children(CB_Submenu(pw))[0])));
+	    DEBUGOUT(_LtDebugDebug2(__FILE__, w, pw, "    Post %s\n", XtName(CB_Submenu(pw))));
+	    DEBUGOUT(_LtDebugDebug2(__FILE__, w, pw, "    Arm %s\n", XtName(MGR_Children(CB_Submenu(pw))[0])));
     	}
     	else if (RC_Type(w) == XmMENU_PULLDOWN || RC_Type(w) == XmMENU_POPUP)
     	{
-	    DEBUGOUT(XdbDebug2(__FILE__, w, pw, "    Arm %s\n", XtName(_XmMenuNextItem(w, pw))));
+	    DEBUGOUT(_LtDebugDebug2(__FILE__, w, pw, "    Arm %s\n", XtName(_XmMenuNextItem(w, pw))));
 	    _XmMenuDisarmItem(pw);
 	    _XmMenuArmItem(_XmMenuNextItem(w,pw));
 
-	    DEBUGOUT(XdbDebug2(__FILE__, w, pw, "    Active menu item is now %s\n", XtName(_XmMenuNextItem(w, pw))));
+	    DEBUGOUT(_LtDebugDebug2(__FILE__, w, pw, "    Active menu item is now %s\n", XtName(_XmMenuNextItem(w, pw))));
     	}
     	else
     	{
@@ -659,11 +687,11 @@ _XmMenuTraversalHandler(Widget w, Widget pw, XmTraversalDirection direction)
     case XmTRAVERSE_UP:
     	if (RC_Type(w) == XmMENU_PULLDOWN || RC_Type(w) == XmMENU_POPUP)
     	{
-	    DEBUGOUT(XdbDebug2(__FILE__, w, pw, "    Arm %s\n", XtName(_XmMenuPrevItem(w, pw))));
+	    DEBUGOUT(_LtDebugDebug2(__FILE__, w, pw, "    Arm %s\n", XtName(_XmMenuPrevItem(w, pw))));
 	    _XmMenuDisarmItem(pw);
 	    _XmMenuArmItem(_XmMenuPrevItem(w,pw));
 
-	    DEBUGOUT(XdbDebug2(__FILE__, w, pw, "    Active menu item is now %s\n", XtName(_XmMenuPrevItem(w, pw))));
+	    DEBUGOUT(_LtDebugDebug2(__FILE__, w, pw, "    Active menu item is now %s\n", XtName(_XmMenuPrevItem(w, pw))));
     	}
     	else
     	{
@@ -700,7 +728,7 @@ _XmMenuTraversalHandler(Widget w, Widget pw, XmTraversalDirection direction)
 	    {
 		if (RC_CascadeBtn(w)  && !RC_TornOff(w))
 		{
-		    DEBUGOUT(XdbDebug2(__FILE__, w, pw, "    RC_LastSelectTopLevel %s %s\n", 
+		    DEBUGOUT(_LtDebugDebug2(__FILE__, w, pw, "    RC_LastSelectTopLevel %s %s\n", 
 			    XtName(RC_LastSelectToplevel(w)),
 			    /* XtName(MGR_ActiveChild(RC_LastSelectToplevel(w))), */
 			    XtName(RC_CascadeBtn(w))));
@@ -751,13 +779,13 @@ _XmMenuTraversalHandler(Widget w, Widget pw, XmTraversalDirection direction)
     		{
 		    if (RC_CascadeBtn(w)  && !RC_TornOff(w))
 		    {
-			DEBUGOUT(XdbDebug2(__FILE__, w, pw, "    RC_LastSelectTopLevel %s %s\n", 
+			DEBUGOUT(_LtDebugDebug2(__FILE__, w, pw, "    RC_LastSelectTopLevel %s %s\n", 
 			    XtName(RC_LastSelectToplevel(w)),
 			    /* XtName(MGR_ActiveChild(RC_LastSelectToplevel(w))), */
 			    XtName(RC_CascadeBtn(w))));
 			if (RC_Type(RC_LastSelectToplevel(w)) == XmMENU_BAR)
 			{
-			    DEBUGOUT(XdbDebug2("TRAV", w, pw, "    RC_LastSelectTopLevel %s %s %s posted\n", 
+			    DEBUGOUT(_LtDebugDebug2("TRAV", w, pw, "    RC_LastSelectTopLevel %s %s %s posted\n", 
 				XtName(RC_LastSelectToplevel(w)),
 				XtName(RC_CascadeBtn(w)),
 				RC_PopupPosted(w) ? XtName(RC_PopupPosted(w)) : "nothing"
@@ -780,17 +808,19 @@ _XmMenuTraversalHandler(Widget w, Widget pw, XmTraversalDirection direction)
     }
 }
 
-int num_saved = 0;
+static int num_saved = 0;
 static String saved_trans[32];
 
-void
+
+extern void
 _XmSaveCoreClassTranslations(Widget widget)
 {
     saved_trans[num_saved] = CoreClassTranslations(widget);
     num_saved++;
 }
 
-void
+
+extern void
 _XmRestoreCoreClassTranslations(Widget widget)
 {
     if (num_saved > 0)
@@ -800,7 +830,8 @@ _XmRestoreCoreClassTranslations(Widget widget)
     }
 }
 
-void
+
+extern void
 XmSetMenuCursor(Display *display, Cursor cursorId)
 {
     int	i;
@@ -815,13 +846,15 @@ XmSetMenuCursor(Display *display, Cursor cursorId)
 		cursorId;
 }
 
-Cursor
+
+extern Cursor
 XmGetMenuCursor(Display *display)
 {
     return _XmGetMenuCursorByScreen(DefaultScreenOfDisplay(display));
 }
 
-XmMenuState
+
+extern XmMenuState
 _XmGetMenuState(Widget widget)
 {
     XmScreenInfo *info = _XmGetScreenInfo(XmGetXmScreen(XtScreen(widget)));
@@ -836,15 +869,16 @@ _XmGetMenuState(Widget widget)
 extern void _XmPopup(Widget shell, XtGrabKind grab_kind);
 extern void _XmPopdown(Widget shell);
 
-void
+extern void
 _XmPopup(Widget shell, XtGrabKind grab_kind)
 {
     XtPopup(shell, grab_kind);
 }
 
-void
+
+extern void
 _XmPopdown(Widget shell)
 {
     XtPopdown(shell);
 }
-#endif
+#endif /* XmVERSION > 1 */
