@@ -1,8 +1,9 @@
 /**
  *
- * $Id: Protocols.c,v 1.7 1999/04/12 17:04:16 strauman Exp $
+ * $Id: Protocols.c,v 1.13 2001/05/20 17:49:38 amai Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
+ * Copyright (C) 1995-2001 LessTif Development Team
  *
  * This file is part of the GNU LessTif Library.
  *
@@ -22,10 +23,14 @@
  *
  **/
 
-static const char rcsid[] = "$Id: Protocols.c,v 1.7 1999/04/12 17:04:16 strauman Exp $";
+static const char rcsid[] = "$Id: Protocols.c,v 1.13 2001/05/20 17:49:38 amai Exp $";
 
 #include <LTconfig.h>
+
+#include <string.h>
+
 #include <XmI/XmI.h>
+#include <XmI/MacrosI.h>
 
 #include <Xm/XmP.h>
 #include <Xm/ProtocolsP.h>
@@ -34,12 +39,9 @@ static const char rcsid[] = "$Id: Protocols.c,v 1.7 1999/04/12 17:04:16 strauman
 #include <XmI/DebugUtil.h>
 
 static void class_initialize(void);
-
 static void class_part_initialize(WidgetClass w_class);
-
 static void initialize(Widget request, Widget new_w,
 		       ArgList args, Cardinal *num_args);
-
 static void destroy(Widget w);
 
 #if 0
@@ -72,20 +74,25 @@ static void _XmProtocolHandler(Widget w, XtPointer client_data, XEvent *ev, Bool
 #ifdef	XmSetWMProtocolHooks
 #undef	XmSetWMProtocolHooks
 #endif
-extern void XmActivateWMProtocol(Widget shell, Atom protocol);
-extern void XmAddWMProtocols(Widget shell, Atom *protocols,
+
+void XmActivateWMProtocol(Widget shell, Atom protocol);
+void XmAddWMProtocols(Widget shell, Atom *protocols,
 			     Cardinal num_protocols);
-extern void XmAddWMProtocolCallback(Widget shell, Atom protocol,
+void XmAddWMProtocolCallback(Widget shell, Atom protocol,
 				   XtCallbackProc callback, XtPointer closure);
-extern void XmRemoveWMProtocolCallback(Widget shell, Atom protocol,
+void XmRemoveWMProtocolCallback(Widget shell, Atom protocol,
 				   XtCallbackProc callback, XtPointer closure);
-extern void XmRemoveWMProtocols(Widget shell, Atom *protocols,
+void XmRemoveWMProtocols(Widget shell, Atom *protocols,
 				Cardinal num_protocols);
-extern void XmSetWMProtocolHooks(Widget shell, Atom protocol,
+void XmSetWMProtocolHooks(Widget shell, Atom protocol,
 				 XtCallbackProc prehook, XtPointer pre_closure,
 				 XtCallbackProc posthook,
 				 XtPointer post_closure);
 
+/* undocumented, but export stuff from libXt */
+extern void _XtAddCallback(XtCallbackList *, XtCallbackProc, XtPointer);
+extern void _XtRemoveCallback(XtCallbackList *, XtCallbackProc, XtPointer);
+
 /*
  * resources 
  */
@@ -104,7 +111,7 @@ static XtResource resources[] =
     }
 };
 
-/* *INDENT-OFF* */
+
 XmProtocolClassRec xmProtocolClassRec = {
     /* Object class part */
     {
@@ -152,49 +159,52 @@ XmProtocolClassRec xmProtocolClassRec = {
         /* extension             */ NULL
     }
 };
-/* *INDENT-ON* */
+
 
 WidgetClass xmProtocolClass = (WidgetClass)&xmProtocolClassRec;
 
+
 static void
 class_initialize(void)
 {
-    DEBUGOUT(XdbDebug(__FILE__, NULL, "Protocol ClassInitialize\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, NULL, "Protocol ClassInitialize\n"));
 }
 
+
 static void
 class_part_initialize(WidgetClass widget_class)
 {
-    DEBUGOUT(XdbDebug(__FILE__, NULL, "Protocol ClassPartInitialize\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, NULL, "Protocol ClassPartInitialize\n"));
 }
 
+
 static void
 initialize(Widget request, Widget new_w,
 	   ArgList args, Cardinal *num_args)
 {
-    DEBUGOUT(XdbDebug(__FILE__, new_w, "Protocol Initialize\n"));
-
-    Protocol_PreHook(new_w).callback = NULL;
-    Protocol_PostHook(new_w).callback = NULL;
+    DEBUGOUT(_LtDebugDebug(__FILE__, new_w, "Protocol Initialize\n"));
 }
 
+
 static void
 destroy(Widget w)
 {
-    DEBUGOUT(XdbDebug(__FILE__, w, "Protocol Destroy\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "Protocol Destroy\n"));
 }
 
+
 #if 0
 static Boolean
 set_values(Widget current, Widget request, Widget new_w,
 	   ArgList args, Cardinal *num_args)
 {
-    DEBUGOUT(XdbDebug(__FILE__, new_w, "Protocol SetValues\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, new_w, "Protocol SetValues\n"));
 
     return True;		/* FIX ME */
 }
 #endif
 
+
 static XmAllProtocolsMgr
 __XmGetAllMgr(Widget shell)
 {
@@ -208,12 +218,13 @@ __XmGetAllMgr(Widget shell)
     newman = (XmAllProtocolsMgr)_XmGetWidgetExtData(shell,
 						    XmPROTOCOL_EXTENSION);
 
-    DEBUGOUT(XdbDebug(__FILE__, shell,
+    DEBUGOUT(_LtDebugDebug(__FILE__, shell,
 		      "Get AllMgr: %08x %08x\n", shell, newman));
 
     return newman;
 }
 
+
 static XmProtocolMgr
 __XmAddProperty(XmAllProtocolsMgr mgrs, Atom property)
 {
@@ -244,6 +255,7 @@ __XmAddProperty(XmAllProtocolsMgr mgrs, Atom property)
     return mgrs->protocol_mgrs[mgrs->num_protocol_mgrs - 1];
 }
 
+
 static XmProtocolMgr
 __XmFindProperty(XmAllProtocolsMgr mgrs, Atom property)
 {
@@ -258,6 +270,7 @@ __XmFindProperty(XmAllProtocolsMgr mgrs, Atom property)
     return NULL;
 }
 
+
 static XmProtocol
 __XmAddProtocol(XmAllProtocolsMgr mgrs, Atom property, Atom protocol)
 {
@@ -295,8 +308,14 @@ __XmAddProtocol(XmAllProtocolsMgr mgrs, Atom property, Atom protocol)
 	_XmError(NULL, "Shell has no extension!\n");
     }
 
-    w = (XmProtocol)XtCreateWidget("protocol_ext", xmProtocolClass,
-				   ve, NULL, 0);
+    w = (XmProtocol)_XmExtObjAlloc(xmProtocolClass->
+					core_class.widget_size);
+    ExtObj_LogicalParent(w) = mgrs->shell;
+    ExtObj_ExtensionType(w) = XmPROTOCOL_EXTENSION;
+
+    memset(&Protocol_PreHook(w), 0, sizeof(XtCallbackRec));
+    memset(&Protocol_PostHook(w), 0, sizeof(XtCallbackRec));
+    Protocol_Callbacks(w) = NULL;
 
     Protocol_Atom(w) = protocol;
     mgr->protocols[mgr->num_protocols] = w;
@@ -305,6 +324,7 @@ __XmAddProtocol(XmAllProtocolsMgr mgrs, Atom property, Atom protocol)
     return w;
 }
 
+
 static XmProtocol
 __XmFindProtocol(XmAllProtocolsMgr mgrs, Atom property, Atom protocol)
 {
@@ -328,6 +348,7 @@ __XmFindProtocol(XmAllProtocolsMgr mgrs, Atom property, Atom protocol)
     return NULL;
 }
 
+
 #if 0
 static XtActionsRec actions[] =
 {
@@ -335,6 +356,7 @@ static XtActionsRec actions[] =
 };
 #endif
 
+
 void
 _XmInitProtocols(Widget w)
 {
@@ -345,7 +367,7 @@ _XmInitProtocols(Widget w)
 
     _XmPushWidgetExtData(w, (XmWidgetExtData)newman, XmPROTOCOL_EXTENSION);
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "Init Protocols: %08x %08x\n", w, newman));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "Init Protocols: %08x %08x\n", w, newman));
 
 #if 0
     XtAppAddActions(XtWidgetToApplicationContext(w),
@@ -364,7 +386,8 @@ _XmInitProtocols(Widget w)
 #endif
 }
 
-void
+
+extern void
 _XmDestroyProtocols(Widget w)
 {
 	XmAllProtocolsMgr allmgrs = NULL;
@@ -374,7 +397,7 @@ _XmDestroyProtocols(Widget w)
 
 	if ( ! allmgrs )
 	{
-	return;
+	    return;
 	}
 
 	/* we could repeatedly call _XmRemoveProtocols here, but it is easier
@@ -388,22 +411,26 @@ _XmDestroyProtocols(Widget w)
 		/* destroy all protocol objects */
 		for (nproto = 0; nproto < mgr->num_protocols; nproto++)
 		{
-			XtDestroyWidget((Widget) mgr->protocols[nproto]);
+		    _XmExtObjFree((XtPointer) mgr->protocols[nproto]);
 		}
 		XtFree((char*) mgr->protocols);
 		XtFree((char*) mgr);
 	}
 	
-	XtFree((char*) allmgrs->protocol_mgrs);
+	if (allmgrs->protocol_mgrs)
+	{
+	    XtFree((char*) allmgrs->protocol_mgrs);
+	}
 	
 	XtFree((char*)allmgrs);
 	XtRemoveEventHandler(w, XtAllEvents, True, _XmProtocolHandler, NULL);
 }
 
+
 /*
  * this ONLY gets called from VendorShellExt's realize callback
  */
-void
+extern void
 _XmInstallProtocols(Widget w)
 {
     XmAllProtocolsMgr mgrs;
@@ -411,7 +438,7 @@ _XmInstallProtocols(Widget w)
     Atom *protocols;
     int i, j;
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "_XmInstallProtocols\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "_XmInstallProtocols\n"));
 
     if ((mgrs = __XmGetAllMgr(w)) == NULL)
     {
@@ -438,6 +465,7 @@ _XmInstallProtocols(Widget w)
     }
 }
 
+
 static void
 #if 0
 _XmProtocolHandler(Widget w, XEvent *evp, String *params, Cardinal *nparams)
@@ -450,14 +478,14 @@ _XmProtocolHandler(Widget w, XtPointer client_data, XEvent *evp, Boolean *ctd)
     XmProtocol p;
     XmAnyCallbackStruct cd;
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "_XmProtocolHandler\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "_XmProtocolHandler\n"));
 
     cd.reason = XmCR_WM_PROTOCOLS;
     cd.event = evp;
 
     if (evp->type != ClientMessage)
     {
-	DEBUGOUT(XdbDebug(__FILE__, w,
+	DEBUGOUT(_LtDebugDebug(__FILE__, w,
 			  "XmProtocolHandler(not a client message)\n"));
 	return;
     }
@@ -467,7 +495,7 @@ _XmProtocolHandler(Widget w, XtPointer client_data, XEvent *evp, Boolean *ctd)
 
     if ((mgrs = __XmGetAllMgr(w)) == NULL)
     {
-	DEBUGOUT(XdbDebug(__FILE__, w,
+	DEBUGOUT(_LtDebugDebug(__FILE__, w,
 			  "XmProtocolHandler(couldn't find manager struct)\n"));
 
 	return;
@@ -475,7 +503,7 @@ _XmProtocolHandler(Widget w, XtPointer client_data, XEvent *evp, Boolean *ctd)
 
     if ((p = __XmFindProtocol(mgrs, property, protocol)) == NULL)
     {
-	DEBUGOUT(XdbDebug(__FILE__, w,
+	DEBUGOUT(_LtDebugDebug(__FILE__, w,
 			  "XmProtocolHandler(couldn't find protocol)\n"));
 
 	return;
@@ -483,7 +511,7 @@ _XmProtocolHandler(Widget w, XtPointer client_data, XEvent *evp, Boolean *ctd)
 
     if (Protocol_Active(p))
     {
-	DEBUGOUT(XdbDebug(__FILE__, w,
+	DEBUGOUT(_LtDebugDebug(__FILE__, w,
 			  "XmProtocolHandler(calling callbacks)\n"));
 
 	if (Protocol_PreHook(p).callback)
@@ -506,12 +534,13 @@ _XmProtocolHandler(Widget w, XtPointer client_data, XEvent *evp, Boolean *ctd)
     }
     else
     {
-	DEBUGOUT(XdbDebug(__FILE__, w,
+	DEBUGOUT(_LtDebugDebug(__FILE__, w,
 			  "XmProtocolHandler(protocol not active)\n"));
     }
 }
 
-void
+
+extern void
 XmActivateProtocol(Widget shell,
 		   Atom property,
 		   Atom protocol)
@@ -519,7 +548,7 @@ XmActivateProtocol(Widget shell,
     XmAllProtocolsMgr mgrs;
     XmProtocol p;
 
-    DEBUGOUT(XdbDebug(__FILE__, shell, "XmActivateProtocol\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, shell, "XmActivateProtocol\n"));
 
     if ((mgrs = __XmGetAllMgr(shell)) == NULL)
     {
@@ -534,7 +563,8 @@ XmActivateProtocol(Widget shell,
     Protocol_Active(p) = True;
 }
 
-void
+
+extern void
 XmDeactivateProtocol(Widget shell,
 		     Atom property,
 		     Atom protocol)
@@ -542,7 +572,7 @@ XmDeactivateProtocol(Widget shell,
     XmAllProtocolsMgr mgrs;
     XmProtocol p;
 
-    DEBUGOUT(XdbDebug(__FILE__, shell, "XmDeactivateProtocol\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, shell, "XmDeactivateProtocol\n"));
 
     if ((mgrs = __XmGetAllMgr(shell)) == NULL)
     {
@@ -557,7 +587,8 @@ XmDeactivateProtocol(Widget shell,
     Protocol_Active(p) = False;
 }
 
-void
+
+extern void
 XmActivateWMProtocol(Widget shell,
 		     Atom protocol)
 {
@@ -566,7 +597,8 @@ XmActivateWMProtocol(Widget shell,
 		       protocol);
 }
 
-void
+
+extern void
 XmAddProtocols(Widget shell,
 	       Atom property,
 	       Atom *protocol,
@@ -580,11 +612,11 @@ XmAddProtocols(Widget shell,
     XmAllProtocolsMgr mgrs;
     XmProtocol p;
 
-    DEBUGOUT(XdbDebug(__FILE__, shell, "XmAddProtocols\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, shell, "XmAddProtocols\n"));
 
     if ((mgrs = __XmGetAllMgr(shell)) == NULL)
     {
-	DEBUGOUT(XdbDebug(__FILE__, shell,
+	DEBUGOUT(_LtDebugDebug(__FILE__, shell,
 			  "XmAddProtocols: Can't find Management structure:"
 			  " not a shell?\n"));
 	return;
@@ -650,7 +682,8 @@ XmAddProtocols(Widget shell,
     }
 }
 
-void
+
+extern void
 XmAddWMProtocols(Widget shell,
 		 Atom *protocols,
 		 Cardinal num_protocols)
@@ -661,7 +694,8 @@ XmAddWMProtocols(Widget shell,
 		   num_protocols);
 }
 
-void
+
+extern void
 XmAddProtocolCallback(Widget shell,
 		      Atom property,
 		      Atom protocol,
@@ -671,11 +705,11 @@ XmAddProtocolCallback(Widget shell,
     XmAllProtocolsMgr mgrs;
     XmProtocol p;
 
-    DEBUGOUT(XdbDebug(__FILE__, shell, "XmAddProtocolCallback\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, shell, "XmAddProtocolCallback\n"));
 
     if ((mgrs = __XmGetAllMgr(shell)) == NULL)
     {
-	DEBUGOUT(XdbDebug(__FILE__, shell,
+	DEBUGOUT(_LtDebugDebug(__FILE__, shell,
 		      "XmAddProtocolCallback: Can't find Management structure:"
 			  " not a shell?\n"));
 
@@ -692,10 +726,11 @@ XmAddProtocolCallback(Widget shell,
     /*
      * Add Callback
      */
-    XtAddCallback((Widget)p, XmNprotocolCallback, callback, closure);
+    _XtAddCallback(&Protocol_Callbacks(p), callback, closure);
 }
 
-void
+
+extern void
 XmAddWMProtocolCallback(Widget shell,
 			Atom protocol,
 			XtCallbackProc callback,
@@ -708,7 +743,8 @@ XmAddWMProtocolCallback(Widget shell,
 			  closure);
 }
 
-void
+
+extern void
 XmRemoveProtocolCallback(Widget shell,
 			 Atom property,
 			 Atom protocol,
@@ -718,11 +754,11 @@ XmRemoveProtocolCallback(Widget shell,
     XmAllProtocolsMgr mgrs;
     XmProtocol p;
 
-    DEBUGOUT(XdbDebug(__FILE__, shell, "XmRemoveProtocolCallback\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, shell, "XmRemoveProtocolCallback\n"));
 
     if ((mgrs = __XmGetAllMgr(shell)) == NULL)
     {
-	DEBUGOUT(XdbDebug(__FILE__, shell,
+	DEBUGOUT(_LtDebugDebug(__FILE__, shell,
 		   "XmRemoveProtocolCallback: Can't find Management structure:"
 			  " not a shell?\n"));
 
@@ -739,10 +775,11 @@ XmRemoveProtocolCallback(Widget shell,
     /*
      * Remove Callback
      */
-    XtRemoveCallback((Widget)p, XmNprotocolCallback, callback, closure);
+    _XtRemoveCallback(&Protocol_Callbacks(p), callback, closure);
 }
 
-void
+
+extern void
 XmRemoveWMProtocolCallback(Widget shell,
 			   Atom protocol,
 			   XtCallbackProc callback,
@@ -755,7 +792,8 @@ XmRemoveWMProtocolCallback(Widget shell,
 			     closure);
 }
 
-void
+
+extern void
 XmRemoveProtocols(Widget shell,
 		  Atom property,
 		  Atom *protocols,
@@ -765,7 +803,7 @@ XmRemoveProtocols(Widget shell,
 	XmProtocolMgr	  mgr;
     int i, mgr_slot, proto_slot;
 
-    DEBUGOUT(XdbDebug(__FILE__, shell, "XmRemoveProtocols\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, shell, "XmRemoveProtocols\n"));
 
     if ((mgrs = __XmGetAllMgr(shell)) == NULL)
     {
@@ -778,24 +816,25 @@ XmRemoveProtocols(Widget shell,
 	    break;
     }
 
-	if ( mgr_slot == mgrs->num_protocol_mgrs )
-	{
-		/* property not found */
-		return;
-	}
-
-	mgr = mgrs->protocol_mgrs[mgr_slot];
-
-	for (i = 0; i < num_protocols; i++) {
-	proto_slot = 0;
-	while ( proto_slot < mgr->num_protocols )
+    if ( mgr_slot == mgrs->num_protocol_mgrs )
     {
-	if (Protocol_Atom(mgr->protocols[proto_slot]) == protocols[i])
-	{
+	/* property not found */
+	return;
+    }
+
+    mgr = mgrs->protocol_mgrs[mgr_slot];
+
+    for (i = 0; i < num_protocols; i++)
+    {
+	proto_slot = 0;
+	while (proto_slot < mgr->num_protocols)
+        {
+	    if (Protocol_Atom(mgr->protocols[proto_slot]) == protocols[i])
+	    {
 		int j;
 
 		/* destroy protocol object */
-	    XtDestroyWidget((Widget)mgr->protocols[proto_slot]);
+	        _XmExtObjFree((XtPointer)mgr->protocols[proto_slot]);
 
 		/* compact the remaining list */
 		for (j = proto_slot; j < mgr->num_protocols - 1; j++)
@@ -809,36 +848,39 @@ XmRemoveProtocols(Widget shell,
 		 */
 		if ( --(mgr->num_protocols) == 0 )
 		{
-			int j;
+		    int j;
 	
-			XtFree((char*)mgr->protocols);
+		    XtFree((char*)mgr->protocols);
 
-			XtFree((char*)mgrs->protocol_mgrs[mgr_slot]);
+		    XtFree((char*)mgrs->protocol_mgrs[mgr_slot]);
 
-			/* compact list of managers */
-			for ( j = mgr_slot; j < mgrs->num_protocol_mgrs - 1; j++)
-			{
-				mgrs->protocol_mgrs[j] = mgrs->protocol_mgrs[j+1];
-			}
-			if ( --(mgrs->num_protocol_mgrs) == 0 )
-			{
-				XtFree((char*) mgrs->protocol_mgrs);
-				mgrs->max_protocol_mgrs = 0;
-			}
-			/* IMPORTANT NOTE: If we should get here, mgr is does not exist
-			 *                 anymore and hence we must leave the loop
-			 *				   immediately.
-			 */
-			return;
+		    /* compact list of managers */
+		    for ( j = mgr_slot; j < mgrs->num_protocol_mgrs - 1; j++)
+		    {
+			mgrs->protocol_mgrs[j] = mgrs->protocol_mgrs[j+1];
+		    }
+		    if ( --(mgrs->num_protocol_mgrs) == 0 )
+		    {
+			XtFree((char*) mgrs->protocol_mgrs);
+			mgrs->protocol_mgrs = NULL;
+			mgrs->max_protocol_mgrs = 0;
+		    }
+		    /* IMPORTANT NOTE: If we should get here, mgr is does
+		     * not exist anymore and hence we must leave the loop
+		     * immediately.
+		     */
+		    return;
 		}
-		break; /* and continue looking for the next protocol to remove */
-	}
-	proto_slot++;
+		break; /* and continue looking for the next protocol to
+			* remove */
+	    }
+	    proto_slot++;
+        }
     }
-	}
 }
 
-void
+
+extern void
 XmRemoveWMProtocols(Widget shell,
 		    Atom *protocols,
 		    Cardinal num_protocols)
@@ -849,7 +891,8 @@ XmRemoveWMProtocols(Widget shell,
 		      num_protocols);
 }
 
-void
+
+extern void
 XmSetProtocolHooks(Widget shell,
 		   Atom property,
 		   Atom protocol,
@@ -861,11 +904,11 @@ XmSetProtocolHooks(Widget shell,
     XmAllProtocolsMgr mgrs;
     XmProtocol p;
 
-    DEBUGOUT(XdbDebug(__FILE__, shell, "XmSetProtocolHooks\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, shell, "XmSetProtocolHooks\n"));
 
     if ((mgrs = __XmGetAllMgr(shell)) == NULL)
     {
-	DEBUGOUT(XdbDebug(__FILE__, shell,
+	DEBUGOUT(_LtDebugDebug(__FILE__, shell,
 			  "XmSetProtocolHooks: Can't find Management structure:"
 			  " not a shell?\n"));
 
@@ -885,7 +928,8 @@ XmSetProtocolHooks(Widget shell,
     Protocol_PostHook(p).closure = post_closure;
 }
 
-void
+
+extern void
 XmSetWMProtocolHooks(Widget shell,
 		     Atom protocol,
 		     XtCallbackProc prehook,
