@@ -1,8 +1,9 @@
 /**
  *
- * $Id: PushB.c,v 1.73 1999/11/11 13:43:21 rwscott Exp $
+ * $Id: PushB.c,v 1.80 2001/05/20 17:49:38 amai Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
+ * Copyright (C) 1995-2001 LessTif Development Team
  *
  * This file is part of the GNU LessTif Library.
  *
@@ -22,11 +23,13 @@
  *
  **/
 
-static const char rcsid[] = "$Id: PushB.c,v 1.73 1999/11/11 13:43:21 rwscott Exp $";
+static const char rcsid[] = "$Id: PushB.c,v 1.80 2001/05/20 17:49:38 amai Exp $";
 
 #include <LTconfig.h>
-#include <XmI/XmI.h>
 
+#include <stdarg.h>
+
+#include <XmI/XmI.h>
 #include <Xm/XmP.h>
 #include <Xm/PushBP.h>
 #include <Xm/RepType.h>
@@ -36,19 +39,15 @@ static const char rcsid[] = "$Id: PushB.c,v 1.73 1999/11/11 13:43:21 rwscott Exp
 #include <Xm/MenuShell.h>
 #include <Xm/CascadeB.h>
 #include <Xm/CascadeBG.h>
-
-#if STDC_HEADERS
-#include <stdarg.h>
-#else
-#include <varargs.h>
+#if XmVERSION > 1
+#include <Xm/TraitP.h>
+#include <Xm/ActivatableT.h>
 #endif
 
 #include <XmI/DebugUtil.h>
 
-#if XmVERSION > 1
-#include <Xm/TraitP.h>
-#include <Xm/ActivatableT.h>
 
+#if XmVERSION > 1
 void _XmPushB_TraitAddCallback(Widget, XtCallbackProc, XtPointer, Boolean);
 
 static XmActivatableTraitRec _XmPushBTraitRec = {
@@ -353,8 +352,8 @@ class_initialize(void)
     _XmPushBCoreClassExtRec.record_type = XmQmotif;
 
 #if XmVERSION > 1
-	if (! XmeTraitSet(xmPushButtonWidgetClass, XmQTactivatable,
-			&_XmPushBTraitRec)) {
+	if (! XmeTraitSet((XtPointer)xmPushButtonWidgetClass, XmQTactivatable,
+			(XtPointer)&_XmPushBTraitRec)) {
 		_XmWarning(NULL,
 			"XmPushButton ClassInitialize: XmeTraitSet failed\n");
 	}
@@ -451,7 +450,7 @@ initialize(Widget request, Widget new_w,
 {
     Dimension margin, width, height;
 
-    DEBUGOUT(XdbDebug(__FILE__, new_w,
+    DEBUGOUT(_LtDebugDebug(__FILE__, new_w,
 		      "initialize: %i args\n"
 		      "\trequest X %5i Y %5i W %5i H %5i\n"
 		      "\t  new_w X %5i Y %5i W %5i H %5i\n",
@@ -460,7 +459,7 @@ initialize(Widget request, Widget new_w,
 		      XtWidth(request), XtHeight(request),
 		      XtX(new_w), XtY(new_w),
 		      XtWidth(new_w), XtHeight(new_w)));
-    DEBUGOUT(XdbPrintArgList(__FILE__, new_w, args, *num_args, False));
+    DEBUGOUT(_LtDebugPrintArgList(__FILE__, new_w, args, *num_args, False));
 
     /* check the RepType resources */
     if (PB_MultiClick(new_w) == (unsigned char)XmUNSPECIFIED ||
@@ -553,8 +552,10 @@ initialize(Widget request, Widget new_w,
 
 	    Lab_TextRect_x(new_w) += margin;
 	    Lab_TextRect_y(new_w) += margin;
-	    Lab_AccTextRect(new_w).x += margin;
-	    Lab_AccTextRect(new_w).y += margin;
+	    if (Lab_AcceleratorText(new_w)) {
+		Lab_AccTextRect(new_w).x += margin;
+		Lab_AccTextRect(new_w).y += margin;
+	    }
 
 	    /* rws 19 Aug 1997
 	     * This does not seem to make sense! Without this the width and
@@ -571,7 +572,7 @@ initialize(Widget request, Widget new_w,
 	    XtWidth(request) = XtWidth(new_w);
 #endif
 
-	    DEBUGOUT(XdbDebug(__FILE__, new_w,
+	    DEBUGOUT(_LtDebugDebug(__FILE__, new_w,
 			      "adjust margins for default button\n"
 			      "\trequest X %5i Y %5i W %5i H %5i\n"
 			      "\t  new_w X %5i Y %5i W %5i H %5i\n",
@@ -591,6 +592,7 @@ destroy(Widget w)
     if (PB_Timer(w) != 0)
     {
 	XtRemoveTimeOut(PB_Timer(w));
+	PB_Timer(w) = 0;
     }
 
     XtReleaseGC(w, PB_FillGC(w));
@@ -604,7 +606,7 @@ set_values(Widget old, Widget request, Widget new_w,
     Boolean refresh_needed = False;
     Dimension margin, width, height;
 
-    DEBUGOUT(XdbDebug(__FILE__, new_w,
+    DEBUGOUT(_LtDebugDebug(__FILE__, new_w,
 		      "set_values: %i args\n"
 		      "\t    old X %5i Y %5i W %5i H %5i\n"
 		      "\trequest X %5i Y %5i W %5i H %5i\n"
@@ -616,7 +618,7 @@ set_values(Widget old, Widget request, Widget new_w,
 		      XtWidth(request), XtHeight(request),
 		      XtX(new_w), XtY(new_w),
 		      XtWidth(new_w), XtHeight(new_w)));
-    DEBUGOUT(XdbPrintArgList(__FILE__, new_w, args, *num_args, False));
+    DEBUGOUT(_LtDebugPrintArgList(__FILE__, new_w, args, *num_args, False));
 
     /* check the RepType resources */
     if (!XmRepTypeValidValue(XmRepTypeGetId(XmRMultiClick),
@@ -702,8 +704,10 @@ set_values(Widget old, Widget request, Widget new_w,
 
 	    Lab_TextRect_x(new_w) += margin;
 	    Lab_TextRect_y(new_w) += margin;
-	    Lab_AccTextRect(new_w).x += margin;
-	    Lab_AccTextRect(new_w).y += margin;
+	    if (Lab_AcceleratorText(new_w) != NULL) {
+		Lab_AccTextRect(new_w).x += margin;
+		Lab_AccTextRect(new_w).y += margin;
+	    }
 #endif
 	    refresh_needed = True;
 	}
@@ -830,7 +834,7 @@ expose(Widget w, XEvent *event, Region region)
 	return;
     }
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "PB expose\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "PB expose\n"));
 
     if (!IN_MENU(w))
     {
@@ -839,7 +843,7 @@ expose(Widget w, XEvent *event, Region region)
 	    inset += Xm3D_ENHANCE_PIXEL + Lab_Shadow(w)
 		  + (PB_DefaultButtonShadow(w) << 1);
 
-	DEBUGOUT(XdbDebug(__FILE__, w,
+	DEBUGOUT(_LtDebugDebug(__FILE__, w,
 			  "x %d y %d w %d h %d b %d\n",
 			  inset, inset,
 			  (XtWidth(w) - (inset << 1)),
@@ -1015,8 +1019,8 @@ Activate(Widget w, XEvent *event, String *params, Cardinal *num_params)
     XmPushButtonCallbackStruct cbs;
     XButtonEvent *ev = (XButtonEvent *)event;
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "Activate()\n"));
-    DEBUGOUT(XdbDebug("ENTRY", w, "Activate()\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "Activate()\n"));
+    DEBUGOUT(_LtDebugDebug("ENTRY", w, "Activate()\n"));
 
     /*
      * This test also broke Accelerators. Refined as well.
@@ -1104,7 +1108,7 @@ ArmTimeout(XtPointer data, XtIntervalId *id)
 {
     Widget w = (Widget)data;
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "ArmTimeout\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "ArmTimeout\n"));
 
     PB_Timer(w) = 0;
 
@@ -1121,8 +1125,8 @@ ArmAndActivate(Widget w, XEvent *event, String *params, Cardinal *num_params)
     XmPushButtonCallbackStruct cbs;
     Boolean poppedUp;
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "ArmAndActivate\n"));
-    DEBUGOUT(XdbDebug("MENU", w, "%s:ArmAndActivate(%d)\n",
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "ArmAndActivate\n"));
+    DEBUGOUT(_LtDebugDebug("MENU", w, "%s:ArmAndActivate(%d)\n",
     	__FILE__, __LINE__
     	));
 
@@ -1168,6 +1172,7 @@ ArmAndActivate(Widget w, XEvent *event, String *params, Cardinal *num_params)
 	if (PB_Timer(w) != 0)
 	{
 	    XtRemoveTimeOut(PB_Timer(w));
+	    PB_Timer(w) = 0;
 	}
 
 	PB_Timer(w) = XtAppAddTimeOut(XtWidgetToApplicationContext(w),
@@ -1191,8 +1196,8 @@ EnterWindow(Widget w, XEvent *event, String *params, Cardinal *num_params)
 {
     XmPushButtonCallbackStruct cbs;
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "PushB Enter\n"));
-    DEBUGOUT(XdbDebug("MENU", w, "%s:EnterWindow(%d)\n",
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "PushB Enter\n"));
+    DEBUGOUT(_LtDebugDebug("MENU", w, "%s:EnterWindow(%d)\n",
     	__FILE__, __LINE__
     	));
 
@@ -1240,8 +1245,8 @@ LeaveWindow(Widget w, XEvent *event, String *params, Cardinal *num_params)
 {
     XmPushButtonCallbackStruct cbs;
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "LeaveWindow()\n"));
-    DEBUGOUT(XdbDebug("MENU", w, "%s:LeaveWindow(%d) - %s %s %s\n",
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "LeaveWindow()\n"));
+    DEBUGOUT(_LtDebugDebug("MENU", w, "%s:LeaveWindow(%d) - %s %s %s\n",
     	__FILE__, __LINE__,
     	IN_MENU(w) ? "menu" : "no-menu",
     	IN_MENU(w) && _XmGetInDragMode(w) ? "dragging" : "not-dragging",
@@ -1295,9 +1300,9 @@ ButtonUp(Widget w,
     Boolean validButton, poppedUp;
     XmPushButtonCallbackStruct cbs;
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "%s(%d) - ButtonUp()\n",
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "%s(%d) - ButtonUp()\n",
     	__FILE__, __LINE__));
-    DEBUGOUT(XdbDebug("MENU", w, "%s:ButtonUp(%d)\n",
+    DEBUGOUT(_LtDebugDebug("MENU", w, "%s:ButtonUp(%d)\n",
     	__FILE__, __LINE__
     	));
 
@@ -1312,9 +1317,9 @@ ButtonUp(Widget w,
     {
 	validButton = False;
     }
-    DEBUGOUT(XdbDebug(__FILE__, w, "%s(%d):ButtonUp() - valid %s %ix%i\n",
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "%s(%d):ButtonUp() - valid %s %ix%i\n",
     	__FILE__, __LINE__,
-    	XdbBoolean2String(validButton),
+    	_LtDebugBoolean2String(validButton),
     	event->xbutton.x, event->xbutton.y));
 
     if (!validButton)
@@ -1380,8 +1385,8 @@ ButtonDown(Widget w, XEvent *event, String *params, Cardinal *num_params)
     XmPushButtonCallbackStruct cbs;
     Boolean poppedUp;
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "ButtonDown()\n"));
-    DEBUGOUT(XdbDebug("MENU", w, "%s:ButtonDown(%d)\n",
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "ButtonDown()\n"));
+    DEBUGOUT(_LtDebugDebug("MENU", w, "%s:ButtonDown(%d)\n",
     	__FILE__, __LINE__
     	));
     /* queue events until the next button event */
@@ -1393,7 +1398,7 @@ ButtonDown(Widget w, XEvent *event, String *params, Cardinal *num_params)
 
 	if (!validButton)
 	{
-	    DEBUGOUT(XdbDebug(__FILE__, w, "ButtonDown() - Invalid button\n"));
+	    DEBUGOUT(_LtDebugDebug(__FILE__, w, "ButtonDown() - Invalid button\n"));
 	    return;
 	}
     }
@@ -1402,7 +1407,7 @@ ButtonDown(Widget w, XEvent *event, String *params, Cardinal *num_params)
 
     RC_MenuShellPopdown(w, event, &poppedUp);
     {
-    int i;
+    Cardinal i;
     Widget menu = XtParent(w);
 
 	for (i = 0; i < MGR_NumChildren(menu); i++)
@@ -1448,7 +1453,7 @@ MultiActivate(Widget w, XEvent *event, String *params, Cardinal *num_params)
     XButtonEvent *ev = (XButtonEvent *)event;
     XmPushButtonCallbackStruct cbs;
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "PushB: MultiClick\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "PushB: MultiClick\n"));
 
     if (PB_MultiClick(w) == XmMULTICLICK_KEEP)
     {
@@ -1498,7 +1503,7 @@ MultiActivate(Widget w, XEvent *event, String *params, Cardinal *num_params)
 static void
 MultiArm(Widget w, XEvent *event, String *params, Cardinal *num_params)
 {
-    DEBUGOUT(XdbDebug(__FILE__, w, "PushB: MultiArm\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "PushB: MultiArm\n"));
 
     if (PB_MultiClick(w) == XmMULTICLICK_KEEP)
 	Arm(w, event, NULL, NULL);
@@ -1507,7 +1512,7 @@ MultiArm(Widget w, XEvent *event, String *params, Cardinal *num_params)
 static void
 KeySelect(Widget w, XEvent *event, String *params, Cardinal *num_params)
 {
-    DEBUGOUT(XdbDebug(__FILE__, w, "KeySelect\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "KeySelect\n"));
 
     /* FIX ME */
 }
@@ -1520,7 +1525,6 @@ _XmClearBCompatibility(Widget w)
     _XmObjectUnlock(w);
 }
 
-#if STDC_HEADERS
 static void
 MenuProcEntry(int proc, Widget w,...)
 {
@@ -1528,17 +1532,6 @@ MenuProcEntry(int proc, Widget w,...)
     Cardinal num_params = 0;
 
     va_start(arg_list, w);
-#else
-MenuProcEntry(proc, w, va_alist)
-     int proc;
-     Widget w;
-     va_dcl
-{
-    va_list arglist;
-    Cardinal num_params = 0;
-
-    va_start(arglist);
-#endif
 
     switch (proc)
     {
