@@ -1,8 +1,9 @@
 /**
  *
- * $Id: PushBG.c,v 1.76 1999/11/11 13:43:22 rwscott Exp $
+ * $Id: PushBG.c,v 1.86 2001/05/20 17:49:38 amai Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
+ * Copyright (C) 1995-2001 LessTif Development Team
  *
  * This file is part of the GNU LessTif Library.
  *
@@ -22,9 +23,13 @@
  *
  **/
 
-static const char rcsid[] = "$Id: PushBG.c,v 1.76 1999/11/11 13:43:22 rwscott Exp $";
+static const char rcsid[] = "$Id: PushBG.c,v 1.86 2001/05/20 17:49:38 amai Exp $";
 
 #include <LTconfig.h>
+
+#include <stdarg.h>
+#include <string.h>
+
 #include <XmI/XmI.h>
 
 #include <Xm/XmP.h>
@@ -38,15 +43,8 @@ static const char rcsid[] = "$Id: PushBG.c,v 1.76 1999/11/11 13:43:22 rwscott Ex
 #include <Xm/RepType.h>
 #include <Xm/RowColumnP.h>
 #include <Xm/LabelP.h>
-#include <X11/Xfuncs.h>
 #include <X11/ShellP.h>
 
-#if STDC_HEADERS
-#include <stdarg.h>
-#else
-#include <varargs.h>
-#endif
-
 #include <XmI/DebugUtil.h>
 
 /*
@@ -428,9 +426,9 @@ secondary_object_create(Widget request, Widget new_w,
     XmBaseClassExt *bce;
     XtPointer nsec, rsec;
     XmWidgetExtData ed;
-    int size;
+    Cardinal size;
 
-    DEBUGOUT(XdbDebug(__FILE__, new_w,
+    DEBUGOUT(_LtDebugDebug(__FILE__, new_w,
 		      "PushBGCacheRec %s being initialized.\n",
 		      XtName(new_w)));
 
@@ -460,7 +458,7 @@ secondary_object_create(Widget request, Widget new_w,
     ed->widget = (Widget)nsec;
     ed->reqWidget = (Widget)rsec;
 
-    bcopy(nsec, rsec, size);
+    memcpy(rsec, nsec, size);
     ((XmExtRec *)rsec)->object.self = (Widget)rsec;
 
     _XmPushWidgetExtData(new_w, ed, XmCACHE_EXTENSION);
@@ -475,7 +473,7 @@ secondary_object_create(Widget request, Widget new_w,
 int
 _XmPushBCacheCompare(XtPointer A, XtPointer B)
 {
-    return !bcmp(((XmPushButtonGCacheObjPart *)A),
+    return !memcmp(((XmPushButtonGCacheObjPart *)A),
 		 ((XmPushButtonGCacheObjPart *)B),
 		 sizeof(XmPushButtonGCacheObjPart));
 }
@@ -513,18 +511,18 @@ class_initialize(void)
 			     &labels, &nlabels);
 
     combined = (XtResourceList)XtMalloc(sizeof(XtResource) * ncom);
-    bcopy(labels, combined, nlabels * sizeof(XtResource));
-    bcopy(cache_resources,
-	  &combined[nlabels],
-	  XtNumber(cache_resources) * sizeof(XtResource));
+    memcpy(combined, labels, nlabels * sizeof(XtResource));
+    memcpy(&combined[nlabels],
+           cache_resources,
+	   XtNumber(cache_resources) * sizeof(XtResource));
     XtFree((char *)labels);
 
     xmPushButtonGCacheObjClassRec.object_class.resources = combined;
     xmPushButtonGCacheObjClassRec.object_class.num_resources = ncom;
 
 #if XmVERSION > 1
-    if (! XmeTraitSet(xmPushButtonGadgetClass, XmQTactivatable,
-		&_XmPushBGTraitRec)) {
+    if (! XmeTraitSet((XtPointer)xmPushButtonGadgetClass, XmQTactivatable,
+		(XtPointer)&_XmPushBGTraitRec)) {
 	_XmWarning(NULL,
 	    "XmPushButtonGadget ClassInitialize: XmeTraitSet failed\n");
     }
@@ -595,7 +593,7 @@ static void
 initialize_prehook(Widget request, Widget new_w,
 		   ArgList args, Cardinal *num_args)
 {
-    DEBUGOUT(XdbDebug(__FILE__, new_w, "PushBG InitializePrehook\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, new_w, "PushBG InitializePrehook\n"));
 }
 #endif
 
@@ -605,7 +603,7 @@ initialize_posthook(Widget request, Widget new_w,
 {
     XmWidgetExtData ext;
 
-    DEBUGOUT(XdbDebug(__FILE__, new_w, "PushBG InitializePosthook\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, new_w, "PushBG InitializePosthook\n"));
 
     /* don't let the null fool you */
     LabG_Cache(new_w) = (XmLabelGCacheObjPart *)
@@ -727,8 +725,10 @@ initialize(Widget request, Widget new_w,
 
 	    LabG_TextRect_x(new_w) += margin;
 	    LabG_TextRect_y(new_w) += margin;
-	    LabG_AccTextRect(new_w).x += margin;
-	    LabG_AccTextRect(new_w).y += margin;
+	    if (LabG_AcceleratorText(new_w) != NULL) {
+		LabG_AccTextRect(new_w).x += margin;
+		LabG_AccTextRect(new_w).y += margin;
+	    }
 	}
     }
 
@@ -746,6 +746,7 @@ destroy(Widget w)
     if (PBG_Timer(w) != 0)
     {
 	XtRemoveTimeOut(PBG_Timer(w));
+	PBG_Timer(w) = 0;
     }
 
     XtReleaseGC(w, PBG_FillGC(w));
@@ -759,7 +760,7 @@ set_values_prehook(Widget old, Widget request, Widget new_w,
 {
     XmBaseClassExt *bce;
     XmWidgetExtData ed;
-    int size;
+    Cardinal size;
     XtPointer nsec, rsec;
 
     bce = _XmGetBaseClassExtPtr(XtClass(new_w), XmQmotif);
@@ -779,12 +780,12 @@ set_values_prehook(Widget old, Widget request, Widget new_w,
     ExtObj_LogicalParent(nsec) = new_w;
     ExtObj_ExtensionType(nsec) = XmCACHE_EXTENSION;
 
-    bcopy(LabG_Cache(new_w),
-	  &((XmLabelGCacheObject)nsec)->label_cache,
-	  sizeof(XmLabelGCacheObjPart));
-    bcopy(PBG_Cache(new_w),
-	  &((XmPushButtonGCacheObject)nsec)->pushbutton_cache,
-	  sizeof(XmPushButtonGCacheObjPart));
+    memcpy(&((XmLabelGCacheObject)nsec)->label_cache,
+           LabG_Cache(new_w),
+	   sizeof(XmLabelGCacheObjPart));
+    memcpy(&((XmPushButtonGCacheObject)nsec)->pushbutton_cache,
+           PBG_Cache(new_w),
+	   sizeof(XmPushButtonGCacheObjPart));
 
     ed = (XmWidgetExtData)XtMalloc(sizeof(XmWidgetExtDataRec));
     ed->widget = (Widget)nsec;
@@ -799,7 +800,7 @@ set_values_prehook(Widget old, Widget request, Widget new_w,
 		   (*bce)->secondaryObjectClass->core_class.num_resources,
 		   args, *num_args);
 
-    bcopy(nsec, rsec, size);
+    memcpy(rsec, nsec, size);
 
     LabG_Cache(new_w) = &(((XmPushButtonGCacheObject)nsec)->label_cache);
     LabG_Cache(request) = &(((XmPushButtonGCacheObject)rsec)->label_cache);
@@ -867,7 +868,7 @@ set_values(Widget old, Widget request, Widget new_w,
     Boolean refresh_needed = False;
     Dimension margin, width, height;
 
-    DEBUGOUT(XdbDebug(__FILE__, new_w,
+    DEBUGOUT(_LtDebugDebug(__FILE__, new_w,
 		      "set_values: %i args\n"
 		      "\t    old X %5i Y %5i W %5i H %5i\n"
 		      "\trequest X %5i Y %5i W %5i H %5i\n"
@@ -879,7 +880,7 @@ set_values(Widget old, Widget request, Widget new_w,
 		      XtWidth(request), XtHeight(request),
 		      XtX(new_w), XtY(new_w),
 		      XtWidth(new_w), XtHeight(new_w)));
-    DEBUGOUT(XdbPrintArgList(__FILE__, new_w, args, *num_args, False));
+    DEBUGOUT(_LtDebugPrintArgList(__FILE__, new_w, args, *num_args, False));
 
     if (new_w->core.being_destroyed)
     {
@@ -955,8 +956,10 @@ set_values(Widget old, Widget request, Widget new_w,
 
 	    LabG_TextRect_x(new_w) += margin;
 	    LabG_TextRect_y(new_w) += margin;
-	    LabG_AccTextRect(new_w).x += margin;
-	    LabG_AccTextRect(new_w).y += margin;
+	    if (LabG_AcceleratorText(new_w)) {
+		LabG_AccTextRect(new_w).x += margin;
+		LabG_AccTextRect(new_w).y += margin;
+	    }
 
 	    refresh_needed = True;
 	}
@@ -1068,7 +1071,7 @@ get_values_prehook(Widget new_w, ArgList args, Cardinal *num_args)
 {
     XmBaseClassExt *bce;
     XmWidgetExtData ed;
-    int size;
+    Cardinal size;
     XtPointer nsec;
 
     bce = _XmGetBaseClassExtPtr(XtClass(new_w), XmQmotif);
@@ -1076,12 +1079,12 @@ get_values_prehook(Widget new_w, ArgList args, Cardinal *num_args)
 
     nsec = _XmExtObjAlloc(size);
 
-    bcopy(LabG_Cache(new_w),
-	  &((XmLabelGCacheObject)nsec)->label_cache,
-	  sizeof(XmLabelGCacheObjPart));
-    bcopy(PBG_Cache(new_w),
-	  &((XmPushButtonGCacheObject)nsec)->pushbutton_cache,
-	  sizeof(XmPushButtonGCacheObjPart));
+    memcpy(&((XmLabelGCacheObject)nsec)->label_cache,
+           LabG_Cache(new_w),
+	   sizeof(XmLabelGCacheObjPart));
+    memcpy(&((XmPushButtonGCacheObject)nsec)->pushbutton_cache,
+           PBG_Cache(new_w),
+	   sizeof(XmPushButtonGCacheObjPart));
 
     /*
      * don't do this and ResInd will blow up.
@@ -1126,7 +1129,7 @@ get_values_posthook(Widget new_w, ArgList args, Cardinal *num_args)
 static void
 get_values_hook(Widget w, ArgList args, Cardinal *num_args)
 {
-    DEBUGOUT(XdbDebug(__FILE__, w, "GetValuesHook\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "GetValuesHook\n"));
 }
 #endif
 
@@ -1135,7 +1138,7 @@ expose(Widget w, XEvent *event, Region region)
 {
     Dimension inset;
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "PBG expose: XY %d %d\n",
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "PBG expose: XY %d %d\n",
 		      XtX(w), XtY(w)));
 
     if (!IN_MENU(w))
@@ -1145,7 +1148,7 @@ expose(Widget w, XEvent *event, Region region)
 	    inset += Xm3D_ENHANCE_PIXEL + LabG_Shadow(w)
 		+ (PBG_DefaultButtonShadow(w) << 1);
 
-	DEBUGOUT(XdbDebug(__FILE__, w,
+	DEBUGOUT(_LtDebugDebug(__FILE__, w,
 			  "x %d y %d w %d h %d b %d\n",
 			  inset, inset,
 			  (XtWidth(w) - (inset << 1)),
@@ -1332,8 +1335,8 @@ Activate(Widget w, XEvent *event, String *params, Cardinal *num_params)
     XmPushButtonCallbackStruct cbs;
     XButtonEvent *ev = (XButtonEvent *)event;
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "Activate\n"));
-    DEBUGOUT(XdbDebug("MENU", w, "%s:Activate(%d)\n",
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "Activate\n"));
+    DEBUGOUT(_LtDebugDebug("MENU", w, "%s:Activate(%d)\n",
     	__FILE__, __LINE__
     	));
 
@@ -1388,7 +1391,7 @@ ArmTimeout(XtPointer data, XtIntervalId *id)
 {
     Widget w = (Widget)data;
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "ArmTimeout\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "ArmTimeout\n"));
 
     PBG_Timer(w) = 0;
 
@@ -1405,8 +1408,8 @@ ArmAndActivate(Widget w, XEvent *event, String *params, Cardinal *num_params)
     XmPushButtonCallbackStruct cbs;
     Boolean poppedUp;
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "ArmAndActivate\n"));
-    DEBUGOUT(XdbDebug("MENU", w, "%s:ArmAndActivate(%d)\n",
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "ArmAndActivate\n"));
+    DEBUGOUT(_LtDebugDebug("MENU", w, "%s:ArmAndActivate(%d)\n",
     	__FILE__, __LINE__
     	));
 
@@ -1453,6 +1456,7 @@ ArmAndActivate(Widget w, XEvent *event, String *params, Cardinal *num_params)
 	if (PBG_Timer(w) != 0)
 	{
 	    XtRemoveTimeOut(PBG_Timer(w));
+	    PBG_Timer(w) = 0;
 	}
 
 	PBG_Timer(w) = XtAppAddTimeOut(XtWidgetToApplicationContext(w),
@@ -1468,8 +1472,8 @@ EnterWindow(Widget w,
 {
     XmPushButtonCallbackStruct cbs;
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "EnterWindow()\n"));
-    DEBUGOUT(XdbDebug("MENU", w, "%s:EnterWindow(%d)\n",
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "EnterWindow()\n"));
+    DEBUGOUT(_LtDebugDebug("MENU", w, "%s:EnterWindow(%d)\n",
     	__FILE__, __LINE__
     	));
 
@@ -1513,7 +1517,7 @@ LeaveWindow(Widget w, XEvent *event, String *params, Cardinal *num_params)
 {
     XmPushButtonCallbackStruct cbs;
 
-    DEBUGOUT(XdbDebug("MENU", w, "%s:LeaveWindow(%d) - %s %s %s\n",
+    DEBUGOUT(_LtDebugDebug("MENU", w, "%s:LeaveWindow(%d) - %s %s %s\n",
     	__FILE__, __LINE__,
     	IN_MENU(w) ? "menu" : "no-menu",
     	IN_MENU(w) && _XmGetInDragMode(w) ? "dragging" : "not-dragging",
@@ -1564,9 +1568,9 @@ ButtonUp(Widget w, XEvent *event, String *params, Cardinal *num_params)
     Boolean validButton, poppedUp;
     XmPushButtonCallbackStruct cbs;
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "ButtonUp()\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "ButtonUp()\n"));
 
-    DEBUGOUT(XdbDebug("MENU", w, "%s:ButtonUp(%d)\n",
+    DEBUGOUT(_LtDebugDebug("MENU", w, "%s:ButtonUp(%d)\n",
     	__FILE__, __LINE__
     	));
     /* queue events until the next button event */
@@ -1642,7 +1646,7 @@ ButtonDown(Widget w, XEvent *event, String *params, Cardinal *num_params)
     XmPushButtonCallbackStruct cbs;
     Boolean poppedUp;
 
-    DEBUGOUT(XdbDebug("MENU", w, "%s:ButtonDown(%d)\n",
+    DEBUGOUT(_LtDebugDebug("MENU", w, "%s:ButtonDown(%d)\n",
     	__FILE__, __LINE__
     	));
     /* queue events until the next button event */
@@ -1661,7 +1665,7 @@ ButtonDown(Widget w, XEvent *event, String *params, Cardinal *num_params)
 
     RC_MenuShellPopdown(w, event, &poppedUp);
     {
-    int i;
+    Cardinal i;
     Widget menu = XtParent(w);
 
 	for (i = 0; i < MGR_NumChildren(menu); i++)
@@ -1730,7 +1734,7 @@ input_dispatch(Widget gadget, XEvent *event, Mask event_mask)
     switch (event_mask)
     {
     case XmARM_EVENT:
-	DEBUGOUT(XdbDebug(__FILE__, gadget, "got arm event\n"));
+	DEBUGOUT(_LtDebugDebug(__FILE__, gadget, "got arm event\n"));
 	if (IN_MENU(gadget))
 	{
 	    ButtonDown(gadget, event, NULL, &num_params);
@@ -1756,7 +1760,7 @@ input_dispatch(Widget gadget, XEvent *event, Mask event_mask)
 	break;
 
     case XmACTIVATE_EVENT:
-	DEBUGOUT(XdbDebug(__FILE__, gadget, "got activate event\n"));
+	DEBUGOUT(_LtDebugDebug(__FILE__, gadget, "got activate event\n"));
 	/*
 	 * The test below is expanded so it also checks for the Shell.
 	 * In tearoff menus, the shell will be a TransientShell, thus we'll
@@ -1792,17 +1796,17 @@ input_dispatch(Widget gadget, XEvent *event, Mask event_mask)
 	break;
 
     case XmENTER_EVENT:
-	DEBUGOUT(XdbDebug(__FILE__, gadget, "got an enter window\n"));
+	DEBUGOUT(_LtDebugDebug(__FILE__, gadget, "got an enter window\n"));
 	EnterWindow(gadget, event, NULL, &num_params);
 	break;
 
     case XmLEAVE_EVENT:
-	DEBUGOUT(XdbDebug(__FILE__, gadget, "got a leave window\n"));
+	DEBUGOUT(_LtDebugDebug(__FILE__, gadget, "got a leave window\n"));
 	LeaveWindow(gadget, event, NULL, &num_params);
 	break;
 
     case XmMOTION_EVENT:
-	DEBUGOUT(XdbDebug(__FILE__, gadget, "got a motion event\n"));
+	DEBUGOUT(_LtDebugDebug(__FILE__, gadget, "got a motion event\n"));
 	break;
 
     case XmFOCUS_IN_EVENT:
@@ -1834,7 +1838,6 @@ _XmClearBGCompatibility(Widget w)
     _XmObjectUnlock(w);
 }
 
-#if STDC_HEADERS
 static void
 MenuProcEntry(int proc, Widget w,...)
 {
@@ -1842,17 +1845,6 @@ MenuProcEntry(int proc, Widget w,...)
     Cardinal num_params = 0;
 
     va_start(arg_list, w);
-#else
-MenuProcEntry(proc, w, va_alist)
-     int proc;
-     Widget w;
-     va_dcl
-{
-    va_list arglist;
-    Cardinal num_params = 0;
-
-    va_start(arglist);
-#endif
 
     switch (proc)
     {
