@@ -1,8 +1,9 @@
 /**
  *
- * $Id: RCUtils.c,v 1.105 1999/09/03 00:17:37 rwscott Exp $
+ * $Id: RCUtils.c,v 1.112 2001/05/14 20:54:10 amai Exp $
  *
  * Copyright (C) 1996 Free Software Foundation, Inc.
+ * Copyright (C) 1996-2001 LessTif Development Team
  *
  * This file is part of the GNU LessTif Library.
  *
@@ -22,12 +23,15 @@
  *
  **/
 
-static const char rcsid[] = "$Id: RCUtils.c,v 1.105 1999/09/03 00:17:37 rwscott Exp $";
+static const char rcsid[] = "$Id: RCUtils.c,v 1.112 2001/05/14 20:54:10 amai Exp $";
 
 #include <LTconfig.h>
-#include <XmI/XmI.h>
 
-#include <X11/Xfuncs.h>
+#include <stdio.h>
+#include <string.h>
+#include <stdarg.h>
+
+#include <XmI/XmI.h>
 #include <Xm/XmP.h>
 #include <Xm/BaseClassP.h>
 #include <Xm/CascadeBP.h>
@@ -40,6 +44,7 @@ static const char rcsid[] = "$Id: RCUtils.c,v 1.105 1999/09/03 00:17:37 rwscott
 #include <Xm/MenuUtilP.h>
 #include <Xm/PushBP.h>
 #include <Xm/PushBGP.h>
+#include <Xm/RCUtilsP.h>
 #include <Xm/RepType.h>
 #include <Xm/RowColumnP.h>
 #include <Xm/ScreenP.h>
@@ -50,11 +55,6 @@ static const char rcsid[] = "$Id: RCUtils.c,v 1.105 1999/09/03 00:17:37 rwscott
 #include <Xm/ToggleBGP.h>
 #include <Xm/TransltnsP.h>
 
-#include <stdio.h>
-#include <string.h>
-#include <stdarg.h>
-#include <assert.h>
-
 #include <XmI/DebugUtil.h>
 
 /*
@@ -184,15 +184,12 @@ PreferredSizeHT(Widget rc, XtWidgetGeometry *rcg,
     Dimension current_y = RC_MarginH(rc) + MGR_ShadowThickness(rc);
     Dimension curh = RC_MarginH(rc) + MGR_ShadowThickness(rc);
 
-    if (XdbInDebug(__FILE__, rc))
-    {
-	DEBUGOUT(XdbDebug(__FILE__, rc, "PreferredSizeHT("));
-	DEBUGOUT(XdbDebug0(__FILE__, rc, "%s",
+    DEBUGOUT(_LtDebugDebug(__FILE__, rc, "PreferredSizeHT("));
+    DEBUGOUT(_LtDebugDebug0(__FILE__, rc, "%s",
 			   RC_FromResize(rc) ? "" : "ParentResize "));
-	DEBUGOUT(XdbDebug0(__FILE__, rc, ")\n"));
-	DEBUGOUT(XdbDebug0(__FILE__, rc, "requesting %s\n",
-		XdbWidgetGeometry2String(rcg)));
-    }
+    DEBUGOUT(_LtDebugDebug0(__FILE__, rc, ")\n"));
+    DEBUGOUT(_LtDebugDebug0(__FILE__, rc, "requesting %s\n",
+		_LtDebugWidgetGeometry2String(rcg)));
 
     *max_height = 0, *max_width = 0;
 
@@ -207,7 +204,7 @@ PreferredSizeHT(Widget rc, XtWidgetGeometry *rcg,
 	    continue;
 	}
 
-	DEBUGOUT(XdbDebug2(__FILE__, rc, kid,
+	DEBUGOUT(_LtDebugDebug2(__FILE__, rc, kid,
 			   "PreferredSizeHT: kid geo %d %d %dx%d\n",
 			   kid_geometry->box.x,
 			   kid_geometry->box.y,
@@ -279,9 +276,9 @@ DoLayoutHT(Widget rc, Widget instig, XtWidgetGeometry *instig_request,
     XmRCKidGeometry help_kid_geometry = NULL;
     Dimension help_kid_width = 0;
 
-    DEBUGOUT(XdbDebug2(__FILE__, rc, instig,
+    DEBUGOUT(_LtDebugDebug2(__FILE__, rc, instig,
 		"DoLayoutHT: req %s, max w %d h %d\n",
-		XdbWidgetGeometry2String(instig_request),
+		_LtDebugWidgetGeometry2String(instig_request),
 		max_width, max_height));
 
     if (RC_HelpPb(rc))
@@ -498,7 +495,7 @@ PreferredSizeHC(Widget rc, XtWidgetGeometry *rcg,
     int i, nrows;
     int number_per_row;
 
-    DEBUGOUT(XdbDebug(__FILE__, rc, "PreferredSizeHC()\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, rc, "PreferredSizeHC()\n"));
 
     if (RC_NCol(rc) == 0)
     {
@@ -707,9 +704,9 @@ PreferredSizeVT(Widget rc, XtWidgetGeometry *rcg,
 	    continue;
 	}
 
-	DEBUGOUT(XdbDebug2(__FILE__, rc, kid_geometry->kid,
+	DEBUGOUT(_LtDebugDebug2(__FILE__, rc, kid_geometry->kid,
 			   "%p:PreferredSizeVT: kid %s\n", rc,
-			   XdbWidgetGeometry2String(&kid_geometry->box)));
+			   _LtDebugWidgetGeometry2String(&kid_geometry->box)));
 
 	if (RC_FromResize(rc) &&
 	    current_y + kid_geometry->box.height > XtHeight(rc))
@@ -742,13 +739,13 @@ PreferredSizeVT(Widget rc, XtWidgetGeometry *rcg,
 	}
     }
 
-    DEBUGOUT(XdbDebug(__FILE__, rc,
+    DEBUGOUT(_LtDebugDebug(__FILE__, rc,
 		      "PreferredSizeVT: MaxWidth %d\n", *max_width));
 
     rcg->request_mode = CWWidth | CWHeight | CWBorderWidth;
     rcg->width = curw + RC_MarginW(rc) + MGR_ShadowThickness(rc);
 
-    if ((!RC_ResizeWidth(rc) || RC_FromResize(rc)) && rcg->width > XtWidth(rc))
+    if ((!RC_ResizeWidth(rc) || RC_FromResize(rc)) && rcg->width > XtWidth(rc) && XtWidth(rc) > 0)
     {
     	/* rws 25 Oct 1998 (rowcolumn/test42, xephem, xmgrace)
     	   If we are being resized then we can't be any wider than the RC.
@@ -799,7 +796,7 @@ DoLayoutVT(Widget rc, Widget instig, XtWidgetGeometry *instig_request,
 	    continue;
 	}
 
-	DEBUGOUT(XdbDebug(__FILE__, rc,
+	DEBUGOUT(_LtDebugDebug(__FILE__, rc,
 			  "Child %p:%s width changed to %d\n",
 			  kid, XtName(kid), max_width));
 
@@ -882,7 +879,7 @@ DoLayoutVT(Widget rc, Widget instig, XtWidgetGeometry *instig_request,
 		(max_width + RC_MarginW(rc) > XtWidth(rc)))
 	    {
 		kid_geometry->box.width = max_width;
-		DEBUGOUT(XdbDebug(__FILE__, rc,
+		DEBUGOUT(_LtDebugDebug(__FILE__, rc,
 			  "Child %p:%s AdjLast too wide width changed to %d\n",
 				  kid, XtName(kid), max_width));
 	    }
@@ -890,7 +887,7 @@ DoLayoutVT(Widget rc, Widget instig, XtWidgetGeometry *instig_request,
 	    {
 		kid_geometry->box.width = (XtWidth(rc) -
 		       (current_x + RC_MarginW(rc) + MGR_ShadowThickness(rc)));
-		DEBUGOUT(XdbDebug(__FILE__, rc,
+		DEBUGOUT(_LtDebugDebug(__FILE__, rc,
 		      "Child %p:%s AdjLast not too wide width changed to %d\n",
 				  kid, XtName(kid), kid_geometry->box.width));
 	    }
@@ -925,7 +922,7 @@ PreferredSizeVC(Widget rc, XtWidgetGeometry *rcg,
     int i, ncols;
     int number_per_column;
 
-    DEBUGOUT(XdbDebug(__FILE__, rc,
+    DEBUGOUT(_LtDebugDebug(__FILE__, rc,
 		      "PreferredSizeVC(ncols %d)\n", RC_NCol(rc)));
 
     if (RC_NCol(rc) == 0)
@@ -1001,7 +998,7 @@ DoLayoutVC(Widget rc, Widget instig, XtWidgetGeometry *instig_request,
     Dimension current_x = RC_MarginW(rc) + MGR_ShadowThickness(rc);
     Dimension current_y = RC_MarginH(rc) + MGR_ShadowThickness(rc);
 
-    DEBUGOUT(XdbDebug(__FILE__, rc,
+    DEBUGOUT(_LtDebugDebug(__FILE__, rc,
 		      "LayoutVC(ncols %d)\n", RC_NCol(rc)));
 
     if (RC_NCol(rc) == 0)
@@ -1048,9 +1045,9 @@ DoLayoutVC(Widget rc, Widget instig, XtWidgetGeometry *instig_request,
 		/* T. Straumann: account for border width */
 		instig_request->width -= 2*kid_geometry->box.border_width;
 		instig_request->height -= 2*kid_geometry->box.border_width;
-	    DEBUGOUT(XdbDebug(__FILE__, rc,
+	    DEBUGOUT(_LtDebugDebug(__FILE__, rc,
 			      "LayoutVC: instig set to %s\n",
-			      XdbWidgetGeometry2String(instig_request)));
+			      _LtDebugWidgetGeometry2String(instig_request)));
 	}
 
 	current_y += RC_Spacing(rc) + max_height;
@@ -1097,9 +1094,9 @@ DoLayoutVC(Widget rc, Widget instig, XtWidgetGeometry *instig_request,
 		/* T. Straumann: account for border width */
 		instig_request->width -= 2*kid_geometry->box.border_width;
 		instig_request->height -= 2*kid_geometry->box.border_width;
-	        DEBUGOUT(XdbDebug(__FILE__, rc,
+	        DEBUGOUT(_LtDebugDebug(__FILE__, rc,
 			          "LayoutVC: instig set to %s after AdjLast\n",
-			          XdbWidgetGeometry2String(instig_request)));
+			          _LtDebugWidgetGeometry2String(instig_request)));
 	    }
 	}
     }
@@ -1177,13 +1174,13 @@ _XmRCGetKidGeo(Widget w, Widget instig,
     Dimension mmt = 0, mmb = 0;
     XmRCKidGeometry boxes = NULL;
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "_XmRCGetKidGeo: instig %s:%s, igeo: %s\n",
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "_XmRCGetKidGeo: instig %s:%s, igeo: %s\n",
 		      instig
 			? XtClass(instig)->core_class.class_name
 			: "(null)",
 		      instig ? XtName(instig) : "(null)",
 		      instig_request
-			? XdbWidgetGeometry2String(instig_request)
+			? _LtDebugWidgetGeometry2String(instig_request)
 			: "(no instig request)"));
 
     boxes = (XmRCKidGeometry)XtCalloc(MGR_NumChildren(w) + 1,
@@ -1209,7 +1206,7 @@ _XmRCGetKidGeo(Widget w, Widget instig,
 	    continue;
 	}
 
-	DEBUGOUT(XdbDebug(__FILE__, w,
+	DEBUGOUT(_LtDebugDebug(__FILE__, w,
 			  "_XmRCGetKidGeo: doing change for %d %p:%s\n",
 			  i, kid_geometry->kid, XtName(kid_geometry->kid)));
 
@@ -1236,10 +1233,10 @@ _XmRCGetKidGeo(Widget w, Widget instig,
 			     instig, instig_request, &kid_geometry->box);
 	}
 
-	DEBUGOUT(XdbDebug(__FILE__, w,
+	DEBUGOUT(_LtDebugDebug(__FILE__, w,
 			  "_XmRCGetKidGeo: did change for %p:%s : geo %s\n",
 			  kid_geometry->kid, XtName(kid_geometry->kid),
-			  XdbWidgetGeometry2String(&kid_geometry->box)));
+			  _LtDebugWidgetGeometry2String(&kid_geometry->box)));
 
 	RCC_WasManaged(kid_geometry->kid) = True;
 
@@ -1256,7 +1253,7 @@ _XmRCGetKidGeo(Widget w, Widget instig,
 	if (mmb < RCC_MarginBottom(kid_geometry->kid))
 	    mmb = RCC_MarginBottom(kid_geometry->kid);
 
-	DEBUGOUT(XdbDebug(__FILE__, w,
+	DEBUGOUT(_LtDebugDebug(__FILE__, w,
 			  "_XmRCGetKidGeo: kid margins: %d %d\n",
 			  RCC_MarginTop(kid_geometry->kid),
 			  RCC_MarginBottom(kid_geometry->kid)));
@@ -1352,15 +1349,21 @@ _XmRCGetKidGeo(Widget w, Widget instig,
 	    if (CBG_Submenu(obg))
 	    {
 		find_largest_option_selection(submenu, &width, &height);
-	    width += (LabG_Highlight(obg) << 1)
-		+ (LabG_StringDirection(obg) == XmSTRING_DIRECTION_L_TO_R
-		   ? LabG_MarginRight(obg) : LabG_MarginLeft(obg))
-		+ LabG_Shadow(obg)
-		- Xm3D_ENHANCE_PIXEL;
-	    if (submenu)
-		width += MGR_ShadowThickness(submenu) << 1;
-	    height += (LabG_Highlight(obg) << 1)
-		+ RCC_MarginTop(obg) + RCC_MarginBottom(obg);
+		width += (LabG_Highlight(obg) << 1)
+		    + (LabG_StringDirection(obg) == XmSTRING_DIRECTION_L_TO_R
+		       ? LabG_MarginRight(obg) : LabG_MarginLeft(obg))
+		    + LabG_Shadow(obg)
+		    - Xm3D_ENHANCE_PIXEL;
+		if (submenu)
+		{
+		    width += MGR_ShadowThickness(submenu) << 1;
+		}
+		height += (LabG_Highlight(obg) << 1);
+		/* rowcolumn/test61 */
+		if (RC_Orientation(w) == XmHORIZONTAL)
+		{
+		    height += RCC_MarginTop(obg) + RCC_MarginBottom(obg);
+		}
 	    }
 	    else
 	    {
@@ -1381,10 +1384,10 @@ _XmRCSetKidGeo(XmRCKidGeometry kg, Widget instigator)
     /* Now we actually lay out our children.  */
     while (kg->kid != NULL)
     {
-	DEBUGOUT(XdbDebug(__FILE__, XtParent(kg->kid),
+	DEBUGOUT(_LtDebugDebug(__FILE__, XtParent(kg->kid),
 		     "In _XmRCSetKidGeo: child %p:%s set to (%s): was %d %d\n",
 			  kg->kid, XtName(kg->kid),
-			  XdbWidgetGeometry2String(&kg->box),
+			  _LtDebugWidgetGeometry2String(&kg->box),
 			  XtWidth(kg->kid), XtHeight(kg->kid)));
 
 	if (!XtIsManaged(kg->kid))
@@ -1436,7 +1439,8 @@ _XmRCSetKidGeo(XmRCKidGeometry kg, Widget instigator)
     }
 }
 
-void
+
+extern void
 _XmRCSetMargins(Widget rc)
 {
     int i;
@@ -1450,7 +1454,7 @@ _XmRCSetMargins(Widget rc)
 	     (XmIsLabelGadget(kg->kid) &&
 	      XtClass(kg->kid) != xmLabelGadgetClass)))
 	{
-	    DEBUGOUT(XdbDebug(__FILE__, kg->kid, "Setting margins to %d %d\n",
+	    DEBUGOUT(_LtDebugDebug(__FILE__, kg->kid, "Setting margins to %d %d\n",
 			      kg->margin_top, kg->margin_bottom));
 	    if (RC_Orientation(rc) == XmHORIZONTAL)
 	    {
@@ -1482,7 +1486,7 @@ _XmRCSetMargins(Widget rc)
 					      kg->margin_bottom);
 		    _XmReCacheLabG(kg->kid);
 
-		    DEBUGOUT(XdbDebug(__FILE__, kg->kid,
+		    DEBUGOUT(_LtDebugDebug(__FILE__, kg->kid,
 			     "Margins now %d %d\n",
 			      LabG_MarginLeft(kg->kid),
 			      LabG_MarginRight(kg->kid)));
@@ -1497,6 +1501,7 @@ _XmRCSetMargins(Widget rc)
     }
 }
 
+
 /*
  * If CW is non-null, make sure not to resize it.
  *
@@ -1514,14 +1519,14 @@ _XmRCSetMargins(Widget rc)
  *                              fit in the size given in XtWidth/XtHeight
  *  - else                      means both Test and Resize are off
  */
-XtGeometryResult
+extern XtGeometryResult
 _XmRCAdjustSize(Widget rc, Widget instig, XtWidgetGeometry *instig_request)
 {
     int max_width, max_height;
     XtWidgetGeometry rcg;
     XtGeometryResult ret = XtGeometryNo;
 
-    DEBUGOUT(XdbDebug(__FILE__, rc,
+    DEBUGOUT(_LtDebugDebug(__FILE__, rc,
 		      "_XmRCAdjustSize: current size %d %d instig %p\n",
 		      XtWidth(rc), XtHeight(rc), instig));
 
@@ -1536,8 +1541,8 @@ _XmRCAdjustSize(Widget rc, Widget instig, XtWidgetGeometry *instig_request)
 	return XtGeometryNo;
     }
 
-    DEBUGOUT(XdbDebug(__FILE__, rc, "XmRCAdjustSize (%s) %s\n",
-		      XdbRcType2String(RC_Type(rc)),
+    DEBUGOUT(_LtDebugDebug(__FILE__, rc, "XmRCAdjustSize (%s) %s\n",
+		      _LtDebugRcType2String(RC_Type(rc)),
 		      RC_FromResize(rc) ? "FromResize" : ""));
 
     if (RC_Boxes(rc))
@@ -1602,25 +1607,25 @@ _XmRCAdjustSize(Widget rc, Widget instig, XtWidgetGeometry *instig_request)
     if (!RC_FromResize(rc))
     {
     /* messagebox/test8, test12 budget */
-	DEBUGOUT(XdbDebug(__FILE__, rc,
+	DEBUGOUT(_LtDebugDebug(__FILE__, rc,
 			  "_XmRCAdjustSize: requesting %s from parent %s\n",
-			XdbWidgetGeometry2String(&rcg), XtName(XtParent(rc))));
+			_LtDebugWidgetGeometry2String(&rcg), XtName(XtParent(rc))));
 
-	if (!RC_ResizeWidth(rc) /* && XtIsRealized(rc) */)
+	if (XtWidth(rc) != 0 && !RC_ResizeWidth(rc) /* && XtIsRealized(rc) */)
 	{
 	    rcg.request_mode &= ~CWWidth;
 	    rcg.width = XtWidth(rc);
-	    DEBUGOUT(XdbDebug(__FILE__, rc,
+	    DEBUGOUT(_LtDebugDebug(__FILE__, rc,
 			  "_XmRCAdjustSize: Dis-allow width change requesting %s from parent %s\n",
-			XdbWidgetGeometry2String(&rcg), XtName(XtParent(rc))));
+			_LtDebugWidgetGeometry2String(&rcg), XtName(XtParent(rc))));
 	}
-	if (!RC_ResizeHeight(rc) /* && XtIsRealized(rc) */)
+	if (XtHeight(rc) != 0 && !RC_ResizeHeight(rc) /* && XtIsRealized(rc) */)
 	{
 	    rcg.request_mode &= ~CWHeight;
 	    rcg.height = XtHeight(rc);
-	    DEBUGOUT(XdbDebug(__FILE__, rc,
+	    DEBUGOUT(_LtDebugDebug(__FILE__, rc,
 			  "_XmRCAdjustSize: Dis-allow height change requesting %s from parent %s\n",
-			XdbWidgetGeometry2String(&rcg), XtName(XtParent(rc))));
+			_LtDebugWidgetGeometry2String(&rcg), XtName(XtParent(rc))));
 	}
 #if 0
 	if (!XtIsRealized(rc))
@@ -1642,22 +1647,22 @@ _XmRCAdjustSize(Widget rc, Widget instig, XtWidgetGeometry *instig_request)
 
 	if ((ret = _XmMakeGeometryRequest(rc, &rcg)) == XtGeometryYes)
 	{
-	    DEBUGOUT(XdbDebug(__FILE__, rc,
+	    DEBUGOUT(_LtDebugDebug(__FILE__, rc,
 		       "_XmMakeGeometryRequest for RC said yes: size %d %d.\n",
 			      XtWidth(rc), XtHeight(rc)));
 	}
 	else
 	{
-	    DEBUGOUT(XdbDebug(__FILE__, rc,
+	    DEBUGOUT(_LtDebugDebug(__FILE__, rc,
 		       "_XmMakeGeometryRequest for RC %s: size stays %d %d (%s).\n",
-			      XdbGeometryResult2String(ret),
+			      _LtDebugGeometryResult2String(ret),
 			      XtWidth(rc), XtHeight(rc),
-			      XdbWidgetGeometry2String(&rcg)));
+			      _LtDebugWidgetGeometry2String(&rcg)));
 	}
     }
     else
     {
-	DEBUGOUT(XdbDebug(__FILE__, rc,
+	DEBUGOUT(_LtDebugDebug(__FILE__, rc,
 			  "_XmRCAdjustSize: RC resize is %d x %d %d\n",
 			  XtWidth(rc), XtHeight(rc), XtBorderWidth(rc)));
     }
@@ -1707,7 +1712,8 @@ _XmRCAdjustSize(Widget rc, Widget instig, XtWidgetGeometry *instig_request)
     return ret;
 }
 
-void
+
+extern void
 _XmRCPreferredSize(Widget rc, XtWidgetGeometry *rcg)
 {
     int max_width, max_height;
@@ -1813,6 +1819,7 @@ FindPrivateShell(Widget parent, Widget w)
     return popup;
 }
 
+
 static Widget
 MakePrivateShell(Widget parent, char *name, Arg *args, Cardinal num_args)
 {
@@ -1850,7 +1857,8 @@ MakePrivateShell(Widget parent, char *name, Arg *args, Cardinal num_args)
     return shell;
 }
 
-Widget
+
+extern Widget
 XmCreateMenuBar(Widget parent, char *name,
 		Arg *arglist, Cardinal argcount)
 {
@@ -2092,7 +2100,7 @@ XmMenuPosition(Widget menu, XButtonPressedEvent *event)
 {
     int x, y;
 
-    DEBUGOUT(XdbDebug(__FILE__, NULL,
+    DEBUGOUT(_LtDebugDebug(__FILE__, NULL,
 		      "MENU POSITION %p %s %d %d\n", menu, XtName(menu),
 		      event->x_root, event->y_root));
 
@@ -2203,11 +2211,12 @@ XmRemoveFromPostFromList(Widget menu_wid, Widget widget)
     /* should getting to this point also be an error? */
 }
 
+
 /*
  * The result of XmGetPostedFromWidget when the callback is activated by
  * an accelerator is not defined in the manual.
  */
-Widget
+extern Widget
 XmGetPostedFromWidget(Widget menu)
 {
     if (XmIsRowColumn(menu))
