@@ -1,8 +1,9 @@
 /**
  *
- * $Id: ResInd.c,v 1.17 1999/09/03 00:17:37 rwscott Exp $
+ * $Id: ResInd.c,v 1.25 2001/05/14 19:46:09 amai Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
+ * Copyright (C) 1995-2001 LessTif Development Team
  *
  * This file is part of the GNU LessTif Library.
  *
@@ -28,13 +29,14 @@
  *
  **/
 
-static const char rcsid[] = "$Id: ResInd.c,v 1.17 1999/09/03 00:17:37 rwscott Exp $";
+static const char rcsid[] = "$Id: ResInd.c,v 1.25 2001/05/14 19:46:09 amai Exp $";
 
 #include <LTconfig.h>
-#include <XmI/XmI.h>
 
+#include <string.h>
+
+#include <XmI/XmI.h>
 #include <Xm/XmP.h>
-#include <X11/Xfuncs.h>
 #include <Xm/ManagerP.h>
 #include <Xm/PrimitiveP.h>
 #include <Xm/GadgetP.h>
@@ -65,7 +67,7 @@ CopyToArg(char *src, XtArgVal *dst, unsigned int size)
     if (size <= sizeof(XtArgVal))
     {
 
-	bcopy((char *)src, (char *)&u, (int)size);
+	memcpy(&u, src, size);
 
 	if (size == sizeof(long))
 	{
@@ -95,12 +97,12 @@ CopyToArg(char *src, XtArgVal *dst, unsigned int size)
 	}
 	else
 	{
-	    bcopy((char *)src, (char *)dst, (int)size);
+	    memcpy(dst, src, size);
 	}
     }
     else
     {
-	bcopy((char *)src, (char *)dst, (int)size);
+	memcpy(dst, src, size);
     }
 }
 
@@ -109,7 +111,7 @@ CopyFromArg(XtArgVal src, char *dst, unsigned int size)
 {
     if (size > sizeof(XtArgVal))
     {
-	bcopy((char *)src, (char *)dst, (int)size);
+	memcpy(dst, (void *)src, size);
     }
     else
     {
@@ -158,7 +160,7 @@ CopyFromArg(XtArgVal src, char *dst, unsigned int size)
 	    p = (char *)&src;
 	}
 
-	bcopy(p, dst, (int)size);
+	memcpy(dst, p, size);
     }
 }
 
@@ -254,7 +256,7 @@ _XmBuildResources(XmSyntheticResource **wc_resources_ptr,
     XmSyntheticResource *cmb;
     int ncmb, i, j, wb;
 
-    DEBUGOUT(XdbDebug(__FILE__, NULL, "BuildResources\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, NULL, "BuildResources\n"));
 
     if (*wc_num_resources_ptr == 0)
     {
@@ -265,12 +267,12 @@ _XmBuildResources(XmSyntheticResource **wc_resources_ptr,
 
     ncmb = *wc_num_resources_ptr + sc_num_resources;
     cmb = (XmSyntheticResource *)XtMalloc(ncmb * sizeof(XmSyntheticResource));
-    bcopy((void *)sc_resources,
-	  (void *)cmb,
-	  sc_num_resources * sizeof(XmSyntheticResource));
-    bcopy((void *)*wc_resources_ptr,
-	  (void *)&cmb[sc_num_resources],
-	  *wc_num_resources_ptr * sizeof(XmSyntheticResource));
+    memcpy((void *)cmb,
+           (void *)sc_resources,
+	   sc_num_resources * sizeof(XmSyntheticResource));
+    memcpy((void *)&cmb[sc_num_resources],
+           (void *)*wc_resources_ptr,
+	   *wc_num_resources_ptr * sizeof(XmSyntheticResource));
 
     for (i = 0; i < sc_num_resources; i++)
     {
@@ -284,9 +286,9 @@ _XmBuildResources(XmSyntheticResource **wc_resources_ptr,
 		cmb[i] = cmb[wb + j];
 		if ((ncmb - (wb + j + 1) > 0))
 		{
-		    bcopy((void *)&cmb[wb + j + 1],
-			  (void *)&cmb[wb + j],
-			  (ncmb - (wb + j + 1)) * sizeof(XmSyntheticResource));
+		    memcpy((void *)&cmb[wb + j],
+		           (void *)&cmb[wb + j + 1],
+			   (ncmb - (wb + j + 1)) * sizeof(XmSyntheticResource));
 		}
 		ncmb--;
 	    }
@@ -309,7 +311,7 @@ _XmInitializeSyntheticResources(XmSyntheticResource *resources,
 {
     int i;
 
-    DEBUGOUT(XdbDebug(__FILE__, NULL, "InitializeSyntheticResources\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, NULL, "InitializeSyntheticResources\n"));
 
     if (!resources || !num_resources)
     {
@@ -318,7 +320,7 @@ _XmInitializeSyntheticResources(XmSyntheticResource *resources,
     for (i = 0; i < num_resources; i++)
     {
 	resources[i].resource_name =
-	    (char *)XrmStringToQuark(resources[i].resource_name);
+	    (char *)(long)XrmStringToQuark(resources[i].resource_name);
     }
 }
 
@@ -349,7 +351,7 @@ _XmPrimitiveGetValuesHook(Widget w,
     XmPrimitiveWidgetClass pwc = (XmPrimitiveWidgetClass)XtClass(w);
     XmManagerWidgetClass mwc = (XmManagerWidgetClass)XtClass(XtParent(w));
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "PrimitiveGetValuesHook\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "PrimitiveGetValuesHook\n"));
 
     if (!XmIsPrimitive(w))
     {
@@ -435,7 +437,7 @@ _XmGadgetGetValuesHook(Widget w,
     XmGadgetClass gwc = (XmGadgetClass)XtClass(w);
     XmManagerWidgetClass mwc = (XmManagerWidgetClass)XtClass(XtParent(w));
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "GadgetGetValuesHook\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "GadgetGetValuesHook\n"));
 
     if (!XmIsGadget(w))
     {
@@ -521,7 +523,7 @@ _XmManagerGetValuesHook(Widget w,
     XmManagerWidgetClass pwc = (XmManagerWidgetClass)XtClass(w);
     XmManagerWidgetClass mwc = (XmManagerWidgetClass)XtClass(XtParent(w));
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "ManagerGetValuesHook\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "ManagerGetValuesHook\n"));
 
     if (!XmIsManager(w))
     {
@@ -609,7 +611,7 @@ _XmExtGetValuesHook(Widget w,
     XrmQuark rq;
 
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "ExtObjectGetValuesHook\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "ExtObjectGetValuesHook\n"));
 
     if (!XmIsExtObject(w))
     {
@@ -653,7 +655,7 @@ _XmExtImportArgs(Widget w,
     int i;
     XrmQuark tq;
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "ExtImportArgs\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "ExtImportArgs\n"));
 
     if (!XmIsExtObject(w))
     {
@@ -710,7 +712,7 @@ _XmPrimitiveImportArgs(Widget w,
     int i;
     XrmQuark tq;
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "PrimitiveImportArgs\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "PrimitiveImportArgs\n"));
 
     if (!XmIsPrimitive(w))
     {
@@ -807,7 +809,7 @@ _XmGadgetImportArgs(Widget w,
     int i;
     XrmQuark tq;
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "GadgetImportArgs\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "GadgetImportArgs\n"));
 
     if (!XmIsGadget(w))
     {
@@ -905,7 +907,7 @@ _XmGadgetImportSecondaryArgs(Widget w,
     int i;
     XrmQuark tq;
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "GadgetImportSecondaryArgs\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "GadgetImportSecondaryArgs\n"));
 
     bce = (XmBaseClassExt *)_XmGetBaseClassExtPtr(XtClass(w), XmQmotif);
 
@@ -957,7 +959,7 @@ _XmManagerImportArgs(Widget w,
     int i;
     XrmQuark tq;
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "ManagerImportArgs\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "ManagerImportArgs\n"));
 
     if (!XmIsManager(w))
     {
@@ -1185,7 +1187,8 @@ _XmSortResourceList(XrmResource *list[],
     }
 
     tmp = list[i];
-    bcopy((void *)&list[0], (void *)&list[1], i * sizeof(XrmResource *));
+/*    memcpy((void *)&list[1], (void *)&list[0], i * sizeof(XrmResource *)); */
+    list[i] = list[0];
     list[0] = tmp;
 }
 
@@ -1271,7 +1274,7 @@ _XmUnitTypeDefault(Widget w,
 	}
     }
 
-    val->addr = (XtPointer)&unit_type;
+    val->addr = (XPointer)&unit_type;
 }
 
 void
@@ -1287,10 +1290,10 @@ _XmExportXmString(Widget w, int offset, XtArgVal *value)
 
     *value = (XtArgVal)str;
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "_XmExportXmString(%p, %d, _) %p -> '%s'\n",
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "_XmExportXmString(%p, %d, _) %p -> '%s'\n",
 		w, offset,
 		str,
-		XdbXmString2String(str)));
+		_LtDebugXmString2String(str)));
 }
 
 void
