@@ -1,8 +1,9 @@
 /**
  *
- * $Id: Scale.c,v 1.37 1999/12/23 02:15:49 rwscott Exp $
+ * $Id: Scale.c,v 1.47 2001/05/14 19:46:10 amai Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
+ * Copyright (C) 1995-2001 LessTif Development Team
  *
  * This file is part of the GNU LessTif Library.
  *
@@ -22,11 +23,16 @@
  *
  **/
 
-static const char rcsid[] = "$Id: Scale.c,v 1.37 1999/12/23 02:15:49 rwscott Exp $";
+static const char rcsid[] = "$Id: Scale.c,v 1.47 2001/05/14 19:46:10 amai Exp $";
 
 #include <LTconfig.h>
-#include <XmI/XmI.h>
 
+#include <stdlib.h>
+#include <stdio.h>
+#include <limits.h>
+#include <math.h>
+
+#include <XmI/XmI.h>
 #include <Xm/XmP.h>
 #include <Xm/ScaleP.h>
 #include <Xm/LabelGP.h>
@@ -36,9 +42,6 @@ static const char rcsid[] = "$Id: Scale.c,v 1.37 1999/12/23 02:15:49 rwscott Exp
 #include <Xm/VendorSP.h>
 #include <Xm/VendorSEP.h>
 
-#include <stdio.h>
-#include <limits.h>
-
 #include <XmI/DebugUtil.h>
 
 /* Forward Declarations */
@@ -49,7 +52,7 @@ static const char rcsid[] = "$Id: Scale.c,v 1.37 1999/12/23 02:15:49 rwscott Exp
 #define CHILD_SB	1
 
 /* FIX ME/BUGS
- *   - higlight
+ *   - highlight
  */
 
 /*
@@ -76,32 +79,22 @@ static const char rcsid[] = "$Id: Scale.c,v 1.37 1999/12/23 02:15:49 rwscott Exp
 #define Scale_ShadowThickness(x) MGR_ShadowThickness(x)
 
 static void class_initialize(void);
-
 static void class_part_initialize(WidgetClass w_class);
-
 static void initialize(Widget request, Widget new_w,
 		       ArgList args, Cardinal *num_args);
-
 static void destroy(Widget w);
-
 static void realize(Widget w, XtValueMask *value_mask,
 		    XSetWindowAttributes *attributes);
-
 static Boolean set_values(Widget current, Widget request, Widget new_w,
 			  ArgList args, Cardinal *num_args);
-
 static XtGeometryResult query_geometry(Widget w,
 				       XtWidgetGeometry *request,
 				       XtWidgetGeometry *reply);
-
 static XtGeometryResult geometry_manager(Widget w,
 					 XtWidgetGeometry *request,
 					 XtWidgetGeometry *reply);
-
 static void expose(Widget w, XEvent *event, Region region);
-
 static void resize(Widget w);
-
 static void change_managed(Widget w);
 
 #if 0
@@ -111,36 +104,25 @@ static void insert_child(Widget w);
 static XmKidGeometry _XmScalePreferredSize(Widget scale, Widget child,
 					   XtWidgetGeometry *childgeom,
 					   Dimension *w, Dimension *ht);
-
 static XtGeometryResult _XmScaleGeomRequest(Widget scale,
 					    Dimension *wd, Dimension *ht);
-
 static void _XmScaleLayout(Widget scale, XmKidGeometry boxes,
 			   Widget child, XtWidgetGeometry *childgeom,
 			   Dimension curw, Dimension curh);
-
 static void _XmScaleConfigureChildren(Widget scale,
 				      Widget child,
 				      XmKidGeometry boxes);
-
 static int _XmScaleConvertWidthToSliderSize(Widget w);
-
 static int _XmScaleConvertSCBValueToScaleValue(Widget w, int value);
-
 static int _XmScaleConvertScaleValueToSCBValue(Widget w);
-
 static void _ScaleValueChanged(Widget sb, XtPointer cd, XtPointer data);
-
 static void _ScaleDrag(Widget sb, XtPointer cd, XtPointer data);
-
 static void computeValueSize(Widget w);
-
 static void showValue(Widget w, int scb_value, int scale_value);
 
 #if 0
 static void GetFocus(Widget w, XEvent *event,
 		     String *params, Cardinal *num_params);
-
 static void LoseFocus(Widget w, XEvent *event,
 		      String *params, Cardinal *num_params);
 #endif
@@ -203,7 +185,7 @@ static XtResource resources[] =
     {
 	XmNdecimalPoints, XmCDecimalPoints, XmRShort,
 	sizeof(short), Offset(decimal_points),
-	XmRImmediate, (XtPointer)0
+	XmRImmediate, (XtPointer)(short)0
     },
     {
 	XmNscaleWidth, XmCScaleWidth, XmRHorizontalDimension,
@@ -238,7 +220,7 @@ static XtResource resources[] =
 #if XmVERSION > 1
     {
 	XmNeditable, XmCEditable, XmRBoolean,
-	sizeof(int), Offset(editable),
+	sizeof(Boolean), Offset(editable),
 	XmRImmediate, (XtPointer)True
     },
 #endif
@@ -279,7 +261,7 @@ static XtActionsRec actions[] =
     {"FocusOut", LoseFocus}
 };
 #endif
-/* *INDENT-OFF* */
+
 static XmBaseClassExtRec _XmScaleCoreClassExtRec = {
     /* next_extension            */ NULL,
     /* record_type               */ NULLQUARK,                             
@@ -386,17 +368,19 @@ XmScaleClassRec xmScaleClassRec =
 	/* extension */ NULL,
     },
 };
-/* *INDENT-ON* */
+
 
 
 WidgetClass xmScaleWidgetClass = (WidgetClass)&xmScaleClassRec;
 
+
 static void
 class_initialize(void)
 {
     _XmScaleCoreClassExtRec.record_type = XmQmotif;
 }
 
+
 static void
 class_part_initialize(WidgetClass widget_class)
 {
@@ -427,6 +411,7 @@ class_part_initialize(WidgetClass widget_class)
     _XmFastSubclassInit(widget_class, XmSCALE_BIT);
 }
 
+
 static void
 CreateForegroundGC(Widget w)
 {
@@ -460,7 +445,7 @@ initialize(Widget request, Widget new_w,
     Dimension wd, ht;
     */
 
-    DEBUGOUT(XdbDebug(__FILE__, new_w,
+    DEBUGOUT(_LtDebugDebug(__FILE__, new_w,
 		      "%s:initialize(%d) - %i args\n"
 		      "\trequest X %5i Y %5i W %5i H %5i\n"
 		      "\t  new_w X %5i Y %5i W %5i H %5i\n",
@@ -470,7 +455,7 @@ initialize(Widget request, Widget new_w,
 		      XtWidth(request), XtHeight(request),
 		      XtX(new_w), XtY(new_w),
 		      XtWidth(new_w), XtHeight(new_w)));
-    DEBUGOUT(XdbPrintArgList(__FILE__, new_w, args, *num_args, False));
+    DEBUGOUT(_LtDebugPrintArgList(__FILE__, new_w, args, *num_args, False));
 
     if (Scale_ScaleWidth(new_w) == 0)
     {
@@ -594,13 +579,20 @@ initialize(Widget request, Widget new_w,
 #endif
 }
 
+
 static void
 destroy(Widget w)
 {
+
+    if (Scale_Title(w))
+     {
+         XmStringFree(Scale_Title(w));
+     }
     XmFontListFree(Scale_FontList(w));
     XtDestroyGC(Scale_ForegroundGC(w));
 }
 
+
 /*
  * called when the user changes a resource
  */
@@ -617,7 +609,7 @@ set_values(Widget old, Widget request, Widget new_w,
     Widget sb, lab;
     int resize = False;
 
-    DEBUGOUT(XdbDebug(__FILE__, new_w, "XmScale set_values()\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, new_w, "XmScale set_values()\n"));
 
     sb = ((XmScaleWidget)new_w)->composite.children[CHILD_SB];
     lab = ((XmScaleWidget)new_w)->composite.children[CHILD_LAB];
@@ -721,8 +713,12 @@ set_values(Widget old, Widget request, Widget new_w,
 	Scale_ScaleHeight(new_w) != Scale_ScaleHeight(old))
     {
 	argc = 0;
-	XtSetArg(argl[argc], XmNwidth, Scale_ScaleWidth(new_w)); argc++;
-	XtSetArg(argl[argc], XmNheight, Scale_ScaleHeight(new_w)); argc++;
+	if (Scale_ScaleWidth(new_w) != Scale_ScaleWidth(old) ) {
+	    XtSetArg(argl[argc], XmNwidth, Scale_ScaleWidth(new_w)); argc++;
+	}
+	if ( Scale_ScaleHeight(new_w) != Scale_ScaleHeight(old) ) {
+	    XtSetArg(argl[argc], XmNheight, Scale_ScaleHeight(new_w)); argc++;
+	}
 
 	XtSetValues(sb, argl, argc);
        
@@ -791,12 +787,13 @@ set_values(Widget old, Widget request, Widget new_w,
     return refresh;
 }
 
+
 static void
 expose(Widget w,
        XEvent *event,
        Region region)
 {
-    DEBUGOUT(XdbDebug(__FILE__, w, "XmScale expose()\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "XmScale expose()\n"));
 
     _XmRedisplayGadgets(w, event, region);
 
@@ -808,6 +805,7 @@ expose(Widget w,
     }
 }
 
+
 /*
  * called when our parent wants to find out how we would like to look.  It
  * doesn't have to honor our preference, though.
@@ -822,7 +820,7 @@ query_geometry(Widget w, XtWidgetGeometry *request, XtWidgetGeometry *reply)
 
 #define Wants(x)	((wants.request_mode & x) == x)
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "XmScale %s QueryGeometry\n", XtName(w)));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "XmScale %s QueryGeometry\n", XtName(w)));
 
     wants = *request;
 
@@ -872,7 +870,7 @@ query_geometry(Widget w, XtWidgetGeometry *request, XtWidgetGeometry *reply)
     XtWidgetGeometry wants;
     Dimension wd, ht;
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "XmScale %s QueryGeometry\n", XtName(w)));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "XmScale %s QueryGeometry\n", XtName(w)));
 
     wants = *request;
 
@@ -885,6 +883,7 @@ query_geometry(Widget w, XtWidgetGeometry *request, XtWidgetGeometry *reply)
 #endif
 }
 
+
 /*
  * called when our parent is resizing us.  We have no choice but to obey.
  */
@@ -894,7 +893,7 @@ resize(Widget w)
     XmKidGeometry boxes;
     Dimension wd, ht;
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "Scale %s Resize: x %d y %d w %d h %d\n",
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "Scale %s Resize: x %d y %d w %d h %d\n",
 		      XtName(w), XtX(w), XtY(w), XtWidth(w), XtHeight(w)));
 
     Scale_SliderSize(w) = _XmScaleConvertWidthToSliderSize(w);
@@ -910,6 +909,7 @@ resize(Widget w)
     XtFree((char *)boxes);
 }
 
+
 /*
  * called when one of our children wants to change.  We control whether
  * the child can, or not
@@ -923,7 +923,7 @@ geometry_manager(Widget w, XtWidgetGeometry *request, XtWidgetGeometry *reply)
     XmKidGeometry boxes;
     XtWidgetGeometry wants;
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "XmScale geometry_manager()\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "XmScale geometry_manager()\n"));
 
     wants = *request;
 
@@ -947,7 +947,7 @@ geometry_manager(Widget w, XtWidgetGeometry *request, XtWidgetGeometry *reply)
 
     *reply = *request;
 
-    reply->request_mode = CWWidth|CWHeight;
+    reply->request_mode &= CWWidth|CWHeight;
 
     if (Wants(CWWidth) && Wants(CWHeight) &&
 	wants.width == reply->width && wants.height == reply->height)
@@ -971,10 +971,19 @@ geometry_manager(Widget w, XtWidgetGeometry *request, XtWidgetGeometry *reply)
 	return XtGeometryYes;
     }
 
+    if (Wants(CWWidth) && wants.width == reply->width)
+    {
+    	reply->request_mode &= ~CWWidth;
+    }
+    if (Wants(CWHeight) && wants.height == reply->height)
+    {
+    	reply->request_mode &= ~CWHeight;
+    }
     return XtGeometryAlmost;
 #undef Wants
 }
 
+
 /*
  * called when our Window is allocated.  We may want to change our layout,
  * and we can ask our parent to allow this
@@ -988,7 +997,7 @@ realize(Widget w, XtValueMask *value_mask, XSetWindowAttributes *attributes)
     XmKidGeometry boxes;
     */
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "XmScale realize()\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "XmScale realize()\n"));
 
 #define superclass (&xmManagerClassRec)
     (*superclass->core_class.realize) (w, value_mask, attributes);
@@ -1010,6 +1019,7 @@ realize(Widget w, XtValueMask *value_mask, XSetWindowAttributes *attributes)
 #endif
 }
 
+
 /*
  * one of our children has become managed or unmanaged.  We need to relayout
  */
@@ -1019,7 +1029,7 @@ change_managed(Widget w)
     Dimension wd, ht;
     XmKidGeometry boxes;
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "XmScale change_managed()\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "XmScale change_managed()\n"));
 
     wd = ht = 0;
     boxes = _XmScalePreferredSize(w, NULL, NULL, &wd, &ht);
@@ -1035,6 +1045,7 @@ change_managed(Widget w)
     _XmNavigChangeManaged(w);
 }
 
+
 #if 0
 /*
  * a child has been added
@@ -1046,13 +1057,14 @@ insert_child(Widget w)
     (*superclass->composite_class.insert_child) (w);
 #undef  superclass
 
-    DEBUGOUT(XdbDebug(__FILE__, w,
+    DEBUGOUT(_LtDebugDebug(__FILE__, w,
 		      "Scale %s: insert child %s\n",
 		      XtName(XtParent(w)), XtName(w)));
 }
 #endif
 
-Widget
+
+extern Widget
 XmCreateScale(Widget parent,
 	      char *name,
 	      Arg *arglist,
@@ -1065,14 +1077,16 @@ XmCreateScale(Widget parent,
 			  argcount);
 }
 
-void
+
+extern void
 XmScaleGetValue(Widget widget,
 		int *value_return)
 {
     *value_return = Scale_Value(widget);
 }
 
-void
+
+extern void
 XmScaleSetValue(Widget widget,
 		int value)
 {
@@ -1099,6 +1113,7 @@ XmScaleSetValue(Widget widget,
     }
 }
 
+
 static void
 valueToString(Widget w, char *buf)
 {
@@ -1656,6 +1671,7 @@ printf("COMPUTED %d %d : %d %d\n", curw, cury, incw, inch);
     return boxes;
 }
 
+
 /*
  * if we can, change our size to match
  */
@@ -1669,7 +1685,7 @@ _XmScaleGeomRequest(Widget scale, Dimension *wd, Dimension *ht)
     request.width = *wd;
     request.height = *ht;
 
-    DEBUGOUT(XdbDebug(__FILE__, scale, "_XmScaleGeomRequest(ing) (%d %d)\n",
+    DEBUGOUT(_LtDebugDebug(__FILE__, scale, "_XmScaleGeomRequest(ing) (%d %d)\n",
 		      *wd, *ht));
 
     result = _XmMakeGeometryRequest(scale, &request);
@@ -1685,12 +1701,13 @@ _XmScaleGeomRequest(Widget scale, Dimension *wd, Dimension *ht)
 	*ht = XtHeight(scale);
     }
 
-    DEBUGOUT(XdbDebug(__FILE__, scale, "_XmScaleLayout (%d %d) => %s\n",
-		      *wd, *ht, XdbGeometryResult2String(result)));
+    DEBUGOUT(_LtDebugDebug(__FILE__, scale, "_XmScaleLayout (%d %d) => %s\n",
+		      *wd, *ht, _LtDebugGeometryResult2String(result)));
 
     return result;
 }
 
+
 static void
 _XmScaleLayout(Widget scale, XmKidGeometry boxes,
 	       Widget child, XtWidgetGeometry *childgeom,
@@ -1739,9 +1756,9 @@ _XmScaleLayout(Widget scale, XmKidGeometry boxes,
 	{
 	    for (i = 2; i < num_children; i++)
 	    {
-		boxes[i].box.width = (Dimension)((float)boxes[i].box.width /
-						 (float)curw *
-						 (float)XtWidth(scale));
+		boxes[i].box.width = (Dimension)((double)boxes[i].box.width /
+						 (double)curw *
+						 (double)XtWidth(scale));
 	    }
 	}
 #endif
@@ -1832,9 +1849,9 @@ _XmScaleLayout(Widget scale, XmKidGeometry boxes,
 	if (XtWidth(scale) > curw)
 	{
 	    for (i = 2; i < num_children; i++)
-		boxes[i].box.height = (Dimension)((float)boxes[i].box.height /
-						  (float)curw *
-						  (float)XtWidth(scale));
+		boxes[i].box.height = (Dimension)((double)boxes[i].box.height /
+						  (double)curw *
+						  (double)XtWidth(scale));
 	}
 #endif
 
@@ -1901,8 +1918,8 @@ _XmScaleConfigureChildren(Widget scale, Widget child, XmKidGeometry boxes)
     _XmSetKidGeo(boxes, child);
 
     Scale_SliderSize(scale) = _XmScaleConvertWidthToSliderSize(scale);
-    inc = (int)((float)(SCB_MAX - SCB_MIN - Scale_SliderSize(scale)) /
-		(float)(Scale_Maximum(scale) - Scale_Minimum(scale)) +
+    inc = (int)((double)(SCB_MAX - SCB_MIN - Scale_SliderSize(scale)) /
+		(double)(Scale_Maximum(scale) - Scale_Minimum(scale)) +
 		0.5);
     inc = _XmMax(inc, 1);	/* FIX ME */
     p_inc = inc * Scale_ScaleMultiple(scale);
@@ -1913,6 +1930,7 @@ _XmScaleConfigureChildren(Widget scale, Widget child, XmKidGeometry boxes)
 			 inc, p_inc, False);
 }
 
+
 static void
 computeValueSize(Widget w)
 {
@@ -1952,6 +1970,7 @@ computeValueSize(Widget w)
         Scale_FontStruct(w)->max_bounds.descent;
 }
 
+
 static void
 showValue(Widget w, int scb_value, int scale_value)
 {
@@ -2026,6 +2045,7 @@ showValue(Widget w, int scb_value, int scale_value)
     }
 }
 
+
 static void
 _ScaleValueChanged(Widget sb, XtPointer cd, XtPointer data)
 {
@@ -2053,6 +2073,7 @@ _ScaleValueChanged(Widget sb, XtPointer cd, XtPointer data)
     }
 }
 
+
 static void
 _ScaleDrag(Widget sb, XtPointer cd, XtPointer data)
 {
@@ -2061,7 +2082,7 @@ _ScaleDrag(Widget sb, XtPointer cd, XtPointer data)
     int scale_value;
 
 #if XmVERSION > 1
-    if (! Scale_Editable(sb))
+    if (! Scale_Editable(XtParent(sb)))
 	return;
 #endif
 
@@ -2084,13 +2105,14 @@ _ScaleDrag(Widget sb, XtPointer cd, XtPointer data)
     }
 }
 
+
 #if 0
 static void
 GetFocus(Widget w, XEvent *event, String *params, Cardinal *num_params)
 {
     XmScrollBarWidget sb;
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "GetFocus\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "GetFocus\n"));
 
     sb = (XmScrollBarWidget)(((XmScaleWidget)w)->composite.children[CHILD_SB]);
 
@@ -2098,12 +2120,13 @@ GetFocus(Widget w, XEvent *event, String *params, Cardinal *num_params)
 		     params, *num_params);
 }
 
+
 static void
 LoseFocus(Widget w, XEvent *event, String *params, Cardinal *num_params)
 {
     XmScrollBarWidget sb;
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "LoseFocus\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "LoseFocus\n"));
 
     sb = (XmScrollBarWidget)(((XmScaleWidget)w)->composite.children[CHILD_SB]);
 
@@ -2112,6 +2135,7 @@ LoseFocus(Widget w, XEvent *event, String *params, Cardinal *num_params)
 }
 #endif
 
+
 static int
 _XmScaleConvertWidthToSliderSize(Widget w)
 {
@@ -2141,9 +2165,9 @@ _XmScaleConvertWidthToSliderSize(Widget w)
 	    if (XtWidth(sb) == 2 * (ht + st))
 	    {
 		/* Note: M*tif 1.2.2 SIGFPE's on this -- PvH */
-		ret = (int)((float)(SCB_MAX - SCB_MIN) /
-			    (float)(Scale_Maximum(w) - Scale_Minimum(w)) *
-			    (float)HSLIDERSIZE);
+		ret = (int)((double)(SCB_MAX - SCB_MIN) /
+			    (double)(Scale_Maximum(w) - Scale_Minimum(w)) *
+			    (double)HSLIDERSIZE);
 	    }
 	    else if (XtWidth(sb) < HSLIDERSIZE)
 	    {
@@ -2183,29 +2207,49 @@ _XmScaleConvertWidthToSliderSize(Widget w)
     return ret;
 }
 
+
 static int
 _XmScaleConvertSCBValueToScaleValue(Widget w, int value)
 {
+    double roundarg;
+    int result;
+
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "_XmScaleConvertSCBValueToScaleValue(,%i)\n",
+                       value));
     if ((SCB_MAX - SCB_MIN - Scale_SliderSize(w)) == 0)
     {
 	return Scale_Minimum(w);
     }
 
-    return (int)((float)(Scale_Maximum(w) - Scale_Minimum(w)) 
-		* (float)value 
-		/ (float)(SCB_MAX - SCB_MIN - Scale_SliderSize(w))
-		+ (float)Scale_Minimum(w) );
+    roundarg = (double)(Scale_Maximum(w) - Scale_Minimum(w)) 
+	     * (double)value 
+	     / (double)(SCB_MAX - SCB_MIN - Scale_SliderSize(w))
+	     + (double)Scale_Minimum(w);
+
+    result = (int)(roundarg + 0.5);
+
+    return result;
 }
 
+
 static int
 _XmScaleConvertScaleValueToSCBValue(Widget w)
 {
-    return (int)((float)(Scale_Value(w) - Scale_Minimum(w)) 
-		* (float)(SCB_MAX - SCB_MIN - Scale_SliderSize(w))
-		/ (float)(Scale_Maximum(w) - Scale_Minimum(w)) 
-		);
+
+    double roundarg;
+    int result;
+
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "_XmScaleConvertScaleValueToSCBValue()\n"));
+    roundarg = (double)(Scale_Value(w) - Scale_Minimum(w)) 
+	     * (double)(SCB_MAX - SCB_MIN - Scale_SliderSize(w))
+	     / (double)(Scale_Maximum(w) - Scale_Minimum(w));
+    /* amai: is this the correct formula? 
+             looks like the old one in _XmScaleConvertSCBValueToScaleValue() */
+    result = (int)(roundarg);
+    return result;
 }
 
+
 static void
 _XmScaleProcessingDirectionDefault(Widget w, int offset, XrmValue *val)
 {
@@ -2221,6 +2265,5 @@ _XmScaleProcessingDirectionDefault(Widget w, int offset, XrmValue *val)
 	direction = XmMAX_ON_RIGHT;	/* FIX ME */
     }
 
-    val->addr = (XtPointer)&direction;
+    val->addr = (XPointer)&direction;
 }
-
