@@ -1,8 +1,9 @@
 /**
  *
- * $Id: ScrollBar.c,v 1.55 1999/09/03 00:17:39 rwscott Exp $
+ * $Id: ScrollBar.c,v 1.60 2001/05/14 19:46:10 amai Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
+ * Copyright (C) 1995-2001 LessTif Development Team
  *
  * This file is part of the GNU LessTif Library.
  *
@@ -22,21 +23,22 @@
  *
  **/
 
-static const char rcsid[] = "$Id: ScrollBar.c,v 1.55 1999/09/03 00:17:39 rwscott Exp $";
+static const char rcsid[] = "$Id: ScrollBar.c,v 1.60 2001/05/14 19:46:10 amai Exp $";
 
 #include <LTconfig.h>
-#include <XmI/XmI.h>
 
+#include <stdlib.h>
+#include <limits.h>
+
+#include <XmI/XmI.h>
 #include <Xm/XmP.h>
 #include <Xm/ScrollBarP.h>
 #include <Xm/ScaleP.h>
-#include <limits.h>
 #include <Xm/ScreenP.h>
 #include <Xm/TransltnsP.h>
 #include <Xm/ScrolledWP.h>
 #include <Xm/RepType.h>
 #include <Xm/VirtKeys.h>
-#include <stdlib.h>
 
 #include <XmI/DebugUtil.h>
 
@@ -596,7 +598,7 @@ CreateUnavailableGC(Widget w)
 static void
 initialize(Widget request, Widget new_w, ArgList args, Cardinal *num_args)
 {
-    DEBUGOUT(XdbDebug(__FILE__, new_w,
+    DEBUGOUT(_LtDebugDebug(__FILE__, new_w,
 		      "%s(%i) - initialize: %i args\n"
 		      "\trequest X %5i Y %5i W %5i H %5i\n"
 		      "\t  new_w X %5i Y %5i W %5i H %5i\n",
@@ -606,7 +608,7 @@ initialize(Widget request, Widget new_w, ArgList args, Cardinal *num_args)
 		      XtWidth(request), XtHeight(request),
 		      XtX(new_w), XtY(new_w),
 		      XtWidth(new_w), XtHeight(new_w)));
-    DEBUGOUT(XdbPrintArgList(__FILE__, new_w, args, *num_args, False));
+    DEBUGOUT(_LtDebugPrintArgList(__FILE__, new_w, args, *num_args, False));
 
     /*
      * First check if the reptypes are valid
@@ -681,7 +683,7 @@ initialize(Widget request, Widget new_w, ArgList args, Cardinal *num_args)
 static void
 check_constraints(Widget sw, Widget current)
 {
-    DEBUGOUT(XdbDebug(__FILE__, sw,
+    DEBUGOUT(_LtDebugDebug(__FILE__, sw,
 	     "Specified scrollbar values: min=%d, max=%d, val=%d, sl_size=%d\n"
 		      " inc=%d, p_inc=%d, p_dir=%d\n",
 		      SCB_Minimum(sw), SCB_Maximum(sw), SCB_Value(sw),
@@ -809,7 +811,7 @@ check_constraints(Widget sw, Widget current)
 	/* Yes, 75 is not equal to the default. */
     }
 
-    DEBUGOUT(XdbDebug(__FILE__, sw,
+    DEBUGOUT(_LtDebugDebug(__FILE__, sw,
 		   "Got scrollbar values: min=%d, max=%d, val=%d, sl_size=%d\n"
 		      " inc=%d, p_inc=%d, p_dir=%d\n",
 		      SCB_Minimum(sw), SCB_Maximum(sw), SCB_Value(sw),
@@ -827,8 +829,10 @@ destroy(Widget w)
 			     SCB_Pixmap(w));
     }
 
-    if (SCB_Timer(w))
+    if (SCB_Timer(w)) {
 	XtRemoveTimeOut(SCB_Timer(w));
+	SCB_Timer(w) = 0;
+    }
 
     XtReleaseGC(w, SCB_UnavailableGC(w));
     XtReleaseGC(w, SCB_ForegroundGC(w));
@@ -849,7 +853,7 @@ set_values(Widget current, Widget request, Widget new_w,
 	Boolean redrawSlider = CNE(Prim_Foreground) || CNE(Prim_TopShadowColor) ||
 		CNE(Prim_BottomShadowColor);
 
-    DEBUGOUT(XdbDebug(__FILE__, new_w,
+    DEBUGOUT(_LtDebugDebug(__FILE__, new_w,
 		      "%s(%d) - set_values: %i args\n"
 		      "\t    old X %5i Y %5i W %5i H %5i\n"
 		      "\trequest X %5i Y %5i W %5i H %5i\n"
@@ -862,7 +866,7 @@ set_values(Widget current, Widget request, Widget new_w,
 		      XtWidth(request), XtHeight(request),
 		      XtX(new_w), XtY(new_w),
 		      XtWidth(new_w), XtHeight(new_w)));
-    DEBUGOUT(XdbPrintArgList(__FILE__, new_w, args, *num_args, False));
+    DEBUGOUT(_LtDebugPrintArgList(__FILE__, new_w, args, *num_args, False));
 
     if (!XmRepTypeValidValue(XmRepTypeGetId(XmROrientation),
 			 SCB_Orientation(new_w),
@@ -1246,7 +1250,7 @@ resize(Widget sw)
 static void
 realize(Widget w, XtValueMask *values, XSetWindowAttributes *attributes)
 {
-    DEBUGOUT(XdbDebug(__FILE__, w, "%s:realize(%d) %dx%d\n",
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "%s:realize(%d) %dx%d\n",
     	__FILE__, __LINE__,
     	XtWidth(w), XtHeight(w)));
 
@@ -1314,7 +1318,7 @@ query_geometry(Widget w,
 static void
 scrollbar_unhighlight(Widget w)
 {
-    DEBUGOUT(XdbDebug(__FILE__, w, "ScrollBarUnHighlight\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "ScrollBarUnHighlight\n"));
 
 #define superclass (&xmPrimitiveClassRec)
     (*superclass->primitive_class.border_unhighlight) (w);
@@ -1324,7 +1328,7 @@ scrollbar_unhighlight(Widget w)
 static void
 scrollbar_highlight(Widget w)
 {
-    DEBUGOUT(XdbDebug(__FILE__, w, "ScrollBarHighlight\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "ScrollBarHighlight\n"));
 
 #define superclass (&xmPrimitiveClassRec)
     (*superclass->primitive_class.border_highlight) (w);
@@ -1595,7 +1599,7 @@ Select(Widget sw,
     int eventPos;
     int change_type;
 
-    DEBUGOUT(XdbDebug(__FILE__, sw, "Select\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, sw, "Select\n"));
 
     if (ev->button == Button1 && !XmIsScrolledWindow(XtParent(sw)))
     {
@@ -1694,8 +1698,10 @@ Select(Widget sw,
     /* handle non-slider movement */
     if (change_type)
     {
-	if (SCB_Timer(sw))
+	if (SCB_Timer(sw)) {
 	    XtRemoveTimeOut(SCB_Timer(sw));
+	    SCB_Timer(sw) = 0;
+	}
 	XSync(XtDisplay(sw), False);
 	SCB_Timer(sw) = XtAppAddTimeOut(XtWidgetToApplicationContext(sw),
 					SCB_InitialDelay(sw),
@@ -1718,7 +1724,7 @@ Release(Widget sw,
     int arrow1, arrow2;
     XmScrollBarCallbackStruct cbs;
 
-    DEBUGOUT(XdbDebug(__FILE__, sw, "Release\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, sw, "Release\n"));
 
     SCB_Flags(sw) &= ~FLG_DRAG;
 
@@ -1746,7 +1752,7 @@ Release(Widget sw,
 	cbs.value = EXPORTED_VALUE(sw);
 	cbs.pixel = (SCB_Orientation(sw) == XmHORIZONTAL ? ev->x : ev->y);
 
-	DEBUGOUT(XdbDebug(__FILE__, sw, "Release: ValueChangedCallback\n"));
+	DEBUGOUT(_LtDebugDebug(__FILE__, sw, "Release: ValueChangedCallback\n"));
 
 	XFlush(XtDisplay(sw));
 	XtCallCallbackList(sw, SCB_ValueChangedCallback(sw), &cbs);
@@ -1771,7 +1777,7 @@ Moved(Widget sw,
 
     old_value = SCB_Value(sw);
 
-    DEBUGOUT(XdbDebug(__FILE__, sw, "Moved\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, sw, "Moved\n"));
 
     if (SCB_Orientation(sw) == XmHORIZONTAL)
     {
@@ -1859,7 +1865,7 @@ Moved(Widget sw,
     /* only call dragcallback if value really changed */
     if (SCB_Value(sw) != old_value)
     {
-	DEBUGOUT(XdbDebug(__FILE__, sw, "Moved: DragCallback\n"));
+	DEBUGOUT(_LtDebugDebug(__FILE__, sw, "Moved: DragCallback\n"));
 
 	cbs.event = event;
 	cbs.value = EXPORTED_VALUE(sw);
@@ -1878,7 +1884,7 @@ TopOrBottom(Widget sw,
 {
     int change_type;
 
-    DEBUGOUT(XdbDebug(__FILE__, sw, "TopOrBottom\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, sw, "TopOrBottom\n"));
 
     if (event->type == KeyPress)
     {
@@ -2064,7 +2070,7 @@ IncrementDownOrRight(Widget w,
 {
     int which;
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "IncrementDownOrRight\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "IncrementDownOrRight\n"));
 
     if (*num_params != 1)
     {
@@ -2088,7 +2094,7 @@ IncrementUpOrLeft(Widget w,
 {
     int which;
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "IncrementUpOrLeft\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "IncrementUpOrLeft\n"));
 
     if (*num_params != 1)
     {
@@ -2294,7 +2300,7 @@ _XmScrollBarProcessingDirectionDefault(Widget w,
     static unsigned char direction;
 
     direction = default_processing_direction(w);
-    val->addr = &direction;
+    val->addr = (XPointer)&direction;
 }
 
 static void
@@ -2319,7 +2325,7 @@ _XmScrollBarTraversalOnDefault(Widget w,
     }
 #endif
 
-    val->addr = (XtPointer)&traversalOn;
+    val->addr = (XPointer)&traversalOn;
 }
 
 /* till 27.5.1999: it seems that their ScrollBar
@@ -2352,7 +2358,7 @@ defBackgroundFromParent(Widget w, int offset, XrmValue *val)
 	if ( XtIsSubclass(XtParent(w),xmScrolledWindowWidgetClass) )
 	{
 		val->size = sizeof(XmParentBackground(w));
-		val->addr = (XtPointer) & XmParentBackground(w);
+		val->addr = (XPointer) & XmParentBackground(w);
 	}
 	else
 	{
