@@ -1,8 +1,9 @@
 /**
  *
- * $Header: /cvsroot/hungry/lesstif/lib/Xm/SelectionBox.c,v 1.44 1999/07/02 02:01:17 rwscott Exp $
+ * $Header: /cvsroot/lesstif/lesstif/lib/Xm/SelectionBox.c,v 1.52 2001/05/22 11:39:30 amai Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
+ * Copyright (C) 1995-2001 LessTif Development Team
  *
  * This file is part of the GNU LessTif Library.
  *
@@ -22,12 +23,14 @@
  *
  **/
 
-static const char rcsid[] = "$Header: /cvsroot/hungry/lesstif/lib/Xm/SelectionBox.c,v 1.44 1999/07/02 02:01:17 rwscott Exp $";
+static const char rcsid[] = "$Header: /cvsroot/lesstif/lesstif/lib/Xm/SelectionBox.c,v 1.52 2001/05/22 11:39:30 amai Exp $";
 
 #include <LTconfig.h>
+
+#include <stdlib.h>
+
 #include <XmI/XmI.h>
 #include <XmI/MacrosI.h>
-
 #include <Xm/XmP.h>
 #include <Xm/SelectioBP.h>
 #include <Xm/CommandP.h>
@@ -45,8 +48,7 @@ static const char rcsid[] = "$Header: /cvsroot/hungry/lesstif/lib/Xm/SelectionBo
 #include <Xm/TextF.h>
 
 #include <XmI/DebugUtil.h>
-#include <stdio.h>
-#include <stdlib.h>
+
 
 /* Forward Declarations */
 
@@ -263,7 +265,6 @@ static XtActionsRec actions[] =
 	{"SelectionBoxRestore", _XmSelectionBoxRestore},
 };
 
-/* *INDENT-OFF* */
 #if 0
 static XmBaseClassExtRec _XmSelectionBCoreClassExtRec = {
     /* next_extension            */ NULL,
@@ -374,10 +375,10 @@ XmSelectionBoxClassRec xmSelectionBoxClassRec = {
     },
     /* XmSelectionBox part */
     {
-	/* extension */ NULL,
+	/* list_callback                */ NULL,
+	/* extension                    */ NULL,
     }
 };
-/* *INDENT-ON* */
 
 
 
@@ -399,6 +400,13 @@ class_initialize(void)
 static void
 class_part_initialize(WidgetClass widget_class)
 {
+    XmSelectionBoxWidgetClass sbwc = (XmSelectionBoxWidgetClass)widget_class;
+    XmSelectionBoxWidgetClass swc = (XmSelectionBoxWidgetClass)widget_class->core_class.superclass;
+
+    if (sbwc->selection_box_class.list_callback == XmInheritCallbackProc)
+    {
+    	sbwc->selection_box_class.list_callback = swc->selection_box_class.list_callback;
+    }
     _XmFastSubclassInit(widget_class, XmSELECTION_BOX_BIT);
 }
 
@@ -449,26 +457,26 @@ ListSingleSelect(Widget w, XtPointer client, XtPointer call)
 
     if (SB_Text(sb) == NULL)
     {
-	DEBUGOUT(XdbDebug(__FILE__, w,
+	DEBUGOUT(_LtDebugDebug(__FILE__, w,
 			  "ListSingleSelect (No TextField)\n"));
 	return;
     }
 
     if (lp == NULL || lp->item == NULL)
     {
-	DEBUGOUT(XdbDebug(__FILE__, w,
+	DEBUGOUT(_LtDebugDebug(__FILE__, w,
 			  "ListSingleSelect (NULL)\n"));
 	return;
     }
 
     if (!XmStringGetLtoR(lp->item, XmFONTLIST_DEFAULT_TAG, &s))
     {
-	DEBUGOUT(XdbDebug(__FILE__, w,
+	DEBUGOUT(_LtDebugDebug(__FILE__, w,
 			  "ListSingleSelect (Couldn't convert to string)\n"));
 	return;
     }
 
-    DEBUGOUT(XdbDebug(__FILE__, w,
+    DEBUGOUT(_LtDebugDebug(__FILE__, w,
 		      "ListSingleSelect '%s'\n", s));
 
     XmTextFieldSetString(SB_Text(sb), s);
@@ -490,7 +498,7 @@ _XmSbButton(Widget w, XtPointer client, XtPointer call)
     XmSelectionBoxCallbackStruct cbs;
     int au = 0;
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "_XmSbButton\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "_XmSbButton\n"));
 
     if (XmIsList(w))
     {
@@ -579,7 +587,7 @@ _XmSbButton(Widget w, XtPointer client, XtPointer call)
 
 	XtCallCallbackList((Widget)sb, sb->manager.help_callback, &cbs);
     }
-
+#if 0
     if (BB_AutoUnmanage(sb) && au)
     {
 	Widget s = XtParent((Widget)sb);
@@ -588,13 +596,13 @@ _XmSbButton(Widget w, XtPointer client, XtPointer call)
 	{
 	    XtUnmanageChild((Widget)sb);
 
-	    DEBUGOUT(XdbDebug2(__FILE__, (Widget)sb, w, "AutoUnmanage\n"));
+	    DEBUGOUT(_LtDebugDebug2(__FILE__, (Widget)sb, w, "AutoUnmanage\n"));
 
 	    /* XtNpopdownCallback */
 	    XtCallCallbackList(s, Shell_PopdownCallback(s), NULL);
 	}
     }
-
+#endif
 }
 
 /*
@@ -615,7 +623,7 @@ initialize(Widget request,
 {
     XmSelectionBoxWidget sb = (XmSelectionBoxWidget)new_w;
 
-    DEBUGOUT(XdbDebug(__FILE__, new_w,
+    DEBUGOUT(_LtDebugDebug(__FILE__, new_w,
 		      "initialize: %i args\n"
 		      "\trequest X %5i Y %5i W %5i H %5i\n"
 		      "\t  new_w X %5i Y %5i W %5i H %5i\n",
@@ -624,7 +632,7 @@ initialize(Widget request,
 		      XtWidth(request), XtHeight(request),
 		      XtX(new_w), XtY(new_w),
 		      XtWidth(new_w), XtHeight(new_w)));
-    DEBUGOUT(XdbPrintArgList(__FILE__, new_w, args, *num_args, False));
+    DEBUGOUT(_LtDebugPrintArgList(__FILE__, new_w, args, *num_args, False));
 
 
     SB_List(sb) = NULL;
@@ -715,14 +723,14 @@ initialize(Widget request,
 		/* T. Straumann: List made a copy of the item list;
 		 *				 we have to adjust SB_ListItems
 		 */
-		XtVaGetValues(SB_List(sb), XmNitems, &SB_ListItems(sb), 0);
+		XtVaGetValues(SB_List(sb), XmNitems, &SB_ListItems(sb), NULL);
     }
 }
 
 static void
 destroy(Widget w)
 {
-    DEBUGOUT(XdbDebug("RWS", w,"%s:destroy(%d)\n",
+    DEBUGOUT(_LtDebugDebug("RWS", w,"%s:destroy(%d)\n",
     	__FILE__, __LINE__
     	));
     /* rws 14 Apr 1998
@@ -751,7 +759,7 @@ set_values(Widget old,
     Arg al[10];
     int ac;
 
-    DEBUGOUT(XdbDebug(__FILE__, new_w,
+    DEBUGOUT(_LtDebugDebug(__FILE__, new_w,
 		      "set_values: %i args\n"
 		      "\t    old X %5i Y %5i W %5i H %5i\n"
 		      "\trequest X %5i Y %5i W %5i H %5i\n"
@@ -763,7 +771,7 @@ set_values(Widget old,
 		      XtWidth(request), XtHeight(request),
 		      XtX(new_w), XtY(new_w),
 		      XtWidth(new_w), XtHeight(new_w)));
-    DEBUGOUT(XdbPrintArgList(__FILE__, new_w, args, *num_args, False));
+    DEBUGOUT(_LtDebugPrintArgList(__FILE__, new_w, args, *num_args, False));
 
     /* this is required for every BB subclass */
     BB_InSetValues(new_w) = True;
@@ -793,6 +801,7 @@ set_values(Widget old,
 	String text;
 
 	SB_TextString(sb) = XmStringCopy(SB_TextString(sb));
+	XmStringFree(SB_TextString(o));
 	if (XmStringGetLtoR(SB_TextString(sb), XmSTRING_DEFAULT_CHARSET, &text))
 	{
 	    XmTextFieldSetString(SB_Text(sb), text);
@@ -805,6 +814,8 @@ set_values(Widget old,
     ac = 0;
     if (NE(SB_SelectionLabelString))
     {
+	SB_SelectionLabelString(sb) = XmStringCopy(SB_SelectionLabelString(sb));
+	XmStringFree(SB_SelectionLabelString(o));
 	XtSetArg(al[ac], XmNlabelString, SB_SelectionLabelString(sb)); ac++;
 	r = True;
     }
@@ -817,6 +828,8 @@ set_values(Widget old,
     ac = 0;
     if (NE(SB_ListLabelString))
     {
+	SB_ListLabelString(sb) = XmStringCopy(SB_ListLabelString(sb));
+	XmStringFree(SB_ListLabelString(o));
 	XtSetArg(al[ac], XmNlabelString, SB_ListLabelString(sb)); ac++;
 	r = True;
     }
@@ -829,6 +842,8 @@ set_values(Widget old,
     ac = 0;
     if (NE(SB_OkLabelString))
     {
+	SB_OkLabelString(sb) = XmStringCopy(SB_OkLabelString(sb));
+	XmStringFree(SB_OkLabelString(o));
 	XtSetArg(al[ac], XmNlabelString, SB_OkLabelString(sb)); ac++;
 	r = True;
     }
@@ -841,6 +856,8 @@ set_values(Widget old,
     ac = 0;
     if (NE(SB_CancelLabelString))
     {
+	SB_CancelLabelString(sb) = XmStringCopy(SB_CancelLabelString(sb));
+	XmStringFree(SB_CancelLabelString(o));
 	XtSetArg(al[ac], XmNlabelString, SB_CancelLabelString(sb)); ac++;
 	r = True;
     }
@@ -853,6 +870,8 @@ set_values(Widget old,
     ac = 0;
     if (NE(SB_ApplyLabelString))
     {
+	SB_ApplyLabelString(sb) = XmStringCopy(SB_ApplyLabelString(sb));
+	XmStringFree(SB_ApplyLabelString(o));
 	XtSetArg(al[ac], XmNlabelString, SB_ApplyLabelString(sb)); ac++;
 	r = True;
     }
@@ -865,6 +884,8 @@ set_values(Widget old,
     ac = 0;
     if (NE(SB_HelpLabelString))
     {
+	SB_HelpLabelString(sb) = XmStringCopy(SB_HelpLabelString(sb));
+	XmStringFree(SB_HelpLabelString(o));
 	XtSetArg(al[ac], XmNlabelString, SB_HelpLabelString(sb)); ac++;
 	r = True;
     }
@@ -1069,7 +1090,7 @@ _XmSelectionBoxGetTextColumns(Widget wid, int resource_offset,
     Arg a;
     int n;
 
-    DEBUGOUT(XdbDebug(__FILE__, wid, "XmSelectionBoxGettTextColumns\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, wid, "XmSelectionBoxGettTextColumns\n"));
 
     XtSetArg(a, XmNtextColumns, &n);
     n = -1;
@@ -1083,9 +1104,13 @@ void
 _XmSelectionBoxGetTextString(Widget wid, int resource_offset,
 			     XtArgVal *value)
 {
-    DEBUGOUT(XdbDebug(__FILE__, wid, "_XmSelectionBoxGetTextString\n"));
+    char *str;
 
-    *value = (XtArgVal)XmStringCreateSimple(XmTextFieldGetString(SB_Text(wid)));
+    DEBUGOUT(_LtDebugDebug(__FILE__, wid, "_XmSelectionBoxGetTextString\n"));
+
+    str = XmTextFieldGetString(SB_Text(wid));
+    *value = (XtArgVal)XmStringCreateSimple(str);
+    XtFree(str);
 }
 
 void
@@ -1095,7 +1120,7 @@ _XmSelectionBoxGetListItems(Widget wid, int resource_offset,
     XmStringTable l;
     Arg a;
 
-    DEBUGOUT(XdbDebug(__FILE__, wid,
+    DEBUGOUT(_LtDebugDebug(__FILE__, wid,
 		      "XmSelectionBoxGetListItems(_, %d, _)\n",
 		      resource_offset));
 
@@ -1122,7 +1147,7 @@ _XmSelectionBoxGetListItemCount(Widget wid, int resource_offset,
     Arg a;
     int n;
 
-    DEBUGOUT(XdbDebug(__FILE__, wid, "XmSelectionBoxGetListItemCount\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, wid, "XmSelectionBoxGetListItemCount\n"));
 
     XtSetArg(a, XmNitemCount, &n);
     n = -1;
@@ -1175,7 +1200,7 @@ _XmSelectionBoxUpOrDown(Widget wid, XEvent *event,
     int *position_list, position_count, new_position, old_position;
     /* char *selected_text; */
 
-    DEBUGOUT(XdbDebug(__FILE__, wid, "_XmSelectionBoxUpOrDown %s\n",
+    DEBUGOUT(_LtDebugDebug(__FILE__, wid, "_XmSelectionBoxUpOrDown %s\n",
 		      argv ? *argv: "NULL"));
     if (list != NULL && text != NULL)
     {
@@ -1264,7 +1289,7 @@ _XmSelectionBoxRestore(Widget wid, XEvent *event,
     int item_count, *position_list, position_count;
     char *selected_text;
 
-    DEBUGOUT(XdbDebug(__FILE__, wid, "_XmSelectionBoxRestore\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, wid, "_XmSelectionBoxRestore\n"));
 
     if (text != NULL && list != NULL)
     {
@@ -1303,7 +1328,7 @@ _XmSelectionBoxGeoMatrixCreate(Widget _w, Widget _from, XtWidgetGeometry *_pref)
     register XmKidGeometry boxPtr;
     Cardinal numKids;
     Boolean newRow;
-    int nrows, i, nextras;
+    Cardinal nrows, i, nextras;
     Widget *extras;
 
     numKids = MGR_NumChildren(_w);
@@ -1693,7 +1718,7 @@ XmCreateSelectionDialog(Widget parent, char *name,
     Widget d, r;
     char *s;
     Arg *al = (Arg *)XtCalloc(argcount + 1, sizeof(Arg));
-    int ac, i;
+    Cardinal ac, i;
 
     s = _XmMakeDialogName(name);
 
@@ -1712,7 +1737,7 @@ XmCreateSelectionDialog(Widget parent, char *name,
     XtFree(s);
 
     r = XtCreateWidget(name, xmSelectionBoxWidgetClass, d, al, ac);
-    XtFree((XtPointer)al);
+    XtFree((XPointer)al);
     return r;
 }
 
@@ -1723,7 +1748,7 @@ XmCreatePromptDialog(Widget parent, char *name,
     Widget d, ret;
     char *s;
     Arg *al;
-    int i;
+    Cardinal i;
 
     s = _XmMakeDialogName(name);
 
@@ -1742,7 +1767,7 @@ XmCreatePromptDialog(Widget parent, char *name,
     XtSetArg(al[argcount - 1], XmNdialogType, XmDIALOG_PROMPT);
     ret = XtCreateWidget(name, xmSelectionBoxWidgetClass, d, al, argcount);
 
-    XtFree((XtPointer)al);
+    XtFree((XPointer)al);
 
     return ret;
 }
@@ -1865,7 +1890,7 @@ _XmDialogTypeDefault(Widget w, int offset, XrmValue *value)
 {
     static unsigned char dialog_type;
     
-    DEBUGOUT(XdbDebug(__FILE__, w, "_XmDialogTypeDefault\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "_XmDialogTypeDefault\n"));
     
     dialog_type = XmDIALOG_WORK_AREA;
     
