@@ -1,8 +1,9 @@
 /**
  *
- * $Id: TearOff.c,v 1.40 1999/07/02 05:45:45 gritton Exp $
+ * $Id: TearOff.c,v 1.47 2001/05/14 19:46:10 amai Exp $
  *
  * Copyright (C) 1996 Free Software Foundation, Inc.
+ * Copyright (C) 1996-2001 LessTif Development Team 
  *
  * This file is part of the GNU LessTif Library.
  *
@@ -22,13 +23,13 @@
  *
  **/
 
-static const char rcsid[] = "$Id: TearOff.c,v 1.40 1999/07/02 05:45:45 gritton Exp $";
+static const char rcsid[] = "$Id: TearOff.c,v 1.47 2001/05/14 19:46:10 amai Exp $";
+
+#include <LTconfig.h>
 
 #include <stdio.h>
 
-#include <LTconfig.h>
 #include <XmI/XmI.h>
-
 #include <Xm/MwmUtil.h>
 #include <Xm/Protocols.h>
 #include <Xm/TearOffP.h>
@@ -52,12 +53,12 @@ static void
 _XmTearOffFocusChangeEventHandler(Widget reportingWidget, XtPointer data,
 				  XEvent *event, Boolean *cont)
 {
-	DEBUGOUT(XdbDebug("TEAROFF", reportingWidget,
+	DEBUGOUT(_LtDebugDebug("TEAROFF", reportingWidget,
 		      "_XmTearOffFocusChangeEventHandler() - %s\n",
-		      XdbEventType2String(event->type)));
-	DEBUGOUT(XdbDebug(__FILE__, reportingWidget,
+		      _LtDebugEventType2String(event->type)));
+	DEBUGOUT(_LtDebugDebug(__FILE__, reportingWidget,
 		      "_XmTearOffFocusChangeEventHandler() - %s\n",
-		      XdbEventType2String(event->type)));
+		      _LtDebugEventType2String(event->type)));
 	switch(event->type)
 	{
 	case FocusIn:
@@ -72,6 +73,15 @@ _XmTearOffFocusChangeEventHandler(Widget reportingWidget, XtPointer data,
 	    /*
 	    _XmMenuDisarmItem(MGR_ActiveChild(reportingWidget));
 	    */
+	    {
+	    Cardinal i;
+	    Widget w = reportingWidget;
+
+		for (i = 0; i < MGR_NumChildren(w); i++)
+		{
+		    _XmMenuDisarmItem(MGR_Children(w)[i]);
+		}
+	    }
 	    break;
 	default:
 		_XmWarning(reportingWidget, "%s(%d) _XmTearOffFocusChangeEventHandler unknown event type",__FILE__, __LINE__);
@@ -85,7 +95,7 @@ _XmTearOffBtnDownEventHandler(Widget reportingWidget, XtPointer data,
 {
 Widget menu;
 
-    DEBUGOUT(XdbDebug(__FILE__, reportingWidget,
+    DEBUGOUT(_LtDebugDebug(__FILE__, reportingWidget,
 		      "_XmTearOffBtnDownEventHandler()\n"));
     if (XmIsRowColumn(reportingWidget))
     {
@@ -122,7 +132,7 @@ Widget menu;
 	 * to it after we're done. */
 	_XmMenuFocus(w, XmMENU_FOCUS_SAVE, CurrentTime);
 
-	DEBUGOUT(XdbDebug(__FILE__, w, "RC FOCUS SET: %s\n", XtName(w)));
+	DEBUGOUT(_LtDebugDebug(__FILE__, w, "RC FOCUS SET: %s\n", XtName(w)));
 
 	_XmMenuFocus(w, XmMENU_FOCUS_SET, CurrentTime);
 #endif
@@ -165,7 +175,7 @@ Widget menu;
     }
     else
     {
-	DEBUGOUT(XdbDebug(__FILE__, reportingWidget,
+	DEBUGOUT(_LtDebugDebug(__FILE__, reportingWidget,
 		      "_XmTearOffBtnDownEventHandler() - Not torn/armed\n"));
     }
 }
@@ -176,7 +186,7 @@ _XmTearOffBtnUpEventHandler(Widget reportingWidget, XtPointer data,
 {
 Widget menu;
 
-    DEBUGOUT(XdbDebug(__FILE__, reportingWidget,
+    DEBUGOUT(_LtDebugDebug(__FILE__, reportingWidget,
 		      "_XmTearOffBtnUpEventHandler()\n"));
     if (XmIsRowColumn(reportingWidget))
     {
@@ -195,7 +205,7 @@ Widget menu;
     }
     else
     {
-	DEBUGOUT(XdbDebug(__FILE__, reportingWidget,
+	DEBUGOUT(_LtDebugDebug(__FILE__, reportingWidget,
 		      "_XmTearOffBtnUpEventHandler() - Not torn/armed\n"));
     }
 }
@@ -203,7 +213,7 @@ Widget menu;
 void
 _XmDestroyTearOffShell(Widget w)
 {
-    DEBUGOUT(XdbDebug(__FILE__, w, "_XmDestroyTearOffShell()\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "_XmDestroyTearOffShell()\n"));
 }
 
 void
@@ -211,9 +221,9 @@ _XmDismissTearOff(Widget shell, XtPointer closure, XtPointer call_data)
 {
     Widget menu;
     ShellWidget sw = (ShellWidget)shell;
-    XEvent *event = call_data;
+    XEvent *event = (XEvent *)call_data;
 
-    DEBUGOUT(XdbDebug(__FILE__, shell, "_XmDismissTearOff()\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, shell, "_XmDismissTearOff()\n"));
 
     if (XtIsSubclass(shell, transientShellWidgetClass))
     {
@@ -229,6 +239,7 @@ _XmDismissTearOff(Widget shell, XtPointer closure, XtPointer call_data)
 
     XtPopdown(shell);
     RC_SetTearOffActive(menu, 0);
+    /*
     _XmCallRowColumnUnmapCallback(menu, event);
     if (RC_TearOffDeactivate_cb(menu))
     {
@@ -238,6 +249,7 @@ _XmDismissTearOff(Widget shell, XtPointer closure, XtPointer call_data)
 	cbs.event = event;
 	XtCallCallbackList(menu, RC_TearOffDeactivate_cb(menu), (XtPointer)&cbs);
     }
+    */
 }
 
 void
@@ -246,7 +258,7 @@ _XmTearOffInitiate(Widget w, XEvent *event)
     Widget menu;
     Widget shell;
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "_XmTearOffInitiate()\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "_XmTearOffInitiate()\n"));
 
     _XmUngrabKeyboard(w, CurrentTime);
     _XmUngrabPointer(w, CurrentTime);
@@ -255,7 +267,7 @@ _XmTearOffInitiate(Widget w, XEvent *event)
 
     if (!XmIsRowColumn(menu))
     {
-	DEBUGOUT(XdbDebug2(__FILE__, menu, w, "Parent is not a menu\n"));
+	DEBUGOUT(_LtDebugDebug2(__FILE__, menu, w, "Parent is not a menu\n"));
 	return;
     }
 
@@ -317,6 +329,7 @@ _XmTearOffInitiate(Widget w, XEvent *event)
     _XmRestoreTearOffToToplevelShell(menu, event);
 #if 1
     RC_CascadeBtn(menu) = NULL;
+#if 0
     if (MGR_NumChildren(menu) > 0)
     {
 	_XmMenuFocus(menu, XmMENU_FOCUS_SAVE, CurrentTime);
@@ -347,6 +360,7 @@ _XmTearOffInitiate(Widget w, XEvent *event)
 		Lab_MenuArm(RC_TearOffFocusItem(menu));
 	}
     }
+#endif
 #endif
     RC_SetTearOffActive(menu, 1);
     if (RC_TearOffActivate_cb(menu))
@@ -358,14 +372,16 @@ _XmTearOffInitiate(Widget w, XEvent *event)
 	XtCallCallbackList(menu, RC_TearOffActivate_cb(menu), (XtPointer)&cbs);
     }
     _XmCallRowColumnMapCallback(menu, event);
+    _XmMenuFocus(menu, XmMENU_FOCUS_SAVE, CurrentTime);
+    RCClass_MenuTraverse(menu, XmTRAVERSE_HOME);
 }
 
 void
 _XmAddTearOffEventHandlers(Widget w)
 {
-int i;
+    Cardinal i;
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "_XmAddTearOffEventHandlers()\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "_XmAddTearOffEventHandlers()\n"));
     XtAddEventHandler(w, FocusChangeMask, False, _XmTearOffFocusChangeEventHandler, NULL);
     /*
     XtAddEventHandler(w, ButtonPressMask, False, _XmTearOffBtnDownEventHandler, NULL);
@@ -383,7 +399,7 @@ int i;
 Boolean
 _XmIsTearOffShellDescendant(Widget w)
 {
-    DEBUGOUT(XdbDebug(__FILE__, w, "_XmIsTearOffShellDescendant()\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "_XmIsTearOffShellDescendant()\n"));
 
     return False;
 }
@@ -392,7 +408,7 @@ void
 _XmLowerTearOffObscuringPoppingDownPanes(Widget ancestor,
 					 Widget tearOff)
 {
-    DEBUGOUT(XdbDebug2(__FILE__, ancestor, tearOff,
+    DEBUGOUT(_LtDebugDebug2(__FILE__, ancestor, tearOff,
 		       "_XmLowerTearOffObscuringPoppingDownPanes()\n"));
 }
 
@@ -400,7 +416,7 @@ void
 _XmRestoreExcludedTearOffToToplevelShell(Widget w,
 					 XEvent *event)
 {
-    DEBUGOUT(XdbDebug(__FILE__, w,
+    DEBUGOUT(_LtDebugDebug(__FILE__, w,
 		      "_XmRestoreExcludedTearOffToToplevelShell()\n"));
 }
 
@@ -415,7 +431,7 @@ CreateTopLevelShell(Widget w)
     Atom delete_atom;
     Widget parent_shell;
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "CreateTopLevelShell()\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "CreateTopLevelShell()\n"));
 
     /* create the tear off shell's name */
     to_shell_name = XtMalloc(strlen(TEAR_OFF_SHELL_NAME) +
@@ -514,7 +530,7 @@ _XmRestoreTearOffToToplevelShell(Widget w, XEvent *event)
     /* w represents the row column */
     Widget menu_shell = XtParent(w);
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "_XmRestoreTearOffToToplevelShell()\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "_XmRestoreTearOffToToplevelShell()\n"));
 
     /* Tear off buttons should not appear in torn off shells */
     if (RC_TearOffControl(w))
@@ -585,7 +601,7 @@ _XmRestoreTearOffToToplevelShell(Widget w, XEvent *event)
     RC_SetTornOff(w, 1);
     RC_SetFromInit(w, 0);
     RC_ParentShell(w) = menu_shell;	/* save off the menu shell */
-    DEBUGOUT(XdbDebug(__FILE__, w, "_XmRestoreTearOffToToplevelShell() new parent %s\n",
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "_XmRestoreTearOffToToplevelShell() new parent %s\n",
 		XtName(XtParent(w))));
     _XmAddTearOffEventHandlers(w);
 }
@@ -598,13 +614,13 @@ _XmRestoreTearOffToMenuShell(Widget w,
     Widget transient = XtParent(w);
     Widget menu_shell;
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "_XmRestoreTearOffToMenuShell()\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "_XmRestoreTearOffToMenuShell()\n"));
 
     /* Tear off buttons should appear in menu shells */
     if (!RC_TearOffControl(w))
     {
 	/* FIX ME: just manage existing one (if any?) */
-	DEBUGOUT(XdbDebug(__FILE__, w,
+	DEBUGOUT(_LtDebugDebug(__FILE__, w,
 			  "_XmRestoreTearOffToMenuShell creating"
 			  " tear off control\n"));
 
@@ -629,7 +645,7 @@ _XmRestoreTearOffToMenuShell(Widget w,
     XGCValues values;
     unsigned long mask = 0;
 
-	DEBUGOUT(XdbDebug(__FILE__, w, "_XmRestoreTearOffToMenuShell() - Grab a copy of the transientShell %s %s %d %d\n",
+	DEBUGOUT(_LtDebugDebug(__FILE__, w, "_XmRestoreTearOffToMenuShell() - Grab a copy of the transientShell %s %s %d %d\n",
 		XtIsManaged(transient) ? "managed" : "not managed",
 		XtIsManaged(w) ? "managed" : "not managed",
 		XtWidth(transient),
@@ -653,6 +669,14 @@ _XmRestoreTearOffToMenuShell(Widget w,
 			    XtWidth(transient),
 			    XtHeight(transient),
 			    transient->core.depth);
+	{
+	Cardinal i;
+
+	    for (i = 0; i < MGR_NumChildren(w); i++)
+	    {
+	    	_XmMenuDisarmItem(MGR_Children(w)[i]);
+	    }
+	}
 	XmUpdateDisplay(transient);
 	XCopyArea(XtDisplay(transient), XtWindow(transient), MenuImage, gc,
 		    0, 0,
@@ -674,6 +698,15 @@ _XmRestoreTearOffToMenuShell(Widget w,
 		XmNbackgroundPixmap, MenuImage,
 		NULL);
 	RC_SetTearOffDirty(w, True);
+	_XmCallRowColumnUnmapCallback(w, event);
+	if (RC_TearOffDeactivate_cb(w))
+	{
+	XmAnyCallbackStruct cbs;
+
+	    cbs.reason = XmCR_TEAR_OFF_DEACTIVATE;
+	    cbs.event = event;
+	    XtCallCallbackList(w, RC_TearOffDeactivate_cb(w), (XtPointer)&cbs);
+	}
     }
 
     menu_shell = RC_ParentShell(w);
