@@ -1,8 +1,9 @@
 /**
  *
- * $Id: ToggleB.c,v 1.70 1999/11/11 13:43:25 rwscott Exp $
+ * $Id: ToggleB.c,v 1.77 2001/05/14 19:46:11 amai Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
+ * Copyright (C) 1995-2001 LessTif Development Team
  *
  * This file is part of the GNU LessTif Library.
  *
@@ -22,11 +23,13 @@
  *
  **/
 
-static const char rcsid[] = "$Id: ToggleB.c,v 1.70 1999/11/11 13:43:25 rwscott Exp $";
+static const char rcsid[] = "$Id: ToggleB.c,v 1.77 2001/05/14 19:46:11 amai Exp $";
 
 #include <LTconfig.h>
-#include <XmI/XmI.h>
 
+#include <stdarg.h>
+
+#include <XmI/XmI.h>
 #include <Xm/XmP.h>
 #include <Xm/ToggleBP.h>
 #include <Xm/RowColumnP.h>
@@ -36,17 +39,11 @@ static const char rcsid[] = "$Id: ToggleB.c,v 1.70 1999/11/11 13:43:25 rwscott E
 #include <Xm/CascadeBG.h>
 #include <Xm/RepType.h>
 #include <Xm/TransltnsP.h>
+#include <Xm/ToggleBGP.h>
 #include <X11/ShellP.h>
 
-#if STDC_HEADERS
-#include <stdarg.h>
-#else
-#include <varargs.h>
-#endif
-
 #include <XmI/DebugUtil.h>
 
-#include <Xm/ToggleBGP.h>
 
 /* Some M*tif versions (e.g. 1.2.6 on Solaris 2.6) leave less room above and
  * below the indicator than others (e.g. 1.2.5 on HP-UX 10, my reference).
@@ -141,7 +138,7 @@ static XtResource resources[] =
 	sizeof(Boolean), Offset(set),
 	XmRImmediate, (XtPointer)False
     },
-#if XmVERSION == 1 || LESSTIF_BACK_COMPAT
+#if XmVERSION == 1
     {
 	XmNindicatorOn, XmCIndicatorOn, XmRBoolean,
 	sizeof(Boolean), Offset(ind_on),
@@ -487,7 +484,7 @@ static void
 initialize(Widget request, Widget new_w,
 	   ArgList args, Cardinal *num_args)
 {
-    DEBUGOUT(XdbDebug(__FILE__, new_w,
+    DEBUGOUT(_LtDebugDebug(__FILE__, new_w,
 		      "initialize: %i args\n"
 		      "\trequest X %5i Y %5i W %5i H %5i\n"
 		      "\t  new_w X %5i Y %5i W %5i H %5i\n",
@@ -496,7 +493,7 @@ initialize(Widget request, Widget new_w,
 		      XtWidth(request), XtHeight(request),
 		      XtX(new_w), XtY(new_w),
 		      XtWidth(new_w), XtHeight(new_w)));
-    DEBUGOUT(XdbPrintArgList(__FILE__, new_w, args, *num_args, False));
+    DEBUGOUT(_LtDebugPrintArgList(__FILE__, new_w, args, *num_args, False));
 
     TB_Armed(new_w) = False;
 
@@ -615,7 +612,8 @@ initialize(Widget request, Widget new_w,
 	    {
 		Lab_MarginLeft(new_w) += margin;
 		Lab_TextRect_x(new_w) += margin;
-		Lab_AccTextRect(new_w).x += margin;
+		if (Lab_AcceleratorText(new_w) != NULL)
+		    Lab_AccTextRect(new_w).x += margin;
 	    }
 	    else
 		Lab_MarginRight(new_w) += margin;
@@ -642,7 +640,8 @@ initialize(Widget request, Widget new_w,
 	    if (!XtHeight(request))
 	    {
 		Lab_TextRect_y(new_w) += margin >> 1;
-		Lab_AccTextRect(new_w).y += margin >> 1;
+		if (Lab_AcceleratorText(new_w) != NULL)
+		    Lab_AccTextRect(new_w).y += margin >> 1;
 		XtHeight(new_w) += margin;
 	    }
 	}
@@ -670,7 +669,7 @@ set_values(Widget old, Widget request, Widget new_w,
 {
     Boolean refresh_needed = False;
 
-    DEBUGOUT(XdbDebug(__FILE__, new_w, "ToggleB set_values()\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, new_w, "ToggleB set_values()\n"));
 
     if (TB_IndType(old) != TB_IndType(new_w) &&
 	!XmRepTypeValidValue(XmRepTypeGetId(XmRIndicatorType),
@@ -942,7 +941,7 @@ resize(Widget w)
 static void
 expose(Widget w, XEvent *event, Region region)
 {
-    DEBUGOUT(XdbDebug(__FILE__, w, "ToggleB Expose\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "ToggleB Expose\n"));
     draw_toggle(w, event, region, True, 0);
 }
 
@@ -1028,7 +1027,7 @@ ArmAndActivate(Widget w, XEvent *event, String *params, Cardinal *num_params)
 {
     Boolean poppedUp;
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "%s:ArmAndActivate(%d)\n",
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "%s:ArmAndActivate(%d)\n",
     	__FILE__, __LINE__));
     Arm(w, event, params, num_params);
 
@@ -1047,7 +1046,7 @@ ButtonUp(Widget w, XEvent *event, String *params, Cardinal *num_params)
     Boolean validButton, poppedUp;
     XmToggleButtonCallbackStruct cbs;
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "ToggleB ButtonUp()\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "ToggleB ButtonUp()\n"));
 
     /* queue events until the next button event */
     XAllowEvents(XtDisplay(w), SyncPointer, CurrentTime);
@@ -1083,7 +1082,7 @@ ButtonUp(Widget w, XEvent *event, String *params, Cardinal *num_params)
     TB_Set(w) = NEXT_TOGGLE(w);
     draw_toggle(w, NULL, NULL, False, TB_Set(w));
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "ButtonUp: HERE\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "ButtonUp: HERE\n"));
 
     cbs.reason = XmCR_VALUE_CHANGED;
     cbs.event = event;
@@ -1120,7 +1119,7 @@ ButtonDown(Widget w, XEvent *event, String *params, Cardinal *num_params)
     /* queue events until the next button event. */
     XAllowEvents(XtDisplay(w), SyncPointer, CurrentTime);
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "ToggleB ButtonDown()\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "ToggleB ButtonDown()\n"));
 
     if (event && (event->type == ButtonPress))
     {
@@ -1135,7 +1134,7 @@ ButtonDown(Widget w, XEvent *event, String *params, Cardinal *num_params)
 
     RC_MenuShellPopdown(w, event, &poppedUp);
     {
-	int i;
+	Cardinal i;
 	Widget menu = XtParent(w);
 
 	for (i = 0; i < MGR_NumChildren(menu); i++)
@@ -1184,7 +1183,7 @@ EnterWindow(Widget w, XEvent *event, String *params, Cardinal *num_params)
 {
     XmToggleButtonCallbackStruct cbs;
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "ToggleB Enter\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "ToggleB Enter\n"));
 
     if (!IN_MENU(w))
     {
@@ -1260,12 +1259,11 @@ LeaveWindow(Widget w, XEvent *event, String *params, Cardinal *num_params)
 static void
 KeySelect(Widget w, XEvent *event, String *params, Cardinal *num_params)
 {
-    DEBUGOUT(XdbDebug(__FILE__, w, "KeySelect\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "KeySelect\n"));
 
     /* FIX ME */
 }
 
-#if STDC_HEADERS
 static void
 MenuProcEntry(int proc, Widget w,...)
 {
@@ -1275,19 +1273,6 @@ MenuProcEntry(int proc, Widget w,...)
     */
 
     va_start(arg_list, w);
-#else
-MenuProcEntry(proc, w, va_alist)
-     int proc;
-     Widget w;
-     va_dcl
-{
-    va_list arglist;
-    /*
-    Cardinal num_params = 0;
-    */
-
-    va_start(arglist);
-#endif
 
     switch (proc)
     {
@@ -1791,7 +1776,7 @@ XmToggleButtonSetState(Widget w, Boolean state, Boolean notify)
 {
     XmToggleButtonCallbackStruct cbs;
 
-    DEBUGOUT(XdbDebug(__FILE__, w, "ToggleB SetState: %d %d\n",
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "ToggleB SetState: %d %d\n",
 		      state, notify));
 
     _XmObjectLock(w);
