@@ -1,8 +1,9 @@
 /**
  *
- * $Id: ToggleBG.c,v 1.67 1999/09/03 00:17:40 rwscott Exp $
+ * $Id: ToggleBG.c,v 1.77 2001/05/14 19:46:11 amai Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
+ * Copyright (C) 1995-2001 LessTif Development Team
  *
  * This file is part of the GNU LessTif Library.
  *
@@ -22,9 +23,13 @@
  *
  **/
 
-static const char rcsid[] = "$Id: ToggleBG.c,v 1.67 1999/09/03 00:17:40 rwscott Exp $";
+static const char rcsid[] = "$Id: ToggleBG.c,v 1.77 2001/05/14 19:46:11 amai Exp $";
 
 #include <LTconfig.h>
+
+#include <stdarg.h>
+#include <string.h>
+
 #include <XmI/XmI.h>
 
 #include <Xm/XmP.h>
@@ -36,18 +41,12 @@ static const char rcsid[] = "$Id: ToggleBG.c,v 1.67 1999/09/03 00:17:40 rwscott
 #include <Xm/MenuShell.h>
 #include <Xm/MenuUtilP.h>
 #include <Xm/CacheP.h>
-#include <X11/Xfuncs.h>
 #include <X11/ShellP.h>
 #include <Xm/ToggleBP.h>
 
-#ifndef __STDC__
-#include <varargs.h>
-#else
-#include <stdarg.h>
-#endif
-
 #include <XmI/DebugUtil.h>
 
+
 /* Some M*tif versions (e.g. 1.2.6 on Solaris 2.6) leave less room above and
  * below the indicator than others (e.g. 1.2.5 on HP-UX 10, my reference).
  */
@@ -179,7 +178,7 @@ static XtResource cache_resources[] =
 	sizeof(Pixmap), Offset(insen_pixmap),
 	XmRImmediate, (XtPointer)XmUNSPECIFIED_PIXMAP
     },
-#if XmVERSION == 1 || LESSTIF_BACK_COMPAT
+#if XmVERSION == 1
     {
 	XmNindicatorOn, XmCIndicatorOn, XmRBoolean,
 	sizeof(Boolean), Offset(ind_on),
@@ -471,9 +470,9 @@ secondary_object_create(Widget request,
     XmBaseClassExt *bce;
     XtPointer nsec, rsec;
     XmWidgetExtData ed;
-    int size;
+    Cardinal size;
 
-    DEBUGOUT(XdbDebug(__FILE__, new_w,
+    DEBUGOUT(_LtDebugDebug(__FILE__, new_w,
 		      "ToggleButtonGCacheRec %s being initialized.\n",
 		      XtName(new_w)));
 
@@ -503,7 +502,7 @@ secondary_object_create(Widget request,
     ed->widget = (Widget)nsec;
     ed->reqWidget = (Widget)rsec;
 
-    bcopy(nsec, rsec, size);
+    memcpy(rsec, nsec, size);
     ((XmExtRec *)rsec)->object.self = (Widget)rsec;
 
     _XmPushWidgetExtData(new_w, ed, XmCACHE_EXTENSION);
@@ -518,7 +517,7 @@ secondary_object_create(Widget request,
 int
 _XmToggleBCacheCompare(XtPointer A, XtPointer B)
 {
-    return !bcmp(((XmToggleButtonGCacheObjPart *)A),
+    return !memcmp(((XmToggleButtonGCacheObjPart *)A),
 		 ((XmToggleButtonGCacheObjPart *)B),
 		 sizeof(XmToggleButtonGCacheObjPart));
 }
@@ -556,10 +555,10 @@ class_initialize(void)
 			     &labels, &nlabels);
 
     combined = (XtResourceList)XtMalloc(sizeof(XtResource) * ncom);
-    bcopy(labels, combined, nlabels * sizeof(XtResource));
-    bcopy(cache_resources,
-	  &combined[nlabels],
-	  XtNumber(cache_resources) * sizeof(XtResource));
+    memcpy(combined, labels, nlabels * sizeof(XtResource));
+    memcpy(&combined[nlabels],
+           cache_resources,
+	   XtNumber(cache_resources) * sizeof(XtResource));
     XtFree((char *)labels);
 
     xmToggleButtonGCacheObjClassRec.object_class.resources = combined;
@@ -631,7 +630,7 @@ initialize_posthook(Widget request, Widget new_w,
 {
     XmWidgetExtData ext;
 
-    DEBUGOUT(XdbDebug(__FILE__, new_w, "ToggleBG InitializePosthook\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, new_w, "ToggleBG InitializePosthook\n"));
 
     /* don't let the null fool you */
     LabG_Cache(new_w) = (XmLabelGCacheObjPart *)
@@ -763,20 +762,49 @@ initialize(Widget request, Widget new_w,
 
     if (TBG_IndOn(new_w))
     {
-	int iheight;
+	int margin;
 
-	/* Make room for the indicator */
-	iheight = TBG_IndicatorDim(new_w);
+	/* Make sure there's enough room on the side for the indicator */
+	margin = (TBG_IndicatorDim(new_w) + TBG_Spacing(new_w))
+	    - (LabG_StringDirection(new_w) == XmSTRING_DIRECTION_L_TO_R
+	       ? LabG_MarginLeft(new_w) : LabG_MarginRight(new_w));
+	if (margin > 0)
+	{
+	    if (LabG_StringDirection(new_w) == XmSTRING_DIRECTION_L_TO_R)
+	    {
+		LabG_MarginLeft(new_w) += margin;
+		LabG_TextRect_x(new_w) += margin;
+		LabG_AccTextRect(new_w).x += margin;
+	    }
+	    else
+		LabG_MarginRight(new_w) += margin;
+	    if (!XtWidth(request))
+		XtWidth(new_w) += margin;
+	}
+
+	/* Make sure there's enough room vertically.
+	 * Non-menu toggles want some padding space.
+	 */
+	margin = TBG_IndicatorDim(new_w) - (LabG_TextRect_height(new_w)
+					    + LabG_MarginTop(new_w)
+					    + LabG_MarginBottom(new_w));
 #ifdef LESS_VERTICAL_PADDING
-	iheight -= LabG_MarginHeight(new_w) << 1;
+	margin -= LabG_MarginHeight(new_w) << 1;
 #else
 	if (!IN_MENU(new_w))
-	    iheight += (LabG_Shadow(new_w) + Xm3D_ENHANCE_PIXEL) << 1;
+	    margin += (LabG_Shadow(new_w) + Xm3D_ENHANCE_PIXEL) << 1;
 #endif
-	_LtLabelGAccomodateIndicator(new_w, request,
-	    TBG_IndicatorDim(new_w) + TBG_Spacing(new_w), iheight,
-	    0, LabG_StringDirection(new_w) == XmSTRING_DIRECTION_L_TO_R,
-	    False);
+	if (margin > 0)
+	{
+	    LabG_MarginTop(new_w) += margin >> 1;
+	    LabG_MarginBottom(new_w) += margin >> 1;
+	    if (!XtHeight(request))
+	    {
+		LabG_TextRect_y(new_w) += margin >> 1;
+		LabG_AccTextRect(new_w).y += margin >> 1;
+		XtHeight(new_w) += margin;
+	    }
+	}
     }
 
     if (IN_MENU(new_w))
@@ -807,7 +835,7 @@ set_values_prehook(Widget old, Widget request, Widget new_w,
 {
     XmBaseClassExt *bce;
     XmWidgetExtData ed;
-    int size;
+    Cardinal size;
     XtPointer nsec, rsec;
 
     bce = _XmGetBaseClassExtPtr(XtClass(new_w), XmQmotif);
@@ -826,12 +854,12 @@ set_values_prehook(Widget old, Widget request, Widget new_w,
     ExtObj_LogicalParent(nsec) = new_w;
     ExtObj_ExtensionType(nsec) = XmCACHE_EXTENSION;
 
-    bcopy(LabG_Cache(new_w),
-	  &((XmLabelGCacheObject)nsec)->label_cache,
-	  sizeof(XmLabelGCacheObjPart));
-    bcopy(TBG_Cache(new_w),
-	  &((XmToggleButtonGCacheObject)nsec)->toggle_cache,
-	  sizeof(XmToggleButtonGCacheObjPart));
+    memcpy(&((XmLabelGCacheObject)nsec)->label_cache,
+           LabG_Cache(new_w),
+	   sizeof(XmLabelGCacheObjPart));
+    memcpy(&((XmToggleButtonGCacheObject)nsec)->toggle_cache,
+           TBG_Cache(new_w),
+	   sizeof(XmToggleButtonGCacheObjPart));
 
     ed = (XmWidgetExtData)XtMalloc(sizeof(XmWidgetExtDataRec));
     ed->widget = (Widget)nsec;
@@ -846,7 +874,7 @@ set_values_prehook(Widget old, Widget request, Widget new_w,
 		   (*bce)->secondaryObjectClass->core_class.num_resources,
 		   args, *num_args);
 
-    bcopy(nsec, rsec, size);
+    memcpy(rsec, nsec, size);
 
     LabG_Cache(new_w) = &(((XmToggleButtonGCacheObject)nsec)->label_cache);
     LabG_Cache(request) = &(((XmToggleButtonGCacheObject)rsec)->label_cache);
@@ -912,7 +940,7 @@ set_values(Widget old, Widget request, Widget new_w,
 {
     Boolean refresh_needed = False;
 
-    DEBUGOUT(XdbDebug(__FILE__, new_w, "ToggleBG set_values\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, new_w, "ToggleBG set_values\n"));
 
     if (TBG_IndType(old) != TBG_IndType(new_w) &&
 	!XmRepTypeValidValue(XmRepTypeGetId(XmRIndicatorType),
@@ -1053,18 +1081,67 @@ set_values(Widget old, Widget request, Widget new_w,
 	 || LabG_MarginTop(new_w) != LabG_MarginTop(old)
 	 || LabG_MarginBottom(new_w) != LabG_MarginBottom(old)))
     {
-	int iheight;
+	int margin, tm;
 
-	iheight = TBG_IndicatorDim(new_w);
+	margin = TBG_IndicatorDim(new_w) + TBG_Spacing(new_w)
+	    - (LabG_StringDirection(new_w) == XmSTRING_DIRECTION_L_TO_R
+	       ? LabG_MarginLeft(new_w) : LabG_MarginRight(new_w));
+	if (margin && (margin > 0 ||
+	    (LabG_StringDirection(new_w) == XmSTRING_DIRECTION_L_TO_R
+	     ? LabG_MarginLeft(new_w) == LabG_MarginLeft(old)
+	     : LabG_MarginRight(new_w) == LabG_MarginRight(old))))
+	{
+	    if (LabG_StringDirection(new_w) == XmSTRING_DIRECTION_L_TO_R)
+	    {
+		LabG_MarginLeft(new_w) += margin;
+		LabG_TextRect_x(new_w) += margin;
+		LabG_AccTextRect(new_w).x += margin;
+	    }
+	    else
+		LabG_MarginRight(new_w) += margin;
+	    if (LabG_RecomputeSize(new_w) || !XtWidth(request))
+		XtWidth(new_w) += margin;
+	}
+
+	margin = TBG_IndicatorDim(new_w) - (LabG_TextRect_height(new_w)
+					    + LabG_MarginTop(new_w)
+					    + LabG_MarginBottom(new_w));
 #ifdef LESS_VERTICAL_PADDING
-	iheight -= LabG_MarginHeight(new_w) << 1;
+	margin -= LabG_MarginHeight(new_w) << 1;
 #else
 	if (!IN_MENU(new_w))
-	    iheight += (LabG_Shadow(new_w) + Xm3D_ENHANCE_PIXEL) << 1;
+	    margin += (LabG_Shadow(new_w) + Xm3D_ENHANCE_PIXEL) << 1;
 #endif
-	_LtLabelGAccomodateIndicator(new_w, request,
-	    TBG_IndicatorDim(new_w) + TBG_Spacing(new_w), iheight,
-	    0, LabG_StringDirection(new_w) == XmSTRING_DIRECTION_L_TO_R, True);
+	if (margin)
+	{
+	    margin /= 2;
+	    tm = margin;
+	    if (tm < (LabG_MarginTop(new_w) == LabG_MarginTop(old)
+		      ? XmDEFAULT_TOP_MARGIN - (int)LabG_MarginTop(new_w)
+		      : 0))
+		tm = (LabG_MarginTop(new_w) == LabG_MarginTop(old)
+		      ? XmDEFAULT_TOP_MARGIN - (int)LabG_MarginTop(new_w)
+		      : 0);
+	    LabG_MarginTop(new_w) += tm;
+	    if (LabG_RecomputeSize(new_w) || !XtHeight(request))
+		XtHeight(new_w) += tm;
+
+	    if (margin < (LabG_MarginBottom(new_w) == LabG_MarginBottom(old)
+		      ? XmDEFAULT_BOTTOM_MARGIN - (int)LabG_MarginBottom(new_w)
+		      : 0))
+		margin = (LabG_MarginBottom(new_w) == LabG_MarginBottom(old)
+		      ? XmDEFAULT_BOTTOM_MARGIN - (int)LabG_MarginBottom(new_w)
+		      : 0);
+	    LabG_MarginBottom(new_w) += margin;
+	    if (LabG_RecomputeSize(new_w) || !XtHeight(request))
+		XtHeight(new_w) += margin;
+
+	    if (tm != margin)
+	    {
+		LabG_TextRect_y(new_w) += (tm - margin) / 2;
+		LabG_AccTextRect(new_w).y += (tm - margin) / 2;
+	    }
+	}
 
 	refresh_needed = True;
     }
@@ -1104,7 +1181,7 @@ get_values_prehook(Widget new_w, ArgList args, Cardinal *num_args)
 {
     XmBaseClassExt *bce;
     XmWidgetExtData ed;
-    int size;
+    Cardinal size;
     XtPointer nsec;
 
     bce = _XmGetBaseClassExtPtr(XtClass(new_w), XmQmotif);
@@ -1112,12 +1189,12 @@ get_values_prehook(Widget new_w, ArgList args, Cardinal *num_args)
 
     nsec = _XmExtObjAlloc(size);
 
-    bcopy(LabG_Cache(new_w),
-	  &((XmLabelGCacheObject)nsec)->label_cache,
-	  sizeof(XmLabelGCacheObjPart));
-    bcopy(TBG_Cache(new_w),
-	  &((XmToggleButtonGCacheObject)nsec)->toggle_cache,
-	  sizeof(XmToggleButtonGCacheObjPart));
+    memcpy(&((XmLabelGCacheObject)nsec)->label_cache,
+           LabG_Cache(new_w),
+	   sizeof(XmLabelGCacheObjPart));
+    memcpy(&((XmToggleButtonGCacheObject)nsec)->toggle_cache,
+           TBG_Cache(new_w),
+	   sizeof(XmToggleButtonGCacheObjPart));
 
     /*
      * don't do this and ResInd will blow up.
@@ -1191,12 +1268,12 @@ resize(Widget w)
 	    }
 	}
     }
-} 
+}
 
 static void
 expose(Widget w, XEvent *event, Region region)
 {
-    DEBUGOUT(XdbDebug(__FILE__, w, "ToggleBG Expose\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, w, "ToggleBG Expose\n"));
     draw_toggle(w, event, region, True, 0);
 }
 
@@ -1400,7 +1477,7 @@ ButtonDown(Widget w, XEvent *event, String *params, Cardinal *num_params)
 	RC_MenuShellPopdown(w, event, &poppedUp);
     }
     {
-	int i;
+	Cardinal i;
 	Widget menu = XtParent(w);
 
 	for (i = 0; i < MGR_NumChildren(menu); i++)
@@ -1543,7 +1620,7 @@ input_dispatch(Widget gadget, XEvent *event, Mask event_mask)
     {
     case XmARM_EVENT:
     case XmMULTI_ARM_EVENT:
-	DEBUGOUT(XdbDebug(__FILE__, gadget,
+	DEBUGOUT(_LtDebugDebug(__FILE__, gadget,
 			  "ToggleButtonGadget got arm event\n"));
 	if (IN_MENU(gadget))
 	{
@@ -1557,7 +1634,7 @@ input_dispatch(Widget gadget, XEvent *event, Mask event_mask)
 
     case XmACTIVATE_EVENT:
     case XmMULTI_ACTIVATE_EVENT:
-	DEBUGOUT(XdbDebug(__FILE__, gadget,
+	DEBUGOUT(_LtDebugDebug(__FILE__, gadget,
 			  "ToggleButtonGadget got activate event\n"));
 	if (IN_MENU(gadget))
 	{
@@ -1571,19 +1648,19 @@ input_dispatch(Widget gadget, XEvent *event, Mask event_mask)
 	break;
 
     case XmENTER_EVENT:
-	DEBUGOUT(XdbDebug(__FILE__, gadget,
+	DEBUGOUT(_LtDebugDebug(__FILE__, gadget,
 			  "ToggleButtonGadget enter window\n"));
 	EnterWindow(gadget, event, NULL, &num_params);
 	break;
 
     case XmLEAVE_EVENT:
-	DEBUGOUT(XdbDebug(__FILE__, gadget,
+	DEBUGOUT(_LtDebugDebug(__FILE__, gadget,
 			  "ToggleButtonGadget leave window\n"));
 	LeaveWindow(gadget, event, NULL, &num_params);
 	break;
 
     case XmMOTION_EVENT:
-	DEBUGOUT(XdbDebug(__FILE__, gadget,
+	DEBUGOUT(_LtDebugDebug(__FILE__, gadget,
 			  "ToggleButtonGadget motion event\n"));
 	break;
 
@@ -1608,7 +1685,6 @@ input_dispatch(Widget gadget, XEvent *event, Mask event_mask)
     }
 }
 
-#ifdef __STDC__
 static void
 MenuProcEntry(int proc, Widget w,...)
 {
@@ -1618,19 +1694,6 @@ MenuProcEntry(int proc, Widget w,...)
     */
 
     va_start(arg_list, w);
-#else
-MenuProcEntry(proc, w, va_alist)
-     int proc;
-     Widget w;
-     va_dcl
-{
-    va_list arglist;
-    /*
-    Cardinal num_params = 0;
-    */
-
-    va_start(arglist);
-#endif
 
     switch (proc)
     {
