@@ -1,8 +1,9 @@
 /*
-
- * $Header: /cvsroot/hungry/lesstif/lib/Xm/XmString.c,v 1.43 1999/08/12 12:09:17 danny Exp $
+ *
+ * $Header: /cvsroot/lesstif/lesstif/lib/Xm/XmString.c,v 1.69 2001/05/14 19:46:11 amai Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
+ * Copyright (C) 1995-2001 LessTif Development Team
  *
  * This file is part of the GNU LessTif Library.
  *
@@ -22,7 +23,7 @@
  *
  **/
 
-static const char rcsid[] = "$Header: /cvsroot/hungry/lesstif/lib/Xm/XmString.c,v 1.43 1999/08/12 12:09:17 danny Exp $";
+static const char rcsid[] = "$Header: /cvsroot/lesstif/lesstif/lib/Xm/XmString.c,v 1.69 2001/05/14 19:46:11 amai Exp $";
 
 /*
  * ** Danny had added a flag here, but I think the behavior he added is always
@@ -45,26 +46,20 @@ static const char rcsid[] = "$Header: /cvsroot/hungry/lesstif/lib/Xm/XmString.c,
 #include <LTconfig.h>
 
 #include <limits.h>
-#include <XmI/XmI.h>
-#include <XmI/MacrosI.h>
-
-#include <Xm/XmP.h>
-#include <X11/IntrinsicP.h>
-#include <X11/Xfuncs.h>
-
 #include <stdio.h>
 #include <locale.h>
 #include <stdlib.h>
-#ifdef HAVE_STRING_H
 #include <string.h>
-#else
-#include <strings.h>
-#endif
 
-#include <XmI/DebugUtil.h>
+#include <XmI/XmI.h>
+#include <XmI/MacrosI.h>
+#include <Xm/XmP.h>
+#include <X11/IntrinsicP.h>
 
 #include "LTmisc.h"
 
+#include <XmI/DebugUtil.h>
+
 /*
  * for information on the external encoding, see doc/XmStrings.txt
  */
@@ -77,19 +72,16 @@ static const char rcsid[] = "$Header: /cvsroot/hungry/lesstif/lib/Xm/XmString.c,
  */
 #define XmSTRING_HEADER_SIZE	(ASN1_HEADER_SIZE * 2)
 
-#ifndef XmUNSPECIFIED
-#define XmUNSPECIFIED (~0)
+/* don't need this in a production build */
+#ifndef LESSTIF_PRODUCTION
+#ifndef XMSTRING_DEBUG
+#define XMSTRING_DEBUG
+#endif
 #endif
 
-#define DEBUG	1
-
-/* For the prototype police */
-extern XmFontList XmStringCreateFontList( XFontStruct *font,
-					  XmStringCharSet charset);
-extern XmString XmStringConcatAndFree(XmString str1, XmString str2);
-
 /**************************** PRIVATE FUNCTIONS ****************************/
-#ifdef DEBUG
+
+#ifdef XMSTRING_DEBUG
 static int
 asn1_dump(unsigned char *string)
 {
@@ -261,7 +253,8 @@ asn1_dump(unsigned char *string)
     }
 }
 
-extern void
+
+static void
 _Xm_dump_fontlist(XmFontList fontlist)
 {
     int i;
@@ -277,19 +270,22 @@ _Xm_dump_fontlist(XmFontList fontlist)
     printf("\n");
 }
 
-extern void
+
+static void
 _Xm_dump_fontlist_cache(void)
 {
 }
 
-extern void
+
+static void
 _Xm_dump_external(XmString str)
 {
     asn1_dump(str);
     printf("\n");
 }
 
-extern void
+
+static void
 _Xm_dump_internal(_XmString str)
 {
     int i;
@@ -337,7 +333,7 @@ _Xm_dump_internal(_XmString str)
     printf("\n");
 }
 
-#endif
+#endif /* XMSTRING_DEBUG */
 
 static _XmString
 __XmAllocNewXmString(int number_of_components)
@@ -429,10 +425,11 @@ __XmStringComponentCopy(struct __XmStringComponentRec *dest,
     dest->data = XtNewString(src->data);
 }
 
+
 static _XmString
 __XmStringFromASN1(XmString string)
 {
-    unsigned length, i, nlen;
+    unsigned i, nlen, nlength, length;
     struct __XmStringExtRec *str = (struct __XmStringExtRec *)string;
     unsigned char *next;
     char *charset;
@@ -533,7 +530,7 @@ __XmStringFromASN1(XmString string)
 		XmSTRING_COMPONENT_CHARSET;
 
 	    charset = XtMalloc(nlen + 1);
-	    bcopy(&str->data[i], charset, nlen);
+	    memcpy(charset, &str->data[i], nlen);
 	    charset[nlen] = 0;
 	    if (_XmStringIsCurrentCharset(charset))
 	    {
@@ -563,8 +560,8 @@ __XmStringFromASN1(XmString string)
 		nlen;
 	    intern->components[intern->number_of_components - 1]->data =
 		XtMalloc(nlen + 1);
-	    bcopy(&str->data[i],
-		  intern->components[intern->number_of_components - 1]->data,
+	    memcpy(intern->components[intern->number_of_components - 1]->data,
+   	           &str->data[i],
 		  nlen);
 	    intern->components[intern->number_of_components - 1]->data[nlen] = 0;
 
@@ -607,8 +604,8 @@ __XmStringFromASN1(XmString string)
 		nlen;
 	    intern->components[intern->number_of_components - 1]->data =
 		XtMalloc(nlen + 1);
-	    bcopy(&str->data[i],
-		  intern->components[intern->number_of_components - 1]->data,
+	    memcpy(intern->components[intern->number_of_components - 1]->data,
+ 	          &str->data[i],
 		  nlen);
 	    intern->components[intern->number_of_components - 1]->data[nlen] = 0;
 
@@ -807,15 +804,14 @@ __XmStringToASN1(_XmString string)
 		    str->data[j] = tlen & 0x00FFU;
 		    tlen >>= 8;
 		}
-		bcopy(tmp,
-		      &str->data[totlen],
+		memcpy(&str->data[totlen], tmp,
 		      nlen);
 		str = (struct __XmStringExtRec *)&str->data[totlen + nlen];
 	    }
 	    else
 	    {
 		str->len = nlen;
-		bcopy(tmp, str->data, nlen);
+		memcpy(str->data, tmp, nlen);
 		str = (struct __XmStringExtRec *)&str->data[nlen];
 	    }
 	    break;
@@ -838,8 +834,8 @@ __XmStringToASN1(_XmString string)
 		    str->data[j] = nlen & 0x00FFU;
 		    nlen >>= 8;
 		}
-		bcopy(string->components[i]->data,
-		      &str->data[totlen],
+		memcpy(&str->data[totlen],
+ 		      string->components[i]->data,
 		      string->components[i]->length);
 		str = (struct __XmStringExtRec *)&str->data[totlen +
 						string->components[i]->length];
@@ -847,9 +843,9 @@ __XmStringToASN1(_XmString string)
 	    else
 	    {
 		str->len = string->components[i]->length;
-		bcopy(string->components[i]->data,
-		      str->data,
-		      string->components[i]->length);
+		memcpy(str->data,
+ 		       string->components[i]->data,
+		       string->components[i]->length);
 		str = (struct __XmStringExtRec *)&str->data[string->components[i]->length];
 	    }
 	    break;
@@ -873,18 +869,18 @@ __XmStringToASN1(_XmString string)
 		    str->data[j] = nlen & 0x00FFU;
 		    nlen >>= 8;
 		}
-		bcopy(string->components[i]->data,
-		      &str->data[totlen],
-		      string->components[i]->length);
+		memcpy(&str->data[totlen],
+ 		       string->components[i]->data,
+		       string->components[i]->length);
 		str = (struct __XmStringExtRec *)&str->data[totlen +
 						string->components[i]->length];
 	    }
 	    else
 	    {
 		str->len = string->components[i]->length;
-		bcopy(string->components[i]->data,
-		      str->data,
-		      string->components[i]->length);
+		memcpy(str->data,
+		       string->components[i]->data,
+		       string->components[i]->length);
 		str = (struct __XmStringExtRec *)&str->data[string->components[i]->length];
 	    }
 	    break;
@@ -930,7 +926,7 @@ __XmStringSegmentExtent(XmFontList flist, _XmStringComponent comp,
 	 comp->type != XmSTRING_COMPONENT_LOCALE_TEXT) ||
 	comp->font == XmUNSPECIFIED)
     {
-	DEBUGOUT(XdbDebug(__FILE__, NULL,
+	DEBUGOUT(_LtDebugDebug(__FILE__, NULL,
 			  "__XmStringSegmentExtent: got NULL Font/bad text\n"));
 
 	return False;
@@ -944,7 +940,7 @@ __XmStringSegmentExtent(XmFontList flist, _XmStringComponent comp,
     switch (flist[comp->font].type)
     {
     case XmFONT_IS_FONT:
-	DEBUGOUT(XdbDebug(__FILE__, NULL,
+	DEBUGOUT(_LtDebugDebug(__FILE__, NULL,
 		"_XmStringSegmentExtent(XmFONT_IS_FONT)\n"));
 	{
 	    XFontStruct *fontstruct = (XFontStruct *)flist[comp->font].font;
@@ -966,7 +962,7 @@ __XmStringSegmentExtent(XmFontList flist, _XmStringComponent comp,
 			     comp->data, comp->length,
 			     &dc, &amax, &dmax, &ov);
 
-		DEBUGOUT(XdbDebug(__FILE__, NULL,
+		DEBUGOUT(_LtDebugDebug(__FILE__, NULL,
 		    "_XmStringSegmentExtent: '%s' len %d\n",
 			comp->data, comp->length));
 	    }
@@ -978,7 +974,7 @@ __XmStringSegmentExtent(XmFontList flist, _XmStringComponent comp,
 	break;
 
     case XmFONT_IS_FONTSET:
-	DEBUGOUT(XdbDebug(__FILE__, NULL,
+	DEBUGOUT(_LtDebugDebug(__FILE__, NULL,
 		"_XmStringSegmentExtent(XmFONT_IS_FONTSET)\n"));
 
 	XmbTextExtents((XFontSet)flist[comp->font].font,
@@ -991,7 +987,7 @@ __XmStringSegmentExtent(XmFontList flist, _XmStringComponent comp,
 	break;
     }
 
-    DEBUGOUT(XdbDebug(__FILE__, NULL,
+    DEBUGOUT(_LtDebugDebug(__FILE__, NULL,
 	"_XmStringSegmentExtent => width %d height %d\n",
 	*width, *height));
 
@@ -999,7 +995,8 @@ __XmStringSegmentExtent(XmFontList flist, _XmStringComponent comp,
 }
 
 /**************************** INTERNAL FUNCTIONS ***************************/
-XFontStruct *
+
+extern XFontStruct *
 _XmGetFirstFont(XmFontListEntry entry)
 {
     XFontStruct **flist;
@@ -1027,7 +1024,8 @@ _XmGetFirstFont(XmFontListEntry entry)
     return NULL;
 }
 
-Boolean
+
+extern Boolean
 _XmFontListGetDefaultFont(XmFontList fontlist,
 			  XFontStruct **font_struct)
 {
@@ -1035,7 +1033,7 @@ _XmFontListGetDefaultFont(XmFontList fontlist,
 
     if (fontlist == NULL)
     {
-	DEBUGOUT(XdbDebug(__FILE__, NULL,
+	DEBUGOUT(_LtDebugDebug(__FILE__, NULL,
 			  "_XmFontListGetDefaultFont(NULL, ...)\n"));
 	*font_struct = NULL;
 	return False;
@@ -1070,7 +1068,8 @@ _XmFontListGetDefaultFont(XmFontList fontlist,
     return False;
 }
 
-Boolean
+
+extern Boolean
 _XmFontListSearch(XmFontList fontlist,
 		  XmStringCharSet charset,
 		  short *indx,
@@ -1138,7 +1137,7 @@ _XmFontListSearch(XmFontList fontlist,
  *
  * Returns True if the argument is external representation.
  */
-Boolean
+extern Boolean
 _XmStringIsXmString(XmString string)
 {
     struct __XmStringExtRec *str = (struct __XmStringExtRec *)string;
@@ -1146,14 +1145,14 @@ _XmStringIsXmString(XmString string)
 
     if (!string)
     {
-	DEBUGOUT(XdbDebug(__FILE__, NULL,
+	DEBUGOUT(_LtDebugDebug(__FILE__, NULL,
 			  "_XmStringIsXmString: string is NULL\n"));
 
 	return False;
     }
     if (str->tag != XmSTRING_TAG || str->len != XmSTRING_LENGTH)
     {
-	DEBUGOUT(XdbDebug(__FILE__, NULL,
+	DEBUGOUT(_LtDebugDebug(__FILE__, NULL,
 			  "_XmStringIsXmString: tag or len is invalid\n"));
 
 	return False;
@@ -1169,7 +1168,8 @@ _XmStringIsXmString(XmString string)
     return True;
 }
 
-Boolean
+
+extern Boolean
 _XmStringEmpty(_XmString string)
 {
     int i;
@@ -1192,7 +1192,40 @@ _XmStringEmpty(_XmString string)
     return True;
 }
 
-_XmString
+
+#if XmVERSION >= 2
+
+/*   Helper function for XmStringIsVoid().
+ *   CheckMe !!
+ */
+static Boolean
+_XmStringIsVoid(_XmString string)
+{
+    int i;
+
+    if (string == NULL || string->number_of_components == 0)
+    {
+	return True;
+    }
+
+    for (i = 0; i < string->number_of_components; i++)
+    {
+	if ( (string->components[i]->type == XmSTRING_COMPONENT_TEXT ||
+	      string->components[i]->type == XmSTRING_COMPONENT_LOCALE_TEXT ||
+	      string->components[i]->type == XmSTRING_COMPONENT_SEPARATOR ||
+	      string->components[i]->type == XmSTRING_COMPONENT_TAB) &&
+	    string->components[i]->length != 0)
+	{
+	    return False;
+	}
+    }
+
+    return True;
+}
+#endif /* XmVERSION >= 2 */
+
+
+extern _XmString
 _XmStringCreate(XmString cs)
 {
     if (!cs)
@@ -1203,7 +1236,8 @@ _XmStringCreate(XmString cs)
     return __XmStringFromASN1(cs);
 }
 
-void
+
+extern void
 _XmStringFree(_XmString string)
 {
     int i;
@@ -1213,7 +1247,10 @@ _XmStringFree(_XmString string)
 
     for (i = 0; i < string->number_of_components; i++)
     {
-	if (string->components[i]->data /*&& string->components[i]->length*/)
+       /* Peter Stein, 07/2000 */                                                       
+       /* in some wired circumstances this may not be the case */                       
+       /* leading to a crash */                                                         
+	if (string->components[i] && string->components[i]->data /*&& string->components[i]->length*/)
 	{
 	    /* rws 30 Aug 1998
 	       If we only free this if the length is greater than 0 we end
@@ -1225,15 +1262,23 @@ _XmStringFree(_XmString string)
 
 	XtFree((char *)string->components[i]);
     }
-
-    XtFree((char *)string->components);
+                                                                                                  
+    if( string->number_of_components > 0 )                                                  
+      {                                                                                       
+       /* Peter Stein, 07/2000: */                                                      
+       /* initialisation in __XmAllocNewXmString() */                                   
+       /*      only done in this case */                                                
+       /* so free should also only happen than*/                                        
+         XtFree((char *)string->components);                                              
+      }
     XtFree((char *)string);
 }
 
-char *
+
+extern char *
 _XmStringGetCurrentCharset(void)
 {
-    char *lang;
+    char *lang, *p;
 
     if ((lang = getenv("LANG")) == NULL)
     {
@@ -1248,16 +1293,22 @@ _XmStringGetCurrentCharset(void)
 	return XmSTRING_OS_CHARSET;
     }
 
-    return lang;
+    if ((p = strchr(lang, '.')) != NULL && *(p+1) != 0) {
+	return p+1;
+    }
+
+    return XmFALLBACK_CHARSET;
 }
 
-char *
+
+extern char *
 _XmCharsetCanonicalize(String charset)
 {
     return NULL;
 }
 
-void
+
+extern void
 _XmStringUpdate(XmFontList fontlist, _XmString string)
 {
     short i, where, def, tf;
@@ -1308,7 +1359,8 @@ _XmStringUpdate(XmFontList fontlist, _XmString string)
     }
 }
 
-_XmString
+
+extern _XmString
 _XmStringCopy(_XmString s)
 {
     _XmString newString;
@@ -1329,7 +1381,8 @@ _XmStringCopy(_XmString s)
     return newString;
 }
 
-Boolean
+
+extern Boolean
 _XmStringByteCompare(_XmString a, _XmString b)
 {
     _XmStringContext context1 = NULL;
@@ -1391,7 +1444,8 @@ _XmStringByteCompare(_XmString a, _XmString b)
     return True;
 }
 
-Boolean
+
+extern Boolean
 _XmStringHasSubstring(_XmString string, _XmString substring)
 {
     int i, j, k;
@@ -1418,12 +1472,11 @@ _XmStringHasSubstring(_XmString string, _XmString substring)
     for (i = 0; i < string->number_of_components; i++)
     {
 	if (string->components[i]->type == substring->components[0]->type &&
-	    string->components[i]->length == substring->components[0]->length &&
+	    string->components[i]->length >= substring->components[0]->length &&
 	    ((string->components[i]->data != NULL &&
 	      substring->components[0]->data != NULL &&
-	      strncmp(string->components[i]->data,
-		      substring->components[0]->data,
-		      substring->components[0]->length) == 0) ||
+               strstr( string->components[i]->data,
+                       substring->components[0]->data ) != NULL) ||
 	     (string->components[i]->data == NULL &&
 	      substring->components[0] == NULL)))
 	{
@@ -1433,13 +1486,12 @@ _XmStringHasSubstring(_XmString string, _XmString substring)
 	    {
 		if (string->components[j]->type !=
 		    substring->components[k]->type ||
-		    string->components[j]->length !=
+		    string->components[j]->length <
 		    substring->components[k]->length ||
 		    ((string->components[j]->data != NULL &&
 		      substring->components[k]->data != NULL &&
-		      strncmp(string->components[j]->data,
-			      substring->components[k]->data,
-			      substring->components[k]->length) != 0) ||
+		      strstr(string->components[j]->data,
+			      substring->components[k]->data) == NULL) ||
 		     (string->components[j]->data != NULL &&
 		      substring->components[k] == NULL) ||
 		     (string->components[j]->data == NULL &&
@@ -1460,7 +1512,8 @@ _XmStringHasSubstring(_XmString string, _XmString substring)
     return False;
 }
 
-XmString
+
+extern XmString
 _XmStringCreateExternal(XmFontList fontlist, _XmString cs)
 {
     /* need to change the string maybe, so that if a font doesn't exist
@@ -1473,7 +1526,8 @@ _XmStringCreateExternal(XmFontList fontlist, _XmString cs)
     return __XmStringToASN1(cs);
 }
 
-char *
+
+extern char *
 _XmStringGetTextConcat(XmString string)
 {
     int ilen = 0, len = 0;
@@ -1501,7 +1555,7 @@ _XmStringGetTextConcat(XmString string)
 		len = comp->length;
 		ret = XtRealloc(ret, ilen + len + 1);
 	    }
-	    bcopy(comp->data, &ret[ilen], len);
+	    memcpy(&ret[ilen], comp->data, len);
 	    ret[ilen + len] = 0;
 	    ilen += len;
 	}
@@ -1514,7 +1568,8 @@ _XmStringGetTextConcat(XmString string)
     return ret;
 }
 
-Boolean
+
+extern Boolean
 _XmStringIsCurrentCharset(XmStringCharSet c)
 {
     if (!c)
@@ -1526,7 +1581,8 @@ _XmStringIsCurrentCharset(XmStringCharSet c)
     return strcmp(c, _XmStringGetCurrentCharset()) == 0;
 }
 
-Boolean
+
+extern Boolean
 _XmStringSingleSegment(XmString str, char **pTextOut,
 		       XmStringCharSet *pCharsetOut)
 {
@@ -1561,7 +1617,8 @@ _XmStringSingleSegment(XmString str, char **pTextOut,
  *
  * FIX ME - deal with the font tags as per the above spec.
  */
-void
+
+extern void
 _XmStringUpdateWMShellTitle(XmString xmstr, Widget shell)
 {
     String title = NULL;
@@ -1596,7 +1653,8 @@ _XmStringUpdateWMShellTitle(XmString xmstr, Widget shell)
     }
 }
 
-Boolean
+
+extern Boolean
 _XmStringInitContext(_XmStringContext *context, _XmString string)
 {
     _XmStringContext p;
@@ -1617,7 +1675,8 @@ _XmStringInitContext(_XmStringContext *context, _XmString string)
     }
 }
 
-Boolean
+
+extern Boolean
 _XmStringGetNextSegment(_XmStringContext context,
 			XmStringCharSet *tag,
 			XmStringDirection *direction,
@@ -1735,13 +1794,15 @@ _XmStringGetNextSegment(_XmStringContext context,
     return valid;
 }
 
-void
+
+extern void
 _XmStringFreeContext(_XmStringContext context)
 {
     XtFree((char *)context);
 }
 
-void
+
+extern void
 _XmStringExtent(XmFontList fontlist, _XmString string,
 		Dimension *width, Dimension *height)
 {
@@ -1759,7 +1820,7 @@ _XmStringExtent(XmFontList fontlist, _XmString string,
 	return;
     }
 
-    DEBUGOUT(XdbDebug(__FILE__, NULL, "_XmStringExtent start\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, NULL, "_XmStringExtent start\n"));
 
     _XmStringUpdate(fontlist, string);
 
@@ -1786,12 +1847,12 @@ _XmStringExtent(XmFontList fontlist, _XmString string,
 	    }
 	    current_width += wd;
 
-	    DEBUGOUT(XdbDebug(__FILE__, NULL,
+	    DEBUGOUT(_LtDebugDebug(__FILE__, NULL,
 		"_XmStringExtent: text segment, line height %d\n", ht));
 	}
 	else if (comp->type == XmSTRING_COMPONENT_SEPARATOR)
 	{
-	    DEBUGOUT(XdbDebug(__FILE__, NULL, "_XmStringExtent: separator\n"));
+	    DEBUGOUT(_LtDebugDebug(__FILE__, NULL, "_XmStringExtent: separator\n"));
 
 	/* What if we don't have a height ... ? */
 	    if (line_height == 0) {
@@ -1808,7 +1869,7 @@ _XmStringExtent(XmFontList fontlist, _XmString string,
 		    __XmStringSegmentExtent(fontlist, blankcomp,
 				&wd, &default_line_height, &amax, &dmax);
 
-		    DEBUGOUT(XdbDebug(__FILE__, NULL,
+		    DEBUGOUT(_LtDebugDebug(__FILE__, NULL,
 			"_XmStringExtent: separator found height %d\n",
 			default_line_height));
 		}
@@ -1843,7 +1904,7 @@ _XmStringExtent(XmFontList fontlist, _XmString string,
 	    {
 		current_height += default_line_height;
 
-		DEBUGOUT(XdbDebug(__FILE__, NULL,
+		DEBUGOUT(_LtDebugDebug(__FILE__, NULL,
 		  "_XmStringExtent-separator: default_line_height %d added\n",
 		  default_line_height));
 	    }
@@ -1851,7 +1912,7 @@ _XmStringExtent(XmFontList fontlist, _XmString string,
 	    {
 		current_height += line_height;
 
-		DEBUGOUT(XdbDebug(__FILE__, NULL,
+		DEBUGOUT(_LtDebugDebug(__FILE__, NULL,
 		  "_XmStringExtent-separator: line_height %d added\n",
 		  line_height));
 	    }
@@ -1884,7 +1945,8 @@ _XmStringExtent(XmFontList fontlist, _XmString string,
     _XmStringFreeContext(context);
 }
 
-Dimension
+
+extern Dimension
 _XmStringWidth(XmFontList fontlist, _XmString string)
 {
     Dimension width = 0, dontcare;
@@ -1894,7 +1956,8 @@ _XmStringWidth(XmFontList fontlist, _XmString string)
     return width;
 }
 
-Dimension
+
+extern Dimension
 _XmStringHeight(XmFontList fontlist, _XmString string)
 {
     Dimension height = 0, dontcare;
@@ -1904,7 +1967,8 @@ _XmStringHeight(XmFontList fontlist, _XmString string)
     return height;
 }
 
-Dimension
+
+extern Dimension
 _XmStringBaseline(XmFontList fontlist, _XmString string)
 {
     _XmStringContext context = NULL;
@@ -1942,11 +2006,13 @@ _XmStringBaseline(XmFontList fontlist, _XmString string)
     return current_baseline;
 }
 
-int
+
+extern int
 _XmStringLineCount(_XmString string)
 {
     _XmStringContext context = NULL;
-    char *text, *tag;
+    char *text;
+    char *tag=NULL;
     short len;
     XmStringDirection dir;
     Boolean sep;
@@ -1973,7 +2039,8 @@ _XmStringLineCount(_XmString string)
     return lc;
 }
 
-void
+
+extern void
 _XmStringDraw(Display *d, Window w, XmFontList fontlist, _XmString string,
 	      GC gc, Position x, Position y, Dimension width,
 	      unsigned char align, unsigned char lay_dir,
@@ -1998,7 +2065,7 @@ _XmStringDraw(Display *d, Window w, XmFontList fontlist, _XmString string,
 	return;
     }
 
-    DEBUGOUT(XdbDebug(__FILE__, XtWindowToWidget(d, w),
+    DEBUGOUT(_LtDebugDebug(__FILE__, XtWindowToWidget(d, w),
 		      "_XmStringDraw x %d y %d wid %d\n",
 		      x, y, width));
 
@@ -2210,7 +2277,8 @@ _XmStringDraw(Display *d, Window w, XmFontList fontlist, _XmString string,
     }
 }
 
-void
+
+extern void
 _XmStringDrawImage(Display *d, Window w,
 		   XmFontList fontlist, _XmString string,
 		   GC gc, Position x, Position y, Dimension width,
@@ -2437,7 +2505,8 @@ _XmStringDrawImage(Display *d, Window w,
     }
 }
 
-void
+
+extern void
 _XmStringDrawUnderline(Display *d, Window w, XmFontList fontlist,
 		       _XmString string, GC gc,
 		       Position x, Position y, Dimension width,
@@ -2460,12 +2529,12 @@ _XmStringDrawUnderline(Display *d, Window w, XmFontList fontlist,
 	return;
     }
 
-    DEBUGOUT(XdbDebug(__FILE__, XtWindowToWidget(d, w),
+    DEBUGOUT(_LtDebugDebug(__FILE__, XtWindowToWidget(d, w),
 		      "_XmStringDrawUnderline x %d y %d wid %d\n",
 		      x, y, width));
     if (!underline)
     {
-	DEBUGOUT(XdbDebug(__FILE__, XtWindowToWidget(d, w),
+	DEBUGOUT(_LtDebugDebug(__FILE__, XtWindowToWidget(d, w),
 		       "_XmStringDrawUnderline, underlined string is NULL\n"));
     }
 
@@ -2787,7 +2856,8 @@ _XmStringDrawUnderline(Display *d, Window w, XmFontList fontlist,
     }
 }
 
-void
+
+extern void
 _XmStringDrawMnemonic(Display *d,
 		      Window w,
 		      XmFontList fontlist,
@@ -2805,9 +2875,9 @@ _XmStringDrawMnemonic(Display *d,
     XmString und;
     _XmString _und;
 
-    DEBUGOUT(XdbDebug(__FILE__, XtWindowToWidget(d, w),
+    DEBUGOUT(_LtDebugDebug(__FILE__, XtWindowToWidget(d, w),
 		      "_XmStringDrawMnemonic(%s, %s)\n",
-		      XdbXmString2String((XmString)string), mnemonic));
+		      _LtDebugXmString2String((XmString)string), mnemonic));
 
     if (mnemonic == NULL)
     {
@@ -2827,10 +2897,11 @@ _XmStringDrawMnemonic(Display *d,
     _XmStringFree(_und);
 }
 
+
 /* This is essentially _XmStringDraw without the drawing part.
  * It expects baselines to point to memory already allocated.
  */
-void
+extern void
 _XmStringBaselines(XmFontList fontlist, _XmString string, Position y,
 		   Dimension *baselines)
 {
@@ -2940,7 +3011,8 @@ _XmStringBaselines(XmFontList fontlist, _XmString string, Position y,
 }
 
 /************************* PUBLIC FUNCTIONS *****************************/
-Dimension
+
+extern Dimension
 XmStringBaseline(XmFontList fontlist,
 		 XmString string)
 {
@@ -2959,7 +3031,8 @@ XmStringBaseline(XmFontList fontlist,
     return baseline;
 }
 
-Boolean
+
+extern Boolean
 XmStringByteCompare(XmString s1,
 		    XmString s2)
 {
@@ -2982,7 +3055,8 @@ XmStringByteCompare(XmString s1,
     return ret;
 }
 
-Boolean
+
+extern Boolean
 XmStringCompare(XmString s1, XmString s2)
 {
  XmStringContext context1 = NULL, context2 = NULL;
@@ -3011,10 +3085,20 @@ XmStringCompare(XmString s1, XmString s2)
 	compare_tags = (strcmp(tag1, XmFONTLIST_DEFAULT_TAG) != 0 &&
 			strcmp(tag2, XmFONTLIST_DEFAULT_TAG) != 0);
 	 /* Don't compare tags when one of them = XmFONTLIST_DEFAULT_TAG */
-	if (strcmp(text1, text2) != 0 /* diferent text */
+	/*
+	printf("%p %p\n>%s< >%s<\n%i %i\n%i %i\n", text1, text2, text1, text2, direction1, direction2, separator1, separator2);
+	*/
+	if (
+	    ((text1 == NULL || text2 == NULL) && text1 != text2) 
+	    || (text1 != text2 && strcmp(text1, text2) != 0) /* diferent text */
 	    || direction1 != direction2
 	    || (compare_tags && strcmp(tag1, tag2) != 0)
-	    || separator1 != separator2) {
+	    || separator1 != separator2
+	   )
+	{
+		/*
+		printf("Different\n");
+		*/
 		XtFree(text1);
 		XtFree(text2);
 		XtFree(tag1);
@@ -3023,6 +3107,9 @@ XmStringCompare(XmString s1, XmString s2)
 		XmStringFreeContext(context2);
 		return False;
 	}
+		/*
+		printf("Equal\n");
+		*/
  }
 
  XtFree(text1);
@@ -3034,7 +3121,8 @@ XmStringCompare(XmString s1, XmString s2)
  return True;
 }
 
-XmString
+
+extern XmString
 XmStringConcat(XmString str1, XmString str2)
 {
     _XmString s1, s2;
@@ -3042,7 +3130,7 @@ XmStringConcat(XmString str1, XmString str2)
     int i, base, total;
     XmString ret;
 
-    DEBUGOUT(XdbDebug(__FILE__, NULL, "XmStringConcat() entering\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, NULL, "XmStringConcat() entering\n"));
 
     if (!_XmStringIsXmString(str1) && !_XmStringIsXmString(str2))
     {
@@ -3095,12 +3183,13 @@ XmStringConcat(XmString str1, XmString str2)
 
     _XmStringFree(newString);
 
-    DEBUGOUT(XdbDebug(__FILE__, NULL, "XmStringConcat() leaving\n"));
+    DEBUGOUT(_LtDebugDebug(__FILE__, NULL, "XmStringConcat() leaving\n"));
 
     return ret;
 }
 
-XmString
+
+extern XmString
 XmStringCopy(XmString s)
 {
     _XmString s1;
@@ -3119,7 +3208,8 @@ XmStringCopy(XmString s)
     return ret;
 }
 
-XmString
+
+extern XmString
 XmStringNConcat(XmString s1, XmString s2, int num_bytes)
 {
     _XmString str1, str2;
@@ -3193,9 +3283,9 @@ XmStringNConcat(XmString s1, XmString s2, int num_bytes)
 	    str1->components[str1->number_of_components - 1]->length = nlen;
 	    str1->components[str1->number_of_components - 1]->data =
 		XtMalloc(nlen + 1);
-	    bcopy(str2->components[i]->data,
-		  str1->components[str1->number_of_components - 1]->data,
-		  nlen);
+	    memcpy(str1->components[str1->number_of_components - 1]->data,
+	           str2->components[i]->data,
+		   nlen);
 	    str1->components[str1->number_of_components - 1]->data[nlen] = 0;
 	}
 	else
@@ -3221,9 +3311,9 @@ XmStringNConcat(XmString s1, XmString s2, int num_bytes)
 	    str1->components[str1->number_of_components - 1]->length = nlen;
 	    str1->components[str1->number_of_components - 1]->data =
 		XtMalloc(nlen + 1);
-	    bcopy(str2->components[i]->data,
-		  str1->components[str1->number_of_components - 1]->data,
-		  nlen);
+	    memcpy(str1->components[str1->number_of_components - 1]->data,
+ 	           str2->components[i]->data,
+		   nlen);
 	    str1->components[str1->number_of_components - 1]->data[nlen] = 0;
 	}
 
@@ -3238,7 +3328,8 @@ XmStringNConcat(XmString s1, XmString s2, int num_bytes)
     return ret;
 }
 
-XmString
+
+extern XmString
 XmStringNCopy(XmString s1, int num_bytes)
 {
     _XmString str1, str2;
@@ -3312,9 +3403,9 @@ XmStringNCopy(XmString s1, int num_bytes)
 	    str1->components[str1->number_of_components - 1]->length = nlen;
 	    str1->components[str1->number_of_components - 1]->data =
 		XtMalloc(nlen + 1);
-	    bcopy(str2->components[i]->data,
-		  str1->components[str1->number_of_components - 1]->data,
-		  nlen);
+	    memcpy(str1->components[str1->number_of_components - 1]->data,
+	           str2->components[i]->data,
+		   nlen);
 	    str1->components[str1->number_of_components - 1]->data[nlen] = 0;
 	}
 	else
@@ -3342,9 +3433,9 @@ XmStringNCopy(XmString s1, int num_bytes)
 	    str1->components[str1->number_of_components - 1]->length = nlen;
 	    str1->components[str1->number_of_components - 1]->data =
 		XtMalloc(nlen + 1);
-	    bcopy(str2->components[i]->data,
-		  str1->components[str1->number_of_components - 1]->data,
-		  nlen);
+	    memcpy(str1->components[str1->number_of_components - 1]->data,
+	           str2->components[i]->data,
+		   nlen);
 	    str1->components[str1->number_of_components - 1]->data[nlen] = 0;
 	}
 
@@ -3359,7 +3450,8 @@ XmStringNCopy(XmString s1, int num_bytes)
     return ret;
 }
 
-XmString
+
+extern XmString
 XmStringSegmentCreate(char *text,
 		      char *tag,
 		      XmStringDirection direction,
@@ -3440,7 +3532,8 @@ XmStringSegmentCreate(char *text,
     return ret;
 }
 
-XmString
+
+extern XmString
 XmStringSeparatorCreate(void)
 {
     _XmString newString = __XmAllocNewXmString(1);
@@ -3456,7 +3549,8 @@ XmStringSeparatorCreate(void)
     return ret;
 }
 
-XmString
+
+extern XmString
 XmStringDirectionCreate(XmStringDirection direction)
 {
     _XmString newString = __XmAllocNewXmString(1);
@@ -3473,7 +3567,8 @@ XmStringDirectionCreate(XmStringDirection direction)
     return ret;
 }
 
-XmString
+
+extern XmString
 XmStringCreate(char *text, char *tag)
 {
     _XmString str;
@@ -3510,7 +3605,8 @@ XmStringCreate(char *text, char *tag)
     return ret;
 }
 
-XmString
+
+extern XmString
 XmStringCreateLtoR(char *text, char *tag)
 {
     XmString ret;
@@ -3629,19 +3725,22 @@ XmStringCreateLtoR(char *text, char *tag)
     return ret;
 }
 
-XmString
+
+extern XmString
 XmStringLtoRCreate(char *text, char *tag)
 {
     return XmStringCreateLtoR(text, tag);
 }
 
-XmString
+
+extern XmString
 XmStringCreateLocalized(char *text)
 {
     return XmStringCreate(text, XmFONTLIST_DEFAULT_TAG);
 }
 
-XmString
+
+extern XmString
 XmStringCreateSimple(char *text)
 {
     _XmString str;
@@ -3669,7 +3768,8 @@ XmStringCreateSimple(char *text)
     return ret;
 }
 
-void
+
+extern void
 XmStringDraw(Display *d,
 	     Window w,
 	     XmFontList fontlist,
@@ -3697,7 +3797,8 @@ XmStringDraw(Display *d,
     _XmStringFree(str);
 }
 
-void
+
+extern void
 XmStringDrawImage(Display *d,
 		  Window w,
 		  XmFontList fontlist,
@@ -3725,13 +3826,14 @@ XmStringDrawImage(Display *d,
     _XmStringFree(str);
 }
 
+
 /*
  * Similar to XmStringDraw, but with an additional parameter
  *
  * If the underline string is contained in string, then that part
  * of string is underlined.
  */
-void
+extern void
 XmStringDrawUnderline(Display *d,
 		      Window w,
 		      XmFontList fontlist,
@@ -3773,7 +3875,8 @@ XmStringDrawUnderline(Display *d,
     _XmStringFree(str);
 }
 
-Boolean
+
+extern Boolean
 XmStringEmpty(XmString s1)
 {
     _XmString str;
@@ -3793,7 +3896,8 @@ XmStringEmpty(XmString s1)
     return ret;
 }
 
-void
+
+extern void
 XmStringExtent(XmFontList fontlist,
 	       XmString string,
 	       Dimension *width,
@@ -3818,17 +3922,18 @@ XmStringExtent(XmFontList fontlist,
 
     if (!string)
     {
-	DEBUGOUT(XdbDebug(__FILE__, NULL,
+	DEBUGOUT(_LtDebugDebug(__FILE__, NULL,
 			  "XmStringExtent() string is NULL\n"));
     }
 
 }
 
+
 /*
  * according to Motif, this does only return the first segment in a
  * multi-segment string
  */
-Boolean
+extern Boolean
 XmStringGetLtoR(XmString string,
 		XmStringCharSet tag,
 		char **text)
@@ -3881,7 +3986,8 @@ XmStringGetLtoR(XmString string,
     return Found;
 }
 
-void
+
+extern void
 XmStringFree(XmString string)
 {
     if (!_XmStringIsSpecified(string))
@@ -3897,7 +4003,8 @@ XmStringFree(XmString string)
     XtFree((char *)string);
 }
 
-Boolean
+
+extern Boolean
 XmStringInitContext(XmStringContext *context,
 		    XmString string)
 {
@@ -3913,7 +4020,8 @@ XmStringInitContext(XmStringContext *context,
     return True;
 }
 
-XmStringComponentType
+
+extern XmStringComponentType
 XmStringGetNextComponent(XmStringContext context,
 			 char **text,
 			 XmStringCharSet *tag,
@@ -3963,7 +4071,7 @@ XmStringGetNextComponent(XmStringContext context,
 	    *unknown_tag = r->type;
 	    *unknown_length = r->length;
 	    *unknown_value = (unsigned char *)XtMalloc(r->length);
-	    bcopy(r->data, *unknown_value, r->length);
+	    memcpy(*unknown_value, r->data, r->length);
 	    break;
 
 	default:
@@ -3978,7 +4086,8 @@ XmStringGetNextComponent(XmStringContext context,
     }
 }
 
-XmStringComponentType
+
+extern XmStringComponentType
 XmStringPeekNextComponent(XmStringContext context)
 {
     if (context == NULL)
@@ -3996,12 +4105,13 @@ XmStringPeekNextComponent(XmStringContext context)
     }
 }
 
+
 /*
  * Assumption: if you change charsets or directions before a separator,
  * then that is another segment.  I don't know if this is right, but it
  * makes sense? MLM
  */
-Boolean
+extern Boolean
 XmStringGetNextSegment(XmStringContext context,
 		       char **text,
 		       XmStringCharSet *tag,
@@ -4020,7 +4130,7 @@ XmStringGetNextSegment(XmStringContext context,
 	return False;
     }
 
-    if (text && context->text)
+    if (text)
     {
 	*text = XtNewString(context->text);
     }
@@ -4043,7 +4153,8 @@ XmStringGetNextSegment(XmStringContext context,
     return True;
 }
 
-void
+
+extern void
 XmStringFreeContext(XmStringContext context)
 {
     _XmStringFree(context->string);
@@ -4051,7 +4162,8 @@ XmStringFreeContext(XmStringContext context)
     XtFree((char *)context);
 }
 
-Boolean
+
+extern Boolean
 XmStringHasSubstring(XmString string,
 		     XmString substring)
 {
@@ -4074,7 +4186,8 @@ XmStringHasSubstring(XmString string,
     return ret;
 }
 
-Dimension
+
+extern Dimension
 XmStringWidth(XmFontList fontlist,
 	      XmString string)
 {
@@ -4085,7 +4198,8 @@ XmStringWidth(XmFontList fontlist,
     return width;
 }
 
-Dimension
+
+extern Dimension
 XmStringHeight(XmFontList fontlist, XmString string)
 {
     Dimension width, height;
@@ -4095,7 +4209,8 @@ XmStringHeight(XmFontList fontlist, XmString string)
     return height;
 }
 
-int
+
+extern int
 XmStringLength(XmString s1)
 {
     unsigned length, i;
@@ -4133,7 +4248,8 @@ XmStringLength(XmString s1)
     return length + XmSTRING_HEADER_SIZE;
 }
 
-int
+
+extern int
 XmStringLineCount(XmString string)
 {
     _XmString str;
@@ -4153,16 +4269,22 @@ XmStringLineCount(XmString string)
     return lc;
 }
 
+
 /*
- * This sucker appears to be undocumented
+ * This sucker appears to be undocumented.
+ * Literal search in OM docs brings up a thread-safe variant
+ * only 'XmStringCreateFontList_r' !?!
  */
-XmFontList
+extern XmFontList
 XmStringCreateFontList( XFontStruct *font, XmStringCharSet charset)
 {
 	return XmFontListCreate(font, charset);
 }
 
-XmString
+
+#if XmVERSION >= 2
+
+extern XmString
 XmStringConcatAndFree(XmString str1, XmString str2)
 {
   XmString tmp;
@@ -4172,13 +4294,13 @@ XmStringConcatAndFree(XmString str1, XmString str2)
   return tmp;
 }
 
-#if XmVERSION > 1
+
 /*
  * MxFTP uses this as
  *	item = (char *)XmStringUnparse(tempstr,NULL,XmCHARSET_TEXT,
  *		XmCHARSET_TEXT,NULL,0,XmOUTPUT_ALL);
  */
-XtPointer
+extern XtPointer
 XmStringUnparse(XmString string, XmStringTag tag, XmTextType tag_type,
 	XmTextType output_type, XmParseTable parse_table,
 	Cardinal parse_count, XmParseModel parse_model)
@@ -4231,10 +4353,11 @@ XmStringUnparse(XmString string, XmStringTag tag, XmTextType tag_type,
 #endif
 		if (comp->data) {
 			if (res) {
-				res = realloc(res, strlen(res) + 1 + strlen(comp->data));
+				res = (char *)realloc(res, strlen(res) + 1 + strlen(comp->data));
 				strcat(res, comp->data);
 			} else {
-				res = strdup(comp->data);
+				res = (char *)malloc(strlen(comp->data)+1);
+				strcpy(res, comp->data);
 			}
 		}
 			
@@ -4251,71 +4374,174 @@ XmStringUnparse(XmString string, XmStringTag tag, XmTextType tag_type,
     return res;
 }
 
-XmString
+
+extern XmString
 XmStringParseText(XtPointer text, XtPointer *text_end, XmStringTag tag,
 	XmTextType type, XmParseTable parse_table, Cardinal parse_count,
 	XtPointer call_data)
 {
-	return NULL;
+	return (XmString)NULL;
 }
 
-Cardinal
+
+extern Cardinal
 XmStringToXmStringTable(XmString string, XmString break_comp, XmStringTable *table)
 {
-	return 0;
+	
+	_XmWarning(NULL, "XmStringToXmStringTable() is not implemented yet!\n");
+	return (Cardinal)0;
 }
 
-XmString
+
+extern XmString
 XmStringTableToXmString(XmStringTable table, Cardinal count, XmString break_component)
 {
-	return NULL;
+	_XmWarning(NULL, "XmStringTableToXmString() is not implemented yet!\n");
+	return (XmString)NULL;
 }
 
-XtPointer *
+
+extern XtPointer *
 XmStringTableUnparse(XmStringTable table, Cardinal count, XmStringTag tag,
 	XmTextType tag_type, XmTextType output_type, XmParseTable parse,
 	Cardinal parse_count, XmParseModel parse_model)
 {
-	return NULL;
+	_XmWarning(NULL, "XmStringTableUnparse() is not implemented yet!\n");
+	return (XtPointer *)NULL;
 }
 
-XmStringTable
+
+extern XmStringTable
 XmStringTableParseStringArray(XtPointer *strings, Cardinal count,
 	XmStringTag tag, XmTextType type, XmParseTable parse,
 	Cardinal parse_count, XtPointer call_data)
 {
-	return NULL;
+	_XmWarning(NULL, "XmStringTableParseStringArray() is not implemented yet!\n");
+	return (XmStringTable)NULL;
 }
 
-XmString
+
+extern XmString
 XmStringPutRendition(XmString string, XmStringTag rendition)
 {
-	return NULL;
+	_XmWarning(NULL, "XmStringPutRendition() is not implemented yet!\n");
+	return (XmString)NULL;
 }
 
-XmParseMapping
+
+extern XmParseMapping
 XmParseMappingCreate(ArgList  arg_list, Cardinal arg_count)
 {
-	return NULL;
+	_XmWarning(NULL, "XmParseMappingCreate() is not implemented yet!\n");
+	return (XmParseMapping)NULL;
 }
 
-void
+
+extern void
 XmParseMappingSetValues(XmParseMapping parse_mapping, ArgList arg_list, Cardinal arg_count)
 {
+	_XmWarning(NULL, "XmParseMappingSetValues() is not implemented yet!\n");
 }
 
-void
+
+extern void
 XmParseMappingGetValues(XmParseMapping parse_mapping, ArgList arg_list, Cardinal arg_count)
 {
+	_XmWarning(NULL, "XmParseMappingGetValues() is not implemented yet!\n");
 }
 
-void
+
+extern void
 XmParseMappingFree(XmParseMapping parse_mapping)
 {
+	_XmWarning(NULL, "XmParseMappingFree() is not implemented yet!\n");
 }
 
-void
+
+extern void
 XmParseTableFree(XmParseTable parse_table, Cardinal parse_count)
 {
+	_XmWarning(NULL, "XmParseTableFree() is not implemented yet!\n");
 }
-#endif
+
+
+extern XmStringComponentType
+XmStringGetNextTriple(XmStringContext context,
+           unsigned int *length,
+	   XtPointer *value)
+{
+	_XmWarning(NULL, "XmStringGetNextTriple() is not implemented yet!\n");
+	return (XmStringComponentType)NULL;
+}
+
+
+extern XmString
+XmStringComponentCreate(XmStringComponentType ctype,
+                                 unsigned int length,
+                                 XtPointer value)
+{
+	_XmWarning(NULL, "XmStringComponentCreate() is not implemented yet!\n");
+	return (XmString)NULL;
+}
+
+
+extern XmIncludeStatus
+XmeGetNextCharacter(XtPointer *text_in_out,
+                    XtPointer text_end,
+                    XmTextType type,
+                    XmStringTag tag,
+                    XmParseMapping entry,
+                    int pattern_length,
+                    XmString *str_include,
+                    XtPointer call_data)
+{
+       _XmWarning(NULL, "XmeGetNextCharacter() is not implemented yet!\n");
+       return XmINSERT; /* this is not sufficient to indicate a failure ... */
+}
+
+
+extern XmIncludeStatus
+XmeGetDirection(XtPointer *text_in_out,
+                XtPointer text_end,
+                XmTextType type,
+                XmStringTag tag,
+                XmParseMapping entry,
+                int pattern_length,
+                XmString *str_include,
+                XtPointer call_data)
+{
+       _XmWarning(NULL, "XmeGetDirection() is not implemented yet!\n");
+       return XmINSERT; /* this is not sufficient to indicate a failure ... */
+}
+
+
+extern XmStringComponentType
+XmStringPeekNextTriple(XmStringContext context)
+{
+       _XmWarning(NULL, "XmStringPeekNextTriple() is not implemented yet!\n");
+       return XmSTRING_COMPONENT_UNKNOWN;
+}
+
+
+/* similar structure to XmStringEmpty() */
+extern Boolean
+XmStringIsVoid(XmString s1)
+{
+    _XmString str;
+    Boolean ret;
+
+    if (!_XmStringIsXmString(s1))
+    {
+	return True;
+    }
+
+    str = _XmStringCreate(s1);
+
+    ret = _XmStringIsVoid(str);
+
+    _XmStringFree(str);
+
+    return ret;
+}
+
+#endif /* #if XmVERSION >= 2 */
