@@ -8,11 +8,15 @@
 
 /* test1.c */
 #include <unistd.h>
+#include <stdio.h>
 #include <Xm/Xm.h>
 #include <Xm/BulletinB.h>
 #include <Xm/ArrowBG.h>
 
-void 
+#include "../../common/Test.h"
+#include "../../common/DumpLayout.h"
+
+int
 main(int argc, char *argv[])
 {
     XtAppContext app;
