@@ -1,4 +1,4 @@
-/*
+/* $Header: /cvsroot/lesstif/lesstif/test/Xm/filesb/test13.c,v 1.7 2001/03/13 21:21:30 amai Exp $
    From:        Yasushi Yamasaki <yamapu@osk3.3web.ne.jp>
    To:          lesstif@hungry.com
    Subject:     bug report
@@ -6,7 +6,9 @@
    Cc:          yamapu@osk3.3web.ne.jp
  */
 
-/* test1.c */
+
+#include <stdlib.h>
+#include <stdio.h>
 #include <unistd.h>
 #include <Xm/Xm.h>
 #include <Xm/FileSB.h>
@@ -16,7 +18,7 @@ static char *FallBack[] = {
 		NULL
 };
 
-void 
+int
 main(int argc, char *argv[])
 {
     XtAppContext app;
@@ -27,7 +29,7 @@ main(int argc, char *argv[])
     int diff = 0;
     int total = 0;
 
-  make_tmp_dir_tree();
+/*  make_tmp_dir_tree(); */
     shell = XtAppInitialize(&app, "Test", NULL, 0,
 			    &argc, argv, FallBack, NULL, 0);
     while (iter < 100)
